;**********************************************************************;
;*                              A T U H R                             *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : Dieses Programm ist ein Uhr-Treiber, mit dessen*;
;*                     Hilfe das DOS bei allen Datums- und Zeit-      *;
;*                     funktionen auf die batteriegepufferte Uhr des  *;
;*                     AT zugreift.                                   *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  :  4.08.1987                                     *;
;*    letztes Update :  3.11.1991                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM ATUHR;                                    *;
;*                     LINK ATUHR;                                    *;
;*                     EXE2BIN ATUHR ATUHR.SYS    oder                *;
;*                                                                    *;
;*                     TASM ATUHR                                     *;
;*                     LINK ATUHR;                                    *;
;*                     EXE2BIN ATUHR ATUHR.SYS                        *;
;*--------------------------------------------------------------------*;
;*    Aufruf         : In das Hauptverzeichnis kopieren, den Befehl   *;
;*                     DEVICE=ATUHR.SYS in die Datei CONFIG.SYS auf-  *;
;*                     nehmen und dann System booten.                 *;
;**********************************************************************;
                                                                        
code     segment
                                                                        
	 assume cs:code,ds:code
                                                                        
         org 0                   ;Programm hat keinen PSP, daher Anfang
                                 ;an der Offsetadresse 0
                                                                        
;== Konstanten =========================================================
                                                                        
befehl   equ 2                   ;Offset Befehls-Feld im Datenblock
status   equ 3                   ;Offset Status-Feld im Datenblock
end_adr  equ 14                  ;Offset Treiber End-Adr. im Datenblock
anzahl   equ 18                  ;Offset Anzahl im Datenblock
p_adr    equ 14                  ;Offset Puffer-Adresse im Datenblock
                                                                        
;== Daten ==============================================================
                                                                        
;-- Kopf des Gerte-Treibers -------------------------------------------
                                                                        
         dw -1,-1                ;Verbindung zum nchsten Treiber
         dw 1000000000001000b    ;Treiber Attribut
         dw offset strat         ;Pointer auf Strategie-Routine
         dw offset intr          ;Pointer auf Interrupt-Routine
         db "$CLOCK "            ;neuer Uhr-Treiber
                                                                        
db_ptr   dw (?),(?)              ;Adresse des bergebenen Datenblocks
                                                                        
mon_tab  db 31                   ;Tabelle mit der Anzahl der Tage in
februar  db 28                   ;den jeweiligen Monaten
         db 31,30,31,30,31,31,30,31,30,31
                                                                        
;== Routinen und Funktionen des Treibers ===============================
                                                                        
strat    proc far                ;Strategie-Routine
                                                                        
         mov  cs:db_ptr,bx       ;Adresse des Datenblocks in der
         mov  cs:db_ptr+2,es     ;Variable DB_PTR merken
                                                                        
         ret                     ;zurck zum Aufrufer
                                                                        
strat    endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
intr     proc far                ;Interrupt-Routine
                                                                        
         push ax                 ;Register auf Stack sichern
         push bx
         push cx
         push dx
         push di
         push si
         push bp
         push ds
         push es
         pushf                   ;auch das Flag-Register merken
                                                                        
         cld                     ;bei Stringbefehlen aufwrts zhlen
                                                                        
         push cs                 ;Datensegment-Register setzen
         pop  ds                 ;Code ist hier mit Daten identisch
                                                                        
         les  di,dword ptr db_ptr;Adresse des Datenblocks nach ES:DI
         mov  bl,es:[di+befehl]  ;Befehls-Code holen
         cmp  bl,4               ;soll Uhrzeit/Datum gelesen werden?
         je   ud_lesen           ;JA --> UD_LESEN
         cmp  bl,8               ;soll Uhrzeit/Datum geschrieben werden?
         je   ud_schr            ;JA --> UD_SCHR
         or   bl,bl              ;soll der Treiber initialisiert werden?
         jne  unb_fkt            ;NEIN --> unbekannte Funktion
                                                                        
         jmp  init               ;Treiber initialisieren
                                                                        
unb_fkt: mov  ax,8003h           ;Code fr "unbekannter Befehl"
                                                                        
         ;-- Ausfhrung der Funktion beendet ---------------------------
                                                                        
intr_end label near
         or   ax,0100h           ;Fertig-Bit setzen
         mov  es:[di+status],ax  ;das Ganze im Status-Feld abspeichern
                                                                        
         popf                    ;Flag-Register zurckholen
         pop  es                 ;andere Register zurckholen
         pop  ds
         pop  bp
         pop  si
         pop  di
         pop  dx
         pop  cx
         pop  bx
         pop  ax
                                                                        
         ret                     ;zurck zum Aufrufer
                                                                        
intr     endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
ud_lesen label near              ;Uhzeit/Datum aus der Uhr auslesen
                                                                        
         mov  byte ptr es:[di+anzahl],6 ;6 Bytes werden bergeben
         les  di,es:[di+p_adr]   ;ES:DI zeigt in den DOS-Puffer
                                                                        
         mov  ah,4               ;Funktionsnummer fr Datum lesen
         int  1Ah                ;BIOS Zeit Interrupt aufrufen
         call date_ofs           ;Datum nach Offset zum 1.1.1980 wandeln
         stosw                   ;in Puffer abspeichern
                                                                        
         mov  ah,2               ;Funktionsnummer fr Zeit lesen
         int  1Ah                ;BIOS Zeit Interrupt aufrufen
         mov  bl,ch              ;Stunde in BL merken
         call bcd_bin            ;Minute umwandeln
         stosb                   ;In Puffer abspeichern
         mov  cl,bl              ;Stunde nach CL
         call bcd_bin            ;Stunde umwandeln
         stosb                   ;In Puffer abspeichern
         xor  al,al              ;Hundertstel Sekunde ist 0
         stosb                   ;In Puffer abspeichern
         mov  cl,dh              ;Sekunde nach CL
         call bcd_bin            ;Sekunde umwandeln
         stosb                   ;In Puffer abspeichern
                                                                        
         xor  ax,ax              ;alles o.k.
         jmp  short intr_end     ;zurck zum Aufrufer
                                                                        
;-----------------------------------------------------------------------
                                                                        
ud_schr   label near             ;Uhzeit/Datum in die Uhr schreiben
                                                                        
         mov  byte ptr es:[di+anzahl],6 ;6 Bytes werden gelesen
         les  di,es:[di+p_adr]   ;ES:DI zeigt in den DOS Puffer
                                                                        
         mov  ax,es:[di]         ;Anzahl der Tage seit 1.1.1980 holen
         push ax                 ;Anzahl merken
         call ofs_date           ;und in ein Datum umrechnen
         mov  ch,19h             ;Jahr beginnt mit 19..
         mov  ah,5               ;Funktionsnummer fr Datum setzen
         int  1AH                ;BIOS Zeit Interrupt aufrufen
                                                                        
         mov  al,es:[di+2]       ;Minute aus Puffer holen
         call bin_bcd            ;nach BCD wandeln
         mov  cl,al              ;und nach CL bringen
         mov  al,es:[di+5]       ;Sekunde aus Puffer holen
         call bin_bcd            ;nach BCD wandeln
         mov  dh,al              ;und nach DH bringen
         mov  al,es:[di+3]       ;Stunde aus Puffer holen
         call bin_bcd            ;nach BCD wandeln
         mov  ch,al              ;und nach CH bringen
         xor  dl,dl              ;keine Sommerzeit
         mov  ah,3               ;Funktionsnummer fr Zeit setzen
         int  1AH                ;BIOS Zeit Interrupt aufrufen
                                                                        
         ;-- Tag der Woche berechnen -----------------------------------
         xor  dx,dx              ;HI-Word fr Division
         pop  ax                 ;Anzahl der Tage wieder vom Stack holen
         or   ax,ax              ;ist Anzahl 0?
         je   nodiv              ;JA --> Division umgehen
         xor  dx,dx              ;HI-Word fr Division
         mov  cx,7               ;sieben Tage hat die Woche
         div  cx                 ;AX durch 7 teilen
nodiv:   add  dl,3               ;der 1.1.80 war ein Dienstag (Tag 3)
         cmp  dl,8               ;ist es Sonntag oder Montag?
         jb   nosomo             ;NEIN --> keine Korrektur notwendig
         sub  dl,cl              ;Wert korrigieren
nosomo:  mov  al,6               ;Speicherst. 6 in RTC ist Tag der Woche
         out  70h,al             ;Adresse an RTC-Adress-Register
         mov  al,dl              ;Tag der Woche nach AL
         out  71h,al             ;Tag der Woche an RTC-Datenregister
                                                                        
         xor  ax,ax              ;alles o.k.
         jmp  intr_end           ;zurck zum Aufrufer
                                                                        
;-- OFS_DATE: Anzahl Tage seit dem 1.1.1980 in Datum umwandeln ---------
;-- Eingabe  : AX = Anzahl der Tage seit dem 1.1.1980
;-- Ausgabe  : CL = Jahr, DH = Monat und DL = Tag
;-- Register : AX, BX, CX, DX, SI und FLAGS werden verndert
;-- Info     : zur Umrechnung des Offsets wird auf das Array MON_TAB
;--            zugegriffen
                                                                        
ofs_date proc near
                                                                        
         mov  cl,80              ;Jahr 1980 vorgeben
         mov  dh,01              ;Januar vorgeben
ja0:     mov  bx,365             ;Anzahl der Tage in einem normalen Jahr
         test cl,3               ;ist das Jahr ein Schaltjahr?
         jne  ja1                ;NEIN --> ja1
         inc  bl                 ;das Schaltjahr hat einen Tag mehr
ja1:     cmp  ax,bx              ;noch ein Jahr vergangen?
         jb   mo                 ;NEIN --> Monat berechnen
         inc  cl                 ;JA --> Jahr inkrementieren
         sub  ax,bx              ;Anzahl Tage in diesem Jahr abziehen
	 jmp  short ja0          ;nchstes Jahr berechnen
                                                                        
mo:      mov  bl,28              ;Tage im Februar in einem normalen Jahr
         test cl,11b             ;ist das Jahr Schaltjahr?
         jne  keinsj2            ;NEIN --> KEINSJ2
         inc  bl                 ;im Schaltjahr hat der Fabruar 29 Tage
keinsj2: mov  februar,bl         ;Anzahl der Tage im Februar speichern
                                                                        
         mov  si,offset mon_tab  ;Adresse der Monats-Tabelle
         xor  bh,bh              ;jeder Monat hat weniger als 256 Tage
mo1:     mov  bl,[si]            ;Anzahl der Tage im Monat holen
         cmp  ax,bx              ;noch ein Monat vergangen?
         jb   ta                 ;NEIN --> Tag berechnen
         sub  ax,bx              ;JA --> Tage des Monats abziehen
         inc  dh                 ;Monat inkrementieren
         inc  si                 ;SI auf nchsten Monat in der Tabelle
         jmp  short mo1          ;nchsten Monat berechnen
                                                                        
ta:      inc  al                 ;der Rest + 1 ist der Tag
         call bin_bcd            ;Tag nach BCD wandeln
         mov  dl,al              ;nach DL bertragen
         mov  al,dh              ;Monat nach AL bertragen
         call bin_bcd            ;nach BCD wandeln
         mov  dh,al              ;wieder nach DH holen
         mov  al,cl              ;Jahr nach AL bertragen
         call bin_bcd            ;nach BCD wandeln
         mov  cl,al              ;wieder nach CL holen
                                                                        
         ret                     ;zurck zum Aufrufer
                                                                        
ofs_date endp
                                                                        
;-- BIN_BCD: Binr-Zahl nach BCD wandeln -------------------------------
;-- Eingabe  : AL = Binrwert
;-- Ausgabe  : AL = entsprechender BCD-Wert
;-- Register : AX, CX und FLAGS werden verndert
                                                                        
bin_bcd  proc near
                                                                        
         xor  ah,ah              ;16 Bit-Division vorbereiten
         mov  ch,10              ;wir arbeiten im 10er System
         div  ch                 ;AX durch 10 teilen
         shl  al,1               ;Quotient um 4 Stellen nach links
         shl  al,1               ;verschieben
         shl  al,1
         shl  al,1
         or   al,ah              ;den Rest einblenden
         ret                     ;zurck zum Aufrufer
                                                                        
bin_bcd  endp
                                                                        
;-- DATE_OFS: Datum in Anzahl der Tage seit dem 1.1.1980 umwandeln -----
;-- Eingabe  : CL = Jahr, DH = Monat und DL = Tag
;-- Ausgabe  : AX = Anzahl der Tage seit dem 1.1.1980
;-- Register : AX, BX, CX, DX, SI und FLAGS werden verndert
;-- Info     : zur Umrechnung des Datums wird auf das Array MON_TAB
;--            zugegriffen
                                                                        
date_ofs proc near
                                                                        
         call bcd_bin            ;Jahr nach binr umwandeln
         mov  bl,al              ;und nach BL bertragen
         mov  cl,dh              ;Monat nach CL bertragen
         call bcd_bin            ;Monat nach binr umwandeln
         mov  dh,al              ;und wieder nach DH bertragen
         mov  cl,dl              ;Tag nach CL bertragen
         call bcd_bin            ;Tag nach binr umwandeln
         mov  dl,al              ;und wieder nach DL bertragen
                                                                        
         xor  ax,ax              ;0 Tage
         mov  ch,bl              ;Jahr merken
         dec  bl                 ;ein Jahr zurckgehen
jahr:    cmp  bl,80              ;bis auf das jahr 1980 heruntergezhlt?
         jb   monat              ;JA --> Monat umrechnen
         test bl,11b             ;ist Jahr ein Schaltjahr?
         jne  keinsj             ;NEIN --> KEINSJ
         inc  ax                 ;ein Schaltjahr hat einen Tag mehr
keinsj:  add  ax,365             ;Tage eines Jahres hinzuaddieren
         dec  bl                 ;ein Jahr zurckgehen
         jmp  short jahr         ;nchstes Jahr bearbeiten
                                                                        
monat:   mov  bl,28              ;Tage im Februar in einem normalen Jahr
         test ch,11b             ;ist aktuelles Jahr Schaltjahr?
         jne  keinsj1            ;NEIN --> KEINSJ1
         inc  bl                 ;im Schaltjahr hat der Fabruar 29 Tage
keinsj1: mov  februar,bl         ;in Monatstabelle abspeichern
         xor  ch,ch              ;jeder Monat hat weniger als 256 Tage
         mov  bx,offset mon_tab  ;Adresse der Monats-Tabelle
monat1:  dec  dh                 ;Anzahl der Monate dekrementieren
         je   tag                ;alle Monate berechnet --> TAG
         mov  cl,[bx]            ;Anzahl der Tage im Monat holen
         add  ax,cx              ;zu den Gesamt-Tagen addieren
         inc  bx                 ;BX auf nchsten Monat in der Tabelle
         jmp  short monat1       ;nchten Monat berechnen
                                                                        
tag:     add  ax,dx              ;aktuellen Tag hinzuaddieren
         dec  ax                 ;einen Tag abziehen (1.1.80 = 0)
         ret                     ;zurck zum Aufrufer
                                                                        
date_ofs  endp
                                                                        
;-- BCD_BIN: BCD nach Binr-Zahl wandeln -------------------------------
;-- Eingabe  : CL = BCD-Wert
;-- Ausgabe  : AL = entsprechender Binr-Wert
;-- Register : AX, CX und FLAGS werden verndert
                                                                        
bcd_bin  proc near               ;BCD-Wert in CL nach Binr wandeln
                                 ;und in AL zurckliefern
                                                                        
         mov  al,cl              ;Wert nach AL bertragen
         shr  al,1               ;um 4 Stellen nach rechts verschieben
         shr  al,1
         shr  al,1
         shr  al,1
         xor  ah,ah              ;AH auf 0 setzen
         mov  ch,10              ;wir arbeiten im 10er System
         mul  ch                 ;AX mit 10 multiplizieren
         mov  ch,cl              ;CL nach CH bertragen
         and  ch,1111b           ;Hi-Nibble in CH auf 0 setzen
         add  al,ch              ;AL und CH addieren
         ret                     ;zurck zum Aufrufer
                                                                        
bcd_bin  endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
init     proc near               ;Initialisierungs-Routine
                                                                        
         ;-- der folgende Code kann nach der Installation der ----------
         ;-- Uhr vom DOS berschrieben werden
                                                                        
         mov  word ptr es:[di+end_adr],offset init  ;Endadresse des
         mov  es:[di+end_adr+2],cs                  ;Treibers setzen
                                                                        
         mov  ah,9               ;Installationsmeldung ausgeben
         mov  dx,offset initm    ;Adresse des Textes
         int  21h                ;DOS Interrupt aufrufen
                                                                        
         xor  ax,ax              ;alles o.k.
         jmp  intr_end           ;zurck zum Aufrufer
                                                                        
initm    db 13,10,"**** ATUHR-Treiber installiert. (c) 1987, 1991 by"
         db " MICHAEL TISCHER",13,10,"$"
                                                                        
init     endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
code     ends
         end
