;**********************************************************************;
;*                              C O N D R V                           *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : Dieses Programm stellt einen normalen Consolen *;
;*                     Treiber (Tastatur und Bildschirm) dar. Es soll *;
;*                     Ihnen als Gerst eines Treibers in der Art des *;
;*                     Treibers ANSI.SYS dienen.                      *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  :  4.08.1987                                     *;
;*    letztes Update :  2.11.1991                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM CONDRV;                                   *;
;*                     LINK CONDRV;                                   *;
;*                     EXE2BIN CONDRV CONDRV.SYS    oder              *;
;*                                                                    *;
;*                     TASM CONDRV                                    *;
;*                     LINK CONDRV;                                   *;
;*                     EXE2BIN CONDRV CONDRV.SYS                      *;
;*--------------------------------------------------------------------*;
;*    Aufruf         : In das Hauptverzeichnis kopieren, den Befehl   *;
;*                     DEVICE=CONDRV.SYS in die Datei CONFIG.SYS auf- *;
;*                     nehmen und dann System booten.                 *;
;**********************************************************************;
                                                                        
code     segment
                                                                        
	 assume cs:code,ds:code
                                                                        
         org 0                   ;Programm hat keinen PSP daher Anfang
                                 ;an der Offsetadresse 0
                                                                        
;== Konstanten =========================================================
                                                                        
befehl   equ 2                   ;Offset Befehls-Feld im Datenblock
status   equ 3                   ;Offset Status-Feld im Datenblock
end_adr  equ 14                  ;Offset Treiber End-Adr. im Datenblock
anzahl   equ 18                  ;Offset Anzahl im Datenblock
p_adr    equ 14                  ;Offset Puffer-Adresse im Datenblock
                                                                        
TAS_GR   equ 20                  ;Gre des Tastaturpuffer
anz_bef  equ 16                  ;die Funktionen 0-16 werden untersttzt
                                                                        
;== Daten ==============================================================
                                                                        
;-- Kopf des Gerte-Treibers -------------------------------------------
                                                                        
         dw -1,-1                ;Verbindung zum nchsten Treiber
         dw 1010100000000011b    ;Treiber Attribut
         dw offset strat         ;Pointer auf Strategie-Routine
         dw offset intr          ;Pointer auf Interrupt-Routine
         db "CONDRV  "           ;neuer Konsolen-Treiber
                                                                        
;-- Sprungtabelle fr die einzelnen Funktionen -------------------------
                                                                        
fkt_tab  dw offset init          ;Funktion  0: Initialisierung
         dw offset dummy         ;Funktion  1: Medium Test
         dw offset dummy         ;Funktion  2: Erstelle BPB
         dw offset no_sup        ;Funktion  3: direktes Lesen
         dw offset lesen         ;Funktion  4: Lesen
         dw offset lesen_p       ;Funktion  5: Lesen, im Puffer bleiben
         dw offset dummy         ;Funktion  6: Eingabe-Status
         dw offset del_ei_p      ;Funktion  7: Lsche Eingabe-Puffer
         dw offset schreibe      ;Funktion  8: Schreiben
         dw offset schreibe      ;Funktion  9: Schreiben & Verifizieren
         dw offset dummy         ;Funktion 10: Ausgabe-Status
         dw offset dummy         ;Funktion 11: Lschen Ausgabe-Puffer
         dw offset no_sup        ;Funktion 12: direktes Schreiben
         dw offset dummy         ;Funktion 13: ffnen (diese Fkt ab 3.0)
         dw offset dummy         ;Funktion 14: Schlieen
         dw offset dummy         ;Funktion 15: wechselbares Medium
         dw offset schreibe      ;Funktion 16: Ausgabe bis beschftigt
                                                                        
db_ptr   dw (?),(?)              ;Adresse des bergebenen Datenblocks
                                                                        
tas_a    dw 0                    ;Pointer auf nchstes Zeichen im TAS_PU
tas_e    dw 0                    ;Pointer auf letztes Zeichen im TAS_PU
tas_pu   db TAS_GR dup (?)       ;interner Tastaturpuffer
                                                                        
;== Routinen und Funktionen des Treibers ===============================
                                                                        
strat    proc far                ;Strategie-Routine
                                                                        
         mov  cs:db_ptr,bx       ;Adresse des Datenblocks in der
         mov  cs:db_ptr+2,es     ;Variable DB_PTR merken
                                                                        
         ret                     ;zurck zum Aufrufer
                                                                        
strat    endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
intr     proc far                ;Interrupt-Routine
                                                                        
         push ax                 ;Register auf Stack sichern
         push bx
         push cx
         push dx
         push di
         push si
         push bp
         push ds
         push es
         pushf                   ;auch das Flag-Register merken
                                                                        
         push cs                 ;Datensegment-Register setzen
         pop  ds                 ;Code ist hier mit Daten identisch
                                                                        
         les  di,dword ptr db_ptr;Adresse des Datenblocks nach ES:DI
         mov  bl,es:[di+befehl]  ;Befehls-Code holen
         cmp  bl,anz_bef         ;ist Befehls-Code erlaubt?
         jle  bc_ok              ;JA --> bc_ok
                                                                        
         mov  ax,8003h           ;Code fr "unbekannter Befehl"
         jmp  short intr_end     ;zurck zum Aufrufer
                                                                        
         ;-- Befehls-Code war o.k. --> Befehl ausfhren ----------------
                                                                        
bc_ok:   shl  bl,1               ;Pointer in Sprungtabelle errechnen
         xor  bh,bh              ;BH lschen
         call [fkt_tab+bx]       ;Funktion aufrufen
         les  di,dword ptr db_ptr;Adresse des Datenblocks nach ES:DI
                                                                        
         ;-- Ausfhrung der Funktion beendet ---------------------------
                                                                        
intr_end label near
         or   ax,0100h           ;Fertig-Bit setzen
         mov  es:[di+status],ax  ;das ganze im Status-Feld abspeichern
                                                                        
         popf                    ;Flag-Register zurckholen
         pop  es                 ;andere Register zurckholen
         pop  ds
         pop  bp
         pop  si
         pop  di
         pop  dx
         pop  cx
         pop  bx
         pop  ax
                                                                        
         ret                     ;zurck zum Aufrufer
                                                                        
intr     endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
dummy    proc near               ;diese Routine bewirkt nichts
                                                                        
         xor  ax,ax              ;Beschftigt-Bit lschen
         ret                     ;zurck zum Aufrufer
                                                                        
dummy    endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
no_sup   proc near               ;diese Routine wird fr alle Funktionen
                                 ;aufgerufen, die eigentlich nicht auf-
                                 ;gerufen werden drfen
                                                                        
         mov  ax,8003h           ;Fehler: Befehl nicht erkannt
         ret                     ;zurck zum Aufrufer
                                                                        
no_sup   endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
store_c  proc near               ;speichert ein Zeichen im internen
                                 ;Tastaturpuffer
                                 ;Eingabe: AL = Zeichen
                                 ;         BX = Position des Zeichen
                                                                        
         mov  [bx+tas_pu],al     ;Zeichen im internen Puffer ablegen
         inc  bl                 ;Pointer auf Ende erhhen
         cmp  bl,TAS_GR          ;Ende des Puffers erreicht?
         jne  store_e            ;NEIN --> STORE_E
                                                                        
         xor  bl,bl              ;neues Ende ist Anfang des Puffers
                                                                        
store_e: ret                     ;zurck zum Aufrufer
                                                                        
store_c  endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
lesen    proc near               ;bestimmte Anzahl Zeichen von der
                                 ;Tastatur in einen Puffer lesen
                                                                        
         mov  cx,es:[di+anzahl]  ;Anzahl der Zeichen lesen
         jcxz lesen_e            ;testen ob gleich 0
         les  di,es:[di+p_adr]   ;Adresse des Zeichen-Puffers nach ES:DI
         cld                     ;bei STOSB aufwrts zhlen
         mov  si,tas_a           ;Pointer auf nchstes Zeichen im TAS_PU
         mov  bx,tas_e           ;Pointer auf letztes Zeichen im TAS_PU
                                                                        
lesen_1: cmp  si,bx              ;sind noch Zeichen im Tastaturpuffer?
         jne  lesen_3            ;JA --> LESEN_3
                                                                        
lesen_2: xor  ah,ah              ;Funktionsnummer fr Lesen ist 0
         int  16h                ;BIOS Tastatur-Interrupt aufrufen
         call store_c            ;Zeichen im internen Puffer speichern
         cmp  al,0               ;testen ob erweiterter Code
         jne  lesen_3            ;nein --> LESEN_3
                                                                        
         mov  al,ah              ;Erweiterter Code steht in AH
         call store_c            ;auch diesen Speichern
                                                                        
lesen_3: mov  al,[si+tas_pu]     ;Zeichen aus Tastaturpuffer lesen
         stosb                   ;in Puffer der aufrufenden Funktion
                                 ;bertragen
         inc  si                 ;Pointer auf nchstes Zeichen erhhen
         cmp  si,TAS_GR          ;Ende des Puffers erreicht?
         jne  lesen_4            ;NEIN --> LESEN_4
                                                                        
         xor  si,si              ;nchstes Zeichen ist das erste Zeichen
                                 ;im Tastaturpuffer
                                                                        
lesen_4: loop lesen_1            ;wiederholen bis alle Zeichen gelesen
         mov  tas_a,si           ;Position des nchsten Zeichens im Tas-
                                 ;taturpuffer speichern
         mov  byte ptr tas_e,bl  ;Position des letzten Zeichens im Tasta-
                                 ;turpuffer speichern
                                                                        
lesen_e: xor  ax,ax              ;alles o.k.
         ret                     ;zurck zum Aufrufer
                                                                        
lesen    endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
lesen_p  proc near               ;das nchste Zeichen von der Tastatur
                                 ;lesen, aber im Puffer belassen
                                                                        
         mov  ah,1               ;Funktionsnummer fr BIOS-Interrupt
         int  16h                ;BIOS Tastatur-Interrupt aufrufen
         je   lesen_p1           ;kein Zeichen da --> LESEN_P1
                                                                        
         mov  es:[di+13],al      ;Zeichen in Datenblock eintragen
         xor ax,ax               ;alles o.k.
         ret                     ;zurck zum Aufurufer
                                                                        
lesen_p1 label near
                                                                        
         mov  ax,0100h           ;Beschftigt-Bit setzen (kein Zeichen)
         ret                     ;zurck zum Aufrufer
                                                                        
lesen_p  endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
del_ei_p proc near               ;den Eingabe-Puffer lschen
                                                                        
         mov  ah,1               ;Funktion: sind noch Zeichen im Puffer?
         int  16h                ;BIOS Tastatur-Interrupt aufrufen
         je   del_e              ;kein Zeichen mehr im Puffer --> ENDE
                                                                        
         xor  ah,ah              ;Funktion: Zeichen aus Puffer entfernen
         int  16h                ;BIOS Tastatur-Interrupt aufrufen
         jmp  short del_ei_p     ;auf weitere Zeichen testen
                                                                        
del_e:   xor ax,ax               ;alles o.k.
         ret                     ;zurck zum Aufrufer
                                                                        
del_ei_p endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
schreibe proc near               ;bestimmte Anzahl Zeichen auf dem Bild-
                                 ;schirm ausgeben
                                                                        
         mov  cx,es:[di+anzahl]  ;Anzahl der Zeichen lesen
         jcxz schr_e             ;testen ob gleich 0
         lds  si,es:[di+p_adr]   ;Adresse des Zeichen-Puffers nach DS:SI
         cld                     ;bei LODSB aufwrts zhlen
                                                                        
         mov  ah,3               ;aktuelle Bildschirmseite lesen
         int  16h                ;BIOS Video-Interrupt aufrufen
                                                                        
         mov  ah,14              ;Funktionsnummer fr BIOS Interrupt
                                                                        
schr_1:  lodsb                   ;auszugebendens Zeichen nach AL lesen
         int  10h                ;BIOS Video-Interrupt aufrufen
         loop schr_1             ;wiederh. bis alle Zeichen ausgegeben
                                                                        
schr_e:  xor ax,ax               ;alles o.k.
         ret                     ;zurck zum Aufrufer
                                                                        
schreibe endp
                                                                        
;-----------------------------------------------------------------------
                                                                        
init     proc near               ;Initialisierungs-Routine
                                                                        
         mov  word ptr es:[di+end_adr],offset init  ;End-Adresse des
         mov  es:[di+end_adr+2],cs                  ;Treibers setzen
                                                                        
         xor  ax,ax              ;alles o.k.
         ret                     ;zurck zum Aufrufer
                                                                        
init     endp
                                                                        
;=======================================================================
                                                                        
code     ends
         end
