;**********************************************************************;
;*                            V H E R C                               *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : stellt einige elementare Funktionen zum Zu-    *;
;*                     griff auf die HERCULES GRAPHICS CARD zu        *;
;*                     Verfgung                                      *;
;*--------------------------------------------------------------------*;
;*    Info           : alle Funktionen unterteilen den Bildschirm     *;
;*                     in die Spalten 0 bis 79 und Zeilen 0 bis 24    *;
;*                     im Textmodus und in die Spalten 0 bis 719 und  *;
;*                     die Zeilen 0 bis 347 im Grafikmodus            *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  : 11.08.1987                                     *;
;*    letztes Update :  1.10.1988                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM VHERC;                                    *;
;*                     LINK VHERC;                                    *;
;*--------------------------------------------------------------------*;
;*    Aufruf         : VHERC                                          *;
;**********************************************************************;
                                                                        
;== Konstanten =========================================================
                                                                        
STEUER_REG  = 03B8h               ;Portadr. des Steuerregisters
ADRESS_6845 = 03B4h               ;Adresseregister des 6845
DATA_6845   = 03B5h               ;Datenregister des 6845
KONFIG_REG  = 03BFh               ;Konfigurationsregister
VIO_SEG     = 0B000h              ;Segmentadresse des Video-RAM
CUR_START   = 10                  ;Registernr. CRTC: Startzeile Cursor
CUR_END     = 11                  ;Registernr. CRTC: Endzeile Cursor
CURPOS_HI   = 14                  ;Registernr. CRTC: Cursorpos. Hi-Byte
CURPOS_LO   = 15                  ;Registernr. CRTC: Cursorpos. Lo-Byte
                                                                        
DELAY       = 20000               ;Schleifenzhler fr Warteschleife
                                                                        
;== Makros =============================================================
                                                                        
setmode   macro modus             ;Steuerregister setzen
                                                                        
          mov  dx,STEUER_REG      ;Adresse des Bildschirmsteuerregisters
          mov  al,modus           ;neuen Modus in das AL-Register
          out  dx,al              ;Modus an Steuerregister senden
                                                                        
          endm
                                                                        
setvk     macro                   ;schreibt einen Wert in eines der
                                  ;CRTC-Register
                                  ;Eingabe: AL = Registernummer
                                  ;         AH = Wert fr das Register
                                                                        
          mov  dx,ADRESS_6845     ;Adresse des Indexregisters
          out  dx,ax              ;Registernr. und neuen Wert ausgeben
                                                                        
          endm
                                                                        
;== Stack ==============================================================
                                                                        
stack     segment para stack      ;Definition des Stack-Segments
                                                                        
          dw 256 dup (?)          ;der Stack ist 256 Words gro
                                                                        
stack     ends                    ;Ende des Stack-Segments
                                                                        
;== Daten ==============================================================
                                                                        
data      segment para 'DATA'     ;Definition des Daten-Segments
                                                                        
;== die Daten fr das Demo-Programm ====================================
                                                                        
initm   db 13,10
        db " VHERC (c) 1987,1988 by Michael Tischer "
        db 13,10,13,10
        db "Dieses Demonstrationsprogramm luft nur in Verbindung",13,10
        db "mit  einer Hercules Grafik-Karte  (HGC). Wenn Ihr  PC",13,10
        db "ber  eine  andere Art  von Bildschirmkarte  verfgt,",13,10
        db "dann  geben Sie jetzt  bitte <S> ein, um das Programm",13,10
        db "zu stoppen.",13,10
        db "Ansonsten  bettigen Sie bitte eine beliebige  andere",13,10
        db "Taste, um das Programm zu starten ...",13,10,"$"
                                                                        
str1    db 1,17,16,2,7,0
str2    db 2,16,17,1,7,0
                                                                        
domes     db 13,10
          db "Sie haben sich fr den Start des Programms ent-"
          db "schieden.",13,10
          db "Es wird nun zunchst eine kleine Grafikdemo und",13,10
          db "dann eine Textdemo gezeigt, die beide durch Be-",13,10
          db "ttigung einer beliebigen Taste beendet werden.",13,10
          db "Bitte bettigen Sie eine Taste zum Start...",13,10,"$"
                                                                        
;== Tabelle mit den Offsetadressen der Zeilenanfnge ===================
                                                                        
zeilen    dw  0*160, 1*160, 2*160, 3*160, 4*160, 5*160, 6*160
          dw  7*160, 8*160, 9*160,10*160,11*160,12*160,13*160
          dw 14*160,15*160,16*160,17*160,18*160,19*160,20*160
          dw 21*160,22*160,23*160,24*160
                                                                        
grafikt   db 35h, 2Dh, 2Eh, 07h, 5Bh, 02h  ;Register-Werte fr den
          db 57h, 57h, 02h, 03h, 00h, 00h  ;Grafik-Modus
                                                                        
textt     db 61h, 50h, 52h, 0Fh, 19h, 06h  ;Register-Werte fr den
          db 19h, 19h, 02h, 0Dh, 0Bh, 0Ch  ;Grafik-Modus
                                                                        
data      ends                    ;Ende des Daten-Segments
                                                                        
;== Code ===============================================================
                                                                        
code      segment para 'CODE'     ;Definition des CODE-Segments
                                                                        
          assume cs:code, ds:data, es:data, ss:stack
                                                                        
;== dies ist nur das Demo-Programm =====================================
                                                                        
demo      proc far
                                                                        
          mov  ax,data            ;Segmentadr. des Datensegments holen
          mov  ds,ax              ;und nach DS laden
          mov  es,ax              ;ebenfalls nach ES
                                                                        
          ;-- Init-Meldung ausgeben und auf Einabe warten --------------
                                                                        
          mov  ah,9               ;Funktionsnummer fr String ausgeben
          mov  dx,offset initm    ;Adresse der Meldung
          int  21h                ;DOS-Interrupt aufrufen
                                                                        
          xor  ah,ah              ;Funktionsnummer fr Taste holen
          int  16h                ;BIOS-Tastatur-Interrupt aufrufen
          cmp  al,"s"             ;wurde <s> eingegeben?
          je   ende               ;JA --> Programm wieder beenden
          cmp  al,"S"             ;wurde <S> eingeben
          jne  startdemo          ;NEIN --> Demo starten
                                                                        
ende:     mov  ax,4C00h           ;Funktionsnr. fr Programm beenden
          int  21h                ;DOS-Interrupt 21(h) aufrufen
                                                                        
          ;-- Demo-Programm, demonstriert Aufruf der Funktionen --------
                                                                        
startdemo label near
          mov  ah,9               ;Funktionsnummer fr String ausgeben
          mov  dx,offset domes    ;Adresse der Meldung
          int  21h                ;DOS-Interrupt aufrufen
                                                                        
          xor  ah,ah              ;Funktionsnummer fr Taste holen
          int  16h                ;BIOS-Tastatur-Interrupt aufrufen
                                                                        
          ;-- Grafikmodus initialisieren -------------------------------
                                                                        
          mov  al,11b             ;Grafik mglich und Seite 2 einblenden
          call konfig             ;konfigurieren
          xor  bp,bp              ;auf Bildschirmseite 0 zugreifen
          call grafik             ;in Grafikmodus umschalten
          xor  al,al
          call cgr                ;Grafikseite 0 lschen
                                                                        
          ;-- Grafik auf dem Bildschirm aufbauen -----------------------
                                                                        
          xor  bx,bx              ;in der oberen linken Bildschirm-
          xor  dx,dx              ;ecke beginnen
          mov  ax,347             ;Punkte vertikal
          mov  cx,719             ;Punkte horizontal
gr1:      push cx                 ;Punkte horizontal merken
          mov  cx,ax              ;Punkte vertikal in Counter
          push ax                 ;Punkte vertikal auf Stack merken
gr2:      call spix               ;Punkt setzen
          inc  dx                 ;Zeile inkrementieren
          loop gr2                ;Linie durchziehen
          pop  ax                 ;Punkte vertikal vom Stack holen
          sub  ax,3               ;nchste Linie 3 Punkte weniger
          pop  cx                 ;Punkte horizontal vom Stack holen
          push cx                 ;Punkte horizontal merken
          push ax                 ;Punkte vertikal auf Stack merken
gr3:      call spix               ;Punkt setzen
          inc  bx                 ;Spalte inkrementieren
          loop gr3                ;Linie durchziehen
          pop  ax                 ;Punkte vertikal vom Stack holen
          pop  cx                 ;Punkte horizontal vom Stack holen
          sub  cx,6               ;nchste Linie 6 Punkte weniger
          push cx                 ;Punkte horizontal merken
          mov  cx,ax              ;Punkte vertikal in Counter
          push ax                 ;Punkte vertikal auf Stack merken
gr4:      call spix               ;Punkt setzen
          dec  dx                 ;Zeile dekrementieren
          loop gr4                ;Linie durchziehen
          pop  ax                 ;Punkte vertikal vom Stack holen
          sub  ax,3               ;nchste Linie 3 Punkte weniger
          pop  cx                 ;Punkte horizontal vom Stack holen
          push cx                 ;Punkte horizontal merken
          push ax                 ;Punkte vertikal auf Stack merken
gr5:      call spix               ;Punkt setzen
          dec  bx                 ;Spalte inkrementieren
          loop gr5                ;Linie durchziehen
          pop  ax                 ;Punkte vertikal vom Stack holen
          pop  cx                 ;Punkte horizontal vom Stack holen
          sub  cx,6               ;nchste Linie 6 Punkte weniger
          cmp  ax,5               ;ist die Vertik.Linie lnger als 5
          ja   gr1                ;JA --> weitermachen
                                                                        
          xor  ah,ah              ;Funktionsnr. fr auf Taste warten
          int  16h                ;BIOS-Tastatur-Interrupt aufrufen
                                                                        
          ;-- Textmodus initialisieren ---------------------------------
                                                                        
          call text               ;Textmodus anschalten
          mov  cx,0d00h           ;Voll-Cursor anschalten
          call cdef
          call cls                ;Bildschirm lschen
                                                                        
          ;-- Strings in die Bildschirmseite 0 ausgeben ----------------
                                                                        
          xor  bx,bx              ;in der o.l. Bildschirmecke beginnen
          call calo               ;in Offsetadresse umrechnen
          mov  si,offset str1     ;Offsetadresse des String1
          mov  cx,16*25           ;der String ist 5 Zeichen lang
demo1:    call print              ;String ausgeben
          loop demo1
                                                                        
          ;-- Strings in die Bildschirmseite 1 ausgeben ----------------
                                                                        
          inc  bp                 ;Bildschirmseite 1 bearbeiten
          xor  bx,bx              ;in der o.l. Bildschirmecke beginnen
          call calo               ;in Offsetadresse umrechnen
          mov  si,offset str2     ;Offsetadresse des String1
          mov  cx,16*25           ;der String ist 5 Zeichen lang
demo2:    call print              ;String ausgeben
          loop demo2
                                                                        
demo3:    setmode 10001000b       ;Textseite 1 anzeigen
                                                                        
          ;-- kleine Pause ---------------------------------------------
                                                                        
          mov  cx,DELAY           ;Schleifenzhler laden
pause:    loop pause              ;bis 65536 zhlen
                                                                        
          setmode 00001000b       ;Textseite 0 anzeigen
                                                                        
          ;-- kleine Pause ---------------------------------------------
                                                                        
          mov  cx,DELAY           ;Schleifenzhler laden
pause1:   loop pause1             ;bis 65536 zhlen
                                                                        
          mov  ah,1               ;Funktionsnr. fr Taste testen
          int  16h                ;BIOS-Tastatur-Interrupt aufrufen
          je   demo3              ;keine Taste --> dann weiter
                                                                        
          ;-- Programmende vorbereiten ---------------------------------
                                                                        
          xor  ah,ah              ;Funktionsnummer fr Taste holen
          int  16h                ;BIOS-Tastatur-Interrupt aufrufen
                                                                        
          mov  bp,0               ;Bildschirmseite 1
          call cls                ;Bildschirm lschen
          mov  cx,0D0Ch           ;normalen Cursor wiederherstellen
          call cdef
          call cls                ;Bildschirm lschen
          jmp  ende               ;Programm beenden
                                                                        
demo      endp
                                                                        
;== jetzt folgen die eigentlichen Funktionen ===========================
                                                                        
;-- KONFIG: konfiguriert die HERCULES-Karte ----------------------------
;-- Eingabe  : AL : Bit 0 = 0 : nur Textdarstellung mglich
;--                         1 : auch Grafikdarstellung mglich
;--                 Bit 1 = 0 : RAM fr Bildschirmseite 2 ausgeblendet
;--                         1 : RAM fr Bildschirmseite 2 eingeblendet
;-- Ausgabe  : keine
;-- Register : AX und DX werden verndert
                                                                        
konfig    proc near
                                                                        
          mov  dx,KONFIG_REG      ;Adresse des Konfigurationsregister
          out  dx,al              ;neuen Wert setzen
          ret                     ;zurck zum Aufrufer
                                                                        
konfig    endp
                                                                        
;-- TEXT: schaltet die Textdarstellung an ------------------------------
;-- Eingabe  : keine
;-- Ausgabe  : keine
;-- Register : AX und DX werden verndert
                                                                        
text      proc near
                                                                        
          mov  si,offset textt    ;Offsetadresse der Register-Tabelle
          mov  bl,00100000b       ;Seite 0 darstellen,Textmodus,blinken
          jmp  short vcprog       ;Video-Controller neu programmieren
                                                                        
text      endp
                                                                        
;-- GRAFIK: schaltet die Grafikdarstellung an --------------------------
;-- Eingabe  : keine
;-- Ausgabe  : keine
;-- Register : AX und DX werden verndert
                                                                        
grafik    proc near
                                                                        
          mov  si,offset grafikt  ;Offsetadresse der Register-Tabelle
          mov  bl,00000010b       ;Seite 0 darstellen, Grafikmodus
                                                                        
grafik    endp
                                                                        
;-- VCPROG: programmiert den Video-Controller --------------------------
;-- Eingabe  : SI = Adresse einer Register-Tabelle
;--            BL = Wert fr Bildschirmsteuerregister
;-- Ausgabe  : keine
;-- Register : AX, SI, BH, DX und FLAGS werden verndert
                                                                        
vcprog    proc near
                                                                        
          setmode bl              ;Bit 3 = 0: Bildschirm aus
                                                                        
          mov  cx,12              ;12 Register werden gesetzt
          xor  bh,bh              ;mit Register 0 beginnen
vcp1:     lodsb                   ;Register-Wert aus Tabelle holen
          mov  ah,al              ;Register-Wert nach AH
          mov  al,bh              ;Nummer des Registers nach AL
          setvk                   ;Wert an Controller bermitteln
          inc  bh                 ;nchstes Register ansprechen
          loop vcp1               ;weitere Register setzen
                                                                        
          or   bl,8               ;Bit 3 = 1: Bildschirm an
          setmode bl              ;neuen Modus setzen
          ret                     ;zurck zum Aufrufer
                                                                        
vcprog    endp
                                                                        
;-- CDEF: setzt die Start- und Endzeile des Cursor ---------------------
;-- Eingabe  : CL = Startzeile
;--            CH = Endzeile
;-- Ausgabe  : keine
;-- Register : AX und DX werden verndert
                                                                        
cdef      proc near
                                                                        
          mov  al,CUR_START       ;Register: Startzeile des Cursor
          mov  ah,cl              ;Startzeile nach AH
          setvk                   ;an Video-Controller bermitteln
          mov  al,CUR_END         ;Register: Endzeile des Cursor
          mov  ah,ch              ;Endzeile nach AH
          setvk                   ;an Video-Controller bermitteln
          ret
                                                                        
cdef      endp
                                                                        
;-- SETBLINK: setzt den blinkenden Bildschirmcurosr --------------------
;-- Eingabe  : DI = Offsetadresse des Cursor
;-- Ausgabe  : keine
;-- Register : BX, AX und DX werden verndert
                                                                        
setblink  proc near
                                                                        
          mov  bx,di              ;Offset nach BX bertragen
          mov  al,CURPOS_HI       ;Register: Hi-Byte des Cursoroffset
          mov  ah,bh              ;HI-Byte des Offset
          setvk                   ;an Video-Controller bermitteln
          mov  al,CURPOS_LO       ;Register: Lo-Byte des Cursoroffset
          mov  ah,bl              ;Lo-Byte des Offset
          setvk                   ;an CRTC bermitteln
          ret
                                                                        
setblink  endp
                                                                        
;-- GETVK: liest ein Byte aus einem der Register des Video-Controllers -
;-- Eingabe  : AL = Nummer des Registers
;-- Ausgabe  : AL = Inhalt des Registers
;-- Register : DX und AL werden verndert
                                                                        
getvk     proc near
                                                                        
          mov  dx,ADRESS_6845     ;Adresse des Indexregisters
          out  dx,al              ;Nummer des Register senden
          jmp  $+2                ;kleine I/O Pause
          inc  dx                 ;Adresse des Indexregister
          in   al,dx              ;Inhalt nach AL lesen
          ret                     ;zurck zum Aufrufer
                                                                        
getvk     endp
                                                                        
;-- SCROLLUP: scrollt ein Fenster um N Zeilen nach oben ----------------
;-- Eingabe  : BL = Zeile oben links
;--            BH = Spalte oben links
;--            DL = Zeile unten rechts
;--            DH = Spalte unten rechts
;--            CL = Anzahl der Zeilen um die gescrollt wird
;--          : BP = Nummer der Bildschirmseite (0 oder 1)
;-- Ausgabe  : keine
;-- Register : nur FLAGS werden verndert
;-- Info     : die frei werdenden Bildschirmzeilen werden gelscht
                                                                        
scrollup  proc near
                                                                        
          cld                     ;bei Stringbefehlen aufwrts zhlen
                                                                        
          push ax                 ;alle vernderten Register auf dem
          push bx                 ;Stack sichern
          push di                 ;in diesem Fall mu die Reihenfolge
          push si                 ;beachtet werden!
                                                                        
          push bx                 ;diese drei Register werden schor vor
          push cx                 ;dem Ende der Routine wieder vom Stack
          push dx                 ;zurckgeholt
          sub  dl,bl              ;Anzahl Zeilen berechnen
          inc  dl
          sub  dl,cl              ;Anzahl der zu scr. Zeilen abziehen
          sub  dh,bh              ;Anzahl Spalten berechnen
          inc  dh
          call calo               ;oben links in Offset umrechnen
          mov  si,di              ;Adresse in SI merken
          add  bl,cl              ;erste Zeile im gescrollten Fenster
          call calo               ;erste Zeile in Offset umrechnen
          xchg si,di              ;SI und DI vertauschen
          push ds                 ;Segmentregister auf dem Stack
          push es                 ;sichern
          mov  ax,VIO_SEG         ;Segmentadresse des Video-RAM
          mov  ds,ax              ;nach DS
          mov  es,ax              ;und ES bringen
sup1:     mov  ax,di              ;DI in AX merken
          mov  bx,si              ;SI in BX merken
          mov  cl,dh              ;Anzahl Spalten in Counter
          rep movsw               ;eine Zeile verschieben
          mov  di,ax              ;DI aus AX zurckholen
          mov  si,bx              ;SI aus BX zurckholen
          add  di,160             ;jeweils in nchste Zeile setzen
          add  si,160
          dec  dl                 ;alle Zeilen verarbeitet?
          jne  sup1               ;NEIN --> noch eine Zeile verschieben
          pop  es                 ;Segmentregister wieder vom Stack
          pop  ds                 ;holen
          pop  dx                 ;untere rechte Ecke zurckholen
          pop  cx                 ;Anzahl Zeilen zurckholen
          pop  bx                 ;obere linke Ecke zurckholen
          mov  bl,dl              ;untere Zeile nach BL
          sub  bl,cl              ;Anzahl Zeilen abziehen
          inc  bl
          mov  ah,07h             ;Farbe : schwarz auf wei
          call clear              ;frei gewordene Zeilen lschen
                                                                        
          pop  si                 ;CX und DX sind bereits wieder her-
          pop  di                 ;untergeholt worden
          pop  bx
          pop  ax
                                                                        
          ret                     ;zurck zum Aufrufer
                                                                        
scrollup  endp
                                                                        
;-- SCROLLDN: scrollt ein Fenster um N Zeilen nach unten ---------------
;-- Eingabe  : BL = Zeile oben links
;--            BH = Spalte oben links
;--            DL = Zeile unten rechts
;--            DH = Spalte unten rechts
;--            CL = Anzahl der Zeilen um die gescrollt wird
;--          : BP = Nummer der Bildschirmseite (0 oder 1)
;-- Ausgabe  : keine
;-- Register : nur FLAGS werden verndert
;-- Info     : die frei werdenden Bildschirmzeilen werden gelscht
                                                                        
scrolldn  proc near
                                                                        
          cld                     ;bei Stringbefehlen aufwrts zhlen
                                                                        
          push ax                 ;alle vernderten Register auf dem
          push bx                 ;Stack sichern
          push di                 ;in diesem Fall mu die Reihenfolge
          push si                 ;beachtet werden!
                                                                        
          push bx                 ;diese drei Register werden schon vor
          push cx                 ;dem Ende der Routine wieder vom Stack
          push dx                 ;zurckgeholt
                                                                        
          sub  dh,bh              ;Anzahl Spalten berechnen
          inc  dh
          mov  al,bl              ;Zeile oben links in AL merken
          mov  bl,dl              ;Zeile u.r. nach Zeile u.l.
          call calo               ;oben links in Offset umrechnen
          mov  si,di              ;Adresse in SI merken
          sub  bl,cl              ;Anzahl zu scrollende Z. abziehen
          call calo               ;oben links in Offset umrechnen
          xchg si,di              ;SI und DI vertauschen
          sub  dl,al              ;Anzahl Zeilen berechnen
          inc  dl
          sub  dl,cl              ;Anzahl der zu scr. Zeilen abziehen
          push ds                 ;Segmentregister auf dem Stack
          push es                 ;sichern
          mov  ax,VIO_SEG         ;Segmentadresse des Video-RAM
          mov  ds,ax              ;nach DS
          mov  es,ax              ;und ES bringen
sdn1:     mov  ax,di              ;DI in AX merken
          mov  bx,si              ;SI in BX merken
          mov  cl,dh              ;Anzahl Spalten in Counter
          rep movsw               ;eine Zeile verschieben
          mov  di,ax              ;DI aus AX zurckholen
          mov  si,bx              ;SI aus BX zurckholen
          sub  di,160             ;jeweils in nchste Zeile setzen
          sub  si,160
          dec  dl                 ;alle Zeilen verarbeitet?
          jne  sdn1               ;NEIN --> noch eine Zeile verschieben
          pop  es                 ;Segmentregister wieder vom Stack
          pop  ds                 ;holen
          pop  dx                 ;untere rechte Ecke zurckholen
          pop  cx                 ;Anzahl Zeilen zurckholen
          pop  bx                 ;obere linke Ecke zurckholen
          mov  dl,bl              ;obere Zeile nach DL
          add  dl,cl              ;Anzahl Zeilen addieren
          dec  dl
          mov  ah,07h             ;Farbe : schwarz auf wei
          call clear              ;frei gewordene Zeilen lschen
                                                                        
          pop  si                 ;CX und DX sind bereits wieder her-
          pop  di                 ;untergeholt worden
          pop  bx
          pop  ax
                                                                        
          ret                     ;zurck zum Aufrufer
                                                                        
scrolldn  endp
                                                                        
;-- CLS: den gesamten Bildschirm lschen -------------------------------
;-- Eingabe  : BP = Nummer der Bildschirmseite (0 oder 1)
;-- Ausgabe  : keine
;-- Register : nur FLAGS werden verndert
                                                                        
cls       proc near
                                                                        
          mov  ah,07h             ;Farbe ist wei auf schwarz
          xor  bx,bx              ;oben links ist (0/0)
          mov  dx,4F18h           ;unten rechts ist (79/24)
                                                                        
          ;-- bergang zu Clear erfolgt automatisch --------------------
                                                                        
cls       endp
                                                                        
;-- CLEAR: fllt einen bestimmten Bildschirmbereich mit Leerzeichen ----
;-- Eingabe  : AH = Attribut/Farbe
;--            BL = Zeile oben links
;--            BH = Spalte oben links
;--            DL = Zeile unten rechts
;--            DH = Spalte unten rechts
;--            BP = Nummer der Bildschirmseite (0 oder 1)
;-- Ausgabe  : keine
;-- Register : nur FLAGS werden verndert
                                                                        
clear     proc near
                                                                        
          cld                     ;bei Stringbefehlen aufwrts zhlen
          push cx                 ;alle im nachfolgenden vernderten
          push dx                 ;Register auf dem Stack sichern
          push si
          push di
          push es
          sub  dl,bl              ;Anzahl Zeilen berechnen
          inc  dl
          sub  dh,bh              ;Anzahl Spalten berechnen
          inc  dh
          call calo               ;Offsetadresse der oberen linken Ecke
          mov  cx,VIO_SEG         ;Segmentadresse des Video-RAM
          mov  es,cx              ;nach ES
          xor  ch,ch              ;Hi-Bytes des Counter auf 0
          mov  al," "             ;Leerzeichen
clear1:   mov  si,di              ;DI in SI merken
          mov  cl,dh              ;Anzahl Spalten in Counter
          rep stosw               ;Leerzeichen speichern
          mov  di,si              ;DI aus SI zurckholen
          add  di,160             ;in nchste Zeile setzen
          dec  dl                 ;alle Zeilen verarbeitet?
          jne  clear1             ;NEIN --> noch eine Zeile lschen
                                                                        
          pop  es                 ;die gesicherten Register wieder
          pop  di                 ;vom Stack holen
          pop  si
          pop  dx
          pop  cx
          ret                     ;zurck zum Aufrufer
                                                                        
clear     endp
                                                                        
;-- PRINT: gibt einen String auf dem Bildschirm aus --------------------
;-- Eingabe  : AH = Attribut/Farbe
;--            DI = Offsetadresse des ersten Zeichen
;--            SI = Offsetadresse des Strings zu DS
;--            BP = Nummer der Bildschirmseite (0 oder 1)
;-- Ausgabe  : DI zeigt hinter das letzte ausgegebene Zeichen
;-- Register : AL, DI und FLAGS werden verndert
;-- Info     : der String mu durch das NUL-Zeichen beendet werden.
;--            andere Steuerzeichen werden nicht erkannt
                                                                        
print     proc near
                                                                        
          cld                     ;bei Stringbefehlen aufwrts zhlen
          push si                 ;SI, DX und ES auf dem Stack sichern
          push es
          push dx
          mov  dx,VIO_SEG         ;Segmentadresse des Video-RAM erst
          mov  es,dx              ;nach DX und dann nach ES
          jmp  print1             ;erstes Zeichen aus dem String holen
print0:   stosw                   ;Attribut und Farbe in V-RAM speichern
print1:   lodsb                   ;nchstes Zeichen aus dem String holen
          or   al,al              ;ist es NUL
          jne  print0             ;NEIN --> ausgeben
                                                                        
printe:   pop  dx                 ;SI, DX und ES wieder vom Stack holen
          pop  es
          pop  si
          ret                     ;zurck zum Aufrufer
                                                                        
print     endp
                                                                        
;-- CALO: rechnet Zeile und Spalte in Offsetadresse um -----------------
;-- Eingabe  : BL = Zeile
;--            BH = Spalte
;--            BP = Nummer der Bildschirmseite (0 oder 1)
;-- Ausgabe  : DI = die Offsetadresse
;-- Register : DI und FLAGS werden verndert
                                                                        
calo      proc near
                                                                        
          push ax                 ;AX auf dem Stack sichern
          push bx                 ;BX auf dem Stack sichern
                                                                        
          shl  bx,1               ;Spalte und Zeile mal 2
          mov  al,bh              ;Spalte nach AL
          xor  bh,bh              ;Hi-Byte
          mov  di,[zeilen+bx]     ;Offsetadresse der Zeile holen
          xor  ah,ah              ;HI-Byte fr Spaltenoffset
          add  di,ax              ;Zeilen- und Spaltenoffset addieren
          or   bp,bp              ;Bildschirmseite 0?
          je   caloe              ;JA --> Adresse ok
                                                                        
          add  di,8000h           ;32 KB fr Bildschirmseite 1 addieren
                                                                        
caloe:    pop  bx                 ;BX wieder vom Stack holen
          pop  ax                 ;AX wieder vom Stack holen
          ret                     ;zurck zum Aufrufer
                                                                        
calo      endp
                                                                        
;-- CGR: den gesamten Grafikbildschirm lschen -------------------------
;-- Eingabe  : BP = Nummer der Bildschirmseite (0 oder 1)
;--            AL = 00(h) : alle Punkte lschen
;--                 FF(h) : alle Punkte setzen
;-- Ausgabe  : keine
;-- Register : AH, BX, CX, DI und FLAGS werden verndert
                                                                        
cgr       proc near
                                                                        
          push es                 ;ES auf dem Stack sichern
          cbw                     ;AL auf AH ausdehnen
          xor  di,di              ;Offsetadresse in Video-RAM
          mov  bx,VIO_SEG         ;Segmentadresse Bildschirmseite 0
          or   bp,bp              ;Seite 1 lschen?
          je   cgr1               ;NEIN --> Seite 0 lschen
                                                                        
          add  bx,0800h           ;Segmentadresse Bildschirmseite 1
                                                                        
cgr1:     mov  es,bx              ;Segmentadresse ins Segmentregister
          mov  cx,4000h           ;eine Seite sind 16KB-Words
          rep stosw               ;Seite fllen
          pop  es                 ;ES vom Stack zurckholen
          ret                     ;zurck zum Aufrufer
                                                                        
cgr       endp
                                                                        
;-- SPIX: setzt einen Punkt im Grafikbildschirm ------------------------
;-- Eingabe  : BP = Nummer der Bildschirmseite (0 oder 1)
;--            BX = Spalte (0 bis 719)
;--            DX = Zeile  (0 bis 347)
;-- Ausgabe  : keine
;-- Register : AX, DI und FLAGS werden verndert
                                                                        
spix      proc near
                                                                        
          push es                 ;ES auf dem Stack sichern
          push bx                 ;BX auf dem Stack sichern
          push cx                 ;CX auf dem Stack sichern
          push dx                 ;DX auf dem Stack sichern
                                                                        
          xor  di,di              ;Offsetadresse in Video-RAM
          mov  cx,VIO_SEG         ;Segmentadresse Bildschirmseite 0
          or   bp,bp              ;auf Seite 1 zugreifen?
          je   spix1              ;NEIN --> auf Seite 0 zugreifen
                                                                        
          add  cx,0800h           ;Segmentadresse Bildschirmseite 1
                                                                        
spix1:    mov  es,cx              ;Segmentadresse ins Segmentregister
          mov  ax,dx              ;Zeile nach AX holen
          shr  ax,1               ;Zeile 2 mal nach rechts schieben
          shr  ax,1               ;und dadurch durch 4 teilen
          mov  cl,90              ;der Faktor ist 90
          mul  cl                 ;Zeile mit 90 multiplizieren
          and  dx,11b             ;bis auf Bit 0 und 1 alle Bits ausbl.
          mov  cl,3               ;3 Verschiebungen
          ror  dx,cl              ;rechts herum rotieren (* 2000(h))
          mov  di,bx              ;Spalte nach DI
          mov  cl,3               ;3 Verschiebungen
          shr  di,cl              ;durch 8 teilen
          add  di,ax              ;+ 90 * int(Zeile/4)
          add  di,dx              ;+ 2000(h) * (Zeile mod 4)
          mov  cl,7               ;maximal 7 Verschiebungen
          and  bx,7               ;Spalte mod 8
          sub  cl,bl              ;7 - Spalte mod 8
          mov  ah,1               ;Bit Wert des Punktes ermitteln
          shl  ah,cl
          mov  al,es:[di]         ;8 Punkte holen
          or   al,ah              ;Punkt setzen
          mov  es:[di],al         ;8 Punkte zurckschreiben;
                                                                        
          pop  dx                 ;DX wieder vom Stack holen
          pop  cx                 ;CX wieder vom Stack holen
          pop  bx                 ;BX wieder vom Stack holen
          pop  es                 ;ES wieder vom Stack holen
          ret                     ;zurck zum Aufrufer
                                                                        
spix      endp
                                                                        
;== Ende ===============================================================
                                                                        
code      ends                    ;Ende des CODE-Segments
          end  demo               ;Programmausfhrung mit DEMO beginnen
                                                                        
