'*****************************************************************************
'*                                 TASTB                                     *
'-----------------------------------------------------------------------------
'*  Aufgabe : stellt eine Funktion zur Verfgung, mit deren Hilfe ein        *
'*            Zeichen von der Tastatur eingelesen werden kann, wobei der     *
'*            Status der Umschalttasten INSERT, CAPS und NUM auf dem Bild-   *
'*            schirm ausgegeben wird                                         *
'-----------------------------------------------------------------------------
'* Autor          : MICHAEL TISCHER                                          *
'* entwickelt am  : 09.06.1991                                               *
'* letztes Update : 01.01.1992                                               *
'*****************************************************************************
'
DECLARE SUB Printinvers (Text AS STRING)
DECLARE SUB WriteChar (Zeichen AS INTEGER, Farbe AS INTEGER)
DECLARE FUNCTION GetSeite% ()
DECLARE SUB Inikey ()
DECLARE FUNCTION NegFlag% (Flag%, FlagReg%, Spalte%, Zeile%, Text AS STRING)
DECLARE FUNCTION GetKey% ()

'$INCLUDE: 'QB.BI'              'Include-Datei enthlt Register-Deklarationen
																		    
CONST TRUE = -1                                     'Wahrheitswerte definieren
CONST FALSE = NOT TRUE
																		    
CONST FZ = 1                         'Zeile in der die Flags Ausgegeben werden
CONST FS = 65                      'Spalte, ab der die Flags ausgegeben werden
																		    
'-- Bits des BIOS-Tastatur-Status --------------------------------------------
CONST SCRL = 16                                               'Scroll-Lock-Bit
CONST NUML = 32                                                  'Num-Lock-Bit
CONST CAPL = 64                                                 'Caps-Lock-Bit
CONST INS = 128                                                    'Insert-Bit
																		    
'-- Codes einiger Tasten, wie sie GETKEY liefert -----------------------------
																		    
CONST BEL = 7                                        'Code des Klingelzeichens
CONST BS = 8                                         'Code der Backspace-Taste
CONST TB = 9                                         'Code der Tabulator-Taste
CONST LF = 10                                              'Code des Line-Feed
CONST CR = 13                                           'Code der Return-Taste
CONST ESC = 27                                          'Code der Escape-Taste
CONST F1 = 315                                                'Funktionstasten
CONST F2 = 316
CONST F3 = 317
CONST F4 = 318
CONST F5 = 319
CONST F6 = 320
CONST F7 = 321
CONST F8 = 322
CONST F9 = 323
CONST F10 = 324
CONST CUP = 328                                             'Cursor-Tasten   
CONST CLEFT = 331                                        'Code des Cursor-Left
CONST CRIGHT = 333                                      'Code des Cursor-Right
CONST CDOEN = 329                                        'Code des Cursor-Down
																		    
'-- globale Variablen --------------------------------------------------------

DIM SHARED Insert AS INTEGER                          'Status der Insert-Taste
DIM SHARED Caps   AS INTEGER                       'Status der Caps-Lock-Taste
DIM SHARED Num    AS INTEGER                        'Status der Num-Lock-Taste
																		    
'-- Hauptproramm --------

DIM Taste AS INTEGER                         'nimmt ASCII-Code einer Taste auf
																	     
Inikey                                           'Tastaturflags initialisieren
CLS                                                        'Bildschirm lschen
PRINT "TASTB  (C) 1987, 91 by MICHAEL TISCHER"
PRINT
PRINT "Sie knnen nun einige Zeichen eingeben und dabei den ";
PRINT "INSERT-, CAPS- oder NUM-"
PRINT "Modus an- oder ausschalten. Der Status der Umschaltmodi wird";
PRINT " jederzeit in "
PRINT "der oberen rechten Bildschirmecke angezeigt"
PRINT "Die Bettigung der <RETURN> oder der <F1>-Taste beendet ";
PRINT "das Programm..."
PRINT : PRINT "Ihre Eingabe : ";
DO                                                            'Eingabeschleife
  Taste = GetKey                                               'Taste einlesen
  IF Taste < 256 THEN                         'Kein erweiterter Tastaturcode ?
    PRINT (CHR$(Taste));                                 'Ausgabe des Zeichens
  END IF
LOOP UNTIL (Taste = CR) OR (Taste = F1)        'wiederholen bis <F1> oder <CR>
END

'*****************************************************************************
'* GetKey  : Einlesen eines Zeichens und Ausgabe des Flag-Status             *
'* Einhabe : keine                                                           *
'* Ausgabe : Code der empfangenen Taste                                      *
'*                   < 256 : normale Taste                                   *  
'*                  >= 256 : erweiterter Tastaturcode                        *
'*****************************************************************************
'
FUNCTION GetKey%

DIM Reg AS RegType                      'Prozessorregister fr Interruptaufruf

DO
  Reg.ax = &H200                     'Funktionsnummer fr Tastaturstatus lesen
  CALL INTERRUPT(&H16, Reg, Reg)             'BIOS-Tastatur-Interrupt aufrufen

  '-- Flags dem neuen Status anpassen----------------------------------------
  Insert = NegFlag(Insert, ((Reg.ax MOD 256) AND INS), FS + 9, FZ, "INSERT")
  Caps = NegFlag(Caps, ((Reg.ax MOD 256) AND CAPL), FS + 3, FZ, " CAPS ")
  Num = NegFlag(Num, ((Reg.ax MOD 256) AND NUML), FS, FZ, "NUM")

  Reg.ax = &H100                         'Funktionsnummer fr Zeichen bereit ?
  CALL INTERRUPT(&H16, Reg, Reg)             'BIOS-Tastatur-Interrupt aufrufen
  IF (Reg.flags AND 64) = 0 THEN                           'Taste steht bereit
    Reg.ax = &H0                           'Funktionsnummer fr Taste einlesen
    CALL INTERRUPT(&H16, Reg, Reg)           'BIOS-Tastatur-Interrupt aufrufen
    IF (Reg.ax MOD 256) = 0 THEN                    'ist es erweiterte Taste ?
      GetKey = (Reg.ax \ 256) OR &H100                'es ist erweiterte Taste
    ELSE                                        'es ist keine erweiterte Taste
      GetKey = (Reg.ax MOD 256)
    END IF
    EXIT DO                                            'die Schleife verlassen
  END IF
LOOP                                          'wiederholen bis Taste empfangen
END FUNCTION

'*****************************************************************************
'*  GetSeite : holt die aktuelle Bildschirmseite                             *
'*  Eingabe  : keine                                                         *
'*  Ausgabe  : die aktuelle Bildschirmseite                                  *
'*****************************************************************************
'
FUNCTION GetSeite%
																		    
DIM Register AS RegType                 'Prozessorregister fr Interruptaufruf
																		    
Register.ax = &H1500                                     'AH = Funktionsnummer
CALL INTERRUPT(&H10, Register, Register)              'BIOS-Interrupt aufrufen
GetSeite = Register.bx \ 256              'Register.BH liefert Bildschirmseite

END FUNCTION

'*****************************************************************************
'*  IniKey  : initialisiert die Tastatur-Flags                               *
'*  Eingabe : keine                                                          *
'*  Ausgabe : keine                                                          *
'*  Info    : Die Tastaturflags werden gegenber ihrem aktuellen             *
'*            Status umgedreht. Dadurch wird erreicht, da ihr aktu-         *
'*            eller Status beim nchsten Aufruf der GETKEY-Funktion          *
'*            ausgegeben wird.                                               *
'*****************************************************************************
'
SUB Inikey
																	     
SHARED Insert AS INTEGER                              'Status der Insert-Taste
SHARED Caps   AS INTEGER                           'Status der Caps-Lock-Taste
SHARED Num    AS INTEGER                            'Status der Num-Lock-Taste

DIM Register AS RegType                 'Prozessorregister fr Interruptaufruf
																		    
Register.ax = &H200                  'Funktionsnummer fr Tastaturstatus lesen
CALL INTERRUPT(&H16, Register, Register)     'BIOS-Tastatur-Interrupt aufrufen
																		    
IF (Register.ax AND INS) THEN                              'Insert-Flag setzen
   Insert = FALSE
ELSE
   Insert = TRUE
END IF

IF (Register.ax AND CAPL) THEN                          'Caps-Lock-Flag setzen
   Caps = FALSE
ELSE
   Caps = TRUE
END IF

IF (Register.ax AND NUML) THEN                           'Num-Lock-Flag setzen
   Num = FALSE
ELSE
   Num = TRUE
END IF

END SUB

'*****************************************************************************
'*  NegFlag : Evtl Flag negieren und zugehrigen Text ausgeben               *
'*  Eingabe : s.u.                                                           *
'*  Ausgabe : der neue Status des Flags (True = an, False = aus )            *
'*****************************************************************************
'
FUNCTION NegFlag% (Flag%, FlagReg%, Spalte%, Zeile%, Text AS STRING)
																		    
DIM AktZeile AS INTEGER                 'speichert die aktuelle Cursorposition
DIM AktSpalte AS INTEGER                'speichert die aktuelle Cursorposition
																		    
'-- testen, ob sich der Status verndert hat ------

IF Flag% AND (FlagReg% = 0) OR (NOT Flag%) AND (FlagReg% <> 0) THEN
  AktZeile = CSRLIN                                 'Ja: aktuelle Zeile Merken
  AktSpalte = POS(0)                                   'aktuelle Spalte merken
  LOCATE Zeile%, Spalte%                   'Cursor auf Position fr Flag-Namen
  IF FlagReg% = 0 THEN                                      'wenn Flag aus ist
    NegFlag = FALSE                          'Ergebnis der Funktion : Flag aus
    PRINT SPACE$(LEN(Text))                               'Flaganzeige lschen
  ELSE                                                            'Flag ist an
    NegFlag% = TRUE                           'Ergebnis der Funktion : Flag an
    Printinvers (Text)                                'Name des Flags ausgeben
  END IF
  LOCATE AktZeile, AktSpalte                'Alte Corsorposition wieder setzen
ELSE                                     ' der Status hat sich nicht verndert
  NegFlag% = Flag%                           'Status des Flags bleibt erhalten
END IF

END FUNCTION

'*****************************************************************************
'* PrintInvers : schreibt eine Zeichenkette in inverser Darstellung an       *
'*               die aktuelle Position der aktuellen Bildschirmseite         *
'* Eingabe     : s.u.                                                        *
'* Ausgabe     : keine                                                       *
'*****************************************************************************
'
SUB Printinvers (Text AS STRING)
																		    
CONST INVERS = &H70                      'Attributbyte fr inverse Darstellung
DIM Zaehler AS INTEGER                                        'Schleifenzhler
																		    
FOR Zaehler = 1 TO LEN(Text)                 'Alle Zeichen des Textes ausgeben
   CALL WriteChar(ASC(MID$(Text, Zaehler, 1)), INVERS)
   LOCATE CSRLIN, POS(0) + 1                'Cursor eine Position weiterrcken
NEXT

END SUB

'*****************************************************************************
'*  WriteChar : schreibt ein Zeichen mit einem vorgegebenen Attribut         *
'*              an die aktuelle Cursorposition der aktuellen B.S.Seite       *
'*  Eingabe   : s.u.                                                         *
'*  Ausgabe   : keine                                                        *
'*****************************************************************************
'
SUB WriteChar (Zeichen AS INTEGER, Farbe AS INTEGER)
																		    
DIM Register AS RegType                 'Prozessorregister fr Interruptaufruf
																		    
Register.ax = &H9 * 256 + Zeichen             'AH = Funkt.Nr Zeichen schreiben
											   'AL = ASCII-Code des Zeichens
Register.bx = GetSeite * 256 + Farbe    'BH = Akt. Bildschirmseite; BL = Farbe
Register.cx = 1                                   'Zeichen nur einmal ausgeben
CALL INTERRUPT(&H10, Register, Register)              'BIOS-Interrupt aufrufen

END SUB

