/**********************************************************************/
/*                   A R G S . C                                      */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Funktionen zur Bearbeitung von Kommandozeilen-   */
/*                   Parametern                                       */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 20.03.1994                                       */
/*  letztes Update : 25.03.1994                                       */
/**********************************************************************/
#ifndef __ARGS_C
#define __ARGS_C

/*- Include Dateien einbinden ----------------------------------------*/
#include <string.h>
#include <stdlib.h>
#include "types.h"
#include "args.h"

/**********************************************************************/
/* GetArg : Ermittle Kommandozeilenparameter                          */
/**-------------------------------------------------------------------*/
/* Eingabe : argc - Argument-Count (s. main(INT argc, CHAR *argv[]))  */
/*           argv - Argument-Values (s. main(INT argc, CHAR *argv[])) */
/*           pPrefix - Parameter-Prefix (z.B: "-o" fr Output)        */
/*           iType - Typ des Parameters (_char, _int, _long, _string, */
/*                   _none = testet ob Parameter vorhanden)           */
/*           pVar - Adresse der Variablen, die Kommandozeilen-        */
/*                   parameterwert entgegenenehmen soll.              */
/*           iNumElements - Falls mehrere Werte durch ',' getrennt    */
/*                          werden, wird bis zu iNumElements          */
/*                          Parameter im in pVar angegebenen Array    */
/*                          gespeichert.                              */
/* Ausgabe : Anzahl der ermittelten Werte, oder 0 falls Kommando-     */
/*           parameter nicht vorhanden.                               */
/**********************************************************************/
INT GetArg( INT argc, PCHAR argv[], PCHAR pPrefix, INT iType,
            PVOID pVar, INT iNumElements )
{
  INT i, j;
  INT iLen;

  PCHAR  cPtr;
  PINT   iPtr;
  PLONG  Ptr;
  PCHAR *sPtr;

  iLen = _fstrlen( pPrefix );

  for( i = 1; i < argc; i++ )
    if( _fstrnicmp( pPrefix, ( LPCHAR )argv[ i ], iLen ) == 0 )
      switch( iType )
      {
        case _int:
          iPtr = (PINT)pVar;
          for( j = 0; ( j < iNumElements ) && argv[ i ][ iLen ]; j++ )
          {
            *iPtr = atoi( ( PCHAR )&argv[ i ][ iLen ] );
            while( argv[ i ][ iLen ] && ( argv[ i ][ iLen ] != ',' ) )
              iLen++;
            if( argv[ i ][ iLen ] == ',' ) iLen++;
            iPtr++;
          }
          return j;

        case _char:
          cPtr = (PCHAR)pVar;
          for( j = 0; ( j < iNumElements ) && argv[ i ][ iLen ]; j++ )
          {
            *cPtr = argv[ i ][ iLen ];
            while( argv[ i ][ iLen ] && ( argv[ i ][ iLen ] != ',' ) )
              iLen++;
            if( argv[ i ][ iLen ] == ',' ) iLen++;
            cPtr++;
          }
          return j;

        case _long:
          Ptr = (PLONG)pVar;
          for( j = 0; ( j < iNumElements ) && argv[ i ][ iLen ]; j++ )
          {
            *Ptr = atol( ( PCHAR )&argv[ i ][ iLen ] );
            while( argv[ i ][ iLen ] && ( argv[ i ][ iLen ] != ',' ) )
              iLen++;
            if( argv[ i ][ iLen ] == ',' ) iLen++;
            Ptr++;
          }
          return j;

        case _string:
          sPtr = ( PCHAR *)pVar;
          for( j = 0; ( j < iNumElements ) && argv[ i ][ iLen ]; j++ )
          {
            *sPtr = &argv[ i ][ iLen ];
            while( argv[ i ][ iLen ] && ( argv[ i ][ iLen ] != ',' ) )
              iLen++;
            if( argv[ i ][ iLen ] == ',' ) argv[ i ][ iLen++ ] = '\0';
            sPtr++;
          }
          return j;

        case _none:
          return TRUE;
      }
  return FALSE;
}

/**********************************************************************/
/* GetNArg : Ermittle Kommandozeilenparameter ohne Prefix             */
/**-------------------------------------------------------------------*/
/* Eingabe : argc - Argument-Count (s. main(INT argc, CHAR *argv[]))  */
/*           argv - Argument-Values (s. main(INT argc, CHAR *argv[])) */
/*           pPrefix - Parameter-Prefix (z.B: "-o" fr Output), das   */
/*                     NICHT in den zu findenden Parameter enthalten  */
/*                     sein darf.                                     */
/*           pString - Adresse des String-Zeiger Arrays, das die      */
/*                     Parameter aufnimmt.                            */
/*           iNumElements - Falls mehrere Parameter in der Kommando-  */
/*                          zeile eenthalten sind, werden nur max.    */
/*                          Parameter im iNumElements abgespeichert.  */
/* Ausgabe : Anzahl der ermittelten Parameter, oder 0 falls Kommando- */
/*           parameter nicht vorhanden.                               */
/**********************************************************************/
INT GetNArg( INT argc, PCHAR argv[], PCHAR pPrefix, 
             PCHAR *pString, INT iNumElements )
{
  INT i, j;
  INT iLen;

  iLen = _fstrlen( pPrefix );

  for( i = 1, j = 0; ( i < argc ) && ( j < iNumElements ); i++ )
    if( _fstrnicmp( pPrefix, ( PCHAR )argv[ i ], iLen ) != 0 )
      pString[ j++ ] = argv[ i ];

  return j;
}

/**********************************************************************/
/* FindString : Suche String in einem String-Array und gib Position   */
/*              des gefundenen Strings innerhalb des Arrays zurck.   */
/**------------------------------------------------------------------**/
/* Eingabe : pArray  - zu durchsundendes String-Array                 */
/*           pFind   - zu suchender String                            */
/*           iNum    - Anzahl der Strings in String-Array             */
/* Ausgabe : Position + 1 des gefundenen Strings in Array oder 0      */
/*           falls Suchstring nicht in Array vorhanden.               */
/**********************************************************************/
INT FindString( PCHAR pArray[], PCHAR pFind, INT iNum )
{ INT i;
  for( i = 0; i < iNum; i++ )
   if( _fstricmp( pArray[ i ], pFind ) == 0 ) return i + 1;
  return 0;
}

/**********************************************************************/
/* htoi : Wandelt bergebenen Hex-String in Zahlenwert                */
/**------------------------------------------------------------------**/
/* Eingabe : pStr - Adresse des zu wandelnden Strings                 */
/*           iDef - Vorgabewert                                       */
/* Ausgabe : Wert des Strings oder 'iDef' falls bergebener String    */
/*           keine gltige Hex-Zahl darstellt.                        */
/**********************************************************************/
INT htoi( PCHAR pStr, INT iDef )
{
  INT iNumChars = 0;
  INT iVal = 0;

  if( pStr )
  {
    while( *pStr == ' ' ) pStr ++;           /* Leerzeichen bergehen */
    for(;;)
    {
      switch( *pStr )
      {
        case '0':   case '1':   case '2':   /* Dezimalziffern wandeln */
        case '3':   case '4':   case '5':
        case '6':   case '7':   case '8':
        case '9':
          iVal *= 0x10;       /* Wert eine stelle nach links schieben */
          iVal += *(pStr++) - '0';         /* 'Einer'-Stelle addieren */
          iNumChars++;          /* Anzahl gewandelter Zeichen erhhen */
        break;
        case 'a':   case 'b':   case 'c':
        case 'd':   case 'e':   case 'f':
          iVal *= 0x10;       /* Wert eine stelle nach links schieben */
          iVal += *(pStr++) - 'a' + 10;    /* 'Einer'-Stelle addieren */
          iNumChars++;          /* Anzahl gewandelter Zeichen erhhen */
        break;
        case 'A':   case 'B':   case 'C':
        case 'D':   case 'E':   case 'F':
          iVal *= 0x10;       /* Wert eine stelle nach links schieben */
          iVal += *(pStr++) - 'A' + 10;    /* 'Einer'-Stelle addieren */
          iNumChars++;          /* Anzahl gewandelter Zeichen erhhen */
        break;
        default:
           return ( iNumChars != 0 ) ? iVal : iDef;
      }
    }
  }
  return iDef;
}

/**********************************************************************/
/* strichr :  Suche erstes Vorkommen eines Teilstrings ohne Gro-/    */
/*            Kleinschreibung zu bercksichtigen.                     */
/**------------------------------------------------------------------**/
/* Eingabe : pStr    - Adresse des zu durchsundenden Strings          */
/*           pSearch - Zu suchender Teilstring                        */
/* Ausgabe : Adresse des Zeichens, das auf das erste Vorkommen des    */
/*           zu suchenden Teilstrings folgt.                          */
/**********************************************************************/
PCHAR strichr( PCHAR pStr, PCHAR pSearch )
{
  if( pStr && pSearch )
  { INT iLen;                                /* Lnge des Suchstrings */

    iLen = _fstrlen( pSearch );  /* Modelunabhngige Stringfunktionen */
    while( *pStr )      /* zu durchsuchender String noch nicht leer ? */
      if( _fstrnicmp( pStr, pSearch, iLen ) == 0 ) 
      {
        /*- Adresse des Zeichen nach gefundenem Suchstring bergeben -*/
        pStr += iLen;  
        if( *pStr ) return pStr;                
      }
      else pStr++;
  }
  return NULL;                           /* Suchstring nicht gefunden */
}

#endif
