/*****************************************************************************
*                           D B L S P C P . C                                *
*----------------------------------------------------------------------------*
*     Aufgabe        : Zeigt Informationen ber alle aktiven Double-         *
*                      Space-Laufwerke an.                                   *
*----------------------------------------------------------------------------*
*     Autor          : MICHAEL TISCHER                                       *
*     entwickelt am  : 23.09.1993                                            *
*     letztes Update : 26.09.1993                                            *
******************************************************************************/

#include <dos.h>
#include <stdio.h>
#include <process.h>

typedef unsigned char BYTE;
#define TRUE 1
#define FALSE 0
typedef unsigned int  BOOL;

/*****************************************************************************
* IsDoubleSpaceInstalled: berprft, ob DoubleSpace installiert ist und      *
*                         liefert Informationen ber DoubleSpace             *
*----------------------------------------------------------------------------*
* Eingabe : pFirstDrive = enthlt nach dem Aufruf die erste fr DoubleSpace  *
*                         verfgbare Gertebezeichnung (65 = A:)             *
*           pNumDrives  = enthlt nach dem Aufruf die Anzahl der fr Double- *
*                         Space reservierten Gertebezeichnungen             *
*           pVerNum     = enthlt nach dem Aufruf die interne DoubleSpace    *
*                         Versionsnummer                                     *
*           pUpperMem   = enthlt nach dem Aufruf TRUE, wenn DoubleSpace     *
*                         im Upper-Memory residiert                          *
* Ausgabe : TRUE wenn DoubleSpace installiert ist, sonst FALSE               *
*****************************************************************************/
                                                                        
BOOL IsDoubleSpaceInstalled( BYTE *pFirstDrive,
                             BYTE *pNumDrives,
                             BYTE *pVerNum,
                             BOOL *pUpperMem ) 

{ union REGS Regs;
  
  Regs.x.ax = 0x4A11;                             /* MUX-Code fr DoubleSpace */
  Regs.x.bx = 0;                                            /* Funtionsnummer */
  int86( 0x2F, &Regs, &Regs );                        /* Multiplexer aufrufen */

  /*-- Informationen aus Prozessorregistern holen ----------------------------*/
  *pFirstDrive = Regs.h.cl;                /* Nummer des ersten DoubleSpace-Lw */
  *pNumDrives = Regs.h.ch;                        /* Anzahl der DoubleSpace-Lw */
  *pVerNum = (Regs.x.dx & 0x7FFF);                   /* interne Versionsnummer */
  *pUpperMem = (Regs.x.dx & 0x8000) == 0;                  /* im Upper-Memory? */
  return (Regs.x.ax == 0);
}

/*****************************************************************************
* IsDoubleSpaceDrive: berprft, ob es sich bei einem gegebenen Laufwerk     *
*                     um ein DoubleSpace-Laufwerk handelt und liefert        *
*                     Informationen ber dieses Laufwerk zurck              *
*----------------------------------------------------------------------------*
* Eingabe : pLW         = Gertebezeichnug des zu berprf}en Laufwerk       *
*                        (0 = A:, 1 = B: usw)                                *
*           pVertauscht = enthlt nach dem Funktionsaufruf TRUE, wenn es     *
*                         sich um ein komprimiertes Laufwerk handelt, das    *
*                         das mit seinem Host-Laufwerk getauscht wurde       *
*           pHostNr     = enthlt nach dem Funktionsaufruf die Gerte-       *
*                         bezeichnung des Host-Laufwerks, sofern  es sich    *
*                         um ein DoubleSpace-Laufwerk handelt                *
*           pCvfNr      = enthlt nach dem Funktionsaufruf die Nummer der    *
*                         CVF-Datei, sofern es sich um ein DoubleSpace-      *
*                         Laufwerk handelt                                   *
* Ausgabe : TRUE, wenn es sich um ein DoubleSpace-Laufwerk handelt, sonst    *
*           FALSE                                                            *
******************************************************************************/

BOOL IsDoubleSpaceDrive( BYTE bLW,
                         BOOL *pVertauscht,
                         BYTE *pHostLw,
                         BYTE *pCvfNr )
{
 BYTE bCvfNr,                              /* lokale Variablen, nehmen zunchst */
      bHostLw;
 BOOL bVertauscht,
      bIsDoubleSpace;

  /*-- zunchst von einem unkomprimierten, nicht vertauschten Laufwerk -------*/
  /*-- ausgehen                                                        -------*/

  bHostLw = bLW;
  bVertauscht = FALSE;
  bIsDoubleSpace = FALSE;
  bCvfNr = 0;

  _asm 
  {
    mov     ax,4A11h                           /* DoubleSpace-Funktion 00001h */
    mov     bx,0001h                           /* aufrufen                    */
    mov     dl,bLW
    int     2Fh
    or      ax,ax                                      /* Aufruf erfolgreich? */
    jnz     idbende                   /* Nein, DoubleSpace nicht installiert */

    /*-- Aufruf erfolgreich --------------------------------------------------*/
    test    bl,80h                                 /* komprimiertes Laufwerk? */
    jz      idbHostLw                       /* nein, eventuell Host-Laufwerk */

    /*-- komprimiertes Laufwerk, jetzt Host-Laufwerk ermitteln ---------------*/
    mov     bIsDoubleSpace,TRUE
    mov     bCvfNr,bh                          /* Nummer der CVF-Datei merken */

    and     bl,7Fh                 /* Nummer des Host-Laufwerks herausfiltern */
    mov     bHostLw,bl                                          /* und merken */

    mov     dl,bl                         /* Funktion 0001h mit Host-Laufwerk */
    mov     ax,4A11h                                       /* erneut aufrufen */
    mov     bx,0001h
    int     2Fh

    and     bl,7Fh                        /* Nummer des Hostlaufwerks filtern */
    cmp     bl,bLW                         /* ist der Host sein eigener Host? */
    mov     bVertauscht,TRUE                  /* von vertauschtem Lw ausgehen */
    je      idbende                                /* vertauscht --> idb}e */

    mov     bVertauscht,FALSE                /* Laufwerk ist nicht vertauscht */
    mov     bHostLw,bl       
    jmp     idbende

    /*-- es handelt sich um ein nicht komprimiertes Host-Laufwerk ------------*/
    idbHostLw:
    and     bl,7Fh                       /* Host-Laufwerksbezeichnung filtern */
    cmp     bl,dl                           /* wurde das Laufwerk vertauscht? */
    je      idbende                                     /* Nein ---> idb}e */

    mov     bVertauscht,TRUE                                            /* Ja */
    mov     bHostLw,bl                      /* echte Gertebezeichnung setzen */

    idbende:
  };                                                               /* ASM */

    *pHostLw   = bHostLw;                       /* Ergebnisse in Variablen des */
    *pVertauscht = bVertauscht;                 /* Aufrufers bernehmen        */
    *pCvfNr      = bCvfNr;
    return bIsDoubleSpace;
};

/*-- Variablen fr das Hauptprogramm -----------------------------------------*/

BYTE i,                                                     /* Schleifenzhler */
    vernum;                                        /* DoubleSpace Versionsnr. */
BYTE firstdrive,                                /* erstes DoubleSpace-Laufwerk */
    numdrive,                                 /* Anzahl DoubleSpace-Laufwerke */
    host,                                           /* nimmt Hostlaufwerk auf */
    cvfnr;                                /* nimmt Cvf-Nummer auf */
BOOL isdbl,                                           /* DoubleSpace-Laufwerk? */
    uppermem,                                 /* DoubleSpace im Upper-Memory? */
    vertauscht;                     /* mit Hostlaufwerk vertauscht? */

/*----------------------------------------------------------------------------*/
/*--- H A U P T P R O G R A M M                                               */
/*----------------------------------------------------------------------------*/
void main( void )
{
  printf( "DBLTSTP.PAS  -  (c)  1993 by Michael Tischer\n" );
  isdbl = IsDoubleSpaceInstalled( &firstdrive, &numdrive,
                                  &vernum, &uppermem );
  if( !isdbl )
  {
    printf ( "DoubleSpace nicht installiert!\n");
    exit(1);                                               /* Programm be}en */
  };

  /*-- DoubleSpace ist installiert -------------------------------------------*/
  printf ("DoubleSpace-Version         : %d\n", vernum );
  printf ("erstes DoubleSpace-Laufwerk : %c:\n", firstdrive );
  printf ("reserviert fr DoubleSpace  : %d Laufwerke\n", numdrive );
  printf ("DoubleSpace im Upper-Memory : " );
  printf ("%s\n\n", uppermem ? "Ja" : "Nein");

  /*-- DoubleSpace-Laufwerke ausgeben ----------------------------------------*/
  printf( "LW   komprimiert   ist eigentlich   CVF-Datei\n");
  printf( "--   -----------   --------------   ------------\n");
  for( i = 0; i < 26; i++ )           /* Laufwerke von A: bis Z: durchlaufen */
    {
      isdbl = IsDoubleSpaceDrive( i, &vertauscht, &host, &cvfnr );
      if( isdbl || vertauscht )
        {
          printf("%c: %s", 'A'+i, isdbl ? "    ja " : "   nein");

          if( vertauscht )
            printf( "             %c: ", 'A'+host);
          else printf("                ");

          if(isdbl)      /* CVF-Nummer fr DoubleSpace-Laufwerke ausgeben */
              printf ( "        DBLSPACE. %03d", cvfnr );
          printf("\n");
        }
    }
    printf("\n");
}
