/**********************************************************************/
/*                          D D P T C . C                             */
/*--------------------------------------------------------------------*/
/*    Aufgabe       : ermglicht die selektive Vernderung einzelner  */
/*                    Werte in der Disk-Drive-Parameter-Table         */
/*--------------------------------------------------------------------*/
/*    Autor         : Michael Tischer                                 */
/*    entwickelt am : 22.08.1991                                      */
/*    letztes Update: 22.09.1991                                      */
/*--------------------------------------------------------------------*/
/*    Speichermodell: SMALL                                           */
/*--------------------------------------------------------------------*/
/**********************************************************************/
 
/*== Include-Dateien einbinden =======================================*/

#include <dos.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <string.h>

/*== Makros ==========================================================*/

#ifdef MK_FP                          /* Makro MK_FP schon definiert? */
  #undef MK_FP                              /* Ja, dann Makro lschen */
#endif

#define MK_FP(seg,ofs) ((void far *) ((unsigned long) (seg)<<16|( ofs)))

/*== Typedefs ========================================================*/

typedef unsigned char byte;                          /* Datentyp Byte */
typedef byte DDPT_Typ[ 11 ];                    /* Feld fr eine DDPT */

/**********************************************************************/
/* upcase        : konvertiert einen Buchstaben in einen Grobuchst.  */
/* Eingabe       : Buchstabe                                          */
/* Ausgabe       : Grobuchstabe                                      */
/**********************************************************************/

byte upcase( byte letter )
{
 if ( ( letter > 0x60 ) && ( letter < 0x7B ) )       /* Konvertieren? */
  return( letter & 0xDF );                      /* Ja, Bit ausblenden */
 else
  return( letter);                   /* Nein, unverndert zurckgeben */
}

/**********************************************************************/
/* D_ziffer : Wandelt eine hexadezimale Ziffer in einen dezimalen     */
/*            Wert                                                    */
/* Eingabe  : hexadezimale Ziffer                                     */
/* Ausgabe  : Zahl                                                    */
/**********************************************************************/

byte D_Ziffer( char Hex )
{
 if ( ( Hex >= 0x41 ) && ( Hex <= 0x46 ) )                  /* A - F? */
  return( Hex - 55 );                                           /* Ja */
 else
  return( Hex ) - 48;                              /* Nein, also 0..9 */
}

/**********************************************************************/
/* hex_byte : Wandelt einen HexString in ein Byte                     */
/* Eingabe  : s.u                                                     */
/* Ausgabe  : Zahl                                                    */
/**********************************************************************/

byte hex_byte( char *hex )                /* zu wandelnder Hex-String */
{
 if ( hex[ 1 ] == 0x58 )                 /* Test auf X also Zahl 0x.. */
   hex += 2;                               /* Zeiger auf erste Ziffer */
 return( ( D_Ziffer( *hex ) << 4 ) | D_Ziffer( hex[ 1 ] ) );
}

/**********************************************************************/
/* GetIntVec: liest einen Interrupt-Vektor aus                        */
/* Eingabe  : NUMMER = Interrupt-Nummer                               */
/* Ausgabe  : Interrupt-Vektor                                        */
/**********************************************************************/

void far *GetIntVec( int Nummer )
{
 return( *( ( void far * far * ) MK_FP( 0, Nummer * 4 ) ) );
}

/**********************************************************************/
/* RAM_DDPT : Testet, ob DDPT im RAM oder im ROM                      */
/* Eingabe  : DDPT = FAR-Zeiger auf DDPT-Variable                     */
/* Ausgabe  : TRUE, wenn DDPT im RAM                                  */
/* Info     : die Funktion schreibt einen Wert in die DDPT, liest     */
/*            diesen wieder aus und vergleicht ob der Wert            */
/*            geschrieben wurde also die DDPT im RAM liegt            */
/**********************************************************************/

int RAM_DDPT( DDPT_Typ far *DDPT )                 /* Zeiger auf DDPT */
{
 byte buffer;                 /* Speicher fr aktuellen Wert der DDPT */
 int  Flag;                              /* Speicher fr Rckgabewert */

 buffer = *DDPT[ 0 ];                      /* Wert der DDPT speichern */
 *DDPT[ 0 ] = buffer ^ 0xFF;                      /* Wert invertieren */
 Flag = ( *DDPT[ 0 ] == ( buffer ^0xFF ) );
 *DDPT[ 0 ] = buffer;                      /* alten Wert restaurieren */
 return( Flag );                                      /* Rckgabewert */
}

/**********************************************************************/
/* Werteanzeigen: Werte der DDPT ausgeben                             */
/* Eingabe      : DDPT = FAR-Zeiger auf DDPT-Variable                 */
/* Ausgabe      : keine                                               */
/**********************************************************************/

void WerteAnzeigen( DDPT_Typ far *DDPT )           /* Zeiger auf DDPT */
{
  printf( "Steprate                (SR): 0x%02x\n\n",
          ( *DDPT )[ 0 ] >> 4 );
  printf( "Head-Unload-Time        (HU): 0x%02x\n",
          ( *DDPT )[ 0 ] & 0x0f );
  printf( "Head-Load-Time          (HL): 0x%02x\n",
          ( *DDPT )[ 1 ] >> 1 );
  printf( "Head-Settle-Time        (HS): 0x%02x\n\n", ( * DDPT )[ 9 ] );
  printf( "Nachlaufzeit des Motors (MN): 0x%02x\n", ( *DDPT )[ 2 ] );
  printf( "Anlaufzeit des Motors   (MA): 0x%02x\n", ( *DDPT )[ 10 ] );
}

/**********************************************************************/
/* NeueWerte: Neue Werte der DDPT setzen                              */
/* Eingabe  : s.u                                                     */
/* Ausgabe  : keine                                                   */
/**********************************************************************/

void NeueWerte( int Anzahl,                       /* Anzahl Kommandos */
                char *Werte[],                  /* Feld mit Kommandos */
                DDPT_Typ far *DDPT )               /* Zeiger auf DDPT */
{
 int  i,j;                                         /* Schleifenzhler */
 char Art[ 4 ],                /* Parameter der verndert werden soll */
      Befehl[ 8 ];                         /* Kommandozeilenparameter */
 byte Wert,                                /* Neuer zu setzender Wert */
      HilfsWert;                           /* Hilfswert zum Speichern */

 /*-- Schleife: alle Parameter durchlaufen ---------------------------*/

 for ( i = 1; i < Anzahl; i++ )
 {
  strcpy( Befehl, Werte[ i ] );                    /* Parameter holen */
  j = 0;
  while ( Befehl[ j ] != 0 )
   Befehl[ j++ ] = upcase( Befehl[ j ] );

  Art[ 0 ] = Befehl[ 0 ];                   /* zu setzender Parameter */
  Art[ 1 ] = Befehl[ 1 ];
  Art[ 2 ] = 0;
  Wert = hex_byte( &Befehl[ 3 ] );               /* zu setzender Wert */
  if ( !strcmp( Art, "SR" ) )                            /* Steprate? */
   {
    Wert = Wert << 4;                        /* Wert in oberes Nibble */
    HilfsWert = ( *DDPT )[ 0 ] & 0x0F;        /* Unteres Nibble lesen */
    ( *DDPT )[ 0 ] = Wert | HilfsWert;              /* Wert schreiben */
   }
  else if ( !strcmp( Art, "HU" ) )               /* Head-Unload-Time? */
   {
    Wert = Wert & 0x0F;                     /* Wert in unteres Nibble */
    HilfsWert = ( *DDPT )[ 0 ] & 0xF0;         /* Oberes Nibble lesen */
    ( *DDPT )[ 0 ] = Wert | HilfsWert;              /* Wert schreiben */
   }
  else if ( !strcmp( Art, "HL" ) )                 /* Head-Load-Time? */
   ( *DDPT )[ 1 ] = Wert << 1;           /* Wert in Bit 1-7 speichern */
  else if ( !strcmp( Art, "HS" ) )               /* Head-Settle-Time? */
   ( *DDPT )[ 9 ] = Wert;                           /* Wert speichern */
  else if ( !strcmp( Art, "MN" ) )              /* Motor Nachlaufzeit */
   ( *DDPT )[ 2 ] = Wert;                           /* Wert speichern */
  else if ( !strcmp( Art, "MA" ) )                /* Motor Anlaufzeit */
   ( *DDPT )[ 10 ] = Wert;                          /* Wert speichern */
 }
}

/**********************************************************************/
/*                         HAUPTPROGRAMM                              */
/**********************************************************************/

void main( int argc, char *argv[] )
{
 DDPT_Typ far *DDPT;                  /* Zeiger auf die aktuelle DDPT */

 printf( "DDPTC  -  (c) 1992 by Michael Tischer\n" );
 printf( "Erlaubt die Verndungerung der aktuellen DDPT\n" );

 DDPT = GetIntVec( 0x1E );               /* Zeiger auf die DDPT holen */
 if ( RAM_DDPT )                       /* DDPT im RAM, also nderbar? */
  {
   if ( argc > 1 )                    /* Sollen Werte gesetzt werden? */
    {
     NeueWerte( argc, argv, DDPT );               /* Neu Werte setzen */
     printf( "\n\nNeue Werte der DDPT:\n" );
     WerteAnzeigen( DDPT );           /* Neue Werte der DDPT ausgeben */
     exit( 0 );
    }
  }
 else                      /* DDPT liegt im ROM, ndern nicht mglich */
  printf( "Eine nderung der Disk-Drive-Parameter-Table" \
          "ist nicht mglich, weil sie im ROM liegt!\n" );
 printf( "\nInhalt der DDPT:\n" );
 WerteAnzeigen( DDPT );               /* Alte Werte der DDPT ausgeben */
}

