/**********************************************************************/
/*                             D F C . C                              */
/*--------------------------------------------------------------------*/
/*    Aufgabe       : Formatiert 3,5" und 5,25" Disketten             */
/*--------------------------------------------------------------------*/
/*    Autor         : Michael Tischer                                 */
/*    entwickelt am : 28.08.1991                                      */
/*    letztes Update: 26.01.1992                                      */
/*--------------------------------------------------------------------*/
/*    Speichermodell: SMALL                                           */
/*--------------------------------------------------------------------*/
/*    Achtung       : Warnmeldung                                     */
/*                      "warning C4059: segment lost in conversion"   */
/*                    bei Microsoft-Compilern ist gewollt.            */
/**********************************************************************/
 
/*== Include-Dateien einbinden =======================================*/

#include <dos.h>
#include <stdio.h>
#include <string.h>

/*== Makros ==========================================================*/

#ifdef MK_FP                          /* Makro MK_FP schon definiert? */
  #undef MK_FP                              /* Ja, dann Makro lschen */
#endif

#define MK_FP(seg,ofs) ((void far *) ((unsigned long) (seg)<<16|( ofs)))
#define LO( wert ) ( ( BYTE ) ( wert & 0xFF ) )
#define HI( wert ) ( ( BYTE ) ( wert >> 8 ) )
#define SEG( p ) ( ( unsigned int ) ( ( ( long ) p ) >> 16 ) )
#define OFS( p ) ( ( unsigned int ) ( p ) )

/*== Konstanten ======================================================*/

#define NEIN         0x4E                               /* N fr Nein */
#define NO_DRIVE     0                    /* Laufwerk nicht vorhanden */
#define DD_525       1                          /* Laufwerk: 5,25" DD */
#define HD_525       2                          /* Laufwerk: 5,25" HD */
#define DD_35        3                           /* Laufwerk: 3,5" DD */
#define HD_35        4                           /* Laufwerk: 3,5" HD */

#define MAXVERSUCHE  5                    /* Maximale Anzahl Versuche */

#define TRUE         ( 0 == 0 )        /* Konstanten, erleichtern das */
#define FALSE        ( 1 == 0 )        /* Lesen des Programmtextes    */

/*== Typedefs ========================================================*/

typedef unsigned char BYTE;                          /* Datentyp Byte */

typedef BYTE DDPTType[ 11 ];                    /* Feld fr eine DDPT */
typedef DDPTType *DDPTPTR;                    /* Zeiger auf eine DDPT */

typedef struct {                      /* physikalische Formatparamter */
                  BYTE    Seiten,                /* Anzahl der Seiten */
                          Spuren,                 /* Spuren pro Seite */
                          Sektoren;              /* Sektoren pro Spur */
                  DDPTPTR DDPT;                   /* Zeiger auf DDPTR */
                } PhysDataType;

typedef struct {                          /* logische Formatparameter */
                  BYTE Media;                           /* Media-Byte */
                  BYTE Cluster;        /* Anzahl Sektoren pro Cluster */
                  BYTE FAT;            /* Anzahl Sektoren fr dir FAT */
                  BYTE RootSize;      /* Eintrge im Hauptverzeichnis */
                } LogDataType;

typedef BYTE SpurBufType[ 18 ][ 512 ];      /* Speicher fr eine Spur */

/*== Globale Variablen ===============================================*/

/*-- Nicht variabler Teil des BOOT-Sektors ---------------------------*/

BYTE BootMaske[ 102 ] =
                   { 0xEB, 0x35,            /* 0000   JMP 0037        */
                     0x90,                  /* 0002   NOP             */
                     /*-- Daten des BPB ------------------------------*/

                     0x50, 0x43, 0x49, 0x4E, 0x54, 0x45, 0x52, 0x4E,
                     0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00,
                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
                     0x00, 0x00, 0x00, 0x00,

                     /*-- Eigentliches Ladeprogramm ------------------*/

                     0xFA,                  /* 0037   CLI             */
                     0xB8, 0x30, 0x00,      /* 0038   MOV     AX,0030 */
                     0x8E, 0xD0,            /* 003B   MOV     SS,AX   */
                     0xBC, 0xFC, 0x00,      /* 003D   MOV     SP,00FC */
                     0xFB,                  /* 0040   STI             */
                     0x0E,                  /* 0041   PUSH    CS      */
                     0x1F,                  /* 0042   POP     DS      */
                     0xBE, 0x66, 0x7C,      /* 0043   MOV     SI,7C66 */
                     0xB4, 0x0E,            /* 0046   MOV     AH,0E   */
                     0xFC,                  /* 0048   CLD             */
                     0xAC,                  /* 0049   LODSB           */
                     0x0A, 0xC0,            /* 004A   OR      AL,AL   */
                     0x74, 0x04,            /* 004C   JZ      0052    */
                     0xCD, 0x10,            /* 004E   INT     10      */
                     0xEB, 0xF7,            /* 0050   JMP     0049    */
                     0xB4, 0x01,            /* 0052   MOV     AH,01   */
                     0xCD, 0x16,            /* 0054   INT     16      */
                     0x74, 0x06,            /* 0056   JZ      005E    */
                     0xB4, 0x00,            /* 0058   MOV     AH,00   */
                     0xCD, 0x16,            /* 005A   INT     16      */
                     0xEB, 0xF4,            /* 005C   JMP     0052    */
                     0xB4, 0x00,            /* 005E   MOV     AH,00   */
                     0xCD, 0x16,            /* 0060   INT     16      */
                     0x33, 0xD2,            /* 0062   XOR     DX,DX   */
                     0xCD, 0x19 };          /* 0064   INT     19      */

char BootMes[] =
  "\nDFC  -  (C) 1992 by Michael Tischer\n\n" \
  "Keine Systemdiskette oder fehlerhafte Diskette!\n" \
  "Bitte Diskette wechseln und danach eine Taste drcken" \
  "\n\n";

/**********************************************************************/
/* upcase        : konvertiert einen Buchstaben in einen Grobuchst.  */
/* Eingabe       : Buchstabe                                          */
/* Ausgabe       : Grobuchstabe                                      */
/**********************************************************************/

char upcase( char letter )
{                                                              /* XXX */
 if ( ( letter > 0x60 ) && ( letter < 0x7B ) )       /* Konvertieren? */
  return (unsigned char) letter & 0xDF;         /* Ja, Bit ausblenden */
 else
  return letter;                     /* Nein, unverndert zurckgeben */
}

/**********************************************************************/
/* GetIntVec: liest einen Interrupt-Vektor aus                        */
/* Eingabe  : NUMMER = Interrupt-Nummer                               */
/* Ausgabe  : Interrupt-Vektor                                        */
/**********************************************************************/

void far *GetIntVec( int Nummer )
{
 return( *( ( void far * far * ) MK_FP( 0, Nummer * 4 ) ) );
}

/**********************************************************************/
/* SetIntVec: setzt einen Interrupt-Vektor                            */
/* Eingabe  : NUMMER = Interrupt-Nummer                               */
/*            ZEIGER = Interrupt-Vektor                               */
/* Ausgabe  : keine                                                   */
/**********************************************************************/

void SetIntVec( int Nummer, void far *Zeiger )
{
 *( ( void far * far * ) MK_FP( 0, Nummer * 4 ) ) = Zeiger;
}

/**********************************************************************/
/* GetDriveType : Ermittelt den Typ eines Diskettenlaufwerks          */
/* Eingabe      : DRIVE = Laufwerksnummer (0, 1 etc.)                 */
/* Ausgabe      : Laufwerkscode als Konstanten (DD_525, HD_525 etc.)  */
/**********************************************************************/

BYTE GetDriveType( BYTE Drive )
{
 union REGS regs;           /* Prozessorregister fr Interrupt-Aufruf */

 regs.h.ah = 0x08;                /* Funktion: Laufwerkstyp ermitteln */
 regs.h.dl = Drive;                                /* Laufwerksnummer */
 int86( 0x13, &regs, &regs );              /* BIOS-Interrupt aufrufen */
 if ( regs.x.cflag )                            /* Fehler bei Aufruf? */
  return( DD_525 );          /* Fkt. 0x08 nicht vorhanden => 360KB XT */
 else
  return( regs.h.bl );                                /* Laufwerkstyp */
}

/**********************************************************************/
/* ResetDisk : Disk-Reset auf alle Laufwerke                          */
/* Eingabe   : keine                                                  */
/* Ausgabe   : keine                                                  */
/* Info      : Unabhngig von der in DL geladenen Laufwerksnummer     */
/*             wird der Reset auf allen Diskletten-Laufwerken         */
/*             ausgefhrt                                             */
/**********************************************************************/
 
void DiskReset( void )
{
 union REGS regs;           /* Prozessorregister fr Interrupt-Aufruf */

 regs.h.ah = 0x00;            /* Funktionsnummer fr Interrupt-Aufruf */
 regs.h.dl = 0;                              /* Laufwerk a: (s. Info) */
 int86( 0x13, &regs, &regs );                     /* Interrupt-Aufruf */
}

/**********************************************************************/
/* GetFormatParamter: Ermittelt die logischen und physikalischen Pa-  */
/*                    rameter, die fr die Formatierung bentigt      */
/*                    werden                                          */
/* Eingabe  : FORMSTRING = Zeiger auf String mit Format               */
/*                          "360", "720", "1200", "1440"              */
/*            DRIVETYPE  = Laufwerkscode, wie er von GetDriveType()   */
/*                         geliefert wurde                            */
/*            PDATAP     = Zeiger auf Struktur, die physikalische     */
/*                         Formatparameter aufnimmt                   */
/*            LDATAP     = Zeiger auf Struktur, die logische Format-  */
/*                         parameter aufnimmt                         */
/* Ausgabe : TRUE, wenn das Format mglich ist, sonst FALSE           */
/* Info    : Neue Formate knnen durch Erweitern dieser Prozedur      */
/*                 hinzugefgt werden                                 */
/**********************************************************************/

BYTE GetFormatParameter( char         *FormString,
                         BYTE         DriveType,
                         PhysDataType *PDataP,
                         LogDataType  *LDataP )

{
 static DDPTType DDPT_360  = { 0xDF, 0x02, 0x25, 0x02, 0x09, 0x2A,
                              0xFF, 0x50, 0xF6, 0x0F, 0x08 };
 static DDPTType DDPT_1200 = { 0xDF, 0x02, 0x25, 0x02, 0x0F, 0x1B,
                              0xFF, 0x54, 0xF6, 0x0F, 0x08 };
 static DDPTType DDPT_1440 = { 0xDF, 0x02, 0x25, 0x02, 0x12, 0x1B,
                              0xFF, 0x6C, 0xF6, 0x0F, 0x08 };
 static DDPTType DDPT_720  = { 0xDF, 0x02, 0x25, 0x02, 0x09, 0x2A,
                              0xFF, 0x50, 0xF6, 0x0F, 0x08 };

 static LogDataType LOG_360  = { 0xFD, 2, 2, 0x70 };
 static LogDataType LOG_1200 = { 0xF9, 1, 7, 0xE0 };
 static LogDataType LOG_720  = { 0xF9, 2, 3, 0x70 };
 static LogDataType LOG_1440 = { 0xF0, 1, 9, 0xE0 };

 static PhysDataType PHYS_360  = { 2, 40,  9, &DDPT_360 };
 static PhysDataType PHYS_1200 = { 2, 80, 15, &DDPT_1200 };
 static PhysDataType PHYS_720  = { 2, 80,  9, &DDPT_720 };
 static PhysDataType PHYS_1440 = { 2, 80, 18, &DDPT_1440 };

 /*-- Format aus String entnehmen und bergebene Strukturen ent- -----*/
 /*-- sprechend mit Daten fllen                                 -----*/

 if ( strcmp( FormString, "1200" ) == 0 )        /* 1,2 MB auf 5,25"? */
   if ( DriveType == HD_525 )     /* Format mit Laufwerk vertrglich? */
    {
     memcpy( PDataP, &PHYS_1200, sizeof( PhysDataType ) );
     memcpy( LDataP, &LOG_1200, sizeof ( LogDataType ) );
     return TRUE;                                 /* Ende ohne Fehler */
    }
    else
     return( FALSE );            /* Laufwerk und Format unvertrglich */
  else if ( strcmp( FormString, "360" ) == 0 )             /* 360 KB? */
   if ( ( DriveType == HD_525 ) || ( DriveType == DD_525 ) )
    {            /* Format und Laufwerk vertrglich, Parameter setzen */
     memcpy ( PDataP, &PHYS_360, sizeof( PhysDataType ) );
     memcpy ( LDataP, &LOG_360, sizeof ( LogDataType ) );
     return TRUE;                                 /* Ende ohne Fehler */
    }
    else
     return( FALSE );            /* Laufwerk und Format unvertrglich */
  else if ( strcmp( FormString, "1440" ) == 0 )  /* 1,44 MB auf 3,5"? */
   if ( DriveType == HD_35 )      /* Format mit Laufwerk vertrglich? */
    {            /* Format und Laufwerk vertrglich, Parameter setzen */
     memcpy ( PDataP, &PHYS_1440, sizeof( PhysDataType ) );
     memcpy ( LDataP, &LOG_1440, sizeof ( LogDataType ) );
     return TRUE;                                 /* Ende ohne Fehler */
    }
    else
     return( FALSE );            /* Laufwerk und Format unvertrglich */
  else if ( strcmp( FormString, "720" ) == 0 )    /* 720 KB auf 3,5"? */
    if ( ( DriveType == HD_35 ) || ( DriveType == DD_35 ) )
    {            /* Format und Laufwerk vertrglich, Parameter setzen */
     memcpy ( PDataP, &PHYS_720, sizeof( PhysDataType ) );
     memcpy ( LDataP, &LOG_720, sizeof ( LogDataType ) );
     return TRUE;                                 /* Ende ohne Fehler */
    }
    else
     return FALSE;               /* Laufwerk und Format unvertrglich */
  else
    return FALSE;                   /* Kein gltiges Format angegeben */
}

/**********************************************************************/
/* DiskPrepare: Vorbereiten des Laufwerks, Einstellen der Daten-      */
/*               transferrate                                         */
/* Eingabe    : DRIVE = Laufwerksnummer                               */
/*              PDATA = Tabelle mit physikalischen Parametern         */
/* Ausgabe    : keine                                                 */
/**********************************************************************/
 
void DiskPrepare( BYTE Drive, PhysDataType PData )
{
 union REGS regs;           /* Prozessorregister fr Interrupt-Aufruf */

 /*-- Media-Typ fr Formatieraufruf setzen ---------------------------*/

 regs.h.ah = 0x18;            /* Funktionsnummer fr Interrupt-Aufruf */
 regs.h.ch = PData.Spuren - 1;             /* Anzahl Spuren pro Seite */
 regs.h.cl = PData.Sektoren;              /* Anzahl Sektoren pro Spur */
 regs.h.dl = Drive;                                 /* Laufwerknummer */
 int86( 0x13, &regs, &regs );                     /* Interrupt-Aufruf */
}

/**********************************************************************/
/* FormatTrack: Formatiert eine Spur                                  */
/* Eingabe    : s.u.                                                  */
/* Ausgabe    : der Fehlerstats                                       */
/**********************************************************************/
 
BYTE FormatTrack( BYTE Laufwerk,               /* die Laufwerksnummer */
                  BYTE Seite,                     /* die Seitennummer */
                  BYTE Spur,                              /* die Spur */
                  BYTE Anzahl )     /* Anzahl Sektoren fr diese Spur */

{
 struct FormatTyp {              /* Sektor-Informationen fr das BIOS */
                  BYTE DSpur, DSeite, DZaehler, DLaenge;
                 };

 BYTE             versuche;    /* Versuchsanzahl fr Interrupt-Aufruf */
 BYTE             Zaehler;                         /* Schleifenzhler */
 struct FormatTyp Datenfeld[ 18 ];             /* maximal 18 Sektoren */
 void far *       dfp = Datenfeld;            /* Zeiger auf Datenfeld */
 union REGS       regs;     /* Prozessorregister fr Interrupt-Aufruf */
 struct SREGS     sregs;                           /* Segmentregister */

 for ( Zaehler = 0; Zaehler < Anzahl; Zaehler++ )
  {
   Datenfeld[ Zaehler ].DSpur = Spur;
   Datenfeld[ Zaehler ].DSeite = Seite;
   Datenfeld[ Zaehler ].DZaehler = Zaehler + 1;
   Datenfeld[ Zaehler ].DLaenge = 2;           /* 512 Byte pro Sektor */
  }

 versuche = MAXVERSUCHE;            /* maximale Versuchsanzahl setzen */
 do
  {
   regs.h.ah = 5;             /* Funktionsnummer fr Interrupt-Aufruf */
   regs.h.al = Anzahl;           /* Anzahl der Sektoren fr eine Spur */
   regs.x.bx = OFS( dfp );                  /* Offsetadr. des Puffers */
   sregs.es = SEG( dfp );                              /* Segmentadr. */
   regs.h.dh = Seite;                             /* Nummer der Seite */
   regs.h.dl = Laufwerk;                           /* Laufwerksnummer */
   regs.h.ch = Spur;                               /* Nummer der Spur */
   int86x( 0x13, &regs, &regs, &sregs );   /* BIOS-Interrupt aufrufen */
   if ( regs.x.cflag )                                     /* Fehler? */
     DiskReset();
  }
 while ( ( --versuche != 0 ) && ( regs.x.cflag ) );
 return( regs.h.ah );                       /* Fehler-Status auslesen */
}

/**********************************************************************/
/* VerifyTrack: Spur berprfen                                       */
/* Eingabe    : s.u                                                   */
/* Ausgabe    : Fehlerstatus                                          */
/**********************************************************************/
 
BYTE VerifyTrack( BYTE Laufwerk,                   /* Laufwerksnummer */
                  BYTE Seite,                         /* Seitennummer */
                  BYTE Spur,                            /* Spurnummer */
                  BYTE Sektoren )         /* Anzahl Sektoren pro Spur */

{
 BYTE         versuche;        /* Versuchsanzahl fr Interrupt-Aufruf */
 union REGS   regs;         /* Prozessorregister fr Interrupt-Aufruf */
 struct SREGS sregs;  /* Prozessorr. fr erweiterten Interrupt-Aufruf */
 SpurBufType  sbuf;                                     /* Spurpuffer */
 void far     *sbptr = sbuf;             /* FAR-Zeiger auf Spurpuffer */

 versuche = MAXVERSUCHE;            /* maximale Versuchsanzahl setzen */
 do
  {
   regs.h.ah = 0x04;           /* Funktionsnummer fr Interruptaufruf */
   regs.h.al = Sektoren;                  /* Anzahl Sektoren pro Spur */
   regs.h.ch = Spur;                               /* Nummer der Spur */
   regs.h.cl = 1;                               /* Start bei Sektor 1 */
   regs.h.dl = Laufwerk;                           /* Laufwerksnummer */
   regs.h.dh = Seite;                                 /* Seitennummer */
   regs.x.bx = OFS( sbptr );                /* Offsetadr. des Puffers */
   sregs.es = SEG( sbptr );                            /* Segmentadr. */
   int86x( 0x13, &regs, &regs, &sregs );   /* BIOS-Interrupt aufrufen */
   if ( regs.x.cflag )                                     /* Fehler? */
     DiskReset();
  }
 while ( ( --versuche != 0 ) && ( regs.x.cflag ) );
 return( regs.h.ah );                       /* Fehler-Status auslesen */
}

/**********************************************************************/
/* WriteTrack: Spur schreiben                                         */
/* Eingabe   : s.u                                                    */
/* Ausgabe   : Fehlercode (0=OK)                                      */
/**********************************************************************/

BYTE WriteTrack( BYTE Laufwerk,                    /* Laufwerksnummer */
                 BYTE Seite,                          /* Seitennummer */
                 BYTE Spur,                             /* Spurnummer */
                 BYTE Start,                      /* Start bei Sektor */
                 BYTE Sektoren,           /* Anzahl Sektoren pro Spur */
                 void far *Daten )            /* Zeiger auf Datenfeld */

{
 BYTE versuche;                /* Versuchsanzahl fr Interrupt-Aufruf */
 union REGS regs;           /* Prozessorregister fr Interrupt-Aufruf */
 struct SREGS sregs;  /* Prozessorr. fr erweiterten Interrupt-Aufruf */

 versuche = MAXVERSUCHE;            /* maximale Versuchsanzahl setzen */
 do
  {
   regs.h.ah = 0x03;           /* Funktionsnummer fr Interruptaufruf */
   regs.h.al = Sektoren;                  /* Anzahl Sektoren pro Spur */
   regs.h.ch = Spur;                               /* Nummer der Spur */
   regs.h.cl = Start;                             /* Start bei Sektor */
   regs.h.dl = Laufwerk;                           /* Laufwerksnummer */
   regs.h.dh = Seite;                                 /* Seitennummer */
   regs.x.bx = OFS( Daten );                /* Offsetadr. des Puffers */
   sregs.es = SEG( Daten );                            /* Segmentadr. */
   int86x( 0x13, &regs, &regs, &sregs );   /* BIOS-Interrupt aufrufen */
   if ( regs.x.cflag )                                     /* Fehler? */
     DiskReset();
  }
 while ( ( --versuche != 0 ) && ( regs.x.cflag ) );
 return( regs.h.ah );                       /* Fehler-Status auslesen */
}

/**********************************************************************/
/* PhysicalFormat: Physikalisches Formatieren der Diskette (Einteilen */
/*                 in Spuren, Sektoren)                               */
/* Eingabe       : s.u.                                               */
/* Ausgabe       : Formatieren ohne Fehler beendet                    */
/**********************************************************************/

BYTE PhysicalFormat( BYTE         Drive,           /* Laufwerksnummer */
                     PhysDataType PData,   /* physikalische Parameter */
                     BYTE         Verify )         /* Flag fr Verify */

{
 union REGS regs;           /* Prozessorregister fr Interrupt-Aufruf */
 BYTE       versuche,          /* Versuchsanzahl fr Interrupt-Aufruf */
            Spur,                   /* Schleifenzhler: aktuelle Spur */
            Seite,                 /* Schleifenzhler: aktuelle Seite */
            Status;       /* Rckgabewert der aufgerufenen Funktionen */

 /*-- Diskette Spur fr Spur formatieren -----------------------------*/

 for ( Spur = 0; Spur < PData.Spuren; Spur++ )
  for ( Seite = 0; Seite < PData.Seiten; Seite++ )
   {
    printf( "\rSpur: %d  Seite: %d", Spur, Seite );
    /*-- Maximal 5 Versuche ein Spur zu formatieren ------------------*/

    versuche = MAXVERSUCHE;         /* maximale Versuchsanzahl setzen */
    do
     {
      Status = FormatTrack( Drive, Seite, Spur, PData.Sektoren );
      if ( Status == 3 )                /* Diskette schreibgeschtzt? */
       {
         printf( "\rDiskette ist schreibgeschtzt" );
         return FALSE;                 /* Prozedur mit Fehler beenden */
       }
      if ( Status == 0  &&  Verify )
       Status = VerifyTrack( Drive, Seite, Spur, PData.Sektoren );
      if ( Status > 0 )                 /* Formatieren fehlgeschlagen */
       DiskReset();
     }
    while ( ( --versuche != 0 ) && ( Status != 0 ) );
    if ( Status > 0 )                     /*  Fehler beim Formatieren */
     {
      printf( "\rSpur fehlerhaft       \n" );
      return FALSE;                    /* Prozedur mit Fehler beenden */
     }
   }
 return TRUE;                         /* Prozedur ohne Fehler beendet */
}

/**********************************************************************/
/* LogicalFormat : Logisches Formatieren der Diskette (Schreiben von  */
/*                 Bootsektore, FAT und Hauptverzeichnis)             */
/* Eingabe       : s.u                                                */
/* Ausgabe       : TRUE, wenn kein Fehler aufgetreten ist             */
/**********************************************************************/

BYTE LogicalFormat( BYTE Drive,                    /* Laufwerksnummer */
                    PhysDataType PData,    /* physikalische Parameter */
                    LogDataType LData )    /* physikalische Parameter */

{
 BYTE        i,                                    /* Schleifenzhler */
             AktSector,
             AktSide,
             AktTrack,
             Status;
 int         GesamtSektoren,           /* Gesamte Anzahl der Sektoren */
             Anzahl;           /* Anzahl noch zu schreibende Sektoren */
 SpurBufType SpurBuffer;             /* nimmt eine komplette Spur auf */

 memset( SpurBuffer, 0, (int) PData.Sektoren * 512 );  /* Spur leeren */

 /*-- Bootsektor: fester Teil ----------------------------------------*/

 memcpy( SpurBuffer, BootMaske, 102 );   /* Maske Bootsektor kopieren */
 memcpy( &SpurBuffer[ 0 ][ 102 ], BootMes, sizeof( BootMes) );
 SpurBuffer[ 0 ][ 510 ] = 0x55;    /* Endekennzeichen des Bootsektors */
 SpurBuffer[ 0 ][ 511 ] = 0xAA;

 /*-- Bootsektor: variabler Teil--------------------------------------*/

 GesamtSektoren = (int) PData.Spuren * (int) PData.Sektoren *
                  (int) PData.Seiten;        /* Gesamtanzahl Sektoren */
 SpurBuffer[ 0 ][ 13 ] = LData.Cluster;               /* Clustergre */
 SpurBuffer[ 0 ][ 17 ] = LData.RootSize;/* Anzahl Eintrge im Hauptv. */
 SpurBuffer[ 0 ][ 19 ] = LO( GesamtSektoren );
 SpurBuffer[ 0 ][ 20 ] = HI( GesamtSektoren );
 SpurBuffer[ 0 ][ 21 ] = LData.Media;             /* Media-Deskriptor */
 SpurBuffer[ 0 ][ 22 ] = LData.FAT;                  /* Gre der FAT */
 SpurBuffer[ 0 ][ 24 ] = PData.Sektoren;         /* Sektoren pro Spur */
 SpurBuffer[ 0 ][ 26 ] = PData.Seiten;           /* Anzahl der Seiten */

 /*-- FAT und FAT-Kopie anlegen --------------------------------------*/

 SpurBuffer[ 1 ][ 0 ] = LData.Media;                /* 1. FAT anlegen */
 SpurBuffer[ 1 ][ 1 ] = 0xFF;
 SpurBuffer[ 1 ][ 2 ] = 0xFF;
 SpurBuffer[ LData.FAT + 1 ][ 0 ] = LData.Media;    /* 2. FAT anlegen */
 SpurBuffer[ LData.FAT + 1 ][ 1 ] = 0xFF;
 SpurBuffer[ LData.FAT + 1 ][ 2 ] = 0xFF;

 /*-- Boot-Sektor und FAT schreiben ----------------------------------*/

 Status = WriteTrack( Drive, 0, 0, 1, PData.Sektoren, SpurBuffer );
 if ( Status )                              /* Fehler beim Schreiben? */
  return FALSE;                              /* Ja, mit Fehler zurck */

 /*-- Rootverzeichnis schreiben --------------------------------------*/

 memset( SpurBuffer, 0, 512 );                       /* Leerer Sektor */
 AktSector = PData.Sektoren;       /* Erste Spur komplett geschrieben */
 AktTrack = 0;                                       /* Aktuelle Spur */
 AktSide = 0;                                       /* Aktuelle Seite */

 /*-- Anzahl der verbleibenden Sektoren ermitteln und schreiben ------*/

 Anzahl = LData.FAT * 2 + (LData.RootSize*32/512) + 1-PData.Sektoren;

 for ( i = 1; i <= Anzahl; i++ )
  {
   if ( ++AktSector > PData.Sektoren )               /* Spur zu Ende? */
    {
     AktSector = 1;                            /* Weiter bei Sektor 1 */
     if ( ++AktSide == PData.Seiten )              /* Schon 2. Seite? */
      {
       AktSide = 0;                             /* Wieder auf Seite 0 */
       AktTrack++;
      }
    }
   Status = WriteTrack( Drive, AktSide, AktTrack,
                        AktSector, 1, SpurBuffer );
   if ( Status )                                           /* Fehler? */
    break;                    /* Ja, FOR-Schleife vorzeitig verlassen */
  }
 return ( Status == 0 );
}

/**********************************************************************/
/*                         HAUPTPROGRAMM                              */
/**********************************************************************/

int main( argc, argv )

int argc;                    /* Anzahl Argumente in der Kommandozeile */
char *argv[];                                  /* Feld mit Parametern */

{
 BYTE         AktDrive;    /* Nummer des zu formatierenden Laufwerks  */
 BYTE         AktDriveType;   /* Typ des aktuellen Diskettenlaufwerks */
 PhysDataType PData;                 /* physikalische Formatparameter */
 LogDataType  LData;                      /* logische Formatparameter */
 void far     *AlteDDPT;                      /* Zeiger auf alte DDPT */
 char         *Param;             /* zur Auswertung der Kommandozeile */
 BYTE         ok;                          /* Flag fr Programmablauf */
 int          ExitCode;

 printf( "DFC  -  (c) 1992 by Michael Tischer\n\n" );

 /*-- Kommandozeile auswerten ----------------------------------------*/

 if ( argc > 1 )                              /* Parameter angegeben? */
  {                                                             /* Ja */
    Param = argv[ 1 ];       /* Laufwerk ermitteln ( 0 = a:, 1 = b: ) */
    AktDrive = upcase( Param[ 0 ] ) - 65;
    AktDriveType = GetDriveType( AktDrive );  /* Typ aktuelles Laufw. */
    if ( AktDriveType > 0 )                    /* Laufwerk vorhanden? */
     if (GetFormatParameter( argv[ 2 ], AktDriveType, &PData, &LData ))
      {
       DiskPrepare( AktDrive, PData );
       AlteDDPT = GetIntVec( 0x1E );              /* Alte DDPT merken */
       SetIntVec( 0x1E, PData.DDPT );             /* Neue DDPT Setzen */

       Param = argv[ 3 ];
       if ( ok = PhysicalFormat( AktDrive, PData,
                 (BYTE) ( upcase( Param[ 0 ] ) != 'N' ) ) )
        {
         printf( "\rBootsektor u. FAT schreiben       " );
         ok = LogicalFormat( AktDrive, PData, LData );
        }

       /*-- Auswertung des Formatiervorgangs -------------------------*/

       if ( ok )
        {
         printf( "\rFormatierung o.k.              \n" );
         ExitCode = 0;                /* Programm erfolgreich beendet */
        }
       else
        {
         printf( "\rFormatierung wg. Fehler abgebrochen\n ");
         ExitCode = 1;          /* Formatieren mit Fehler abgebrochen */
        }
       SetIntVec( 0x1E, AlteDDPT );         /* Alte DDPT restaurieren */
      }
     else
      {
       printf( "Das angegeben Format kann auf dem Laufwerk " \
               "nicht formatiert werden!\n" );
       ExitCode = 2;          /* Laufwerk und Format nicht kompatibel */
      }
    else
     {
      printf( "Das angegebene Laufwerk existiert nicht!\n" );
      ExitCode = 3;                       /* Laufwerk nicht vorhanden */
     }
  }
 else
   {
    printf( "Aufruf: DFP Laufwerk   Format   [ NV ]" );
    printf( "                                " );
    printf( "                                " );
    printf( "  A: oder B:                    " );
    printf( "                                 " );
    printf( "   360, 720, 1200, 1440          " );
    printf( "                                  " );
    printf( "                 NV = kein Verify " );
    ExitCode = 4;                                    /* Aufruf falsch */
   }
 return( ExitCode );             /* Programm mit Rckgabewert beenden */
}
