/**********************************************************************/
/*                              D I R C 1                             */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : Zeigt alle Dateien in einem beliebigen Ver-    */
/*                     zeichnis inklusive Unterverzeichnissen und     */
/*                     Volume-Namen auf dem Bildschirm an.            */
/*                     Die Abfrage der Dateien erfolgt dabei ber     */
/*                     direkte Aufrufer der DOS-Funktionen 4Eh und    */
/*                     4Fh.                                           */
/*                     Siehe in diesem Zusammenhang auch DIR2.C       */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 15.08.1987                                     */
/*    letztes Update : 15.10.1991                                     */
/*--------------------------------------------------------------------*/
/*    Speichermodell : SMALL                                          */
/**********************************************************************/
                                                              
/*== Include-Dateien einbinden =======================================*/
                                                              
#include <dos.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>
#include <conio.h>

/*== Typedefs ========================================================*/
                                                              
typedef unsigned char BYTE;               /* wir basteln uns ein BYTE */
typedef struct {           /* DIR-Struktur der Funktionen 4Eh und 4Fh */
                BYTE          Reserviert[21];
                BYTE          Attribut;
                unsigned int  Time;
                unsigned int  Date;
                unsigned long Size;
                char          Name[13];
               } DIRSTRUCT;
                                                              
/*== Konstanten ======================================================*/
                                                              
/*-- Attribute fr die Dateisuche ------------------------------------*/

#define ATTR_RDONLY 0x01                               /* Read-Only   */
#define ATTR_HIDDEN 0x02                               /* Versteckt   */
#define ATTR_SYSTEM 0x04                               /* System      */
#define ATTR_LABEL  0x08                               /* Volume-Name */
#define ATTR_DIREC  0x10                               /* Directory   */
#define ATTR_ARCH   0x20                               /* Archiviert  */
#define ATTR_ALL    0x3F                    /* alle Arten von Dateien */

#define TRUE   ( 0 == 0 )        /* Konstanten, erleichtern das Lesen */
#define FALSE  ( 0 == 1 )        /* des Programmtextes                */

#define EINTR  14         /* soviele Directory-Eintrge sind sichtbar */
#define EZ     (20-EINTR >> 1)  /* erste Zeile des Directory-Fensters */
#define NOF    0x07              /* weie Schrift auf schwarzem Grund */
#define INV    0x70     /* schwarze Schrift auf weiem Grund (invers) */

/*== Makros ==========================================================*/

#ifdef MK_FP                          /* Makro MK_FP schon definiert? */
  #undef MK_FP                              /* Ja, dann Makro lschen */
#endif

#define MK_FP(seg,ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))

/**********************************************************************/
/* PRINT : Wie PRINTF, schreibt die Zeichenkette aber direkt in den   */
/*         Video-RAM                                                  */
/* Eingabe : SPALTE = die Ausgabespalte                               */
/*           ZEILE  = die Ausgabezile                                 */
/*           FARBE  = die Ausgabefarbe                                */
/*           STRING = Zeiger auf den PRINTF-String                    */
/*           ...    = weitere Argumente                               */
/* Ausgabe : keine                                                    */
/**********************************************************************/

void Print( int Spalte, int Zeile, BYTE Farbe, char * String, ...)
{
 struct vr {                   /* eine Bildschirmposition als 2 Bytes */
            BYTE zeichen,                           /* der ASCII-Code */
                 attribut;                 /* das zugehrige Attribut */
           } far * lptr;      /* Laufzeiger zum Zugriff auf Video-RAM */
 va_list parameter;              /* Parameter-Liste fr VA_... Macros */
 char ausgabe[255],                 /* Puffer fr formatierten String */
      *aptr = ausgabe;                 /* zum Durchlaufen des Strings */
 static unsigned int vioseg = 0;
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                              
 if ( vioseg == 0 )                                 /* erster Aufruf? */
  {                     /* Ja, Segmentadresse des Video-RAM ermitteln */
   Register.h.ah = 0x0F;
   int86(0x10, &Register, &Register);
   vioseg = ( Register.h.al == 7 ? 0xb000 : 0xb800 );
  }

 va_start( parameter, String );                /* Parameter umwandeln */
 vsprintf( ausgabe, String, parameter );               /* formatieren */
 lptr = (struct vr far *)
          MK_FP( vioseg, ( Zeile * 80 + Spalte ) << 1 );

 for ( ; *aptr ; )                              /* String durchlaufen */
  {
   lptr->zeichen = *aptr++;                   /* Zeichen in Video-RAM */
   lptr++->attribut = Farbe;           /* Attribut des Zeichen setzen */
  }
}

/**********************************************************************/
/* SCROLLUP: schiebt einen Bildschirmbereich um eine oder mehrere     */
/*           Zeilen nach oben oder lscht ihn                         */
/* Eingabe : ANZAHL   = Anzahl der Zeilen, um die gescrollt wird      */
/*           FARBE    = Farbe bzw. Attribut fr die Leerzeilen        */
/*           SPALTEOL = Spalte der oberen linken Ecke des Bereichs    */
/*           ZEILEOL  = Zeile der oberen linken Ecke des Bereichs     */
/*           SPALTEUR = Spalte der unteren rechten Ecke des Bereichs  */
/*           ZEILEUR  = Zeile der unteren rechten Ecke des Bereichs   */
/* Ausgabe : keine                                                    */
/* Info    : wird als Anzahl 0 bergeben, so wird der Bildschirm-     */
/*           bereich mit Leerzeichen gefllt                          */
/**********************************************************************/
                                                              
void ScrollUp( BYTE Anzahl, BYTE Farbe, BYTE SpalteOL,
               BYTE ZeileOL, BYTE SpalteUR, BYTE ZeileUR)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                              
 Register.h.ah = 6;                                /* Funktionsnummer */
 Register.h.al = Anzahl;                             /* Anzahl Zeilen */
 Register.h.bh = Farbe;                     /* Farbe der Leerzeile(n) */
 Register.h.ch = ZeileOL;               /* Koordinaten des zu scroll- */
 Register.h.cl = SpalteOL;              /* enden oder zu lschenden   */
 Register.h.dh = ZeileUR;               /* Bildschirmfensters setzen  */
 Register.h.dl = SpalteUR;
 int86(0x10, &Register, &Register);         /* Interrupt 10h aufrufen */
}
                                                              
/**********************************************************************/
/* SETPOS: Stellt die Position des Cursors in der aktuellen Bild-     */
/*         schirmseite ein                                            */
/* Eingabe : SPALTE = neue Cursorspalte                               */
/*           ZEILE  = neue Cursorzeile                                */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                              
void SetPos( BYTE Spalte, BYTE Zeile)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                              
 Register.h.ah = 2;                                /* Funktionsnummer */
 Register.h.bh = 0;                                /* Bildschirmseite */
 Register.h.dh = Zeile;                            /* Bildschirmzeile */
 Register.h.dl = Spalte;                          /* Bildschirmspalte */
 int86(0x10, &Register, &Register);         /* Interrupt 10h aufrufen */
}
                                                              
/**********************************************************************/
/* CLS: aktuelle Bildschirmseite lschen und Cursor in obere linke    */
/*      Ecke setzen                                                   */
/* Eingabe : keine                                                    */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                              
void Cls( void )

{
 ScrollUp(0, NOF, 0, 0, 79, 24);                /* Bildschirm lschen */
 SetPos(0, 0);                                       /* Cursor setzen */
}
                                                              
/**********************************************************************/
/* BILDSCHIRMAUFBAU: bereitet den Bildschirm fr die Ausgabe des      */
/*                   Directory vor                                    */
/* Eingabe : keine                                                    */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                              
void BildschirmAufbau( void )
{
 BYTE i;                                           /* Schleifenzhler */
                                                              
 Cls();                                         /* Bildschirm lschen */
 Print( 14, EZ, NOF,
        "ͻ");
 Print( 14, EZ+1, NOF,
        " Dateiname   Gre    Datum      Zeit   RHSVD");
 Print( 14, EZ+2, NOF,
        "Ķ");
 for (i = EZ+3; i < EZ+3+EINTR; i++)
  Print( 14, i, NOF,
         "                                             ");
 Print( 14, EZ+EINTR+3, NOF,
        "ͼ");
}
                                                              
/**********************************************************************/
/* PRINTDATA: Informationen ber einen Eintrag ausgeben               */
/* Eingabe : EINTRAG = Zeiger auf eine DirStrukt mit Datei-           */
/*                      informationen                                 */
/*           ZEILE   = Bildschirmzeile des Eintrags                   */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                              
void PrintData(DIRSTRUCT *Eintrag, BYTE Zeile)
{
 BYTE i, j;                                        /* Schleifenzhler */
 static char *Monat[] =        /* Vektor mit Pointern auf Monatsnamen */
                       {
                        "JAN", "FEB", "MR", "APR", "MAI", "JUN",
                        "JUL", "AUG", "SEP", "OKT", "NOV", "DEZ"
                       };
                                                              
 /*-- die verschiedenen Dateiinformationen ausgeben ------------------*/

 Print(15, Zeile, NOF, "%s", Eintrag->Name);

 Print( 28, Zeile, NOF, "%7lu", Eintrag->Size);
 Print( 36, Zeile, NOF, "%2d. %s %4d", Eintrag->Date & 31,
        Monat[(Eintrag->Date >> 5 & 15) - 1],
        (Eintrag->Date >> 9) + 1980);
 Print(49, Zeile, NOF, "%2d Uhr %2d", Eintrag->Time >> 11,
                                      Eintrag->Time >> 5 & 63 );

 for (i = j = 1; i <= 16; i <<= 1, ++j)    /* Dateiattribute ausgeben */
  Print( 58+j, Zeile, NOF, "%c", (Eintrag->Attribut & i) ? 'X' : ' ' );
}
                                                              
/**********************************************************************/
/* FINDFIRST: den ersten Directory-Eintrag einlesen                   */
/* Eingabe : PFAD     = Zeiger auf Suchpfad mit Dateimaske            */
/*           ATTRIBUT = Such-Attribute                                */
/* Ausgabe : TRUE, wenn ein Eintrag gefunden wurde, sonst FALSE       */
/* Info    : der Eintrag wird in den DTA eingelesen                   */
/**********************************************************************/
                                                              
BYTE findfirst( char *Pfad, BYTE Attribut )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
 struct SREGS Segmente;              /* nimmt die Segmentregister auf */
                                                              
 segread(&Segmente);           /* Inhalt der Segmentregister einlesen */
 Register.h.ah = 0x4E;        /* Funktionsnummer fr Search for first */
 Register.x.cx = Attribut;      /* Attribute, nach denen gesucht wird */
 Register.x.dx = (unsigned int) Pfad;   /* Offsetadresse des Suchpfad */
 intdosx(&Register, &Register, &Segmente);  /* DOS-Intr. 21h aufrufen */
 return( !Register.x.cflag );  /* Carry-Flag = 0: eine Datei gefunden */
}
                                                              
/**********************************************************************/
/* FINDNEXT: den folgenden Directory-Eintrag einlesen                 */
/* Eingabe : keine                                                    */
/* Ausgabe : TRUE, wenn ein Eintrag gefunden wurde, sonst FALSE       */
/* Info    : der Eintrag wird in den DTA eingelesen                   */
/**********************************************************************/
                                                              
BYTE findnext( void )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                              
 Register.h.ah = 0x4F;         /* Funktionsnummer fr Search for next */
 intdos(&Register, &Register);              /* DOS-Intr. 21h aufrufen */
 return( !Register.x.cflag );  /* Carry-Flag = 0: eine Datei gefunden */
}
                                                              
/**********************************************************************/
/* SETDTA: setzt die DTA auf eine Variable im Datensegment            */
/* Eingabe : OFFSET = Offset der DTA innerhalb des Datensegments      */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                              
void SetDTA( unsigned int Offset )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
 struct SREGS Segmente;              /* nimmt die Segmentregister auf */
                                                              
 segread(&Segmente);           /* Inhalt der Segmentregister einlesen */
 Register.h.ah = 0x1A;              /* Funktionsnummer fr DTA setzen */
 Register.x.dx = Offset;             /* Offsetadresse ins DX-Register */
 intdosx(&Register, &Register, &Segmente);  /* DOS-Intr. 21h aufrufen */
}
                                                              
/**********************************************************************/
/* DIR: kontrolliert das Einlesen und die Ausgabe des Directories     */
/* Eingabe : PFAD     = Zeiger auf Suchpfad mit Dateimaske            */
/*           ATTRIBUT = Such-Attribute                                */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                              
void Dir(char *Pfad, BYTE Attribut )
{
 int       AnzEintraege,      /* Gesamtanzahl der gefundenen Eintrge */
           AnzBild;                    /* Anzahl der Eintrge in Bild */
 DIRSTRUCT Eintrag;                          /* ein Directory-Eintrag */
                                                              
 SetDTA( (unsigned int) &Eintrag );       /* EINTRAG ist der neue DTA */
 BildschirmAufbau();     /* Bildschirm fr Directory-Ausgabe aufbauen */
                                                              
 AnzBild = AnzEintraege = 0;/* noch kein Eintrag im Fenster angezeigt */
                                      /* noch keinen Eintrag gefunden */
 if (findfirst(Pfad, Attribut))              /* ersten Eintrag suchen */
  {                                            /* eine Datei gefunden */
   do          /* Datei ausgeben und mit GetNext() die nchste suchen */
    {
     PrintData(&Eintrag, EZ+EINTR+2);             /* Eintrag ausgeben */
     if (++AnzBild == EINTR )                /* ist das Fenster voll? */
      {
       AnzBild = 0;                 /* wieder ein Fenster voll machen */
       Print(14, EZ+4+EINTR, INV,
          "                 Bitte Taste drcken               ");
       getch();                                   /* auf Taste warten */
       Print(14, EZ+4+EINTR, NOF,
          "                                                   ");
      }
     ScrollUp(1, NOF, 15, EZ+3, 63, EZ+2+EINTR);
     Print(15, EZ+2+EINTR, NOF,
        "                                             ");
     ++AnzEintraege;
    }
   while ( findnext() );    /* Schleife beende, wenn keine Datei mehr */
  }

 SetPos(14, EZ+4+EINTR);
 switch (AnzEintraege)
  {
   case 0  : printf("keine Dateien gefunden");
             break;
   case 1  : printf("eine Datei gefunden");
             break;
   default : printf("%d Dateien gefunden", AnzEintraege);
             break;
  }
}
                                                              
/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/
                                                              
void main( int Anzahl, char *Argumente[] )
{
 switch ( Anzahl )                  /* je nach Anzahl der bergebenen */
  {                                 /* Argumente reagieren            */
   case 1  : Dir("*.*", ATTR_ALL );      /* alle Dateien im aktuellen */
             break;                      /* Verzeichnis anzeigen      */
   case 2  : Dir( Argumente[1], ATTR_ALL ); /*alle Dateien im angege- */
             break;                         /*benen Verz. anzeigen    */
   default : printf("Ungltige Anzahl Parameter\n");
  }
}
