/**********************************************************************/
/*                   D M A U T I L  . H                               */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Header-Datei fr DMA_UTIL.C                      */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 20.03.1994                                       */
/*  letztes Update : 25.03.1994                                       */
/**********************************************************************/
#ifndef __INC_DMA_UTIL_H
#define __INC_DMA_UTIL_H

#include "types.h"

/*- Bits des Statusregisters (0x08, 0xD0 ) Lesen ---------------------*/
#define STATUS_REQ3 0x80         /* Bit Gesetzt: jeweiliger DMA-Kanal */
#define STATUS_REQ2 0x40         /* erhielt DMA-Anforderung.          */
#define STATUS_REQ1 0x20         /* Request                           */
#define STATUS_REQ0 0x10
#define STATUS_TC3  0x08        /* Bit gesetzt: Seit letzem Lesen des */
#define STATUS_TC2  0x04        /* Statusregisters wurde eine DMA-    */
#define STATUS_TC1  0x02        /* bertragung abgeschlossen.         */
#define STATUS_TC0  0x01        /* Terminal Count                     */

/*- Bit des Befehlsregisters (0x08, 0xD0) Schreiben ------------------*/
/*- Bit 7 gesetzt: DMA Acknoledge Leitung HIGH aktiv -----------------*/
#define COMMAND_DACKLEVEL 0x80

/*- Bit 6 gesetzt: REQ Acknoledge Leitung LOW aktiv ------------------*/
#define COMMAND_DREQLEVEL 0x40

/**- Nach DMA-Request sendet Controller DMA-Acknowledge! -------------*/
/*- Bit 5 gesetzt: EXTENDED Write, sonst LATE Write ------------------*/
#define COMMAND_EXTWRITE 0x20

/*- Bit 4 gesetzt: feste Prioritt -----------------------------------*/
#define COMMAND_FIXEDPRI 0x10

/*- Bit 3 gesetzt: komprimierte Taktung ------------------------------*/
#define COMMAND_COMPRESS 0x08

/*- Bit 2 gesetzt: Controller deaktiviert ----------------------------*/
#define COMMAND_INACTIVE 0x04

/*- Bit 1 gesetzt: Adress Hold fr Kanal 0/4 deaktiviert -------------*/
#define COMMAND_ADH0     0x02

/*- Bit 0 gesetzt: Speicher/Speicher, sonst Speicher/Peripherie ------*/
#define COMMAND_MEM2MEM  0x01

/*- Bits des Request-Registers ( 0x09, 0xD2 ) Schreiben --------------*/
#define REQUEST_RESERVED 0xF8      /* Reserverte Bits immer =0 setzen */
#define REQUEST_SET      0x04                   /* DMA Request setzen */
#define REQUEST_CLR      0x00                  /* DMA-Request lschen */
#define REQUEST_MSK      0x03 /* In unteren beiden Bits Kanal angeben */

/*- Bits des Kanalmaskierungsregisters ( 0x0A, 0xD4 ) Schreiben ------*/
#define CHANNEL_RESERVED 0xF8      /* Reserverte Bits immer =0 setzen */
#define CHANNEL_SET      0x04          /* DMA Kanal maskieren/sperren */
#define CHANNEL_CLR      0x00                  /* DMA Kanal freigeben */
#define CHANNEL_MSK      0x03 /* In unteren beiden Bits Kanal angeben */

/*- Bits des Modusregisters (0x0B, 0xD6) Schreiben -------------------*/
#define MODE_DEMAND     0x00            /* "Auf Abruf" transferieren  */
#define MODE_SINGLE     0x40             /* Einzelwerte transferieren */
#define MODE_BLOCK      0x80                         /* Blocktransfer */
#define MODE_CASCADE    0xC0              /* Kaskadiert transferieren */

#define MODE_DECREMENT  0x20                       /* Dekrementierten */
#define MODE_AUTOINIT   0x10         /* Autoinitialisierung nach Ende */

#define MODE_VERIFY     0x00                          /* Verifizieren */
#define MODE_WRITE      0x04                 /* In Speicher Schreiben */
#define MODE_READ       0x08                    /* Aus Speicher Lesen */
#define MODE_INVALID    0x0C                              /* Ungltig */

#define MODE_CHANNELMSK 0x03  /* In unteren beiden Bits Kanal angeben */

/*- Prototypen -------------------------------------------------------*/
VOID   dma_SetRequest  ( INT iChan );
VOID   dma_ClrRequest  ( INT iChan );
VOID   dma_SetMask     ( INT iChan );
VOID   dma_ClrMask     ( INT iChan );
VOID   dma_ClrFlipFlop ( INT iChan );
BYTE   dma_ReadStatus  ( INT iChan );
WORD   dma_ReadCount   ( INT iChan );
VOID   dma_Mem2Mem( BYTE bPage, WORD uSource, WORD uDest, WORD  uSize );
WORD   dma_Until64kPage( LPVOID lpMem, WORD uSize );
VOID   dma_SetChannel  ( INT iChan, LPVOID lpMem,
                         WORD uSize, BYTE bMode );
#define DMA_8BIT  ( BYTE) 0x00       
#define DMA_16BIT ( BYTE )0x01

LPVOID dma_AllocMem    ( WORD uSize );
VOID   dma_Free        ( LPVOID lpMem );
#endif
