/**********************************************************************/
/*                   D S P . C                                        */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Demo-Programm zur Veranschaulichung der DSP-     */
/*                   Programmierung                                   */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 20.03.1994                                       */
/*  letztes Update : 25.03.1994                                       */
/* Compiler Optionen: 1 Byte Struct Alignment, Keine Optimierungen    */
/* COMPILER: Borland C++ 3.1, Microsoft Visual C++ 1.5                */
/**********************************************************************/

/*- Include Dateien einbinden ----------------------------------------*/
#include <fcntl.h>
#include <dos.h>
#include <process.h>

#include "sbutil.h"
#include "dmautil.h"
#include "dsputil.h"
#include "mixutil.h"
#include "irqutil.h"
#include "args.h"

/*- "Bibliotheken" einbinden -----------------------------------------*/
#include "sbutil.c"
#include "dmautil.c"
#include "dsputil.c"
#include "mixutil.c"
#include "irqutil.c"
#include "args.c"

#define DSP_NAME "DSP"

SBBASE SBB;

#define BUFSIZE 8192U

/**********************************************************************/
/*                 H A U P T P R O G R A M M                          */
/**********************************************************************/
VOID main( INT argc, PCHAR argv[] )
{
  PCHAR      pFile;
  LONG       lFrequency;
  INT        iADC, iDuration, iHandle;
  PCHAR      lpRecordSource;
  INT        iSource, iSt;
  LPBYTE     lpBuffer;
  WORD       uMemSize, uFrq;
  DSPRECPLAY dspHeader;

  if( sb_GetEnviron( &SBB, getenv( "BLASTER" ) ) != NO_ERROR )
  {
    printf("BLASTER-Umgebungsvariable nicht vorhanden\n");
    exit(0);
  }

  if( dsp_SetBase( &SBB, TRUE ) == NO_ERROR )
  {
    pFile = NULL;       /* Dateiname der die Daten enthaltende Datei */
    GetNArg( argc, argv, "-", ( PCHAR *)&pFile, 1 );

    if( pFile == NULL )
    {
      printf("Aufruf: %s [-bX][-dX][-fX][-r][-s][-w] Dateiname\n",
             DSP_NAME );
      printf("\n-bX   : DMA Buffer Gre \n");
      printf("-dX   : Lnge der Aufnahme in Sekunden\n");
      printf("-fX   : Frequenz in Hz\n");
      printf("-r    : Aufnahme (sonst Wiedergabe)\n");
      printf("-s    : Stereo (sonst Mono)\n");
      printf("-w    : 16-Bit Transfer (Word) \n");
      printf("-iMIC/CD/LINE : Aufnahmequelle (Input)\n");
      exit(0);
    }

    iADC = 0;                                    /* Aufnahme (Record) */
    iADC = GetArg( argc, argv, "-r", _none, NULL, 0 );

    dspHeader.iStereo = FALSE;                     /* Defaults setzen */
    dspHeader.uFrequency = 10000;
    dspHeader.iBits = 8;

    dsp_FileOpen( pFile, iADC, &iHandle );
    if( !iADC ) dsp_ReadHeader( iHandle, &dspHeader );

    if( GetArg( argc, argv, "-s", _none, NULL, 0 ) )
        dspHeader.iStereo = TRUE;

    lFrequency = dspHeader.uFrequency;                    /* Frequenz */
    if( GetArg( argc, argv, "-f", _long, &lFrequency, 1 ) )
       dspHeader.uFrequency = ( WORD )lFrequency;

    if( SBB.uDspVersion >= DSP_4XX )
        if( GetArg( argc, argv, "-w", _none, NULL, 0 ) )
            dspHeader.iBits = 16;

    if( iADC )  dsp_WriteHeader( iHandle, &dspHeader );

    iDuration = 10;                        /* Spieldauer bei Aufnahme */
    GetArg( argc, argv, "-d", _int, &iDuration, 1 );

    uMemSize = BUFSIZE;   /* fr DMA zur Verfgung stehender Speicher */
    GetArg( argc, argv, "-b", _int, &uMemSize, 1 );

    lpRecordSource = NULL;
    GetArg( argc, argv, "-i", _string, &lpRecordSource, 1 );
    iSource = -1;
    if( _fstrcmp( lpRecordSource, "MIC" ) == 0 )  iSource = MIC;
    else
    if( _fstrcmp( lpRecordSource, "CD" ) == 0 )   iSource = CD;
    else
    if( _fstrcmp( lpRecordSource, "LINE" ) == 0 ) iSource = LINE;

    sb_Print( &SBB );
  }
  else
  {
    printf("Keine DSP-Ansteuerung mglich!");
    exit( 0 );
  }

  if( ( dspHeader.iStereo ) && ! dsp_CanStereo() )
    printf("WARNUNG! Stereomodus wird NICHT untersttzt.\n");
  if( dspHeader.iBits > dsp_MaxBits() )
    printf("WARNUNG! 16-BitModus wird NICHT untersttzt!\n");

  uFrq = dspHeader.uFrequency;
  iSt  = dspHeader.iStereo;
  dsp_AdjustFrq( &uFrq, iADC, &iSt );
  if( uFrq < dspHeader.uFrequency )
    printf("WARNUNG! Frequenz zu hoch! Wird angepasst!\n");

  uMemSize &= ~0x0003U;           /* uMemSize ist nun durch 4 Teilbar */
  lpBuffer = dma_AllocMem( uMemSize );     /* DMA-Speicher allokieren */
  if( lpBuffer )
  {
    dsp_DoRECPLAY( iHandle, iADC, iSource, &dspHeader, iDuration,
                   lpBuffer, uMemSize );
    dma_Free( lpBuffer );                       /* Speicher freigeben */
  }
  else printf("Kein freier Speicher mehr!\n");
  dsp_FileClose( iHandle );                        /* Datei schlieen */
}
