/**********************************************************************/
/*                   D S P U T I L  . C                               */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Stellt Funktionen zur Programmierung des Sound-  */
/*                   blaster DSP's zur Verfgung.                     */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 20.03.1994                                       */
/*  letztes Update : 25.03.1994                                       */
/**------------------------------------------------------------------**/
/* Hinweis: Wird dieses Modul mit definiertem Symbol DSP_VERSIONONLY  */
/*          compiliert, werden nur die zur Ermittlung der Version be- */
/*          ntigten Funktionen compiliert!                           */
/**********************************************************************/
#ifndef __DSPUTIL_C          /* DSPUTIL.C kann auch #Included werden! */
#define __DSPUTIL_C

/*-- Include-Dateien einbinden ---------------------------------------*/

#include <dos.h>
#include <io.h>
#include <stdio.h>
#include <errno.h>

#include "dsputil.h"
#include "dmautil.h"
#include "mixutil.h"
#include "irqutil.h"

#define _SB_NONE 0
#define _SB_1XX 1   /* DSP-Version 1.00 bis 1.99 */
#define _SB_2XX 2   /* DSP-Version 2.00 bis 2.99 */
#define _SB_3XX 3   /* DSP-Version 3.00 bis 3.99 */
#define _SB_4XX 4   /* DSP-Version 4.00 bis 4.99 */
#define _SB_XXX 5   /* DSP-Version > 4.99 */

/*-- Globale Variablen -----------------------------------------------*/
static LPCHAR pDspNames[ ]= { "Unbekannte Soundkarte",
                              "SB 1.5 oder SB MCV",
                              "SB 2.0",
                              "SB Pro oder SB Pro MCV",
                              "SB 16, SB 16 ASP",
                              "> SB16" };

SBBASE _FP DSPBASE;

#ifndef DSP_VERSIONONLY
VOID ( _FP *lpUserFunc )( LONG ) = NULL;
VOID ( _FP *lpDSP4UserFunc )( BYTE ) = NULL;
VOID ( _interrupt _FP *lpOldInt )() = NULL;

static LONG lIRQServed = 0L;
static LONG lOldIRQServed =0L;
#endif

/*-- Funktionen ------------------------------------------------------*/

/**********************************************************************/
/* dsp_SetBase : Setze SBBase-Struktur fr Gebrauch der dsp_???       */
/*               Funktionen.                                          */
/**------------------------------------------------------------------**/
/* Eingabe : pSBBASE - Adresse der durch 'SB_SetEnviron'              */
/*                     initialisierten Struktur.                      */
/*           iReset   - DSP soll zurckgesetzt werden                 */
/* Ausgabe :              == 0 - DSP-Funktionen konnten ordnungsgem */
/*                               initialisiert werden                 */
/*           DSP_ERRRESET    == 1 - Fehler bei DSP-Reset              */
/*           DSP_ERRVERSION  == 2 - Fehler bei Ermittlung der V-Nummer*/
/*                           ==-1 - Keine SBBASE-Struktur bergeben   */
/*                                  oder iDspPort == -1               */
/**********************************************************************/
WORD dsp_SetBase( PSBBASE pSBBASE, WORD iReset )
{
  if( pSBBASE )
    if( pSBBASE->iDspPort != -1 )
    {
      DSPBASE = *pSBBASE;
      if( iReset ) if( dsp_Reset() ) return DSP_ERRRESET;
      if( dsp_GetVersion( pSBBASE ) ) return DSP_ERRVERSION;
      if( pSBBASE->uDspVersion >= DSP_4XX )
        if( pSBBASE->iDspDmaW == -1 )
            pSBBASE->iDspDmaW = pSBBASE->iDspDmaB;

#ifndef DSP_VERSIONONLY
      mix_SetBase( pSBBASE, FALSE );
#endif

      DSPBASE = *pSBBASE;
      return NO_ERROR;
    }
  return ERROR;
}

/**********************************************************************/
/* dsp_write :  Daten an DSP bergeben                                */
/**------------------------------------------------------------------**/
/* Eingabe : iVal - zu bermittelndes Daten-Byte oder Kommando        */
/* Ausgabe : == 0 - Datum konnte nicht bertragen werden              */
/*           <> 0 - Datum wurde an DSP bermittelt                    */
/**********************************************************************/
WORD dsp_Write( WORD iVal )
{ WORD cx = 65535U;    /* Interner Zhler == 0 => bertragungsfehler! */
      /* Write-Status lesen, und warten bis OK. Zhler dekrementieren */
  while( ( inp( DSPBASE.iDspPort + DSP_WRITESTATUS ) & 0x80 ) && cx )
    cx--;
        /* Zhler != 0, dann Write-Status innerhalb vorgegebener Zeit */
        /* bertragung eines Datenbytes also erlaubt:                 */
  if( cx ) outp( DSPBASE.iDspPort + DSP_WRITECMDDATA, ( BYTE )iVal );
  return cx;                /* Zhler als Fehlerindikator zurckgeben */
}

/**********************************************************************/
/* dsp_Read : Daten vom DSP lesen                                     */
/**------------------------------------------------------------------**/
/* Eingabe : pVal - Adresse des 'int', das den gelesenen Wert         */
/*                   entgegennehmen soll ('Ausgabeparameter')         */
/* Ausgabe : == 0 - DSP stellte kein Datum zur Verfgung              */
/*           <> 0 - DSP stellte Datum zur Verfgung                   */
/**------------------------------------------------------------------**/
/* Info : - Das Auslesen der DSP-Daten macht nur Sinn, wenn vorher    */
/*          ein entsprechender Abfrage-Befehl gesendet wurde.         */
/**********************************************************************/
WORD dsp_Read( PBYTE pVal )
{ WORD cx = 65535U ;   /* Interner Zhler == 0 => bertragungsfehler! */
       /* Read-Status lesen, und warten bis OK. Zhler dekrementieren */
  while( !( inp( DSPBASE.iDspPort + DSP_READSTATUS ) & 0x80 ) && cx )
    cx--;
     /* Zhler != 0, dann Read-Status innerhalb vorgegebener Zeit OK: */
     /* Lesen eines Datenbytes also erlaubt:                          */
  if( cx ) *pVal = ( BYTE )inp( DSPBASE.iDspPort + DSP_READDATA );
  return cx;                /* Zhler als Fehlerindikator zurckgeben */
}

/**********************************************************************/
/* dsp_Reset : DSP in Ausgangszustand zurckversetzen                 */
/**-------------------------------------------------------------------*/
/* Ausgabe : NO_ERROR( 0 ) - Reset ordnungsgem ausgefhrt           */
/*           ERROR ( -1 )  - Reset nicht ausgefhrt                   */
/**------------------------------------------------------------------**/
/* Info : - Reset-Sequenz - Zuerst 1, dann 0 an Reset-Port des DSP    */
/*                          bermitteln, dann DSP-Auslesen und auf    */
/*                          Empfang von 0xAA warten.                  */
/**********************************************************************/
WORD dsp_Reset( VOID )
{ BYTE bVal = 0;
                   /* Reset-Port zuerst mit 1, dann mit 0 beschreiben */
  outp( DSPBASE.iDspPort + DSP_RESET, 1 );
  inp( DSPBASE.iDspPort + DSP_RESET );        /* mind. 3              */
  inp( DSPBASE.iDspPort + DSP_RESET );        /* Microsekunden warten */
  inp( DSPBASE.iDspPort + DSP_RESET );
  outp( DSPBASE.iDspPort + DSP_RESET, 0 );
  inp( DSPBASE.iDspPort + DSP_RESET );
  inp( DSPBASE.iDspPort + DSP_RESET );
  inp( DSPBASE.iDspPort + DSP_RESET );
  dsp_Read( &bVal );                            /* Wert von DSP-Lesen */
  return (bVal == 0xAA) ? NO_ERROR : ERROR;      /* Wert = 0xAA => OK */
}

/**********************************************************************/
/* dsp_GetVersion : DSP-Versionsnummer ermitteln                      */
/*--------------------------------------------------------------------*/
/* Eingabe : pSBBASE - Adresse der zu initialisierenden               */
/*                      SBase-Struktur                                */
/* Ausgabe : NO_ERROR ( 0 )       - kein Fehler                       */
/*           DSP_ERRVERSION ( 2 ) - bertragungsfehler                */
/**------------------------------------------------------------------**/
/* Info : - Neben dem Feld uDspVersion wird auch der Name der         */
/*          entdeckten SoundBlaster-Karte angegeben                   */
/**********************************************************************/
WORD dsp_GetVersion( PSBBASE pSBBASE)
{ BYTE bV1, bV2;
  if( dsp_Write(DSP_GETVERSION) && dsp_Read(&bV1) && dsp_Read(&bV2) )
  {                                           /* Versions-Wort bilden */
    pSBBASE->uDspVersion = MAKEWORD( bV1, bV2 );
    if((bV1>=1)&&(bV1<=4))  pSBBASE->pDspName = pDspNames[ bV1 ];
    else                    pSBBASE->pDspName = pDspNames[ _SB_NONE ];
    return NO_ERROR;
  }
  return DSP_ERRVERSION;           /* Fehler bei DSP-Datenbertragung */
}

#ifndef DSP_VERSIONONLY             /* Nur Versionkontrolle verwenden */
/**********************************************************************/
/* dsp_AdjustFrq : Frequenz an Soundkarte anpassen                    */
/**------------------------------------------------------------------**/
/* Eingabe : pFrq    - Adresse des INT's das gewnschte Frequenz      */
/*                     enthlt                                        */
/*           iADC    - Aufnahme (TRUE) oder Wiedergabe (FALSE)        */
/*           iStereo - Stereomodus (TRUE/FALSE)                       */
/* Ausgabe : TRUE - Mono/Stereo gltig                                */
/*           FALSE - kein Stereomodus                                 */
/**------------------------------------------------------------------**/
/* Info : Diese Funktion berprft ob die gegebene Frequenz auf der   */
/*        aktuell vorhandenen Soundblaster Karte verwendet werden     */
/*        kann, bzw. setzt andernfalls die grtmgliche Frequenz.    */
/*        Wird versucht, den Stereomodus einer Karte zu aktivieren,   */
/*        die dies nicht zult, wird FALSE zurckgegegen.            */
/*        Einige der hohen Frequenzen knnen nur von den HI-SPEED     */
/*        Kommandos verwendet werden.                                 */
/**********************************************************************/
WORD dsp_AdjustFrq( PWORD pFrq, INT iADC, PINT pStereo )
{
  if( iADC )
  {
    if( DSPBASE.uDspVersion >= DSP_4XX )
    {
      *pFrq = *pFrq > FRQ4_ADC ? FRQ4_ADC : *pFrq;
      return TRUE;
    }
    if( DSPBASE.uDspVersion >= DSP_3XX )
    {
      if( *pStereo )
        *pFrq = *pFrq > FRQ3_STEREO_ADC ? FRQ3_STEREO_ADC : *pFrq;
      else
        *pFrq = *pFrq > FRQ3_HIMONO_ADC ? FRQ3_HIMONO_ADC : *pFrq;
      return TRUE;
    }
    if( DSPBASE.uDspVersion >= DSP_201 )
    {
      *pFrq = *pFrq > FRQ2p_HIMONO_ADC ? FRQ2p_HIMONO_ADC : *pFrq;
      if( *pStereo )
      {
        *pStereo = FALSE; return FALSE;
      }
      return TRUE;
    }
    if( DSPBASE.uDspVersion >= DSP_1XX )
    {
      *pFrq = *pFrq > FRQ1_MONO_ADC ? FRQ1_MONO_ADC : *pFrq;
      if( *pStereo )
      {
        *pStereo = FALSE; return FALSE;
      }
      return TRUE;
    }
  }
  else
  {
    if( DSPBASE.uDspVersion >= DSP_4XX )
    {
      *pFrq = *pFrq > FRQ4_DAC ? FRQ4_DAC : *pFrq;
      return TRUE;
    }

    if( DSPBASE.uDspVersion >= DSP_3XX )
    {
      if( *pStereo )
        *pFrq = *pFrq > FRQ3_STEREO_DAC ? FRQ3_STEREO_DAC : *pFrq;
      else
        *pFrq = *pFrq > FRQ3_HIMONO_DAC ? FRQ3_HIMONO_DAC : *pFrq;
      return TRUE;
    }

    if( DSPBASE.uDspVersion >= DSP_201 )
    {
      *pFrq = *pFrq > FRQ2p_HIMONO_DAC ? FRQ2p_HIMONO_DAC : *pFrq;
      if( *pStereo )
      {
        *pStereo = FALSE; return FALSE;
      }
      return TRUE;
    }
    if( DSPBASE.uDspVersion >= DSP_1XX )
    {
      *pFrq = *pFrq > FRQ1_MONO_DAC ? FRQ1_MONO_DAC : *pFrq;
      if( *pStereo )
      {
        *pStereo = FALSE; return FALSE;
      }
      return TRUE;
    }
  }
  return FALSE;                                 /* DSP-Version < 1.00 */
}

/**********************************************************************/
/* dsp4_DACFrq : Ausgabefrequenz fr DSP4.00  setzen                  */
/**------------------------------------------------------------------**/
/* Eingabe : uFrq - einzustellende Ausgabefrequenz in Hertz [Hz]      */
/* Ausgabe : NO_ERROR( 0 )        - Frequenz wurde gesetzt            */
/*           DSP_ERR4DACFRQ ( 4 ) - bertragungsfehler                */
/**------------------------------------------------------------------**/
/* Info : - Diese Funktion gibt es erst ab dem DSP4.00!               */
/**********************************************************************/
WORD dsp4_DACFrq( WORD uFrq )
{
  if( dsp_Write( DSP4_DACFRQ ) &&
      dsp_Write( HIBYTE( uFrq ) ) &&
      dsp_Write( LOBYTE( uFrq ) ) ) return NO_ERROR;
  return DSP_ERR4DACFRQ;
}

/**********************************************************************/
/* dsp4_ADCFrq : Eingabefrequenz des DSP4.00 setzen                   */
/**------------------------------------------------------------------**/
/* Eingabe : uFrq - einzustellende Eingabefrequenz in Hertz           */
/* Ausgabe : NO_ERROR( 0 )        - Frequenz wurde gesetzt            */
/*           DSP_ERR4ADCFRQ ( 5 ) - bertragungsfehler                */
/**------------------------------------------------------------------**/
/* Info : - Diese Funktion gibt es erst ab dem DSP4.00!               */
/**********************************************************************/
WORD dsp4_ADCFrq( WORD uFrq )
{
  if( dsp_Write( DSP4_ADCFRQ ) &&
      dsp_Write( HIBYTE( uFrq ) ) &&
      dsp_Write( LOBYTE( uFrq ) ) ) return NO_ERROR;
  return DSP_ERR4ADCFRQ;
}

/**********************************************************************/
/* dsp_SetFrq : Ein-/Ausgabefrequenz des DSP setzen                   */
/**------------------------------------------------------------------**/
/* Eingabe : pFrq - Adresse des WORD's, das Ein-/Ausgabefrequenz in   */
/*                  Hertz [Hz] enthlt. Nach dem Aufruf dieser        */
/*                  Funktion steht in diesem WORD die tatschlich     */
/*                  verwendete Frequenz.                              */
/* Ausgabe : NO_ERROR( 0 )    - Frequenz wurde gesetzt                */
/*           DSP_ERRFRQ ( 3 ) - bertragungsfehler                    */
/**------------------------------------------------------------------**/
/* Info : - Beim DSP3.XX kann die Requenz in 256'er Schritten einge-  */
/*          stellt werden. Welche Frequenz tatschlich verwendet wird,*/
/*          wird von dieser Funktion zurckgegeben.                   */
/**********************************************************************/
WORD dsp_SetFrq( PWORD pFrq )
{
  BYTE bTC;                                          /* Zeitkonstante */
  bTC = (BYTE)( 256L - ( ( 1000000L + ( *pFrq / 2 ) ) / *pFrq ) );
  if( dsp_Write( DSP_SETTIMECONSTANT ) && dsp_Write( bTC ) )
  {
    *pFrq = ( WORD ) ( 1000000L / (256L - bTC ) );
    return NO_ERROR;
  }
  return ERROR;
}

/**********************************************************************/
/* dsp_CanStereo: Untersttzt aktuelle Soundkarte Stereomodus?        */
/*--------------------------------------------------------------------*/
/* Ausgabe : TRUE  - Stereomodus wird untersttzt (ab DSP3.00)        */
/*           FALSE - Stereomodus wird nicht untersttzt.              */
/**********************************************************************/
WORD dsp_CanStereo( void )
{
  return ( DSPBASE.uDspVersion >= DSP_3XX ) ? TRUE : FALSE;
}

/**********************************************************************/
/* dsp_HIMONOADCFrq : Ist angegebene Frequenz eine HISPEED-Frequenz?  */
/*--------------------------------------------------------------------*/
/* Ausgabe : Frquenz fr Mono-Aufnahmen                               */
/**********************************************************************/
WORD dsp_IsHIMONOADCFrq( WORD uFrq )
{
  if( DSPBASE.uDspVersion <= DSP_1XX ) return FALSE;
  if( DSPBASE.uDspVersion <= DSP_201 )
    return ( uFrq <= FRQ2p_HIMONO_ADC );
  if( DSPBASE.uDspVersion <= DSP_3XX )
    return ( uFrq <= FRQ3_HIMONO_ADC );
  if( DSPBASE.uDspVersion <= DSP_4XX )
    return ( uFrq <= FRQ4_ADC );
  return TRUE;
}

/**********************************************************************/
/* dsp_HIMONODACFrq : Ist angegebene Frequenz eine HISPEED-Frequenz?  */
/*--------------------------------------------------------------------*/
/* Ausgabe : Frquenz fr Mono-Wiedergaben                             */
/**********************************************************************/
WORD dsp_IsHIMONODACFrq( WORD uFrq )
{
  if( DSPBASE.uDspVersion <= DSP_1XX ) return FALSE;
  if( DSPBASE.uDspVersion <= DSP_201 )
    return ( uFrq <= FRQ2p_HIMONO_DAC );
  if( DSPBASE.uDspVersion <= DSP_3XX )
    return ( uFrq <= FRQ3_HIMONO_DAC );
  if( DSPBASE.uDspVersion <= DSP_4XX )
    return ( uFrq <= FRQ4_DAC );
  return TRUE;
}

/**********************************************************************/
/* dsp_MaxBits : Hchste Sampleauflsung ermitteln                    */
/*--------------------------------------------------------------------*/
/* Ausgabe : Maximale Anzahl der Bits eines Samples (8, 16)           */
/**********************************************************************/
int dsp_MaxBits( VOID )
{
  return ( DSPBASE.uDspVersion >= DSP_4XX ) ? 16 : 8;
}

/**********************************************************************/
/* dsp_SetSpeaker : DSP Ausgabe an-/ abschalten.                      */
/*--------------------------------------------------------------------*/
/* Eingabe : iState - == 0 DSP Ausgabe auschalten                     */
/*                    <> 0 DSP Ausgabe anschalten                     */
/* Ausgabe : NO_ERROR( 0 )        - Lautsprecher wurde geschaltet     */
/*           DSP_ERRSPEAKER ( 6 ) - bertragungsfehler                */
/**********************************************************************/
WORD dsp_SetSpeaker( WORD iState )
{
   return dsp_Write( iState ? DSP_SPEAKERON : DSP_SPEAKEROFF ) ?
    NO_ERROR : DSP_ERRSPEAKER;
}

/**********************************************************************/
/* dsp_SetTransferSize : Anzahl zu bertragender Bytes an DSP ber-   */
/*                       mitteln.                                     */
/*--------------------------------------------------------------------*/
/* Eingabe : uSize - Anzahl der Bytes in bertragung                  */
/* Ausgabe : NO_ERROR( 0 )         - Anzahl gesetzt                   */
/*           DSP_ERRTRANSSIZE( 7 ) - bertragungsfehler               */
/**********************************************************************/
WORD dsp_SetTransferSize( WORD uSize )
{
   if( uSize )   /* Nur Wenn Anzahl != 0 bergeben wurd  weitermachen */
   {
      uSize--;                            /* bergeben wird Anzahl - 1*/
      if( dsp_Write( DSP_SETTRANSFERSIZE ) &&  /* DSP-Kommando senden */
      dsp_Write( LOBYTE( uSize ) ) &&               /* LO-Byte senden */
      dsp_Write( HIBYTE( uSize ) ) )                /* HI-Byte senden */
        return NO_ERROR;
      return DSP_ERRTRANSSIZE;
   }
   return DSP_ERRILLSIZE;
}

/**********************************************************************/
/* dsp_IrqHandler : Demo DSP Interrupt-Handler                        */
/**------------------------------------------------------------------**/
/* Info : - Dies ist ein gewhnlicher Interrupt-Handler.              */
/*          Um die Programmierung eines eigenen Handlers zu           */
/*          vereinfachen, wird in diesem Handler alles ntige unter-  */
/*          nommen, um die Interrupts zu besttigen. Im Anschlu      */
/*          daran kann eine Benutzer-Funktion aufgerufen werden.      */
/**********************************************************************/
VOID _interrupt _FP dsp_IrqHandler( )
{
  lIRQServed++;              /* Anzahl der bisherigen Aufrufe erhhen */

  if( DSPBASE.uDspVersion >= 0x0400 )          /* DSP-Version >= 4.00 */
  { BYTE who;
                                  /* Welcher Chip hat IRQ ausgelst ? */
    who = ( BYTE )mix_Read( MIX4_IRQSOURCE );

                         /* Benutzerfunktion aufrufen falls vorhanden */
    if( lpDSP4UserFunc ) lpDSP4UserFunc( who );
                                   /* 8-Bit DMA oder Midi-bertragung */
    if (who & MIX4_IRQ8DMA )  inp( DSPBASE.iDspPort + 0x0e );
                                                        /* 16-Bit DMA */
    if (who & MIX4_IRQ16DMA ) inp( DSPBASE.iDspPort + 0x0f );
                                                      /* MPU-401 UART */
    if (who & MIX4_IRQMPU )   inp( DSPBASE.iMpuPort );
  }
                           /* IRQ-Acknowledge aller anderen DSP's.    */
                           /* Einzige Verursacher: Midi und 8-Bit DMA */
  else inp( DSPBASE.iDspPort + 0x0e );

  if( lpUserFunc ) lpUserFunc( lIRQServed );

              /* Interruptkontroller das Ende des IRQ's signalisieren */
  irq_SendEOI( DSPBASE.iDspIrq );
}

/**********************************************************************/
/* dsp_RestoreIrqHandler : Original IRQ-Handler wieder einrichten.    */
/**********************************************************************/
VOID dsp_RestoreIrqHandler( VOID )
{
  irq_SetHandler( DSPBASE.iDspIrq, lpOldInt );
}

/**********************************************************************/
/* dsp_InitIrqHandler : eigenen DSP-IRQ-Handler einrichten.           */
/**********************************************************************/
VOID dsp_InitIrqHandler( VOID )
{
   lpOldInt = irq_SetHandler( DSPBASE.iDspIrq, dsp_IrqHandler );
}

/**********************************************************************/
/* dsp_SetUserIRQ : Benutzerdefinierte IRQ-Funktion angeben.          */
/**------------------------------------------------------------------**/
/* Eigabe : lpFunc - Adresse der Benutzerfunktion.                    */
/* Info : - Die hier angegebene Funktion wird vom eigenen DSP-Handler */
/*          aufgerufen.                                               */
/*          PROTOTYP: void far User( LONG irqcnt ){}                  */
/*          Der Benutzerfunktion wird die Anzahl der bisherigen IRQ-  */
/*          Aufrufe bergeben.                                        */
/*          Durch 'dsp_SetUserIRQ(NULL);' wird der Aufruf der eigenen */
/*          Funktion abgebrochen.                                     */
/**********************************************************************/
VOID dsp_SetUserIRQ( VOID ( _FP *lpFunc)( LONG ) )
{
  lpUserFunc = lpFunc;
}

/**********************************************************************/
/* dsp4_SetUserIRQ : Benutzerdefinierte IRQ-Funktion angeben.         */
/**------------------------------------------------------------------**/
/* Eigabe : lpFunc - Adresse der Benutzerfunktion.                    */
/* Info : - Die hier angegebene Funktion wird vom eigenen DSP-Handler */
/*          aufgerufen.                                               */
/*          PROTOTYP: void far DSP4User( BYTE who ){}                 */
/*          Der Benutzerfunktion wird die Anzahl der bisherigen IRQ-  */
/*          Aufrufe bergeben.                                        */
/*          Durch 'dsp4_SetUserIRQ(NULL);' wird der Aufruf der eigenen*/
/*          Funktion abgebrochen.                                     */
/**********************************************************************/
VOID dsp4_SetUserIRQ( VOID ( _FP *lpFunc)( BYTE ) )
{
  lpDSP4UserFunc = lpFunc;
}

/**********************************************************************/
/* dsp_WaitForNextIRQ : Warte auf nchsten vom DSP ausgelsten        */
/*                      Interrupt                                     */
/**------------------------------------------------------------------**/
/* Eingabe : pDoSomething - Adresse einer whrend des wartens auszu-  */
/*                           fhrenden Funktion.                      */
/*           lPar          - Parameter, der der Funktion bergeben    */
/*                           werden soll.                             */
/* Ausgabe : TRUE  : Seit dem letzen Aufruf von WaitForNextIRQ sind   */
/*                   mehr als 1 IRQ-Aufruf vergangen.                 */
/*           FALSE : Seit dem letzen Aufruf von WaitForNextIRQ ist    */
/*                   nur 1 IRQ-Aufruf vergangen.                      */
/**********************************************************************/
WORD dsp_WaitForNextIRQ( VOID ( _FP *lpDoSomething)( LONG ), LONG lPar )
{ WORD iRet;

  iRet = ( lIRQServed - lOldIRQServed > 1L );
  lOldIRQServed = lIRQServed;         /* aktuellen Zhlerstand retten */
                    /* Schleife ausfhren, bis sich IRQ-Zhler ndert */
  while( lOldIRQServed == lIRQServed )           /* Funktion aufrufen */
    if( lpDoSomething ) lpDoSomething( lPar );

  return iRet;
}

/**********************************************************************/
/* dsp_InitWaitForIRQ : Interruptzhler angleichen                    */
/**********************************************************************/
VOID dsp_InitWaitForIRQ( VOID )
{                        /* alter Zhlerstand = aktuellen Zhlerstand */
  lOldIRQServed = lIRQServed;
}

/**********************************************************************/
/* dsp_8DAC : Direktes 8 Bit "Digital->Analog Converten" (Ausgabe)    */
/*--------------------------------------------------------------------*/
/* Eingabe : bVal - auszugebender 8-Bit Wert                          */
/* Ausgabe : NO_ERROR ( 0 )  - Byte konnte ausgegeben werden          */
/*           DSP_ERR8DAC( 9 )- bertragungsfehler                     */
/**********************************************************************/
WORD dsp_8DAC( BYTE bVal )
{
  return ( dsp_Write( DSP_8DAC ) && dsp_Write( bVal ) ) ?
   NO_ERROR  : DSP_ERR8DAC;
}

/**********************************************************************/
/* dsp_8ADC : Direktes 8 Bit "Analog->Digital Converten" (Samplen)    */
/*--------------------------------------------------------------------*/
/* Eingabe : pVal - Adresse des Bytes, das gewandelten Wert           */
/*                  aufnehmen soll.                                   */
/* Ausgabe : NO_ERROR ( 0 )    - Byte konnte eingelesen werden        */
/*           DSP_ERR8ADC( 10 ) - bertragungsfehler                   */
/**********************************************************************/
WORD dsp_8ADC( PBYTE pVal )
{
  return ( dsp_Write( DSP_8ADC ) && dsp_Read( pVal ) ) ?
   NO_ERROR  : DSP_ERR8DAC;
}

/**********************************************************************/
/* dsp_PLAY : Abspielen von Musikdaten eines Arrays                   */
/*--------------------------------------------------------------------*/
/* Eingabe : pBuffer - Adresse des Musikdaten enthaltenden Arrays     */
/*           uSize    - Gre des Arrays                              */
/*           uDelay   - Wert fr Verzgerungsschleife (fr Frequenz)  */
/* Ausgabe : NO_ERROR ( 0 )     - Byte konnte ausgegeben werden       */
/*           DSP_ERRPLAY ( 11 ) - bertragungsfehler                  */
/**********************************************************************/
WORD dsp_PLAY( PBYTE pBuffer, WORD uSize, WORD uDelay )
{ WORD i,j;

  if( dsp_SetSpeaker( ON ) ) return DSP_ERRPLAY;     /* Tonausgabe an */
  for( i = 0; i < uSize; i++ )     /* Jedes einzelne Byte ausgeben... */
  {
    if( dsp_8DAC( pBuffer[ i ] ) ) return DSP_ERRPLAY;
    for( j = 0; j < uDelay; j++ );                  /* ... und warten */
  }
  if( dsp_SetSpeaker( OFF ) ) return DSP_ERRPLAY;
  return NO_ERROR;
}

/**********************************************************************/
/* dsp_RECORD : Aufnehmen von Musikdaten in ein Array                 */
/*--------------------------------------------------------------------*/
/* Eingabe : pBuffer - Adresse des Musikdaten aufnehmenden Arrays     */
/*           uSize    - Gre des Arrays                              */
/*           uDelay   - Wert fr Verzgerungsschleife (fr Frequenz)  */
/* Ausgabe : NO_ERROR ( 0 )       - Byte konnte ausgegeben werden     */
/*           DSP_ERRRECORD ( 12 ) - bertragungsfehler                */
/**********************************************************************/
WORD dsp_RECORD( PBYTE pBuffer, WORD uSize, WORD uDelay )
{ WORD i, j;
             /* Bei Aufnahmen mu DSP-Lautsprecher abgeschaltet sein! */
  if( dsp_SetSpeaker( OFF ) ) return DSP_ERRRECORD;
  for( i = 0; i < uSize; i++ )
  {                                                    /* Einlesen... */
     if( dsp_8ADC( &pBuffer[ i ] ) ) return DSP_ERRRECORD;
     for( j = 0; j < uDelay; j++ );                 /* ... und warten */
  }
  return NO_ERROR;
}

 /* Welcher Bufferteil soll als nchstes geladen/gespeichert werden ? */
INT iBufCnt = 0; 

/**********************************************************************/
/* dsp_InitBuffers: Buffer fr DSP-Aufnahme/Wiedergabe vorbereiten    */
/**********************************************************************/
VOID dsp_InitBuffers( VOID )
{
  iBufCnt = 0;
}

/**********************************************************************/
/* dsp_FileOpen: Datei fr DSP-Aufnahme/Wiedergabe ffnen             */
/**------------------------------------------------------------------**/
/* Eingabe : pFile   - Dateiname                                      */
/*           iADC    - Aufnahme (TRUE) / Wiedergabe (FALSE)           */
/*           pHandle - Adresse eines INTs zur Aufnahme des Datei-     */
/*                     handles                                        */
/* Ausgabe : 0    - kein Fehler                                       */
/*           <> 0 - aufgetretener DOSERROR                            */
/**********************************************************************/
INT dsp_FileOpen( PCHAR pFile, INT iADC, PINT pHandle )
{
  if( iADC ) return _dos_creat( pFile, _A_NORMAL, pHandle );
  else       return _dos_open ( pFile, O_RDWR, pHandle );
}

/**********************************************************************/
/* dsp_FileClose: DSP-Aufnahme/Wiedergabe Datei schlieen             */
/**------------------------------------------------------------------**/
/* Eingabe : iHandle - Dateihandle der Datei                          */
/* Ausgabe : 0    - kein Fehler                                       */
/*           <> 0 - aufgetretener DOSERROR                            */
/**********************************************************************/
INT dsp_FileClose( INT iHandle )
{
  return _dos_close( iHandle );
}

/**********************************************************************/
/* dsp_ReadHeader : Dateikopf lesen                                   */
/**------------------------------------------------------------------**/
/* Eingabe : iHandle - Dateihandle                                    */
/*           pDRP    - Adresse der RECPLAY-Struktur die Dateikopf     */
/*                     aufnehmen soll                                 */
/* Ausgabe : Anzahl der gelesenen Zeichen                             */
/**********************************************************************/
INT dsp_ReadHeader( INT iHandle, PDSPRECPLAY pDRP )
{ WORD n;
  _dos_read( iHandle, &pDRP[ 0 ], sizeof( DSPRECPLAY ), &n );
  return n;
}

/**********************************************************************/
/* dsp_WriteHeader : Dateikopf schreiben                              */
/**------------------------------------------------------------------**/
/* Eingabe : iHandle - Dateihandle                                    */
/*           pDRP    - Adresse der RECPLAY-Struktur die Dateikopf     */
/*                     enthlt                                        */
/* Ausgabe : Anzahl der geschriebenen Zeichen                         */
/**********************************************************************/
INT dsp_WriteHeader( INT iHandle, PDSPRECPLAY pDRP )
{ WORD n;
   _dos_write( iHandle, &pDRP[ 0 ], sizeof( DSPRECPLAY ), &n );
  return n;
}

/**********************************************************************/
/* dsp_ReadBuffer : Nchsten Block fr Wiedergabe lesen               */
/**------------------------------------------------------------------**/
/* Eingabe : iHandle   - Dateihandle                                  */
/*           lpBuffer  - Adresse des Speichers                        */
/*           iHalfSize - (Gre des Speichers) / 2                    */
/* Ausgabe : Anzahl der gelesenen Zeichen                             */
/**------------------------------------------------------------------**/
/* Hinweis : Diese Funktion wacht automatisch darber, ob die erste   */
/*           oder zweite Hlfte des Buffers gelesen werden mu. Dazu  */
/*           werden alle geladenen Buffer gezhlt. Welcher Teil des   */
/*           Buffers mit neuen Daten geladen wird, wird aufgrund der  */
/*           "Geradheit" dieses Zhlers entschieden.                  */
/**********************************************************************/
INT dsp_ReadBuffer( INT iHandle, LPBYTE lpBuffer, INT iHalfSize )
{ WORD n;
  _dos_read( iHandle,
           ( LPVOID )&lpBuffer[ ( iBufCnt & 1 ) ? iHalfSize : 0 ],
             iHalfSize, &n );
  iBufCnt++;
  return n;
}

/**********************************************************************/
/* dsp_WriteBuffer : Nchsten Block der Aufnahme sichern              */
/**------------------------------------------------------------------**/
/* Eingabe : iHandle   - Dateihandle                                  */
/*           lpBuffer  - Adresse des Speichers                        */
/*           iHalfSize - (Gre des Speichers) / 2                    */
/* Ausgabe : Anzahl der geschriebenen Zeichen                         */
/**------------------------------------------------------------------**/
/* Hinweis : Diese Funktion wacht automatisch darber, ob die erste   */
/*           oder zweite Hlfte des Buffers gesichert werden mu.     */
/**********************************************************************/
INT dsp_WriteBuffer( INT iHandle, LPBYTE lpBuffer, INT iHalfSize )
{ WORD n;
  _dos_write( iHandle,
              ( LPVOID )&lpBuffer[ ( iBufCnt & 1 ) ? iHalfSize : 0 ],
              iHalfSize, &n );
  iBufCnt++;
  return n;
}

/**********************************************************************/
/* dsp_ClearBuffer : Buffer lschen                                   */
/**------------------------------------------------------------------**/
/* Eingabe : lpBuffer - Adresse des Buffers                           */
/*           uMemSize - Gre des Buffers                             */
/**------------------------------------------------------------------**/
/* Hinweis : Nachdem der letzte wiederzugebende Block geladen wurde,  */
/*           wird diese Funktion aufgerufen, um ein "Echo" zu ver-    */
/*           meiden.                                                  */
/**********************************************************************/
VOID dsp_ClearBuffer( LPBYTE lpBuffer, WORD uMemSize )
{ WORD i;
  for( i = 0; i < uMemSize; i++ ) lpBuffer[ i ] = 0;
}

/**********************************************************************/
/* dsp_DORecPlay : Harddisk-Recorder / Player                         */
/**------------------------------------------------------------------**/
/* Eingabe : iHandle  - Dateihandle der Datei, die wiederzugebende    */
/*                      Daten enthlt, bzw. Daten aufnehmen soll.     */                              
/*           iADC     - Aufnahme (TRUE) oder Wiedergabe (FALSE)       */
/*           iASource - Aufnahmequelle (CD, LINE, MIC)                */
/*                      -1: aktuelle Mixereinstellung verwenden       */
/*           pDRP     - Dateikopf                                     */
/*           iSecs    - Anzahl der aufzunehmenden Sekunden            */
/*                      (wird bei Wiedergabe ignoriert)               */
/*           lpBuffer - DMA-fhiger Speicher                          */
/*           uMemSize - gre des DMA-Speichers                       */
/**********************************************************************/
VOID dsp_DoRECPLAY( INT         iHandle,
                    INT         iADC,
                    INT         iSource,
                    PDSPRECPLAY pDRP,
                    INT         iSecs,
                    LPBYTE      lpBuffer,
                    UINT        uMemSize )
{ INT iEncore;                        /* Zu - ga - be!, Zu - ga - be! */
  BYTE DSP_cmd;                             /* Ausgefhrtes Kommandos */
  LONG lSamples;  /* Anzahl der aufzunehmenden/abzuspielenden Samples */

  pDRP->iStereo = pDRP->iStereo ? dsp_CanStereo() : FALSE;
  pDRP->iBits = ( pDRP->iBits == 16 ) ? dsp_MaxBits() : 8;
  dsp_AdjustFrq( &pDRP->uFrequency, iADC, &pDRP->iStereo );

  dsp_InitIrqHandler();        /* DSP-Interrupt Funktion installieren */

  dsp_InitBuffers();      /* DMA-Double/Splitbuffering initialisieren */
  dsp_ClearBuffer( lpBuffer, uMemSize );
  
  if( !iADC ) /* Wiedergabe: Buffer fllen */
  {
    lSamples = filelength( iHandle ) - lseek( iHandle, 0L, SEEK_SET );
    dsp_ReadBuffer( iHandle, lpBuffer, uMemSize / 2 );
    dsp_ReadBuffer( iHandle, lpBuffer, uMemSize / 2 );
  }
  else lSamples = ( LONG ) pDRP->uFrequency * ( LONG )iSecs;

  /*- DSP3xx ---------------------------------------------------------*/
  if( DSPBASE.uDspVersion < DSP_4XX )            
  {
    if( pDRP->iStereo ) mix3_PrepareForStereo( iADC ? ADC : DAC );

    if( iADC )                                            /* Aufnahme */
    {
      if( iSource >= 0 ) mix3_SetADCSource( iSource );
      dsp_SetSpeaker( OFF );   /* Bei Aufnahmen immer DSP-Speaker aus */
                                        /* Mono oder Stereo Aufnahme? */
      if( DSPBASE.uDspVersion >= DSP_3XX )
        dsp_Write( pDRP->iStereo ? DSP3_STEREOADC : DSP3_MONOADC );
             /* Im Stereomodus oder bei Hochgeschwindigkeitstransfers */
             /* HI_SPEED DSP-Befehle verwenden.                       */

      DSP_cmd = ( BYTE )DSP_8DMAAUTOADC;
      if( DSPBASE.uDspVersion >= DSP_201 )
        if( pDRP->iStereo || dsp_IsHIMONOADCFrq( pDRP->uFrequency ) )
           DSP_cmd = ( BYTE )DSP2p_8DMAHIAUTOADC;
    }
    else                                                /* Wiedergabe */
    { /* DSP3XX */
      dsp_Write( DSP3_MONOADC );          /* Aufnahmemodus immer MONO */

      dsp_SetSpeaker( ON );                    /* DSP-Lautsprecher an */
      if( pDRP->iStereo )                        /* Stereo-Wiedergabe */
      { char cBackUp;     /* Bei Stereo-Wiedergabe erst mal 0x80 ber */
                          /* DSP unter DMA-Verwendung ausgeben.       */
          /* erstes Byte des DMA fhigen Speicher durch 0x80 ersetzen */
        cBackUp = lpBuffer[ 0 ];
        lpBuffer[ 0 ]= 0x80;
                /* DMA-Kanal zur Ausgabe eines Bytes programmieren... */
        dma_SetChannel( DSPBASE.iDspDmaB, lpBuffer, 1,
                        MODE_SINGLE | MODE_READ );

        dsp_Write( DSP_8DMADAC );/* ...und ein Byte ber DSP ausgeben */
        dsp_Write( 0 );
        dsp_Write( 0 );
                                   /* Ende des DMA-Transfers abwarten */
        dsp_WaitForNextIRQ( NULL, 0 );
        lpBuffer[ 0 ] = cBackUp;      /* DMA-Buffer wieder herstellen */
        DSP_cmd = DSP2p_8DMAHIAUTODAC;/* Stereowiederg. immer HiSpeed */
      }
      else                                         /* Mono-Wiedergabe */
      {
        DSP_cmd = ( BYTE )DSP_8DMAAUTODAC;                 /* DSP_201 */
        if( dsp_IsHIMONODACFrq( pDRP->uFrequency ) )
             DSP_cmd = ( BYTE )DSP2p_8DMAHIAUTODAC;
      }
    }
                         /* DMA fr Aufnahme/Wiedergabe programmieren */
    dma_SetChannel( DSPBASE.iDspDmaB, lpBuffer, uMemSize,
                    ( BYTE ) ( iADC ?
                    ( BYTE )(MODE_SINGLE|MODE_AUTOINIT|MODE_WRITE) :
                    ( BYTE )(MODE_SINGLE|MODE_AUTOINIT|MODE_READ) ) );

    dsp_SetFrq( &pDRP->uFrequency );
            /* Nach jeder Bufferhlfte soll ein IRQ ausgelst werden: */
    dsp_SetTransferSize( uMemSize / 2 );
    dsp_Write( DSP_cmd );                    /* DSP-Kommando absetzen */
  }
  else 
  /*- DSP 4.XX -------------------------------------------------------*/
  { BYTE dsp4_Mode = 0;
    INT  iDivisor = 2;
    if( iADC )
      switch( iSource )
      {
        case CD:
          mix4_SetADCSourceL( CD_L, TRUE );/* Bei Mono-Aufnahmen wird */
          mix4_SetADCSourceR( CD_R, TRUE );/* nur linker ADC verwendet*/
        break;
        case LINE:
          mix4_SetADCSourceL( LINE_L, TRUE );
          mix4_SetADCSourceR( LINE_R, TRUE );
        break;
        case MIC:
          mix4_SetADCSourceL( MIC, TRUE );
          mix4_SetADCSourceR( MIC, TRUE );
        break;
      }
    if( !iADC ) mix4_SetVolume( VOICE, 255, 255 );
    else if( !pDRP->iStereo ) mix4_PrepareForMonoADC();
                             /* DMA-Kanal fr 8 oder 16 Bit Programmieren */
    dma_SetChannel( ( pDRP->iBits == 16 ) ? DSPBASE.iDspDmaW : DSPBASE.iDspDmaB,
                    lpBuffer, uMemSize,
                    ( BYTE )( iADC ?
                    ( MODE_SINGLE | MODE_AUTOINIT | MODE_WRITE ) :
                    ( MODE_SINGLE | MODE_AUTOINIT | MODE_READ ) ) );
                                       /* Ein-/Ausgabefrequenz setzen */
    if( iADC ) dsp4_ADCFrq( pDRP->uFrequency );
    else       dsp4_DACFrq( pDRP->uFrequency );

            /* Aufnahme/Wiedergabe wird durch Kommando-Byte angegeben */
    DSP_cmd  = ( iADC ) ? DSP4_CMDADC : DSP4_CMDDAC;
    DSP_cmd |= DSP4_CMDAUTOINIT | DSP4_CMDFIFO;
    DSP_cmd |= ( pDRP->iBits == 16) ? DSP4_CMD16DMA : DSP4_CMD8DMA;

    dsp4_Mode  = ( pDRP->iStereo ) ? DSP4_MODESTEREO : DSP4_MODEMONO;
    dsp4_Mode |= DSP4_MODESIGNED;

          /* Kommando, Modus und bertragungslnge an DSP bermitteln */
    dsp_Write( DSP_cmd );
    dsp_Write( dsp4_Mode );
                                 /* Halber Buffer in WORDS bzw. BYTES */
    if( pDRP->iBits == 16 ) iDivisor *= 2;
    dsp_Write( LOBYTE( ( uMemSize / iDivisor ) - 1 ) );
    dsp_Write( HIBYTE( ( uMemSize / iDivisor ) - 1 ) );
  }

  dsp_InitWaitForIRQ();              /* Auf IRQ-Warten initialisieren */
  do
  {                                        /* Auf nchsten IRQ warten */
    if( dsp_WaitForNextIRQ( NULL, 0 ) ) printf("Daten verloren!\n");
    printf("." );
    if( iADC )
    {
      iEncore = FALSE;          /* erst mal nicht weitermachen, oder? */
      if( dsp_WriteBuffer( iHandle, lpBuffer, uMemSize / 2 ) )
        if( lSamples > 0 )
        {
          iEncore = TRUE;
          lSamples -= uMemSize / 2;
        }
    }
    else iEncore = dsp_ReadBuffer( iHandle, lpBuffer, uMemSize / 2 );
  } while( iEncore );
  dsp_ClearBuffer( lpBuffer, uMemSize );

      /* HISpeed-Kommandos knnen nur durch Reset unterbrochen werden */
  if( ( DSP_cmd == DSP2p_8DMAHIAUTODAC ) ||
      ( DSP_cmd == DSP2p_8DMAHIAUTOADC ) )
  {
    dsp_RestoreIrqHandler();         /* alten IRQ-Handler instalieren */
    dsp_Reset();
  }
  else
  {
    if( ( !pDRP->iStereo ) && ( DSPBASE.uDspVersion>= DSP_4XX ) )
       mix4_PrepareForMonoADC();

    if( ( DSP_cmd & 0xF0 ) == DSP4_CMD8DMA )
      dsp_Write( DSP4_EXIT8DMA );     /* Nach aktuellem Block beenden */
    else
    if( ( DSP_cmd & 0xF0 ) == DSP4_CMD16DMA )
      dsp_Write( DSP4_EXIT16DMA );    /* Nach aktuellem Block beenden */
    else
    {
      if( ( pDRP->iStereo )  && ( DSPBASE.uDspVersion <= DSP_3XX ) )
        mix3_RestoreFromStereo();
      dsp_SetSpeaker( OFF );                 /* DSP-Laustsprecher aus */
      dsp_Write( DSP2p_EXITAUTOINIT );/* Nach aktuellen Block beenden */
    }
    dsp_WaitForNextIRQ( NULL, 0 );               /* Auf Ende warten */
    dsp_RestoreIrqHandler();         /* alten IRQ-Handler instalieren */
  }             
}
#endif                                     /* #ifndef DSP_VERSIONONLY */
#endif                                         /* #define __DSPUTIL_C */
