/**********************************************************************/
/*                               D V I C                              */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : demonstriert den dirketen Zugriff auf den      */
/*                     Video-RAM.                                     */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  :  1.10.1988                                     */
/*    letztes Update :  1.10.1988                                     */
/*--------------------------------------------------------------------*/
/*    (MICROSOFT C)                                                   */
/*    Erstellung     : CL /AS DVIC.C                                  */
/*    Aufruf         : DVIC                                           */
/*--------------------------------------------------------------------*/
/*    (BORLAND TURBO C)                                               */
/*    Erstellung     : ber den Befehl RUN in der Menzeile           */
/*                     (ohne Project-Datei)                           */
/**********************************************************************/
                                                                        
/*== Include-Dateien einbinden =======================================*/
                                                                        
#include <dos.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <bios.h>
                                                                        
/*== Typedefs ========================================================*/
                                                                        
typedef unsigned char BYTE;               /* wir basteln uns ein Byte */
typedef struct velb far * VP;    /* VP = FAR-Pointer in den Video-RAM */
typedef BYTE BOOL;                           /* wie BOOLEAN in Pascal */
                                                                        
/*== Strukturen ======================================================*/

struct velb {       /* beschreibt eine Bildschirmposition als 2 Bytes */
             BYTE zeichen,                          /* der ASCII-Code */
                  attribut;                /* das zugehrige Attribut */
            };

/*== Makros ==========================================================*/

/*-- MK_FP erstellt aus einer Segment- und Offsetadresse einen -------*/
/*-- FAR-Pointer auf ein Objekt                                -------*/

#ifndef MK_FP                    /* wurde MK_FP noch nicht definiert? */
#define MK_FP(seg, ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))
#endif

#define FARBE(VG, HG) ((VG << 3) + HG)

/*== Konstanten ======================================================*/

#define TRUE  1                     /* Konstanten zur Arbeit mit BOOL */
#define FALSE 0

/*-- die folgende Konstanten liefern Pointer auf Variablen aus dem ---*/
/*-- BIOS-Variablen-Segment ab der Segmentadresse 0x40             ---*/

#define CRT_START ((unsigned far *) MK_FP(0x40, 0x4E))
#define ADDR_6845 ((unsigned far *) MK_FP(0x40, 0x63))

#define NORMAL        0x07        /* Definition der Zeichen-Attribute */
#define HELL          0x0f        /* in bezug auf eine monochrome     */
#define INVERS        0x70        /* Bildschirmkarte                  */
#define UNTERSTRICHEN 0x01
#define BLINKEN       0x80

#define SCHWARZ       0x00         /* Farbattribute der Color-Karte   */
#define BLAU          0x01
#define GRUEN         0x02
#define KOBALTBLAU    0x03
#define ROT           0x04
#define VIOLETT       0x05
#define BRAUN         0x06
#define HELLGRAU      0x07
#define DUNKELGRAU    0x01
#define HELLBLAU      0x09
#define HELLGRUEN     0x0A
#define HELLKOBALT    0x0B
#define HELLROT       0x0C
#define HELLVIOLETT   0x0D
#define GELB          0x0E
#define WEISS         0x0F
                                                                        
/*== globale Variablen ===============================================*/
                                                                        
VP vptr;                /* Pointer auf das erste Zeichen im Video-RAM */
                                                                        
/***********************************************************************
*  Funktion         : D P R I N T                                      *
**--------------------------------------------------------------------**
*  Aufgabe          : Schreibt einen String direkt in den Video-RAM.   *
*                                                                      *
*  Eingabe-Parameter: - SPALTE   = Die Ausgabespalte.                  *
*                     - ZEILE    = Die Ausgabezeile.                   *
*                     - FARBE    = Attribut fr die Zeichen.           *
*                     - STRING   = Pointer auf String.                 *
*  Return-Wert      : keiner                                           *
***********************************************************************/

void dprint(BYTE spalte, BYTE zeile, BYTE farbe, char * string)

{
 register VP lptr;                     /* Laufzeiger in den VIDEO-RAM */
 register BYTE i;                     /* zhlt die Anzahl der Zeichen */

 /*-- Pointer auf die Ausgabeposition im VRAM setzen -----------------*/
 lptr = (VP) ((BYTE far *) vptr + *CRT_START) + zeile * 80 + spalte;
 for (i=0 ; *string ; ++lptr, ++i)          /* den String durchlaufen */
  {
   lptr->zeichen = *(string++);           /* das Zeichen in VIDEO-RAM */
   lptr->attribut = farbe;             /* Attribut fr Zeichen setzen */
  }
}

/***********************************************************************
*  Funktion         : I N I T _ D P R I N T                            *
**--------------------------------------------------------------------**
*  Aufgabe          : Ermittelt die Segmentadresse des Video-RAM fr   *
*                     DPRINT:                                          *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : keiner                                           *
*  Info             : Die Segmentadresse des Video-RAM wird in der     *
*                     globalen Variablen VPTR abgelegt.                *
***********************************************************************/
                                                                        
void init_dprint( void )
{
 vptr = (VP) MK_FP( (*ADDR_6845 == 0x3B4) ? 0xB000 : 0xB800, 0 );
}
                                                                        
/***********************************************************************
*  Funktion         : C L S                                            *
**--------------------------------------------------------------------**
*  Aufgabe          : Lscht den Bildschirm mit Hilfe von DPRINT.      *
*                                                                      *
*  Eingabe-Parameter: - FARBE    = Attribut fr die Zeichen.           *
*  Return-Wert      : keiner                                           *
***********************************************************************/
                                                                        
void cls( BYTE farbe )
                                                                        
{
 static char leerzeile[] =
   "                                        " \
   "                                        ";
 register BYTE i;                                  /* Schleifenzhler */

 for (i=0; i<24; ++i)             /* die einzelnen Zeilen durchlaufen */
  dprint(0, i, farbe, leerzeile);           /* ein Leerzeile ausgeben */
}
                                                                        
/***********************************************************************
*  Funktion         : N O K E Y                                        *
**--------------------------------------------------------------------**
*  Aufgabe          : Testet, ob eine Taste bettigt worden ist.       *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : TRUE, wenn eine Taste bettigt wurde, sonst      *
*                     FALSE.                                           *
***********************************************************************/
                                                                        
BOOL nokey( void )
                                                                        
{
#ifdef __TURBOC__                     /* wird mit TURBO-C gearbeitet? */
 return( bioskey( 1 ) == 0 );      /* JA, Tastatur ber BIOS abfragen */
#else                           /* es wird mit Microsoft C gearbeitet */
 return( _bios_keybrd( _KEYBRD_READY ) == 0 );   /* Abfrage ber BIOS */
#endif
}
                                                                        
/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/
                                                                        
void main()
{
 BYTE firstcol,        /* Farbe fr erstes Quadrat auf dem Bildschirm */
      farbe,                           /* Farbe fr aktuelles Quadrat */
      spalte,                             /* aktuelle Ausgabeposition */
      zeile;
                                                                        
 init_dprint();             /* Segmentadresse des Video-RAM ermitteln */
 cls( FARBE(SCHWARZ, GRUEN) );              /* den Bildschirm lschen */
 dprint(22, 0, WEISS, "DVIC  - (c) 1988 by Michael Tischer");
 firstcol = SCHWARZ;                          /* mit SCHWARZ beginnen */
 while( nokey() )        /* wiederholen, bis eine Taste gedrckt wird */
  {
   if (++firstcol > WEISS)                  /* letzte Farbe erreicht? */
    firstcol = BLAU;                    /* JA, mit  BLAU weitermachen */
   farbe = firstcol;         /* erste Farbe auf dem Bildschirm setzen */
                                                                        
   /*-- den Bildschirm mit Quadraten fllen --------------------------*/
                                                                        
   for ( spalte=0; spalte < 80; spalte += 4)
    for (zeile=1; zeile < 24; zeile += 2)
     {
      dprint( spalte, zeile,   farbe, "");
      dprint( spalte, zeile+1, farbe, "");
      farbe = ++farbe & 15;
     }
  }
}
