/**********************************************************************/
/*                               E M M C                              */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : stellt einige Funktionen zum Zugriff auf den   */
/*                     EMS-Speicher (Expanded Memory) zur Verfgung.  */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 30.08.1988                                     */
/*    letztes Update : 19.02.1992                                     */
/*--------------------------------------------------------------------*/
/*    Speichermodell : eines mit FAR-Zeiger auf die Daten, also       */
/*                     Compact, Large oder Huge                       */
/**********************************************************************/
                                                              
/*== Include-Dateien einbinden =======================================*/
                                                              
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <conio.h>
#include <dos.h>
                                                              
/*== Typedefs ========================================================*/
                                                              
typedef unsigned char BYTE;               /* wir basteln uns ein Byte */
typedef unsigned int WORD;
typedef BYTE BOOL;                           /* wie BOOLEAN in Pascal */
                                                              
/*== Makros ==========================================================*/
                                                              
/*-- MK_FP erstellt aus einer Segment- und Offsetadresse einen -------*/
/*-- FAR-Pointer auf ein Objekt                                -------*/
                                                              
#ifdef MK_FP                        /* wurde MK_FP bereits definiert? */
 #undef MK_FP
#endif
#define MK_FP(seg, ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))
                                                              
/*-- PAGE_ADR liefert einen Pointer auf die physikalische Seite X ----*/
/*-- innheralb des Page-Frame des EMS-Speichers                   ----*/
                                                              
#define PAGE_ADR(x) ((void *) MK_FP(ems_frame_seg() + ((x) << 10), 0))
                                                              
/*== Konstanten ======================================================*/
                                                              
#define TRUE  1                     /* Konstanten zur Arbeit mit BOOL */
#define FALSE 0
                                                              
#define EMS_INT 0x67          /* Interruptnr. zum Zugriff auf den EMM */
#define EMS_ERR -1           /* wird bei einem Fehler zurckgeliefert */
                                                              
/*== globale Variablen ===============================================*/
                                                              
BYTE emm_ec;             /* hier werden die EMM-Fehler-Codes abgelegt */
                                                              
/***********************************************************************
*  Funktion         : E M S _ I N S T                                  *
**--------------------------------------------------------------------**
*  Aufgabe          : Stellt fest, ob EMS-Speicher und ein zugehriger *
*                     EMS-Treiber (EMM) installiert sind.              *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : TRUE, wenn EMS-Speicher installiert ist, sonst   *
*                     FALSE.                                           *
***********************************************************************/
                                                              
BOOL ems_inst()
 {
  static char emm_name[] = { 'E', 'M', 'M', 'X', 'X', 'X', 'X', '0' };
  union REGS regs;       /* Prozessorregister fr den Interruptaufruf */
  struct SREGS sregs;     /* Segmentregister fr den Interrupt-Aufruf */
                                                              
  /*-- Pointer auf Namen im Kopf eines Gerte-Treibers aufbauen ------*/
                                                              
  regs.x.ax = 0x3567;          /* Fkt.nr.: Interruptvektor 0x67 holen */
  intdosx(&regs, &regs, &sregs);   /* den DOS-Interrupt 0x21 aufrufen */
  return !memcmp( MK_FP(sregs.es, 10), emm_name, sizeof emm_name );
 }
                                                              
/***********************************************************************
*  Funktion         : E M S _ N U M _ P A G E                          *
**--------------------------------------------------------------------**
*  Aufgabe          : Ermittelt die Gesamtanzahl der EMS-Pages.        *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : EMS_ERR bei einem Fehler, sonst die Anzahl der   *
*                     EMS-Pages.                                       *
***********************************************************************/
                                                              
int ems_num_page()
 {
  union REGS regs;       /* Prozessorregister fr den Interruptaufruf */
                                                              
  regs.h.ah = 0x42;            /* Fkt.nr.: Anzahl der Pages ermitteln */
  int86(EMS_INT, &regs, &regs);                       /* EMM aufrufen */
  if ((int) (emm_ec = regs.h.ah))      /* ist ein Fehler aufgetreten? */
   return(EMS_ERR);                            /* JA, Fehler anzeigen */
  else                                                 /* kein Fehler */
   return( regs.x.dx );       /* Gesamtanzahl der Pages zurckliefern */
 }
                                                              
/***********************************************************************
*  Funktion         : E M S _ F R E E _ P A G E                        *
**--------------------------------------------------------------------**
*  Aufgabe          : Ermittelt die Anzahl der noch freien EMS-Pages.  *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : EMS_ERR bei einem Fehler, sonst die Anzahl der   *
*                     freien EMS-Pages.                                *
***********************************************************************/
                                                              
int ems_free_page()
 {
  union REGS regs;       /* Prozessorregister fr den Interruptaufruf */
                                                              
  regs.h.ah = 0x42;            /* Fkt.nr.: Anzahl der Pages ermitteln */
  int86(EMS_INT, &regs, &regs);                       /* EMM aufrufen */
  if ((int) (emm_ec = regs.h.ah))      /* ist ein Fehler aufgetreten? */
   return(EMS_ERR);                            /* JA, Fehler anzeigen */
  else                                                 /* kein Fehler */
   return( regs.x.bx );      /* Anzahl der freien Pages zurckliefern */
 }
                                                              
/***********************************************************************
*  Funktion         : E M S _ F R A M E _ S E G                        *
**--------------------------------------------------------------------**
*  Aufgabe          : Ermittelt die Segmentadresse des EMS-Page-Frames *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : EMS_ERR bei einem Fehler, sonst die Segment-     *
*                     adresse des Page-Frames.                         *
***********************************************************************/
                                                              
WORD ems_frame_seg()
 {
  union REGS regs;       /* Prozessorregister fr den Interruptaufruf */
                                                              
  regs.h.ah = 0x41;          /* Fkt.nr.: Segmentadr. Page-Frame holen */
  int86(EMS_INT, &regs, &regs);                       /* EMM aufrufen */
  if ((int) (emm_ec = regs.h.ah))      /* ist ein Fehler aufgetreten? */
   return(EMS_ERR);                            /* JA, Fehler anzeigen */
  else                                                 /* kein Fehler */
   return( regs.x.bx );               /* Segmentadresse zurckliefern */
 }
                                                              
/***********************************************************************
*  Funktion         : E M S _ A L L O C                                *
**--------------------------------------------------------------------**
*  Aufgabe          : Allokiert die angegebene Zahl von Pages und      *
*                     liefert ein Handle zum Zugriff auf diese Pages   *
*                     zurck.                                          *
*  Eingabe-Parameter: PAGES : die Anzahl der zu allokierenden Pages    *
*                             (jeweils 16 KByte)                       *
*  Return-Wert      : EMS_ERR bei einem Fehler, sonst das EMS-Handle.  *
***********************************************************************/
                                                              
int ems_alloc(int pages)
 {
  union REGS regs;       /* Prozessorregister fr den Interruptaufruf */
                                                              
  regs.h.ah = 0x43;                      /* Fkt.nr.: Pages allokieren */
  regs.x.bx = pages;         /* Anzahl zu allokierender Seiten setzen */
  int86(EMS_INT, &regs, &regs);                       /* EMM aufrufen */
  if ((int) (emm_ec = regs.h.ah))      /* ist ein Fehler aufgetreten? */
   return(EMS_ERR);                            /* JA, Fehler anzeigen */
  else                                                 /* kein Fehler */
   return( regs.x.dx );                   /* EMS-Handle zurckliefern */
 }
                                                              
/***********************************************************************
*  Funktion         : E M S _ M A P                                    *
**--------------------------------------------------------------------**
*  Aufgabe          : Bildet eine der unter dem angegebenen Handle     *
*                     allokierten Seiten auf eine physikalische Seite  *
*                     im Page-Frame ab.                                *
*  Eingabe-Parameter: HANDLE: das bei EMS_ALLOC zurckgel. Handle.     *
*                     LOGP  : die logische Page (0 bis n-1)            *
*                     PHYSP : die physikalische Page (0 bis 3)         *
*  Return-Wert      : FALSE bei einem Fehler, sonst TRUE.              *
***********************************************************************/
                                                              
BOOL ems_map(int handle, int logp, BYTE physp)
 {
  union REGS regs;       /* Prozessorregister fr den Interruptaufruf */
                                                              
  regs.h.ah = 0x44;                        /* Fkt.nr.: Mapping setzen */
  regs.h.al = physp;                    /* physikalische Seite setzen */
  regs.x.bx = logp;                          /* logische Seite setzen */
  regs.x.dx = handle;                            /* EMS-Handle setzen */
  int86(EMS_INT, &regs, &regs);                       /* EMM aufrufen */
  return (!(emm_ec = regs.h.ah));
 }
                                                              
/***********************************************************************
*  Funktion         : E M S _ F R E E                                  *
**--------------------------------------------------------------------**
*  Aufgabe          : Gibt den unter einem Handle allokierten Speicher *
*                     wieder frei.                                     *
*  Eingabe-Parameter: HANDLE: das bei EMS_ALLOC zurckgel. Handle.     *
*  Return-Wert      : FALSE bei einem Fehler, sonst TRUE.              *
***********************************************************************/
                                                              
BOOL ems_free(int handle)
 {
  union REGS regs;       /* Prozessorregister fr den Interruptaufruf */
                                                              
  regs.h.ah = 0x45;                       /* Fkt.nr.: Pages freigeben */
  regs.x.dx = handle;                            /* EMS-Handle setzen */
  int86(EMS_INT, &regs, &regs);                       /* EMM aufrufen */
  return (!(emm_ec = regs.h.ah));     /* enthlt AH 0, ist alles o.k. */
 }
                                                              
/***********************************************************************
*  Funktion         : E M S _ V E R S I O N                            *
**--------------------------------------------------------------------**
*  Aufgabe          : Ermittelt die EMM-Versionsnummer.                *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : EMS_ERR bei einem Fehler, sonst die EMM-         *
*                     Versionsnummer.                                  *
*  Info             : Bei der Versionsnr. steht 10 fr 1.0, 11 fr     *
*                     1.1, 34 fr 3.4 etc.                             *
***********************************************************************/
                                                              
BYTE ems_version()
 {
  union REGS regs;       /* Prozessorregister fr den Interruptaufruf */
                                                              
  regs.h.ah = 0x46;                 /* Fkt.nr.: EMM-Version ermitteln */
  int86(EMS_INT, &regs, &regs);                       /* EMM aufrufen */
  if ((int) (emm_ec = regs.h.ah))      /* ist ein Fehler aufgetreten? */
   return(EMS_ERR);                            /* JA, Fehler anzeigen */
  else           /* kein Fehler Versionsnummer aus BCD-Zahl errechnen */
   return( (regs.h.al & 15) + (regs.h.al >> 4) * 10);
 }
                                                              
/***********************************************************************
*  Funktion         : E M S _ S A V E _ M A P                          *
**--------------------------------------------------------------------**
*  Aufgabe          : Speichert die Abbildung (Mapping) zwischen lo-   *
*                     gischen und physikalischen Pages.                *
*  Eingabe-Parameter: HANDLE: das bei EMS_ALLOC zurckgel. Handle.     *
*  Return-Wert      : FALSE bei einem Fehler, sonst TRUE.              *
***********************************************************************/
                                                              
BOOL ems_save_map(int handle)
 {
  union REGS regs;       /* Prozessorregister fr den Interruptaufruf */
                                                              
  regs.h.ah = 0x47;                     /* Fkt.nr.: Mapping speichern */
  regs.x.dx = handle;                            /* EMS-Handle setzen */
  int86(EMS_INT, &regs, &regs);                       /* EMM aufrufen */
  return (!(emm_ec = regs.h.ah));     /* enthlt AH 0, ist alles o.k. */
 }
                                                              
/***********************************************************************
*  Funktion         : E M S _ R E S T O R E _ M A P                    *
**--------------------------------------------------------------------**
*  Aufgabe          : Holt eine zuvor ber EMS_SAVE_MAP gespeicherte   *
*                     Abbildung zischen logischen und physikalischen   *
*                     Pages wieder zurck.                             *
*  Eingabe-Parameter: HANDLE: das bei EMS_ALLOC zurckgel. Handle.     *
*  Return-Wert      : FALSE bei einem Fehler, sonst TRUE.              *
***********************************************************************/
                                                              
BOOL ems_restore_map(int handle)
 {
  union REGS regs;       /* Prozessorregister fr den Interruptaufruf */
                                                              
  regs.h.ah = 0x48;                   /* Fkt.nr.: Mapping zurckholen */
  regs.x.dx = handle;                            /* EMS-Handle setzen */
  int86(EMS_INT, &regs, &regs);                       /* EMM aufrufen */
  return (!(emm_ec = regs.h.ah));     /* enthlt AH 0, ist alles o.k. */
 }
                                                              
/***********************************************************************
*  Funktion         : P R I N T _ E R R                                *
**--------------------------------------------------------------------**
*  Aufgabe          : Gibt eine EMS-Fehlermeldung auf dem Bildschirm   *
*                     aus und beendet das Programm.                    *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : keiner                                           *
*  Info             : Diese Funktion darf nur aufgerufen werden, wenn  *
*                     bei einem vorhergehenden Aufruf einer Funktion   *
*                     des EMM ein Fehler aufgetreten ist.              *
***********************************************************************/
                                                              
void print_err()
 {
  static char nid[] = "nicht identifizierbar";
  static char *err_vec[] =
   { "Fehler im EMS-Treiber (EMM zerstrt)",                  /* 0x80 */
      "Fehler in der EMS-Hardware",                           /* 0x81 */
      nid,                                                    /* 0x82 */
      "ungltiges EMM-Handle",                                /* 0x83 */
      "aufgerufene EMS-Funtkion existiert nicht",             /* 0x84 */
      "keine freien EMS-Handles mehr verfgbar",              /* 0x85 */
      "Fehler beim Sichern oder Zurckspeichern des Mapping", /* 0x86 */
      "mehr Seiten angefordert als physikalisch vorhanden",   /* 0x87 */
      "mehr Seiten angefordert als noch frei sind",           /* 0x88 */
      "Null Seiten angefordert",                              /* 0x89 */
      "logische Seite gehrt nicht zum Handle",               /* 0x8A */
      "ungltige physikalische Seitennummer",                 /* 0x8B */
      "Mapping-Speicherbereich ist voll",                     /* 0x8C */
      "Sichern des Mapping wurde bereits durchgefhrt",       /* 0x8D */
      "Zurckspeichern des Mapping ohne verhergehendes Sichern"
   };
                                                              
  printf("\nACHTUNG! Fehler bei Zugriff auf EMS-Speicher\n");
  printf("         ... %s\n", (emm_ec<0x80 || emm_ec>0x8E) ?
                               nid : err_vec[emm_ec-0x80]);
  exit( 1 );                       /* Programm mit Fehlercode beenden */
 }
                                                              
/***********************************************************************
*  Funktion         : V R _ A D R                                      *
**--------------------------------------------------------------------**
*  Aufgabe          : Liefert einen Pointer auf den Video-RAM.         *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : VOID-Pointer auf den Video-RAM.                  *
***********************************************************************/
                                                              
void *vr_adr()
 {
  union REGS regs;       /* Prozessorregister fr den Interruptaufruf */
                                                              
  regs.h.ah = 0x0f;                 /* Fkt.nr.: Videmomodus ermitteln */
  int86(0x10, &regs, &regs);         /* BIOS-Video-Interrupt aufrufen */
  return ( MK_FP((regs.h.al==7) ? 0xb000 : 0xb800, 0) );
 }
                                                              
/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/
                                                              
void main()
{
 int  anzpage,                                /* Anzahl der EMS-Pages */
      handle,              /* Handle zum Zugriff auf den EMS-Speicher */
      i;                                           /* Schleifenzhler */
 WORD pageseg ;                      /* Segmentadresse des Page-Frame */
 BYTE emmver;                               /* Versionsnummer des EMM */
                                                              
 printf("EMMC  -  (c) 1988, 92 by MICHAEL TISCHER\n\n");
 if ( ems_inst() )                   /* ist EMS-Speicher installiert? */
  {                                                             /* Ja */
   /*-- Informationen ber den EMS-Speicher ausgeben -----------------*/
                                                              
   if ( (int) (emmver = ems_version()) == EMS_ERR) /*Versionsnr. holen*/
    print_err();   /* Fehler: Fehlermeldung ausgeben und Prg. beenden */
   else                                                /* kein Fehler */
    printf("EMM-Versionsnummer           : %d.%d\n",
           emmver/10, emmver%10);
                                                              
   if ( (anzpage = ems_num_page()) == EMS_ERR)   /* Anz. Seiten holen */
    print_err();   /* Fehler: Fehlermeldung ausgeben und Prg. beenden */
   printf("Anzahl der EMS-Pages         : %d (%d KByte)\n",
          anzpage, anzpage << 4);
                                                              
   if ( (anzpage = ems_free_page()) == EMS_ERR)
    print_err();   /* Fehler: Fehlermeldung ausgeben und Prg. beenden */
   printf("... davon frei               : %d (%d KByte)\n",
          anzpage, anzpage << 4);
                                                              
   if ( (int) (pageseg = ems_frame_seg()) == EMS_ERR)
    print_err();   /* Fehler: Fehlermeldung ausgeben und Prg. beenden */
   printf("Segmentadresse des Page-Frame: %X\n", pageseg);
                                                              
   printf("\nNun wird eine Seite aus dem EMS-Speicher allokiert und\n");
   printf("der Bildschirminhalt aus dem VIDEO-RAM in  diese Seite\n");
   printf("kopiert.\n");
   printf("                    ... bettigen Sie bitte eine Taste\n");
   getch();                                  /* Auf eine Taste warten */
                                                              
   /*-- eine Seite allokieren und sie auf die erste logische Seite ---*/
   /*-- im Page-Frame abbilden                                     ---*/
                                                              
   if ( (handle = ems_alloc(1)) == EMS_ERR)
    print_err();   /* Fehler: Fehlermeldung ausgeben und Prg. beenden */
   if ( !ems_map(handle, 0, 0) )                    /* Mapping setzen */
    print_err();   /* Fehler: Fehlermeldung ausgeben und Prg. beenden */
                                                              
   /*-- 4000 Bytes aus dem Video-RAM in den EMS-Speicher kopieren ----*/
                                                              
   memcpy(PAGE_ADR(0), vr_adr(), 4000);
                                                              
   for (i=0; i<24; ++i)                     /* den Bildschirm lschen */
    printf("\n");
                                                              
   printf("Der alte Bildschirminhalt ist nun gelscht worden und\n");
   printf("damit  unwiderbringlich verloren. Da er aber im  EMS-\n");
   printf("Speicher  gesichert  wurde, kann er  von dort  wieder\n");
   printf("zurck in den Video-RAM kopiert werden.\n");
   printf("                   ... bettigen Sie bitte eine Taste\n");
   getch();                                  /* Auf eine Taste warten */
                                                              
   /*-- Inhalt des Video-RAM aus dem EMS-Speicher zurckkopieren  ----*/
   /*-- und den allokierten EMS-Speicher wieder freigeben         ----*/
                                                              
   memcpy(vr_adr(), PAGE_ADR(0), 4000);       /* V-RAM zurckkopieren */
   if ( !ems_free(handle) )                     /* Speicher freigeben */
    print_err();   /* Fehler: Fehlermeldung ausgeben und Prg. beenden */
   printf("ENDE");
  }
 else                         /* der EMS-Treiber wurde nicht entdeckt */
  printf("ACHTUNG! Kein EMS-Speicher installiert\n");
}
