/***********************************************************************
*                                E X T C . C                           *
**--------------------------------------------------------------------**
*  Demmonstriert den Zugriff auf das Extended-Memory ber die BIOS-    *
*  Funktionen des Interrupts 15h bei Bercksichtigung von RAM-Disks    *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    : 18.05.1989                                       *
*  letztes Update am: 19.02.1992                                       *
**--------------------------------------------------------------------**
*  Speichermodell   : SMALL                                            *
**--------------------------------------------------------------------**
*  Microsoft C      : Die Warnmeldung "Segment lost in conversation"   *
*                     lt sich leider nicht vermeiden.                *
***********************************************************************/

/*-- Include-Dateien einbinden ---------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <conio.h>
#include <string.h>
#include <dos.h>
                                                              
/*== Typedefs ========================================================*/
                                                              
typedef unsigned char BYTE;               /* wir basteln uns ein Byte */
typedef unsigned int WORD;
typedef BYTE BOOL;                           /* wie BOOLEAN in Pascal */

#define TRUE  ( 0 == 0 )
#define FALSE ( 0 == 1 )

/*-- Markos ----------------------------------------------------------*/

#ifndef __TURBOC__
  #define random(x) rand()
  #define randomize() srand(1)
#endif

/*-- globale Variablen -----------------------------------------------*/

int  RdLen;                             /* Gre der RAM-Disks in KB  */
BOOL ExtAvail;                          /* extended Memory verfgbar? */
long ExtStart;    /* Startadresse des extended Memory als linear Adr. */
int  ExtLen;                    /* Gre des extended Memory in KByte */

/***********************************************************************
*  ExtAdrConv : konvertiert einen FAR-Pointer in eine 32-Bit groe     *
*               lineare Adresse, die in Form eines longs zurck-       *
*               geliefert wird                                         *
**--------------------------------------------------------------------**
*  Eingabe : Adr = der zu konvertierende Pointer                       *
*  Ausgabe : die konvertierte Adresse                                  *
***********************************************************************/

long ExtAdrConv( void far *Adr )
{
 return (((long) Adr >> 16) << 4 ) + (unsigned int) Adr;
}

/***********************************************************************
*  ExtCopy : kopiert Daten zwischen zwei beliebigen Puffern innerhalb  *
*            des 16-MB groen Adressraums des 80286/386/486.           *
**--------------------------------------------------------------------**
*  Eingabe : Start = Adresse des Start-Puffers als 32-Bit lineare Adr. *
*            Ziel  = Adresse des Ziel-Puffers als 32-Bit lineare Adr.  *
*            Len   = Anzahl der zu kopierenden Bytes                   *
*  Ausgabe : keine                                                     *
*  Info    : - Die Anzahl der zu kopierenden Bytes mu gradzahlig sein.*
***********************************************************************/

void ExtCopy( long Start, long Ziel, WORD Len )
{
 /*-- Datenstrukturen zum Zugriff auf den extended RAM ---------------*/

 typedef struct {                               /* Segmentbeschreiber */
                 WORD Laenge,        /* Lnge des Segments in Byte    */
                      AdrLo;         /* Bit  0 bis 15 der Segmentadr. */
                 BYTE AdrHi,         /* Bit 16 bis 23 der Segmentadr. */
                      Attribut;      /* Segmentattribut               */
                 WORD Res;           /* reserviert fr 80386          */
               } SDES;

 typedef struct {                          /* Global Descriptor Table */
                 SDES Dummy,
                      GDTS ,
                      Start,                       /* kopiere von ... */
                      Ziel ,                       /* ... nach        */
                      Code ,
                      Stack;
                } GDT;

 #define LOWORD(x) ((unsigned int) (x))
 #define HIBYTE(x) (*((BYTE *)&x+2))

 GDT          GTab;                        /* Global Descriptor Table */
 union REGS   Regs;             /* Prozessorregs. fr Interruptaufruf */
 struct SREGS SRegs;                               /* Segmentregister */
 long         Adr;                     /* zur Umrechnung der Adressen */

 memset( &GTab, 0, sizeof GTab );                /* alle Felder auf 0 */

 /*-- Segmentdescriptor des Start-Segments aufbauen ------------------*/

 GTab.Start.AdrLo    = LOWORD(Start);
 GTab.Start.AdrHi    = HIBYTE(Start);
 GTab.Start.Attribut = 0x92;
 GTab.Start.Laenge   = Len;

  /*-- Segmentdescriptor des Ziel-Segments aufbauen ------------------*/

 GTab.Ziel.AdrLo    = LOWORD(Ziel);
 GTab.Ziel.AdrHi    = HIBYTE(Ziel);
 GTab.Ziel.Attribut = 0x92;
 GTab.Ziel.Laenge   = Len;

 /*-- Speicherbereiche mit Hilfe der Funtion 0x87 des Kassetten- -----*/
 /*-- Interrupts 0x15 kopieren                                   -----*/

 Regs.h.ah = 0x87;         /* Funktionsnummer fr 'Speicher kopieren' */
 SRegs.es = (long) (void far *) &GTab >> 16;       /* Adresse der GDT */
 Regs.x.si = (int) &GTab;                          /* nach ES:SI      */
 Regs.x.cx = Len >> 1;         /* Anzahl zu kopierender Words nach CX */
 int86x( 0x15, &Regs, &Regs, &SRegs );           /* Funktion aufrufen */
 if ( Regs.h.ah )                                          /* Fehler? */
  {                                     /* Ja, AH enthlt Fehler-Code */
   printf( "\nFehler bei Zugriff auf extended-Ram (%d)\n", Regs.h.ah);
   exit( 1 );                    /* Programm mit Fehlercode abbrechen */
  }
}

/***********************************************************************
*  ExtRead : liest eine bestimmte Anzahl von Bytes aus dem extended    *
*            Memory in den Hauptspeicher.                              *
**--------------------------------------------------------------------**
*  Eingabe : ExtAdr = Source-Adresse im extended-RAM (lineare Adresse) *
*            BuPtr  = Pointer auf den Ziel-Puffer im Hauptspeicher     *
*            Len    = Anzahl der zu kopierenden Bytes                  *
*  Ausgabe : keine                                                     *
***********************************************************************/

void ExtRead( long  ExtAdr,  void far *BuPtr, WORD Len )
{
 ExtCopy( ExtAdr, ExtAdrConv( BuPtr ), Len );
}

/***********************************************************************
*  ExtWrite : schreibt eine bestimmte Anzahl von Bytes aus dem Haupt-  *
*             speicher in das extended Memory.                         *
**--------------------------------------------------------------------**
*  Eingabe : BuPtr  = Pointer auf den Source-Puffer im Hauptspeicher   *
*            ExtAdr = Ziel-Adresse im extended-RAM (lineare Adresse)   *
*            Len    = Anzahl der zu kopierenden Bytes                  *
*  Ausgabe : keine                                                     *
***********************************************************************/

void ExtWrite( void far *BuPtr, long ExtAdr, WORD Len)
{
 ExtCopy( ExtAdrConv( BuPtr ), ExtAdr, Len );
}

/***********************************************************************
*  ExtGetInfo : ermittelt die Startadresse des extended RAM und seine  *
*               Gre und bercksichtigt dabei evtl. dort abgelegte    *
*               RAM-Disks vom Typ VDISK                                *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : keine                                                     *
*  Globals : ExtAvail/W, ExtStart/W, ExtLen/W                          *
***********************************************************************/

void ExtGetInfo( void )
{
 typedef struct {                       /* Boot-Sektor einer RAM-Disk */
                 BYTE dummy1[3];
                 char Name[5];
                 BYTE dummy2[3];
                 WORD BpS;
                 BYTE dummy3[6];
                 WORD Sektoren;
                 BYTE dummy4;
                } BOOT_SEKTOR;

 static char VDiskName[5] = { 'V', 'D', 'I', 'S', 'K' };

 BOOT_SEKTOR BootSek;         /* nimmt vermeintlichen Boot-Sektor auf */
 union REGS Regs;               /* Prozessorregs. fr Interruptaufruf */

 /*-- Gre des extended Memory ermitteln und dabei feststellen, ob  -*/
 /*-- berhaupt extended Memory zur Verfgung steht                  -*/

 Regs.h.ah = 0x88; /* Funktionsnr.: "Gre des extended-RAM ermitteln"*/
 int86( 0x15, &Regs, &Regs );         /* Kassetten-Interrupt aufrufen */
 if ( Regs.x.ax == 0 )
  {                                    /* kein extended RAM vorhanden */
   ExtAvail = FALSE;
   ExtLen   = ExtStart = 0;
   return;                                     /* zurck zum Aufrufer */
  }

 ExtAvail = TRUE;                        /* extended Memory verfgbar */
 ExtLen   = Regs.x.ax;        /* extended RAM vorhanden, Gre merken */

 /*-- nach RAM-Disks vom Typ VDISK suchen ----------------------------*/

 ExtStart = 0x100000l;                            /* bei 1 MB starten */
 while ( TRUE )                                    /* Abfrageschleife */
  {
   ExtRead( ExtStart, &BootSek, sizeof BootSek );
   if ( memcmp( BootSek.Name, VDiskName,
        sizeof VDiskName ) == 0 )  /* ist Boot-Sektor einer RAM-Disk? */
     ExtStart += (long) BootSek.Sektoren * BootSek.BpS;         /* Ja */
   else
     break;                                       /* Schleife beenden */
  }

  /*-- Gre der RAM-Disks vom freien extended RAM abziehen ----------*/

  ExtLen -= (int) ((ExtStart - 0x100000l) >> 10);
}

/***********************************************************************
*  CheckExt : berprft die Konsistenz des freien extended RAM         *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : keine                                                     *
***********************************************************************/

void CheckExt( void )
{
 long  AdrTest;                             /* Adresse des Testblocks */
 int   i, j;                                       /* Schleifenzhler */
 BYTE  WriteBuf[1024],                                  /* Testblcke */
       ReadBuf[1024];
 BOOL  Fehler = FALSE;                    /* Zeiger Speicherfehler an */

 randomize();                      /* Zufallsgenerator initialisieren */
 AdrTest = ExtStart;
 for ( i = 1; i <= ExtLen; ++i, AdrTest += 1024 )
  {                            /* Speicher in 1 KB-Blocks durchlaufen */
   for ( j = 0; j < 1024; )         /* Block mit Zufallszahlen fllen */
    WriteBuf[ j++ ] = random( 255 );

    printf("\r%ld", AdrTest );   /* Adresse des berprften KB-Blocks */

    /*-- Buffer WriteBuf schreiben, dann wieder nach ReadBuf einlesen-*/

    ExtWrite( WriteBuf, AdrTest, 1024 );
    ExtRead( AdrTest, ReadBuf, 1024 );

    /*-- Identitt von WriteBuf und ReadBuf feststellen --------------*/

   for ( j = 0; j < 1024; ++j )     /* Block mit Zufallszahlen fllen */
    if ( WriteBuf[j] != ReadBuf[j] )       /* Bufferinhalt identisch? */
     {                                               /* Nein, Fehler! */
      printf( "\n  Fehler! Speicherstelle %ld\n", AdrTest + j - 1);
      Fehler = TRUE;
     }
   }                                 /* setzen                        */
  printf( "\n\n" );
 if ( !Fehler )                        /* ist ein Fehler aufgetreten? */
  printf( "Alles o.k.!\n" );                                  /* Nein */
}

/***********************************************************************
*  H A U P T P R O G R A M M                                           *
***********************************************************************/

void main( void )
{
 printf ("EXTC - (c) 1989, 92 by Michael Tischer\n\n");
 ExtGetInfo(); /*Verfgbarkeit und Gre des extended Memory ermitteln*/
 if ( ExtAvail )                       /* ist extended RAM vorhanden? */
  {                                                             /* Ja */
   RdLen = (int) ( (ExtStart - 0x100000l ) >> 10 );
   if ( RdLen == 0 )                   /* sind RAM-Disks installiert? */
     printf( "Es sind keine RAM-Disks installiert.\nDer freie extended"\
             " RAM beginnt mit der 1 MB Speichergrenze.\n" );
   else                                    /* Ja, RAM-Disks vorhanden */
    printf( "Eine oder mehrere RAM-Disks belegen %d KB des extended " \
            "RAM.\nDer freie extended RAM beginnt %d KB hinter der "\
            "1 MB-Speichergrenze.\n", RdLen, RdLen);
   printf( "Die Gre des freien extended RAM betrgt %d KB\n", ExtLen);
   printf( "\nDer extended RAM wird nun auf seine Konsistenz" \
           " hin berprft...\n\n" );
   CheckExt();
  }
 else
  printf( "In Ihrem Rechner ist kein extended RAM installiert!\n" );
}
