/**********************************************************************/
/*                          F L O C K C . C                           */
/*--------------------------------------------------------------------*/
/*    Aufgabe       : Demonstriert das File-Locking in einem Netzwerk */
/*                    mit Hilfe der Funktionen aus dem Modul NETFILEC */
/*--------------------------------------------------------------------*/
/*    Speichermodell: SMALL                                           */
/*--------------------------------------------------------------------*/
/*    Autor         : Michael Tischer                                 */
/*    entwickelt am : 10.02.1992                                      */
/*    letztes Update: 13.02.1992                                      */
/**********************************************************************/

#include <stdio.h>
#include "netfilec.c"               /* die Netzwerkroutinen einbinden */

/*== Konstanten ======================================================*/

#define DATEINAME "flockc.dat"             /* Dateiname der Testdatei */

/*== Typdefinitionen =================================================*/

typedef char Test[ 5 ];                             /* Test fr Datei */
typedef unsigned char BYTE;

/*== Bildschirmroutinen fr Microsoft C ==============================*/

#ifndef __TURBOC__                                    /* Microsoft C? */

  #define clrscr() clearwindow( 1, 1, 80, 25 )

  /********************************************************************/
  /* Gotoxy        : setzt den Cursor                                 */
  /* Eingabe       : Koordinaten des Cursors                          */
  /* Ausgabe       : keine                                            */
  /********************************************************************/

  void gotoxy( int x, int y )
  {
   regs.h.ah = 0x02;          /* Funktionsnummer fr Interrupt-Aufruf */
   regs.h.bh = 0;                                  /* Bildschirmseite */
   regs.h.dh = y - 1;
   regs.h.dl = x - 1;
   int86( 0x10, &regs, &regs );                    /* InterruptAufruf */
  }
#endif

/**********************************************************************/
/* clearwindow   : lscht einen Teil des Bildschirms                  */
/* Eingabe       : s.u.                                               */
/* Ausgabe       : keine                                              */
/**********************************************************************/

void clearwindow( int x1, int y1, int x2, int y2 )
{
 regs.h.ah = 0x07;            /* Funktionsnummer fr Interrupt-Aufruf */
 regs.h.al = 0x00;
 regs.h.bh = 0x07;
 regs.h.ch = y1 - 1;
 regs.h.cl = x1 - 1;
 regs.h.dh = y2 - 1;
 regs.h.dl = x2 - 1;
 int86( 0x10, &regs, &regs );                      /* InterruptAufruf */
 gotoxy( x1, y1 );                                   /* Cursor setzen */
}

/**********************************************************************/
/* FModus   : Dateimodus aus Zugriffsart und Locking erzeugen         */
/* Eingabe  : s.u.                                                    */
/* Ausgabe  : Dateimodus                                              */
/**********************************************************************/

int FModus( int Zugriff,                     /* Zugriffsart auf Datei */
            int Schutz )                     /* Schutzstufe der Datei */
{
 static BYTE Zugriff_Art[ 3 ] = { FM_R, FM_W, FM_RW };
 static BYTE Schutz_Art[ 5 ]  = { SM_COMP, SM_RW, SM_R,
                                  SM_W, SM_NO };

 return Zugriff_Art[ Zugriff-1 ] | Schutz_Art[ Schutz-1 ];
}

/**********************************************************************/
/* DateiTest   : Demonstration der Zugriffskonflikte oder Datei-      */
/*               sperrren mit und ohne File-Locking                   */
/* Eingabe     : s.u.                                                 */
/* Ausgabe     : keine                                                */
/**********************************************************************/

void DateiTest( int ZugriffA,                 /* Zugriffsart 1. Datei */
                int SchutzA,                  /* Schutzstufe 1. Datei */
                int ZugriffB,                 /* Zugriffsart 2. Datei */
                int SchutzB )                 /* Schutzstufe 2. Datei */
{
 Test  TestAAus = "AAAA\0";                         /* Testdatenstze */
 Test  TestBAus = "BBBB\0";

 Test  TestAEin,                            /* zum Einlesen der Daten */
       TestBEin;

 NFILE DateiA;                 /* Testdateien fr gemeinsamen Zugriff */
 NFILE DateiB;
 char  SDummy[ 50 ];                  /* Status nach Netzwerkfunktion */

 clearwindow( 1, 11, 80, 25 );
 printf( "Datei A: Name = %s Zugriffsart = %2i Schutzmodus = %2i\n",
         DATEINAME, ZugriffA, SchutzA );
 printf( "Datei B: Name = %s Zugriffsart = %2i Schutzmodus = %2i\n\n",
         DATEINAME, ZugriffB, SchutzB );

 /*-- Dateien erffnen -----------------------------------------------*/

 printf( "ffnen der Datei A:  " );
 NetReset( DATEINAME, FModus( ZugriffA, SchutzA ),
           sizeof( Test ), &DateiA );
 if ( NetError == NE_FileNotFound )
  NetRewrite( DATEINAME, FModus( ZugriffA, SchutzA ),
              sizeof( Test ), &DateiA );
 NetErrorMsg( NetError, SDummy );
 printf( "Status %2u = %s\n", NetError, SDummy );

 printf( "ffnen der Datei B:  " );
 NetReset( DATEINAME, FModus( ZugriffB, SchutzB ),
           sizeof( Test ), &DateiB );
 NetErrorMsg( NetError, SDummy );
 printf( "Status %2u = %s\n\n", NetError, SDummy );

 /*-- Dateien schreiben ----------------------------------------------*/

 printf( "Schreiben in Datei A:" );
 if ( Is_NetWriteOk( &DateiA ) )                /* Schreiben erlaubt? */
 {
  NetWrite( &DateiA, TestAAus );
  printf( " Satz '%s' geschrieben\n", TestAAus );
 }
 else
  printf( " Datei nicht zur Ausgabe geffnet\n" );

 printf( "Schreiben in Datei B:" );
 if ( Is_NetWriteOk( &DateiB ) )                /* Schreiben erlaubt? */
 {
  NetWrite( &DateiB, TestBAus );
  printf( " Satz '%s' geschrieben\n\n", TestBAus );
 }
 else
  printf( " Datei nicht zur Ausgabe geffnet\n\n" );

 /*-- Dateizeiger fr beide Dateien wieder auf den Anfang ------------*/

 if ( Is_NetOpen( &DateiA ) )                      /* Datei geffnet? */
  NetSeek( &DateiA, 0L );
 if ( Is_NetOpen( &DateiB ) )                      /* Datei geffnet? */
  NetSeek( &DateiB, 0L );

 /*-- Dateien auslesen -----------------------------------------------*/

 TestAEin[0] = TestBEin[0] = '\0';
 printf( "Lesen aus Datei A:" );
 if ( Is_NetReadOk( &DateiA ) )                     /* Lesen erlaubt? */
 {
  NetRead( &DateiA, TestAEin );
  printf( " Satz '%s' gelesen\n", TestAEin );
 }
 else
  printf( " Datei nicht zur Eingabe geffnet\n" );

 printf( "Lesen aus Datei B:" );
 if ( Is_NetReadOk( &DateiB ) )                     /* Lesen erlaubt? */
 {
  NetRead( &DateiB, TestBEin );
  printf( " Satz '%s' gelesen\n\n", TestBEin );
 }
 else
  printf( " Datei nicht zur Eingabe geffnet\n\n" );

 /*-- Dateien schliessen ---------------------------------------------*/

 NetClose( &DateiA );
 NetClose( &DateiB );
}

/**********************************************************************/
/*               H A U P T P R O G R A M M                            */
/**********************************************************************/

void main( void )
{
 int ZugriffA;                           /* Zugriffsarten der Dateien */
 int ZugriffB;
 int SchutzA;                                /* Lock-Modi der Dateien */
 int SchutzB;

 clrscr();
 gotoxy( 1, 1 );
 printf( "FLOCKC Demonstration des File-Locking mit DOS       " \
         "(c) 1992 by Michael Tischer\n" );
 printf( "====================================================" \
         "===========================\n\n" );

 if ( ShareInst() )                             /* SHARE installiert? */
 {
  /*-- Datei-Modi auswhlen ------------------------------------------*/

  printf( "Mgliche Zugriffsarten:         Mgliche Schutzarten:\n" );
  printf( " 1: Nur Lesen                    " ),
  printf( " 1: Kompatibilittsmodus (kein Schutz)\n" );
  printf( " 2: Nur Schreiben                " );
  printf( " 2: Andere Dateizugriffe generell verboten\n" );
  printf( " 3: Lesen und Schreiben          " );
  printf( " 3: Nur Lese-Zugriffe erlaubt\n" );
  printf( "                                 " );
  printf( " 4: Nur Schreib-Zugriffe erlaubt\n" );
  printf( "                                 " );
  printf( " 5: Alles Erlaubt (Schutz ber Record-Locking)\n" );

  printf( "\nZugriffsart Testdatei A: " );
  scanf( "%i", &ZugriffA );
  printf( "Schutzmodus Testdatei A: " );
  scanf( "%i", &SchutzA );
  printf( "Zugriffsart Testdatei B: " );
  scanf( "%i", &ZugriffB );
  printf( "Schutzmodus Testdatei B: " );
  scanf( "%i", &SchutzB );

  DateiTest( ZugriffA, SchutzA, ZugriffB, SchutzB );
 }
 else
  printf( "\nTest nicht mglich, SHARE mu installiert sein\n" );
}
