/***********************************************************************
*                             H M A C . C                              *
**--------------------------------------------------------------------**
*  Aufgabe        : Demonstriert den direkten Zugriff auf die HMA ohne *
*                   die Beihilfe eines speziellen Treibers.            *
**--------------------------------------------------------------------**
*  Autor          : MICHAEL TISCHER                                    *
*  entwickelt am  : 27.07.1990                                         *
*  letztes Update : 29.07.1990                                         *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung     : CL /AS /Zp hmac.c hmaca                            *
*  Aufruf         : hmac                                               *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung     : ber eine Projekt-Datei mit folgendem Inhalt:      *
*                     hmac.c                                           *
*                     hmaca.obj                                        *
***********************************************************************/

/*-- Include-Dateien einbinden ---------------------------------------*/

#include <dos.h>                              /* fr Interruptaufrufe */

#ifdef __TURBOC__
  #include <alloc.h>
#else
  #include <malloc.h>
#endif

/*-- Konstanten ------------------------------------------------------*/


#define TRUE  ( 0 == 0 )
#define FALSE ( 0 == 1 )

/*-- Makros ----------------------------------------------------------*/

#ifndef MK_FP
  #define MK_FP(seg,ofs) \
         ((void far *) (((unsigned long)(seg) << 16) | (unsigned)(ofs)))
#endif

#define Hi(x) (*((BYTE *) &x+1))                /* Hi-Byte eines ints */
#define Lo(x) (*((BYTE *) &x))                  /* Lo-Byte eines ints */

/*-- Typdeklarationen ------------------------------------------------*/

typedef unsigned char BYTE;
typedef BYTE BOOL;
typedef unsigned WORD;

/*-- extern Deklarationen --------------------------------------------*/

extern BOOL HMAAvail( void );                       /* HMA verfgbar? */
extern BOOL GateA20( BOOL frei );         /* A20 freischalten/sperren */
extern BOOL IsA20On( void );                   /* A20 freigeschaltet? */

/***********************************************************************
* HMATest : Demonstrtiert den Zugriff auf die HMA                      *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
***********************************************************************/

void HMATest( void)

{
 BYTE far * hmap;                               /* Zeiger auf die HMA */
 WORD i,                                           /* Schleifenzhler */
      err;                       /* Anzahl der Fehler bei HMA-Zugriff */

 if ( IsA20On() )
  printf( "Die Adressleitung A20 ist bereits freigeschaltet!\n" );
 else
  if ( GateA20( TRUE ) == FALSE  || IsA20On() == FALSE )
   {
    printf( "Achtung! Die Adressleitung A20 konnte nicht frei" \
             "geschaltet werden." );
    return;
   }
  else
    printf( "Der Zugriff auf die HMA wurde freigeschaltet.\n" );

 hmap = MK_FP( 0xFFFF, 0x0010 );                    /* Zeiger auf HMA */
 err = 0;                             /* bis jetzt noch keinen Fehler */
 for ( i = 1; i < 65520; ++i, ++hmap )
  {                             /* jede Speicherstelle einzeln testen */
   printf( "\rSpeicherstelle: %u", i );
   *hmap = i % 256;                     /* Speicherstelle beschreiben */
   if ( *hmap != i % 256 )                     /* und wieder auslesen */
    {                                                      /* Fehler! */
     printf( " FEHLER!\n" );
     ++err;
    }
  }

 printf( "\n" );
 if ( err == 0 )                      /* Resultat des Tests auswerten */
  printf( "HMA ok, keine Speicherstelle fehlerhaft.\n" );
 else
  printf( "ACHTUNG! %d fehlerhafte Speicherstellen in der HMA " \
          "entdeckt!\n", err );
 GateA20( FALSE );                        /* Adressleitung abschalten */
}

/***********************************************************************
*                        H A U P T P R O G R A M M                     *
***********************************************************************/

void main( void )
{
 int i;                                            /* Schleifenzhler */

 for ( i = 1; i < 25; ++i )                     /* Bildschirm lschen */
  printf ( "\n" );

 printf("HMAC  -  HMA-Demoprogramm von MICHAEL TISCHER\n\n" );
 if ( HMAAvail() )
  {
   HMATest();                                           /* HMA testen */
   printf( "\n" );
  }
 else
  printf( "Kein Zugriff auf HMA mglich.\n" );
}


