;**********************************************************************;
;*                           H M A C A . A S M                        *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : Assemblerroutine zur Einbindung in das C-      *;
;*                     Programm HMAC.C. Stellt eine Routine zur Ab-   *;
;*                     frage auf extended Memory und zum Freischal-   *;
;*                     ten bzw. Sperren der Adressleitung A20 zur     *;
;*                     Verfgung.                                     *;
;*                     Implementierung hier fr das Speichermodell    *;
;*                     SMALL.                                         *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  : 28.07.1990                                     *;
;*    letztes Update : 28.07.1990                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM HAMCA;   oder   TASM HMACA;               *;
;*                     ... dann mit dem compilierten C-Programm       *;
;*                         HMAC.C verbinden                           *;
;**********************************************************************;

;== Konstanten =========================================================

KB_COMMAND equ 64h                ;Kommandoport der Tastatur
KB_STATUS  equ 64h                ;ist auch Statusport
KB_DATA    equ 60h                ;Datenport der Tastatur
IB_FREE    equ 2                  ;Maske fr Test auf freien Input-Buf
WO_COMMAND equ 0D1h               ;Kommando fr Schreiben d. Outpur-Port

GATE_ON    equ 11011111b          ;Code fr Adressleitung A20 Frei
GATE_OFF   equ 11011101b          ;Code fr Adressleitung A20 Gesperrt

;== Segmentdeklarationen fr das C-Programm ============================

IGROUP group _text                ;Zusammenfassung der Programm-Segmente
DGROUP group _bss, _data          ;Zusammenfassung der Daten-Segmente
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP

_BSS   segment word public 'BSS'  ;dieses Segment nimmt alle nicht ini-
_BSS   ends                       ;tialisierten statischen Variablen auf

_DATA  segment word public 'DATA' ;alle initialisierten globalen und
                                  ;statischen Variablen werden in diesem
                                  ;Segment untergebracht

_DATA  ends

;== Programm ===========================================================

_TEXT  segment byte public 'CODE' ;das Programmsegment

public     _HMAAvail
public     _GateA20
public     _IsA20On

;-----------------------------------------------------------------------
;-- HMAAvail : stellt fest, ob eine HMA zur Verfgung steht
;-- Aufruf von C: BOOL HMAAvail( void )
;-- Return-Wert : TRUE, wenn die HMA verfgbar ist, sonst FALSE

_HMAAvail  proc near
                                                                        
	   ;-- zunchst testen ob AT oder 386-Rechner -----------------

	   xor    ax,ax           ;Diese Befehle wurden von Intel fr
	   push   ax              ;die berprfung des Prozessortyps
	   popf                   ;empfohlen. Sie belegen die Existenz
	   pushf                  ;eines 8088- oder 8086-Prozessors
	   pop    ax
	   and    ax,0F000h
	   cmp    ax,0F000h
	   je     keinhma         ;ein normaler PC ---> keinhma

	   ;-- AT oder 386-Rechner, aber auch mit extended Memory? ----

	   mov    ah,88h          ;Gre des extended Memory ber BIOS
	   int    15h             ;abfragen
	   cmp    ax,64           ;mehr als 64 KB extended Memory?
	   jb     keinhma         ;Nein, keine HMA

	   mov    ax,0FFFFh       ;Ja, HMA steht bereit
	   ret                    ;TRUE zurckliefern

keinhma:   xor    ax,ax           ;0 fr FALSE zurckliefern
           ret                    ;zurck zum C-Programm

_HMAAvail  endp

;-----------------------------------------------------------------------
;-- GateA20 : Die Adressleitung A20 Feischalten oder Sperren
;-- Aufruf von C: BOOL GateA20( BOOL frei )
;-- Return-Wert : TRUE, wenn die Operation geglckt ist, sonst FALSE

_GateA20   proc near

	   ;-- Struktur zum leichten Zugriff auf den Parameter ---------

sframe     struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
ret_adr    dw ?                   ;Rcksprungadresse zum Aufrufer
frei       dw ?                   ;Leitung sperren oder freigeben
sframe     ends                   ;Ende der Struktur

frame      equ [ bp - bp0 ]       ;adressiert die Elemente der Struktur

	   ;-- Makro zum Abfragen der Bereitschaft des KB-Controllers --

kbc_ready  macro                  ;Keyboard-Controller bereit?
	   local  notready

notready:  in     al,KB_STATUS    ;Status-Port abfragen
	   test   al,IB_FREE      ;Input-Buffer frei?
	   loopne notready        ;Nein, und CX noch nicht Null -->

	   endm

	   ;-- hier beginnt der Prozedur-Code -------------------------

	   push   bp              ;BP auf dem Stack sichern
	   mov    bp,sp           ;SP nach BP bertragen

	   mov    ah,11011101b    ;Wert fr "Leitung sperren"
	   cmp    frame.frei,0    ;Leitung sperren?
	   je     g1              ;Ja, Code ist o.k.

	   mov    ah,11011111b    ;Nein, Code fr "Leitung frei" laden

g1:        xor    cx,cx           ;Zhler fr Time-Out
	   cli                    ;keine Interrupts jetzt
	   kbc_ready              ;auf Tastatur-Controller warten
	   jne    gerr            ;Time-Out? Ja ---> GERR

	   mov    al,WO_COMMAND   ;Code zum Zugriff auf Output-P.
	   out    KB_COMMAND,al   ;an den Kommando-Port senden

	   kbc_ready              ;auf Tastatur-Controller warten
	   jne    gerr            ;Time-Out? Ja ---> GERR

	   mov    al,ah           ;Kommando zum Freischalten/Sperren
	   out    KB_DATA,al      ;an den Datenport senden

	   kbc_ready              ;auf Tastatur-Controller warten
	   sti                    ;Interrupts wieder an
	   jne    gerr            ;Time-Out? Ja ---> GERR

	   mov    ax,0FFFFh       ;kein Time-Out, alles o.k.
	   pop    bp              ;BP wieder vom Stack holen
	   ret

gerr:      xor    ax,ax           ;keine Umschaltung mglich
	   pop    bp              ;BP wieder vom Stack holen
	   ret

_GateA20   endp

;-----------------------------------------------------------------------
;-- IsA20On : Stellt fest, ob die Adressleitung A20 freigeschaltet ist
;-- Aufruf von C: BOOL IsA20( void )
;-- Return-Wert : TRUE, wenn die Leitung frei ist, sonst FALSE

ramptr     dd 000000000h          ;Zeiger in den konventionellen RAM
extptr     dd 0FFFF0010h          ;Zeiger in den extended Memory

_IsA20On   proc near

	   push   ds
	   push   es

	   lds    si,cs:ramptr   ;bei gesperrter Leitung zeigen beide
	   les    di,cs:extptr   ;Ptr auf eine identitsche Speicherst.

	   mov    cx,64          ;128 Bytes vergleichen
	   cld                    ;bei Stringbefehlen aufwrts zhlen
	   repe   cmpsw          ;die beiden Bereich vergleichen

	   pop    es             ;die Register zurckholen
	   pop    ds
	   jcxz   a20off         ;CX = 0 --> Bereiche identisch

	   mov    ax,0FFFFh      ;Bereich ungleich ---> A20 ist aktiv
	   ret

a20off:    xor    ax,ax          ;Bereich gleich ---> A20 ist gesperrt
	   ret

_IsA20On   endp

;-------------------------------------------------------------------------
                                                                        
_text      ends                   ;Ende des Codesegments
           end                    ;Ende des Programms
