/**********************************************************************/
/*                   I R Q S T A T . C                                */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Zeigt Status der IRQ-Controller an               */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 12.03.1994                                       */
/*  letztes Update : 26.03.1994                                       */
/* COMPILER: Borland C++ 3.1, Microsoft Visual C++ 1.5                */
/* Achtung: Stack Checking abschalten!                                */
/*          Keine Optimierungen                                       */
/*          Dieses Programm ist ein reines DEMO Programm, das sich    */
/*          als TSR installiert und nicht wieder deinstalliert werden */
/*          kann.                                                     */
/**********************************************************************/
#include <dos.h>
#include <stdio.h>
#include "types.h"
#include "win.h"
#include "irqutil.h"

/*- Folgende Zeilen innerhalb eines Projektes entfernen: -------------*/
#include "win.c"
#include "irqutil.c"


/*- Globale Variablen ------------------------------------------------*/
WINDOW IRQStat1, IRQStat2, IRQCnt;
INT    iCounts[ 16 ];

/**********************************************************************/
/* PrintIRQ : Alle anstehenden (Request) und zur Zeit ausgefhrten    */
/*            (In Service) Interrupts ausgeben.                       */
/**------------------------------------------------------------------**/
/* Ausgabe : Nummer des zur Zeit ausgefhrten IRQ's mit der hchsten  */
/*           Prioritt.                                               */
/**------------------------------------------------------------------**/
/* Info : Der zur Zeit in Ausfhrung befindliche IRQ mit der hchsten */
/*        Prioritt wird ber das ISR (In Service Register) er-       */
/*        mittelt, ber das jeder PIC (Master und Slave) verfgt.     */
/*        Zu beachten ist, das die Priortten wie folgt verteilt      */
/*        sind :                                                      */
/*        PRI: 15 14 13 12 11 10 09 08 07 06 05 04 03 02 01 00        */
/*        IRQ:  0  1  2  8  9 10 11 12 13 14 15  3  4  5  6  7        */
/*        Die Ausgabe der IRR und ISR Flags in dieser Funktion        */
/*        erfolgt in dieser Reihenfolge!                              */
/**********************************************************************/
int _FP PrintIRQ( void )
{ static BYTE bIRRm, bIRRs, bISRm, bISRs, i;
  static INT iIRQ;
  static CHAR szIRR[ 17 ],
              szISR[ 17 ],
              szCntEiner [ 17 ],
              szCntZehner[ 17 ];

  bIRRm = irq_ReadIRR( MASTER_PIC );
  bIRRs = irq_ReadIRR( SLAVE_PIC );
  bISRm = irq_ReadISR( MASTER_PIC );
  bISRs = irq_ReadISR( SLAVE_PIC );

  win_GotoXY( &IRQStat1, 0, 0 );
  win_Print ( &IRQStat1, "IRR ");
  win_GotoXY( &IRQStat2, 0, 0 );
  win_Print ( &IRQStat2, "ISR ");
  win_GotoXY( &IRQCnt,   0, 0 );
  win_Print ( &IRQCnt,   "CNT ");

  iIRQ = -1;
  for( i = 0; i < 2; i++ )                              /* MASTER 0,1 */
  {
    szIRR[ i ] = bIRRm & 0x01 ? 'X': '-';
    bIRRm >>= 1;
    if( bISRm & 0x01 )
      if( iIRQ == -1) iIRQ = i;
    szISR[ i ] = bISRm & 0x01 ? 'X' : '-';
    bISRm >>= 1;
  }

  for( i = 8; i < 16; i++ )                             /* SLAVE 8-15 */
  {
    szIRR[ i ] = bIRRs & 0x01 ? 'X' : '-';
    bIRRs >>= 1;
    if( bISRs & 0x01 )
     if( iIRQ == -1)
     {
       iIRQ = i;
       iCounts[ 2 ]++;     /* Kaskadiertern MASTER-IRQ inkrementieren */
       iCounts[ 2 ] %= 100;
     }

    szISR[ i ] = bISRs & 0x01 ? 'X' : '-';
    bISRs >>= 1;
  }

  for( i = 2; i < 8; i++ )                            /* MASTER 2 - 7 */
  {
    szIRR[ i ] = bIRRm & 0x01 ? 'X' : '-';
    bIRRm >>= 1;
    if( bISRm & 0x01 )
     if( iIRQ == -1)
      iIRQ = i;
    szISR[ i ] = bISRm & 0x01 ? 'X' : '-';
    bISRm >>= 1;
  }
  szISR[ 16 ] = '\0';
  szIRR[ 16 ] = '\0';

  iCounts[ iIRQ ]++;
  iCounts[ iIRQ ] %= 100;                                   /* 0 - 99 */
  for( i = 0; i < 16; i++ )
  {
    szCntEiner[ i ]  = '0' + iCounts[ i ] / 10;
    szCntZehner[ i ] = '0' + iCounts[ i ] % 10;
  }
  szCntEiner[ 16 ] = '\0';
  szCntZehner[ 16 ] = '\0';

  win_Print ( &IRQStat1, szIRR );
  win_Print ( &IRQStat2, szISR );
  win_GotoXY( &IRQCnt,   4, 0 );
  win_Print ( &IRQCnt,   szCntEiner );
  win_GotoXY( &IRQCnt,   4, 1 );
  win_Print ( &IRQCnt,   szCntZehner );

  return iIRQ;
}


/**********************************************************************/
/* IRQ : Interrupt-Routine fr alle Hardware-Interrupts               */
/**------------------------------------------------------------------**/
/* Info : Durch Untersuchung der IS-Register wird der IRQ mit der     */
/*        hchsten Prioritt ermittelt. Dessen alte IRQ-Routine wird  */
/*        dann aufgerufen. Dies ist allerdings nur dadurch mglich,   */
/*        da die PIC's im AT auf "feste Prioritt" programmiert      */
/*        wurden.                                                     */
/**********************************************************************/
void (_interrupt _FP *lpOldIRQ[16])( void );
void _interrupt _FP IRQ()
{ INT iIRQ;
  void ( _interrupt _FP *lpFun)( void );

  _disable();
  iIRQ = PrintIRQ();
  #ifdef __TURBO_C__ 
    __asm pushf
  #endif
  lpFun = lpOldIRQ[ iIRQ ];
  lpFun();
  PrintIRQ();
  _enable();
}

/**********************************************************************/
/* InstallIRQs : Alle Hardware-IRQ-Routinen auf obigen Handler setzen */
/**********************************************************************/
void InstallIRQs( void )
{ BYTE i;

  win_Init( &IRQStat1, 80-20, 0, 20, 1, 0x1F, 0x1F, 0);
  win_Clr( &IRQStat1 );
  win_Init( &IRQStat2, 80-20, 1, 20, 1, 0x1F, 0x1F, 0);
  win_Clr( &IRQStat2 );
  win_Init( &IRQCnt  , 80-20, 2, 20, 2, 0x1F, 0x1F, 0);
  win_Clr( &IRQCnt );
  for( i = 0; i < 16; i++ )
  {
    iCounts[ i ] = 0;
    lpOldIRQ[i] = irq_SetHandler( i, IRQ );
  }
}

/**********************************************************************/
/* UninstallIRQs : Hardware-IRQ-Routinen auf alte Handler setzen.     */
/**********************************************************************/
void UninstallIRQs( void )
{ BYTE i;
  for( i = 0; i < 16; i++ ) irq_SetHandler( i, lpOldIRQ[i] );
}

/**********************************************************************/
/*                 H A U P T P R O G R A M M                          */
/**********************************************************************/
void main( void )
{
  InstallIRQs();
  _dos_keep(0, 20 * 64 );                       /* ca. 20K Allokieren */
}

