/***********************************************************************
*                              L O G O C . C                           *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Definition eigener Zeichenstze *
*                     bei EGA- und VGA-Karten am Beispiel einer Rou-   *
*                     tine zur Darstellung von Logos innerhalb des     *
*                     Textmodus                                        *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    :  6.08.1990                                       *
*  letztes Update am: 14.01.1991                                       *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung     : CL /AS /Zp logoc.c logoca                          *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung     : ber eine Projekt-Datei mit folgendem Inhalt:      *
*                     logoc.c                                          *
*                     logoca.obj                                       *
**--------------------------------------------------------------------**
*  Aufruf         : logoc                                              *
***********************************************************************/

#include <dos.h>                         /* Include-Dateien einbinden */
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <conio.h>

#ifdef __TURBOC__                        /* Kompilierung mit Turbo C? */
  #define CLI()         disable()
  #define STI()         enable()
  #define outpw( p, w ) outport( p, w )
  #ifndef inp
    #define outp( p, b )  outportb( p, b )
    #define inp( p )   inportb( p )
  #endif
#else                                /* Nein, mit QuickC 2.0 oder MSC */
  #include <conio.h>
  #define MK_FP(seg,ofs) ((void far *)\
                         (((unsigned long)(seg) << 16) | (ofs)))
  #define CLI()          _disable()
  #define STI()          _enable()
#endif

#define EGA       0                                   /* Karten-Typen */
#define VGA       1
#define WEDERNOCH 2

#define EGAVGA_SEQUENCER 0x3C4          /* Sequencer Adre-/Datenport */
#define EGAVGA_MONCTR    0x3D4                  /* Monitor-Controller */
#define EGAVGA_GRAPHCTR  0x3CE  /*Graphics-Controller Adre-/Datenport*/

/*-- Typdeklarationen ------------------------------------------------*/

typedef unsigned char BYTE;

extern void defchar( BYTE ascii, BYTE table, BYTE height, /*Assembler-*/
                     BYTE anzahl, void far * buf );       /*Routine   */

/***********************************************************************
*  SetCursor : legt die Position des blinkenden Bildschirmcursors fest *
**--------------------------------------------------------------------**
*  Eingabe-Parameter: SPALTE = die neue Cursor-Spalte (0-79)           *
*                     ZEILE  = die neue Cursor-Zeile (0-24)            *
*  Return-Wert      : keiner                                           *
***********************************************************************/

void SetCursor( BYTE spalte, BYTE zeile )
{
 union REGS regs;               /* Prozessorregs. fr Interruptaufruf */

 regs.h.ah = 2;                     /* Funktionsnummer fr Set Cursor */
 regs.h.bh = 0;                /* auf die Bildschirmseite 0 zugreifen */
 regs.h.dh = zeile;                                   /* Zeile merken */
 regs.h.dl = spalte;                                 /* Spalte merken */
 int86(0x10, &regs, &regs);              /* BIOS-Video-Intr. aufrufen */
}

/***********************************************************************
*  PrintfAt : gibt einen formatierten String an einer beliebigen Bild- *
*             schirmposition aus                                       *
**--------------------------------------------------------------------**
*  Eingabe-Parameter: SPALTE = die Ausgabeposition                     *
*                     ZEILE                                            *
*                     FARBE  = das Attribut fr die Zeichen            *
*                     STRING = Pointer auf den String                  *
*  Return-Wert      : keiner                                           *
*  Info             : Diese Funktion darf nur aufgerufen werden, nach- *
*                     dem zuvor die Existenz einer EGA- oder VGA-Karte *
*                     festgestellt wurde                               *
***********************************************************************/

void PrintfAt( BYTE spalte, BYTE zeile, BYTE farbe, char * string, ... )
{
 va_list parameter;              /* Parameter-Liste fr VA_... Macros */
 char ausgabe[255],                 /* Puffer fr formatierten String */
      *ausptr;
 BYTE far *vptr;                           /* Zeiger in den Video-RAM */

 va_start( parameter, string );                /* Parameter umwandeln */
 vsprintf( ausgabe, string, parameter );               /* formatieren */

 vptr = (BYTE far *) MK_FP( 0xB800, spalte*2+zeile*160 );

 for ( ausptr = ausgabe; *ausptr ; )            /* String durchlaufen */
  {
   *vptr++ = *(ausptr++);           /* Zeichen in Video-Ram schreiben */
   *vptr++ = farbe;                /* Attribut in Video-Ram schreiben */
  }
}

/***********************************************************************
*  ClrScr: lscht den Bildschirm                                       *
**--------------------------------------------------------------------**
*  Eingabe-Parameter: FARBE  = das Attribut fr die Zeichen            *
*  Return-Wert      : keiner                                           *
***********************************************************************/

void ClrScr( BYTE farbe )
{
 BYTE far *vptr;                           /* Zeiger in den Video-RAM */
 int  count = 2000;               /* Anzahl der zu lschenden Zeichen */

 vptr = (BYTE far *) MK_FP( 0xB800, 0 ); /*Zeiger auf Video-RAM setzen*/

 for ( ; count--; )                      /* den Video-RAM durchlaufen */
  {
   *vptr++ = ' ';                   /* Zeichen in Video-Ram schreiben */
   *vptr++ = farbe;                /* Attribut in Video-Ram schreiben */
  }
}

/***********************************************************************
*  SetCharWidth: Legt die Breite der Zeichen bei VGA-Karten auf 8 oder *
*                9 Punkte fest.                                        *
**--------------------------------------------------------------------**
*  Eingabe : BREITE = Breite der Zeichen (8 oder 9)                    *
***********************************************************************/

void SetCharWidth( BYTE breite )
{
 union REGS Regs;            /* Prozessorregister zum Interruptaufruf */
 unsigned char x;              /* nimmt Wert fr Misc-Output-Reg. auf */

 Regs.x.bx = ( breite == 8 ) ? 0x0001 : 0x0800;

 x = inp( 0x3CC ) & (255-12);    /* horizontale Auflsung von 720 auf */
 if ( breite == 9 )              /* 640 Punkte umschalten und umge-   */
  x |= 4;                        /* kehrt                             */
 outp( 0x3C2, x);

 CLI();  /* Sequencer von acht auf neun Punkte schalten und umgekehrt */
 outpw( EGAVGA_SEQUENCER, 0x0100 );
 outpw( EGAVGA_SEQUENCER, 0x01 + ( Regs.h.bl << 8 ) );
 outpw( EGAVGA_SEQUENCER, 0x0300 );
 STI();

 Regs.x.ax = 0x1000;               /* Bildschirm horizontal justieren */
 Regs.h.bl = 0x13;
 int86( 0x10, &Regs, &Regs );
}

/***********************************************************************
*  IsEgaVga : Stellt fest, ob eine EGA- oder VGA-Karte installiert ist.*
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : EGA, VGA oder WEDERNOCH                                   *
***********************************************************************/

BYTE IsEgaVga( void )
{
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */

 Regs.x.ax = 0x1a00;              /* Funktion 1Ah gibt es nur bei VGA */
 int86( 0x10, &Regs, &Regs );
 if ( Regs.h.al == 0x1a )              /* ist die Funktion verfgbar? */
  return VGA;
 else
  {
   Regs.h.ah = 0x12;                    /* Funktion 12h Unterfunktion */
   Regs.h.bl = 0x10;                    /* 10h aufrufen               */
   int86(0x10, &Regs, &Regs );                 /* Video-BIOS aufrufen */
   return ( Regs.h.bl != 0x10 ) ? EGA : WEDERNOCH;
  }
}

/***********************************************************************
*  BuildLogo : Baut ein Logo auf dem Bildschirm auf, indem das Logo    *
*              aus verschiedenen Zeichen zusammengesetzt wird, die im  *
*              deutschen Sprachgebrauch nicht vorkommen.               *
**--------------------------------------------------------------------**
*  Eingabe : SPALTE = Startspalte des Logos (1-80)                     *
*            ZEILE  = Startzeile des Logos (1-25)                      *
*            TIEFE  = Anzahl der Logo-Scan-Zeilen                      *
*            FARBE  = Ausgabefarbe des Logos                           *
*            BUFP   = Zeiger auf Array mit Zeigern auf Strings, die    *
*                     das Zeichenmuster des Logos enthalten            *
*  Info    : - Ein Beispiel fr den Aufbau des zu bergebenden Logo-   *
*              Puffers liefert die Funktion TEST.                      *
*            - Das Logo wird innerhalb seines Zeichenblocks zentriert  *
*              dargestellt.                                            *
***********************************************************************/

void BuildLogo( BYTE spalte, BYTE zeile, BYTE tiefe,
                BYTE farbe, char **bufp )
{
 #define MAX_CHAR 32     /* soviele Zeichen knnen umdefiniert werden */

 static BYTE UseChars[MAX_CHAR] =             /* die umzudef. Zeichen */
                  { 128, 130, 131, 133, 134, 135, 136, 137, 138, 139,
                    140, 141, 143, 144, 145, 146, 147, 149, 150, 151,
                    152, 155, 156, 157, 158, 159, 160, 161, 162, 163,
                    164, 165 };

 BYTE       videok;               /* Typ der installierten Videokarte */
 BYTE       chardef[16],                  /* Bitmuster eines Zeichens */
            charhoehe,          /* Anzahl der Scan-Zeilen pro Zeichen */
            i, j, k, l,                         /* Schleifenvariablen */
            maske,       /* Bit-Maske fr Erstellung einer Scan-Zeile */
            breite,                             /* Breite der Strings */
            index,        /* Index zum Durchlaufen des UseChars-Array */
            dx,              /* Breite des Logo-Blocks (Text-Spalten) */
            dy,                /* Tiefe des Logo-Blocks (Text-Zeilen) */
            lspalte,                   /* aktuell durchlaufene Spalte */
            lzeile,                     /* aktuell durchlaufene Zeile */
            links,                           /* linker Rand in Pixeln */
            rechts,                         /* rechter Rand in Pixeln */
            oben,                            /* oberer Rand in Pixeln */
            unten;

 videok = IsEgaVga();                        /* Video-Karte ermitteln */
 switch ( videok )
  {
   case WEDERNOCH :
     printf( "Fehler: Kein EGA- oder VGA-Karte installiert!\n" );
     return;

   case EGA       :
     charhoehe = 14;               /* EGA: 14 Scan-Zeilen pro Zeichen */
     break;

   case VGA       :
     SetCharWidth( 8 );              /* Zeichen nur noch 8 Pix. breit */
     charhoehe = 16;                    /* 16 Scan-Zeilen pro Zeichen */
     break;
  }

 breite = strlen( *bufp ); /* Stringlnge und damit Logo-Breite holen */
 dx = ( breite + 7 ) / 8;       /* Anzahl der bentigten Zeichen ber. */
 dy = ( tiefe + charhoehe - 1 ) / charhoehe;
 if ( dx*dy > MAX_CHAR )
  printf( "Fehler: Logo in BuildLogo zu gro!\n" );
 else
  {
   oben   = ( dy*charhoehe-tiefe ) / 2;             /* Rand berechnen */
   unten  = tiefe + oben - 1;
   links  = ( dx*8-breite ) / 2;
   rechts = breite + links - 1;

   for ( index = 0, i = 0; i < dy; ++ i)
    {                                  /* die Text-Zeilen durchlaufen */
     for (j = 0; j < dx; ++j, ++index )   /* Text-Spalten durchlaufen */
      {
       PrintfAt( spalte+j, zeile+i, farbe,        /* Zeichen ausgeben */
                 "%c", UseChars[ index ] );

       /*-- neues Zeichenmuster fr das ausg. Zeichen berechnen ------*/

       for ( k = 0; k < charhoehe; ++ k )  /* Scan-Zeilen durchlaufen */
        {
         maske = 0;                             /* Bit-Maske bisher 0 */
         for ( l = 0; l <= 7; ++l )  /* jedes Zeichen ist 8 Pix breit */
          {
           maske <<= 1;                  /* Maske nach links schieben */
           lzeile = i * charhoehe + k;
           lspalte = j * 8 + l;

           if ( lzeile>=oben   && lzeile<=unten  &&  /*Pixel auerhalb*/
                lspalte>=links && lspalte<=rechts )  /*des Rands?     */
            if ( *(*(bufp+lzeile-oben)+lspalte-links) != ' ' )
             maske |= 1;                     /* Punkt im Logo gesetzt */
          }
         chardef[ k ] = maske;          /* Bitmuster in Zeiche-Puffer */
        }
       defchar( UseChars[ index ], 0, charhoehe, 1, chardef );
      }
    }
  }
}

/***********************************************************************
*  ResetLogo : Ldt wieder die ursprnglichen Zeichendefinitionen      *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************/

void ResetLogo( void )
{
 union REGS Regs;           /* Prozessorregister fr Interrupt-Aufruf */

 switch ( IsEgaVga() )
  {
   case EGA  :
    Regs.x.ax = 0x1101;        /* 8*14 Zeichensatz komplett neu laden */
    Regs.h.bl = 0;
    int86( 0x10, &Regs, &Regs );
    break;

   case VGA  :
    SetCharWidth( 9 );         /* Zeichen wieder mit 9 Punkten darst. */
    Regs.x.ax = 0x1104;        /* 8*16 Zeichensatz komplett neu laden */
    Regs.h.bl = 0;
    int86( 0x10, &Regs, &Regs );
   }
}

/***********************************************************************
*  Test : demonstriert die Arbeit mit der Prozedur BuildLogo           *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************/

void Test( void )
{
 static char *MyLogo[32] =
                { "                  **                  ",
                  "                 ****                 ",
                  "                 ****                 ",
                  "                  **                  ",
                  "                                      ",
                  "                                      ",
                  "                                      ",
                  "                 ****                 ",
                  "                 ****                 ",
                  "**************************************",
                  "**************************************",
                  "***              ****              ***",
                  "**               ****               **",
                  "*                ****                *",
                  "                 ****                 ",
                  "                 ****                 ",
                  "      ********   ****   ********      ",
                  "      ****  **** **** ****  ****      ",
                  "      ****     ********     ****      ",
                  "      ****      ******      ****      ",
                  "      ****       ****       ****      ",
                  "      ****       ****       ****      ",
                  "      ****       ****       ****      ",
                  "      ****       ****       ****      ",
                  "      ****       ****       ****      ",
                  "      ****       ****       ****      ",
                  "      ****       ****       ****      ",
                  "      ****       ****       ****      ",
                  "      ****       ****       ****      ",
                  "      ****       ****       ****      ",
                  "      ****       ****       ****      ",
                  "      ****       ****       ****      " };

 int  i, j;
 BYTE farbe;

 static BYTE NeuDef[MAX_CHAR] =               /* die umzudef. Zeichen */
                  { 128, 130, 131, 133, 134, 135, 136, 137, 138, 139,
                    140, 141, 143, 144, 145, 146, 147, 149, 150, 151,
                    152, 155, 156, 157, 158, 159, 160, 161, 162, 163,
                    164, 165 };

 ClrScr( 0 );
 for ( i = 0; i < 256; ++i )   /* den kompletten Zeichensatz ausgeben */
  {
   for ( j = 0; j < MAX_CHAR; ++ j )     /* wurde das Zeichen umdef.? */
    {
     if ( NeuDef[ j ] == i )                  /* gehrt Zeichen dazu? */
      break;                           /* Ja, aus Schleife ausbrechen */
    }
   farbe = ( j < MAX_CHAR ) ? 15 : 14;

   PrintfAt( (i % 13) * 6 + 1, i / 13, farbe, "%3d:%c", i, i );
  }

 PrintfAt( 22, 22, 14, "LOGOC  -  (c) 1990 by MICHAEL TISCHER" );
 BuildLogo( 60, 21, 32, 0x3F, MyLogo );                 /* Logo aufb. */
 getch();
 ResetLogo();                                  /* Logo wieder lschen */
 ClrScr( 15 );
 SetCursor( 0, 0 );
}

/***********************************************************************
*  H A U P T P R O G R A M M                                           *
***********************************************************************/

void main( void )
{
 Test();
}

