/**********************************************************************/
/*                               M C B C . C                          */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : Verfolgt die Kette der ber DOS allokierten    */
/*                     Speicherblcke.                                */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 23.08.1988                                     */
/*    letztes Update : 20.10.1981                                     */
/*--------------------------------------------------------------------*/
/*    Speichermodell : SMALL                                          */
/**********************************************************************/
                                                                        
/*== Include-Dateien einbinden =======================================*/
                                                                        
#include <dos.h>
#include <stdlib.h>
                                                                        
/*== Typedefs ========================================================*/
                                                                        
typedef unsigned char BYTE;               /* wir basteln uns ein Byte */
typedef unsigned int WORD;
typedef BYTE BOOLEAN;
typedef BYTE far *FB;                     /* FAR-Pointer auf ein Byte */
                                                                        
/*== Konstanten ======================================================*/
                                                                        
#define TRUE  ( 0 == 0 )    /* werden zur Arbeit mit BOOLEAN bentigt */
#define FALSE ( 1 == 0 )

/*== Strukturen und Unions ===========================================*/
                                                                        
struct MCB {                      /* beschreibt einen MCB im Speicher */
            BYTE id_code;     /* 'M' = es folgt ein Block, 'Z' = Ende */
            WORD psp;           /* Segmentadresse des zugehrigen PSP */
            WORD abstand;      /* Anzahl der reservierten Paragraphen */
           };
                                                                        
typedef struct MCB far *MCBPtr;          /* FAR Pointer auf einen MCB */
                                                                        
/*== Makros ==========================================================*/
                                                                        
#ifdef MK_FP                          /* wurde MK_FP schon definiert? */
  #undef MK_FP
#endif

#define MK_FP(seg, ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))

/***********************************************************************
*  Funktion         : F I R S T _ M C B                                *
**--------------------------------------------------------------------**
*  Aufgabe          : Liefert einen Pointer auf den ersten MCB.        *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : Pointer auf den ersten MCB                       *
***********************************************************************/
                                                                        
MCBPtr first_mcb( void )
{
 union REGS   regs;                /* nimmt die Prozessorregister auf */
 struct SREGS sregs;                 /* nimmt die Segmentregister auf */
                                                                        
 regs.h.ah = 0x52;       /* Fkt.nr.: Adresse des DOS-Info-Block holen */
 intdosx( &regs, &regs, &sregs );      /* DOS-Interrupt 0x21 aufrufen */
                                                                        
 /*-- ES:(BX-4) zeigt auf den ersten MCB, Pointer erstellen ----------*/
                                                                        
 return( *((MCBPtr far *) MK_FP( sregs.es-1, regs.x.bx+12 )) );
}
                                                                        
/***********************************************************************
*  Funktion         : D U M P                                          *
**--------------------------------------------------------------------**
*  Aufgabe          : Gibt Hex- und ASCII-Dump eines Speicherbereiches *
*                     aus.                                             *
*  Eingabe-Parameter: BPTR : Pointer auf den Speicherbereich           *
*                     ANZ  : Anzahl der Dumpzeilen (je 16 Byte)        *
*  Return-Wert      : keiner                                           *
***********************************************************************/
                                                                        
void dump( FB bptr, BYTE anz)
{
 FB   lptr;                /* Laufzeiger zur Ausgabe einer Dump-Zeile */
 WORD offset;                        /* Offsetadresse relativ zu BPTR */
 BYTE i;                                           /* Schleifenzhler */
                                                                        
 printf("\nDUMP  0123456789ABCDEF        00 01 02 03 04 05 06 07 08");
 printf(" 09 0A 0B 0C 0D 0E 0F\n");
 printf("");
 printf("\n");
                                                                        
 for (offset=0;  anz-- ; offset += 16, bptr += 16)
  {                               /* die Schleife ANZ mal durchlaufen */
   printf("%04x  ", offset);
   for (lptr=bptr, i=16; i-- ; ++lptr)  /* Zeichen als ASCII ausgeben */
    printf("%c", (*lptr<32) ? ' ' : *lptr);
   printf("        ");
   for (lptr=bptr, i=16; i-- ; )          /* Zeichen als Hex ausgeben */
    printf("%02X ", *lptr++);
   printf("\n");                     /* in die nchste Zeile schalten */
  }
}
                                                                        
/***********************************************************************
*  Funktion         : T R A C E _ M C B                                *
**--------------------------------------------------------------------**
*  Aufgabe          : Verfolgt die Kette der MCBs.                     *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : keine                                            *
***********************************************************************/
                                                                        
void trace_mcb( void )
{
 static char fenv[] = {                  /* erster Environment-String */
                       'C', 'O', 'M', 'S', 'P', 'E', 'C', '='
                      };       
                                                                        
 MCBPtr  akt_mcb;            /* Pointer auf den jeweils aktuellen MCB */
 BOOLEAN ende;         /* wird TRUE, wenn der letzte MCB gefunden ist */
 BYTE    nr_mcb,               /* Nummer des aktuell bearbeiteten MCB */
         i;                                      /* Schleifenvariable */
 FB      lptr;                       /* Laufzeiger in das Environment */

 ende = FALSE;                           /* jetzt geht's erst mal los */
 nr_mcb = 1;                      /* der erste MCB trgt die Nummer 1 */
 akt_mcb = first_mcb();           /* Pointer auf den ersten MCB holen */
 do                                  /* die einzelnen MCBs abarbeiten */
  {
   if ( akt_mcb->id_code == 'Z' )            /* letzten MCB erreicht? */
    ende = TRUE;                                                /* Ja */
   printf("MCB-Nummer    = %d\n", nr_mcb++);
   printf("MCB-Adresse   = %Fp\n", akt_mcb);
   printf("Speicher-Adr. = %Np:0000\n", FP_SEG(akt_mcb)+1);
   printf("ID            = %c\n", akt_mcb->id_code);
   printf("PSP-Adresse   = %Fp\n", (FB) MK_FP(akt_mcb->psp, 0) );
   printf("Gre         = %u Paragraphen ( %lu Bytes )\n",
          akt_mcb->abstand, (unsigned long) akt_mcb->abstand << 4);
   printf("Inhalt        = ");
                                                                        
   /*-- handelt es sich um ein Environment? --------------------------*/
                                                                        
   for (i=0, lptr=(FB)akt_mcb+16; /* ersten ENV-String mit FENV vergl.*/
        ( i<sizeof fenv ) && ( *(lptr++) == fenv[i++] ) ; )
    ;
   if ( i == sizeof fenv )              /* wurde der String entdeckt? */
    {                       /* Ja, es handelt sich um ein Environment */
     printf("Environment\n");
     if ( _osmajor >= 3 )              /* DOS-Version 3.0 oder hher? */
      {                                /* Ja, Programmnamen ermitteln */
       printf("Programmname  = ");
       for ( ; !(*(lptr++)==0 && *lptr==0) ; )
        ;                                /* letzten ENV-String suchen */
       if ( *(int far *)(lptr + 1) == 1 )
        {                       /* hier befindet sich ein Prgrammname */
         for ( lptr += 3; *lptr ; )  /* den Programmnamen durchlaufen */
          printf( "%c", *(lptr++) );  /* jeweils ein Zeichen ausgeben */
        }
       else                    /* es wurde kein Programmname entdeckt */
        printf("unbekannt");
       printf("\n");                 /* in die nchste Zeile schalten */
      }
                                                                        
     /*-- die Environment-Strings ausgeben ---------------------------*/

     printf("Environment-Strings\n");
     for (lptr=(FB) akt_mcb +16; *lptr ; ++lptr)
      {                                      /* einen String ausgeben */
       printf("           ");
       for ( ; *lptr ; ) /* den String bis zum NUL-Zeichen durchlaufen*/
        printf( "%c", *(lptr++) );    /* jeweils ein Zeichen ausgeben */
       printf("\n");                 /* in die nchste Zeile schalten */
      }
    }
   else                                           /* kein Envrionment */
    {
                                                                        
     /*- handelt es sich um einen PSP? -------------------------------*/
     /*-- (wird durch Befehl INT 20 (Code=0xCD 0x20) eingeleitet) ----*/

     if (*((unsigned far *) MK_FP( akt_mcb->psp, 0 )) == 0x20cd)
      printf("PSP (mit nachfolgendem Programm)\n");             /* Ja */
     else         /* der Befehl INT 0x20 konnte nicht entdeckt werden */
       {
        printf("nicht identifizierbar (Programm oder Daten)\n");
        dump( (FB) akt_mcb + 16, 5);  /* die ersten 5*16 Bytes dumpen */
       }
    }
                                                                        
   printf("");
   printf(" Bitte Taste drcken \n");
   if ( !ende )                               /* folgt noch ein MCB ? */
    {                      /* Ja, Pointer auf den nchsten MCB setzen */
     akt_mcb = (MCBPtr)
                MK_FP( FP_SEG(akt_mcb) + akt_mcb->abstand + 1, 0 );
     getch();                                /* auf eine Taste warten */
    }
  }
 while ( !ende );   /* wiederholen, bis der letzte MCB bearbeitet ist */
}
                                                                        
/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/
                                                                        
void main( void )
{
 printf("\nMCBC (c) 1988, 91 by Michael Tischer\n\n");
 trace_mcb();                      /* die Kette der MCB nachverfolgen */
}
