/**********************************************************************/
/*                         M E M D E M O C . C                        */
/*--------------------------------------------------------------------*/
/*    Aufgabe       : Demonstration der DOS-Speicherverwaltung        */
/*--------------------------------------------------------------------*/
/*    Speichermodell: SMALL                                           */
/*--------------------------------------------------------------------*/
/*    Autor         : Michael Tischer                                 */
/*    entwickelt am : 08.10.1991                                      */
/*    letztes Update: 20.10.1991                                      */
/**********************************************************************/

/*== Include-Dateien einbinden =======================================*/

#include <dos.h>
#include <conio.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

/*== Konstanten ======================================================*/

#define TRUE  ( 0 == 0 )
#define FALSE ( 0 == 1 )

/*-- Funktionsnummern fr Interrupt 0x21 -----------------------------*/

#define GET_MEM      0x48                 /* RAM-Speicher reservieren */
#define FREE_MEM     0x49                   /* RAM-Speicher freigeben */
#define CHANGE_MEM   0x4A      /* Gre eines Speicherbereichs ndern */
#define GET_STRATEGY 0x5800   /* Konzept der Speicherverteilung lesen */
#define SET_STRATEGY 0x5801  /* Konzept der Speicherverteilung setzen */
#define GET_UMB      0x5802                   /* UMB-Einbindung lesen */
#define SET_UMB      0x5803                  /* UMB-Einbindung setzen */

/*-- Suchstrategien fr SetzeStrategie -------------------------------*/

#define SUCHE_UNTEN  0x00     /* Verwende ersten freien Speicherblock */
#define SUCHE_BESTEN 0x01            /* Verwende besten Speicherblock */
#define SUCHE_OBEN   0x02    /* Verwende letzten freien Speicherblock */
#define ZUERST_UMB   0x80 /* Zuerst im UMB suchen (in Kombination mit */
                          /* SUCHE_.... zu verwenden)                 */

/*-- Konstanten fr SetzeUMB -----------------------------------------*/

#define UMB_AUS      0x00                /* UMB nicht bercksichtigen */
#define UMB_EIN      0x01         /* Speichervergabe auch aus dem UMB */

/*-- Konstanten fr Demo ---------------------------------------------*/

#define TEST_GR        (10240-1) /* 10239 Paragraphen fr Testumgebung*/
#define TEST_GR_UMB    (2560-1) /*2559 Pragraphen UMB fr Testumgebung*/
#define TEST_GR_KB     160            /* Testumgebung ist 160 KB gro */
#define TEST_GR_UMB_KB 40          /* Testumgebung UMB ist 40 KB gro */
#define BLOCKANZ       26  /* Anzahl der Adressen fr Ergebnisanzeige */

/*-- Tastencodes fr Bedienereingabe ---------------------------------*/

#define ESC  27                              /* Abbruch mit ESC-Taste */
#define F1   59                                 /* Funktionstaste F1  */
#define F2   60                                 /* Funktionstaste F2  */
#define F3   61                                 /* Funktionstaste F3  */
#define F8   66                                 /* Funktionstaste F8  */
#define F9   67                                 /* Funktionstaste F9  */
#define F10  68                                 /* Funktionstaste F10 */

/*== Typdeklarationen ================================================*/

typedef struct { unsigned int Adresse,              /* Segmentadresse */
                              Groesse;
               } BlockTyp;                          /* Gre Speicher */

typedef unsigned char BYTE;

/*== Makros ==========================================================*/

#ifdef MK_FP                          /* Makro MK_FP schon definiert? */
  #undef MK_FP                              /* Ja, dann Makro lschen */
#endif

#define MK_FP(seg,ofs) ((void far *) ((unsigned long) (seg)<<16|(ofs)))

/*== Typisiertre Konstanten ==========================================*/

char *JaNein[]     = { "Nein", "Ja  " };
char *SText[]      = { "ersten freien Speicherblock verwenden ",
                       "besten freien Speicherblock verwenden ",
                       "letzten freien Speicherblock verwenden" };
BYTE FarbFeld[]    = { 0x07, 0x70 };
char *TastenText[] = { " [F1] Speicher allokieren",
                       " [F2] Speicher freigeben",
                       " [F3] Gre ndern",
                       " [ESC] Programmende" };

/*== Globale Variablen ===============================================*/

union REGS   regs;                   /* Register fr Interrupt-Aufruf */
struct SREGS sregs;         /* Segmentregister fr erweiterten Inter. */
unsigned int AdressFeld[ 1000 ];                 /* Feld fr Speicher */
unsigned int AnzAdress;                            /* Anzahl Adressen */
unsigned int KonvSeg;    /* Adresse Testblock im konvention. Speicher */
unsigned int UMBSeg;                  /* Adresse Testblock in den UMB */
BlockTyp     BlockFeld[ BLOCKANZ ];                       /* Speicher */

/*== Bildschirmroutinen fr Microsoft C ==============================*/

#ifndef __TURBOC__                                    /* Microsoft C? */

  #define textcolor( Farbe )
  #define textbackground( Farbe )

  /********************************************************************/
  /* Gotoxy        : setzt den Cursor                                 */
  /* Eingabe       : Koordinaten des Cursors                          */
  /* Ausgabe       : keine                                            */
  /********************************************************************/

  void gotoxy( int x, int y )
  {
   regs.h.ah = 0x02;          /* Funktionsnummer fr Interrupt-Aufruf */
   regs.h.bh = 0;                                            /* Farbe */
   regs.h.dh = y - 1;
   regs.h.dl = x - 1;
   int86( 0x10, &regs, &regs );                    /* InterruptAufruf */
  }

  /********************************************************************/
  /* clrscr        : lscht den Bildschirm                            */
  /* Eingabe       : s.u.                                             */
  /* Ausgabe       : keine                                            */
  /********************************************************************/

  void clrscr( void )
  {
   regs.h.ah = 0x07;          /* Funktionsnummer fr Interrupt-Aufruf */
   regs.h.al = 0x00;
   regs.h.ch = 0;
   regs.h.cl = 0;
   regs.h.dh = 24;
   regs.h.dl = 79;
   int86( 0x10, &regs, &regs );                    /* InterruptAufruf */
   gotoxy( 1, 1 );                                   /* Cursor setzen */
  }

#endif

/**********************************************************************/
/* PRINT : Wie PRINTF, schreibt die Zeichenkette aber direkt in den   */
/*         Video-RAM                                                  */
/* Eingabe : SPALTE = die Ausgabespalte                               */
/*           ZEILE  = die Ausgabezile                                 */
/*           FARBE  = die Ausgabefarbe                                */
/*           STRING = Zeiger auf den PRINTF-String                    */
/*           ...    = weitere Argumente                               */
/* Ausgabe : keine                                                    */
/**********************************************************************/

void Print( int Spalte, int Zeile, BYTE Farbe, char * String, ...)
{
 struct vr {                   /* eine Bildschirmposition als 2 Bytes */
            BYTE zeichen,                           /* der ASCII-Code */
                 attribut;                 /* das zugehrige Attribut */
           } far * lptr;      /* Laufzeiger zum Zugriff auf Video-RAM */
 va_list parameter;              /* Parameter-Liste fr VA_... Macros */
 char ausgabe[255],                 /* Puffer fr formatierten String */
      *aptr = ausgabe;                 /* zum Durchlaufen des Strings */
 static unsigned int vioseg = 0;
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                              
 if ( vioseg == 0 )                                 /* erster Aufruf? */
  {                     /* Ja, Segmentadresse des Video-RAM ermitteln */
   Register.h.ah = 0x0F;
   int86(0x10, &Register, &Register);
   vioseg = ( Register.h.al == 7 ? 0xb000 : 0xb800 );
  }

 va_start( parameter, String );                /* Parameter umwandeln */
 vsprintf( ausgabe, String, parameter );               /* formatieren */
 lptr = (struct vr far *)
          MK_FP( vioseg, ( (Zeile-1) * 80 + (Spalte-1) ) << 1 );

 for ( ; *aptr ; )                              /* String durchlaufen */
  {
   lptr->zeichen = *aptr++;                   /* Zeichen in Video-RAM */
   lptr++->attribut = Farbe;           /* Attribut des Zeichen setzen */
   ++Spalte;
  }
 gotoxy( Spalte, Zeile );             /* Cursor hinter Ausgabe setzen */
}

/**********************************************************************/
/* DOS_GetMem : Reserviert Speicher                                   */
/* Eingabe    : Gewnschte Speichergre in Paragraphen               */
/* Ausgabe    : Segmentadresse des allokierten Speicherblocks,        */
/*              Anzahl der allokierten Paragraphen oder Anzahl der    */
/*              maximal verfgbaren Paragraphen                       */
/**********************************************************************/

void DOS_GetMem( unsigned int Gr,
                 unsigned int *Adr,
                 unsigned int *Erg )
{
 regs.h.ah = GET_MEM;                              /* Funktionsnummer */
 regs.x.bx = Gr;          /* Anzahl der zu reservierenden Paragraphen */
 intdos( &regs, &regs );                          /* Interrupt-Aufruf */
 if ( !regs.x.cflag )                          /* Aufruf erfolgreich? */
 {
  *Adr = regs.x.ax;              /* Ja, Adresse und Gre zurckgeben */
  *Erg = Gr;
 }
 else                                                 /* Nein, Fehler */
 {
  *Adr = 0;                               /* Kein Speicher reserviert */
  *Erg = regs.x.bx;                       /* maximal verfgbare Gre */
 }
}

/**********************************************************************/
/* DOS_FreeMem: Gibt reservierten Speicher frei                       */
/* Eingabe    : Segmentadresse des Speichers                          */
/* Ausgabe    : keine                                                 */
/**********************************************************************/

void DOS_FreeMem( unsigned int Adr )
{
 regs.h.ah = FREE_MEM;                             /* Funktionsnummer */
 sregs.es = Adr;          /* Anzahl der zu reservierenden Paragraphen */
 intdosx( &regs, &regs, &sregs );                 /* Interrupt-Aufruf */
}

/**********************************************************************/
/* DOS_ChangeMem : ndert reservierte Speichergre                   */
/* Eingabe       : Alte Segementadresse, neue gewnschte Speichergre*/
/* Ausgabe       : Segmentadresse des allokierten Speicherblocks      */
/*                 Anzahl der allokierten Paragraphen oder Anzahl der */
/*                 maximal verfgbaren Paragraphen                    */
/**********************************************************************/

void DOS_ChangeMem( unsigned int Gr,
                    unsigned int *Adr,
                    unsigned int *Erg )
{
 regs.h.ah = CHANGE_MEM;                           /* Funktionsnummer */
 regs.x.bx = Gr;          /* Anzahl der zu reservierenden Paragraphen */
 sregs.es = *Adr;         /* Anzahl der zu reservierenden Paragraphen */
 intdosx( &regs, &regs, &sregs );                 /* Interrupt-Aufruf */
 if ( !regs.x.cflag )                          /* Aufruf erfolgreich? */
  *Erg = Gr;                            /* Ja, neue Gre zurckgeben */
 else                                                 /* Nein, Fehler */
  *Erg = regs.x.bx;                   /* maximal verfgbarer Speicher */
}

/**********************************************************************/
/* LeseStrategie : Liest das Konzept der DOS-Speicherverteilung       */
/* Eingabe       : keine                                              */
/* Ausgabe       : Konzept der Speicherverteilung                     */
/**********************************************************************/

int LeseStrategie( void )
{
 regs.x.ax = GET_STRATEGY;                  /* Funktionsnummer setzen */
 intdos( &regs, &regs );
 return regs.x.ax;                                /* Konzept ausgeben */
}

/**********************************************************************/
/* LeseUMB       : Liest den Status der UMB-Einbindung                */
/* Eingabe       : keine                                              */
/* Ausgabe       : UMB wird bei der Speicherverteilung bercksichtigt */
/* Info          : Erst ab MS-DOS Version 5.0 verfgbar               */
/**********************************************************************/

int LeseUMB( void )
{
 regs.x.ax = GET_UMB;                       /* Funktionsnummer setzen */
 intdos( &regs, &regs );
 return regs.h.al;                                 /* Status ausgeben */
}

/**********************************************************************/
/* SetzeStrategie: Setzt das Konzept der DOS-Speicherverteilung       */
/* Eingabe       : Neues Konzept                                      */
/* Ausgabe       : keine                                              */
/**********************************************************************/

void SetzeStrategie( unsigned int Strategie )
{
 regs.x.ax = SET_STRATEGY;                  /* Funktionsnummer setzen */
 regs.x.bx = Strategie;
 intdos( &regs, &regs );
}

/**********************************************************************/
/* SetzeUMB      : Setzt den Status der UMB-Einbindung                */
/* Eingabe       : Neuer Status der UMB                               */
/* Ausgabe       : keine                                              */
/* Info          : Erst ab MS-DOS Version 5.0 verfgbar               */
/**********************************************************************/

void SetzeUMB( unsigned int UMB )
{
 regs.x.ax = SET_UMB;                       /* Funktionsnummer setzen */
 regs.x.bx = UMB;
 intdos( &regs, &regs );
}

/**********************************************************************/
/* AllokiereSpeicher: Erzeugt die Testumgebung fr den Speichertest   */
/* Eingabe          : keine                                           */
/* Ausgabe          : keine                                           */
/**********************************************************************/

void AllokiereSpeicher( void )
{
 unsigned int SegAdr;     /* Segmentadresse des allokierten Speichers */
 unsigned int Versuch;                    /* gewnschte Speichergre */
 unsigned int Groesse;      /* Gre des allokierten Speicherbereichs */

 /*-- 1. Testblock allokieren ----------------------------------------*/

 SetzeUMB( UMB_AUS );
 DOS_GetMem( TEST_GR, &KonvSeg, &Groesse );        /* Testblock holen */
 if ( KonvSeg == 0 )                                       /* Fehler? */
  return;                                  /* Ja, zurck zum Aufrufer */

 SetzeUMB( UMB_EIN );
 SetzeStrategie( SUCHE_OBEN | ZUERST_UMB );
 DOS_GetMem( TEST_GR_UMB, &UMBSeg, &Groesse );
 if ( UMBSeg != 0  &&  UMBSeg < 0xA000 )       /* Kein UMB vorhanden? */
 {
  DOS_FreeMem( UMBSeg );                        /* Speicher freigeben */
  UMBSeg = 0;                                          /* Keine UMB's */
 }

 /*-- 2. restlichen in 1KB Blcken Speicher allokieren ---------------*/

 Versuch = 63;         /* Versuche zuerst 14 Paragaphen zu allokieren */
 AnzAdress = 0;
 do
 {
  DOS_GetMem( Versuch, &SegAdr, &Groesse );     /* Speicher anfordern */
  if ( SegAdr != 0 )                            /* Speicher erhalten? */
   AdressFeld[ AnzAdress++ ] = SegAdr;          /* Ja, Adresse merken */
 }
 while ( SegAdr != 0 );                /* Gesamter Speicher allokiert */

 /*-- 3. Testbcke wieder freigeben ----------------------------------*/

 if ( KonvSeg > 0 )
  DOS_FreeMem( KonvSeg-- );             /* MCB ist auch frei geworden */
 if ( UMBSeg > 0 )
  DOS_FreeMem( UMBSeg-- );              /* MCB ist auch frei geworden */
}

/**********************************************************************/
/* SpeicherFreigeben: den bei ErmitteleSpeicher rerservierten Speicher*/
/*                    wieder freigeben                                */
/* Eingabe          : keine                                           */
/* Ausgabe          : keine                                           */
/* Globale Varibalen: AdressFeld/R                                    */
/**********************************************************************/

void SpeicherFreigeben( void )

{
 unsigned int i;                                   /* Schleifenzhler */

 if ( AnzAdress > 0 )   /* die einzelnen 1-KB-Blcke wieder freigeben */
  for ( i = 0; i < AnzAdress; i++ )
   DOS_FreeMem( AdressFeld[ i ] );
}

/**********************************************************************/
/* ErgebnisAnzeigen : zeigt die Speicherbelegung an                   */
/* Eingabe          : MITRAHMEN = TRUE, wenn auch der Rahmen ausgege- */
/*                                ben werden soll                     */
/* Ausgabe          : keine                                           */
/**********************************************************************/

void ErgebnisAnzeigen( BYTE MitRahmen )
{

 char          SFeld[ TEST_GR_KB ];
 char          SFeld_UMB[ TEST_GR_UMB_KB ];
 unsigned int  i, j;                               /* Schleifenzhler */
 unsigned int  Position;                             /* Hilfsvariable */
 char          Sdummy[ 20 ];
 char          AltZeichen;  /* Speicher fr letztes ausgegeb. Zeichen */
 int           AktFarbe;                     /* Aktuelle Ausgabefarbe */

 memset( SFeld, 32, TEST_GR_KB );           /* Anzeigefelder initial. */
 memset( SFeld_UMB, 32, TEST_GR_UMB_KB );

 /*-- Speichertabelle fllen -----------------------------------------*/

 for ( i = 0; i < BLOCKANZ; i++ )
 {
  if ( BlockFeld[ i ].Adresse > 0xA000 )                      /* UMB? */
  {
   Position = ( BlockFeld[ i ].Adresse - UMBSeg ) / 64;
   for ( j = 0; j <= BlockFeld[ i ].Groesse / 64; j++ )
    SFeld_UMB[ Position + j ] = i + 65;
  }
  else if ( BlockFeld[ i ].Adresse > 0 )
  {
   Position = ( BlockFeld[ i ].Adresse - KonvSeg ) / 64;
   for ( j = 0; j <= BlockFeld[ i ].Groesse / 64; j++ )
    SFeld[ Position + j ] = i + 65;
  }
 }

 /*-- Rahmen der Speichertabelle malen -------------------------------*/

 if ( MitRahmen )
  {
   Print( 1, 7, 0x07, "Konventioneller Speicher:" );
   Print( 1, 8, 0x07,
              "                   1         2         3         4" );
   Print( 1, 9, 0x07,
              "          1        0         0         0         0" );
   Print( 1, 10, 0x07,
              "ͻ" );
   for ( i = 0; i < 4; i++ )
    Print( 1, 11 + i, 0x07, " %3i %s%s", i * 40,
               "KB                                         ",
               TastenText[ i ] );
   Print( 1, 15, 0x07,
              "ͼ" );
   if ( UMBSeg > 0 )
   {
    Print( 1, 17, 0x07, "UMB:" );
    Print( 1, 18, 0x07,
               "                   1         2         3         4" );
    Print( 1, 19, 0x07,
               "          1        0         0         0         0" );
    Print( 1, 20, 0x07,
               "ͻ" );
    Print( 1, 21, 0x07,
               "   0 KB                                         " );
    Print( 1, 22, 0x07,
               "ͼ" );
   }
   else
    Print( 1, 17, 0x07, "Keine UMB verfgbar" );
  }

 /*-- Tabelle fr konventionellen Speicher ausgeben ------------------*/

 AltZeichen = 0;                      /* letztes ausgegebenes Zeichen */
 AktFarbe = 1;                                 /* letzte Ausgabefarbe */
 for ( i = 0; i < 4; i++ )
  for ( j = 0; j < 40; j++ )
  {
   if ( AltZeichen != SFeld[ i * 40 + j ] )           /* Farbwechsel? */
   {
    AktFarbe = ( AktFarbe + 1 ) % 2;              /* Neue Farbenummer */
    AltZeichen = SFeld[ i * 40 + j ];            /* Vergleichszeichen */
   }
   Print( j + 11, i + 11, FarbFeld[ AktFarbe ],
              "%c", SFeld[ i * 40 + j ] );
  }

 /*-- Tabelle fr UMB ausgeben ---------------------------------------*/

 if ( UMBSeg > 0 )
 {
  for ( j = 0; j < 40; j++ )
  {
   if ( AltZeichen != SFeld_UMB[ j ] )                /* Farbwechsel? */
   {
    AktFarbe = ( AktFarbe + 1 ) % 2;              /* Neue Farbenummer */
    AltZeichen = SFeld_UMB[ j ];                 /* Vergleichszeichen */
   }
   Print( j + 11, 21, FarbFeld[ AktFarbe ],
              "%c", SFeld_UMB[ j ] );
  }
 }
}

/**********************************************************************/
/* Demo          : Demonstration der Speicherverwaltung               */
/* Eingabe       : UMB einbinden, UMB zuerst durchsuchen,             */
/*                 Konzept der Speicherverteilung                     */
/* Ausgabe       : keine                                              */
/**********************************************************************/

void Demo( int Mit_UMB,
           int UMB_zuerst,
           int Strategie )
{
 int          i;                                   /* Schleifenzhler */
 int          Taste;                                  /* Eingabetaste */
 char         Zeiger[ 5 ];           /* Zeiger (A-Z) fr Reservierung */
 unsigned int Versuch;                      /* Gre der Reservierung */
 unsigned int Groesse;
 char         *sdummy[ 20 ];

 /*-- Adressfeld initialisieren --------------------------------------*/

 for ( i = 0; i < BLOCKANZ; i++ )     /* Alle Adressen initialisieren */
 {
  BlockFeld[ i ].Adresse = 0;   /* Segment-Adresse des Speicherblocks */
  BlockFeld[ i ].Groesse = 0;             /* Gre des Speicherblocks */
 }

 ErgebnisAnzeigen( TRUE );                /* Speichertabelle anzeigen */

 /*-- Demonstrationsschleife -----------------------------------------*/

 do
 {
  /*-- Gewhltes Konzept der Speicherverwaltung setzen ---------------*/

  if ( Mit_UMB )                                    /* UMB verwenden? */
   SetzeUMB( UMB_EIN );
  else
   SetzeUMB( UMB_AUS );

  if ( UMB_zuerst )
   SetzeStrategie( Strategie | ZUERST_UMB );
  else
   SetzeStrategie( Strategie );

  /*-- Aktuelles Konzept anzeigen ------------------------------------*/

  Print( 1, 3, 0x07,
             "Konzept der Speicherverwaltung: %s  [F8]",
             SText[ Strategie ] );
  Print( 1, 4, 0x07,
             "Zuerst in den UMB suchen      : %s         %s",
             JaNein[ UMB_zuerst ], "                           [F9]" );
  Print( 1, 5, 0x07,
             "UMB verwenden                 : %s         %s",
             JaNein[ Mit_UMB ], "                           [F10]" );
  Print( 1, 6, 0x07, "---------------------------------------" );
  Print( 40, 6, 0x07,"---------------------------------------" );

  /*-- Eingabe und Verarbeitung --------------------------------------*/

  while ( ! kbhit() );                             /* Warte auf Taste */
  Taste = getch();                                     /* Taste lesen */
  if ( ( Taste == 0 ) && ( kbhit() ) )              /* Funktionstaste */
   Taste = getch();                            /* 2. Tastencode holen */

  switch(  Taste )
  {
   case F1 :                              /* Speicherblock allokieren */
     i = -1;         /* noch keinen gltigen Speicherblock eingegeben */
     do                                 /* Eingabe bis gltiger Block */
      {
       Print( 1, 23, 0x07, "Welchen Block reservieren [ A-Z ]: " );
       scanf( "%s", Zeiger );
       Zeiger[0] = toupper( Zeiger[ 0 ] );
       if (( Zeiger[0] >= 'A' ) && ( Zeiger[0] <= 'Z' ))
        if ( BlockFeld[ (int) Zeiger[0] - 65 ].Adresse == 0 )
         i = (int) Zeiger[0] - 65;
      }
     while ( i == -1 );
     Print( 1, 24, 0x07,
            "Wie viele KB sollen reserviert werden  : " );
     scanf( "%i", &Versuch );
     Versuch = Versuch * 64 - 1;          /* Umrechnen in Paragraphen */
     DOS_GetMem( Versuch, &BlockFeld[ i ].Adresse,
                          &BlockFeld[ i ].Groesse );
     if ( BlockFeld[ i ].Groesse != Versuch )              /* Fehler? */
      {
       Print( 1, 25, 0x07, "Es sind nur noch %4d KB frei",
                           ( BlockFeld[ i ].Groesse + 1 ) /  64  );
       while ( !kbhit() )
        ;
       while ( kbhit() )
        Taste = getch();
       Taste = 0;
      }
     Print(1, 23, 0x07, "                                            ");
     Print(1, 24, 0x07, "                                            ");
     Print(1, 25, 0x07, "                                            ");
     ErgebnisAnzeigen( FALSE );           /* Speichertabelle anzeigen */
     break;

   case F2 :                    /* allokierten Block wieder freigeben */
     i = -1;         /* noch keinen gltigen Speicherblock eingegeben */
     do                                 /* Eingabe bis gltiger Block */
      {
       Print( 1, 23, 0x07, "Welchen Block freigeben [ A-Z ]: " );
       scanf( "%s", Zeiger );
       Zeiger[0] = toupper( Zeiger[ 0 ] );
       if (( Zeiger[0] >= 'A' ) && ( Zeiger[0] <= 'Z' ))
        if ( BlockFeld[ (int) Zeiger[0] - 65 ].Adresse != 0 )
         i = (int) Zeiger[0] - 65;
      }
     while ( i == -1 );
     DOS_FreeMem( BlockFeld[ i ].Adresse );
     BlockFeld[ i ].Adresse = 0;
     BlockFeld[ i ].Groesse = 0;
     Print(1, 23, 0x07, "                                            ");
     ErgebnisAnzeigen( FALSE );           /* Speichertabelle anzeigen */
     break;

   case F3 :                          /* Gre eines Blocks verndern */
     i = -1;         /* noch keinen gltigen Speicherblock eingegeben */
     do                                 /* Eingabe bis gltiger Block */
      {
       Print( 1, 23, 0x07, "Welchen Block freigeben [ A-Z ]: " );
       scanf( "%s", Zeiger );
       Zeiger[0] = toupper( Zeiger[ 0 ] );
       if (( Zeiger[0] >= 'A' ) && ( Zeiger[0] <= 'Z' ))
        if ( BlockFeld[ (int) Zeiger[0] - 65 ].Adresse != 0 )
         i = (int) Zeiger[0] - 65;
      }
     while ( i == -1 );
     Print( 1, 24, 0x07, "Wie viele KB sollen reserviert werden  : " );
     scanf( "%i", &Versuch );
     Versuch = Versuch * 64 - 1;          /* Umrechnen in Paragraphen */
     DOS_ChangeMem( Versuch, &BlockFeld[ i ].Adresse, &Groesse );
     if ( Groesse != Versuch )                             /* Fehler? */
      {
       Print( 1, 23, 0x07, "Es sind nur noch %4i KB frei",
                         ( Groesse + 1 ) /  64  );
       while ( !kbhit() );
       while ( kbhit() )
        Taste = getch();
       Taste = 0;
      }
     else
      BlockFeld[ i ].Groesse = Groesse;           /* neu Gre setzen */
     Print(1, 23, 0x07,"                                            ");
     Print(1, 24, 0x07,"                                            ");
     Print(1, 25, 0x07,"                                            " );
     ErgebnisAnzeigen( FALSE );           /* Speichertabelle anzeigen */
     break;

   case F8 :                                    /* Strategie wechseln */
     Strategie = ( Strategie + 1 ) % 3;
     break;

   case F9 :                      /* Umschalten: Zuerst in UMB suchen */
     UMB_zuerst = ! UMB_zuerst;
     break;

   case F10:                           /* Umschalten: UMB einbeziehen */
     Mit_UMB = ! Mit_UMB;
     break;
  }
 }
 while ( Taste != ESC );
}

/**********************************************************************/
/*                     H A U P T P R O G R A M M                      */
/**********************************************************************/

void main( void )
{
 int StartStrategie;          /* Speicherverteilung bei Programmstart */
 int StartUMB;                    /* UMB-Einbindung bei Programmstart */
 int Akt_UMB_ein;                          /* UMB Verwenden (ja/nein) */
 int Akt_UMB_zuerst;                          /* zuerst im UMB suchen */
 int Akt_Strategie;                         /* aktuelle Suchstrategie */

 /*-- Bildschirm aufbauen --------------------------------------------*/

 clrscr();
 Print( 1, 1, 0x07,
            "Demonstrationsprogramm zur DOS-Speicherverwaltung " );
 Print( 51, 1, 0x07,
             " (C) 1991 by Michael Tischer" );
 Print( 1, 2, 0x07, "=======================================" );
 Print( 40, 2, 0x07, "=======================================" );
 Print( 25, 5, 0x07, "Speicherbelegung wird ermittelt" );

 /*-- DOS-Werte beim Programmstart sichern ---------------------------*/

 StartStrategie = LeseStrategie();  /* Strategie der Speicherbelegung */
 StartUMB = LeseUMB();                             /* UMB eingebunden */
 AllokiereSpeicher();                        /* Testumgebung schaffen */
 SetzeStrategie( StartStrategie );     /* alte Strategie restaurieren */
 SetzeUMB( StartUMB );

 if ( KonvSeg == 0 )           /* konventionellen Speicher allokiert? */
  {                                /* Nein, mit Fehlermeldung beenden */
   clrscr();
   printf( "MEMDEMOC: Zu wenig Speicher!\n" );
   exit(1);
  }

 /*-- Startwerte fr die Speicherverwaltung --------------------------*/

 Akt_UMB_ein = ( StartUMB == UMB_EIN );
 Akt_UMB_zuerst = ( ( StartStrategie & ZUERST_UMB ) == ZUERST_UMB );
 Akt_Strategie = ( StartStrategie & ( 0xFF ^ ZUERST_UMB ) );

 /*-- Demonstration der Speicherverteilung ---------------------------*/

 clrscr();
 Print( 1, 1, 0x07,
            "Demonstrationsprogramm zur DOS-Speicherverwaltung " );
 Print( 51, 1, 0x07,
             " (C) 1991 by Michael Tischer" );
 Print( 1, 2, 0x07, "=======================================" );
 Print( 40, 2, 0x07, "=======================================" );
 Demo( Akt_UMB_ein, Akt_UMB_zuerst, Akt_Strategie );

 /*-- Alte DOS-Werte restaurieren ------------------------------------*/

 SpeicherFreigeben();          /* den reservierten Speicher freigeben */
 SetzeStrategie( StartStrategie );
 SetzeUMB( StartUMB );
 clrscr();
}
