/**********************************************************************/
/*                   M I X U T I L  . H                               */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Header-Datei fr MIXUTIL.H                       */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 20.03.1994                                       */
/*  letztes Update : 25.03.1994                                       */
/**********************************************************************/
#ifndef __INC_MIX_UTIL_H
#define __INC_MIX_UTIL_H

#include "types.h"

                                                 /* Mixer-Portoffsets */
#define MIX_REGISTERPORT   0x04             /* Register einstellen... */
#define MIX_DATAPORT       0x05              /* Daten lesen/schreiben */

                                        /* CT1335-Mixer (im DSP 2.xx) */
#define MIX_RESET          0x00                 /* Mixer zurcksetzen */
#define MIX2_MASTERVOL     0x02         /* Gesamt-Volume (BITS 3,2,1) */
#define MIX2_MIDIVOL       0x06       /* MIDI(FM) Volume (BITS 3,2,1) */
#define MIX2_CDVOL         0x08             /* CD Volume (BITS 3,2,1) */
#define MIX2_VOICEVOL      0x0A       /* VOICE(DSP) Volume (BITS 2,1) */

                                       /* CT1345 Mixer (im DSP 3.xx ) */
#define MIX3_VOICEVOL       0x04    /* VOICE-Vol. (Bits 7,6,5  3,2,1) */
#define MIX3_MICVOL         0x0A       /* Mikrophon Volume (BITS 2,1) */

#define MIX3_ADCSTATE       0x0C            /* DSP-Eingang und Filter */
                                                    /* Eingangsfilter */
  #define MIX3_ADCFILTEROFF    0x20      /* Bit 5: Low-Pass Filter an */
  #define MIX3_LOWPASS88       0x08 /* Bit 3: 8.8 kHz Low-Pass Filter */
                                                    /* Eingangsquelle */
  #define MIX3_MICSRC         0x00    /* BIT 2,1 = 00: Mikrophon      */
  #define MIX3_MICSRC_        0x04    /* BIT 2,1 = 10: auch Mikrophon */
  #define MIX3_CDSRC          0x02    /* BIT 2,1 = 01: CD             */
  #define MIX3_LINESRC        0x06    /* BIT 2,1 = 11: LINE IN        */
  #define MIX3_SRCMSK         0x06    /* Bitmaske der Source-Bits     */


#define MIX3_DACSTATE     0x0E    /* Ausgabefilter und Stereoschalter */
  #define MIX3_DACFILTEROFF 0x20  /* Bit 5: Low-Pass Filter an (s.o.) */
  #define MIX3_STEREOON     0x02              /* Bit 1: Stereoausgabe */


#define MIX3_MASTERVOL    0x22   /* Gesamtvol. (BITS 7,6,5 und 3,2,1) */
#define MIX3_MIDIVOL      0x26    /* MIDI-Vol. (BITS 7,6,5 und 3,2,1) */
#define MIX3_CDVOL        0x28       /* CD-Vol (BITS 7,6,5 und 3,2,1) */
#define MIX3_LINEVOL      0x2E    /* LINE-Vol. (BITS 7,6,5 und 3,2,1) */

                              /* CT1745 Mixer (im DSP 4.xx) sowie ASP */
                                                      /* Lautstrken: */
#define MIX4_MASTERVOL_L  0x30       /* Gesamt links (BITS 7,6,5,4,3) */
#define MIX4_MASTERVOL_R  0x31      /* Gesamt rechts (BITS 7,6,5,4,3) */
#define MIX4_VOICEVOL_L   0x32   /* VOICE(DSP) links (BITS 7,6,5,4,3) */
#define MIX4_VOICEVOL_R   0x33  /* VOICE(DSP) rechts (BITS 7,6,5,4,3) */
#define MIX4_MIDIVOL_L    0x34         /* MIDI links (BITS 7,6,5,4,3) */
#define MIX4_MIDIVOL_R    0x35        /* MIDI rechts (BITS 7,6,5,4,3) */
#define MIX4_CDVOL_L      0x36           /* CD links (BITS 7,6,5,4,3) */
#define MIX4_CDVOL_R      0x37          /* CD rechts (BITS 7,6,5,4,3) */
#define MIX4_LINEVOL_L    0x38         /* LINE links (BITS 7,6,5,4,3) */
#define MIX4_LINEVOL_R    0x39        /* LINE rechts (BITS 7,6,5,4,3) */
#define MIX4_MICVOL       0x3A         /* Mikrophon (BITS 7,6,5,4,3 ) */
#define MIX4_PCSPEAKERVOL 0x3B         /* PC-Lautsprecher (BITS 7,6 ) */

#define MIX4_OUTSOURCE    0x3C                      /* Ausgabequellen */
#define MIX4_ADCSOURCE_L     0x3D               /* Sample-Quellen L/R */
#define MIX4_ADCSOURCE_R     0x3E

 #define MIX4_MIDI_L         0x40           /* Aktive Aufnahme-Quellen*/
 #define MIX4_MIDI_R         0x20
 #define MIX4_LINE_L         0x10
 #define MIX4_LINE_R         0x08
 #define MIX4_CD_L           0x04
 #define MIX4_CD_R           0x02
 #define MIX4_MIC            0x01

   /* Vorverstrker fr Ausgabe(OUT) und Samplen(ADC) (Bits 7 und 6 ) */
#define MIX4_ADCGAIN_L   0x3F
#define MIX4_ADCGAIN_R   0x40
#define MIX4_OUTGAIN_L   0x41
#define MIX4_OUTGAIN_R   0x42

#define MIX4_AGC   0x43               /* Mikrofonvorverstrker (20dB) */
 #define MIX4_AGCON 0x01                   /* Mikrofonvoverstrker an */

        /* Hhen und Bsse des Vorverstrkers (jeweils BITS 7,6,5,4 ) */
#define MIX4_TREBLE_L       0x44
#define MIX4_TREBLE_R       0x45
#define MIX4_BASS_L         0x46
#define MIX4_BASS_R         0x47
              /* Welche Interrupts und DMA-Leitungen werden benutzt ? */
#define MIX4_IRQ 0x80
 #define MIX4_IRQ2  0x01             /* 4 mgliche Interruptleitungen */
 #define MIX4_IRQ5  0x02
 #define MIX4_IRQ7  0x04
 #define MIX4_IRQ10 0x08

#define MIX4_DMA 0x81           /*- Welche DMA Leitung wird benutzt ? */
 #define MIX4_DMA0 0x01
 #define MIX4_DMA1 0x02
 /*#define MIX4_DMA2 0x04                           Diskettenlaufwerk */
 #define MIX4_DMA3 0x08
 /* #define MIX4_DMA4 0x10                               Kaskadierung */
 #define MIX4_DMA5 0x20
 #define MIX4_DMA6 0x40
 #define MIX4_DMA7 0x80

#define MIX4_IRQSOURCE 0x82
 #define MIX4_IRQ8DMA  0x01        /* Interupt von 8-Bit DMA und Midi */
 #define MIX4_IRQ16DMA 0x02                /* Interupt von 16-Bit DMA */
 #define MIX4_IRQMPU   0x04                       /* Interupt von MPU */

/* MIXUTIL-spezifische Konstanten                                     */
/* Fr Zugriff auf Laustrken-Arrays werden Konstanten benutzt,       */
/* die angeben, ob sich eine Portadresse auf den rechten, linken,     */
/* oder beide Kanle bezieht.                                         */
#define PORT     0    /* Fr Array-Zugriff: 1. Element = Port         */
#define CHANNEL  1                      /*  2. Element = Zugriffscode */

#define L 0                                                  /* links */
#define R 1                                                 /* rechts */

#define CH_LEFT  1                        /* Kennung fr linken Kanal */
#define CH_RIGHT 2                                   /* rechter Kanal */
#define CH_BOTH  3                                    /* beide Kanle */
#define CH_NONE  0                                      /* kein Kanal */
#define CH_MAX   6        /* Maximum von rechtem und linkem verwenden */

              /* Durchnumerierung der zur Verfgung stehenden Quellen */
#define CD          0
#define LINE        1
#define VOICE       2
#define MASTER      3
#define MIDI        4
#define MIC         5
#define PCSPEAKER   6
#define NUM_SOURCES 7

#define CD_L        7
#define LINE_L      8
#define VOICE_L     9
#define MASTER_L    10
#define MIDI_L      11
#define CD_R        12
#define LINE_R      13
#define VOICE_R     14
#define MASTER_R    15
#define MIDI_R      16

#define MAX_SRC     17

#define DAC TRUE                       /* fr 'mix3_PrepareForStereo' */
#define ADC FALSE

              /* Prototypen */
WORD mix_SetBase( PSBBASE pSBBASE, WORD iReset );
VOID mix_Write( WORD iReg, WORD iData );
WORD mix_Read( WORD iReg );
VOID mix_Reset( VOID );
VOID mix3_SetADCFilter( WORD iState );
WORD mix3_GetADCFilter( VOID );
VOID mix3_SetDACFilter( WORD iState );
WORD mix3_GetDACFilter( VOID );
VOID mix3_SetDACStereo( WORD iState );
WORD mix3_GetDACStereo( VOID );
VOID mix3_SetADDACLowPass( WORD iState );
WORD mix3_GetADDACLowPass( VOID );
VOID mix3_PrepareForStereo( WORD iMode );
VOID mix3_RestoreFromStereo( VOID );
VOID mix3_SetVolume( WORD iSource, WORD iVolL, WORD iVolR );
WORD mix3_GetVolume( WORD iSource );
VOID mix3_SetADCSource( WORD iSource );
WORD mix3_GetADCSource( VOID );

VOID mix4_PrepareForMonoADC( VOID );
VOID mix4_RestoreFromMonoADC( VOID );
VOID mix4_SetVolume( WORD iSource, WORD iVolL, WORD iVolR );
WORD mix4_GetVolume( WORD iSource );
VOID mix4_SetADCSourceL( WORD iSource, WORD iState );
VOID mix4_SetADCSourceR( WORD iSource, WORD iState );
WORD mix4_GetADCSourceL( WORD iSource );
WORD mix4_GetADCSourceR( WORD iSource );
VOID mix4_SetOUTSource( WORD iSource, WORD iState );
WORD mix4_GetOUTSource( WORD iSource );
VOID mix4_SetADCGain( WORD iGainL, WORD iGainR );
WORD mix4_GetADCGain( WORD iChannel );
VOID mix4_SetOUTGain( WORD iGainL, WORD iGainR );
WORD mix4_GetOUTGain( WORD iChannel );
VOID mix4_SetAGC( WORD iState );
WORD mix4_GetAGC( VOID );
VOID mix4_SetTrebble( WORD iTrebbleL, WORD iTrebbleR );
WORD mix4_GetTrebble( WORD iChannel );
VOID mix4_SetBass( WORD iBassL, WORD iBassR );
WORD mix4_GetBass( WORD iChannel );
#endif

