/**********************************************************************/
/*                           M O U S E C . C                          */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : Stellt verschiedene Funktionen zur Arbeit mit  */
/*                     der Maus zur Verfgung                         */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 20.04.1989                                     */
/*    letztes Update : 22.04.1989                                     */
/*--------------------------------------------------------------------*/
/*    Speichermodell : SMALL                                          */
/**********************************************************************/

/*== Include-Dateien einbinden =======================================*/

#include <dos.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>

extern void far AssHand( void );            /* Extern-Deklaration des */
                                            /* Assembler-Handlers     */
/*== Typedefs ========================================================*/

typedef unsigned char BYTE;               /* wir basteln uns ein Byte */
typedef unsigned long PTRVIEW;               /* Maske fr Maus-Cursor */
typedef struct {                      /* beschreibt einen Mausbereich */
                BYTE x1,             /* Koordinaten der oberen linken */
                     y1,             /* und unteren rechten Ecke des  */
                     x2,             /* spezifizierten Bereichs       */
                     y2;
                PTRVIEW ptr_mask;            /* Maske fr Maus-Cursor */
               } BEREICH;
typedef void (far * MOUHAPTR)( void );   /* Pointer auf Event-Handler */

/*== Konstanten ======================================================*/

#define TRUE  ( 1 == 1 )
#define FALSE ( 1 == 0 )

/*-- Event-Codes -----------------------------------------------------*/

#define EV_MOU_MOVE      1                             /* Maus bewegt */
#define EV_LEFT_PRESS    2            /* linker Mausknopf niedergedr. */
#define EV_LEFT_REL      4            /* linker Mausknopf losgelassen */
#define EV_RIGHT_PRESS   8           /* rechter Mausknopf niedergedr. */
#define EV_RIGHT_REL    16           /* rechter Mausknopf losgelassen */
#define EV_MOU_ALL      31                        /* alle Maus-Events */

#define KEIN_BEREICH 255           /* Maus-Cursor nicht in Bereich xy */

/*-- Makros ----------------------------------------------------------*/

#define MouGetCol()         (ev_col)        /* liefern Mausposition & */
#define MouGetRow()         (ev_row)        /* -bereich im Moment des */
#define MouGetBereich()     (ev_ber)        /* Event-Eintritts        */
#define MouAvail()          ( mavail )  /* liefert TRUE, wenn Maus v. */
#define MouGetAktCol()      ( moucol )       /* liefern jeweils aktu- */
#define MouGetAktRow()      ( mourow )       /* elle Mausposition und */
#define MouGetAktBer()      ( mouber )       /* aktuellen Mausbereich */
#define MouIsLeftPress()    ( mouevent & EV_LEFT_PRESS )
#define MouIsLeftRel()      ( mouevent & EV_LEFT_REL )
#define MouIsRightPress()   ( mouevent & EV_RIGHT_PRESS )
#define MouIsRightRel()     ( mouevent & EV_RIGHT_REL )
#define MouSetMoveAreaAll() MouSetMoveArea( 0, 0, (BYTE) (tcol-1), \
                                                  (BYTE) (tline-1) );

#define ELVEK(x) ( sizeof(x) / sizeof(x[0]) ) /* Anzahl Elemente in X */

/*-- Makros zur Erstellung der Bit-Maske, die das Erscheinungsbild ---*/
/*-- des Maus-Cursors definiert.                                   ---*/
/*-- Der Aufruf von MouPtrMask lautet beispielsweise:              ---*/
/*--   MouPtrMask( PTRDIFCHAR( 'x' ), PTRINVCOL )                  ---*/
/*-- um den Maus-Cursor als ein kleines X darzustellen, das die    ---*/
/*-- invertierte Farbe des Zeichens trgt, das von ihm berdeckt   ---*/
/*-- wird.                                                         ---*/

#define MouPtrMask( z, f )\
  ( (( (PTRVIEW) f) >> 8 << 24) + ((( PTRVIEW) z) >> 8 << 16) +\
    (((f) & 255) << 8) + ((z) & 255) )

#define PTRSAMECHAR   ( 0x00ff )        /* gleiches Zeichen           */
#define PTRDIFCHAR(z) ( (z) << 8 )      /* anderes Zeichen            */
#define PTRSAMECOL    ( 0x00ff )        /* gleiche Farbe              */
#define PTRINVCOL     ( 0x7777 )        /* Farbe invertiert           */
#define PTRSAMECOLB   ( 0x807f )        /* gleiche Farbe blinkend     */
#define PTRINVCOLB    ( 0xF777 )        /* invertierte Farbe blinkend */
#define PTRDIFCOL(f)  ( (f) << 8 )      /* andere Farbe               */
#define PTRDIFCOLB(f) (((f)|0x80) << 8) /* andere Farbe blinkend      */

#define UND  0              /* Event-Verknpfungen fr MouEventWait() */
#define ODER 1

#define MOUINT(rin, rout) int86(0x33, &rin, &rout)
#define MOUINTX(rin, rout, sr) int86x(0x33, &rin, &rout, &sr)

/*-- Makros zur Umrechnung zwischen Maus-Koordinaten in Bezug auf den */
/*-- virtuellen Maus-Bildschirm und dem Textbildschirm                */

#define XTOCOL(x) ( (x) >> 3 )                           /* X durch 8 */
#define YTOROW(y) ( (y) >> 3 )                       /* Zeile durch 8 */
#define COLTOX(c) ( (c) << 3 )                             /* C mal 8 */
#define ROWTOY(r) ( (r) << 3 )                         /* Zeile mal 8 */

/*== globale Variablen ===============================================*/

BYTE tline,                                     /* Anzahl Text-Zeilen */
     tcol,                                     /* Anzahl Text-Spalten */
     mavail = FALSE;                 /* ist TRUE, wenn Maus verfgbar */

/*-- Maske fr den Standard-Maus-Cursor ------------------------------*/

PTRVIEW stdptr = MouPtrMask( PTRSAMECHAR, PTRINVCOL );

BYTE    * bpuf,              /* Ptr auf Puffer fr Bereichs-Erkennung */
        anz_bereich = 0;           /* bisher keine Bereiche definiert */

BEREICH * akt_bereich;            /* Pointer auf akt. Bereichs-Vektor */
int     blen;                               /* Lnge des BPUF in Byte */

/*-- Variablen, die mit jedem Aufruf des Maus-Handlers geladen werden */

BYTE mouber = KEIN_BEREICH,                 /* aktueller Maus-Bereich */
     moucol,                          /* Mausspalte (Text-Bildschirm) */
     mourow;                          /* Mauszeile  (Text-Bildschirm) */
int  mouevent = EV_LEFT_REL + EV_RIGHT_REL;            /* Event-Maske */

/*-- Variablen, die nur bei Eintritt eines erwarteten Events durch ---*/
/*-- den Maus-Handler geladen werden                               ---*/

BYTE ev_ber,                /* Bereich, in dem sich die Maus befindet */
     ev_col,                                            /* Mausspalte */
     ev_row;                                             /* Mauszeile */

/***********************************************************************
*  Funktion         : M o u D e f i n e P t r                          *
**--------------------------------------------------------------------**
*  Aufgabe          : Definiert die Cursor- und die Screen-Maks, die   *
*                     das Erscheinungsbild des Maus-Cursors bestimmen  *
*  Eingabe-Parameter: MASK = die beiden Bit-Masken, vereint zu einem   *
*                            32-Bit-Wert vom Typ UNSIGNED LONG         *
*  Return-Wert      : keiner                                           *
*  Info             : - Die hherwertigen 16-Bit von MASK reprsen-    *
*                       tieren die Screen-Mask, die niederwertigen 16  *
*                       Bit die Cursor-Mask                            *
***********************************************************************/

#pragma check_stack(off)                  /* kein Stack-Checking hier */

void MouDefinePtr( PTRVIEW mask )
{
 static PTRVIEW altercursor = (PTRVIEW) 0;   /* letzter Wert fr MASK */
 union REGS regs;                /* Prozessorregs fr Interruptaufruf */

 if ( altercursor != mask )  /* Vernderung gegenber letztem Aufruf? */
  {                                                             /* Ja */
   regs.x.ax = 0x000a;          /* Fktnr. fr "Set text pointer type" */
   regs.x.bx = 0;                       /* Software-Cursor einstellen */
   regs.x.cx = mask;                         /* Lo-Word ist AND-Maske */
   regs.x.dx = mask >> 16;                   /* Hi-Word ist XOR-Maske */
   MOUINT(regs, regs);                       /* Maus-Treiber aufrufen */
   altercursor = mask;                       /* neue Bit-Maske merken */
  }
}

/***********************************************************************
*  Funktion         : M o u E v e n t H a n d l e r                    *
**--------------------------------------------------------------------**
*  Aufgabe          : Wird vom Maustreiber ber die Assembler-Routine  *
*                     AssHand aufgerufen, sobald ein Ereignis in Ver-  *
*                     bindung mit der Maus eingetreten ist.            *
*  Eingabe-Parameter: EvFlags  = Event-Makse des Ereignis              *
*                     ButState = Status der Mausknpfe                 *
*                     X, Y     = die aktuelle Position des Maus-Cur-   *
*                                sors, bereits umgerechnet in das Ko-  *
*                                ordinatensystem des Textbildschirms   *
*  Return-Wert      : keiner                                           *
*  Info             : - Diese Funktion ist nur zum Aufruf durch den    *
*                       Maus-Treiber bestimmt und darf nicht von einer *
*                       anderen Funktion aufgerufen werden.            *
***********************************************************************/

void MouEventHandler( int EvFlags, int ButState, int x, int y )
{
 #define LBITS ( EV_LEFT_PRESS | EV_LEFT_REL )
 #define RBITS ( EV_RIGHT_PRESS | EV_RIGHT_REL )

 unsigned neuber;                        /* Nummer des neuen Bereichs */

 mouevent &= ~1;                                  /* Bit 0 ausblenden */
 mouevent |= ( EvFlags & 1 );           /* Bit 0 aus EvFlags kopieren */

 if ( EvFlags & LBITS )      /* linker Mausknopf losg. oder niederg.? */
  {                                                             /* Ja */
   mouevent &= ~LBITS;                /* bisherigen Status ausblenden */
   mouevent |= ( EvFlags & LBITS );        /* neuen Status einblenden */
  }

 if ( EvFlags & RBITS )     /* rechter Mausknopf losg. oder niederg.? */
  {                             /* Ja, Bits ausblenden und einblenden */
   mouevent &= ~RBITS;                /* bisherigen Status ausblenden */
   mouevent |= ( EvFlags & RBITS );        /* neuen Status einblenden */
  }

 moucol = x;                       /* Spalte in Textspalten umrechnen */
 mourow = y;                         /* Zeile in Textzeilen umrechnen */

 /*-- Bereich ermitteln, in dem sich die Maus befindet und fest-   ---*/
 /*-- stellen, ob sich der Bereich seit dem letzten letzten Aufruf ---*/
 /*-- des Handlers verndert hat. In diesem Fall mu das Erschei-  ---*/
 /*-- nungsbild des Maus-Cursors neu definiert werden.             ---*/

 neuber = *(bpuf + mourow * tcol + moucol);          /* Bereich holen */
 if ( neuber != mouber )                            /* neuer Bereich? */
  MouDefinePtr((neuber==KEIN_BEREICH) ? stdptr :
                                        (akt_bereich+neuber)->ptr_mask);
 mouber = neuber;           /* Bereichsnummer in globaler Var. merken */
}

#pragma check_stack              /* alten Zustand im Hinblick auf das */
#pragma check_stack              /* Stack-Checking wieder herstellen  */

/***********************************************************************
*  Funktion         : M o u I B u f F i l l                            *
**--------------------------------------------------------------------**
*  Aufgabe          : Speichert den Bereichscode fr einen bestimmten  *
*                     Bildschirmbereich innerhalb des Modul-internen   *
*                     Bildschirmspeichers                              *
*  Eingabe-Parameter: x1, y1 = obere linke Ecke des Bildschirmbereichs *
*                     x2, y2 = untere rechte Ecke des Bildschirmb.     *
*                     CODE   = Bereichscode.                           *
*  Return-Wert      : keiner                                           *
*  Info             : Diese Funktion darf nur innerhalb dieses Moduls  *
*                     aufgerufen werden.                               *
***********************************************************************/

static void MouIBufFill( BYTE x1, BYTE y1,
                         BYTE x2, BYTE y2, BYTE code )
{
 register BYTE * lptr;                /* Laufzeiger in den Bereichsp. */
 BYTE i, j;                                        /* Schleifenzhler */

 lptr = bpuf + y1 * tcol + x1;              /* Pointer in erste Zeile */

 /*-- die einzelnen Zeilen durchlaufen -------------------------------*/
 for (j=x2 - x1 + 1 ; y1 <= y2; ++y1, lptr+=tcol )
  memset( lptr, code, j );                             /* Code setzen */
}

/***********************************************************************
*  Funktion         : M o u D e f B e r e i c h                        *
**--------------------------------------------------------------------**
*  Aufgabe          : Erlaubt die Definition verschiedener Bildschirm- *
*                     berreiche, die zur Arbeit mit der Maus mit einem *
*                     unterschiedlichen code versehen werden.          *
*  Eingabe-Parameter: - ANZAHL = Anzahl der Bildschirmbereiche         *
*                     - PTR    = Pointer auf Vektor mit den Bereichs-  *
*                                beschreibern vom Typ BEREICH          *
*  Return-Wert      : keiner                                           *
*  Info             : - Den freibleibenden Bildschirmbereichen wird    *
*                       der Code KEIN_BEREICH zugewiesen.              *
*                     - Beim Eintritt in einen der angegegebenen Bild- *
*                       schirmbereiche schaltet der Maus-Handler auto- *
*                       matisch auf das Erscheinungsbild des Maus-     *
*                       Cursors aus dem zugehrigen Bereichsbeschrei-  *
*                       ber um.                                        *
*                     - Da nur der bergebene Poniter gespeichert,     *
*                       nicht aber der bergebene Vektor in einen se-  *
*                       paraten Puffer kopiert wird, darf der Inhalt   *
*                       des Vektors bis zum nchsten Aufruf dieser     *
*                       Funktion nicht verndert werden.               *
***********************************************************************/

void MouDefBereich( BYTE anzahl, BEREICH * ptr )
{
 register BYTE i,                                  /* Schleifenzhler */
               bereich;                                /* Mausbereich */

 akt_bereich = ptr;                     /* Pointer auf Vektor und An- */
 anz_bereich = anzahl;                  /* zahl der Bereiche merken   */
 memset( bpuf, KEIN_BEREICH, blen );
 for (i=0 ; i<anzahl ; ++ptr )
  MouIBufFill( ptr->x1, ptr->y1, ptr->x2, ptr->y2, i++);

 /*-- Mauscursor neu definieren --------------------------------------*/

 bereich = *(bpuf + mourow * tcol + moucol);           /*akt. Mausber.*/
 MouDefinePtr( ( bereich == KEIN_BEREICH ) ? stdptr
               : (akt_bereich+bereich)->ptr_mask );
}

/***********************************************************************
*  Funktion         : M o u E v e n t W a i t                          *
**--------------------------------------------------------------------**
*  Aufgabe          : Wartet auf den Eintritt eines bestimmten Ereig-  *
*                     nisses von der Tastatur.                         *
*  Eingabe-Parameter: TYP        = legt die Verknpfung zwischden den  *
*                                  verschiedenen Ereignissen fest.     *
*                     WAIT_EVENT = Bit-Maske, die das zu erwartende    *
*                                  Ereignis spezifiziert.              *
*  Return-Wert      : Bit-Maske, die das oder die eingetretenen Er-    *
*                     eignisse beschreibt                              *
*  Info             : - WAIT_EVENT kann durch Oder-Verknpfung der     *
*                       verschiedenen Konstanten wie z.B. EV_MOU_MOVE  *
*                       oder EV_LEFT_PRESS gebildet werden             *
*                     - Mgliche Angaben fr Typ sind UND oder ODER.   *
*                       UND hat zur Folge, da die Funktion erst dann  *
*                       zum Aufrufer zurckkehrt, wenn alle erwarteten *
*                       Ereignisse gleichzeitig eintreten, whrend sich*
*                       ODER mit dem Eintritt mindestens eines Ereig-  *
*                       nisses begngt.                                *
***********************************************************************/

int MouEventWait( BYTE typ, int wait_event )
{
 int akt_event;                   /* die jeweils aktuelle Event-Maske */
 register BYTE spalte = moucol,                /* letzte Mausposition */
               zeile  = mourow;
 BYTE ende = FALSE;           /* wird TRUE, wenn Ereignis eingetreten */

 while ( !ende )             /* wiederholen, bis Ereignis eingetreten */
  {
   /*-- warten bis eines der Ereignisse eintritt ---------------------*/

   if ( typ == UND )         /* UND: alle Ereignisse mssen eintreten */
     while ( (akt_event = mouevent) != wait_event)
      ;
   else                /* ODER: wenigstens ein Ereignis mu eintreten */
     while ( ( (akt_event = mouevent) & wait_event) == 0)
      ;

   akt_event &= wait_event;            /* nur Event-Bits stehenlassen */

   /*-- wenn auf die Bewegung der Maus gewartet wird, wird das Ereig- */
   /*-- nis nur akzeptiert, wenn der Maus-Cursor in eine anderer Zei- */
   /*-- le und/oder Spalte des Textbildschirms bewegt wurde           */

   if ((wait_event & EV_MOU_MOVE) && spalte==moucol && zeile==mourow)
    {                 /* Maus bewegt, aber gleiche Bildschirmposition */
     akt_event &= (~EV_MOU_MOVE);              /* Move-Bit ausblenden */
     ende = (akt_event != 0);                /* bleiben Events brig? */
    }
   else                                       /* Ereignis eingetreten */
    ende = TRUE;
  }
 ev_col = moucol;                       /* aktuelle Mausposition und  */
 ev_row = mourow;                       /* -bereich in globalen Vari- */
 ev_ber = mouber;                       /* ablen festhalten           */
 return( akt_event );                    /* Event-Maske zurckliefern */
}

/***********************************************************************
*  Funktion         : M o u I S e t E v e n t H a n d l e r            *
**--------------------------------------------------------------------**
*  Aufgabe          : Installiert einen Event-Handler, der beim Ein-   *
*                     tritt eines bestimmten Maus-Ereignisses vom      *
*                     Maus-Treiber aufgerufen wird.                    *
*  Eingabe-Parameter: EVENT = Bit-Maske, die das Ereignis spezifi-     *
*                             ziert, bei dessen Eintritt der Event-    *
*                             Handler aufgerufen werden soll.          *
*                     PTR   = Pointer auf den Maus-Handler             *
*  Return-Wert      : keiner                                           *
*  Info             : - EVENT kann durch Oder-Verknpfung der ver-     *
*                       schiedenen Konstanten wie z.B. EV_MOU_MOVE     *
*                       oder EV_LEFT_PRESS aus der Datei gebildet      *
*                       werden                                         *
***********************************************************************/

static void MouISetEventHandler( unsigned event, MOUHAPTR ptr )
{
 union REGS regs;                /* Prozessorregs fr Interruptaufruf */
 struct SREGS sregs;           /* Segmentregister fr Interruptaufruf */

 regs.x.ax = 0x000C;                /* Fktnr. fr "Set Mouse Handler" */
 regs.x.cx = event;                              /* Event-Maske laden */
 regs.x.dx = FP_OFF( ptr );             /* Offsetadresse des Handlers */
 sregs.es  = FP_SEG( ptr );            /* Segmentadresse des Handlers */
 MOUINTX( regs, regs, sregs );               /* Maus-Treiber aufrufen */
}

/***********************************************************************
*  Funktion         : M o u I G e t X                                  *
**--------------------------------------------------------------------**
*  Aufgabe          : Ermittelt die (Text-) Spalte, in der sich der    *
*                     Maus-Cursor befindet.                            *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : die Maus-Spalte in Bezug auf den Textbildschirm  *
***********************************************************************/

static BYTE MouIGetX( void )
{
 union REGS regs;                /* Prozessorregs fr Interruptaufruf */

 regs.x.ax= 0x0003;               /* Fktnr.: fr "Get mouse position" */
 MOUINT( regs, regs );                       /* Maus-Treiber aufrufen */
 return XTOCOL( regs.x.cx );         /* Spalte umrechnen und zurckl. */
}

/***********************************************************************
*  Funktion         : M o u I G e t Y                                  *
**--------------------------------------------------------------------**
*  Aufgabe          : Ermittelt die (Text-) Zeile, in der sich der     *
*                     Maus-Cursor befindet.                            *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : die Maus-Zeile in Bezug auf den Textbildschirm   *
***********************************************************************/

static BYTE MouIGetY( void )
{
 union REGS regs;                /* Prozessorregs fr Interruptaufruf */

 regs.x.ax= 0x0003;               /* Fktnr.: fr "Get mouse position" */
 MOUINT(regs, regs);                         /* Maus-Treiber aufrufen */
 return YTOROW(regs.x.dx);            /* Zeile umrechnen und zurckl. */
}

/***********************************************************************
*  Funktion         : M o u S h o w M o u s e                          *
**--------------------------------------------------------------------**
*  Aufgabe          : Maus-Cursor auf dem Bildschirm anzeigen.         *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : keiner                                           *
*  Info             : Die Aufrufe von MouHidemMouse() und MouShow-     *
*                     Mouse() mssen ausbalanciert sein, damit sie     *
*                     Wirkung zeigen.                                  *
***********************************************************************/

void MouShowMouse( void )
{
 union REGS regs;                /* Prozessorregs fr Interruptaufruf */

 regs.x.ax = 0x0001;                      /* Fktnr.: fr "Show Mouse" */
 MOUINT(regs, regs);                         /* Maus-Treiber aufrufen */
}

/***********************************************************************
*  Funktion         : M o u H i d e M o u s e                          *
**--------------------------------------------------------------------**
*  Aufgabe          : Maus-Cursor vom dem Bildschirm entfernen.        *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : keiner                                           *
*  Info             : Die Aufrufe von MouHidemMouse() und MouShow-     *
*                     Mouse() mssen ausbalanciert sein, damit sie     *
*                     Wirkung zeigen.                                  *
***********************************************************************/

void MouHideMouse( void )
{
 union REGS regs;                /* Prozessorregs fr Interruptaufruf */

 regs.x.ax = 0x0002;                       /* Fktnr. fr "Hide Mouse" */
 MOUINT(regs, regs);                         /* Maus-Treiber aufrufen */
}

/***********************************************************************
*  Funktion         : M o u S e t M o v e A r e a                      *
**--------------------------------------------------------------------**
*  Aufgabe          : Definiert den Bildschirmbereich, innerhalb dessen*
*                     sich der Maus-Cursor bewegen darf.               *
*  Eingabe-Parameter: x1, y1 = Koordinaten der oberen linken Ecke      *
*                     x2, y2 = Koordinaten der unteren rechten Ecke    *
*  Return-Wert      : keiner                                           *
*  Info             : - Beide Angaben beziehen sich auf den Textbild-  *
*                       schirm und nicht auf den virtuellen Grafik-    *
*                       bildschirm des Maustreibers                    *
***********************************************************************/

void MouSetMoveArea( BYTE x1, BYTE y1, BYTE x2, BYTE y2 )
{
 union REGS regs;                /* Prozessorregs fr Interruptaufruf */

 regs.x.ax = 0x0008;              /* Fktnr. fr "Set vertical Limits" */
 regs.x.cx = ROWTOY( y1 );                /* Umrechnung in virtuellen */
 regs.x.dx = ROWTOY( y2 );                /* Mausbildschirm           */
 MOUINT(regs, regs);                         /* Maus-Treiber aufrufen */
 regs.x.ax = 0x0007;            /* Fktnr. fr "Set horizontal Limits" */
 regs.x.cx = COLTOX( x1 );                /* Umrechnung in virtuellen */
 regs.x.dx = COLTOX( x2 );                /* Mausbildschirm           */
 MOUINT(regs, regs);                         /* Maus-Treiber aufrufen */
}

/***********************************************************************
*  Funktion         : M o u S e t S p e e d                            *
**--------------------------------------------------------------------**
*  Aufgabe          : Legt das Verhltnis zwischen der Lnge einer     *
*                     Mausbewegung und der daraus resultierenden Be-   *
*                     wegung des Maus-Cursors fest.                    *
*  Eingabe-Parameter: - XSPEED = Geschwindigkeit horizontal            *
*                     - YSPEED = Geschwindigkeit vertikal              *
*  Return-Wert      : keiner                                           *
*  Info             : - Beide Angaben beziehen sich auf die Einheit    *
*                       Mickey / 8 Pixel.                              *
***********************************************************************/

void MouSetSpeed( int xspeed, int yspeed )
{
 union REGS regs;                /* Prozessorregs fr Interruptaufruf */

 regs.x.ax = 0x000f;           /* Fktnr. "Set mickeys to pixel ratio" */
 regs.x.cx = xspeed;
 regs.x.dx = yspeed;
 MOUINT(regs, regs);                         /* Maus-Treiber aufrufen */
}

/***********************************************************************
*  Funktion         : M o u M o v e P t r                              *
**--------------------------------------------------------------------**
*  Aufgabe          : Bewegt den Maus-Cursor an eine bestimmte Bild-   *
*                     schirmposition.                                  *
*  Eingabe-Parameter: - COL = neue Bildschirmspalte                    *
*                     - ROW = neue Bildschirmzeile                     *
*  Return-Wert      : keiner                                           *
*  Info             : - Beide Angaben beziehen sich auf den Textbild-  *
*                       schirm und nicht auf den virtuellen Grafik-    *
*                       bildschirm des Maustreibers                    *
***********************************************************************/

void MouMovePtr( int col, int row )
{
 union REGS regs;                /* Prozessorregs fr Interruptaufruf */
 unsigned neuber;             /* Bereich, in den die Maus bewegt wird */

 regs.x.ax = 0x0004;           /* Fktnr. "Set mouse pointer position" */
 regs.x.cx = COLTOX( moucol = col );  /* Koordinaten umrechnen und in */
 regs.x.dx = ROWTOY( mourow = row );  /* globalen Variablen speichern */
 MOUINT(regs, regs);                         /* Maus-Treiber aufrufen */

 neuber = *(bpuf + mourow * tcol + moucol);          /* Bereich holen */
 if ( neuber != mouber )                            /* neuer Bereich? */
  MouDefinePtr((neuber==KEIN_BEREICH) ? stdptr :
                                        (akt_bereich+neuber)->ptr_mask);
 mouber = neuber;              /* Bereichsnr. in globaler Var. merken */
}

/***********************************************************************
*  Funktion         : M o u S e t D e f a u l t P t r                  *
**--------------------------------------------------------------------**
*  Aufgabe          : Definiert das Erscheinungsbild des Maus-Cursors  *
*                     fr Bildschirmbereiche, die nicht mit Hilfe von  *
*                     MouDefBereich definiert wurden.                  *
*  Eingabe-Parameter: STANDARD = Bit-Maske fr den Standard-Maus-Cursor*
*  Return-Wert      : keiner                                           *
***********************************************************************/

void MouSetDefaultPtr( PTRVIEW standard )
{
 stdptr = standard;              /* Bit-Maske in globaler Var. merken */

 /*-- befindet sich die Maus im Augenblick in keinem Bereich, wird ---*/
 /*-- direkt auf das neue Erscheinungsbild umgeschaltet            ---*/

 if ( MouGetBereich() == KEIN_BEREICH )         /* in keinem Bereich? */
  MouDefinePtr( standard );                                   /* Nein */
}

/***********************************************************************
*  Funktion         : M o u E n d                                      *
**--------------------------------------------------------------------**
*  Aufgabe          : Beendet die Arbeit mit den Funktionen des Mousec-*
*                     Moduls.                                          *
*  Eingabe-Parameter: keine                                            *
*  Return-Wert      : keiner                                           *
*  Info             : Die Funktion wird bei der Beendigung eines Pro-  *
*                     gramms automatisch aufgerufen, sofern zuvor      *
*                     MouInstall aufgerufen wurde.                     *
***********************************************************************/

void MouEnd( void )
{
 union REGS regs;                /* Prozessorregs fr Interruptaufruf */

 MouHideMouse();              /* Maus-Cursor vom Bildschirm entfernen */
 regs.x.ax = 0;                            /* Reset des Maus-Treibers */
 MOUINT(regs, regs);                         /* Maus-Treiber aufrufen */

 free( bpuf );               /* allokierten Speicher wieder freigeben */
}

/***********************************************************************
*  Funktion         : M o u I n i t                                    *
**--------------------------------------------------------------------**
*  Aufgabe          : Leitet die Arbeit mit dem Mousec-Modul ein und   *
*                     initialisiert die verschiedenen Variablen        *
*  Eingabe-Parameter: Spalten, = die Auflsung des Textbildschirms     *
*                     Zeilen                                           *
*  Return-Wert      : TRUE, wenn eine Maus installiert ist,sonst FALSE *
*  Info             : Diese Funktion mu als erste Funktion aus diesem *
*                     Modul aufgerufen werden.                         *
***********************************************************************/

BYTE MouInit( BYTE spalten, BYTE zeilen )
{
 union REGS regs;                /* Prozessorregs fr Interruptaufruf */

 tline = zeilen;                     /* Anzahl Zeilen- und Spalten in */
 tcol  = spalten;                    /* globalen Variablen speichern  */

 atexit( MouEnd );                 /* am Programmende MouEnd aufrufen */

 /*-- Puffer fr Maus-Bereiche allokieren und fllen -----------------*/

 bpuf = (BYTE *) malloc( blen = tline * tcol );
 MouIBufFill( 0, 0, (BYTE) (tcol-1), (BYTE) (tline-1), KEIN_BEREICH );

 regs.x.ax = 0;                       /* Mouse-Treiber initialisieren */
 MOUINT(regs, regs);                         /* Maus-Treiber aufrufen */
 if ( regs.x.ax != 0xffff )              /* Maus-Treiber installiert? */
  return FALSE;                                               /* Nein */

 MouSetMoveAreaAll();                       /* Bewegunsbereich setzen */

 moucol = MouIGetX();                     /* aktuelle Mausposition in */
 mourow = MouIGetY();                     /* globale Variablen laden  */

 /*-- den Assembler-Event-Handler "AssHand" installieren -------------*/
 MouISetEventHandler( EV_MOU_ALL, (MOUHAPTR) AssHand );

 return mavail = TRUE;                        /* Maus ist installiert */
}

/***********************************************************************
*                          H A U P T P R O G R A M M                   *
***********************************************************************/

int main( void )
{
 static BEREICH bereiche[] =        /* die verschiedenen Mausbereiche */
  {
   {  0,  0, 79,  0, MouPtrMask( PTRDIFCHAR(0x18), PTRINVCOL)  },
   {  0,  1,  0, 23, MouPtrMask( PTRDIFCHAR(0x1b), PTRINVCOL)  },
   {  0, 24, 78, 24, MouPtrMask( PTRDIFCHAR(0x19), PTRINVCOL)  },
   { 79,  1, 79, 23, MouPtrMask( PTRDIFCHAR(0x1a), PTRINVCOL)  },
   { 79, 24, 79, 24, MouPtrMask( PTRDIFCHAR('X'),  PTRDIFCOLB(0x40) ) },
  };

 printf("\nMOUSEC - (c) 1989, 91 by MICHAEL TISCHER\n\n");
 if ( MouInit( 80, 25 ) )               /* Mouse-Modul initialisieren */
  {                 /* alles o.k., es ist ein Maustreiber installiert */
   printf("Wenn Sie den  Maus-Cursor ber den  Bildschirm und vor\n"\
          "allem an dessen Rand entlang bewegen, werden Sie fest-\n"\
          "feststellen, da  sich das Erscheinungsbild  des Maus-\n"\
          "Cursors abhngig von Seiner Position verndert.\n\n"
          "Um diese Demo zu beenden, bewegen Sie  den Maus-Cursor\n"\
          "bitte in die untere rechte  Bildschirmecke und betti-\n"\
          "gen dann gleichzeitig den linken und den rechten Maus-\n"\
          "knopf.\n" );

   MouSetDefaultPtr( MouPtrMask( PTRDIFCHAR( '' ), PTRDIFCOL( 3 ) ) );
   MouDefBereich( ELVEK( bereiche ), bereiche );     /* Bereiche def. */
   MouShowMouse();          /* Mauscursor auf dem Bildschirm anzeigen */

   /*-- warten, bis der linke und der rechte Mausknopf gleichzeitig --*/
   /*-- niedergedrckt werden und sich der Maus-Cursor dabei in dem --*/
   /*-- Bereich 4 befindet                                          --*/

   do                                              /* Abfrageschleife */
    MouEventWait( UND, EV_LEFT_PRESS | EV_RIGHT_PRESS );
   while ( MouGetBereich() != 4 );

   return 0;                          /* OK-Code an DOS zurckliefern */
  }
 else                 /* keine Maus oder kein Maustreiber installiert */
  {
   printf("Es ist kein Maustreiber installiert!\n");
   return 1;                       /* Fehlercode an DOS zurckliefern */
  }
}

