/*****************************************************************************
*                                 M R C I C . C                              *
*----------------------------------------------------------------------------*
*     Aufgabe        : Demonstriert die Nutzung eines MRCI-Servers zur       *
*                      Kompression und Dekompression von Daten               *
*----------------------------------------------------------------------------*
*     Autor          : MICHAEL TISCHER                                       *
*     entwickelt am  : 25.09.1993                                            *
*     letztes Update : 10.10.1994                                            *
*----------------------------------------------------------------------------*
*     Compiler       : QuickC und VC                                         *
*     Speichermodell : nach Wahl                                             *
*****************************************************************************/

#include "dos.h"
#include "stdio.h"

#pragma pack(1)   /* kein Auto-Word-Alignment in Strukturen */

/*-- Typen und Strukturen --------------------------------------------------*/
typedef unsigned short WORD;
typedef int BOOL;
typedef unsigned char BYTE;

typedef struct {                               /* beschreibt MRCI-Server */
      long HerstellerID;                  /* 4-byte ASCII-Code mit */
                                                     /* Hersteller-Namen      */
      WORD HerstellerVersion;        /* Versionsnummer des MRC-Servers     */
                                        /* High-Byte is Hauptversion, Lo-Byte */
                                        /* ist Unterversion                   */
      WORD MrciVersion;       /* Versionsnummer des MRCI, das dieser */
                                       /* Server untersttzt                  */
      void far * Einsprungspunkt;         /* Einsprungspunkt fr Ausfhrung */
                                            /* von Server-Funktionen          */
      WORD Flags;                        /* Server-Fhigkeiten */
      WORD HardwareFlags;            /* Server-Fhigkeiten in Hardware */
      WORD MaxBytes;             /* grter Block, den der Server */
                                             /* verarbeiten kann              */
    } MRCINFO;

typedef struct {          /* fr Aufruf von Server-Funktionen */
       void far *SourcePtr;                        /* Zeiger in Quellpuffer */
       WORD SourceLen;                  /* Lnge des Quellpuffers in Byte */
       WORD Reserviert1;
       void far *DestPtr;                         /* Zeiger in Zielpuffer */
       WORD DestLen;                           /* Lnge des Zielpuffers */
       WORD ChunkLen;               /* Blockgre fr komprimierte Daten */
       long IncDecomp;           /* Status fr inkrementelles Dekompr. */
     } MRCIREQUEST;

typedef struct {          /* beschreibt einen Puffer fr Compress/Decompress */
   void far *adresse;                                    /* Zeiger auf Puffer */
   WORD lenb;                                  /* Lnge des Puffers in Byte */
 } BUF;

/*-- Konstanten --------------------------------------------------------------*/

#define TRUE  (0==0)
#define FALSE (0==1)

/*-- Flags und Befehlscodes --------------------------------------------------*/
#define STANDARD_COMPRESS    1                         /* Standard-Komprimieren */
#define STANDARD_DECOMPRESS  2                     /* Standard-Dekomprimieren */
#define MAX_COMPRESS         8                            /* maximales Komprimieren  */
#define INC_DECOMPRESS      32                   /* inkrementelles Dekomprimieren */

/*-- Art des MRCI-Clients ----------------------------------------------------*/
#define TYP_ANWENDUNG    0                   /* MRCI-Client ist eine Anw}ung */
#define TYP_TSRTREIBER   1           /* MRCI-Client ist TSR oder Gertetreiber */

/*-- Fehlercodes -------------------------------------------------------------*/
#define MRCI_OK             0                                  /* alles o.k. */
#define MRCI_ERR_FUNKTION   1                      /* falscher Funktionscode */
#define MRCI_ERR_BUSY       2                      /* Server ist beschftigt */
#define MRCI_ERR_OVERFLOW   3                         /* Zielpuffer zu klein */
#define MRCI_ERR_NOCOMPRESS 4       /* Daten knnen nicht komprimiert werden */

/*-- Konstanten fr Demoprogramm ---------------------------------------------*/
#define TYP_APP       TYP_ANWENDUNG                 /* das hier ist kein TSR */
#define CHUNK         1               /* wenn es geht, bis auf 1 Byte kompr. */

/*-- globale Variablen -------------------------------------------------------*/

MRCINFO mrci;                  /* nimmt MRCI-Info-Block nach Aufruf von */
                                     /* MrciQuery auf                         */

/*****************************************************************************
* MrciQuery : berprft die Existenz eines MRCI-Servers in Hardware oder     *
*             Software und liefert Informationen ber diesen Server          *
*----------------------------------------------------------------------------*
* Eingabe : MrciInfo = enthlt nach einem erfolgreichen Funktionsaufruf      *
*                      einen MRCI-Info-Block mit Informationen ber den      *
*                      Server                                                *
* Ausgabe : TRUE wenn Mrci-Server installiert ist, sonst FALSE               *
*****************************************************************************/

int MrciQuery ( MRCINFO* pmrci  )
{
 BOOL server; /* gibt es einen MRCI-Server? */
 union REGS regs;
 struct SREGS segregs;

 server = FALSE;
 regs.x.ax = 0x4A12;                               /* MUX-Code fr MRCI-Server */
 regs.x.cx = ('M' << 8) + 'R';                  /* CX = 'MR' */
 regs.x.dx = ('C' << 8) + 'I';                  /* DX = 'CI' */
 int86x( 0x2f, &regs, &regs, &segregs);
 if ( (regs.x.cx == ( 'I' << 8 ) + 'C')  &&     /* CX == 'IC' */
      (regs.x.dx == ( 'R' << 8 ) + 'M')  )     /* DX == 'RM' */
    server = TRUE;                     /* CX und DX o.k. ---> Server gefunden */
 else
  {             /* kein Sofware-Server gefunden, Hardware-Server testen */
   regs.x.ax = 0xB001;                    /* Int 1A, Funktion B001h aufrufen */
   regs.x.cx = ( 'M' << 8 ) + 'R' ;              /* CX = 'MR' */
   regs.x.dx = ( 'C' << 8 ) +  'I';              /* DX = 'CI' */
   int86x( 0x1A, &regs, &regs, &segregs);
   if ( (regs.x.cx == ( 'I' << 8 ) + 'C') &&
        (regs.x.dx == ( 'R' << 8 ) + 'M') )
   server = TRUE;                 /* CX und DX o.k. ---> Server gefunden */
  }

 if (server)      /* wenn Server gefunden, dann zeigt ES:DI auf Info-Block */
  * (MRCINFO far *) pmrci = * (MRCINFO far * )(((long) segregs.es << 16) + regs.x.di);
 return server;
}

/*****************************************************************************
* MrciCall : Ruft einen der verschiedenen Server-Dienste auf                 *
*----------------------------------------------------------------------------*
* Eingabe : OperationCode = Befehlscode (siehe Konstanten)                   *
*           SourceBuf     = Quellpuffer-Beschreiber                          *
*           DestBuf       = Zielpuffer-Beschreiber                           *
* Ausgabe : Fehlercode des MRCI-Servers                                      *
* Info    : Diese Funktion darf erst nach vorhergehendem und erfolgreichem   *
*           Aufruf von MrciQuery aufgerufen werden                           *
* Globals : mrci
*****************************************************************************/

int MRCICall( WORD OperationCode, BUF *sourcebuf, BUF *destbuf)
{
 MRCIREQUEST mrcir;              /* zum Aufbau eines MRCI-Request-Blocks */
 void far * mrcrp;                                       /* zeigt auf MRCIR */
 int  err;                          /* zurckgelieferter Fehlercode */

 mrcir.SourcePtr = sourcebuf->adresse;          /* Source-Puffer in Request- */
 mrcir.SourceLen = sourcebuf->lenb;             /* Block verzeichnen         */

 mrcir.DestPtr = destbuf->adresse;                /* Ziel-Puffer in Request- */
 mrcir.DestLen = destbuf->lenb;                   /* Block verzeichnen       */

 mrcir.ChunkLen = CHUNK;                 /* Chunk-Gre aus Konstante holen */
 mrcrp = &mrcir;                         /* Zeiger auf Request-Block merken */

 _asm
   {
    push    bp                                            /* Register sichern */
    push    ds

    /*-- zunchst in eine Windows-Critical-Section eintreten, damit unter ----*/
    /*-- Windows im 486 Enhanced Mode nicht mehrere VMs gleichzeitig den  ----*/
    /*-- MRCI-Server aufrufen knnen                                      ----*/

    push    ax                               /* Windows erwartet genau diese  */
    mov     ax,8001h                         /* Befehlsfolge und keine andere */
    int     2ah                     
    pop     ax                      

    /*--- Aufruf dss MRCI-Servers vorbereiten --------------------------------*/
    mov     ax,OperationCode
    mov     cx,TYP_APP

    mov     dx,seg mrci              /* Zeiger auf MRCI-Info-Block nach ES:BX */
    mov     es,dx
    mov     bx,offset mrci
    lds     si,mrcrp                        /* DS:SI auf Request-Block setzen */
    call    dword ptr es:[bx+8]               /* Server aufrufen */

    /*-- Windows-Critical-Section wieder verlassen ---------------------------*/
    push    ax                             /* auch hier mu die Befehlssquenz */
    mov     ax,8101h                       /* genau eingehalten werden        */
    int     2ah                 
    pop     ax                  

    pop     ds                                        /* Register zurckholen */
    pop     bp

    mov     err,ax                  /* zurckgelieferten Fehlercode speichern */
   }

  destbuf->lenb = mrcir.DestLen;       /* Lnge des Zielpuffers zurckliefern */
  return err;              /* Fehlercode als Funktionsergebnis zurckliefern */
}

/*****************************************************************************
* Compress : komprimiert einen Datenblock mit Hilfe des MRCI-Servers         *
*----------------------------------------------------------------------------*
* Eingabe : TypCompress = eine der Beiden Konstanten STANDARD_COMPRESS oder  *
*                         MAX_COMPRESS                                       *
*           SourceBuf   = Quellpuffer                                        *
*           DestBuf     = Zielpuffer                                         *
* Ausgabe : Fehlercode des MRCI-Servers                                      *
* Info    : Diese Funktion darf erst nach vorhergehendem und erfolgreichem   *
*           Aufruf von MrciQuery aufgerufen werden                           *
*****************************************************************************/

int Compress( WORD TypCompress, BUF *sourcebuf, BUF *destbuf )
{
 return MRCICall( TypCompress, sourcebuf, destbuf );
}

/*****************************************************************************
* Decompress : dekomprimiert einen Datenblock mit Hilfe des MRCI-Servers     *
*----------------------------------------------------------------------------*
* Eingabe : SourceBuf = Quellpuffer                                          *
*           DestBuf   = Zielpuffer                                           *
* Ausgabe : Fehlercode des MRCI-Servers                                      *
* Info    : - Diese Funktion darf erst nach vorhergehendem und erfolgreichem *
*             Aufruf von MrciQuery aufgerufen werden                         *
*           - Der Datenblock mu vom MRCI-Server komprimiert worden sein     *
*****************************************************************************/

int Decompress( BUF *sourcebuf, BUF *destbuf )
{
 return MRCICall( STANDARD_DECOMPRESS, sourcebuf, destbuf );
}

/*****************************************************************************
* JaNein : berprft Flags und gibt Ja/Nein-Meldung aus                      *
*----------------------------------------------------------------------------*
* Eingabe : Status = Status, in dem sich das gewnschte Flag versteckt       *
*           Flag   = Bitwert des Flags                                       *
* Ausgabe : Ja oder Nein als String, jeweils 4 Zeichen breit                 *
*****************************************************************************/

char * JaNein( WORD Status, WORD Flag)
{
 if ( (Status & Flag) == Flag )
  return "Ja  ";
 else
  return "Nein";
}

/*****************************************************************************
* PrintMrciInfos: Gibt Statusinformationen ber MRCI-Server aus              *
*----------------------------------------------------------------------------*
* Eingabe : keine                                                            *
* Ausgabe : keine                                                            *
* Info    : Diese Funktion darf erst nach vorhergehendem und erfolgreichem   *
*           Aufruf von MrciQuery aufgerufen werden                           *
* Globals : mrci
*****************************************************************************/

void PrintMrciInfos()
{
  printf( "Hersteller                   : %c%c%c%c\n",
          (char) (mrci.HerstellerID & 255),
          (char) (( mrci.HerstellerID >> 8 ) & 255 ) ,
          (char) (( mrci.HerstellerID >> 16 ) & 255 ),
          (char) ( mrci.HerstellerID >> 24) );
  printf( "Hersteller-Version           : %d.%d\n",
           mrci.HerstellerVersion >> 8,
           mrci.HerstellerVersion & 255 );
  printf( "MRCI-Version                 : %d.%d\n\n",
          mrci.MrciVersion >> 8,
          mrci.MrciVersion & 255 );

  printf( "Features                       Software   Hardware\n");
  printf( "--------------------------------------------------\n");
  printf( "Standard-Komprimieren        :   %s      %s\n",
          JaNein( mrci.Flags, STANDARD_COMPRESS ),
          JaNein( mrci.HardwareFlags, STANDARD_COMPRESS ) );
  printf( "Standard-Dekomprimieren      :   %s      %s\n",
          JaNein( mrci.Flags, STANDARD_DECOMPRESS ),
          JaNein( mrci.HardwareFlags, STANDARD_DECOMPRESS ) );
  printf( "Maximal Komprimieren         :   %s      %s\n",
          JaNein( mrci.Flags, MAX_COMPRESS ),
          JaNein( mrci.HardwareFlags, MAX_COMPRESS ) );
  printf( "Inkrementelles Dekomprimieren:   %s      %s\n\n",
          JaNein( mrci.Flags, INC_DECOMPRESS ),
          JaNein( mrci.HardwareFlags, INC_DECOMPRESS ) );
};

/*----------------------------------------------------------------------------*/
/*-- Hauptprogramm -----------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
#define SBUFLEN 12196                 /* Umfang der zu komprimier}en Daten */

BYTE sourcebuf[SBUFLEN];                             /* Quell- und Zielpuffer */
BYTE destbuf[SBUFLEN];

void main( int argc, char *argv[] )
{
 BUF  sb,                                  /* beschreiben Quell- und Zielpuffer */
      db;
 int  err,                                                        /* Fehlercode */
      i;                                  /* Schleifenzhler */

 printf( "MRCIC  -  (c) 1993, 94 by Michael Tischer\n\n");
 if (!MrciQuery( &mrci ))
  {
   printf("MRCI nicht installiert\n");
   return;
  }

 /*-- MRCI ist installiert, zunchst Infos ber MRCI-Server ausgeben --------*/

 printf( "MRCI installiert\n");
 PrintMrciInfos();

 /*-- Puffer mit Daten fllen, komprimieren und dekomprimieren --------------*/
 sb.adresse = &sourcebuf;                     /* Beschreiber von Quell- und */
 sb.lenb = SBUFLEN;                           /* Zielpuffer aufbauen        */
 db.adresse = &destbuf;
 db.lenb = SBUFLEN;

 for (i = 0; i < SBUFLEN; sourcebuf[i++] = 1 );                /* Quellpuffer mit Daten fllen */

 printf("Komprimieren\n");
 printf("Lnge vorher: %d Bytes\n", SBUFLEN);
 err = Compress( STANDARD_COMPRESS, &sb, &db );
 printf ( "Fehlercode = %d\n", err );
 printf("Lnge nachher: %d Bytes\n\n", db.lenb);

 printf("Dekomprimieren\n");
 printf("Lnge vorher: %d Bytes\n", db.lenb );
 err = Decompress( &db, &sb );
 printf ( "Fehlercode = %d\n", err );
 printf ("Lnge nachher: %d Bytes\n", sb.lenb);
}

