/**********************************************************************/
/*                         N E T F I L E C . C                        */
/*--------------------------------------------------------------------*/
/*    Aufgabe       : Stellt verschiedene Prozeduren und Funktionen   */
/*                    zur Netzwerkprogrammierung unter DOS bereit     */
/*--------------------------------------------------------------------*/
/*    Speichermodell: SMALL                                           */
/*--------------------------------------------------------------------*/
/*    Autor         : Michael Tischer                                 */
/*    entwickelt am : 10.02.1992                                      */
/*    letztes Update: 13.02.1991                                      */
/*--------------------------------------------------------------------*/
/*    Microsoft C   : Wanrmeldung "Segment lost in conversation" ist  */
/*                    leider nicht zu vermeiden, strt die Kompi-     */
/*                    lierung des Programms jedoch nicht.             */
/**********************************************************************/

#include <dos.h>
#include <string.h>
#include <stdlib.h>

/*== Makros ==========================================================*/

#ifdef FP_OFF
  #undef FP_OFF
  #undef FP_SEG
#endif

#define FP_OFF(fp)      ((unsigned)(fp))
#define FP_SEG(fp)      ((unsigned)((unsigned long)(fp) >> 16))

/*== Konstanten ======================================================*/

/*-- Wahrheitswerte --------------------------------------------------*/

#define TRUE  ( 1 == 1 )
#define FALSE ( 0 == 1 )

/*-- Arten des Dateizugriffs -----------------------------------------*/

#define FM_R       0                                     /* Nur Lesen */
#define FM_W       1                                 /* Nur schreiben */
#define FM_RW      2                           /* Lesen und Schreiben */

/*-- Arten des Dateischutzes -----------------------------------------*/

#define SM_COMP 0x00   /* Kompatibilittsmodus, kein Schutz der Datei */
#define SM_RW   0x10     /* Lesen und Schreiben durch Andere verboten */
#define SM_R    0x20 /* Lesen durch Andere erlaubt, Schreiben verbot. */
#define SM_W    0x30 /* Lesen durch Andere verbot., Schreiben erlaubt */
#define SM_NO   0x40        /* Alles erlaubt, Schutz ber Record-Lock */

/*-- Mgliche Fehler bei Aufruf der Prozeduren -----------------------*/

#define NE_OK            0x00                          /* Kein Fehler */
#define NE_FileNotFound  0x02         /* Fehler: Datei nicht gefunden */
#define NE_PathNotFound  0x03          /* Fehler: Pfad nicht gefunden */
#define NE_TooManyFiles  0x04      /* Fehler: zu viele offene Dateien */
#define NE_AccessDenied  0x05 /* Fehler: Zugriff auf Datei verweigert */
#define NE_InvalidHandle 0x06         /* Fehler: Dateihandle ungltig */
#define NE_AccessCode    0x07       /* Fehler: Zugriffsart unzulssig */
#define NE_Share         0x20          /* Verletzung der Share Rechte */
#define NE_Lock          0x21   /* Fehler: (Ent)sperren eines Records */
#define NE_ShareBuffer   0x24             /* Pufferberlauf bei Share */

/*-- Funktionsnummern fr DOS-Aufruf ---------------------------------*/

#define FKT_OPEN     0x3D    /* Funktion: Datei mit Handle erffnen   */
#define FKT_CLOSE    0x3E    /* Funktion: Datei mit Handle schliessen */
#define FKT_CREATE   0x3C    /* Funktion: Datei mit Handle erzeugen   */
#define FKT_WRITE    0x40    /* Funktion: in Datei schreiben          */
#define FKT_READ     0x3F    /* Funktion: aus Datei lesen             */
#define FKT_LSEEK    0x42    /* Funktion: Dateizeiger setzen          */
#define FKT_REC_LOCK 0x5C    /* Funktion: Record-Locking              */

/*-- Funktions- u. Interrupt-Nummern sonstiger Interupt-Aufrufe ------*/

#define MULTIPLEX    0x2F                      /* Multiplex-Interrupt */
#define FKT_SHARE    0x1000              /* Test ob Share installiert */

/*-- Kennungen fr Dateien (Werte analog Turbo-Pascal) ---------------*/

#define FMCLOSED     0xD7B0                      /* Datei geschlossen */
#define FMINPUT      0xD7B1               /* Datei zum Lesen geffnet */
#define FMOUTPUT     0xD7B2           /* Datei zum Schreiben geffnet */
#define FMINOUT      0xD7B3  /* Datei zum Lesen u. Schreiben geffnet */

/*== Typdeklarationen ================================================*/

typedef struct { unsigned int Handle, RecS, Mode; } NFILE;

/*== Globale Variablen ===============================================*/

int          NetError;            /* Fehlernummer nach DOS-Interrupts */
union  REGS  regs;          /* Prozessorregister fr Interrupt-Aufruf */
struct SREGS sregs;           /* Segmentregister fr Interrupt-Aufruf */

/**********************************************************************/
/* ShareInst   : Test ob Share installiert                            */
/* Eingabe     : keine                                                */
/* Ausgabe     : TRUE wenn Share installiert                          */
/* Globale Var.: NetError/W (Fehlerstatus nach Aufruf)                */
/**********************************************************************/

int ShareInst( void )
{
 regs.x.ax = FKT_SHARE;              /* Test ob Share installiert ist */
 int86( MULTIPLEX, &regs, &regs );    /* Multiplex-Interrupt aufrufen */
 NetError = NE_OK;                                     /* Kein Fehler */
 return ( regs.h.al == 0xFF );                  /* Rckgabe auswerten */
}

/**********************************************************************/
/* NetErrorMsg : Text der Fehlermeldung                               */
/* Eingabe     : s.u.                                                 */
/* Ausgabe     : s.u.                                                 */
/**********************************************************************/

void NetErrorMsg( int  Nummer,                        /* Fehlernummer */
                  char *Text )                          /* Fehlertext */
{
 char Sdummy[ 5 ];                                    /* Fehlernummer */

 switch ( Nummer )
 {
  case NE_OK           : strcpy( Text, "Kein Fehler                 " );
                         break;
  case NE_FileNotFound : strcpy( Text, "Datei nicht gefunden        " );
                         break;
  case NE_PathNotFound : strcpy( Text, "Pfad nicht gefunden         " );
                         break;
  case NE_TooManyFiles : strcpy( Text, "Zu viele offene Dateien     " );
                         break;
  case NE_AccessDenied : strcpy( Text, "Zugriff auf Datei verweigert" );
                         break;
  case NE_InvalidHandle: strcpy( Text, "Dateihandle ungltig        " );
                         break;
  case NE_AccessCode   : strcpy( Text, "Zugriffsart unzulssig      " );
                         break;
  case NE_Share        : strcpy( Text, "Verletzung der Share Rechte " );
                         break;
  case NE_Lock         : strcpy( Text, "Fehler beim Record-Locking  " );
                         break;
  case NE_ShareBuffer  : strcpy( Text, "Pufferberlauf bei Share    " );
                         break;
  default              : {
                          itoa( Nummer, Sdummy, 2 );
                          strcpy( Text, "DOS-Fehler:               " );
                          strcat( Text, Sdummy );
                        }
 }
}

/**********************************************************************/
/* NetReset    : ffnet eine bestehende Datei                         */
/* Eingabe     : s.u.                                                 */
/* Ausgabe     : s.u                                                  */
/* Globale Var.: NetError/W (Fehlerstatus nach Aufruf)                */
/**********************************************************************/

void NetReset( char far     *DName,                      /* Dateiname */
               unsigned int Modus,                 /* Erffnungsmodus */
               unsigned int RecS,                      /* Recordgre */
               NFILE        *Datei )              /* Zeiger auf Datei */
{
 regs.x.dx = FP_OFF( DName );             /* Adresse des Dateinmamens */
 regs.h.ah = FKT_OPEN;              /* Funktionsnummer Datei erffnen */
 regs.h.al = Modus;                                 /* Dateiattribute */
 sregs.ds  = FP_SEG( DName );
 intdosx( &regs, &regs, &sregs );                 /* Interrupt-Aufruf */
 if ( !regs.x.cflag )                             /* ffnen gelungen? */
 {
  Datei->Handle = regs.x.ax;                         /* Handle merken */
  Datei->RecS = RecS;                           /* Recordgre merken */
  switch ( Modus & 0x0F )                        /* Dateimodus setzen */
  {
   case FM_R  : Datei->Mode = FMINPUT;
                break;
   case FM_W  : Datei->Mode = FMOUTPUT;
                break;
   case FM_RW : Datei->Mode = FMINOUT;
                break;
  }
  NetError = NE_OK;                         /* Kein Fehler bei Aufruf */
 }
 else
   NetError = regs.x.ax;                             /* Fehler merken */
}

/**********************************************************************/
/* NetRewrite  : Erzeugen einer Datei                                 */
/* Eingabe     : s.u.                                                 */
/* Ausgabe     : s.u                                                  */
/* Globale Var.: NetError/W (Fehlerstatus nach Aufruf)                */
/**********************************************************************/

void NetRewrite( char far     *DName,                    /* Dateiname */
                 unsigned int Modus,               /* Erffnungsmodus */
                 unsigned int RecS,                    /* Recordgre */
                 NFILE        *Datei )            /* Zeiger auf Datei */
{
 regs.x.dx = FP_OFF( DName );             /* Adresse des Dateinmamens */
 regs.h.ah = FKT_CREATE;            /* Funktionsnummer Datei erffnen */
 regs.x.cx = 0 ;                                    /* Dateiattribute */
 sregs.ds  = FP_SEG( DName );
 intdosx( &regs, &regs, &sregs );                 /* Interrupt-Aufruf */
 if ( !regs.x.cflag )                             /* ffnen gelungen? */
 {
  regs.x.bx = regs.x.ax;                     /* Handle in Register BX */
  regs.h.ah = FKT_CLOSE;          /* Funktionsnummer Datei schliessen */
  intdos( &regs, &regs );
  if ( !regs.x.cflag )                        /* Schliessen gelungen? */
    NetReset( DName, Modus, RecS, Datei );          /* Datei erffnen */
  else
    NetError = regs.x.ax;                      /* Fehlernummer merken */
 }
 else
  NetError = regs.x.ax;                        /* Fehlernummer merken */
}

/**********************************************************************/
/* NetClose  : Schliessen einer Datei                                 */
/* Eingabe   : s.u.                                                   */
/* Ausgabe   : keine                                                  */
/**********************************************************************/

void NetClose( NFILE *Datei )                /* zu schliessende Datei */
{
 if ( Datei->Mode != FMCLOSED )                       /* Datei offen? */
 {
  regs.x.bx = Datei->Handle;                  /* Dateihandle zuweisen */
  regs.h.ah = FKT_CLOSE;          /* Funktionsnummer Datei schliessen */
  intdos( &regs, &regs );
  if ( !regs.x.cflag )                        /* Schliessen gelungen? */
  {
   Datei->Handle = 0;                               /* Handle lschen */
   Datei->Mode = FMCLOSED;                       /* Datei geschlossen */
   NetError = NE_OK;                                   /* Kein Fehler */
  }
  else
    NetError = regs.x.ax;
 }
 else
  NetError = NE_InvalidHandle;                   /* Datei nicht offen */
}

/**********************************************************************/
/* Locking     : Sperren bzw. Entsperren eines Dateibereiches         */
/* Eingabe     : s.u                                                  */
/* Ausgabe     : true wenn erfolgreich                                */
/* Globale Var.: NetError/W (Fehlerstatus fr Aufruf)                 */
/* Info        : Nur interne Verwendung, Aufruf durch NetLock         */
/*               und NetUnlock                                        */
/**********************************************************************/

int Locking( int           Handle,                /* Handle der Datei */
             int           Operation,    /* Oper. Sperren, Entsperren */
             unsigned long Offset, /* Offset zum Dateianfang in Bytes */
             unsigned long Laenge )   /* Lnge des Bereiches in Bytes */
{
 regs.h.ah = FKT_REC_LOCK;    /* Funktionsnummer fr interrupt-Aufruf */
 regs.h.al = Operation;                       /* 0 = Lock, 1 = Unlock */
 regs.x.bx = Handle;                                   /* Dateihandle */
 regs.x.cx = Offset >> 16;                          /* Hi-Word Offset */
 regs.x.dx = Offset & 0xFFFF;                       /* Lo-Word Offset */
 regs.x.si = Laenge >> 16;                           /* Hi-Word Lnge */
 regs.x.di = Laenge & 0xFFFF;                        /* Lo-Word Lnge */
 intdos( &regs, &regs );                          /* Interrupt-Aufruf */
 if ( ! regs.x.cflag )                           /* Locking gelungen? */
 {
  NetError = NE_OK;
  return TRUE;                                         /* Kein Fehler */
 }
 else
 {
  NetError = regs.x.ax;                        /* Fehlernummer merken */
  return FALSE;                                             /* Fehler */
 }
}

/**********************************************************************/
/* NetUnLock   : Freigebenn von Records                               */
/* Eingabe     : s.u.                                                 */
/* Ausgabe     : true wenn erfolgreich                                */
/* Globale Var.: NetError/W (Fehlerstatus fr Aufruf)                 */
/**********************************************************************/

int NetUnLock( NFILE         *Datei,                         /* Datei */
               unsigned long RecNo,                  /* Record-Nummer */
               unsigned long Anzahl )               /* Anzahl Records */
{
 return Locking( Datei->Handle, 1, Datei->RecS * RecNo,
                 Datei->RecS * Anzahl );
}

/**********************************************************************/
/* NetLock     : Sperren von Records                                  */
/* Eingabe     : s.u.                                                 */
/* Ausgabe     : true wenn erfolgreich                                */
/* Globale Var.: NetError/W (Fehlerstatus fr Aufruf)                 */
/**********************************************************************/

int NetLock( NFILE         *Datei,                           /* Datei */
             unsigned long RecNo,                    /* Record-Nummer */
             unsigned long Anzahl )                 /* Anzahl Records */
{
 return Locking( Datei->Handle, 0, Datei->RecS * RecNo,
                 Datei->RecS * Anzahl );
}

/**********************************************************************/
/* Is_NetReadOk  : Eingabe von Datei erlaubt                          */
/* Eingabe       : s.u                                                */
/* Ausgabe       : true wenn Eingabe erlaubt                          */
/**********************************************************************/

int Is_NetReadOk( NFILE *Datei )
{
 return ( ( Datei->Mode == FMINPUT ) ||
          ( Datei->Mode == FMINOUT ) );
}

/**********************************************************************/
/* Is_NetOpen    : Datei erffnet                                     */
/* Eingabe       : s.u                                                */
/* Ausgabe       : true wenn Datei erffnet                           */
/**********************************************************************/

int Is_NetOpen( NFILE *Datei )
{
 return ( ( Datei->Mode == FMOUTPUT ) ||
          ( Datei->Mode == FMINPUT ) ||
          ( Datei->Mode == FMINOUT ) );
}

/**********************************************************************/
/* Is_NetWriteOk : Ausgabe in Datei erlaubt                           */
/* Eingabe       : s.u                                                */
/* Ausgabe       : true wenn Ausgabe erlaubt                          */
/**********************************************************************/

int Is_NetWriteOk( NFILE *Datei )
{
 return ( ( Datei->Mode == FMOUTPUT ) ||
          ( Datei->Mode == FMINOUT ) );
}

/**********************************************************************/
/* NetWrite    : In Datei schreiben                                   */
/* Eingabe     : s.u.                                                 */
/* Ausgabe     : keine                                                */
/**********************************************************************/

void NetWrite( NFILE     *Datei,                             /* Datei */
               void  far *Daten )            /* Zeiger auf Datenblock */
{
 regs.x.dx = FP_OFF( Daten );            /* Adresse des Datenbereichs */
 regs.h.ah = FKT_WRITE;            /* Funktionsnummer Datei schreiben */
 regs.x.bx = Datei->Handle;                            /* Dateihandle */
 regs.x.cx = Datei->RecS;                             /* Anzahl Bytes */
 sregs.ds  = FP_SEG( Daten );
 intdosx( &regs, &regs, &sregs );
 if ( !regs.x.cflag )
  NetError = NE_OK;                                    /* Kein Fehler */
 else
  NetError = regs.x.ax;                        /* Fehlernummer merken */
}

/**********************************************************************/
/* NetRead     : Aus Datei lesen                                      */
/* Eingabe     : s.u.                                                 */
/* Ausgabe     : keine                                                */
/**********************************************************************/

void NetRead( NFILE     *Datei,                              /* Datei */
              void  far *Daten )             /* Zeiger auf Datenblock */
{
 regs.x.dx = FP_OFF( Daten );            /* Adresse des Datenbereichs */
 regs.h.ah = FKT_READ;                 /* Funktionsnummer Datei lesen */
 regs.x.bx = Datei->Handle;                            /* Dateihandle */
 regs.x.cx = Datei->RecS;                             /* Anzahl Bytes */
 sregs.ds  = FP_SEG( Daten );
 intdosx( &regs, &regs, &sregs );
 if ( !regs.x.cflag )
  NetError = NE_OK;                                    /* Kein Fehler */
 else
  NetError = regs.x.ax;                        /* Fehlernummer merken */
}

/**********************************************************************/
/* NetSeek     : Dateizeiger setzen                                   */
/* Eingabe     : s.u                                                  */
/* Ausgabe     : keine                                                */
/**********************************************************************/

void NetSeek( NFILE         *Datei,                          /* Datei */
              unsigned long RecNo )                   /* Recordnummer */
{
 regs.h.ah = FKT_LSEEK;         /* Funktionsnummer Dateizeiger setzen */
 regs.h.al = 0;                   /* Position absolut zum Dateianfang */
 regs.x.bx = Datei->Handle;                            /* Dateihandle */
 RecNo = RecNo * Datei->RecS;                      /* Offset in Bytes */
 regs.x.cx = RecNo >> 16;                           /* Hi-Word Offset */
 regs.x.dx = RecNo & 0xFFFF;                        /* Lo-Word Offset */
 intdos( &regs, &regs );
 if ( !regs.x.cflag )
  NetError = NE_OK;                                    /* Kein Fehler */
 else
  NetError = regs.x.ax;                        /* Fehlernummer merken */
}
