/**********************************************************************/
/*                         R A W C O O K C . C                        */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : stellt zwei Funktionen zur Verfgung, mit      */
/*                     denen ein Zeichen-Gerte-Treiber in den RAW-   */
/*                     oder in den COOKED-Modus umgeschaltet werden   */
/*                     kann                                           */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 16.08.1987                                     */
/*    letztes Update : 29.10.1991                                     */
/*--------------------------------------------------------------------*/
/*    SPEICHERMODELL : SMALL                                          */
/**********************************************************************/
                                                                        
/*== Include-Dateien einbinden =======================================*/
                                                                        
#include <dos.h>
#include <stdio.h>
#include <conio.h>
                                                                        
/*== Konstanten ======================================================*/
                                                                        
#define STANDEIN 0           /* Handle 0 ist das Standardeingabegert */
#define STANDAUS 1           /* Handle 1 ist das Standardausgabegert */
                                                                        
/**********************************************************************/
/* GETMODE: liest das Attribut eines Gerte-Treibers ein              */
/* Eingabe : HANDLE = Handle des anzusprechenden Gerts               */
/* Ausgabe : das Gerte-Attribut                                      */
/**********************************************************************/
                                                                        
int GetMode( int Handle )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                                        
 Register.x.ax = 0x4400;       /* Funktionsnummer fr IOCTL: Get Mode */
 Register.x.bx = Handle;
 intdos(&Register, &Register);        /* DOS-Interrupt 21(h) aufrufen */
 return(Register.x.dx);                  /* Gerte-Attribut bergeben */
}
                                                                        
/**********************************************************************/
/* SETRAW: versetzt einen Zeichen-Treiber in den RAW-Modus            */
/* Eingabe : HANDLE = Handle des anzusprechenden Gerts               */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                                        
void SetRaw( int Handle )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                                        
 Register.x.ax = 0x4401;       /* Funktionsnummer fr IOCTL: Set Mode */
 Register.x.bx = Handle;
 Register.x.dx = GetMode(Handle) & 255 | 32; /* neues Gerte-Attribut */
 intdos(&Register, &Register);        /* DOS-Interrupt 21(h) aufrufen */
}
                                                                        
/**********************************************************************/
/* SETCOOKED: versetzt einen Zeichen-Treiber in den COOKED-Modus      */
/* Eingabe : HANDLE = Handle des anzusprechenden Gerts               */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                                        
void SetCooked( int Handle )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                                        
 Register.x.ax = 0x4401;       /* Funktionsnummer fr IOCTL: Set Mode */
 Register.x.bx = Handle;
 Register.x.dx = GetMode(Handle) & 223;      /* neues Gerte-Attribut */
 intdos(&Register, &Register);        /* DOS-Interrupt 21(h) aufrufen */
}
                                                                        
/**********************************************************************/
/* TESTAUSGABE: gibt einen Test-String 1000mal auf dem Standardaus-   */
/*              gabegert aus                                         */
/* Eingabe : keine                                                    */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                                        
void TestAusgabe( void )
{
 int i = 1000;                                 /* Anzahl der Ausgaben */
 static char Test[] = "Test.... ";           /* der auszugebende Text */
                                                                        
 printf("\n");
 for ( ; i--; )                   /* Teststring immer wieder ausgeben */
  fputs(Test, stdout); /* String auf das Standardausgabeger. ausgeben */
 printf("\n");
}
                                                                        
/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/
                                                                        
void main( void )
{
 printf("\nRAWCOOKC  -  (c) 1987, 91 by Michael Tischer\n\n");
                                                                        
 printf("Der CON-Treiber (Tastatur, Bildschirm) befindet sich" \
        "jetzt im RAW-Modus.\nBei den folgenden Ausgaben werden " \
        "deshalb Steuerzeichen wie z.B.\n<CTRL-S> nicht erkannt. " \
        "Probieren Sie das ruhig aus.\n\n" \
        "Bitte zum Start eine Taste drcken..." );
 getch();                                         /* auf Taste warten */
 SetRaw( STANDEIN );             /* Consolen-Treiber in den RAW-Modus */
 TestAusgabe();

 while ( kbhit() )          /* in der Zwischenzeit eingegebe Tasten   */
  getch();                  /* aus dem Tastaturpuffer entfernen       */

 printf("\nDer CON-Treiber befindet sich jetzt im COOKED-Modus. " \
        "Steuertasten wie z.B.\n<CTRL-S> werden bei der Ausgabe " \
        "erkannt und entsprechend beantwortet!\n" \
        "Bitte zum Start ein Taste bettigen..." );
 getch();                                         /* auf Taste warten */
 SetCooked(STANDEIN);         /* Consolen-Treiber in den COOKED-Modus */
 TestAusgabe();
}
