/**********************************************************************/
/*                              R T C C                               */
/*--------------------------------------------------------------------*/
/*    Aufgabe        : stellt zwei Funktionen zur Verfgung, mit der  */
/*                     Daten in die batteriegepufferte Echtzeituhr    */
/*                     geschrieben und ausgelesen werden knnen       */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 15.08.1987                                     */
/*    letztes Update : 17.02.1991                                     */
/*--------------------------------------------------------------------*/
/*    (MICROSOFT C)                                                   */
/*    Erstellung     : CL /AS RTCC.C                                  */
/*    Aufruf         : RTCC                                           */
/*--------------------------------------------------------------------*/
/*    (BORLAND TURBO C)                                               */
/*    Erstellung     : ber den Befehl RUN in der Befehlszeile        */
/*                     (ohne Project-Datei)                           */
/**********************************************************************/
                                                                        
/*== Include-Dateien einbinden =======================================*/
                                                                        
#include <dos.h>
#include <conio.h>
#include <stdio.h>

/*== Typedefs ========================================================*/
                                                                        
typedef unsigned char BYTE;           /* so bastelt man sich ein BYTE */
                                                                        
/*== Konstanten ======================================================*/
                                                                        
#define RTCAdrPort  0x70                   /* Adress-Register der RTC */
#define RTCDtaPort  0x71                    /* Daten-Register der RTC */
                                                                        
#define SEKUNDE      0x00 /* Adressen einiger Speicherstellen der RTC */
#define MINUTE       0x02
#define STUNDE       0x04
#define TAGDERWOCHE  0x06
#define TAG          0x07
#define MONAT        0x08
#define JAHR         0x09
#define STATUSA      0x0A
#define STATUSB      0x0B
#define STATUSC      0x0C
#define STATUSD      0x0D
#define DIAGNOSE     0x0E
#define JAHRHUNDERT  0x32
                                                                        
/**********************************************************************/
/* RTCREAD: den Inhalt einer der Speicherstellen der RTC auslesen     */
/* Eingabe : ADRESSE = die Adresse der Speicherstelle in der RTC      */
/* Ausgabe : der Inhalt dieser Speicherstelle                         */
/**********************************************************************/
                                                                        
BYTE RTCRead(BYTE Adresse)
{
 outp(RTCAdrPort, Adresse);         /* Adresse an die RTC bermitteln */
 return(inp(RTCDtaPort));   /* Inhalt auslesen und Aufrufer bergeben */
}
                                                                        
/**********************************************************************/
/* RTCDT: liest eine der Speicherstellen des Datums oder der Zeit aus */
/*        und setzt das Ergebnis in einen Binr-Wert um, falls die    */
/*        Uhr mit dem BCD-Format arbeitet                             */
/* Eingabe : ADRESSE = die Adresse der Speicherstelle in der RTC      */
/* Ausgabe : der Inhalt dieser Speicherstelle als Binr-Wert          */
/* Info    : liegt die Adresse auerhalb des erlaubten Bereichs       */
/*           (0 bis 63) wird der Wert -1 zurckgeliefert              */
/**********************************************************************/
                                                                        
BYTE RTCDt(BYTE Adresse)
                                                                        
{
 if (RTCRead(STATUSB) & 2)                  /* BCD- oder Binr-Modus? */
  return((RTCRead(Adresse) >> 4) * 10 + (RTCRead(Adresse) & 15));
 else return(RTCRead(Adresse));                    /* ist Binr-Modus */
}
                                                                        
/**********************************************************************/
/* RTCWRITE: einen Wert in einer der Speicherst. der RTC schreiben    */
/* Eingabe : ADRESSE = die Adresse der Speicherstelle in der RTC      */
/*           INHALT  = der neue Inhalt dieser Speicherstelle          */
/* Ausgabe : keine                                                    */
/* Info    : die Adresse mu im Bereich 0 bis 63 liegen               */
/**********************************************************************/
                                                                        
void RTCWrite(BYTE Adresse, BYTE Inhalt)
                                                                        
{
 outp(RTCAdrPort, Adresse);         /* Adresse an die RTC bermitteln */
 outp(RTCDtaPort, Inhalt);                    /* neuen Wert schreiben */
}
                                                                        
/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/
                                                                        
void main()
                                                                        
{
 static char *WochenTag[] =               /* die Namen der Wochentage */
                           {
                            "Sonntag", "Montag", "Dienstag", "Mittwoch",
                            "Donnerstag", "Freitag", "Samstag"
                           };
                                                                        
 printf("\nRTCC (c) 1987, 91 by Michael Tischer\n\n");
 printf("Informationen aus der batteriegepufferten Echtzeituhr\n");
 printf("=====================================================\n\n");
 if (!(RTCRead(DIAGNOSE) & 128))            /* ist die Batterie o.k.? */
  {                                          /* die Batterie ist o.k. */
   printf("- die Uhr wird im %d Stunden-Modus betrieben\n",
          (RTCRead(STATUSB) & 2)*6+12);
   printf("- die Uhrzeit: %2d:%02d:%02d\n",
          RTCDt(STUNDE), RTCDt(MINUTE), RTCDt(SEKUNDE));
   printf("- das Datum: ");
   printf("%s, der %d.%02d.%d%d\n", WochenTag[RTCDt(TAGDERWOCHE)-1],
          RTCDt(TAG), RTCDt(MONAT), RTCDt(JAHRHUNDERT), RTCDt(JAHR));
  }
 else printf("       ACHTUNG! Die Batterie der Uhr ist leer\n");
}
