/***********************************************************************
*                           S 3 2 4 0 C . C                            *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Arbeit mit Sprites im VGA-      *
*                     320*400-Grafikmodus bei 256 Farben und zwei      *
*                     Bildschirmseiten.                                *
*                     Das Programm arbeitet dabei mit den Assembler-   *
*                     Routinen aus den Moduln S3240CA.ASM und          *
*                     V3240CA.ASM zusammen.                            *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    :  9.09.1990                                       *
*  letztes Update am: 14.01.1991                                       *
**--------------------------------------------------------------------**
*  Speichermodell   : SMALL                                            *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung       : CL /AS s3240c.c v3240ca s3240ca                  *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung       : ber eine Projekt-Datei mit folgendem Inhalt:    *
*                      s3240c.c                                        *
*                      v3240ca.obj                                     *
*                      s3240ca.obj                                     *
**--------------------------------------------------------------------**
*  Aufruf           : s3240c                                           *
***********************************************************************/

#include <dos.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <conio.h>

/*-- Compiler-abhngige Deklarationen --------------------------------*/

#ifdef __TURBOC__
  #include <alloc.h>
#else
  #include <malloc.h>
  #define random(x) ( rand() % (x+1) )             /* Zufallsfunktion */
#endif

/*-- Typdeklarationen ------------------------------------------------*/

typedef unsigned char BYTE;
typedef BYTE BOOL;

typedef struct {        /* Pixel-Puffer fr GetVideo() und PutVideo() */
                BYTE *bitptr[4],             /* Zeiger auf Bit-Planes */
                     byproz[4],        /* Anzahl zu kopierender Bytes */
                     hoehe;                      /* Anzahl der Zeilen */
                /*-- hier folgen die Bytes aus den Bit-Planes --------*/
               } PIXBUF;
typedef PIXBUF *PIXPTR;              /* Zeiger auf einen Pixel-Puffer */

typedef struct {                    /* Erscheinungsbild eines Sprites */
                BYTE   breite,                       /* Breite gesamt */
                       hoehe;                  /* Hhe in Punktzeilen */
                PIXPTR pixbp;               /* Zeiger auf Pixel-Block */
               } SPLOOK;

typedef struct {                           /* Sprite-Beschreiber (ID) */
                SPLOOK *splookp;       /* Zeiger auf Erscheinungsbild */
                int    x[2], y[2];    /* Koordinaten in Seite 0 und 1 */
                PIXPTR hgptr[2];     /* Zeiger auf Hintergrund-Puffer */
               } SPID;

/*-- externe Referenzen auf die Assembler-Routinen -------------------*/

extern void init320400( void );
extern void setpix( int x, int y, unsigned char farbe);
extern BYTE getpix( int x, int y );
extern void setpage( BYTE page );
extern void showpage( BYTE page );
extern void far * getfontptr( void );

extern void copybuf2plane( BYTE *bufptr, BYTE page,
                           int nachx, int nachy, BYTE breite,
                           BYTE hoehe, BOOL bg );
extern void copyplane2buf( BYTE *bufptr, BYTE page,
                           int vonx, int vony, BYTE breite,
                           BYTE hoehe );

/*-- Konstanten ------------------------------------------------------*/

#define TRUE  ( 0 == 0 )
#define FALSE ( 0 == 1 )

#define MAXX 319                        /* Maximale X- und Y-Ordinate */
#define MAXY 399

#define OUT_LEFT   1      /* fr Kollisionsmarkierung in SpriteMove() */
#define OUT_TOP    2
#define OUT_RIGHT  4
#define OUT_BOTTOM 8
#define OUT_NO     0                             /* nirgendwo drauen */

#define ALLOKBUF ((PIXPTR) 0)        /* GetVideo(): Puffer allokieren */

/***********************************************************************
*  IsVga: Stellt fest, ob eine VGA-Karte installiert ist.              *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : 0, wenn keine VGA angetroffen wurde, sonst ungleich 0     *
***********************************************************************/

BYTE IsVga( void )
{
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */

 Regs.x.ax = 0x1a00;              /* Funktion 1Ah gibt es nur bei VGA */
 int86( 0x10, &Regs, &Regs );
 return (BYTE) ( Regs.h.al == 0x1a );  /* ist die Funktion verfgbar? */
}

/***********************************************************************
*  Line: Zieht eine Linie nach dem Bresenham-Algorithmus               *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endkoordinate                                    *
*            FARBE  = Farbe der Linie                                  *
***********************************************************************/

/*-- Unterfunktion, vertauscht zwei Integer-Variablen ----------------*/

void SwapInt( int *i1, int *i2 )
{
 int dummy;

 dummy = *i2;
 *i2   = *i1;
 *i1   = dummy;
}

/*-- Hauptteil der Funktion ------------------------------------------*/

void Line( int x1, int y1, int x2, int y2, BYTE farbe )
{
 int d, dx, dy,
     aincr, bincr,
     xincr, yincr,
     x, y;

 if ( abs(x2-x1) < abs(y2-y1) )       /* ber X- oder Y-Achse laufen? */
  {                                               /* ber die Y-Achse */
   if ( y1 > y2 )                                    /* y1 grer y2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   xincr = ( x2 > x1 ) ?  1 : -1;       /* Spalteninkrement festlegen */

   dy = y2 - y1;
   dx = abs( x2-x1 );
   d  = 2 * dx - dy;
   aincr = 2 * (dx - dy);
   bincr = 2 * dx;
   x = x1;
   y = y1;

   setpix( x, y, farbe );                      /* ersten Punkt setzen */
   for (y=y1+1; y<= y2; ++y )         /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       x += xincr;
       d += aincr;
      }
     else
      d += bincr;
     setpix(x, y, farbe);
    }
  }
 else                                             /* ber die X-Achse */
  {
   if ( x1 > x2 )                                    /* x1 grer x2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   yincr = ( y2 > y1 ) ? 1 : -1;         /* Zeileninkrement festlegen */

   dx = x2 - x1;
   dy = abs( y2-y1 );
   d  = 2 * dy - dx;
   aincr = 2 * (dy - dx);
   bincr = 2 * dy;
   x = x1;
   y = y1;

   setpix(x, y, farbe);                        /* ersten Punkt setzen */
   for (x=x1+1; x<=x2; ++x )          /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       y += yincr;
       d += aincr;
      }
     else
      d += bincr;
     setpix(x, y, farbe);
    }
  }
}

/***********************************************************************
*  PrintChar : Baut ein Zeichen auerhalb des sichtbaren Bereichs des  *
*              Video-RAMs auf.                                         *
**--------------------------------------------------------------------**
*  Eingabe :   zeichen = das aufzubauende Zeichen                      *
*              x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************/

void PrintChar( char zeichen, int x, int y, BYTE vg, BYTE hg )
{
 typedef BYTE ZDEF[256][8];                 /* Aufbau des Zeichensatz */
 typedef ZDEF far *ZPTR;                    /* Zeiger auf Zeichensatz */

 BYTE         i, k,                                /* Schleifenzhler */
              Maske;              /* Bitmaske fr Aufbau des Zeichens */

 static ZPTR fptr = (ZPTR) 0;        /* Zeiger auf Zeichensatz im ROM */

 if ( fptr == (ZPTR) 0 )   /* Zeiger auf Zeichensatz schon ermittelt? */
  fptr = getfontptr();  /*Nein, mit Hilfe der Assembler-Funktion laden*/

 /*- das Zeichen Punkt fr Punkt aufbauen ----------------------------*/

 if ( hg == 255 )                    /* Zeichen transparent zeichnen? */
  for ( i = 0; i < 8; ++i )      /* Ja, nur Vordergrund-Punkte setzen */
   {
    Maske = (*fptr)[zeichen][i];    /* Bitmuster fr eine Zeile holen */
    for ( k = 0; k < 8; ++k, Maske <<= 1 ) /* die Spalten durchlaufen */
     if ( Maske & 128 )                              /* Punkt setzen? */
      setpix( x+k, y+i, vg );                                   /* Ja */
   }
 else                                     /* Nein, jeden Punkt setzen */
  for ( i = 0; i < 8; ++i )                     /* Zeilen durchlaufen */
  {
   Maske = (*fptr)[zeichen][i];     /* Bitmuster fr eine Zeile holen */
   for ( k = 0; k < 8; ++k, Maske <<= 1 )  /* die Spalten durchlaufen */
    setpix( x+k, y+i, ( Maske & 128 ) ? vg : hg );
  }
}


/***********************************************************************
*  GrafPrintf: Gibt einen formatierten String im Grafikbildschirm aus  *
*  Eingabe : X, Y   = Startkoordinate (0 - ...)                        *
*            VG     = Vordergrundfarbe                                 *
*            HG     = Hintergrundfarbe (255 = transparent)             *
*            STRING = String mit Formatinformationen                   *
*            ...    = Ausdrcke wie bei Printf                         *
***********************************************************************/

void GrafPrintf( int x, int y, BYTE vg, BYTE hg, char * string, ... )
{
 va_list parameter;              /* Parameter-Liste fr VA_... Macros */
 char ausgabe[255],                 /* Puffer fr formatierten String */
      *cp;

 va_start( parameter, string );                /* Parameter umwandeln */
 vsprintf( ausgabe, string, parameter );               /* formatieren */
 for ( cp = ausgabe; *cp; ++cp, x+= 8 )    /* formatierten String     */
   PrintChar( *cp, x, y, vg, hg );         /* ber PrintChar ausgeben */
}

/***********************************************************************
*  GetVideo: Holt den Inhalt eines rechteckigen Bereichs aus dem       *
*            Video-RAM in einen Puffer                                 *
**--------------------------------------------------------------------**
*  Eingabe : PAGE   = Bildschirmseite (0 oder 1)                       *
*            X1, Y1 = Startkoordinate                                  *
*            BREITE = Breite des rechteckigen Bereichs in Punkten      *
*            HOEHE  = Hhe des rechteckigen Bereichs in Punkten        *
*            BUFPTR = Zeiger auf Pixel-Puffer, in dem die Informa-     *
*                     tionen abgelegt werden sollen                    *
*  Ausgabe : Zeiger auf den angelegten Pixel-Puffer mit dem Inhalt     *
*            des angegebenen Bereichs                                  *
*  Info    : Wird fr den Parameter BUFPTR der Wert ALLOKBUF bergeben,*
*            wird ein neuer Pixel-Puffer ber den Heap allokiert und   *
*            zurckgeliefert. Dieser Puffer kann bei einem neuer-      *
*            lichen Aufruf wieder angegeben werden, sofern der bis-    *
*            herige Inhalt nicht mehr bentigt wird und die Gre des  *
*            rechteckigen Bereichs gegenber dem vorhergehenden Aufruf *
*            unverndert bleibt.                                       *
***********************************************************************/

PIXPTR GetVideo( BYTE page, int x1, int y1, BYTE breite, BYTE hoehe,
                 PIXPTR bufptr )
{
 BYTE i,                                          /* Schleifenzhnler */
      aktplane,                      /* aktuell bearbeitete Bit-Plane */
      sb,                            /* Bit-Plane der Startkoordinate */
      eb,                                /* Bit-Plane der }koordinate */
      b,                            /* Byte-Anzahl in einer Bit-Plane */
      am;      /* Anzahl Bytes in der Mitte der beiden Vierer-Gruppen */
 BYTE *rptr;             /* Zeiger fr Pos. einer Bit-Plane im Puffer */

 if ( bufptr == ALLOKBUF )      /* kein Puffer beim Aufruf bergeben? */
  bufptr = malloc( sizeof( PIXBUF ) + breite*hoehe ); /* Nein, allok. */

 /*-- die Anzahl der Bytes pro Bit-Plane berechnen -------------------*/

 am = (BYTE) ( ( (x1+breite-1) & ~3 ) -  /* Anzahl Bytes in der Mitte */
                ( (x1+4) & ~3) ) >> 2;
 sb = (BYTE) (x1 % 4);                             /* Start-Bit-Plane */
 eb = (BYTE) ((x1+breite-1) % 4);                    /* End-Bit-Plane */

 rptr = (BYTE *) bufptr + sizeof( PIXBUF );

 /*-- die vier Bit-Planes durchlaufen --------------------------------*/

 for ( i=0; i<4; ++i )
  {
   aktplane = (sb+i) % 4;
   b = am;                      /* Grundanzahl der zu kopier}en Bytes */
   if ( aktplane >= sb )               /* auch im Start-Vierer-Block? */
    ++b;                     /* Ja, in dieser Bit-Plane ein Byte mehr */
   if ( aktplane <= eb )                 /* auch im End-Vierer-Block? */
    ++b;                     /* Ja, in dieser Bit-Plane ein Byte mehr */
   bufptr->bitptr[i] = rptr;    /* Zeiger auf Start im Puffer ablegen */
   bufptr->byproz[i] = b;                 /* Anzahl im Puffer ablegen */
   copyplane2buf( rptr, page, x1+i,                /* Inhalt der Bit- */
                  y1, b, hoehe );                  /* Plane holen     */
   rptr += (b * hoehe);                    /* Zeiger auf nchste Bit- */
  };                                       /* Plane im Puffer setzen  */

 bufptr->hoehe = hoehe;                                /* Hhe merken */

 return bufptr;        /* Zeiger auf Puffer an Aufrufer zurckliefern */
}

/***********************************************************************
*  PutVideo: Schreibt den Inhalt eines zuvor ber GetVideo gesicherten *
*            rechteckigen Bildschirmbereichs wieder in den Video-RAM   *
**--------------------------------------------------------------------**
*  Eingabe : BUFPTR = Beim vorherigen Aufruf von Getvideo zurck-      *
*                     gelieferter Zeiger auf den Pixel-Puffer          *
*            PAGE   = Bildschirmseite (0 oder 1)                       *
*            X1, Y1 = Startkoordinate                                  *
*            BG     = Sollen Hintergrundpunkte (Farbcode 255) nicht    *
*                     in den Video-RAM geschrieben werden?             *
*  Info    : Der Pixel-Puffer wird durch diese Prozedur nicht gelscht,*
*            dafr mu die Funktion FreePixBuf() aufgerufen werden.    *
***********************************************************************/

void PutVideo( PIXPTR bufptr, BYTE page, int x1, int y1, BOOL bg )
{
 BYTE aktplane,                     /* aktuell durchlaufene Bit-Plane */
      hoehe;

 hoehe = bufptr->hoehe;                          /* Hhe des Bereichs */
 for ( aktplane=0; aktplane<4; ++aktplane )   /* 4 Bit-Planes durchl. */
  copybuf2plane( bufptr->bitptr[aktplane], page, x1+aktplane,
                 y1, bufptr->byproz[aktplane], hoehe, bg );
}

/***********************************************************************
*  FreePixBuf: Lscht einen Pixel-Puffer, der bei einem vorherigen     *
*              Aufruf von GetVideo() ber den Heap allokiert wurde     *
**--------------------------------------------------------------------**
*  Eingabe : BUFPTR = Beim vorherigen Aufruf von Getvideo zurck-      *
*                     gelieferter Zeiger auf den Pixel-Puffer          *
*            BREITE = Breite des rechteckigen Bereichs in Punkten      *
*            HOEHE  = Hhe des rechteckigen Bereichs in Punkten        *
***********************************************************************/

void FreePixBuf( PIXPTR bufptr )
{
 free( bufptr );
}

/***********************************************************************
*  CreateSprite: Erzeugt ein Sprite auf der Basis einer bereits kompi- *
*                lierten Punktmusters                                  *
**--------------------------------------------------------------------**
*  Eingabe : SPLOOKP = Zeiger auf die von CompileSpriteI() erzeugte    *
*                      Datenstruktur                                   *
*  Ausgabe : Zeiger auf die erstellte Sprite-Struktur                  *
*  Info    : Fr die Ablege des Sprite-Hintergrundes werden zwei ne-   *
*            beneinanderliegende Flchen von der Gre des jeweiligen  *
*            Sprites bentigt.                                         *
***********************************************************************/

SPID *CreateSprite( SPLOOK *splookp )
{
 SPID *spidp;                  /* Zeiger auf erzeugte Sprite-Struktur */

 spidp = (SPID *) malloc( sizeof(SPID) );   /* Sprite-Struktur allok. */
 spidp->splookp = splookp;                    /* Daten in die Sprite- */

 /*- zwei Hintergrund-Puffer erzeugen, indem ein entsprechend groer -*/
 /*- Bereich aus dem Video-RAM ber GetVideo gesichert wird          -*/

 spidp->hgptr[0] = GetVideo( 0, 0, 0, splookp->breite,
                             splookp->hoehe, ALLOKBUF );
 spidp->hgptr[1] = GetVideo( 0, 0, 0, splookp->breite,
                             splookp->hoehe, ALLOKBUF );
 return spidp;            /* Zeiger auf Sprite-Struktur zurckliefern */
}

/***********************************************************************
*  CompileSprite: Erzeugt das Punkt- und Bitmusters eines Sprites,     *
*                 auf der Basis einer zur Laufzeit angegebenen Sprite- *
*                 definition                                           *
**--------------------------------------------------------------------**
*  Eingabe : BUFP    = Zeiger auf Array mit Zeigern auf Strings, die   *
*                      das Zeichenmuster des Sprites enthalten         *
*            HOEHE   = Hoehe des Sprites und damit Anzahl der Strings  *
*            SEITE   = Grafik-Seite zum Aufbau des Sprites             *
*            FB      = ASCII-Zeichen, das fr die kleinste Farbe steht *
*            VGFARBE = erste Farbcode fr FB                           *
*  Info    : Die Sprites werden in der angegebenen Punktzeile ab dem   *
*            linken Zeilenrand angelegt                                *
***********************************************************************/

SPLOOK *CompileSprite( char **bufp, BYTE hoehe, BYTE seite,
                       char fb, BYTE vgfarbe )
{
 BYTE   breite,                                 /* Breite der Strings */
        c,                    /* nimmt Zeichen aus C-Sprite-Array auf */
        i, k;                                   /* Schleifenvariablen */
 SPLOOK *splookp;              /* Zeiger auf erzeugte Sprite-Struktur */
 PIXPTR pbptr;               /* nimmt Sprite-Hintergrund temporr auf */

 /*-- Sprite-Look-Struktur erzeugen und mit Daten fllen -------------*/

 splookp = (SPLOOK *) malloc( sizeof(SPLOOK) );
 breite = (BYTE) strlen( *bufp );
 splookp->breite = breite;
 splookp->hoehe = hoehe;

 /*-- Sprite in der angegebenen Seite ab (0/0) aufbauen --------------*/

 setpage( seite );                    /* Seite zum Zeichnen festlegen */
 showpage( seite );
 pbptr = GetVideo( seite, 0, 0, breite, hoehe, ALLOKBUF );/*Hgr. holen*/

 for (i = 0; i < hoehe; ++i )               /* die Zeilen durchlaufen */
  for ( k = 0; k < breite; ++k )           /* die Spalten durchlaufen */
   {
    c = *(*(bufp+i)+k);
    setpix( k, i, (BYTE) (c == ' ' ? 255 : vgfarbe+(c-fb)));
   }

 /*-- Sprite in Puffer holen und Hintergrund restaurieren ------------*/

 splookp->pixbp = GetVideo( seite, 0, 0, breite, hoehe, ALLOKBUF );
 PutVideo( pbptr, seite, 0, 0, FALSE );
 FreePixBuf( pbptr );                      /* Puffer wieder freigeben */

 return splookp;        /* Zeiger auf den Sprite-Puffer zurckliefern */
}

/***********************************************************************
*  PrintSprite : Sprite in einer bestimmten Seite ausgeben             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in der das Sprite gezeichnet werden soll   *
*                    (0 oder 1)                                        *
***********************************************************************/

void PrintSprite( register SPID *spidp, BYTE seite )
{
 PutVideo( spidp->splookp->pixbp,
            seite, spidp->x[seite], spidp->y[seite], TRUE );
}

/***********************************************************************
*  GetSpriteBg: Hintergrund eines Sprites holen und an der dafr vor-  *
*               gesehenen Position ablegen                             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, aus der der Hintergrund geholt werden soll *
*                    (0 oder 1)                                        *
***********************************************************************/

void GetSpriteBg( register SPID *spidp, BYTE seite )
{
 GetVideo( seite, spidp->x[seite],  spidp->y[seite],
           spidp->splookp->breite, spidp->splookp->hoehe,
           spidp->hgptr[seite] );
}

/***********************************************************************
*  RestoreSpriteBg: Zuvor gesicherten Hintergrund eines Sprites wieder *
*                   in die ursprngliche Grafikseite kopieren          *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in die der Hinntergrund zurckkopiert      *
*                    werden soll (0 oder 1)                            *
***********************************************************************/

void RestoreSpriteBg( register SPID *spidp, BYTE seite )
{
 PutVideo( spidp->hgptr[seite], seite,
           spidp->x[seite], spidp->y[seite], FALSE );
}

/***********************************************************************
*  MoveSprite: Sprite innerhlicherten Hintergrund eines Sprites wieder *
*                   in die ursprngliche Grafikseite kopieren          *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP  = Zeiger auf die Sprite-Struktur                   *
*            SEITE  = Seite, in die der Hintergrund zurckkopiert      *
*                     werden soll (0 oder 1)                           *
*            DELTAX = Bewegungszhler in X- und Y-Richtung             *
*            DELTAY                                                    *
*  Ausgabe : Kollisionsmarkierung, siehe Konstanten OUT_...            *
***********************************************************************/

BYTE MoveSprite( SPID *spidp, BYTE seite, int deltax, int deltay )
{
 int  neux, neuy;                           /* neue Sprite-Koordinate */
 BYTE out;                             /* Kollision mit Rand anzeigen */

 /*-- X-Koordinate verschieben und Kollision mit Rand testen ---------*/

 if ( ( neux = spidp->x[seite] + deltax ) < 0 )
  {
   neux = 0 - deltax - spidp->x[seite];
   out = OUT_LEFT;
  }
 else
  if ( neux > MAXX - spidp->splookp->breite )
   {
    neux = (2*(MAXX+1))-neux-2*(spidp->splookp->breite);
    out = OUT_RIGHT;
   }
  else
    out = OUT_NO;

 /*-- Y-Koordinate verschieben und Kollision mit Rand testen ---------*/

 if ( ( neuy = spidp->y[seite] + deltay ) < 0 )       /* oberer Rand? */
  {                                    /* Ja, deltay mu negativ sein */
   neuy = 0 - deltay - spidp->y[seite];
   out |= OUT_TOP;
  }
 else
  if ( neuy + spidp->splookp->hoehe > MAXY+1  )      /* unterer Rand? */
   {                                   /* Ja, deltay mu positiv sein */
    neuy = (2*(MAXY+1))-neuy-2*(spidp->splookp->hoehe);
    out |= OUT_BOTTOM;
   }

 /*-- neue Position nur setzen, wenn gegenber der alten verndert ---*/

 if ( neux != spidp->x[seite]  ||  neuy != spidp->y[seite] )
  {                                              /* ist neue Position */
   RestoreSpriteBg( spidp, seite );       /* Hintergrund zurcksetzen */
   spidp->x[seite] = neux;                 /* neue Koordinaten merken */
   spidp->y[seite] = neuy;
   GetSpriteBg( spidp, seite );            /* neuen Hintergrund holen */
   PrintSprite( spidp, seite );  /* Sprite in angegeb. Seite zeichnen */
  }
 return out;
}

/***********************************************************************
*  SetSprite: Sprite an eine bestimmte Position versetzen              *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            x0, y0 = Sprite-Koordinate fr die Seite 0                *
*            x1, y1 = Sprite-Koordinate fr die Seite 1                *
*  Info    : Der Aufruf dieser Funktion sollte dem ersten Aufruf von   *
*            MoveSprite() vorausgehen                                  *
***********************************************************************/

void SetSprite( SPID *spidp, int x0, int y0, int x1, int y1 )
{
 spidp->x[0] = x0;           /* Koordinaten in Sprite-Struktur merken */
 spidp->x[1] = x1;
 spidp->y[0] = y0;
 spidp->y[1] = y1;

 GetSpriteBg( spidp, 0 );              /* Sprite-Hintergrund in Seite */
 GetSpriteBg( spidp, 1 );              /* 0 und 1 holen               */
 PrintSprite( spidp, 0 );                        /* Sprite in Seite 0 */
 PrintSprite( spidp, 1 );                        /* und 1 zeichnen    */
}

/***********************************************************************
*  RemoveSprite: Entfernt ein Sprite von seiner augenblicklichen       *
*                Position und macht es dadurch unsichtbar              *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*  Info    : Nach dem Aufruf dieser Funktion mu erst wieder die Funk- *
*            tion SetSprite() auferufen werden, bevor das Sprite ber  *
*            MoveSprite() bewegt werden kann                           *
***********************************************************************/

void RemoveSprite( SPID *spidp )
{
 RestoreSpriteBg( spidp, 0 );          /* Sprite-Hintergrund in Seite */
 RestoreSpriteBg( spidp, 1 );          /* 0 und 1 wieder zurcksetzen */
}

/***********************************************************************
*  Demo: Demonstriert den Einsatz der verschiedenen Funktionen aus     *
*        diesem Modul                                                  *
***********************************************************************/

void Demo( void )
{
 static char *GleiterRauf[20] =
                { "               AA               ",
                  "              AAAA              ",
                  "              AAAA              ",
                  "               AA               ",
                  "             GGBBGG             ",
                  "            GBBCCBBG            ",
                  "           GBBBCCBBBG           ",
                  "          GBBBBBBBBBBG          ",
                  "          GBBBBBBBBBBG          ",
                  " G       GBBBBBBBBBBBBG       G ",
                  "GCG     GGDBBBBBBBBBBDGG     GCG",
                  "GCG   GGBBBDBBB  BBBDBBBGG   GCG",
                  "GCBGGGBBBBBDBB    BBDBBBBBGGGBCG",
                  "GCBBBBBBBBBBDB    BDBBBBBBBBBBCG",
                  "BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB",
                  "GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ",
                  "  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ",
                  "    GGBBDDDDDGGGGGDDDDDDBBG     ",
                  "      GDDDDGGG    GGGDDDDG      ",
                  "       DDDD          DDDD       "  };


 static char *GleiterRunter[20] =
                {
                  "       DDDD          DDDD       ",
                  "      GDDDDGGG    GGGDDDDG      ",
                  "    GGBBDDDDDGGGGGDDDDDDBBG     ",
                  "  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ",
                  "GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ",
                  "BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB",
                  "GCBBBBBBBBBBDB    BDBBBBBBBBBBCG",
                  "GCBGGGBBBBBDBB    BBDBBBBBGGGBCG",
                  "GCG   GGBBBDBBB  BBBDBBBGG   GCG",
                  "GCG     GGDBBBBBBBBBBDGG     GCG",
                  " G       GBBBBBBBBBBBBG       G ",
                  "          GBBBBBBBBBBG          ",
                  "          GBBBBBBBBBBG          ",
                  "           GBBBCCBBBG           ",
                  "            GBBCCBBG            ",
                  "             GGBBGG             ",
                  "               AA               ",
                  "              AAAA              ",
                  "              AAAA              ",
                  "               AA               " };

 #define SPRANZ 6                               /* Anzahl der Sprites */
 #define BREITE 38         /* Breite der Copyright-Meldung in Zeichen */
 #define HOEHE  6                                   /* Hhe in Zeilen */
 #define SX     (MAXX-(BREITE*8)) / 2              /* Startordinate X */
 #define SY     (MAXY-(HOEHE*8)) / 2               /* Startordinate Y */

 struct {                       /* beschreibt die verwalteten Sprites */
          SPID *spidp;                        /* Zeiger auf Sprite-Id */
          int  deltax[2],          /* X-Verschiebung fr Page 0 und 1 */
               deltay[2];          /* Y-Verschiebung fr Page 0 und 1 */
        } sprites[ SPRANZ ];
 BYTE   page,                            /* aktuell bearbeitete Seite */
        out;                   /* nimmt Flags fr Seitenkollision auf */
 int    x, y, i,                                   /* Schleifenzhler */
        dx, dy;                                     /* Bewegungswerte */
 char   lc;
 SPLOOK *gleiterupp, *gleiterdnp;           /* Zeiger auf die Sprites */

 srand( *(int far *) 0x0040006cl );     /* Zufallsgen. initialisieren */

 /*-- Vorlagen fr die verschiedenen Sprites aufbauen ----------------*/

 gleiterupp = CompileSprite( GleiterRauf,   20, 0, 'A', 1 );
 gleiterdnp = CompileSprite( GleiterRunter, 20, 40, 'A', 1 );

 /*-- die ersten beiden Grafikseiten mit Buchstaben fllen -----------*/

 for ( page = 0; page < 2; ++ page )
  {
   setpage( page );
   for ( lc = 0, y = 0; y < (MAXY+1)-8; y += 12 )
     for ( x = 0; x < (MAXX+1)-8; x += 8 )
       GrafPrintf( x, y, lc % 255, 255, "%c", lc++ & 127 );

   /*-- Copyright-Meldung ausgeben -----------------------------------*/

   Line( SX-1, SY-1, SX+BREITE*8, SY-1, 15 );
   Line( SX+BREITE*8, SY-1, SX+BREITE*8, SY+HOEHE*8,15 );
   Line( SX+BREITE*8, SY+HOEHE*8, SX-1, SY+HOEHE*8, 15 );
   Line( SX-1, SY+HOEHE*8, SX-1, SY-1, 15 );
   GrafPrintf( SX, SY,    15, 4,
               "                                      "  );
   GrafPrintf( SX, SY+8,  15, 4,
               " S3240C.C (c) 1990 by Michael Tischer " );
   GrafPrintf( SX, SY+16, 15, 4,
               "                                      "  );
   GrafPrintf( SX, SY+24, 15, 4,
               "  Sprite-Demo fuer den 320*400 Punkte "  );
   GrafPrintf( SX, SY+32, 15, 4,
               "  Grafikmodus von VGA-Karten          "  );
   GrafPrintf( SX, SY+40, 15, 4,
               "                                      "  );
  }

 /*-- die verschiedenen Sprites erzeugen -----------------------------*/

 for ( i = 0; i < SPRANZ ; ++ i)
  {
   sprites[ i ].spidp = CreateSprite( gleiterupp );
   do                            /* Bewegungswerte der Sprites whlen */
    {
     dx = 0;
     dy = random(8) - 4;
    }
   while ( dx==0  &&  dy==0 );

   sprites[ i ].deltax[0] = sprites[ i ].deltax[1] = dx * 2;
   sprites[ i ].deltay[0] = sprites[ i ].deltay[1] = dy * 2;

   x = ( 320 / SPRANZ * i ) + (320 / SPRANZ - 40) / 2 ;
   y = random( (MAXY+1) - 40 );
   SetSprite( sprites[ i ].spidp, x, y, x - dx, y - dy );
  }

 /*-- die Sprites bewegen und an den Seitenrndern reflektieren ------*/

 page = 1;                                      /* mit Page 1 starten */
 while ( !kbhit() )          /* ein Tastenderuck beendet die Schleife */
  {
   showpage( (BYTE) (1 - page) );        /* die andere Seite anzeigen */

   for ( i = 0; i < SPRANZ; ++ i)          /* die Sprites durchlaufen */
    {                /* Sprite bewegen und Kontakt mit Seite abfragen */
     out = MoveSprite( sprites[i].spidp, page, sprites[i].deltax[page],
                       sprites[i].deltay[page] );
     if ( out & OUT_TOP  ||  out & OUT_BOTTOM )  /*Kontakt oben/unten?*/
      {  /* Ja, Bewegungsrichtung umdrehen und neues Erscheinungsbild */
       sprites[i].deltay[page] = 0 - sprites[i].deltay[page];
       sprites[i].spidp->splookp = ( out & OUT_TOP ) ? gleiterdnp
                                                     : gleiterupp;
      }
     if ( out & OUT_LEFT  ||  out & OUT_RIGHT )
      sprites[i].deltax[page] = 0 - sprites[i].deltax[page];
    }
   page = (page+1) & 1;       /* von 1 auf 0 und umgekehrt umschalten */
  }
}

/*--------------------------------------------------------------------*/
/*-- H A U P T P R O G R A M M                                      --*/
/*--------------------------------------------------------------------*/

void main( void )
{
 union REGS regs;

 if ( IsVga() )                             /* VGA-Karte installiert? */
  {                                                  /* Ja, los gehts */
   init320400();                        /* Grafikmodus initialisieren */
   Demo();
   getch();                                       /* auf Taste warten */
   regs.x.ax = 0x0003;          /* wieder in Textmodus zurckschalten */
   int86( 0x10, &regs, &regs );
  }
 else
  printf( "S3240C.C - (c) 1990 by MICHAEL TISCHER\n\nAchtung! Fr "\
          "die Ausfhrung dieses Programms wird eine VGA-Karte "\
          "bentigt.\n\n" );
}

