;**********************************************************************;
;*                         S 6 4 3 5 C A . A S M                      *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : enthlt die Routinen zur Arbeit mit Sprites im *;
;*                     640*350 Punkte Grafikmodus der EGA- und VGA-   *;
;*                     Karte                                          *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  :  8.12.1990                                     *;
;*    letztes Update : 14.01.1991                                     *;
;*--------------------------------------------------------------------*;
;*    Speichermodell : SMALL                                          *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM /mx S6435CA;    oder   TASM -mx S6435CA   *;
;*                     ... dann mit S6435C.C linken                   *;
;**********************************************************************;

IGROUP group _text                ;Zusammenfassung der Programm-Segmente
DGROUP group _bss,  _data         ;Zusammenfassung der Daten-Segmente
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP

_BSS   segment word public 'BSS'  ;dieses Segment nimmt alle nicht ini-
_BSS   ends                       ;tialisierten statischen Variablen auf

_DATA  segment word public 'DATA' ;alle initialisierten globalen und
                                  ;statischen Variablen werden in diesem
                                  ;Segment untergebracht

_DATA  ends

;== Konstanten =========================================================

SC_INDEX       = 3c4h             ;Indexregister des Sequencer-Ctrl.
SC_MAP_MASK    = 2                ;Nummer des Map-Mask-Registers
SC_MEM_MODE    = 4                ;Nummer des Memory-Mode-Registers

GC_INDEX       = 3ceh             ;Indexregister des Graphics-Ctrl.
GC_READ_MAP    = 4                ;Nummer des Read-Map-Registers
GC_BIT_MASK    = 8                ;Nummer des Bit-Mask-Registers

PIXX           = 640              ;horizontale Auflsung

;== Programm ===========================================================

_TEXT  segment byte public 'CODE' ;das Programmsegment

;-- Public-Deklarationen -----------------------------------------------

public     _copybuf2video
public     _mergeandcopybuf2video
public     _copyvideo2buf

;-----------------------------------------------------------------------
;-- CopyBuf2Video: kopiert den Inhalt eines rechteckigen Bereichs aus
;--                dem Video-RAM, der zuvor mit Hilfe von CopyVideo2Buf
;--                gesichert wurde wieder in den Video-RAM
;-- Deklaration: CopyBuf2Plane( byte *bufptr,
;--                             byte nachpage,
;--                             int  nachx,
;--                             int  nachy,
;--                             byte breite,
;--                             byte hoehe );

_copybuf2video proc near

sfr0    struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
stofs0     dw ?                   ;lok. Var.: Startoffset in Video-RAM
ret_adr0   dw ?                   ;Rcksprungadresse zum Aufrufer
bufptr0    dw ?                   ;Zeiger auf Puffer
nachpage   dw ?                   ;nach Seite
nachx      dw ?                   ;nach X-Ordinate
nachy      dw ?                   ;nach Y-Ordinate
breite0    dw ?                   ;Breite
hoehe0     dw ?                   ;Hhe des Bereichs
sfr0    ends                   ;Ende der Struktur

fr         equ [ bp - bp0 ]          ;adr. die Elemente der Struktur
bfr        equ byte ptr [ bp - bp0 ] ;adr. Stack-Element als Byte

	   sub   sp,2             ;Platz fr lokale Variablen

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  si               ;SI und DI sichern
	   push  di

	   ;-- Segmentadresse fr Zugriff auf Video-RAM berechnen ------

	   mov   ax,0A000h        ;ES auf Anfang NACH-Seite
	   cmp   bfr.nachpage,0   ;ist es Seite 0?
	   je    cv0              ;Ja, AL schon o.k

	   mov   ax,0A6D6h        ;Nein, Seite 1 bei A6D6h

cv0:       mov   es,ax

	   ;-- Offset fr Zielposition in Page berechnen ---------------

	   mov   ax,PIXX / 8      ;AX auf Zielposition VON
	   mul   fr.nachy
	   mov   bx,fr.nachx
	   shr   bx,1
	   shr   bx,1
	   shr   bx,1
	   add   bx,ax
	   mov   fr.stofs0,bx     ;Startoffset in lokaler Var. merken

	   mov   si,fr.bufptr0    ;DS:SI auf den Puffer setzen

	   ;-- Zhler fr die Kopierschleife laden ---------------------

	   mov   dl,bfr.breite0   ;DL = Bytes
	   mov   bx,PIXX / 8      ;BX als Offset zur nchsten Zeile
	   sub   bl,dl
	   xor   ch,ch            ;Hi-Byte des Counters ist immer 0

	   ;-- die zu adressierende Bit-Plane einstellen ---------------

	   mov   ah,1             ;Plane-Nummer als Bit-Maske laden
	   mov   al,SC_MAP_MASK   ;die Registernummer nach AL

cv1:       mov   dx,SC_INDEX      ;Zugriff auf die zu bearbeitende
	   out   dx,ax            ;Plane erschlieen

	   ;-- Kopierroutine fr eine Bit-Plane, ohne Bercksichti- ----
	   ;-- gung des Hintergrundes

	   mov   di,fr.stofs0     ;DI auf Startoffset
	   mov   dh,bfr.hoehe0    ;DH = Zeilen
	   mov   dl,bfr.breite0   ;DL = Bytes

cv2:       mov   cl,dl            ;Anzahl Bytes nach CL

	   rep movsb              ;Zeile kopieren
	   add   di,bx            ;DI in nchste Zeile
	   dec   dh               ;noch eine Zeile?
	   jne   cv2              ;Ja ---> weiter

	   shl   ah,1             ;auf die nchste Plane umschalten
	   test  ah,16            ;alle Planes bearbeitet?
	   je    cv1              ;Nein, mit nchster Plane fortfahren

	   mov   ax,(0Fh shl 8)+ SC_MAP_MASK   ;Zugriff auf alle Bit-
	   mov   dx,SC_INDEX                   ;Planes erlauben
	   out   dx,ax

	   pop   di               ;DI und SI zurckholen
	   pop   si
	   pop   bp               ;BP zurckholen

	   add   sp,2             ;lokale Variablen entfernen
	   ret                    ;zurck zum Aufrufer

_copybuf2video endp

;-----------------------------------------------------------------------
;-- MergeAndCopyBuf2Video: verknpft den Inhalt eines Hintergrundpuffers
;--                        mit dem Inhalt eines Sprite-Puffers ber eine
;--                        Bit-Maske und kopiert das Resultat in den
;--                        Video-RAM
;-- Deklaration : MergeAndCopyBuf2Video( void * spribufptr,
;--                                      void * hgbufptr,
;--                                      void * andbufptr,
;--                                      BYTE page,
;--                                      int  nachx,
;--                                      int  nachy,
;--                                      BYTE breite,
;--                                      BYTE hoehe );
;-- Info         : siehe CopyVideo2Buf

_mergeandcopybuf2video proc near

sfr2    struc                  ;Struktur zum Zugriff auf den Stack
bp2        dw ?                   ;nimmt BP auf
andptr2    dw ?                   ;lok. Var.: Zeiger in AND-Puffer
stofs2     dw ?                   ;lok. Var.: Startoffset in Video-RAM
ret_adr2   dw ?                   ;Rcksprungadresse zum Aufrufer
spribufptr dw ?                   ;Zeiger auf den Spritepuffer
hgbufptr   dw ?                   ;Zeiger auf Hintergrundpuffer
andbufptr  dw ?                   ;Zeiger auf AND-Puffer
nachpage2  dw ?                   ;nach Seite
nachx2     dw ?                   ;nach X-Ordinate
nachy2     dw ?                   ;nach Y-Ordinate
breite2    dw ?                   ;Breite
hoehe2     dw ?                   ;Hhe des Bereichs
sfr2    ends                   ;Ende der Struktur

fr         equ [ bp - bp2 ]          ;adr. die Elemente der Struktur
bfr        equ byte ptr [ bp - bp2 ] ;adr. Stack-Element als Byte

	   sub   sp,4             ;Platz fr lokale Variablen

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  ds               ;DS merken
	   push  si               ;SI und DI sichern
	   push  di

	   ;-- Segmentadresse fr Zugriff auf Video-RAM berechnen ------

	   mov   ax,0A000h        ;ES auf Anfang NACH-Seite
	   cmp   bfr.nachpage2,0  ;ist es Seite 0?
	   je    cm0              ;Ja, AL schon o.k

	   mov   ax,0A6D6h        ;Nein, Seite 1 bei A6D6h

cm0:       mov   es,ax

	   ;-- Offset fr Zielposition in Page berechnen ---------------

	   mov   ax,PIXX / 8      ;AX auf Zielposition VON
	   mul   fr.nachy2
	   mov   bx,fr.nachx2
	   shr   bx,1
	   shr   bx,1
	   shr   bx,1
	   add   bx,ax
	   mov   fr.stofs2,bx     ;Startoffset in lokaler Var. merken

	   ;-- Zhler fr die Kopierschleife laden ---------------------

	   mov   dl,bfr.breite2   ;DL = Bytes
	   mov   bx,PIXX / 8      ;BX als Offset zur nchsten Zeile
	   sub   bl,dl
	   xor   ch,ch            ;Hi-Byte des Counters ist immer 0

	   ;-- die zu adressierende Bit-Plane einstellen ---------------

	   mov   ah,1             ;Plane-Nummer als Bit-Maske laden

cm1:       mov   al,SC_MAP_MASK   ;die Registernummer nach AL
	   mov   dx,SC_INDEX      ;Zugriff auf die zu bearbeitende
	   out   dx,ax            ;Plane erschlieen

	   ;-- Kopierroutine fr eine Bit-Plane, ohne Bercksichti- ----
	   ;-- gung des Hintergrundes

	   mov   dx,fr.andbufptr  ;Offsettadresse. des AND-Zeigers
	   mov   fr.andptr2,dx    ;in lokale Variable kopieren
	   mov   di,fr.stofs2     ;DI auf Startoffset
	   mov   dh,bfr.hoehe2    ;DH = Zeilen
	   mov   dl,bfr.breite2   ;DL = Bytes

cm2:       mov   cl,dl            ;Anzahl Bytes nach CL

cm3:       mov   si,fr.hgbufptr   ;Zeiger auf Hintrgr.-Puffer laden
	   lodsb                  ;Byte aus Hintergrund-Puffer laden
	   mov   fr.hgbufptr,si   ;inkrementierten Offset abspeichern

	   mov   si,fr.andptr2    ;Zeiger in AND-Puffer laden
	   and   al,[si]          ;Hintergrund mit AND-Maske verkn.
	   inc   si               ;Offset in AND-Puffer inkrementieren
	   mov   fr.andptr2,si    ;und speichern

	   mov   si,fr.spribufptr ;Zeiger auf Sprite-Puffer laden
	   or    al,[si]          ;Byte aus Sprite-Puffer ORen
	   inc   si               ;Offset in Sprite-Puffer inkrementieren
	   mov   fr.spribufptr,si ;und speichern

	   stosb                  ;Byte in Video-RAM schreiben
	   loop  cm3              ;nchstes Byte bearbeiten

	   add   di,bx            ;DI in nchste Zeile
	   dec   dh               ;noch eine Zeile?
	   jne   cm2              ;Ja ---> weiter

	   shl   ah,1             ;auf die nchste Plane umschalten
	   test  ah,16            ;alle Planes bearbeitet?
	   je    cm1              ;Nein, mit nchster Plane fortfahren

	   mov   ax,(0Fh shl 8)+ SC_MAP_MASK   ;Zugriff auf alle Bit-
	   mov   dx,SC_INDEX                   ;Planes erlauben
	   out   dx,ax

	   pop   di               ;DI und SI zurckholen
	   pop   si
	   pop   ds               ;DS zurckholen
	   pop   bp

	   add   sp,4             ;lokale Variablen entfernen
	   ret                    ;zurck zum Aufrufer

_mergeandcopybuf2video endp

;-----------------------------------------------------------------------
;-- CopyVideo2Buf: Kopiert ein rechtecikgen Teil aus dem Video-RAM in
;--                einen Puffer
;-- Deklaration : CopyPlane2Buf( byte *bufptr,
;--                              byte vonpage,
;--                              int  vonx,
;--                              int  vony,
;--                              byte breite,
;--                              byte hoehe );
;-- Info         : In dieser Version der Routine mu der zu kopierende
;--                Bereich an einer durch acht teilbaren Punktspalte
;--                beginnen und sich ber ein vielfaches von acht Punk-
;--                ten erstrecken.
;--                BREITE ist hier die Anzahl der Bytes pro Zeile
;--                einer Bit-Plane!

_copyvideo2buf proc near

sfr1    struc                  ;Struktur zum Zugriff auf den Stack
bp1        dw ?                   ;nimmt BP auf
stofs1     dw ?                   ;lok. Var.: Startoffset in Video-RAM
ret_adr1   dw ?                   ;Rcksprungadresse zum Aufrufer
bufptr1    dw ?                   ;Zeiger auf Puffer
vonpage    dw ?                   ;von Seite
vonx       dw ?                   ;von X-Ordinate
vony       dw ?                   ;von Y-Ordinate
breite1    dw ?                   ;Breite des Bereichs in Punkten
hoehe1     dw ?                   ;Hhe des Bereichs in Punktzeilen
sfr1    ends                   ;Ende der Struktur

fr         equ [ bp - bp1 ]          ;adr. die Elemente der Struktur
bfr        equ byte ptr [ bp - bp1 ] ;adr. Stack-Element als Byte

	   sub   sp,2             ;Platz fr lokale Variablen

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  ds               ;DS merken
	   push  si               ;SI und DI sichern
	   push  di

	   push  ds               ;ES fr Zugriff auf Puffer mit
	   pop   es               ;DS laden

	   ;-- Segmentadresse fr Zugriff auf Video-RAM berechnen ------

	   mov   ax,0A000h        ;ES auf Anfang VON-Seite
	   cmp   bfr.vonpage,0    ;ist es Seite 0?
	   je    cc0              ;Ja, AL schon o.k

	   mov   ax,0A6D6h        ;Nein, Seite 1 bei A6D6h

cc0:       mov   ds,ax

	   ;-- Offset in die auszulesende Seite bilden -----------------

	   mov   ax,PIXX / 8      ;AX auf Zielposition VON
	   mul   fr.vony
	   mov   bx,fr.vonx
	   shr   bx,1
	   shr   bx,1
	   shr   bx,1
	   add   bx,ax
	   mov   fr.stofs1,bx     ;Startoffset in lokaler Var. merken

	   mov   di,fr.bufptr1    ;ES:DI auf den Puffer setzen

	   ;-- Zhler fr die Kopierschleife laden ---------------------

	   mov   dl,bfr.breite1   ;DL = Bytes
	   mov   bx,PIXX / 8      ;BX als Offset zur nchsten Zeile
	   sub   bl,dl
	   xor   ch,ch            ;Hi-Byte des Counters ist immer 0

	   ;-- die zu adressierende Bit-Plane einstellen ---------------

	   xor   ah,ah            ;mit Plane #0 beginnen
	   mov   al,GC_READ_MAP   ;nach AL die Registernummer

cc1:       mov   dx,GC_INDEX      ;Indexadresse des Graphic-Ctrl laden
	   out   dx,ax            ;Read-Map-Register laden

	   ;-- Kopierroutine fr eine Bit-Plane, ohne Bercksichti- ----
	   ;-- gung des Hintergrundes

	   mov   dh,bfr.hoehe1    ;DH = Zeilen
	   mov   dl,bfr.breite1   ;DL = Bytes
	   mov   si,fr.stofs1     ;Startoffset nach SI laden

cc2:       mov   cl,dl            ;Anzahl Bytes nach CL

	   rep movsb              ;Zeile kopieren
	   add   si,bx            ;SI in nchste Zeile
	   dec   dh               ;noch eine Zeile?
	   jne   cc2              ;Ja, weiter

	   inc   ah               ;auf die nchste Plane umschalten
	   cmp   ah,4             ;alle Planes bearbeitet?
	   jne   cc1              ;Nein, mit nchster Plane fortfahren

	   pop   di               ;DI und SI zurckholen
	   pop   si
	   pop   ds               ;DS zurckholen

	   pop   bp

	   add   sp,2             ;lokale Variablen entfernen
	   ret                    ;zurck zum Aufrufer

_copyvideo2buf endp

;== Ende ===============================================================

_text    ends                     ;Ende des Programm-Segments
         end                      ;Ende des Assembler-Source


