/**********************************************************************/
/*                     S E R I R Q  . C                               */
/**------------------------------------------------------------------**/
/*  Aufgabe        : Demonstriert mittels Chat-Programm die           */
/*                   Programierung des seriellen Schnittstellen IRQ's */
/**------------------------------------------------------------------**/
/*  Autor          : Michael Tischer / Bruno Jennrich                 */
/*  entwickelt am  : 08.04.1994                                       */
/*  letztes Update : 18.04.1994                                       */
/**------------------------------------------------------------------**/
/* Compiler-Hinweis: Stackberprfung ausschalten!                    */
/*                   Zeigerprfung ausschalten!                       */
/*                   Optimierungen ALLE ausschalten!                  */
/* COMPILER: Borland C++ 3.1, Microsoft Visual C++ 1.5                */
/**********************************************************************/
#include <dos.h>
#include <ctype.h>
#include <process.h>
#include <stdio.h>
#include <conio.h>

#include "types.h"
#include "win.h"
#include "serutil.h"
#include "irqutil.h"
#include "args.h"

/*- Folgende Zeilen innnerhalb eines Projektes ausdokumentieren ------*/
#include "args.c"
#include "win.c"
#include "serutil.c"
#include "irqutil.c"

/*- Globale Variablen ------------------------------------------------*/
WINDOW Screen;
WINDOW Remote;
WINDOW Local;
WINDOW Status;

INT iSerPort;
INT iSerIRQ;

/**********************************************************************/
/* GetSer : Interruptroutine, ausgelst durch serielle Schnittstelle  */
/**------------------------------------------------------------------**/
/* Info : In dieser Funktion werden Daten von der seriellen Schnitt-  */
/*        stelle entgegengenommen und im "Remote"-Fenster dar-        */
/*        gestellt. Diese Funktion enthlt auerdem das Gerst fr    */
/*        alle weiteren Schnittstellenereignisse, die einen IRQ aus-  */
/*        lsen.                                                      */
/**********************************************************************/
VOID __interrupt _FP GetSer( VOID )
{ static BYTE bIRQID;
  static CHAR c[16 + 1];
  static int i;

  bIRQID = ( BYTE )inp( iSerPort + SER_IRQ_ID );
  if( ! ( bIRQID & SER_ID_PENDING ) )               /* steht IRQ an ? */
  {
    switch( bIRQID & SER_ID_MASK )
    {
      case SER_ID_RECEIVED:                      /* Nach Datenempfamg */
        i = 0;
        while( ser_IsDataAvaiable( iSerPort ) && ( i < 16 ) )
        {
          c[ i++ ] = ( BYTE )inp( iSerPort + SER_RXBUFFER );
          c[ i ] = '\0';
        }
        win_Print( &Remote, c );
      break;
      case SER_ID_SENT:                          /* Nach Datensendung */
      break;
      case SER_ID_MODEMSTATUS:  /* Nach nderung der Leitungszustnde */
      break;
    }
  }
  irq_SendEOI( iSerIRQ );                         /* End Of Interrupt */
}

/**********************************************************************/
/* H A U P T P R O G R A M M                                          */
/**********************************************************************/
VOID main( INT argc, PCHAR argv[] )
{
  static CHAR c[2];
  static INT iCom;
  static LONG lBaud;
  VOID (_interrupt _FP *lpOldIRQ)();
  PBYTE pSaved;                              /* geretteter Bildschirm */
  INT iUART;

  if( FindString( argv, "?", argc ) )
  {
    printf("Aufruf:\n" );
    printf("SERIRQ [-COM:comport] [-BAUD:baudrate]\n");
    printf("comport  = 1 oder 2    (Default: 1 )\n");
    printf("baudrate = 50 - 115200 (Default: 9600)\n");
    exit(0);
  }

  if( GetArg( argc, argv, "-COM:", _int, &iCom, 1 ) )
  {
    if( iCom == 1 )
    {
      iSerPort = SER_COM1;   /* Nur COM1 und COM2 sind "standardisiert" */
      iSerIRQ  = SER_IRQ_COM1;
    }
    else
    if( iCom == 2 )
    {
      iSerPort = SER_COM2;   /* Nur COM1 und COM2 sind "standardisiert" */
      iSerIRQ  = SER_IRQ_COM2;
    }
    else
    {
      printf("Nicht untersttzter Com-Port!\n");
      exit(0);
    }
  }
  else
  {
    iSerPort = SER_COM1;   /* Nur COM1 und COM2 sind "standardisiert" */
    iSerIRQ  = SER_IRQ_COM1;
  }

  if( !GetArg( argc, argv, "-BAUD:", _long, &lBaud, 1 ) )
      lBaud = 9600L;              /* Maximale Baudrate bei UART 8450A */
  if( lBaud > SER_MAXBAUD )
  {
    printf("Baudrate zu hoch!\n");
    printf("Maximum: %ld Bd\n", SER_MAXBAUD );
  }

  iUART = ser_Init( iSerPort, lBaud,
                  SER_LCR_8BITS | SER_LCR_1STOPBIT | SER_LCR_NOPARITY );


  if( iUART == NOSER )
  {
    printf("Keine Schnittstelle!\n");
    exit( 0 );
  }
  if( iUART > INS8250 ) ser_FIFOLevel( iSerPort, SER_FIFO_TRIGGER14 );

  win_GetScreenSettings( &Screen );              /* Bildschirm retten */
  pSaved = win_Save( &Screen );
                                        /* Ausgabe Fenster einrichten */
  win_Init( &Remote, 0, 0, 80, 9, 0x17, 0x1f, WIN_SCROLL | WIN_CRLF );
  win_Init( &Local,  0,10, 80, 9, 0x17, 0x1f, WIN_SCROLL | WIN_CRLF |
                                        WIN_ACTIVE | WIN_HASCURSOR );
  win_Init( &Status,  0,20, 80, 2, 0x17, 0x1f, WIN_SCROLL | WIN_CRLF );

  win_Clr( &Screen );
  win_Clr( &Remote );
  win_Clr( &Local );
  win_Clr( &Status );

  win_printf( &Status, "COM-Port: 0x%04X, IRQ: 0x%02X, Baud: %ld\n",
              iSerPort, iSerIRQ, lBaud );
  win_printf( &Status, "<ESC> = Ende        ");
  win_printf( &Status, "Aufruf mit '?' zeigt Parameter          ");
  win_printf( &Status, "(c) BHJ, MITI");

  lpOldIRQ = ser_SetIRQHandler( iSerPort,      /* seriellen Interrupt */
                                iSerIRQ,       /* installieren.       */
                                GetSer,
                                SER_IER_RECEIVED | SER_IER_SENT );

  c[0] = 0;
  c[1] = 0;

  win_GotoXY( &Local, 0, 0);
  while( c[0] != 27 )
  {
    c[0] = ( BYTE ) _getch();
    if( c[0] )
    {           /* eigegebenes Zeichen ber Schnittstelle ausgeben... */
      ser_WriteByte( iSerPort, c[0], 0x8000, 0, 0 );
                                 /* ...und in Fenster protokollieren. */
      if( isprint( c[ 0 ] ) ) win_Print( &Local, c );
      else win_printf( &Local, "<%02X>", ( UINT )c[ 0 ] );
    }
    else _getch();
  }
                                          /* Alten Handler einrichten */
  ser_RestoreIRQHandler( iSerPort, iSerIRQ, lpOldIRQ );
  win_Restore( pSaved, TRUE );       /* Bildschirminhalt restaurieren */
  if( iUART > INS8250 ) ser_FIFOLevel( iSerPort, 0 );
}

