;**********************************************************************;
;*                            S O U N D C A                           *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : stellt eine Funktion zur Einbidung in C zur    *;
;*                     Verfgung, mit der die Tne der Oktaven 3, 4   *;
;*                     und 5 gespielt werden knnen                   *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  : 15.08.87                                       *;
;*    letztes Update : 16.02.91                                       *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM -mx SOUNDCA;                              *;
;*                     ... dann mit SOUNDC.C verbinden                *;
;**********************************************************************;
                                                                        
IGROUP group _text                ;Zusammenfassung der Programm-Segmente
DGROUP group _bss,  _data         ;Zusammenfassung der Daten-Segmente
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP
                                                                        
       public _Sound              ;Funktion wird fr andere Programme
                                  ;zugnglich
                                                                        
_BSS   segment word public 'BSS'  ;dieses Segment nimmt alle nicht ini-
_BSS   ends                       ;tialisierten statischen Variablen auf
                                                                        
_DATA  segment word public 'DATA' ;alle initialisierten globalen und
                                  ;statischen Variablen werden in diesem
                                  ;Segment untergebracht
                                                                        
time_alt  dw (?),(?)              ;Adresse des alten Timer-Interrupts
s_zaehler db (?)                  ;verbleibende Dauer eines Tones in
                                  ;1/18 Sekunden Schritten
                                                                        
s_ende    db (?)                  ;zeigt an, ob Ton bereits gespielt ist
toene     dw 9121,8609,8126,7670  ;Ton-Werte fr die Oktave 3
          dw 7239,6833,6449,6087
          dw 5746,5423,5119,4831
          dw 4560,4304,4063,3834  ;Ton-Werte fr die Oktave 4
          dw 3619,3416,3224,3043
          dw 2873,2711,2559,2415
          dw 2280,2152,2031,1917  ;Ton-Werte fr die Oktave 5
          dw 1809,1715,1612,1521
          dw 1436,1355,1292,1207
                                                                        
_DATA  ends
                                                                        
;== Programm ===========================================================
                                                                        
_TEXT  segment byte public 'CODE' ;das Programmsegment
                                                                        
;-- SOUND: einen Ton spielen -------------------------------------------
;-- Aufruf von C: Sound((int) Ton, (int) Dauer);
;-- Ausgabe     : keine
;-- Info        : Ton ist die Nummer des Tones relativ zum C der dritten
;--               Oktave
;--               Dauer ist die Tondauer in 1/18-Sekunden-Schritten
                                                                        
_Sound    proc near
                                                                        
          push bp                 ;BP auf dem Stack merken
          mov  bp,sp              ;SP nach BP bertragen
                                                                        
          ;-- Timer Interrupt auf eigen Routine verbiegen --------------

          mov  word ptr cs:setds+1,ds ;DS fr neuen Timer-Int. speichern
          mov  ax,351ch           ;Adresse des Timer-Interrupts holen
          int  21h                ;DOS-Interrupt aufrufen
          mov  time_alt,bx        ;Offsetadresse des alten Interrupts
          mov  time_alt+2,es      ;und Segmentadresse merken

          mov  word ptr cs:stjump+1,bx ;fr neuen Timer-Interrupt
          mov  word ptr cs:stjump+3,es ;speichern
          mov  bx,ds              ;DS in BX merken
          push cs                 ;Stack auf den Stack bringen
          pop  ds                 ;und als DS zurckholen
          mov  dx,offset sound_ti ;Offsetadresse der neuen Timer-Routine
          mov  ax,251ch           ;neue Timer-Routine setzen
          int  21h                ;DOS-Interrupt aufrufen
          mov  ds,bx              ;DS wieder restaurieren
                                                                        
          mov  al,182             ;Erzeugung eines Tones vorbereiten
          out  43h,al             ;Wert an Timer-Command-Register senden
                                                                        
          mov  bx,[bp+4]          ;Ton holen
          xor  bh,bh              ;BH fr Adressierung der Ton-Tab. = 0
          shl  bx,1               ;Ton-Nr. verdoppeln (da Word-Tabelle)
          mov  ax,[toene+bx]      ;Ton-Wert holen
          out  42h,al             ;LO-Byte an Timer-Counter-Register
          mov  al,ah              ;HI-Byte an AL bertragen
          out  42h,al             ;und auch an Timer-Counter-Register
          in   al,61h             ;Lautsprecher-Kontroll-Bit einlesen
          or   al,11b             ;untersten beiden Bits schalten Ls. an
          mov  s_ende,1           ;Ton mu erst noch gespielt werden
          mov  dl,[bp+6]          ;Ton-Dauer holen
          mov  s_zaehler,dl       ;und speichern
          out  61h,al             ;Lautsprecher anschalten
                                                                        
play:     cmp  s_ende,0           ;Ton beendet?
          jne  play               ;NEIN --> noch warten
                                                                        
          in   al,61h             ;Lautsprecher-Kontroll-Bit einlesen
          and  al,11111100b       ;untersten beiden Bits lschen
          out  61h,al             ;Lautsprecher ausschalten
                                                                        
          ;-- alten Timer-Interrupt wieder aktivieren ------------------

          mov cx,ds               ;DS merken
          mov ax,251ch            ;Fkt.nr. fr Interrupt-Vektor setzen
          lds dx,dword ptr time_alt ;alte Adresse nach DS:DX laden
          int 21h                 ;DOS-Interrupt aufrufen
          mov ds,cx               ;DS zurckholen
                                                                        
          mov  sp,bp              ;Stackpointer wieder restaurieren
          pop  bp                 ;BP vom Stack zurckholen
          ret                     ;zurck zum Aufrufer
                                                                        
_Sound    endp
                                                                        
;-- neuer Timer Interrupt ----------------------------------------------
                                                                        
sound_ti  proc far                ;wird 18mal in der Sekunde aufgerufen
                                                                        
          push ax                 ;AX und DS auf dem Stack merken
          push ds
setds:    mov  ax,0000h           ;hier mu das C-DS eingetragen werden
          mov  ds,ax
          dec  s_zaehler          ;Zeit-Zhler dekrementieren
          jne  st_ende            ;wenn noch nicht 0 dann Ende
          mov  s_ende,0           ;Notendauer vergangen signalisieren
st_ende:  pop  ds                 ;altes DS wieder setzen
          pop  ax                 ;auch AX wieder vom Stack holen
                                                                        
stjump:   db   0EAh,0,0,0,0       ;FAR-JUMP zum alten Timer-Interrupt
                                                                        
sound_ti  endp
                                                                        
;== Ende ===============================================================
                                                                        
_text    ends                     ;Ende des Programm-Segments
         end                      ;Ende des Assembler-Source
