/**********************************************************************/
/*                             T A S T C                              */
/*--------------------------------------------------------------------*/
/*    Aufgabe : stellt eine Funktion zur Verfgung, mit deren Hilfe   */
/*              ein Zeichen von der Tastatur eingelesen werden kann,  */
/*              wobei der Status der Umschaltmodi INSERT, CAPS und    */
/*              NUM auf dem Bildschirm angezeigt wird.                */
/*--------------------------------------------------------------------*/
/*    Autor          : MICHAEL TISCHER                                */
/*    entwickelt am  : 13.08.1987                                     */
/*    letztes Update :  1.01.1992                                     */
/*--------------------------------------------------------------------*/
/*    Speichermodell : SMALL                                          */
/**********************************************************************/
                                                                        
/*== Include-Dateien einbinden =======================================*/
                                                                        
#include <dos.h>
#include <bios.h>
#include <stdio.h>
                                                                        
/*== Typedefs ========================================================*/
                                                                        
typedef unsigned char BYTE;               /* wir basteln uns ein BYTE */
                                                                        
/*== Makros ==========================================================*/

#ifdef __TURBOC__                         /* Definitionen fr TURBO C */

   #define GetKbKey()        ( bioskey( 0 ) )
   #define GetKbReady()      ( bioskey( 1 ) != 0 )
   #define GetKbStatus()     ( bioskey( 2 ) )

#else                        /* Definitionen fr Microsoft C-Compiler */

   #define GetKbKey()        ( _bios_keybrd( _KEYBRD_READ ) )
   #define GetKbReady()      ( _bios_keybrd( _KEYBRD_READY ) != 0 )
   #define GetKbStatus()     ( _bios_keybrd( _KEYBRD_SHIFTSTATUS ) )

#endif

/*== Konstanten ======================================================*/
                                                                        
/*-- Bit-Belegung innerhalb des BIOS-Tastatur-Status -----------------*/
                                                                        
#define SCRL  16                                   /* Scroll-Lock-Bit */
#define NUML  32                                      /* Num-Lock-Bit */
#define CAPL  64                                     /* Caps-Lock-Bit */
#define INS  128                                        /* Insert-Bit */
                                                                        
#define TRUE  ( 0 == 0 )         /* Konstanten, erleichtern das Lesen */
#define FALSE ( 0 == 1 )         /* des Programmtextes                */
                                                                        
#define FZ      0        /* Zeile, in der die Flags ausgegeben werden */
#define FS      65      /* Spalte, in der die Flags ausgegeben werden */
#define FlagFarbe 0x70           /* schwarze Schrift auf weiem Grund */
                                                                        
/*-- Codes einiger Tasten, wie sie GETKEY() liefert ------------------*/
#define BEL     7                         /* Code des Klingelzeichen  */
#define BS      8                         /* Code der Backspace-Taste */
#define TAB     9                         /* Code der Tabulator-Taste */
#define LF      10                        /* Code des Linefeed        */
#define CR      13                        /* Code der Return-Taste    */
#define ESC     27                        /* Code der Escape-Taste    */
#define F1      315                       /* Funktionstasten          */
#define F2      316
#define F3      317
#define F4      318
#define F5      319
#define F6      320
#define F7      321
#define F8      322
#define F9      323
#define F10     324
#define CUP     328                       /* Cursor-Tasten            */
#define CLEFT   331
#define CRIGHT  333
#define CDOWN   328
                                                                        
/*== globale Variablen ===============================================*/
                                                                        
BYTE Insert,                               /* Status des INSERT-Flags */
     Num,                                     /* Status des NUM-Flags */
     Caps;                                   /* Status des CAPS-Flags */
                                                                        
/**********************************************************************/
/* GETSEITE: holt die aktuelle Bildschirmseite                        */
/* Eingabe : keine                                                    */
/* Ausgabe : s.u.                                                     */
/**********************************************************************/
                                                                        
BYTE GetSeite( void )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                                        
 Register.h.ah = 15;                               /* Funktionsnummer */
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
 return(Register.h.bh);               /* Nummer der aktuellen B.Seite */
}
                                                                        
/**********************************************************************/
/* SETPOS: setzt die Position des Cursors in der aktuellen Bs.seite   */
/* Eingabe : SPALTE = neue Cursorspalte                               */
/*           ZEILE  = neue Cursorzeile                                */
/* Ausgabe : keine                                                    */
/* Info    : die Position des blinkenden Bildschirmcursors ndert     */
/*           sich durch den Aufruf dieser Funktion nur dann, wenn die */
/*           angegebene Bildschirmseite die aktuelle Bildschirmseite  */
/*           ist                                                      */
/**********************************************************************/
                                                                        
void SetPos(BYTE Spalte, BYTE Zeile)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                                        
 Register.h.ah = 2;                                /* Funktionsnummer */
 Register.h.bh = GetSeite();                       /* Bildschirmseite */
 Register.h.dh = Zeile;                            /* Bildschirmzeile */
 Register.h.dl = Spalte;                          /* Bildschirmspalte */
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
}
                                                                        
/**********************************************************************/
/* GETPOS: holt die Position des Cursors in der aktuellen Bs.seite    */
/* Eingabe : keine                                                    */
/* Ausgabe : - SPALTE = Pointer auf Var. die die akt. Spalte aufnimmt */
/*             ZEILE  = Pointer auf Var. die die akt. Zeile aufnimmt  */
/**********************************************************************/
                                                                        
void GetPos(BYTE * Spalte, BYTE * Zeile)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                                        
 Register.h.ah = 3;                                /* Funktionsnummer */
 Register.h.bh = GetSeite();                       /* Bildschirmseite */
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
 *Spalte = Register.h.dl;                  /* Ergebnisse der Funktion */
 *Zeile = Register.h.dh;                   /* aus den Registern lesen */
}
                                                                        
/**********************************************************************/
/* WRITECHAR: schreibt ein Zeichen mit einem vorgegebenen Attribut an */
/*            die aktuelle Cursorposition in der aktuellen Bs.seite   */
/* Eingabe : - ZEICHEN = ASCII-Code des auszugebenden Zeichen         */
/*           - FARBE   = Attribut des Zeichen                         */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                                        
void WriteChar(char Zeichen, BYTE Farbe)
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                                        
 Register.h.ah = 9;                                /* Funktionsnummer */
 Register.h.al = Zeichen;                 /* das auszugebende Zeichen */
 Register.h.bh = GetSeite();                       /* Bildschirmseite */
 Register.h.bl = Farbe;            /* Farbe des auszugebenden Zeichen */
 Register.x.cx = 1;                /* das Zeichen nur einmal ausgeben */
 int86(0x10, &Register, &Register);       /* Interrupt 10(h) aufrufen */
}
                                                                        
/**********************************************************************/
/* WRITETEXT: schreibt eine Zeichenkette mit konstanter Farbe ab      */
/*            einer vorgegebenen Position innerhalb der aktuellen     */
/*            Bildschirmseite                                         */
/* Eingabe : - SPALTE = die Ausgabespalte                             */
/*           - ZEILE  = die Ausgabezeile                              */
/*           - TEXT   = Pointer auf den auszugebengen String          */
/*           - FARBE  = Attribut fr die auszugebenden Zeichen        */
/* Ausgabe : keine                                                    */
/* Info    : Text ist ein Pointer auf einen Zeichen-Vektor, der den   */
/*           auszugebenden Text enthlt und mit einem '\0' Zeichen    */
/*           abgeschlossen wird                                       */
/**********************************************************************/
                                                                        
void WriteText(BYTE Spalte, BYTE Zeile, char *Text, BYTE Farbe)
{
 union REGS InRegister,     /* Register-Variable zum Interrupt-Aufruf */
            OutRegister;
                                                                        
 SetPos(Spalte, Zeile);                              /* Cursor setzen */
 InRegister.h.ah = 14;                             /* Funktionsnummer */
 InRegister.h.bh = GetSeite();                     /* Bildschirmseite */
 while (*Text)                      /* Text bis '\0' Zeichen ausgeben */
  {
   WriteChar(' ', Farbe);               /* Farbe fr Zeichen vorgeben */
   InRegister.h.al = *Text++;             /* das auszugebende Zeichen */
   int86(0x10, &InRegister, &OutRegister);      /* Interrupt aufrufen */
  }
}
                                                                        
/**********************************************************************/
/* CLS: aktuelle Bildschirmseite lschen                              */
/* Eingabe : keine                                                    */
/* Ausgabe : keine                                                    */
/**********************************************************************/
                                                                        
void Cls( void )
{
 union REGS Register;       /* Register-Variable zum Interrupt-Aufruf */
                                                                        
 Register.h.ah = 6;                  /* Funktionsnummer fr Scroll-UP */
 Register.h.al = 0;                            /* 0 steht fr lschen */
 Register.h.bh = 7;              /* weie Schrift auf schwarzem Grund */
 Register.x.cx = 0;                     /* obere linke Bildschirmecke */
 Register.h.dh = 24;                       /* Koordinaten der unteren */
 Register.h.dl = 79;                       /* rechten Bildschirmecke  */
 int86(0x10, &Register, &Register);  /* BIOS-Video-Interrupt aufrufen */
}
                                                                        
/**********************************************************************/
/* NEGFLAG: evtl. Flag negieren und zugehrigen Text ausgeben         */
/* Eingabe : FLAG    = der letzte Status des Flags                    */
/*           FLAGREG = der aktuelle Status des Flags (0 = aus )       */
/*           SPALTE  = Ausgabe-Spalte fr den Flag-Namen              */
/*           ZEILE   = Ausgabe-Zeile fr den Flag-Namen               */
/*           TEXT    = der Flag-Name                                  */
/* Ausgabe : der neue Status des Flags (TRUE = an, FALSE = aus)       */
/**********************************************************************/

BYTE NegFlag(BYTE Flag, unsigned int FlagReg,
             BYTE Spalte, BYTE Zeile, char * Text)
{
 BYTE AktZeile,                                     /* aktuelle Zeile */
      AktSpalte;

 if (!(Flag == (FlagReg != 0)))        /* hat sich das Flag gendert? */
  {                                                             /* JA */
   GetPos(&AktSpalte, &AktZeile);    /* aktuelle Cursorposition holen */
   WriteText(Spalte, Zeile, Text, (BYTE) ((Flag) ? 0 : FlagFarbe));
   SetPos(AktSpalte, AktZeile);         /* alte Cursorposition setzen */
   return(Flag ^1);                       /* Bit 0 des Flags umdrehen */
  }
 else return(Flag);                        /* alles bleibt beim alten */
}

/**********************************************************************/
/* GETKEY: Einlesen eines Zeichen und Ausgabe des Flag-Status         */
/* Eingabe : keine                                                    */
/* Ausgabe : Code der empfangenen Taste                               */
/*               < 256 : normale Taste                                */
/*              >= 256 : erweiterter Tastaturcode                     */
/**********************************************************************/
                                                                        
unsigned int GetKey( void )
{
 int Taste,                             /* die zurckgelieferte Taste */
     Status;                                   /* der Tastatur-Status */

 do
  {
   Status = GetKbStatus();                   /* Tastatur-Status lesen */
   Insert = NegFlag(Insert, Status & INS, FS+9, FZ, "INSERT");
   Caps = NegFlag(Caps, Status & CAPL, FS+3, FZ, " CAPS ");
   Num = NegFlag(Num, Status & NUML, FS, FZ, "NUM");
  }
 while ( !GetKbReady() );        /* wiederhole bis Taste bereit steht */

 Taste = GetKbKey();                                   /* Taste holen */
 return ((Taste & 255) == 0) ? (Taste >> 8) + 256 : Taste & 255;
}
                                                                        
/**********************************************************************/
/* INIKEY: Tastatur-Flags initialisieren                              */
/* Eingabe : keine                                                    */
/* Ausgabe : keine                                                    */
/* Info    : die Tastatur-Flags werden gegenber ihrem aktuellen      */
/*           Status umgedreht. Dadurch wird erreicht, da ihr aktu-   */
/*           eller Status beim nchsten Aufruf der GETKEY-Funktion    */
/*           ausgegeben wird.                                         */
/**********************************************************************/
                                                                        
void IniKey( void )
{
 int Status;                                   /* der Tastatur-Status */
                                                                        
 Status = GetKbStatus();                     /* Tastatur-Status lesen */
 Insert = (Status & INS) ? FALSE : TRUE;          /* jeweils den aktu */
 Caps = (Status & CAPL)? FALSE : TRUE;            /* ellen Inhalt um- */
 Num = (Status & NUML) ? FALSE : TRUE;            /* drehen           */
}
                                                                        
/**********************************************************************/
/**                           HAUPTPROGRAMM                          **/
/**********************************************************************/
                                                                        
void main( void )
{
 unsigned int Taste;
                                                                        
 Cls();                                          /* Bildschirmlschen */
 SetPos(0,0);             /* Cursor in die obere linke Bildschirmecke */
 printf("TASTC  -  (c) 1987, 92 by Michael Tischer\n\n");
 printf("Sie knnen im folgenden einige Zeichen eingeben und dabei ");
 printf("den INSERT-, CAPS-\noder NUM-Status verndern. Jede Ver");
 printf("nderung wird in der oberen rechten Bild-\nschirmecke proto");
 printf("kolliert.\n<RETURN> oder <F1> beendet die Eingabe...\n\n");
 printf("Ihre Eingabe: ");
 IniKey();                           /* Tastatur-Flags initialisieren */
 do
  {
   if ((Taste = GetKey()) < 256)                    /* Taste einlesen */
    printf("%c", (char) Taste);           /* ausgeben (falls normale) */
  }
 while (!(Taste == CR || Taste == F1)); /* wiederholen bis F1 oder CR */
 printf("\n");
}
