;**********************************************************************;
;*                             T S R C A                              *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : stellt die Assemblerschnittstelle zu einem     *;
;*                     C-Programm dar, da als TSR-Programm ber      *;
;*                     einen Hotkey aktiviert werden kann             *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  : 10.08.1988                                     *;
;*    letztes Update : 18.11.1991                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM -mx TSRCA;    oder TASM /mx TSRCA;        *;
;*                     ... mit C-Programm verknpfen                  *;
;**********************************************************************;
                                                                        
IGROUP group _text                ;Zusammenfassung der Programm-Segmente
DGROUP group _bss,  _data         ;Zusammenfassung der Daten-Segmente
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP

_BSS   segment word public 'BSS'  ;dieses Segment nimmt alle nicht ini-
_BSS   ends                       ;tialisierten statischen Variablen auf
                                                                        
_DATA  segment word public 'DATA' ;alle initialisierten globalen und
                                  ;statischen Variablen werden in diesem
                                  ;Segment untergebracht
                                                                        
extrn  __psp : word               ;Segmentadresse des PSP des C-Programm
                                                                        
_DATA  ends
                                                                        
;== Konstanten =========================================================
                                                                        
TC_STACK   equ 512                ;Stack fr Turbo C
ARG_WORDS  equ 32                 ;kopierte Stack-Words bei TsrCall

I2F_FKT_0  equ 0AAh               ;Code fr INT 2F, Funktion 0
I2F_FKT_1  equ 0BBh               ;Code fr INT 2F, Funktion 1
TIME_OUT   equ 9                  ;Time-Out fr Aktivierung in Ticks
                                                                        
;== Programm ===========================================================
                                                                        
_TEXT  segment byte public 'CODE' ;das Programmsegment
                                                                        
;-- Referenz zu externen (C-) Funktionen -------------------------------
                                                                        
extrn      _GetHeapEnd:near             ;liefert Endadresse des Heap
                                                                        
;-- Public-Deklarationen interner Funktionen ---------------------------
                                                                        
public     _TsrInit               ;ermglicht Aufruf aus C-Programm
public     _TsrIsInst
public     _TsrUnInst
public     _TsrCanUnInst
public     _TsrCall
public     _TsrSetHotkey
public     _TsrSetPtr

;-- Variablen fr die Interrupt-Handler --------------------------------
;-- (nur ber das Codesegment zugnglich) ------------------------------
                                                                        
call_ptr   equ this dword
call_ofs   dw 0                   ;Offsetadresse fr TSRCall
call_seg   dw 0                   ;Segmentadresse noch nicht initial.

ret_ax     dw 0                   ;Sichern des Funktionsergebnisses bei
ret_dx     dw 0                   ;TsrCall

;-- Variablen, die zur Aktivierung des C-Programms bentigt werden -------
                                                                        
c_ss       dw 0                   ;C Stacksegment
c_sp       dw 0                   ;C Stackpointer
c_ds       dw 0                   ;C Datensegment
c_es       dw 0                   ;C Extrasegment
                                                                        
c_dta_ofs  dw 0                   ;DTA-Adresse des C-Programms
c_dta_seg  dw 0
                                                                        
c_psp      dw 0                   ;Segmentadresse des PSP des C-Programm
break_adr  dw 0                   ;Break-Adresse des Heap
fkt_adr    dw 0                   ;Addresse der C-TSR-Funktion
                                                                        
;-- Variablen, zum Test auf die Bettigung des Hotkey ------------------
                                                                        
key_mask   dw 3                   ;Hotkey-Maske fr BIOS-Tastaturflag
                                  ;Default: Shift links + Shift Rechts
sc_code    db 128                 ;Scan-Code des Hotkeys
                                  ;Default: keine Taste
i2F_code   db 0                   ;Funktionsnummer fr INT 2F

;-- Variablen fr die TSR-Aktivierung ----------------------------------

tsrnow     db 0                   ;wartet TSR auf Aktivierung?
tsraktiv   db 0                   ;ist TSR bereits aktiv?
in_bios    db 0                   ;zeigt BIOS-Disk-Aktivitt an

daptr      equ this dword         ;Pointer auf das DOS Indos-Flag
daptr_ofs  dw 0                   ;Offsetadresse
daptr_seg  dw 0                   ;Segmentadresse
                                                                        
;-- die folgenden Variablen nehmen die alten Adressen der Interrupt- ---
;-- Handler auf, die durch neue Interrupt-Handler ersetzt wurden     ---
                                                                        
int8_ptr   equ this dword         ;alter Interrupt-Vektor 8h
int8_ofs   dw 0                   ;Offsetadresse des alten Handlers
int8_seg   dw 0                   ;Segmentadresse des alten Handlers

int9_ptr   equ this dword         ;alter Interrupt-Vektor 9h
int9_ofs   dw 0                   ;Offsetadresse des alten Handlers
int9_seg   dw 0                   ;Segmentadresse des alten Handlers
                                                                        
int13_ptr  equ this dword         ;alter Interrupt-Vektor 13h
int13_ofs  dw 0                   ;Offsetadresse des alten Handlers
int13_seg  dw 0                   ;Segmentadresse des alten Handlers
                                                                        
int28_ptr  equ this dword         ;alter Interrupt-Vektor 28h
int28_ofs  dw 0                   ;Offsetadresse des alten Handlers
int28_seg  dw 0                   ;Segmentadresse des alten Handlers
                                                                        
int2F_ptr  equ this dword         ;alter Interrupt-Vektor 2Fh
int2F_ofs  dw 0                   ;Offsetadresse des alten Handlers
int2F_seg  dw 0                   ;Segmentadresse des alten Handlers

;-- Variablen, die Informationen ber das unterbrochene Programm -------
;-- aufnehmen                                                    -------
                                                                        
u_dta_ofs  dw 0                   ;DTA-Adresse des unterbrochenen Prg.
u_dta_seg  dw 0
                                                                        
u_psp      dw 0                   ;Segmentadresse des PSP des unt. Prg.
                                                                        
uprg_ss    dw 0                   ;SS und SP des unterbrochenen Prg.
uprg_sp    dw 0
                                                                        
;-----------------------------------------------------------------------
;-- TSRINIT: beendet das C-Programm und aktiviert die neuen Interrupt- -
;--          Handler
;-- Aufruf von C: void TsrInit(  bool TC,
;--                              void (fkt *)(void),
;--                              unsigned heap_byte );

_TsrInit  proc    near

sframe0    struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
ret_adr0   dw ?                   ;Rcksprungadresse
tc0        dw ?                   ;Compiler (1 = TURBO-C, 0 = MSC )
fktptr0    dw ?                   ;Zeiger auf C-TSR-Funktion
heap0      dw ?                   ;bentigte Heap-Bytes
sframe0    ends                   ;Ende der Struktur
                                                                        
frame      equ [ bp - bp0 ]
                                                                        
           push bp                ;BP auf dem Stack sichern
           mov  bp,sp             ;SP nach BP bertragen
                                                                        
           ;-- die C-Segmentregister sichern ---------------------------
                                                                        
           mov  c_ss,ss           ;die Register in den entsprechenden
           mov  c_sp,sp           ;Variablen merken
           mov  c_es,es
           mov  c_ds,ds
                                                                        
           ;-- die bergebenen Parameter speichern ---------------------
                                                                        
           mov  ax,frame.fktptr0  ;Pointer auf C-TSR-Funktion holen
           mov  fkt_adr,ax        ;und merken

           ;-- DTA-Adresse des C-Programms ermitteln -------------------
                                                                        
           mov  ah,2fh            ;Fkt.nr.: DTA-Adresse holen
           int  21h               ;DOS-Interrupt aufrufen
           mov  c_dta_ofs,bx      ;Adresse in den entsprechenden
           mov  c_dta_seg,es      ;Variablen merken
                                                                        
           ;-- Adresse des INDOS-Flags ermitteln -----------------------
                                                                        
           mov  ah,34h            ;Fkt.nr.: Adresse INDOS-Flag holen
           int  21h               ;DOS-Interrupt aufrufen
           mov  daptr_ofs,bx      ;Adresse in den entsprechenden
           mov  daptr_seg,es      ;Variablen merken
                                                                        
           ;-- die Adressen der umzuleitenden Interrupt-Handler holen --
                                                                        
           mov  ax,3508h          ;Interrupt-Vektor 8h holen
           int  21h               ;DOS-Interrupt aufrufen
           mov  int8_ofs,bx       ;Adresse des Handlers in den ent-
           mov  int8_seg,es       ;sprechenen Variablen merken

           mov  ax,3509h          ;Interrupt-Vektor 9h holen
           int  21h               ;DOS-Interrupt aufrufen
           mov  int9_ofs,bx       ;Adresse des Handlers in den ent-
           mov  int9_seg,es       ;sprechenen Variablen merken

           mov  ax,3513h          ;Interrupt-Vektor 13h holen
           int  21h               ;DOS-Interrupt aufrufen
           mov  int13_ofs,bx      ;Adresse des Handlers in den ent-
           mov  int13_seg,es      ;sprechenden Variablen merken

           mov  ax,3528h          ;Interrupt-Vektor 28h holen
           int  21h               ;DOS-Interrupt aufrufen
           mov  int28_ofs,bx      ;Adresse des Handlers in den ent-
           mov  int28_seg,es      ;sprechenden Variablen merken

           mov  ax,352Fh          ;Interrupt-Vektor 2Fh holen
           int  21h               ;DOS-Interrupt aufrufen
           mov  int2F_ofs,bx      ;Adresse des Handlers in den ent-
           mov  int2F_seg,es      ;sprechenden Variablen merken

           ;-- die neuen Interrupt-Handler installieren ----------------

           push ds                ;Datensegment merken
           mov  ax,cs             ;CS nach AX und dann nach DS laden
           mov  ds,ax
                                                                        
           mov  ax,2508h          ;Fkt.nr.: Interrupt 8h setzen
           mov  dx,offset int08   ;DS:DX nimmt Adresse des Handler auf
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           mov  ax,2509h          ;Fkt.nr.: Interrupt 9h setzen
           mov  dx,offset int09   ;DS:DX nimmt Adresse des Handler auf
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           mov  ax,2513h          ;Fkt.nr.: Interrupt 13h setzen
           mov  dx,offset int13   ;DS:DX nimmt Adresse des Handler auf
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           mov  ax,2528h          ;Fkt.nr. Interrupt 28h setzen
           mov  dx,offset int28   ;DS:DX nimmt Adresse des Handler auf
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           mov  ax,252Fh          ;Fkt.nr. Interrupt 2Fh setzen
           mov  dx,offset int2F   ;DS:DX nimmt Adresse des Handler auf
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           pop  ds                ;DS wieder vom Stack kholen
                                                                        
           ;-- Anzahl der Paragraphen berechnen, die im Speicher ver- ----
           ;-- bleiben mssen                                         ----

           call _GetHeapEnd       ;C-Funktion im TSR-Modul aufrufen
           add  ax,frame.heap0    ;bentigten Heap-Speicher addieren
                                                                        
           ;-- bei TURBO-C befindet sich der Stack hinter dem Heap und -
           ;-- beginnt mit dem Ende des Segments, er mu deshalb an
           ;-- den Heap herangezogen werden.
                                                                        
           cmp  byte ptr frame.tc0,0  ;wird mit TURBO-C gearbeitet?
           je   msc2                  ;NEIN, mit MSC

           add  ax,TC_STACK-1     ;neuen Stack-Pointer fr TC berechnen
           mov  c_sp,ax           ;und spreichern
           inc  ax                ;Break-Adresse setzen
                                                                        
           ;-- Anzahl der Paragraphen berechnen, die resident im  ------
           ;-- Speicher verbleiben mssen
                                                                        
msc2:      mov  dx,ax             ;Breakadrese nach DX holen
           add  dx,15             ;Velust durch Integer-Div. vermeiden
           mov  cl,4              ;4 mal nach rechts verschieben und da-
           shr  dx,cl             ;mit durch 16 teilen
           mov  ax,ds             ;DS nach AX holen
           mov  bx,__psp          ;Segmentadresse des PSP holen
           mov  c_psp,bx          ;in einer Variablen merken
           sub  ax,bx             ;DS von PSP abziehen
           add  dx,ax             ;und zur Anzahl der Para. addieren
           mov  ax,3100h          ;Fkt.nr.: Programm resident beenden
           int  21h               ;DOS-Interrupt aufrufen und damit das
                                  ;Programm beenden
                                                                        
_TsrInit  endp
                                                                        
          assume CS:IGROUP, DS:nothing, ES:nothing, SS:nothing

;-----------------------------------------------------------------------
;-- TSRSETHOTKEY: Stellt den Hotkey des Programms ein
;-- Aufruf von C: void TsrSetHotKey( unsigned KeyMask,
;--                                  byte     ScanCode );
;-- Info            : Diese Prozedur ist FAR, damit sie auch in einem
;--                   bereits installierten TSR aufgerufen werden kann.
;--

_TsrSetHotkey  proc far

sframe1    struc                  ;Struktur zum Zugriff auf den Stack
bp1        dw ?                   ;nimmt BP auf
ret_adr1   dd ?                   ;Rcksprungadresse
keymask1   dw ?                   ;Maske fr Hotkey
sc_code1   dw ?                   ;Scan-Code des Hotkey
sframe1    ends                   ;Ende der Struktur

frame      equ [ bp - bp1 ]

           push bp                ;BP auf dem Stack sichern
           mov  bp,sp             ;SP nach BP bertragen

           ;-- die bergebenen Parameter speichern ---------------------
                                                                        
           mov  ax,frame.keymask1 ;Maske fr Hotkey holen und merken
           mov  key_mask,ax       ;und sichern
           mov  al,byte ptr frame.sc_code1 ;Scan-Code des Hotkey holen
           mov  sc_code,al                 ;und sichern

           pop  bp                ;BP wieder vom Stack
           ret                    ;zurck 

_TsrSetHotkey  endp

;-----------------------------------------------------------------------
;-- TSRISINST: stellt fest, ob das Programm bereits installiert ist ----
;-- Aufruf von C: bool TsrIsInst( byte i2f_fktnr);
;-- Return-Wert : TRUE, wenn das Programm bereits installiert war,
;--               sonst FALSE

_TsrIsInst proc    near
                                                                        
sframe2    struc                  ;Struktur zum Zugriff auf den Stack
bp2        dw ?                   ;nimmt BP auf
ret_adr2   dw ?                   ;Rcksprungadresse
i2F_code2  dw ?                   ;Funktionsnummer fr INT 2F
sframe2    ends                   ;Ende der Struktur

frame      equ [ bp - bp2 ]

           push bp                ;BP auf dem Stack sichern
           mov  bp,sp             ;SP nach BP bertragen

           mov  ah,byte ptr frame.i2F_code2 ;Fktnr. fr INT 2F
           mov  i2F_code,ah                 ;und sichern
           mov  al,I2F_FKT_0                ;Unterfktnr.
           mov  bx,ax                       ;beide Nummern merken
           int  2Fh
           xchg bh,bl                       ;Nummern verdrehen
           cmp  ax,bx                       ;mit Rckgabe vergleichen
           mov  ax,0                        ;nicht von Install. ausgehen
           jne  isi_end                     ;ungleich --> nicht install.

           ;-- Segmentadresse der bereits installierten Kopie ermitteln

           mov  ah,i2f_code       ;Nein, Segmentadresse ber INT 2Fh
           mov  al,I2F_FKT_1      ;Unterfunktion 01h laden
           int  2Fh
           mov  call_seg,ax       ;und in Variablen merken
           mov  ax,-1             ;ist installiert

isi_end:   pop  bp                ;BP wieder vom Stack
           ret                    ;zurck

_TsrIsInst endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- TSRCANUNIST: Feststellen, ob installierte Kopie des TSR wieder -----
;--              reinstalliert werden kann
;-- Aufruf von C: bool TsrCanUnInst( void );
;-- Ausgabe     : TRUE, wenn Reinstallation mglich, sonst FALSE
;-- Info        : Das Programm kann nur reinstalliert werden, wenn
;--               keiner der Interrupt-Vektoren des Programms in
;--               der Zwischenzeit auf ein anderes Programm um-
;--               geleitet wurde.

tsrlist    db  08h,09h,13h,28h,2Fh,00h ;Liste der umgeleiteten INT
                                       ;00h markiert Ende
_TsrCanUninst proc  near

           push di                  ;DI sichern (falls Registervar.)
           mov  dx,call_seg         ;Segment der install. Kopie laden
           mov  di,offset tsrlist-1 ;DI auf Liste

tcu_1:     inc  di                ;DI auf die nchste Intr-Nummer
           mov  al,cs:[di]        ;nchste Intr-Nummer nach AL
           or   al,al             ;Ende der Liste erreicht?
           je   tcu_ok            ;Ja, alle Vektoren ok

           mov  ah,35h            ;Fktnr. fr "Get Interrupt"
           int  21h               ;DOS-Interrupt aufrufen
           mov  cx,es             ;ES zum Vergleich nach CX
           cmp  dx,cx             ;noch im gleichen Segment?
           je   tcu_1             ;Ja, keine Reinstallation mglich

           xor  ax,ax             ;Nein, keine Reinstallation mglich
           pop  di                ;DI zurckholen
           ret

tcu_ok:    mov  ax,-1
           pop  di                ;DI zurckholen
           ret

_TsrCanUninst endp

;-----------------------------------------------------------------------
;-- TSRUNINST: Reinstalliert das TSR-Programm und gibt den allokierten -
;--            Speicher wieder frei.
;-- Aufruf von C: void TsrUnInst( void );
;-- Info             : Diese Routine darf nur nach einem vorhergehenen
;--                    und erfolgreichen Aufruf von TSRCANUNINST()
;--                    aufgerufen werden

_TsrUninst    proc    near
                                                                        
           push ds
           mov  es,call_seg       ;Segment des installierten TSR laden

           ;-- die Interrupt-Handler der TSR-Programm wieder -----------
           ;-- reinstallieren                                -----------

           cli                    ;keine Interrupts mehr zulassen
           mov  ax,2508h          ;Fkt.nr: Handler fr INT 8 setzen
           mov  ds,es:int8_seg    ;Segmentadresse des alten Handler
           mov  dx,es:int8_ofs    ;Offsetadresse des alten Handler
           int  21h               ;den alten Handler wieder installieren

           mov  ax,2509h          ;Fkt.nr: Handler fr INT 9 setzen
           mov  ds,es:int9_seg    ;Segmentadresse des alten Handler
           mov  dx,es:int9_ofs    ;Offsetadresse des alten Handler
           int  21h               ;den alten Handler wieder installieren
                                                                        
           mov  ax,2513h          ;Fkt.nr: Handler fr INT 13 setzen
           mov  ds,es:int13_seg   ;Segmentadresse des alten Handler
           mov  dx,es:int13_ofs   ;Offsetadresse des alten Handler
           int  21h               ;den alten Handler wieder installieren
                                                                        
           mov  ax,2528h          ;Fkt.nr: Handler fr INT 28 setzen
           mov  ds,es:int28_seg   ;Segmentadresse des alten Handler
           mov  dx,es:int28_ofs   ;Offsetadresse des alten Handler
           int  21h               ;den alten Handler wieder installieren
                                                                        
           mov  ax,252Fh          ;Fkt.nr: Handler fr INT 2F setzen
           mov  ds,es:int2F_seg   ;Segmentadresse des alten Handler
           mov  dx,es:int2F_ofs   ;Offsetadresse des alten Handler
           int  21h               ;den alten Handler wieder installieren

           sti                    ;Interrupts wieder erlauben
                                                                        
           ;-- Speicher wieder freigeben -------------------------------

           mov  es,es:c_psp       ;Segadr. des PSP des TSR-Programm
           mov  cx,es             ;in CX merken
           mov  es,es:[ 02ch ]    ;Segadr. des Environment aus PSP holen
           mov  ah,49h            ;Fkt.nr.: allok. Speicher freigeben
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           mov  es,cx             ;ES aus CX zurckholen
           mov  ah,49h            ;Fkt.nr.: allok. Speicher freigeben
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           pop  ds                ;DS und BP wieder vom Stack holen
           ret                    ;zurck zum Aufrufer

_TsrUninst endp                   ;Ende der Prozedur
                                                                        
;-----------------------------------------------------------------------
;-- TSRSETPTR: Speichert die Adresse der Routine, die bei einem nach- --
;--            folgenden Aufruf von TSRCALL aufgerufen werden soll
;--            und liefert einen Zeiger auf TSRCALL zurck
;-- Aufruf von C: void (*)(void) TsrSetPtr( void far * Fkt );

_TsrSetPtr  proc    near

sframe3    struc                  ;Struktur zum Zugriff auf den Stack
bp3        dw ?                   ;nimmt BP auf
ret_adr3   dw ?                   ;Rcksprungadresse
fktptr3    dd ?                   ;Zeiger auf die aufzurufenden Routine
sframe3    ends                   ;Ende der Struktur

frame      equ [ bp - bp3 ]

           push bp                ;BP auf dem Stack sichern
           mov  bp,sp             ;SP nach BP bertragen

           mov  ax,word ptr frame.fktptr3  ;Offsetadresse nach AX
           mov  call_ofs,ax                ;und speichern

           mov  ax,offset _TsrCall;Near-Zeiger auf TSRCall zurckliefern
           pop  bp                ;BP wieder vom Stack

           ret                    ;zurck und Argumente vom Stack

_TsrSetPtr endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- TSRCALL: Ruft eine Routine in der zuvor installierten Kopie des ----
;--          TSR-Programms auf.
;-- Aufruf von C: void TsrCall( void )
;-- Achtung          : - Die Argumente vom Stack des Aufrufers werden
;--                      von dieser Routine auf den Stack des aufzu-
;--                      rufenden Programms kopiert und der Stack um-
;--                      geschaltet, damit weiterhin gilt DS=SS

_TsrCall   proc near

           ;-- Kontextwechsel zum Turbo-Programm durchfhren und die ---
           ;-- angegebene Prozedur aufrufen
                                                                        
           push di                ;DS, SI und DI merken
           push si
           push ds

           mov  ah,2fh            ;Fkt.nr.: DTA-Adresse holen
           int  21h               ;DOS-Interrupt aufrufen
           mov  u_dta_ofs,bx      ;Adresse der DTA des unterbrochenen
           mov  u_dta_seg,es      ;Programms sichern

           mov  es,call_seg       ;Segmentadresse des installierten TSR
                                  ;nach ES
           mov  ah,50h            ;Fkt.nr.: Adresse des PSP setzen
           mov  bx,es:c_psp       ;Segmentadresse des PSP holen
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           mov  ah,1ah            ;Fkt.nr.: DTA-Adresse setzen
           mov  dx,es:c_dta_ofs   ;Offsetadresse der neuen DTA und
           mov  ds,es:c_dta_seg   ;Segmentadresse der neuen DTA holen
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           ;-- Argumente auf den Stack des installierten TSR kopieren --

           push ss                ;DS:SI auf die Argumente auf dem
           pop  ds                ;aktuellen Stack setzen
           mov  si,sp
           add  si,8              ;ber RET und PUSH DS,DI,SI hinweg

	   mov  cx,ARG_WORDS*2
           mov  di,es:c_sp        ;ES:DI auf den Stack des installierten
	   sub  di,cx
           mov  es,es:c_ss
	   rep movsb              ;Argumente kopieren

           ;-- Segmentregister des install. TSR einstellen und aufrufen

           mov  es,call_seg       ;Segmentadresse des installierten TSR
                                  ;nach ES

           cli                    ;keine Interrupts zulassen
           mov  uprg_ss,ss        ;aktuelles Stacksegment und Stack-
           mov  uprg_sp,sp        ;zeiger merken

           mov  ss,es:c_ss        ;den Stack der installierten Kopie des
           mov  sp,es:c_sp        ;Programms aktivieren
	   sub  sp,ARG_WORDS*2    ;ber die Argumente hinweg
           sti                    ;Interrupts wieder erlauben

           mov  ds,es:c_ds        ;Segment-Register fr das Turbo-
           mov  es,es:c_es        ;Programm setzen

           call [call_ptr]        ;die ber TSRSETPTR genannte Funktion
           mov  ret_ax,ax         ;Funktionsergebnis merken
           mov  ret_dx,dx

           cli                    ;keine Interrupts zulassen
           mov  ss,uprg_ss        ;wieder auf den eigenen Stack
           mov  sp,uprg_sp        ;umschalten
           sti                    ;Interrupts wieder erlauben

           ;-- Kontextwechsel zum aktuellen Programm durchfhren -------
                                                                        
           mov  ah,1ah            ;Fkt.nr.: DTA-Adresse setzen
           mov  dx,u_dta_ofs      ;Offset- und Segmentadresse der DTA
           mov  ds,u_dta_seg      ;des unterbrochenen Programms laden
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           mov  es,call_seg       ;ES und DS zurckholen
           pop  ds                ;zurckholen

           mov  ah,50h            ;Fkt.nr.: Adresse des PSP setzen
           mov  bx,cs             ;CS nach BX holen
           sub  bx,10h            ;Segmentadresse des PSP berechnen
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           mov  ax,ret_ax         ;Funktionsergebnis zurckholen
           mov  dx,ret_dx

           pop  si                ;Register zurckholen
           pop  di
           ret                    ;zurck zum Aufrufer

_TsrCall   endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- DOSAKTIV: Stellt anhand des DOS-INDOS-Flags fest, ob DOS derzeit
;--           unterbrochen werden darf.
;-- Eingabe:  keine
;-- Ausgabe:  Zero-Flag = 1 : DOS darf unterbrochen werden

dosaktiv   proc near

           push ds                ;DS und BX auf dem Stack merken
           push bx
           lds  bx,daptr          ;DS:BX zeigen jetzt auf das INDOS-Flag
           cmp  byte ptr [bx],0   ;DOS-Funktion aktiv?
           pop  bx                ;BX und DS wieder vom Stack holen
           pop  ds

           ret                    ;zurck zum Aufrufer

dosaktiv   endp

;-----------------------------------------------------------------------
;-- es folgen die neuen Interrupt-Handler ------------------------------
;-----------------------------------------------------------------------

;-- der neue Interrupt 08h-Handler (Timer) -----------------------------

int08      proc far

           cmp  tsrnow,0          ;soll TSR aktiviert werden?
           je   i8_end            ;Nein, weiter zum alten Handler

           dec  tsrnow            ;Ja, Aktivierungs-Flag dekrementieren

           ;-- TSR soll aktiviert werden, aber ist das mglich? --------

           cmp  in_bios, 0        ;BIOS-Disk-Interrupt gerade aktiv?
           jne  i8_end            ;JA --> keine Aktivierung mglich

           call dosaktiv          ;darf DOS unterbrochen werden?
           je   i8_tsr            ;Ja, TSR aufrufen

i8_end:    jmp  [int8_ptr]        ;zum alten Handler springen

           ;-- TSR aktivieren ------------------------------------------

i8_tsr:    mov  tsrnow,0          ;TSR wartet nicht mehr auf Aktivierung
           mov  tsraktiv,1        ;TSR ist gleich aktiv
           pushf                  ;Aufruf des alten Handler ber den
           call [int8_ptr]        ;Befehl INT 8h simulieren
           call start_tsr         ;das TSR-Programm starten
           iret                   ;zurck zum unterbrochenen Programm

int08      endp

;-- der neue Interrupt 09h-Handler (Tastatur) --------------------------

int09      proc far

           push ax
           in   al,60h            ;Tastatur-Port auslesen

           cmp  tsraktiv,0        ;ist das TSR-Programm bereits aktiv?
           jne  i9_end            ;JA: alten Handler aufr, dann zurck

           cmp  tsrnow,0          ;wartet TSR auf seine Aktivierung?
           jne  i9_end            ;JA: alten Handler aufr, dann zurck

           ;-- Auf Hotkey testen ---------------------------------------

           cmp  sc_code,128       ;gibt es berhaupt einen Scan-Code?
           je   i9_ks             ;Nein, nur Umschalttasten berprfen

           cmp  al,128            ;Doch, ist Release-Code?
           jae  i9_end            ;Ja, ist nicht Hotkey

           cmp  sc_code,al        ;Make-Code, mit Taste vergleichen
           jne  i9_end            ;stimmt nicht, keine Aktivierung

i9_ks:     ;-- Status der Umschalttasten prfen ------------------------

           push ds
           mov  ax,040h              ;DS auf das Variablen-Segment des
           mov  ds,ax                ;ROM-BIOS
           mov  ax,word ptr ds:[17h] ;BIOS-Tastatur-Flag holen
           and  ax,key_mask          ;die nicht-Hotkey-Bits ausblenden
           cmp  ax,key_mask          ;nur noch die Hotkey-Bits brig?
           pop  ds                
           jne  i9_end            ;Hotkey entdeckt? NEIN --> zurck

           cmp  in_bios, 0        ;BIOS-Disk-Interrupt gerade aktiv?
           jne  i9_e1             ;JA --> keine Aktivierung mglich

           call dosaktiv          ;darf DOS unterbrochen werden?
           je   i9_tsr            ;Ja, TSR starten

i9_e1:     mov  tsrnow,TIME_OUT;TSR wartet auf Aktivierung

i9_end:    pop   ax               ;AX wieder zurckholen
           jmp   [int9_ptr]       ;zum alten Handler springen

i9_tsr:    mov   tsraktiv,1       ;TSR ist jetzt (gleich) aktiv
           mov   tsrnow,0         ;kein verzgerter Start gewnscht
           pushf
           call  [int9_ptr]       ;alten Handler aufrufen
           pop   ax               ;AX wieder zurckholen
           call  start_tsr        ;das TSR-Programm starten
           iret                   ;zurck zum unterbrochenen Programm

int09      endp
                                                                        
;-- der neue Interrupt 13h-Handler (Diskette/Festplatte) ---------------

int13      proc far
                                                                        
           inc  in_bios           ;BIOS-Disk-Flag inkrementieren
           pushf                  ;Aufruf der alten Interrupt-Handler
           call [int13_ptr]       ;ber INT 13h simulieren
           dec  in_bios           ;BIOS-Disk-Flag wieder zurcksetzen

           sti                    ;Interruptaufrufe wieder erlauben
           ret  2                 ;zurck zum Aufrufer, aber das Flag
                                  ;Register zuvor nicht vom Stack holen
int13      endp
                                                                        
;-- der neue Interrupt 28h-Handler (DOS-idle) --------------------------

int28      proc far
                                                                        
           cmp  tsrnow,0          ;wartet TSR auf Aktivierung?
           je   i28_end           ;Nein, zurck zum Aufrufer

           cmp  in_bios, 0        ;Ja, aber ist Disk-Intr gerade aktiv?
           je   i28_tsr           ;Ja, also nicht aktivieren

i28_end:   jmp  [int28_ptr]       ;zum alten Handler

           ;-- TSR starten ---------------------------------------------

i28_tsr:   mov  tsrnow,0          ;TSR wartet nicht mehr auf Aktivierung
           mov  tsraktiv,1        ;TSR ist (gleich) aktiv
           pushf                  ;Aufruf des alten Interrupt-Handler
           call [int28_ptr]       ;ber INT 28h simulieren
           call start_tsr         ;das TSR-Programm starten
           iret                   ;zurck zum Aufrufer

int28      endp
                                                                        
;-- der neue Interrupt 2Fh-Handler (Multiplexer) -----------------------

int2F      proc far

           cmp  ah,i2F_code       ;Aufruf fr dieses TSR?
           jne  i2F_end           ;Nein, zum alten Handler

           cmp  al,I2F_FKT_0      ;Ja, ist es Unterfunktion 00h?
           je   i2F_0             ;Ja, ausfhren

           cmp  al,I2F_FKT_1      ;Vielleicht Unterfunktion 01h?
           je   i2F_1             ;Ja, ausfhren

           iret                   ;Nein, Aufruf ignorieren

i2F_end:   ;-- TSR ist nicht gemeint, Aufruf weiterleiten --------------

           jmp  [int2F_ptr]       ;zum alten Handler

i2F_0:     ;-- Unterfunktion 00: Installations-Check -------------------

           xchg ah,al             ;Funktions- und Unterfkt-nummer aust.
           iret                   ;zurck zum Aufrufer

i2F_1:     ;-- Unterfunktion 01: Segmentadresse zurckliefern ----------

           mov  ax,cs             ;Segmentadresse nach AX
           iret                   ;zurck zum Aufrufer

int2F      endp

;-- START_TSR: das TSR-Programm aktivieren -----------------------------

start_tsr  proc near
                                                                        
           ;-- Kontextwechsel zum C-Programm durchfhren ---------------
                                                                        
           cli                    ;keine Interrupts zulassen
           mov  uprg_ss,ss        ;aktuelles Stacksegment und Stack-
           mov  uprg_sp,sp        ;zeiger merken

           mov  ss,c_ss           ;den Stack des C-Programms aktivieren
           mov  sp,c_sp
           sti                    ;Interrupts wieder erlauben

           push ax                ;die Prozessorregister auf dem C-Stack
           push bx                ;sichern
           push cx
           push dx
           push bp
           push si
           push di
           push ds
           push es
                                                                        
           ;-- 64 Words vom DOS-Stack sichern ----------------------------
                                                                        
           mov  cx,64             ;Schleifenzhler
           mov  ds,uprg_ss        ;DS:SI auf das Ende des DOS-Stack setzen
           mov  si,uprg_sp
                                                                        
tsrs1:     push word ptr [si]     ;Word aus dem DOS-Stack auf dem C-Stack
           inc  si                ;sichern und SI auf das nchste Stack-
           inc  si                ;Word setzen
           loop tsrs1             ;alle 64 Words abarbeiten
                                                                        
           mov  ah,51h            ;Fkt.nr.: Adresse des PSP ermitteln
           int  21h               ;DOS-Interrupt aufrufen
           mov  u_psp,bx          ;Segmentadresse des PSP merken

           mov  ah,2fh            ;Fkt.nr.: DTA-Adresse holen
           int  21h               ;DOS-Interrupt aufrufen
           mov  u_dta_ofs,bx      ;Adresse der DTA des unterbrochenen
           mov  u_dta_seg,es      ;Programms sichern

           mov  ah,50h            ;Fkt.nr.: Adresse des PSP setzen
           mov  bx,c_psp          ;Segmentadresse PSP des C-Prg. holen
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           mov  ah,1ah            ;Fkt.nr.: DTA-Adresse setzen
           mov  dx,c_dta_ofs      ;Offsetadresse der neuen DTA und
           mov  ds,c_dta_seg      ;Segmentadresse der neuen DTA holen
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           mov  ds,c_ds           ;Segment-Register fr das C-Programm
           mov  es,c_es           ;setzen

           call [fkt_adr]         ;die Startfunktion des C-Prg. aufrufen

           ;-- Kontextwechsel zum unterbrochenen Programm durchfhren --
                                                                        
           mov  ah,1ah            ;Fkt.nr.: DTA-Adresse setzen
           mov  dx,u_dta_ofs      ;Offset- und Segmentadresse der DTA
           mov  ds,u_dta_seg      ;des unterbrochenen Programms laden
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           mov  ah,50h            ;Fkt.nr.: Adresse des PSP setzen
           mov  bx,u_psp          ;Segadr. PSP des untebrochenen Prg.
           int  21h               ;DOS-Interrupt aufrufen
                                                                        
           ;-- DOS-Stack wieder restaurieren -----------------------------
                                                                        
           mov  cx,64             ;Schleifenzhler
           mov  ds,uprg_ss        ;DS:SI mit der Endadresse des DOS-Stack
           mov  si,uprg_sp        ;laden
           add  si,128            ;SI auf den Anfang des DOS-Stack setzen
tsrs2:     dec  si                ;SI auf das vorhergehende Stack-Word
           dec  si
           pop  word ptr [si]     ;Word vom C-Stack auf DOS-Stack holen
           loop tsrs2             ;alle 64 Words abarbeiten
                                                                        
           pop  es                ;die gesicherten Register wieder vom
           pop  ds                ;C-Stack zurckholen
           pop  di
           pop  si
           pop  bp
           pop  dx
           pop  cx
           pop  bx
           pop  ax
                                                                        
           cli                    ;keine Interrupts mehr zulassen
           mov  ss,uprg_ss        ;Stackzeiger und Stacksegment des un-
           mov  sp,uprg_sp        ;terbrochenen Programms wieder setzen
                                                                        
           mov  tsraktiv,0        ;TSR ist nicht mehr aktiv
           sti                    ;Interruptaufrufe wieder erlauben

           ret                    ;zurck zum Aufrufer
                                                                        
start_tsr  endp
                                                                        
;-------------------------------------------------------------------------
                                                                        
_text      ends                   ;Ende des Codesegments
           end                    ;Ende des Programms

