;**********************************************************************;
;*                             T Y P M C A                            *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : Assemblerroutine zur Einbindung in ein C-      *;
;*                     Programm, mit dessen Hilfe die Typematic-Rate  *;
;*                     der MF-II-Tastatur eingestellt werden kann.    *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  : 27.08.1988                                     *;
;*    letztes Update : 27.08.1988                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM TYPMCA;    oder     TASM TYPMCA           *;
;*                     ... dann mit TYPMC.C verknpfen                *;
;**********************************************************************;
                                                                        
;== Konstanten =========================================================
                                                                        
KB_STATUS_P   equ 64h             ;Statusport der Tastatur
KB_DATEN_P    equ 60h             ;Datenport der Tastatur
                                                                        
OB_FULL       equ 1               ;Bit 0 im Status-Port der Tastatur
                                  ;ein Zeichen im Ausgabe-Puffer
IB_FULL       equ 2               ;Bit 1 im Status-Port der Tastatur
                                  ;ein Zeichen im Eingabe-Puffer
                                                                        
ACK_SIGNAL    equ 0fah            ;Acknowledge-Signal der Tastatur
SET_TYPEM     equ 0f3h            ;Set-Typematic Code
                                                                        
MAX_TRY       equ 3               ;Anzahl der erlaubten Fehlversuche
                                                                        
;== Segmentdeklarationen fr das C-Programm ============================
                                                                        
IGROUP group _text                ;Zusammenfassung der Programm-Segmente
DGROUP group _bss,  _data   ;Zusammenfassung der Daten-Segmente
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP
                                                                        
_BSS   segment word public 'BSS'  ;dieses Segment nimmt alle nicht ini-
_BSS   ends                       ;tialisierten statischen Variablen auf
                                                                        
_DATA  segment word public 'DATA' ;alle initialisierten globalen und
                                  ;statischen Variablen werden in diesem
                                  ;Segment untergebracht
_DATA  ends
                                                                        
;== Programm ===========================================================
                                                                        
_TEXT  segment byte public 'CODE' ;das Programmsegment
                                                                        
public     _set_typm
                                                                        
;-----------------------------------------------------------------------
;-- SET_TYPM: bermittelt die zu setzende Typematic-Rate an den  -------
;--           Tastatur-Controller
;-- Aufruf von C: bool set_typem( byte trate );
;-- Return-Wert : TRUE, wenn die Typematic-Rate gesetzt werden konnte,
;--               FALSE wenn dabei ein Fehler aufgetreten ist
                                                                        
_set_typm  proc near
                                                                        
sframe0    struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
ret_adr0   dw ?                   ;Rcksprungadresse zum Aufrufer
trate0     dw ?                   ;die zu setzende Typematic-Rate
sframe0    ends                   ;Ende der Struktur
                                                                        
frame      equ [ bp - bp0 ]       ;adressiert die Elemente der Struktur
                                                                        
           push bp                ;BP auf dem Stack sichern
           mov  bp,sp             ;SP nach BP bertragen
                                                                        
           xor  dx,dx             ;von fehlerhafter bertragung ausgehen
           mov  ah,SET_TYPEM      ;Befehlscode fr Typematic-Rate setzen
           cli                    ;keine Interrupts mehr zulassen
           call send_kb           ;an den Controller senden
           jne  error             ;Fehler? JA --> Error
                                                                        
           mov  ah,byte ptr frame.trate0 ;Typematic-Rate holen
           call send_kb           ;an den Controller senden
           jne  error             ;Fehler? JA --> Error
                                                                        
           inc  dl                ;alles o.k., TRUE zurckliefern
                                                                        
error:     sti                    ;Interrupts wieder erlauben
           mov  ax,dx             ;Return-Wert nach AX
           pop  bp                ;BP wieder vom Stack holen
           ret                    ;zurck zum C-Programm
                                                                        
_set_typm  endp
                                                                        
;-----------------------------------------------------------------------
;-- SEND_KB: sendet ein Byte an den Tastatur-Controller ----------------
;-- Eingabe  : AH = das zu sendende Byte
;-- Ausgabe  : Zero-Flag: 0=Fehler, 1=o.k.
;-- Register : AX und das Flag-Register werden verndert
;-- Info     : diese Routine ist nur zum internen Aufruf innerhalb
;--            dieses Moduls bestimmt
                                                                        
send_kb   proc near
                                                                        
          push cx                 ;alle Register, die innerhalb dieser
          push bx                 ;Routine verndert werden, auf dem
                                  ;Stack sichern
                                                                        
          mov  bl,MAX_TRY         ;maximal MAX_TRY-Fehlversuche
                                                                        
          ;-- warten, bis Controller zur Aufnahme von Daten bereit ist -
                                                                        
skb_1:    xor  cx,cx              ;maximal 65536 Schleifendurchlufe
skb_2:    in   al,KB_STATUS_P     ;Inhalt des Statusport einlesen
          test al,IB_FULL         ;noch ein Zeichen im Input-Buffer?
          loopne skb_2            ;JA --> SKB_2
                                                                        
          ;-- Zeichen an den Controller senden -------------------------
                                                                        
          mov  al,ah              ;Zeichen nach AL holen
          out  KB_DATEN_P,al      ;Zeichen an den Datenport senden
skb_3:    in   al,KB_STATUS_P     ;Inhalt des Statusport einlesen
          test al,OB_FULL         ;Anwort im Output-Buffer?
          loope skb_3             ;NEIN --> SKB_3
                                                                        
          ;-- Antwort des Controllers holen und auswerten --------------
                                                                        
          in   al,KB_DATEN_P      ;Anwort vom Datenport einlesen
          cmp  al,ACK_SIGNAL      ;wurde des Zeichen akzeptiert?
          je   skb_end            ;JA --> alles o.k.
                                                                        
          ;-- das Zeichen wurde nicht akzeptiert -----------------------
                                                                        
          dec  bl                 ;Fehlerzhler dekrementieren
          jne  skb_2              ;noch Fehlerversuche frei?
                                  ;JA --> SKB_2
                                                                        
          or   bl,1               ;NEIN, Zero-Flag auf 0 setzen und
                                  ;damit Fehler anzeigen
                                                                        
skb_end:  pop  bx                 ;die gesicherten Register wieder vom
          pop  cx                 ;Stack holen
          ret                     ;zurck zum Aufrufer
                                                                        
send_kb   endp
                                                                        
;-------------------------------------------------------------------------
                                                                        
_text      ends                   ;Ende des Codesegments
           end                    ;Ende des Programms
