;**********************************************************************;
;*                       V 1 6 C O L C A . A S M                      *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : enthlt verschiedene Routinen zur Arbeit in    *;
;*                     den EGA- und VGA-Grafikmodi mit 16 Farben      *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  :  5.12.1990                                     *;
;*    letztes Update : 14.01.1991                                     *;
;*--------------------------------------------------------------------*;
;*    Speichermodell : SMALL                                          *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM /mx V16COLCA    oder   TASM -mx V16COLCA  *;
;**********************************************************************;

IGROUP group _text                ;Zusammenfassung der Programm-Segmente
DGROUP group _bss,  _data         ;Zusammenfassung der Daten-Segmente
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP

_BSS   segment word public 'BSS'  ;dieses Segment nimmt alle nicht ini-
_BSS   ends                       ;tialisierten statischen Variablen auf

_DATA  segment word public 'DATA' ;alle initialisierten globalen und
                                  ;statischen Variablen werden in diesem
                                  ;Segment untergebracht

_DATA  ends

;== Konstanten =========================================================

SC_INDEX       = 3c4h             ;Indexregister des Sequencer-Ctrl.
SC_MAP_MASK    = 2                ;Nummer des Map-Mask-Registers
SC_MEM_MODE    = 4                ;Nummer des Memory-Mode-Registers

GC_INDEX       = 3ceh             ;Indexregister des Graphics-Ctrl.
GC_FN_SELECT   = 3                ;Nummer des Funktion-Select-Reg.
GC_READ_MAP    = 4                ;Nummer des Read-Map-Registers
GC_GRAPH_MODE  = 5                ;Nummer des Graphics-Mode-Registers
GC_MISCELL     = 6                ;Nummer des Miscellaneous-Registers
GC_BIT_MASK    = 8                ;Nummer des Bit-Mask-Registers

CRTC_INDEX     = 3d4h             ;Indexregistes des CRT-Controllers
CC_MAX_SCAN    = 9                ;Nummer des Maximum-Scan-Line-Reg.
CC_START_HI    = 0Ch              ;Nummer des Hi-Start-Registers
CC_UNDERLINE   = 14h              ;Nummer des Underline-Registers
CC_MODE_CTRL   = 17h              ;Nummer des Mode-Control-Registers

DAC_WRITE_ADR  = 3C8h             ;DAC-Write-Adresse
DAC_READ_ADR   = 3C7h             ;DAC-Read-Adresse
DAC_DATA       = 3C9h             ;DAC-Datenregister

VERT_RETRACE   = 3DAh             ;Input Status-Register #1

PIXX           = 640              ;horizontale Auflsung

;== Daten ==============================================================

_DATA  segment word public 'DATA'

vio_seg    dw 0A000h              ;Video-Segment mit akt. Seite
zbreite    dw 0                   ;Breite einer Punktzeile in Bytes
pageofs    dw 0                   ;Page-Offset in der Segmentadresse

_DATA  ends

;== Programm ===========================================================

_TEXT  segment byte public 'CODE' ;das Programmsegment

;-- Public-Deklarationen -----------------------------------------------

public    _init640350             ;640*350-Punkte-Modus initialisieren
public    _init640480             ;640*480-Punkte-Modus initialisieren
public    _init640200             ;640*200-Punkte-Modus initialisieren
public    _init320200             ;320*200-Punkte-Modus initialisieren
public    _setpix                 ;Punkt setzen
public    _getpix                 ;Farbe eines Punktes ermitteln
public    _showpage               ;Seite 0 oder 1 anzeigen
public    _setpage                ;Seite fr setpix und getpix festlegen
public    _getfontptr             ;liefert Zeiger auf 8*8-Zeichensatz

;-----------------------------------------------------------------------
;-- INIT640350: initialisiert den 640*350 EGA-Grafikmodus bei 16 Farben
;-- Deklaration : void init640350( void );

_init640350 proc near

	   mov   al,10h           ;Modus 10h einstellen
	   mov   cx,28000 / 16    ;Page-Offset

init16:    mov   bx,640/8         ;Zeilenbreite

init:      mov   zbreite,bx       ;Zeilenbreite speichern
           mov   pageofs,cx       ;Page-Offset fr Segmentadresse merken

           xor   ah,ah            ;Funktion 00h zum Setzen des Modus
	   int   10h              ;aufrufen

	   ret                    ;zurck zum Aufrufer

_init640350 endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- INIT640480: initialisiert den 640*480 VGA-Grafikmodus bei 16 Farben
;-- Deklaration : void init640480( void );

_init640480 proc near

	   mov   al,12h           ;Modus 12h  einstellen

           ;-- der Page-Offset ist uninteressant, weil nur eine Seite
           ;-- darstellbar ist

           jmp   init16

_init640480 endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- INIT640200: initialisiert den 640*200 EGA-Grafikmodus bei 16 Farben
;-- Deklaration : void init640200( void );

_init640200 proc near

	   mov   al,0Eh           ;Modus 0Eh  einstellen
           mov   cx,( 64000 / 4 ) / 16 ;Page-Offset

           jmp   init16

_init640200 endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- INIT320200: initialisiert den 320*200 EGA-Grafikmodus bei 16 Farben
;-- Deklaration : void init320200( void );

_init320200 proc near

	   mov   al,0Dh           ;Modus 0Dh  einstellen
	   mov   bx,320/8         ;Zeilenbreite
           mov   cx,( 32000 / 4 ) / 16 ;Page-Offset
           jmp   init

_init320200 endp                   ;Ende der Prozedur

;-- SETPIX: verleiht einem Punkt eine bestimmte Farbe ------------------
;-- Deklaration : void setpix( int x, int y, unsigned char farbe );

_setpix    proc near

sframe     struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
ret_adr0   dw ?                   ;Rcksprungadresse zum Aufrufer
x0         dw ?                   ;X-Ordinate
y0         dw ?                   ;Y-Ordinate
farbe      dw ?                   ;Farbe
sframe     ends                   ;Ende der Struktur

frame      equ [ bp - bp0 ]       ;adressiert die Elemente der Struktur

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   ;-- zunchst Offset in Video-RAM und Shift-Wert berechnen ---

	   mov   ax,frame.y0      ;Y-Koordinate laden
	   mov   dx,zbreite       ;mit Zeilenbreite multiplizieren
	   mul   dx
	   mov   bx,frame.x0      ;X-Koordinate laden
	   mov   cl,bl            ;Lo-Byte fr Shift-Berechnung merken

	   shr   bx,1             ;X-Koordinate durch acht teilen
	   shr   bx,1
	   shr   bx,1
	   add   bx,ax            ;dazu Offset aus Multiplikation add.

	   and   cl,7             ;Bit-Maske aus X-Koordinate berechnen
	   xor   cl,7
	   mov   ah,1
	   shl   ah,cl

	   mov   dx,GC_INDEX      ;Zugriff auf Graphics-Controller
	   mov   al,GC_BIT_MASK   ;Bit-Maske in das Bit-Mask-Register
	   out   dx,ax            ;schreiben

	   mov   ax,(02h shl 8) + GC_GRAPH_MODE;Write-Modus 2 & Read-
	   out   dx,ax                         ;Modus 0 einstellen

	   mov   ax,vio_seg       ;Segmentadresse des Video-RAM inklu-
	   mov   es,ax            ;sive Page nach ES

	   mov   al,es:[bx]               ;Latch-Register laden
	   mov   al,byte ptr frame.farbe  ;Farbwert fr Punkt setzen
	   mov   es:[bx],al               ;Latch-Reg. zurckschreiben

	   ;-- Defaultwerte in den verschiedenen Registern des Graphics-
	   ;-- Controllers einstellen, die verndert wurden

	   mov   ax,(0FFh shl 8 ) + GC_BIT_MASK
	   out   dx,ax

	   mov   ax,(00h shl 8) + GC_GRAPH_MODE
	   out   dx,ax

	   pop   bp
	   ret                    ;zurck zum Aufrufer

_setpix    endp                   ;Ende der Prozedur

;-- GETPIX: liefert die Farbe eines Punktes ----------------------------
;-- Deklaration : unsigned char getpix( int x, int y );

_getpix    proc near

sframe1    struc                  ;Struktur zum Zugriff auf den Stack
bp1        dw ?                   ;nimmt BP auf
ret_adr1   dw ?                   ;Rcksprungadresse zum Aufrufer
x1         dw ?                   ;X-Ordinate
y1         dw ?                   ;Y-Ordinate
sframe1    ends                   ;Ende der Struktur

frame      equ [ bp - bp1 ]       ;adressiert die Elemente der Struktur

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  si

	   ;-- zunchst Offset in Video-RAM und Shift-Wert berechnen ---

	   mov   ax,frame.y1      ;Y-Koordinate laden
	   mov   dx,zbreite       ;mit Zeilenbreite multiplizieren
	   mul   dx
	   mov   si,frame.x1      ;X-Koordinate laden
	   mov   cx,si            ;fr Shift-Berechnung merken

	   shr   si,1             ;X-Koordinate durch acht teilen
	   shr   si,1
	   shr   si,1
	   add   si,ax            ;dazu Offset aus Multiplikation add.

	   and   cl,7             ;Bit-Maske aus X-Koordinate berechnen
	   xor   cl,7
	   mov   ch,1
	   shl   ch,cl

	   mov   ax,vio_seg       ;Segmentadresse des Video-RAM inklu-
	   mov   es,ax            ;sive Page nach ES

	   mov   dx,GC_INDEX      ;Zugriff auf Graphics-Controller
	   mov   ax,(3 shl 8)+ GC_READ_MAP    ;zunchst Plane #3
	   xor   bl,bl                        ;auslesen

gp1:       out   dx,ax            ;zu lesende Bit-Plane angeben
	   mov   bh,es:[si]       ;Wert aus Latch-Register laden
	   and   bh,ch            ;nur gewnschten Pixel stehenlassen
	   neg   bh               ;Bit 7 entsprechend des Pixels setzen
	   rol   bx,1             ;Bit 7 aus BH nach Bit 1 in BL rot.

	   dec   ah               ;nchste Bit-Plane bearbeiten
	   jge   gp1              ;grer oder gleich Null? ---> weiter

	   mov   al,bl            ;Funktionsergebnis nach AL

	   pop   si
	   pop   bp
	   ret                    ;zurck zum Aufrufer

_getpix    endp                   ;Ende der Prozedur

;-- SETPAGE: Whlt die Seite aus, auf die bei Funktionsaufrufen von ----
;--          setpix und getpix zugegriffen werden soll
;-- Deklaration : void setpage( int page );

_setpage   proc near

	   pop   cx               ;Rcksprungadresse vom Stack
	   pop   ax               ;Argument vom Stack

           push  ax               ;alles wieder auf den Stack
	   push  cx
           mul   pageofs          ;Seitennummer mit Page-Ofsfset multip.

           add   ax,0A000h        ;dazu die Basis-Segmentadresse add.
           mov   vio_seg,ax       ;neue Segmentadresse merken

	   ret                    ;zurck zum Aufrufer, Argument wurde
                                  ;bereits vom Stack entfernt

_setpage   endp                   ;Ende der Prozedur

;-- SHOWPAGE: Bringt eine der beiden Bildschirmseiten zur Anzeige ------
;-- Deklaration : void showpage( int page );

_showpage  proc near

	   pop   cx               ;Rcksprungadresse vom Stack
	   pop   ax               ;Argument vom Stack

           push  ax               ;alles wieder auf den Stack
	   push  cx

           mul   pageofs          ;Seitennummer mit Page-Ofsfset multip.
           mov   cl,4             ;das ganze noch mit 16 malnehmen
           shl   ax,cl

           mov   bl,al            ;Lo-Byte merken

           mov   dx,CRTC_INDEX    ;CRT-Controller adressieren
	   mov   al,CC_START_HI   ;zunchst Hi-Byte ausgeben
	   out   dx,ax
           inc   al               ;jetzt Lo-Byte ausgeben
           mov   ah,bl
           out   dx,ax

	   ;-- wieder auf den Start eines Bildschirmaufbaus warten -----

	   mov   dx,VERT_RETRACE  ;zunchst auf Ende des vertikalen
sp3:       in    al,dx            ;Rcklaufs warten
	   test  al,8
	   jne   sp3

sp4:       in    al,dx            ;dann auf den Start des Rcklaufs
	   test  al,8
	   je    sp4

	   ret                    ;zurck zum Aufrufer

_showpage  endp                   ;Ende der Prozedur

;-- GETFONTPTR: liefert FAR-Pointer auf 8*8 Zeichentabelle -------------
;-- Deklaration : void far * getfontptr( void )

_getfontptr proc near

	  push  bp                ;BP sichern

	  mov   ax,1130h          ;Register fr Funktionsaufruf laden
	  mov   bh,3
	  int   10h               ;BIOS-Video-Interrupt aufrufen

	  mov   dx,es             ;Zeiger ES:BP in DX:AX zurckliefern
	  mov   ax,bp

	  pop   bp                ;BP vom Stack zurckholen
	  ret                     ;zurck zum Aufrufer

_getfontptr endp                  ;Ende der Prozedur

;== Ende ===============================================================

_text    ends                     ;Ende des Programm-Segments
         end                      ;Ende des Assembler-Source


