/***********************************************************************
*                           V 3 2 2 0 C . C                            *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Grafikprogrammierung im         *
*                     VGA-320*200-Grafikmodus bei 256 Farben und vier  *
*                     Bildschirmseiten. Das Programm greift dabei auf  *
*                     die Assembler-Routinen aus dem Modul V3220CA.ASM *
*                     zurck.                                          *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    :  4.09.1990                                       *
*  letztes Update am: 14.01.1991                                       *
**--------------------------------------------------------------------**
*  Speichermodell   : SMALL                                            *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung       : CL /AS v3220c.c v3220ca                          *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung       : ber eine Projekt-Datei mit folgendem Inhalt:    *
*                       v3220c.c                                       *
*                       v3220ca.obj                                    *
**--------------------------------------------------------------------**
*  Aufruf           : v3220c                                           *
***********************************************************************/

#include <dos.h>
#include <stdarg.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>

/*-- Typdeklarationen ------------------------------------------------*/

typedef unsigned char BYTE;

/*-- externe Referenzen auf die Assembler-Routinen -------------------*/

extern void init320200( void );
extern void setpix( int x, int y, unsigned char farbe);
extern BYTE getpix( int x, int y );
extern void setpage( BYTE page );
extern void showpage( BYTE page );
extern void far * getfontptr( void );

/*-- Konstanten ------------------------------------------------------*/

#define MAXX 319                        /* Maximale X- und Y-Ordinate */
#define MAXY 199

/***********************************************************************
*  IsVga: Stellt fest, ob eine VGA-Karte installiert ist.              *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : 0, wenn keine VGA angetroffen wurde, sonst ungleich 0     *
***********************************************************************/

BYTE IsVga( void )
{
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */

 Regs.x.ax = 0x1a00;              /* Funktion 1Ah gibt es nur bei VGA */
 int86( 0x10, &Regs, &Regs );
 return ( Regs.h.al == 0x1a );         /* ist die Funktion verfgbar? */
}

/***********************************************************************
*  PrintChar : Baut ein Zeichen auerhalb des sichtbaren Bereichs des  *
*              Video-RAMs auf.                                         *
**--------------------------------------------------------------------**
*  Eingabe :   zeichen = das aufzubauende Zeichen                      *
*              x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************/

void PrintChar( char zeichen, int x, int y, BYTE vg, BYTE hg )
{
 typedef BYTE ZDEF[256][8];                 /* Aufbau des Zeichensatz */
 typedef ZDEF far *ZPTR;                    /* Zeiger auf Zeichensatz */

 BYTE         i, k,                                /* Schleifenzhler */
              Maske;              /* Bitmaske fr Aufbau des Zeichens */

 static ZPTR fptr = (ZPTR) 0;        /* Zeiger auf Zeichensatz im ROM */

 if ( fptr == (ZPTR) 0 )   /* Zeiger auf Zeichensatz schon ermittelt? */
  fptr = getfontptr();  /*Nein, mit Hilfe der Assembler-Funktion laden*/

 /*- das Zeichen Punkt fr Punkt aufbauen ----------------------------*/

 if ( hg == 255 )                    /* Zeichen transparent zeichnen? */
  for ( i = 0; i < 8; ++i )      /* Ja, nur Vordergrund-Punkte setzen */
   {
    Maske = (*fptr)[zeichen][i];    /* Bitmuster fr eine Zeile holen */
    for ( k = 0; k < 8; ++k, Maske <<= 1 ) /* die Spalten durchlaufen */
     if ( Maske & 128 )                              /* Punkt setzen? */
      setpix( x+k, y+i, vg );                                   /* Ja */
   }
 else                                     /* Nein, jeden Punkt setzen */
  for ( i = 0; i < 8; ++i )                     /* Zeilen durchlaufen */
  {
   Maske = (*fptr)[zeichen][i];     /* Bitmuster fr eine Zeile holen */
   for ( k = 0; k < 8; ++k, Maske <<= 1 )  /* die Spalten durchlaufen */
    setpix( x+k, y+i, (BYTE) (( Maske & 128 ) ? vg : hg) );
  }
}

/***********************************************************************
*  Line: Zieht eine Linie nach dem Bresenham-Algorithmus               *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endkoordinate                                    *
*            FARBE  = Farbe der Linie                                  *
***********************************************************************/

/*-- Unterfunktion, vertauscht zwei Integer-Variablen ----------------*/

void SwapInt( int *i1, int *i2 )
{
 int dummy;

 dummy = *i2;
 *i2   = *i1;
 *i1   = dummy;
}

/*-- Hauptteil der Funktion ------------------------------------------*/

void Line( int x1, int y1, int x2, int y2, BYTE farbe )
{
 int d, dx, dy,
     aincr, bincr,
     xincr, yincr,
     x, y;

 if ( abs(x2-x1) < abs(y2-y1) )       /* ber X- oder Y-Achse laufen? */
  {                                               /* ber die Y-Achse */
   if ( y1 > y2 )                                    /* y1 grer y2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   xincr = ( x2 > x1 ) ?  1 : -1;       /* Spalteninkrement festlegen */

   dy = y2 - y1;
   dx = abs( x2-x1 );
   d  = 2 * dx - dy;
   aincr = 2 * (dx - dy);
   bincr = 2 * dx;
   x = x1;
   y = y1;

   setpix( x, y, farbe );                      /* ersten Punkt setzen */
   for (y=y1+1; y<= y2; ++y )         /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       x += xincr;
       d += aincr;
      }
     else
      d += bincr;
     setpix(x, y, farbe);
    }
  }
 else                                             /* ber die X-Achse */
  {
   if ( x1 > x2 )                                    /* x1 grer x2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   yincr = ( y2 > y1 ) ? 1 : -1;         /* Zeileninkrement festlegen */

   dx = x2 - x1;
   dy = abs( y2-y1 );
   d  = 2 * dy - dx;
   aincr = 2 * (dy - dx);
   bincr = 2 * dy;
   x = x1;
   y = y1;

   setpix(x, y, farbe);                        /* ersten Punkt setzen */
   for (x=x1+1; x<=x2; ++x )          /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       y += yincr;
       d += aincr;
      }
     else
      d += bincr;
     setpix(x, y, farbe);
    }
  }
}

/***********************************************************************
*  GrafPrintf: Gibt einen formatierten String im Grafikbildschirm aus  *
**--------------------------------------------------------------------**
*  Eingabe : X, Y   = Startkoordinate (0 - ...)                        *
*            VG     = Vordergrundfarbe                                 *
*            HG     = Hintergrundfarbe (255 = transparent)             *
*            STRING = String mit Formatinformationen                   *
*            ...    = Ausdrcke wie bei Printf                         *
***********************************************************************/

void GrafPrintf( int x, int y, BYTE vg, BYTE hg, char * string, ... )
{
 va_list parameter;              /* Parameter-Liste fr VA_... Macros */
 char ausgabe[255],                 /* Puffer fr formatierten String */
      *cp;

 va_start( parameter, string );                /* Parameter umwandeln */
 vsprintf( ausgabe, string, parameter );               /* formatieren */
 for ( cp = ausgabe; *cp; ++cp, x+= 8 )    /* formatierten String     */
   PrintChar( *cp, x, y, vg, hg );         /* ber PrintChar ausgeben */
}

/***********************************************************************
*  ColorBox: Zeichnet ein Rechteckt und fllt es mit einem Muster aus  *
*            Linien.                                                   *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Koordinate der oberen linken Fensterecke         *
*            X2, Y2 = Koordinate der unteren rechnte Fensterecke       *
*            FARBMAX= grter Farbwert                                 *
*  Info    : Die Farben der Linien werden in einem Zyklus von 0 bis    *
*            FARBMAX gewhlt                                           *
***********************************************************************/

void ColorBox( int x1, int y1, int x2, int y2, int farbmax )
{
 int x, y,                                      /* Schleifenvariablen */
     sx, sy;                 /* Ausgangspunkt fr letzte Farbschleife */

 Line( x1, y1, x1, y2, 15 );              /* Rahmen um die BOX ziehen */
 Line( x1, y2, x2, y2, 15 );
 Line( x2, y2, x2, y1, 15 );
 Line( x2, y1, x1, y1, 15 );

 for ( y = y2-1; y > y1; --y ) /* von links unten an den rechten Rand */
  Line( x1+1, y2-1, x2-1, y, (BYTE) (y % farbmax) );

 for ( y = y2-1; y > y1; --y ) /* von rechts unten an den linken Rand */
  Line( x2-1, y2-1, x1+1, y, (BYTE) (y % farbmax) );

 /*-- von der Boxmitte an den oberen Rand ----------------------------*/

 for ( x=x1+1, sx=x1+(x2-x1)/2, sy=y1+(y2-y1)/ 2; x < x2; ++x )
  Line( sx, sy, x, y1+1, (BYTE) (x % farbmax) );
}

/***********************************************************************
*  DrawAxis: Zeichnet die Achsen am linken und oberen Rand des Bild-   *
*            schirms.                                                  *
**--------------------------------------------------------------------**
*  Eingabe : XSTEP = Schrittweite fr die X-Achse                      *
*            YSTEP = Schrittweite fr die Y-Achse                      *
*            VG    = Vordergrundfarbe                                  *
*            HG    = Hintergrundfarbe (255 = transparent)              *
***********************************************************************/

void DrawAxis( int stepx, int stepy, BYTE vg, BYTE hg )
{
 int x, y;                                      /* Schleifenordinaten */

 Line( 0, 0, MAXX, 0, vg );                         /* X-Achse ziehen */
 Line( 0, 0, 0, MAXY, vg );                         /* Y-Achse ziehen */

 for ( x = stepx; x < MAXX; x += stepx )         /* X-Achse skalieren */
  {
   Line( x, 0, x, 5, vg );
   GrafPrintf( x < 100 ? x - 8 : x - 12, 8, vg, hg, "%d", x );
  }

 for ( y = stepy; y < MAXY; y += stepy )         /* Y-Achse skalieren */
  {
   Line( 0, y, 5, y, vg );
   GrafPrintf( 8, y-4, vg, hg, "%3d", y );
  }
}

/***********************************************************************
*  Demo: Demonstriert den Einsatz der verschiedenen Funktionen aus     *
*        diesem Modul                                                  *
***********************************************************************/

void Demo( void )
{
 #define PAUSE 100000                /* Pausenzhler, Rechnerabhngig */

 int x;                                            /* Ordinatenzhler */
 BYTE seite;                                          /* Seitenzhler */
 long delay;                                          /* Pausenzhler */

 for ( seite = 0; seite < 4; ++seite )
  {
   setpage( seite );                              /* Seite bearbeiten */
   ColorBox( 80, 25, 308, 175, ( seite + 1 ) * 16 ); /* Farbbox malen */
   DrawAxis( 30, 20, 15, 255 );                      /* Axen zeichnen */
   GrafPrintf( 46, MAXY-10, 15, 255,
               "V3220C  -  (c) by MICHAEL TISCHER" );
  }

 /*-- die vier Grafikseiten im Wechsel anzeigen ----------------------*/

 for ( x = 0; x < 50; ++x )                          /* 50 Durchlufe */
  {
   showpage( (BYTE) (x % 4) );                      /* Seite anzeigen */
   for ( delay = 1; delay < PAUSE; ++delay )           /* kurze Pause */
    ;
  }
}

/*--------------------------------------------------------------------*/
/*-- H A U P T P R O G R A M M                                      --*/
/*--------------------------------------------------------------------*/

void main( void )
{
 union REGS regs;

 if ( IsVga() )                             /* VGA-Karte installiert? */
  {                                                  /* Ja, los gehts */
   init320200();                        /* Grafikmodus initialisieren */
   Demo();
   getch();                                       /* auf Taste warten */
   regs.x.ax = 0x0003;          /* wieder in Textmodus zurckschalten */
   int86( 0x10, &regs, &regs );
  }
 else
  printf( "V3220C  -  (c) 1990 by MICHAEL TISCHER\n\nAchtung! Fr "\
          "die Ausfhrung dieses Programms wird eine VGA-Karte "\
          "bentigt.\n\n" );
}


