;**********************************************************************;
;*                         V 3 2 4 0 C A . A S M                      *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : enthlt verschiedene Routinen zur Arbeit im    *;
;*                     320*400-256-Punktemodus der VGA-Karte          *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  :  5.09.1990                                     *;
;*    letztes Update : 14.01.1991                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM /mx V3240CA;    oder   TASM -mx V3240CA   *;
;*                     ... dann mit V3240C.C linken                   *;
;**********************************************************************;

IGROUP group _text                ;Zusammenfassung der Programm-Segmente
DGROUP group _bss,  _data         ;Zusammenfassung der Daten-Segmente
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP

_BSS   segment word public 'BSS'  ;dieses Segment nimmt alle nicht ini-
_BSS   ends                       ;tialisierten statischen Variablen auf

_DATA  segment word public 'DATA' ;alle initialisierten globalen und
                                  ;statischen Variablen werden in diesem
                                  ;Segment untergebracht

_DATA  ends

;== Konstanten =========================================================

SC_INDEX       = 3c4h             ;Indexregister des Sequencer-Ctrl.
SC_MAP_MASK    = 2                ;Nummer des Map-Mask-Registers
SC_MEM_MODE    = 4                ;Nummer des Memory-Mode-Registers

GC_INDEX       = 3ceh             ;Indexregister des Graphics-Ctrl.
GC_READ_MAP    = 4                ;Nummer des Read-Map-Registers
GC_GRAPH_MODE  = 5                ;Nummer des Graphics-Mode-Registers
GC_MISCELL     = 6                ;Nummer des Miscellaneous-Registers

CRTC_INDEX     = 3d4h             ;Indexregistes des CRT-Controllers
CC_MAX_SCAN    = 9                ;Nummer des Maximum-Scan-Line-Reg.
CC_START_HI    = 0Ch              ;Nummer des Hi-Start-Registers
CC_UNDERLINE   = 14h              ;Nummer des Underline-Registers
CC_MODE_CTRL   = 17h              ;Nummer des Mode-Control-Registers

PIXX           = 320              ;horizontale Auflsung

VERT_RETRACE   = 3DAh             ;Input Status-Register #1

;== Daten ==============================================================

_DATA  segment word public 'DATA'

vio_seg    dw 0a000h              ;Video-Segment mit akt. Seite

_DATA  ends

;== Programm ===========================================================

_TEXT  segment byte public 'CODE' ;das Programmsegment

;-- Public-Deklarationen -----------------------------------------------

public    _init320400             ;320*400-Punkte-Modus initialisieren
public    _setpix                 ;Punkt setzen
public    _getpix                 ;Farbe eines Punktes ermitteln
public    _showpage               ;Seite 0 oder 1 anzeigen
public    _setpage                ;Seite fr setpix und getpix festlegen
public    _getfontptr             ;liefert Zeiger auf 8*8-Zeichensatz

;-- INIT320400: initialisiert den 320*400 Punkte-Modus -----------------
;-- Deklaration : void init320400( void );

_init320400 proc near

	   ;-- zunchst ganz normal den Modus 13h einstellen, damit das
	   ;-- BIOS die meiste Arbeit der Initialisierung bernimmt.
	   ;-- Danach die Register verndern, die sich noch nicht in
	   ;-- der gewnschten Stellung befinden.

	   mov   ax,0013h         ;normalen Modus 13h einstellen
	   int   10h

	   mov   dx,GC_INDEX      ;Auftrennung der Speicheradressen
	   mov   al,GC_GRAPH_MODE ;im Graphics-Mode-Register des
	   out   dx,al            ;Gaphics-Controllers ber das
	   inc   dx               ;Bit #4 abschalten
	   in    al,dx
	   and   al,11101111b
	   out   dx,al
	   dec   dx

	   mov   al,GC_MISCELL    ;und das Ganze noch einmal im
	   out   dx,al            ;Miscellanous-Register ber das
	   inc   dx               ;Bit #1
	   in    al,dx
	   and   al,11111101b
	   out   dx,al

	   mov   dx,SC_INDEX      ;das Memory-Mode-Register im
	   mov   al,SC_MEM_MODE   ;Sequencer-Controlller dahingehend
	   out   dx,al            ;modifizieren, da keine Auftrennung
	   inc   dx               ;der Adressen in verschiedene Bit-
	   in    al,dx            ;Planes mehr erfolgt und die aktuelle
	   and   al,11110111b     ;Plane ber das Bit-Mask-Register
	   or    al,4             ;festgelegt wird
	   out   dx,al

	   mov   ax,vio_seg       ;alle vier Bit-Planes komplett mit
	   mov   es,ax            ;dem Farbcode 00h fllen und damit
	   xor   di,di            ;den Bildschirm zunchst lschen
	   mov   ax,di
	   mov   cx,8000h
	   rep   stosw

	   mov   dx,CRTC_INDEX    ;Verdopplung der Punktzeilen im
	   mov   al,CC_MAX_SCAN   ;Maximum-Scan-Line-Register des
	   out   dx,al            ;CRT-Controllers ber das Bit 7
	   inc   dx               ;abschalten, gleichzeitig die
	   in    al,dx            ;Zeichenhhe in den Bits 0-4 auf
	   and   al,01110000b     ;1 einstellen
	   out   dx,al
	   dec   dx               ;DX wieder auf das CRT-Index-Reg

	   mov   al,CC_UNDERLINE  ;Doppelwort-Modus ber das Bit 6
	   out   dx,al            ;im Underline-Register des CRT-
	   inc   dx               ;Controllers abstellen
	   in    al,dx
	   and   al,10111111b
	   out   dx,al
	   dec   dx

	   mov   al,CC_MODE_CTRL  ;ber das Bit 6 im Mode-Control-
	   out   dx,al            ;Register des CRT-Controllers jetzt
	   inc   dx               ;vom Word- in den Byte-Modus wechseln
	   in    al,dx
	   or    al,01000000b
	   out   dx,al

	   ret                    ;zurck zum Aufrufer

_init320400 endp                  ;Ende der Prozedur

;-- SETPIX: verleiht einem Punkt eine bestimmte Farbe ------------------
;-- Deklaration : void setpix( int x, int y, unsigned char farbe );

_setpix    proc near

sframe     struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
ret_adr0   dw ?                   ;Rcksprungadresse zum Aufrufer
x0         dw ?                   ;X-Ordinate
y0         dw ?                   ;Y-Ordinate
farbe      dw ?                   ;Farbe
sframe     ends                   ;Ende der Struktur

frame      equ [ bp - bp0 ]       ;adressiert die Elemente der Struktur

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  di               ;DI auf dem Stack sichern

	   mov   ax,PIXX / 4      ;Offset in den Video-RAM berechnen
	   mul   frame.y0         ;und nach DI laden
	   mov   cx,frame.x0
	   mov   bx,cx
	   shr   bx,1
	   shr   bx,1
	   add   ax,bx
	   mov   di,ax

	   and   cl,3             ;Bit-Maske fr die zu adressierende
	   mov   ah,1             ;Map in AH berechnen
	   shl   ah,cl
	   mov   al,SC_MAP_MASK   ;nach AL die Registernummer
	   mov   dx,SC_INDEX      ;Indexadresse des Sequencers laden
	   out   dx,ax            ;Bit-Mask-Register laden

	   mov   ax,vio_seg       ;ES auf den Video-RAM setzen
	   mov   es,ax
	   mov   al,byte ptr frame.farbe  ;Farbe des Punktes laden
	   stosb                  ;und in die gewhlte Bit-Map schreiben

	   pop   di               ;Register wieder vom Stack holen
	   pop   bp

	   ret                    ;zurck zum Aufrufer

_setpix    endp                   ;Ende der Prozedur

;-- GETPIX: liefert die Farbe eines Punktes ----------------------------
;-- Deklaration : unsigned char getpix( int x, int y );

_getpix    proc near

sframe1    struc                  ;Struktur zum Zugriff auf den Stack
bp1        dw ?                   ;nimmt BP auf
ret_adr1   dw ?                   ;Rcksprungadresse zum Aufrufer
x1         dw ?                   ;X-Ordinate
y1         dw ?                   ;Y-Ordinate
sframe1    ends                   ;Ende der Struktur

frame      equ [ bp - bp1 ]       ;adressiert die Elemente der Struktur

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

	   push  si               ;SI auf dem Stack sichern

	   mov   ax,PIXX / 4      ;Offset in den Video-RAM berechnen
	   mul   frame.y1         ;und nach SI laden
	   mov   si,frame.x1
	   mov   cx,si
	   shr   si,1
	   shr   si,1
	   add   si,ax

	   and   cl,3             ;Bit-Maske fr die zu adressierende
	   mov   ah,cl            ;Map in AH berechnen
	   mov   al,GC_READ_MAP   ;nach AL die Registernummer
	   mov   dx,GC_INDEX      ;Indexadresse des Graphic-Ctrl laden
	   out   dx,ax            ;Read-Map-Register laden

	   mov   ax,vio_seg       ;ES auf den Video-RAM setzen
	   mov   es,ax
	   mov   al,es:[si]       ;Farbe des Punktes laden

	   pop   si               ;Register wieder vom Stack holen
	   pop   bp

	   ret                    ;zurck zum Aufrufer

_getpix    endp                   ;Ende der Prozedur

;-- SETPAGE: Whlt die Seite aus, auf die bei Funktionsaufrufen von ----
;--          setpix und getpix zugegriffen werden soll
;-- Deklaration : void setpage( unsigend char page );

_setpage   proc near

	   pop   bx               ;Rcksprungadresse vom Stack
	   pop   ax               ;Argument vom Stack

	   push  ax               ;und beides wieder rauf
	   push  bx

	   mov   bl,0a0h          ;von Seite 0 ausgehen
	   or    al,al            ;ist es die Seite 0?
	   je    sp1              ;Ja, Segmentadresse merken

	   mov   bl,0a8h          ;Nein, Seite 1

sp1:       mov   byte ptr vio_seg + 1,bl  ;neue Segmentadresse merken

	   ret                    ;zurck zum Aufrufer

_setpage   endp                   ;Ende der Prozedur

;-- SHOWPAGE: Bringt eine der beiden Bildschirmseiten zur Anzeige ------
;-- Deklaration : void showpage( unsigend char page );

_showpage  proc near

	   pop   bx               ;Rcksprungadresse vom Stack
	   pop   ax               ;Argument vom Stack

	   push  ax               ;und beides wieder rauf
	   push  bx

	   or    al,al            ;ist es die Seite 0?
	   je    sp2              ;Ja, Nummer ist auf Hi-Byte des Offset

	   mov   al,80h           ;Nein, Seite 1, mit Offset 8000h

sp2:       mov   dx,CRTC_INDEX    ;CRT-Controller adressieren
	   mov   ah,al            ;Hi-Byte des Offsets nach AH
	   mov   al,CC_START_HI   ;Registernummer nach AL
	   out   dx,ax            ;und ab gehts

	   ;-- wieder auf den Start eines Bildschirmaufbaus warten -----

	   mov   dx,VERT_RETRACE  ;zunchst auf Ende des vertikalen
sp3:       in    al,dx            ;Rcklaufs warten
	   test  al,8
	   jne   sp3

sp4:       in    al,dx            ;dann auf den Start des Rcklaufs
	   test  al,8
	   je    sp4

	   ret                    ;zurck zum Aufrufer

_showpage  endp                   ;Ende der Prozedur

;-- GETFONTPTR: liefert FAR-Pointer auf 8*8 Zeichentabelle -------------
;-- Deklaration : void far * getfontptr( void )

_getfontptr proc near

	  push  bp                ;BP sichern

	  mov   ax,1130h          ;Register fr Funktionsaufruf laden
	  mov   bh,3
	  int   10h               ;BIOS-Video-Interrupt aufrufen

	  mov   dx,es             ;Zeiger ES:BP in DX:AX zurckliefern
	  mov   ax,bp

	  pop   bp                ;BP vom Stack zurckholen
	  ret                     ;zurck zum Aufrufer

_getfontptr endp                  ;Ende der Prozedur

;== Ende ===============================================================

_text    ends                     ;Ende des Programm-Segments
         end                      ;Ende des Assembler-Source


