/***********************************************************************
*                          V 8 0 6 0 C . C                             *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Grafikprogrammierung im 800*    *
*                     600 Punkte-Grafikmodus der Super-VGA-Karten bei  *
*                     16 Farben. Das Programm greift dabei auf die     *
*                     Assembler-Routinen aus dem Modul V8060CA.ASM     *
*                     zurck.                                          *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    : 14.01.1991                                       *
*  letztes Update am: 14.01.1991                                       *
**--------------------------------------------------------------------**
*  Speichermodell   : SMALL                                            *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung       : CL /AS v8060c.c v8060ca                          *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung       : ber eine Projekt-Datei mit folgendem Inhalt:    *
*                       v8060c.c                                       *
*                       v8060ca.obj                                    *
**--------------------------------------------------------------------**
*  Aufruf           : v8060c                                           *
***********************************************************************/

#include <dos.h>
#include <stdarg.h>
#include <stdlib.h>
#include <io.h>
#include <stdio.h>
#include <conio.h>

/*-- Typdeklarationen ------------------------------------------------*/

typedef unsigned char BYTE;

/*-- externe Referenzen auf die Assembler-Routinen -------------------*/

extern int  init800600( void );
extern void setpix( int x, int y, unsigned char farbe);
extern BYTE getpix( int x, int y );
extern void far * getfontptr( void );

/*-- Compiler-abhngige Deklarationen --------------------------------*/

#ifndef __TURBOC__
  #define random(x) ( rand() % (x+1) )             /* Zufallsfunktion */
#endif

/*-- Konstanten ------------------------------------------------------*/

#define MAXX      799                   /* maximale X- und Y-Ordinate */
#define MAXY      599
#define ANZLINE   2500                           /* Anzahl der Linien */
#define XABSTAND  40                 /* Abstand der Line-Box vom Rand */
#define YABSTAND  30
#define X1        ( 2 * XABSTAND )          /* Ordinaten der Line-Box */
#define Y1        ( 2 * YABSTAND )
#define X2        ( MAXX-XABSTAND )
#define Y2        ( MAXY-YABSTAND )
#define XRAND     random( X2 - X1 - 1 ) + X1 + 1         /* Zufalls-  */
#define YRAND     random( Y2 - Y1 - 1 ) + Y1 + 1         /* Ordinaten */

/***********************************************************************
*  IsVga: Stellt fest, ob eine VGA-Karte installiert ist.              *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : 0, wenn keine VGA angetroffen wurde, sonst ungleich 0     *
***********************************************************************/

BYTE IsVga( void )
{
 union REGS Regs;            /* Prozessorregister fr Interruptaufruf */

 Regs.x.ax = 0x1a00;              /* Funktion 1Ah gibt es nur bei VGA */
 int86( 0x10, &Regs, &Regs );
 return (BYTE) ( Regs.h.al == 0x1a );  /* ist die Funktion verfgbar? */
}

/***********************************************************************
*  PrintChar : Baut ein Zeichen auerhalb des sichtbaren Bereichs des  *
*              Video-RAMs auf.                                         *
**--------------------------------------------------------------------**
*  Eingabe :   zeichen = das aufzubauende Zeichen                      *
*              x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************/

void PrintChar( char zeichen, int x, int y, BYTE vg, BYTE hg )
{
 typedef BYTE ZDEF[256][8];                 /* Aufbau des Zeichensatz */
 typedef ZDEF far *ZPTR;                    /* Zeiger auf Zeichensatz */

 BYTE         i, k,                                /* Schleifenzhler */
              Maske;              /* Bitmaske fr Aufbau des Zeichens */

 static ZPTR fptr = (ZPTR) 0;        /* Zeiger auf Zeichensatz im ROM */

 if ( fptr == (ZPTR) 0 )   /* Zeiger auf Zeichensatz schon ermittelt? */
  fptr = getfontptr();  /*Nein, mit Hilfe der Assembler-Funktion laden*/

 /*- das Zeichen Punkt fr Punkt aufbauen ----------------------------*/

 if ( hg == 255 )                    /* Zeichen transparent zeichnen? */
  for ( i = 0; i < 8; ++i )      /* Ja, nur Vordergrund-Punkte setzen */
   {
    Maske = (*fptr)[zeichen][i];    /* Bitmuster fr eine Zeile holen */
    for ( k = 0; k < 8; ++k, Maske <<= 1 ) /* die Spalten durchlaufen */
     if ( Maske & 128 )                              /* Punkt setzen? */
      setpix( x+k, y+i, vg );                                   /* Ja */
   }
 else                                     /* Nein, jeden Punkt setzen */
  for ( i = 0; i < 8; ++i )                     /* Zeilen durchlaufen */
  {
   Maske = (*fptr)[zeichen][i];     /* Bitmuster fr eine Zeile holen */
   for ( k = 0; k < 8; ++k, Maske <<= 1 )  /* die Spalten durchlaufen */
    setpix( x+k, y+i, ( Maske & 128 ) ? vg : hg );
  }
}

/***********************************************************************
*  Line: Zieht eine Linie nach dem Bresenham-Algorithmus               *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endkoordinate                                    *
*            FARBE  = Farbe der Linie                                  *
***********************************************************************/

/*-- Unterfunktion, vertauscht zwei Integer-Variablen ----------------*/

void SwapInt( int *i1, int *i2 )
{
 int dummy;

 dummy = *i2;
 *i2   = *i1;
 *i1   = dummy;
}

/*-- Hauptteil der Funktion ------------------------------------------*/

void Line( int x1, int y1, int x2, int y2, BYTE farbe )
{
 int d, dx, dy,
     aincr, bincr,
     xincr, yincr,
     x, y;

 if ( abs(x2-x1) < abs(y2-y1) )       /* ber X- oder Y-Achse laufen? */
  {                                               /* ber die Y-Achse */
   if ( y1 > y2 )                                    /* y1 grer y2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   xincr = ( x2 > x1 ) ?  1 : -1;       /* Spalteninkrement festlegen */

   dy = y2 - y1;
   dx = abs( x2-x1 );
   d  = 2 * dx - dy;
   aincr = 2 * (dx - dy);
   bincr = 2 * dx;
   x = x1;
   y = y1;

   setpix( x, y, farbe );                      /* ersten Punkt setzen */
   for (y=y1+1; y<= y2; ++y )         /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       x += xincr;
       d += aincr;
      }
     else
      d += bincr;
     setpix(x, y, farbe);
    }
  }
 else                                             /* ber die X-Achse */
  {
   if ( x1 > x2 )                                    /* x1 grer x2? */
    {
     SwapInt( &x1, &x2 );                    /* Ja, X1 mit X2 und     */
     SwapInt( &y1, &y2 );                    /* Y1 mit Y2 vertauschen */
    }

   yincr = ( y2 > y1 ) ? 1 : -1;         /* Zeileninkrement festlegen */

   dx = x2 - x1;
   dy = abs( y2-y1 );
   d  = 2 * dy - dx;
   aincr = 2 * (dy - dx);
   bincr = 2 * dy;
   x = x1;
   y = y1;

   setpix(x, y, farbe);                        /* ersten Punkt setzen */
   for (x=x1+1; x<=x2; ++x )          /* Line auf X-Achse durchlaufen */
    {
     if ( d >= 0 )
      {
       y += yincr;
       d += aincr;
      }
     else
       d += bincr;
     setpix(x, y, farbe);
    }
  }
}

/***********************************************************************
*  GrafPrintf: Gibt einen formatierten String im Grafikbildschirm aus  *
**--------------------------------------------------------------------**
*  Eingabe : X, Y   = Startkoordinate (0 - ...)                        *
*            VG     = Vordergrundfarbe                                 *
*            HG     = Hintergrundfarbe (255 = transparent)             *
*            STRING = String mit Formatinformationen                   *
*            ...    = Ausdrcke wie bei Printf                         *
***********************************************************************/

void GrafPrintf( int x, int y, BYTE vg, BYTE hg, char * string, ... )
{
 va_list parameter;              /* Parameter-Liste fr VA_... Macros */
 char ausgabe[255],                 /* Puffer fr formatierten String */
      *cp;

 va_start( parameter, string );                /* Parameter umwandeln */
 vsprintf( ausgabe, string, parameter );               /* formatieren */
 for ( cp = ausgabe; *cp; ++cp, x+= 8 )    /* formatierten String     */
   PrintChar( *cp, x, y, vg, hg );         /* ber PrintChar ausgeben */
}

/***********************************************************************
*  DrawAxis: Zeichnet die Achsen am linken und oberen Rand des Bild-   *
*            schirms.                                                  *
**--------------------------------------------------------------------**
*  Eingabe : XSTEP = Schrittweite fr die X-Achse                      *
*            YSTEP = Schrittweite fr die Y-Achse                      *
*            VG    = Vordergrundfarbe                                  *
*            HG    = Hintergrundfarbe (255 = transparent)              *
***********************************************************************/

void DrawAxis( int stepx, int stepy, BYTE vg, BYTE hg )
{
 int x, y;                                      /* Schleifenordinaten */

 Line( 0, 0, MAXX, 0, vg );                         /* X-Achse ziehen */
 Line( 0, 0, 0, MAXY, vg );                         /* Y-Achse ziehen */

 for ( x = stepx; x < MAXX; x += stepx )         /* X-Achse skalieren */
  {
   Line( x, 0, x, 5, vg );
   GrafPrintf( x < 100 ? x - 8 : x - 12, 8, vg, hg, "%d", x );
  }

 for ( y = stepy; y < MAXY; y += stepy )         /* Y-Achse skalieren */
  {
   Line( 0, y, 5, y, vg );
   GrafPrintf( 8, y-4, vg, hg, "%3d", y );
  }
}

/***********************************************************************
*  Demo: Demonstriert den Einsatz der verschiedenen Funktionen aus     *
*        diesem Modul                                                  *
***********************************************************************/

void Demo( void )
{
 int i;                                            /* Schleifenzhler */

 DrawAxis( 30, 20, 15, 255 );                        /* Axen zeichnen */

 GrafPrintf( X1, MAXY-10, 15, 255,
             "V8060C.C  -  (c) by MICHAEL TISCHER" );

 Line( X1, Y1, X1, Y2, 15 );         /* Rahmen um die Line-Box ziehen */
 Line( X1, Y2, X2, Y2, 15 );
 Line( X2, Y2, X2, Y1, 15 );
 Line( X2, Y1, X1, Y1, 15 );

 /*-- zufllige Linien innerhalb der Line-Box erzeugen ---------------*/

 for ( i = ANZLINE; i > 0 ; --i )
  Line( XRAND, YRAND, XRAND, YRAND, (BYTE) (i % 16) );
}

/*--------------------------------------------------------------------*/
/*-- H A U P T P R O G R A M M                                      --*/
/*--------------------------------------------------------------------*/

void main( void )
{
 union REGS regs;

 printf( "V8060C.C  - (c) 1991 by MICHAEL TISCHER\n\n" );
 if ( IsVga() )                             /* VGA-Karte installiert? */
  {             /* Ja, aber kann der Modus auch initialisiert werden? */
   if ( init800600() )
    {                                                     /* Modus ok */
     Demo();                                        /* Demo ausfhren */
     getch();                                     /* auf Taste warten */
     regs.x.ax = 0x0003;        /* wieder in Textmodus zurckschalten */
     int86( 0x10, &regs, &regs );
    }
   else
    {
     printf( "Achtung! 800*600-Modus konnte nicht " \
             "initialisiert werden!\n" );
     exit( 1 );
    }
  }
 else
  {                                          /* Nein, keine VGA-Karte */
   printf( "Achtung! Fr die Ausfhrung dieses Programms " \
           "\nwird eine VGA-Karte bentigt!\n" );
   exit(1);                                       /* Programm beenden */
  }
}
