/***********************************************************************
*                           V O N O F F C . C                          *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Ab- und Anschaltung der Bild-   *
*                     schirmdarstellung bei EGA- und VGA-Karten        *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    : 26.08.1990                                       *
*  letztes Update am: 14.01.1991                                       *
**--------------------------------------------------------------------**
*  (MICROSOFT C)                                                       *
*  Erstellung       : CL /AS vonoffc.c                                 *
**--------------------------------------------------------------------**
*  (BORLAND TURBO C)                                                   *
*  Erstellung       : in der integrierten Entwicklungsumgebung         *
***********************************************************************/

#include <dos.h>                         /* Include-Dateien einbinden */
#include <conio.h>
#include <stdio.h>

#ifdef __TURBOC__                        /* Kompilierung mit Turbo C? */
  #define CLI()         disable()
  #define STI()         enable()
  #define outpw( p, w ) outport( p, w )
  #ifndef inp
    #define outp( p, b )  outportb( p, b )
    #define inp( p )      inportb( p )
  #endif
#else                                /* Nein, mit QuickC 2.0 oder MSC */
  #include <conio.h>
  #define MK_FP(seg,ofs) ((void far *)\
                         (((unsigned long)(seg) << 16) | (ofs)))
  #define CLI()          _disable()
  #define STI()          _enable()
#endif

/*-- Konstanten ------------------------------------------------------*/

#define EV_STATC 0x3DA        /* EGA-/VGA-Karten Statusregister Color */
#define EV_STATM 0x3BA         /* EGA-/VGA-Karten Statusregister Mono */
#define EV_ATTR  0x3C0         /* EGA-/VGA-Karten Attribut-Controller */

/***********************************************************************
*  ScrOff : Schaltet den Bildschirm bei EGA- und VGA-Karten ab.        *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************/

void ScrOff( void )
{
 CLI();                                     /* keine Interrupts jetzt */
 inp( EV_STATC );                       /* Reset auf Status-Reg Color */
 inp( EV_STATM );                        /* Reset auf Status-Reg Mono */
 outp( EV_ATTR, 0x00 );       /* Bit 5 ausblenden, dadurch Verbindung */
                        /* zum CRT-Controller unterbinden             */
 STI();                                       /* Interrupts wieder an */
}

/***********************************************************************
*  ScrOn : Schaltet den Bildschirm bei EGA- und VGA-Karten an.         *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************/

void ScrOn( void )
{
 CLI();                                     /* keine Interrupts jetzt */
 inp( EV_STATC );                       /* Reset auf Status-Reg Color */
 inp( EV_STATM );                        /* Reset auf Status-Reg Mono */
 outp( EV_ATTR, 0x20 );      /* Bit 5 einblenden, dadurch Verbindung  */
                        /* zum CRT-Controller wieder herstellten      */
 STI();                                       /* Interrupts wieder an */
}

/***********************************************************************
*  IsEgaVga : Stellt fest, ob eine EGA- oder VGA-Karte installiert ist.*
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : TRUE, wenn EGA- oder VGA-KArte, sonst FALSE               *
***********************************************************************/

int IsEgaVga( void )
{
 union REGS Regs;                     /* Register zum Interruptaufruf */

 Regs.x.ax = 0x1a00;              /* Funktion 1Ah gibt es nur bei VGA */
 int86( 0x10, &Regs, &Regs );
 if ( Regs.h.al == 0x1a )              /* ist die Funktion verfgbar? */
  return 1;
 else
  {
   Regs.h.ah = 0x12;                    /* Funktion 12h Unterfunktion */
   Regs.h.bl = 0x10;                    /* 10h aufrufen               */
   int86(0x10, &Regs, &Regs);                  /* Video-BIOS aufrufen */
   return ( Regs.h.bl != 0x10 );
  }
}

/***********************************************************************
*  Delay : Realisiert eine Zeitverzgerung auf der Basis des BIOS      *
*          Timers.                                                     *
**--------------------------------------------------------------------**
*  Eingabe : Lnge der Verzgerung in Sekunden                         *
*  Ausgabe : keine                                                     *
***********************************************************************/

void Delay( int pauslen )
{
 unsigned int zeit_hi,                                  /* Zeitzhler */
              zeit_lo,
              ticks;
 union REGS   inregs,                            /* Prozessorregister */
              outregs;

 ticks = pauslen * 182 / 10;
 inregs.h.ah = 0;               /* Funktion 00h = Zeitzhler auslesen */
 int86( 0x1a, &inregs, &outregs );           /* Zeit holen und merken */
 zeit_hi = outregs.x.cx;
 zeit_lo = outregs.x.dx;

 while ( ticks )                      /* wiederholen, bis ticks auf 0 */
  {                                 /* heruntergezhlt wurde          */
   int86( 0x1a, &inregs, &outregs );                    /* Zeit holen */

   /*-- neuer Tick angebrochen? --------------------------------------*/

   if ( zeit_hi != outregs.x.cx  ||  zeit_lo != outregs.x.dx )
    {                                                           /* Ja */
     zeit_hi = outregs.x.cx;      /* neue Werte der Zeitzhler merken */
     zeit_lo = outregs.x.dx;
     --ticks;            /* Anzahl verbleibender Ticks dekrementieren */
    }
  }
}

/**********************************************************************/
/*-- H A U P T P R O G R A M M                                      --*/
/**********************************************************************/

void main( void )                             /* nimmt eine Taste auf */
{
 int i;                                            /* Schleifenzhler */

 for ( i=0; i<25; ++i )                         /* Bildschirm lschen */
  printf( "\n" );

 printf( "VONOFFC  -  (c) 1990 by MICHAEL TISCHER\n\n" );
 if ( IsEgaVga() )                            /* EGA- oder VGA-Karte? */
  {                                                      /* Ja, do it */
   printf( "Achtung! Nach fnf Sekunden wird der Bildschirm " \
           "abgeschaltet.\nBettigen Sie dann eine beliebige Taste" \
           ", um ihn wieder anzuschalten..." );
   Delay( 5 );                                /* fnf Sekunden warten */
   while ( kbhit() )  /* alle Tasten aus dem Tastaturpuffer entfernen */
     getch();
   ScrOff();                                        /* Bildschirm aus */
   getch();                                       /* auf Taste warten */
   ScrOn();                                   /* Bildschirm wieder an */
   printf( "\n\n\nDas war's ....\n" );
  }
 else                                     /* Nein, keine EGA oder VGA */
  printf( "Achtung! Dieses Programm bentigt eine " \
          "EGA- oder VGA-Karte.\n" );
}
