/**********************************************************************/
/*                          W I N D A C . C                           */
/*--------------------------------------------------------------------*/
/*    Aufgabe       : Feststellen ob und in welchem Modus Windows     */
/*                    aktiv ist.                                      */
/*--------------------------------------------------------------------*/
/*    Autor         : Michael Tischer                                 */
/*    entwickelt am : 22.08.1991                                      */
/*    letztes Update: 22.10.1991                                      */
/*--------------------------------------------------------------------*/
/*    Speichermodell: SMALL                                           */
/**********************************************************************/
 
/*== Include-Dateien einbinden =======================================*/

#include <dos.h>
#include <stdio.h>

/*== Konstanten ======================================================*/

#define MULTIPLEX    0x2F /* Interrupt-Nummer des Multiplex-Interrupt */

#define NO_WIN       0x00                      /* Windows nicht aktiv */
#define W_386_X      0x01                  /* Windows /386 V2.x luft */
#define W_REAL       0x81              /* Windows luft im Real-Modus */
#define W_STANDARD   0x82          /* Windows luft im Standard-Modus */
#define W_ENHANCED   0x83       /* Windows luft im Erweiterten-Modus */

/**********************************************************************/
/* windows : stellt fest ob Windows aktiv ist                         */
/* Eingabe : HVERSION = Zeiger auf INT-Variable, die die Haupt-       */
/*                      versionsnummer aufnehmen soll                 */
/*           NVERSION = Zeiger auf INT-Variable, die die Unter-       */
/*                      versionsnummer aufnehmen soll                 */
/* Ausgabe : Windows-Status, eine der Konstanten NO_WIN, W_386_X,     */
/*           W_STANDARD, W_STANDARD oder W_ENHANCED                   */
/* Info    : Nur im Enhanced-Modus von Windows 3 kann die Versions-   */
/*           nummer ermittelt und zurckgeliefert werden              */
/**********************************************************************/

int windows( int *HVersion, int *NVersion )
{
 union  REGS  regs;                  /* Register fr Interrupt-Aufruf */
 struct SREGS sregs;          /* Segmentregister fr Interrupt-Aufruf */

 *HVersion = 0;                       /* Initialisiere Versionsnummer */
 *NVersion = 0;

 /*-- Identifiziere Windows x.y im Erweiterten-Modus -----------------*/

 regs.x.ax = 0x1600;                    /* Installationscheck Windows */
 segread( &sregs );                          /* Segmentregister lesen */
 int86x( MULTIPLEX, &regs, &regs, &sregs );

 switch ( regs.h.al )
 {
  case 0x01:
  case 0xFF:  *HVersion = 2;                          /* Hauptversion */
              *NVersion = 0;                /* Nebenversion unbekannt */
              return W_386_X;             /* Windows /386 Version 2.x */

  case 0x00:
  case 0x80:  regs.x.ax = 0x4680;           /* Real- u. Standardmodus */
              int86x( MULTIPLEX, &regs, &regs, &sregs );
              if ( regs.h.al == 0x80 )
               return NO_WIN;                  /* Windows luft nicht */
              else
              {
               /*-- Windows im Real- oder Standardmodus --------------*/

               regs.x.ax = 0x1605;      /* Initialisierung simulieren */
               regs.x.bx = regs.x.si = regs.x.cx =
                           sregs.es = sregs.ds = 0x0000;
               regs.x.dx = 0x0001;
               int86x( MULTIPLEX, &regs, &regs, &sregs );
               if ( regs.x.cx == 0x0000 )
               {
                /*-- Windows im Real-Modus ---------------------------*/

                regs.x.ax = 0x1606;
                int86x( MULTIPLEX, &regs, &regs, &sregs );
                return W_REAL;
               }
               else
                return W_STANDARD;
              }

  /*-- Windows im Erweiterten-Modus, ax enthlt Versionsnummer -------*/

  default:  *HVersion = regs.h.al;        /* Windows Version ausgeben */
            *NVersion = regs.h.ah;
            return W_ENHANCED;        /* Windows im Erweiterten-Modus */
 }
}

/**********************************************************************/
/*                     H A U P T P R O G R A M M                      */
/**********************************************************************/

int main( void )
{
 int WindowsAktiv,                                   /* Windows-Modus */
     HVer,                                /* Hauptversion von Windows */
     NVer;                                /* Nebenversion von Windows */

 printf(" WINDAC  -  (c) 1991 by Michael Tischer \n\n");
 WindowsAktiv = windows( &HVer, &NVer );
 switch ( WindowsAktiv )
 {
  case NO_WIN:     printf( "Windows nicht aktiv\n" );
                   break;
  case W_REAL:     printf( "Windows aktiv im Real-Modus\n" );
                   break;
  case W_STANDARD: printf( "Windows aktiv im Standard-Modus\n" );
                   break;
  case W_386_X:    printf( "Windows/386 V 2.x aktiv" );
                   break;
  case W_ENHANCED: printf( "Windows V %d.%d aktiv im %s\n",
                           HVer, NVer, "erweiterten Modus" );
                   break;
 }
 return( WindowsAktiv );
}
