;**********************************************************************;
;*                           X M S C A . A S M                        *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : Assemblerroutine zur Einbindung in das C-      *;
;*                     Programm XMSC.C. Stellt eine Routine zum Auf-  *;
;*                     ruf des XMS-Treibers zur Verfgung.            *;
;*                     Implementierung hier fr das Speichermodell    *;
;*                     SMALL.                                         *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  : 27.07.1990                                     *;
;*    letztes Update : 27.07.1990                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM XMSCA;   oder   TASM XMSCA;               *;
;*                     ... dann mit dem compilierten C-Programm       *;
;*                         XMSC.C verbinden                           *;
;**********************************************************************;

;== Segmentdeklarationen fr das C-Programm ============================

IGROUP group _text                ;Zusammenfassung der Programm-Segmente
DGROUP group _bss,  _data         ;Zusammenfassung der Daten-Segmente
       assume CS:IGROUP, DS:DGROUP, ES:DGROUP, SS:DGROUP

_BSS   segment word public 'BSS'  ;dieses Segment nimmt alle nicht ini-
_BSS   ends                       ;tialisierten statischen Variablen auf

_DATA  segment word public 'DATA' ;alle initialisierten globalen und
                                  ;statischen Variablen werden in diesem
                                  ;Segment untergebracht

extrn _XMSPtr : dword            ;Referenz auf den XMS-Zeiger

_DATA  ends

;== Programm ===========================================================

_TEXT  segment byte public 'CODE' ;das Programmsegment

public     _XMSCall

;-----------------------------------------------------------------------
;-- XMSCall : Allgmeine Routine zum Aufruf einer XMS-Funktion
;-- Aufruf von C: void XMSCall( BYTE FktNr, XMSRegs *Xr )    mit
;--             typedef struct { WORD AX, BX, DX, SI, Segment } XMSRegs;
;-- Return-Wert : keiner
;-- Info    : - Es mssen vor dem Aufruf dieser Prozedur immer nur die
;--             Register geladen werden, die fr den Aufruf der angege-
;--             benen Funktion tatschlich bentigt werden.
;--           - Nach dem XMS-Funktionsaufruf werden die Inhalte der ver-
;--             schiedenen Prozessorregister in die korrespondierenden
;--             Komponenten der bergebenen Struktur kopiert.
;--           - Vor dem ersten Aufruf dieser Prozedur mu die Prozedur
;--             XMSInit erfolgreich aufgerufen worden sein.

_XMSCall   proc near
                                                                        
sframe     struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
ret_adr    dw ?                   ;Rcksprungadresse zum Aufrufer
fktnr      dw ?                   ;die Nummer der XMS-Funktion
xrptr      dw ?                   ;Zeiger auf die Register-Struktur
sframe     ends                   ;Ende der Struktur
                                                                        
frame      equ [ bp - bp0 ]       ;adressiert die Elemente der Struktur
                                                                        
           push   bp                ;BP auf dem Stack sichern
           mov    bp,sp             ;SP nach BP bertragen
                                                                        
           push   si                ;SI und DI sichern
           push   di

	   mov    cx,ds                   ;DS in CX merken
	   push   cx                      ;und auf dem Stack sichern
	   mov    di,frame.xrptr          ;Funktionsnr. laden
	   mov    ah,byte ptr frame.fktnr ;Zeiger auf Struktur laden
	   mov    bx,[di+2]               ;Register aus den Komponenten
	   mov    dx,[di+4]               ;der Struktur laden
           mov    si,[di+6]
           mov    ds,[di+8]
	   mov    es,cx                   ;ES mit DS laden
	   call   es:[_XMSPtr]            ;XMS-Handler aufrufen
	   mov    cx,ds                   ;DS in CX merken
	   pop    ds                      ;altes DS zurckholen
	   mov    di,frame.xrptr          ;Zeiger auf Struktur laden
	   mov    [di],ax                 ;Register in die zugehrigen
	   mov    [di+02],bx              ;Komponenten der Struktur
	   mov    [di+04],dx              ;eintragen
           mov    [di+06],si
           mov    [di+08],cx

           pop    di              ;SI und DI zurckholen
           pop    si

           pop    bp              ;BP wieder vom Stack holen
           ret                    ;zurck zum C-Programm

_XMSCall   endp

;-------------------------------------------------------------------------
                                                                        
_text      ends                   ;Ende des Codesegments
           end                    ;Ende des Programms
