{********************************************************************}
{                          A R G S . P A S                           }
{*------------------------------------------------------------------*}
{ Aufgabe        : Funktionen zur Bearbeitung von Kommandozeilen-    }
{                  Parametern                                        }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 20.03.1994                                        }
{ letztes Update :  6.10.1994                                        }
{********************************************************************}

Unit ARGS;

Interface
Const
 _char      = 0;                            { Typen-Codes fr GetArg }
 _int       = 1;
 _long      = 2;
 _String    = 3;
 _none      = 4;
 _maxparams = 20;         { maximale Anzahl Parameter fr NArgString }

type NArgStrings = Array[0.._maxparams-1] of string;

Function GetArg( Prefix  : String;
                 iType   : Integer;
                 pVar    : Pointer;
                 iNumEl  : Integer ) : Integer;

Function GetNArg( Prefix      : String;
                  var Strings : NArgStrings ) : Integer;

Function FindString( var Strings : NArgStrings;
                         Find    : String;
                         Anzahl  : integer) : Integer;

Function htoi( Hstring : String;
               iDef    : Integer) : Integer;

Function strichr( SuchStr   : String;
                  SearchStr : String ) : String;

Function Up( s : String ) : String;

Implementation
Uses DOS;

{********************************************************************}
{ Up: Wandele String in Grobuchstaben                               }
{*-------------------------------------------------------------------}
{ Eingabe : s - der umzuwandelnde String                             }
{ Ausgabe : Der umgewandelte String                                  }
{********************************************************************}
Function Up( s : String) : String;
var i : Integer;

Begin
 for i := 1 to Length( s ) do s[i] := Upcase( s[i] );
 Up:=s;
End;

{********************************************************************}
{ GetArg : Ermittle Kommandozeilenparameter                          }
{*-------------------------------------------------------------------}
{ Eingabe : Prefix - Parameter-Prefix (z.B: "-o" fr Output)         }
{           iType  - Typ des Parameters (_char, _int, _long, _String }
{                     _none = testet ob Parameter vorhanden)         }
{           pVar   - Adresse der Variablen, die Kommandozeilen-      }
{                    parameterwert entgegennehmen soll.              }
{           iNumEl - Falls mehrere Werte durch ',' getrennt werden   }
{                    wird bis zu iNumElements Parameter im in pVar   }
{                    angegebenen Array gespeichert.                  }
{*-------------------------------------------------------------------}
{ Ausgabe : Anzahl der ermittelten Werte, oder 0 falls Kommando-     }
{           parameter nicht vorhanden.                               }
{*-------------------------------------------------------------------}
{ Info    : Soll ein Stringargument ermittelt werden, mu fr den    }
{           den Parameter pVar die Adresse eines Strings angegeben   }
{           werden, sofern iNumElements = 1 ist. Ist iNumElements    }
{           grer, wird ein Zeiger auf eine Variable vom Typ        }
{           NArgStrings erwartet!                                    }
{********************************************************************}
Function GetArg( Prefix  : String;
                 iType   : Integer;
                 pVar    : Pointer;
                 iNumEl  : Integer ):Integer;

Var i, j,
    iLen  : Integer;
    Ende  : Boolean;
    cPtr  : ^Char;
    iPtr  : ^Integer;
    Ptr   : ^Longint;
    sPtr  : ^String;
    arg   : string;
    ArgAr : ^NArgStrings;

Begin
  iLen := Length( Prefix );
  GetArg := 0;

  i := 1;
  while ( i <= ParamCount ) and
        ( Pos( Up( Prefix ), Up( ParamStr(i) ) ) <> 1 ) do
    i := i + 1;

  if i <= ParamCount then
    begin                                        { Argument gefunden }
      arg := ParamStr(i);
      case iType of
        _int:
        Begin
          iPtr := pVar;
          j := 0;
          Repeat
            iptr^ := 0;
            Inc( j );
            While ( arg[ilen+1] in
                    ['1','2','3','4','5','6','7','8','9','0'])
              and ( ilen < length( arg ) ) do
              Begin
                Inc( iLen );
                iPtr^ := iptr^ * 10;
                iPtr^ := iPtr^ + Ord( arg[iLen]) - Ord( '0' )
              end;
            if ( ( arg[ilen+1] = ',' ) and
               ( iLen < Length( arg ) ) and
               ( j < iNumEl ) ) then
            Begin Inc(iptr); Inc(iLen); Ende:=FALSE; End
            else Ende:=TRUE;
          Until Ende;
          GetArg := j;
        End;

        _char:
        Begin
          cPtr := pVar;
          if iLen < Length( arg ) then cPtr^ := arg[iLen+1]
            else cptr^:=#0;
          GetArg:=1;
        End;

        _long:
        Begin
          j := 1;
          Ptr := pVar;
          j := 0;
          Repeat
            ptr^ := 0;
            inc( j );
            while ( arg[iLen+1] in
                    ['1','2','3','4','5','6','7','8','9','0'] )
              and ( ilen < Length( arg ) ) do Begin
              Inc( ilen );
              Ptr^ := Ptr^ * 10;
              Ptr^ := Ptr^ + Ord( arg[iLen] ) - Ord( '0' )
            End;
            if ( ( arg[iLen+1] = ',' ) and
               ( iLen < Length( arg ) ) and
               ( j < iNumEl ) ) then
            Begin inc( Ptr ); inc( iLen ); Ende:=FALSE; End
            else Ende:=TRUE;
          Until Ende;
          GetArg := j;
        End;

        _String:
        Begin
          if iNumEl = 1 then
            begin
              sPtr := pVar;
              Repeat
                Inc( iLen );
                sPtr^ := sPtr^ + arg[iLen];
              Until iLen = Length( arg );
              GetArg:=1;
            end
          else
            begin
              ArgAr := pVar;
              j := 0;
              ArgAr^[j] := '';
              Inc( iLen );
              Repeat
                ArgAr^[j] := ArgAr^[j] + arg[iLen];
                Inc( iLen );
                if iLen <  Length( arg ) then
                  if arg[iLen] = ',' then
                    begin
                      inc( j );
                      ArgAr^[j] := '';
                      inc( iLen )
                    end;
              Until (iLen > Length( arg )) or (j = iNumEl);
              if(Length( ArgAr^[j] )>0) then Inc(j);
              GetArg:=j;
            end;
        end;

        _none:
          GetArg := -1;
      End;
   End;
End;

{********************************************************************}
{ GetNArg : Ermittle alle Kommandozeilenparameter, denen kein Para-  }
{           meter-Prefix-Zeichen vorangeht.                          }
{*-------------------------------------------------------------------}
{ Eingabe : Prefix  - Parameter-Prefix (z.B: "-o" fr Output). Alle  }
{                     Parameter, die nicht mit diesem Prefix begin-  }
{                     nen, werden zurckgeliefert.                   }
{           Strings - String-Array, in das die einzelnen Parameter-  }
{                     Strings eingetragen werden.                    }
{ Ausgabe : Anzahl der ermittelten Parameter, oder 0 falls Kommando- }
{           parameter nicht vorhanden.                               }
{********************************************************************}
Function GetNArg(     Prefix   : String;
                  var Strings  : NArgStrings ) : Integer;

Var i, j, iLen : Integer;

Begin
  iLen := Length( Prefix );
  i := 1;
  j := 0;
  While ( ( i <= ParamCount ) and ( j < _maxparams ) ) do
  Begin
    if( Pos( Prefix, ParamStr(i) ) = 0 ) then
    Begin
      Strings[j] := ParamStr(i);
      Inc( j );
    End;
    Inc( i );
  End;
  GetNArg := j;
End;

{********************************************************************}
{ FindString : Suche String in einem String-Array und gib Position   }
{              des gefundenen Strings innerhalb des Arrays zurck.   }
{*------------------------------------------------------------------*}
{ Eingabe : Strings - Array mit den zu durchsuchenden Strings        }
{           Find    - zu suchender String                            }
{           Anzahl  - Anzahl der Array-Eintrge, die gefllt sind    }
{ Ausgabe : Position + 1 des gefundenen Strings in Array oder 0      }
{           falls SuchString nicht in Array vorhanden.               }
{********************************************************************}
Function FindString( var Strings : NArgStrings;
                         Find    : String;
                         Anzahl  : integer ) : Integer;

Var i : Integer;

Begin
  FindString := 0;
  for i := 0 to Anzahl - 1 do
   if Up( Find ) = Up( Strings[i] ) then FindString := i + 1;
End;

{********************************************************************}
{ htoi : Wandelt bergebenen Hex-String in Zahlenwert                }
{*------------------------------------------------------------------*}
{ Eingabe : Hstring - der umzuwandelnde String                       }
{           iDef    - Vorgabewert                                    }
{ Ausgabe : Wert des Strings oder 'iDef', falls bergebener String   }
{           keine gltige Hex-Zahl darstellt.                        }
{********************************************************************}
Function htoi( Hstring : String; iDef : Integer) : Integer;

Var iNumChars : Integer;
    iVal,i    : Integer;

Begin
  iNumChars := 0;
  iVal := 0;
  if ( Hstring <> '' ) then
  Begin
    i := 1;
    While Hstring[i] = ' ' do Inc( i );
    While i <= Length( Hstring ) do
    Begin
      case Hstring[i] of
  	 '0'..'9':                          { Dezimalziffern wandeln }
	 Begin
	   iVal := iVal * $10;
                              { Wert eine stelle nach links schieben }
	   iVal := iVal + Integer( Hstring[i] ) - Integer( '0' );
	                                   { 'Einer'-Stelle addieren }
           iNumChars := Succ( iNumChars );
	                        { Anzahl gewandelter Zeichen erhhen }
	 End;
	 'a'..'f':
	 Begin
           iVal := iVal * $10;
                              { Wert eine stelle nach links schieben }
	   iVal := iVal + Integer( Hstring[i] ) - Integer( 'a' ) + 10;
	                                   { 'Einer'-Stelle addieren }
    	   iNumChars := Succ( iNumChars );
                                { Anzahl gewandelter Zeichen erhhen }
	 End;
	 'A'..'F':
	 Begin
	   iVal := iVal * $10;
                                    {eine Stelle nach links schieben }
	   iVal := iVal + Integer( Hstring[i] ) - Integer( 'A' ) + 10;
                                           { 'Einer'-Stelle addieren }
	   iNumChars := Succ( iNumChars );
	                        { Anzahl gewandelter Zeichen erhhen }
	 End;

	 else
         Begin
	   if iNumChars <> 0 then htoi := iVal else htoi:= iDef;
	   Exit;
	 End;
      End;
      Inc( i );
      End;
  End
  else htoi := idef;
End;

{********************************************************************}
{ strichr :  Suche erstes Vorkommen eines TeilStrings ohne Gro-/    }
{            Kleinschreibung zu bercksichtigen.                     }
{*------------------------------------------------------------------*}
{ Eingabe : SuchStr   - zu durchsuchender String                     }
{           SearchStr - zu suchender TeilString                      }
{ Ausgabe : Adresse des Zeichens, das auf das erste Vorkommen des    }
{           zu suchenden TeilStrings folgt.                          }
{********************************************************************}
Function strichr( SuchStr, SearchStr : String ) : String;
var iLen, i : Integer;
Begin
  strichr:='';
  if ( SuchStr <> '' ) and ( SearchStr <> '') then
  Begin

    iLen := Length( SearchStr );             { Lnge des SuchStrings }
    i := Pos( Up( SearchStr ), Up( SuchStr ) );
    if i > 0 then
      strichr := Copy( SuchStr, i+iLen, Length( SuchStr ) - i - iLen)
  End;
End;

begin
end.