{********************************************************************}
{                       C D R O M . P A S                            }
{*------------------------------------------------------------------*}
{ Aufgabe        : Demo Programm fr MSCDEX-Programmierung           }
{                  Verwendet CDUTIL                                  }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 08.04.1994                                        }
{ letztes Update :  8.10.1994                                        }
{********************************************************************}

{$A-}                           { kein Word-Alignment von Strukturen }
{$X+}   { erweiterte Syntax, Auswertung von Fkt-Ergebnissen optional }

Uses ARGS,CDDEV,CDUTIL,DOS;

Var
  DS         : MSCDEX_DevStat;
  MC         : MSCDEX_MedChng;
  Di         : MSCDEX_DiskInfo;
  TI         : MSCDEX_TnoInfo;
  UPC        : MSCDEX_UPCode;
  i,j,
  iVersion,
  iresult,
  iIsHsg,
  iSize      : Integer;
  arg        : String;
  lpFileName : String;
  lSektor    : Longint;
  Sektor     : Array[0..2047] of Byte;
  cSektor    : Array[0..2351] of Char;
  DirEntry   : DIR_ENTRY ;

Const
  iDrive    : Integer = 0;
  iNumTrack : Integer = 5;


{********************************************************************}
{--  H A U P T P R O G R A M M                                     --}
{********************************************************************}
Begin
  if ParamCount = 0 then
  Begin
    Writeln( 'Parameter:');
    Writeln( 'A-Z:            Laufwerksbuchstabe');
    Writeln( ' -DEVSTAT       zeigt Device-Status');
    Writeln( ' -VTOC          zeigt Volume Table Of Contents');
    Writeln( ' -UPC           zeigt Universal Product Code');
    Writeln( ' -CONTENTS      zeigt Tracks');
    Writeln( ' -SHOW:X        zeigt Sektor Nummer X [mit -RAW als Raw-Data]');
    Writeln( ' -PLAY:X        spielt Audio-Track Nummer X');
    Writeln( '                [mit -WAIT werden Status-Infos angezeigt]');
    Write  ( ' -ENTRY:name    zeigt Directory-Entry der');
    Writeln( ' angegebenen Datei');
    Halt(0);
  End;

  iDrive := -1;
  for i := 1 to ParamCount do
    for j := 0 to  25 do
      begin
        arg := ParamStr(i);
        if( ( upcase( arg[1] ) = char ( j + ord( 'A' ) ) ) and
            ( arg[ 2 ] = ':' ) and
            ( length(arg) = 2 ) ) then iDrive := j;
      end;

  if iDrive < 0 then
    Begin
      Writeln( 'Ungltige Laufwerksangabe!' );
      Halt( 0 );
  End;

  if not MSCDEX_Installed then
    Begin
      Writeln( 'Konnte MSCDEX nicht finden!' );
      Halt( 0 );
  End;

  iVersion := MSCDEX200_GetVersion;
  Writeln('MSCDEX V', HI( iVersion ), '.', LO( iVersion ):2,' entdeckt');

  if iVersion > $200 then
    if not MSCDEX200_CDRomDriveCheck( iDrive ) then
      Begin
        Writeln( 'Angegebenes Laufwerk ist kein CD-Rom Laufwerk!' );
        Halt( 0 );
      End;

  {- Dafr sorgen, da evtl. CD-Wechsel erkannt werden ---------------}
  cd_GetMediaChanged( iDrive, MC );
  cd_GetDevStat( iDrive, DS );
  if GetArg( '-DEVSTAT', _none, NIL, 0 ) <> 0 then
    cd_PrintDevStat( DS )
  else
    if ( DS.lDeviceStatus and DS_NO_DISC_IN_DRIVE ) <> 0 then
      Begin
        Writeln( 'Keine CD im Laufwerk!' );
        Halt( 0 );
      End;

  {- Display Volume Table Of Contents -------------------------------}
  if GetArg( '-VTOC', _none, NIL, 0 ) <> 0 then
  Begin
    i := 0;
    repeat
      iResult := MSCDEX_ReadVTOC( iDrive, @Sektor, i );
      if ( iResult ) <> 0 then
        cd_PrintSektor( @Sektor, 2048, 16, 24 );
      Inc(i);
    Until iResult = 0;
  End;

  {- Universal Product Code anzeigen --------------------------------}
  if GetArg( '-UPC', _none, NIL, 0 ) <> 0 then
  Begin
    cd_GetUPCode( iDrive, UPC );
    cd_PrintUPCode( UPC );
  End;

  {- Inhaltsverzeichnis (Titel) aznzeigen ---------------------------}
  if GetArg( '-CONTENTS', _none, NIL, 0 ) <> 0 then
    cd_PrintDiskTracks( iDrive );

  {- Sektorinhalt anzeigen ------------------------------------------}
  lSektor := 0;
  if GetArg( '-SHOW:', _long, @lSektor, 1 ) <> 0 then
    Begin                                          { Auch RAW erlauben }
      iSize := 2048; i := COOKED;
      if GetArg( '-RAW', _none, NIL, 1 ) <> 0 then
        Begin
          iSize := 2352;
          i := RAW;
        End;
              { Achtung! Audio-CD's werden nicht gelesen Copyight! }
    if not cd_IsError( cd_ReadLong( iDrive, HSG, lSektor, 1, @cSektor, i ) ) then
      cd_PrintSektor( @cSektor, iSize, 16, 24 )
    else
      Writeln( 'Kann Sektor nicht lesen! (Audio CD?)' );
  End;

  {- Verzeichniseintrag anzeigen ------------------------------------}
  if GetArg( '-ENTRY:', _string, @lpFileName, 1 ) <> 0 then
    Begin
      if MSCDEX200_GetDirectoryEntry( iDrive, STRUCT_COPY,
                                      lpFileName, @DirEntry, iIsHsg ) then
        cd_PrintDirEntry( DirEntry )
      else
         Writeln( 'Error: ', DOSERROR );
    End;

  {- Titel abspielen ------------------------------------------------}
  iNumTrack := 0;
  if GetArg( '-PLAY:', _int, @iNumTrack, 1 ) <> 0 then
    Begin
      if ( DS.lDeviceStatus and DS_AUDIO_VIDEO ) = 0 then
        Begin
          Writeln( 'Laufwerk untersttzt keine Audio-Wiedergabe!' );
          Halt( 0 );
        End;
      if ( DS.lDeviceStatus and DS_DOOR_OPEN ) <> 0 then
        cd_CloseTray( iDrive );
      cd_GetAudioDiskInfo( iDrive, DI );
      if ( ( iNumTrack < DI.bLowestTrack ) or
           ( iNumTrack > DI.bHighestTrack ) ) then
        Begin
          Writeln( 'ungltige Liednummer' );
          Halt(0);
        End;
      cd_GetAudioTrackInfo( iDrive, iNumTrack, TI );
      if cd_IsDataTrack( TI ) then
        Begin
          Writeln('Kein Audio Track!');
          Halt(0);
        End;
       cd_StopAudio( iDrive );
       if not cd_IsError ( cd_PlayAudio( iDrive, REDBOOK, TI.lStartingPoint,
                           cd_GetTrackLen( iDrive, iNumTrack ) ) ) then
         Begin
           if GetArg( '-WAIT', _none, NIL, 0 ) <> 0 then
             Begin
               Repeat Until not ( cd_PrintActPlay( iDrive ) );
               writeln;
             End;
           Writeln( 'Play - OK' );
         End;
    End;
End.
