{*****************************************************************************
*                           D B L S P C P . P A S                            *
*----------------------------------------------------------------------------*
*     Aufgabe        : Zeigt Informationen ber alle aktiven Double-         *
*                      Space-Laufwerke an.                                   *
*----------------------------------------------------------------------------*
*     Autor          : MICHAEL TISCHER                                       *
*     entwickelt am  : 23.09.1993                                            *
*     letztes Update : 26.09.1993                                            *
*****************************************************************************}

program DblSpcP;

uses Dos, Crt;

{*****************************************************************************
* IsDoubleSpaceInstalled: berprft, ob DoubleSpace installiert ist und      *
*                         liefert Informationen ber DoubleSpace             *
*----------------------------------------------------------------------------*
* Eingabe : FirstDrive = enthlt nach dem Aufruf die erste fr DoubleSpace   *
*                        verfgbare Gertebezeichnung (65 = A:)              *
*           NumDrives  = enthlt nach dem Aufruf die Anzahl der fr Double-  *
*                        Space reservierten Gertebezeichnungen              *
*           VerNum     = enthlt nach dem Aufruf die interne DoubleSpace     *
*                        Versionsnummer                                      *
*           UpperMem   = enthlt nach dem Aufruf TRUE, wenn DoubleSpace      *
*                        im Upper-Memory residiert                           *
* Ausgabe : TRUE wenn DoubleSpace installiert ist, sonst FALSE               *
*****************************************************************************}
                                                                        
function IsDoubleSpaceInstalled( var FirstDrive : byte;
                                 var NumDrives  : byte;
                                 var VerNum     : integer;
                                 var UpperMem   : boolean ) : boolean;

var Regs : Registers;            { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ax := $4A11;                               { MUX-Code fr DoubleSpace }
  Regs.bx := 0;                                             { Funtionsnummer }
  intr( $2F, Regs );                                  { Multiplexer aufrufen }

  {-- Informationen aus Prozessorregistern holen ----------------------------}
  FirstDrive := Regs.cl;                  { Nummer des ersten DoubleSpace-Lw }
  NumDrives := Regs.ch;                          { Anzahl der DoubleSpace-Lw }
  VerNum := (Regs.dx and $7FFF);                    { interne Versionsnummer }
  UpperMem := (Regs.dx and $8000) = 0;                    { im Upper-Memory? }
  IsDoubleSpaceInstalled := (Regs.ax = 0);
end;

{*****************************************************************************
* IsDoubleSpaceDrive: berprft, ob es sich bei einem gegebenen Laufwerk     *
*                     um ein DoubleSpace-Laufwerk handelt und liefert        *
*                     Informationen ber dieses Laufwerk zurck              *
*----------------------------------------------------------------------------*
* Eingabe : LW         = Gertebezeichnug des zu berprfenden Laufwerk      *
*                        (0 = A:, 1 = B: usw)                                *
*           Vertauscht = enthlt nach dem Funktionsaufruf TRUE, wenn es      *
*                        sich um ein komprimiertes Laufwerk handelt, das     *
*                        das mit seinem Host-Laufwerk getauscht wurde        *
*           HostNr     = enthlt nach dem Funktionsaufruf die Gerte-        *
*                        bezeichnung des Host-Laufwerks, sofern  es sich     *
*                        um ein DoubleSpace-Laufwerk handelt                 *
*           CvfNr      = enthlt nach dem Funktionsaufruf die Nummer der     *
*                        CVF-Datei, sofern es sich um ein DoubleSpace-       *
*                        Laufwerk handelt                                    *
* Ausgabe : TRUE, wenn es sich um ein DoubleSpace-Laufwerk handelt, sonst    *
*           FALSE                                                            *
*****************************************************************************}

function IsDoubleSpaceDrive(     lw         : byte;
                             var Vertauscht : boolean;
                             var HostLw     : byte;
                             var CvfNr      : byte ) : boolean;

var lCvfNr,                              { lokale Variablen, nehmen zunchst }
    lHostLw        : byte;               { die Funktionsergebnisse auf       }
    lVertauscht,
    lIsDoubleSpace : boolean;

begin
  {-- zunchst von einem unkomprimierten, nicht vertauschten Laufwerk -------}
  {-- ausgehen                                                        -------}

  lHostLw := lw;
  lVertauscht := FALSE;
  lIsDoubleSpace := FALSE;
  lCvfNr := 0;

  asm
    mov     ax,4A11h                           { DoubleSpace-Funktion 00001h }
    mov     bx,0001h                           { aufrufen                    }
    mov     dl,lw
    int     2Fh
    or      ax,ax                                      { Aufruf erfolgreich? }
    jnz     @idbEnde                   { Nein, DoubleSpace nicht installiert }

    {-- Aufruf erfolgreich --------------------------------------------------}
    test    bl,80h                                 { komprimiertes Laufwerk? }
    jz      @idbHostLw                       { nein, eventuell Host-Laufwerk }

    {-- komprimiertes Laufwerk, jetzt Host-Laufwerk ermitteln ---------------}
    mov     lIsDoubleSpace,TRUE
    mov     lCvfNr,bh                          { Nummer der CVF-Datei merken }

    and     bl,7Fh                 { Nummer des Host-Laufwerks herausfiltern }
    mov     lHostLw,bl                                          { und merken }

    mov     dl,bl                         { Funktion 0001h mit Host-Laufwerk }
    mov     ax,4A11h                                       { erneut aufrufen }
    mov     bx,0001h
    int     2Fh

    and     bl,7Fh                        { Nummer des Hostlaufwerks filtern }
    cmp     bl,lw                          { ist der Host sein eigener Host? }
    mov     lVertauscht,TRUE                  { von vertauschtem Lw ausgehen }
    je      @idbEnde                                { vertauscht --> idbEnde }

    mov     lVertauscht,FALSE                { Laufwerk ist nicht vertauscht }
    mov     lHostLw,bl       
    jmp     @idbEnde

    {-- es handelt sich um ein nicht komprimiertes Host-Laufwerk ------------}
    @idbHostLw:
    and     bl,7Fh                       { Host-Laufwerksbezeichnung filtern }
    cmp     bl,dl                           { wurde das Laufwerk vertauscht? }
    je      @idbEnde                                     { Nein ---> idbEnde }

    mov     lVertauscht,TRUE                                            { Ja }
    mov     lHostLw,bl                      { echte Gertebezeichnung setzen }

    @idbEnde:
    end;                                                               { ASM }

    HostLw   := lHostLw;                       { Ergebnisse in Variablen des }
    Vertauscht := lVertauscht;                 { Aufrufers bernehmen        }
    CvfNr      := lCvfNr;
    IsDoubleSpaceDrive := lIsDoubleSpace;
end;

{-- Variablen fr das Hauptprogramm -----------------------------------------}

var i,                                                     { Schleifenzhler }
    vernum     : integer;                          { DoubleSpace Versionsnr. }
    firstdrive,                                { erstes DoubleSpace-Laufwerk }
    numdrive,                                 { Anzahl DoubleSpace-Laufwerke }
    host,                                           { nimmt Hostlaufwerk auf }
    cvfnr      : byte;                                { nimmt Cvf-Nummer auf }
    isdbl,                                           { DoubleSpace-Laufwerk? }
    uppermem,                                 { DoubleSpace im Upper-Memory? }
    vertauscht : boolean;                     { mit Hostlaufwerk vertauscht? }
    cvfstr     : string;         { zur Umwandlung der Cvf-Nummer nach String }

{----------------------------------------------------------------------------}
{--- H A U P T P R O G R A M M                                               }
{----------------------------------------------------------------------------}

begin
  clrscr;
  writeln( 'DBLTSTP.PAS  -  (c)  1993 by Michael Tischer' );
  writeln;
  isdbl := IsDoubleSpaceInstalled( firstdrive, numdrive,
                                   vernum, uppermem );
  if isdbl = false then
    begin
      writeln ( 'DoubleSpace nicht installiert!');
      exit;                                               { Programm beenden }
    end;

  {-- DoubleSpace ist installiert -------------------------------------------}
  writeln ('DoubleSpace-Version         : ', vernum );
  writeln ('erstes DoubleSpace-Laufwerk : ', chr(firstdrive), ':');
  writeln ('reserviert fr DoubleSpace  : ', numdrive, ' Laufwerke' );
  write   ('DoubleSpace im Upper-Memory : ' );
  if uppermem then writeln ( 'Ja')
              else writeln ('Nein');

  {-- DoubleSpace-Laufwerke ausgeben ----------------------------------------}
  writeln;
  writeln( 'LW   komprimiert   ist eigentlich   CVF-Datei');
  writeln( '--   -----------   --------------   ------------');
  for i := 0 to 25 do                  { Laufwerke von A: bis Z: durchlaufen }
    begin
      isdbl := IsDoubleSpaceDrive( i, vertauscht, host, cvfnr );
      if isdbl or vertauscht then
        begin
          write( chr(65+i) + ':   ' );
          if isdbl then write( '    ja ' )
                   else write( '   nein' );

          if vertauscht then write( '             ' + chr(65+host) +': ' )
                        else write( '                ' );

          if isdbl then      { CVF-Nummer fr DoubleSpace-Laufwerke ausgeben }
            begin
              str( cvfnr:3, cvfstr );
              if cvfstr[1] = ' ' then cvfstr[1] := '0';
              if cvfstr[2] = ' ' then cvfstr[2] := '0';
              write ( '        DBLSPACE.', cvfstr );
            end;
          writeln;
        end;
    end;
    writeln;
end.
