{**********************************************************************}
{*                            D F P . P A S                           *}
{*--------------------------------------------------------------------*}
{*    Aufgabe       : Formatiert 3,5" und 5,25" Disketten             *}
{*--------------------------------------------------------------------*}
{*    Autor         : Michael Tischer                                 *}
{*    entwickelt am : 23.08.1991                                      *}
{*    letztes Update: 26.01.1992                                      *}
{**********************************************************************}

program DFP;

Uses Dos;                        { die Crt- und die DOS-Unit einbinden }

{-- Konstanten --------------------------------------------------------}

const NO_DRIVE    = 0;                      { Laufwerk nicht vorhanden }
      DD_525      = 1;                            { Laufwerk: 5,25" DD }
      HD_525      = 2;                            { Laufwerk: 5,25" HD }
      DD_35       = 3;                             { Laufwerk: 3,5" DD }
      HD_35       = 4;                             { Laufwerk: 3,5" HD }
      MaxVersuche = 5;                      { Maximale Anzahl Versuche }

{-- Typdeklarationen --------------------------------------------------}

type DdptType = array[ 0..10 ] of byte;            { Struktur fr DDPT }
     DdptPtr = ^DdptType;                            { Zeiger auf DDPT }

     PhysDataType = record            { physikalische Format-Parameter }
       Seiten,                    { gewnschte Seitenzahl der Diskette }
       Spuren,                               { Anzahl Spuren pro Seite }
       Sektoren : byte;                     { Anzahl Sektoren pro Spur }
       DDPT     : DdptPtr;     { Zeiger auf Disk-Drive-Parameter-Table }
     end;

     LogDataType = record                       { DOS-Format-Parameter }
       Media,                                             { Media-Byte }
       Cluster,                          { Anzahl Sektoren pro Cluster }
       FAT,                              { Anzahl Sektoren fr die FAT }
       RootSize : byte;                 { Eintrge im Hauptverzeichnis }
    end;

    SpurBufType = array[ 1..18, 1..512 ] of byte;    { Puffer fr Spur }

{-- Initialisierte globale Variablen ----------------------------------}

const {-- Vordefinierte Tabellen fr die einzelnen Formate ------------}


      {-- Vorgaben fr den BOOT-Sektor mit Ladeprogramm ---------------}

      BootMaske  : array[ 1..102 ] of byte =
                   ( $EB, $35,                { 0000   JMP 0037        }
                     $90,                     { 0002   NOP             }
                     {-- Daten des BPB --------------------------------}

                     $50, $43, $49, $4E, $54, $45, $52, $4E,
                     $00, $00, $00, $01, $00, $00, $00, $00,
                     $00, $00, $00, $00, $00, $00, $00, $00,
                     $00, $00, $00, $00, $00, $00, $00, $00,
                     $00, $00, $00, $00, $00, $00, $00, $00,
                     $00, $00, $00, $00, $00, $00, $00, $00,
                     $00, $00, $00, $00,

                     {-- Eigentliches Ladeprogramm --------------------}

                     $FA,                     { 0037   CLI             }
                     $B8, $30, $00,           { 0038   MOV     AX,0030 }
                     $8E, $D0,                { 003B   MOV     SS,AX   }
                     $BC, $FC, $00,           { 003D   MOV     SP,00FC }
                     $FB,                     { 0040   STI             }
                     $0E,                     { 0041   PUSH    CS      }
                     $1F,                     { 0042   POP     DS      }
                     $BE, $66, $7C,           { 0043   MOV     SI,7C66 }
                     $B4, $0E,                { 0046   MOV     AH,0E   }
                     $FC,                     { 0048   CLD             }
                     $AC,                     { 0049   LODSB           }
                     $0A, $C0,                { 004A   OR      AL,AL   }
                     $74, $04,                { 004C   JZ      0052    }
                     $CD, $10,                { 004E   INT     10      }
                     $EB, $F7,                { 0050   JMP     0049    }
                     $B4, $01,                { 0052   MOV     AH,01   }
                     $CD, $16,                { 0054   INT     16      }
                     $74, $06,                { 0056   JZ      005E    }
                     $B4, $00,                { 0058   MOV     AH,00   }
                     $CD, $16,                { 005A   INT     16      }
                     $EB, $F4,                { 005C   JMP     0052    }
                     $B4, $00,                { 005E   MOV     AH,00   }
                     $CD, $16,                { 0060   INT     16      }
                     $33, $D2,                { 0062   XOR     DX,DX   }
                     $CD, $19 );              { 0064   INT     19      }

      BootMes : string =
       #13#10'DFP  -  (C) 1992 by Michael Tischer'+ #13#10 +
       #13#10'Keine Systemdiskette oder fehlerhafte Diskette!'#13#10 +
       'Bitte Diskette wechseln und danach eine Taste drcken' +
       #13#10;

{-- nicht initialisierte globale Variablen ----------------------------}

var AktDrive     : byte; { Nummer des zu formatierenden Laufwerks 0, 1 }
    AktDriveType : byte;        { Typ des aktuellen Diskettenlaufwerks }
    PData        : PhysDataType;   { physikalische Formatinformationen }
    LData        : LogDataType;         { logische Formatinformationen }
    AlteDDPT     : pointer;                     { Zeiger auf alte DDPT }
    OK           : boolean;                  { Flag fr Programmablauf }
    EndeCode     : word;         { Rckgabewert an aufrufenden Prozess }
    Param        : string;          { zur Auswertung der Kommandozeile }

{**********************************************************************}
{* GetDriveType  : Ermittelt den Typ eines Diskettenlaufwerks         *}
{* Eingabe       : DRIVE = Laufwerksnummer (0, 1 etc.)                *}
{* Ausgabe       : Laufwerks-Code als Konstante (DD_525, HD_525 etc.) *}
{**********************************************************************}

function GetDriveType( Drive : byte ) : byte;

var Regs     : Registers;     { Prozessorregister fr Interrupt-Aufruf }

begin
  Regs.ah := $08;                   { Funktion: Laufwerkstyp ermitteln }
  Regs.dl := Drive;                                  { Laufwerksnummer }
  intr( $13, Regs );                         { BIOS-Interrupt aufrufen }
  if ( Regs.flags and fcarry = 0 ) then  { Aufruf ohne Fehler beendet? }
    GetDriveType := Regs.bl                             { Laufwerkstyp }
  else
    GetDriveType := DD_525;        { Funktion $08 des Interrupts nicht }
end;                               { vorhanden  => Computertyp = XT    }

{**********************************************************************}
{* ResetDisk     : Disk-Reset auf alle Laufwerke                      *}
{* Eingabe       : keine                                              *}
{* Ausgabe       : keine                                              *}
{* Info          : Unabhngig von der in DL geladenen Laufwerksnummer *}
{*                 wird der Reset auf alle Laufwerke durchgefhrt     *}
{**********************************************************************}

procedure DiskReset;

var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }

begin
  with Regs do
    begin
      ah := $00;                  { Funktionummer fr Interrupt-Aufruf }
      dl := 0;                                 { Laufwerk a: (s. Info) }
    end;
  intr( $13, Regs );                                { Interrupt-Aufruf }
end;

{**********************************************************************}
{* GetFormatParamter: Ermittelt die logischen und physikalischen Pa-  *}
{*                    rameter, die fr die Formatierung bentigt      *}
{*                    werden                                          *}
{* Eingabe       : FORMSTRING = gewnschte Kapazitt als String       *}
{*                              "360", "1200", "720", "1440"          *}
{*                 DRIVETYPE  = Laufwerkscode, wie er von GetDrive-   *}
{*                              Type zurckgeliefert wird             *}
{*                 PDATA      = wird durch die Prozedur mit den Eck-  *}
{*                              daten des Physikalischen Formats ge-  *}
{*                              laden                                 *}
{*                 LDATA      = wie PDATA, nur DOS-Eckdaten           *}
{* Ausgabe       : TRUE, wenn das Format mglich ist, sonst FALSE     *}
{* Info          : Neue Formate knnen durch Erweitern dieser Prozedur*}
{*                 hinzugefgt werden                                 *}
{**********************************************************************}

function GetFormatParameter(     FormString : string;
                                 DriveType  : byte;
                             var PData      : PhysDataType;
                             var LData      : LogDataType  ) : boolean;

const DDPT_360  : DdptType = ( $DF, $02, $25, $02, $09, $2A,
                               $FF, $50, $F6, $0F, $08 );
      DDPT_1200 : DdptType = ( $DF, $02, $25, $02, $0F, $1B,
                               $FF, $54, $F6, $0F, $08 );
      DDPT_720  : DdptType = ( $DF, $02, $25, $02, $09, $2A,
                               $FF, $50, $F6, $0F, $08 );
      DDPT_1440 : DdptType = ( $DF, $02, $25, $02, $12, $1B,
                               $FF, $6C, $F6, $0F, $08 );

      LOG_360  : LogDataType = ( Media : $FD; Cluster  : 2;
                                 FAT   : 2;   RootSize : $70 );
      LOG_1200 : LogDataType = ( Media : $F9; Cluster  : 1;
                                 FAT   : 7;   RootSize : $E0 );
      LOG_720  : LogDataType = ( Media : $F9; Cluster  : 2;
                                 FAT   : 3;   RootSize : $70 );
      LOG_1440 : LogDataType = ( Media : $F0; Cluster  : 1;
                                 FAT   : 9;   RootSize : $E0 );

      PHYS_360  : PhysDataType = ( Seiten   :  2; Spuren : 40;
                                   Sektoren :  9; DDPT   : @DDPT_360  );
      PHYS_1200 : PhysDataType = ( Seiten   :  2; Spuren : 80;
                                   Sektoren : 15; DDPT   : @DDPT_1200 );
      PHYS_1440 : PhysDataType = ( Seiten   :  2; Spuren : 80;
                                   Sektoren : 18; DDPT   : @DDPT_1440 );
      PHYS_720  : PhysDataType = ( Seiten   :  2; Spuren : 80;
                                   Sektoren :  9; DDPT   : @DDPT_720  );

begin
  if ( FormString = '1200' ) then                  { 1,2 MB auf 5,25"? }
    if ( DriveType = HD_525 ) then  { Format mit Laufwerk vertrglich? }
      begin                                     { Ja, Parameter setzen }
        PData := PHYS_1200;
        LData := LOG_1200;
        GetFormatParameter := true;                 { Ende ohne Fehler }
      end
    else
      GetFormatParameter := false  { Laufwerk und Format unvertrglich }
  else if ( FormString = '360' ) then                        { 360 KB? }
    if ( DriveType = HD_525 ) or ( DriveType = DD_525 ) then
      begin        { Format und Laufwerk vertrglich, Parameter setzen }
        PData := PHYS_360;
        LData := LOG_360;
        GetFormatParameter := true;                 { Ende ohne Fehler }
      end
    else
      GetFormatParameter := false  { Laufwerk und Format unvertrglich }
  else if ( FormString = '1440' ) then             { 1,44 MB auf 3,5"? }
    if ( DriveType = HD_35 ) then   { Format mit Laufwerk vertrglich? }
      begin                                     { Ja, Parameter setzen }
        PData := PHYS_1440;
        LData := LOG_1440;
        GetFormatParameter := true;                 { Ende ohne Fehler }
      end
    else
      GetFormatParameter := false  { Laufwerk und Format unvertrglich }
  else if ( FormString = '720' ) then               { 720 KB auf 3,5"? }
    if ( DriveType = HD_35 ) or ( DriveType = DD_35 ) then
      begin        { Format und Laufwerk vertrglich, Parameter setzen }
        PData := PHYS_720;
        LData := LOG_720;
        GetFormatParameter := true;                 { Ende ohne Fehler }
      end
    else
      GetFormatParameter := false  { Laufwerk und Format unvertrglich }
  else
    GetFormatParameter := false;      { Kein gltiges Format angegeben }
end;

{**********************************************************************}
{* DiskPrepare   : Vorbereiten des Laufwerks, Einstellen der Daten-   *}
{*                 transferrate                                       *}
{* Eingabe       : DRIVE = Laufwerksnummer                            *}
{*                 PDATA = physikalische Parameter                    *}
{* Ausgabe       : keine                                              *}
{**********************************************************************}

procedure DiskPrepare( Drive : byte; PData : PhysDataType );

var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }

begin
  {-- Media-Typ fr Formatieraufruf setzen ----------------------------}

  with Regs do
    begin
      ah := $18;                { Funktionsnummer fr Interrupt-Aufruf }
      ch := PData.Spuren - 1;                { Anzahl Spuren pro Seite }
      cl := PData.Sektoren;                 { Anzahl Sektoren pro Spur }
      dl := Drive;                                    { Laufwerknummer }
    end;
  intr( $13, Regs );                                { Interrupt-Aufruf }
end;

{**********************************************************************}
{* FormatTrack   : formatiert eine Spur                               *}
{* Eingabe       : s.u.                                               *}
{* Ausgabe       : der Fehler Status                                  *}
{**********************************************************************}

function Formattrack( Laufwerk,         { die Nummer Diskettenlaufwerk }
                      Seite,                        { die Seitennummer }
                      Spur,                { die zu formatierende Spur }
                      Anzahl      : byte ) : byte; { Sektoren pro Spur }

type FormatTyp = record
                   DSpur, DSeite, DZaehler, DLaenge : byte;
                 end;

var Regs      : Registers;    { Prozessorregister fr Interrupt-Aufruf }
    DatenFeld : array[ 1..18 ] of FormatTyp;     { maximal 18 Sektoren }
    Zaehler   : byte;                                { Schleifenzhler }
    versuche  : byte;                       { Maximale Anzahl Versuche }

begin
  for Zaehler := 1 to Anzahl do
    with Datenfeld[ Zaehler ] do
      begin
        DSpur := Spur;                               { Nummer der Spur }
        DSeite := Seite;                              { Diskettenseite }
        DZaehler := Zaehler;                      { Nummer des Sektors }
        DLaenge := 2;                   { Anzahl Byte pro Sektor (512) }
      end;
  versuche := MaxVersuche;            { maximale Versuchsanzahl setzen }
  repeat
    with Regs do
      begin
        ah := 5;                { Funktionsnummer fr Interrupt-Aufruf }
        al := Anzahl;              { Anzahl der Sektoren fr eine Spur }
        es := Seg( DatenFeld );              { Adresse des Datenfeldes }
        bx := Ofs( DatenFeld );              { in die Register es:bx   }
        dh := Seite;                                { Nummer der Seite }
        dl := Laufwerk;                              { Laufwerksnummer }
        ch := Spur;                                  { Nummer der Spur }
      end;
    intr( $13, Regs );                       { BIOS-Interrupt aufrufen }
    if ( Regs.flags and fcarry = 1 ) then                    { Fehler? }
      DiskReset;                { Ja, Diskettenreset vor neuem Versuch }
    dec( versuche );
  until ( Regs.flags and fcarry = 0 ) or ( versuche = 0 );
  Formattrack := Regs.ah;                     { Fehler-Status auslesen }
end;

{**********************************************************************}
{* VerifyTrack   : Spur berprfen                                    *}
{* Eingabe       : Laufwerk, Seite, Spur, Sektorzahl                  *}
{* Ausgabe       : Fehlercode (0=OK)                                  *}
{**********************************************************************}

function VerifyTrack( Laufwerk, Seite, Spur, Sektoren : byte ) : byte;

var versuche   : byte;                      { maximale Anzahl Versuche }
    Regs       : Registers;   { Prozessorregister fr Interrupt-Aufruf }
    SpurBuffer : SpurBufType;                 { Speicher fr eine Spur }

begin
  versuche := MaxVersuche;            { maximale Versuchsanzahl setzen }
  repeat
    with Regs do
      begin
        ah := $04;              { Funktionsnummer fr Interrupt-Aufruf }
        al := Sektoren;                     { Anzahl Sektoren pro Spur }
        ch := Spur;                                  { Nummer der Spur }
        cl := 1;                                  { Start bei Sektor 1 }
        dl := Laufwerk;                              { Laufwerksnummer }
        dh := Seite;                                    { Seitennummer }
        es := Seg( SpurBuffer );                  { Adresse fr Buffer }
        bx := Ofs( SpurBuffer );
      end;
    intr( $13, Regs );                      { BIOS Interrupt  aufrufen }
    if ( Regs.flags and fcarry = 1 ) then                    { Fehler? }
      DiskReset;                { Ja, Diskettenreset vor neuem Versuch }
    dec( versuche );
  until ( Regs.flags and fcarry = 0 ) or ( versuche = 0 );
  VerifyTrack := Regs.ah;
end;

{**********************************************************************}
{* WriteTrack    : Spur schreiben                                     *}
{* Eingabe       : Laufwerk, Seite, Spur, Startsektor, Anzahl, Daten  *}
{* Ausgabe       : Fehlercode (0=OK)                                  *}
{**********************************************************************}

function WriteTrack(     Laufwerk, Seite, Spur,
                         Start, Anzahl           : byte;
                     var Buffer ) : byte;

var versuche : byte;                        { maximale Anzahl Versuche }
    Regs     : Registers;     { Prozessorregister fr Interrupt-Aufruf }

begin
  versuche := MaxVersuche;            { maximale Versuchsanzahl setzen }
  repeat
    with Regs do
      begin
        ah := $03;              { Funktionsnummer fr Interrupt-Aufruf }
        al := Anzahl;                       { Anzahl Sektoren pro Spur }
        ch := Spur;                                  { Nummer der Spur }
        cl := Start;                              { Start bei Sektor 1 }
        dl := Laufwerk;                              { Laufwerksnummer }
        dh := Seite;                                    { Seitennummer }
        es := Seg( Buffer );                      { Adresse fr Buffer }
        bx := Ofs( Buffer );
      end;
    intr( $13, Regs );                       { BIOS-Interrupt aufrufen }
    if ( Regs.flags and fcarry = 1 ) then                    { Fehler? }
      DiskReset;                { Ja, Diskettenreset vor neuem Versuch }
    dec( versuche );
  until ( Regs.flags and fcarry = 0 ) or ( versuche = 0 );
  WriteTrack := Regs.ah;
end;

{**********************************************************************}
{* PhysicalFormat: Physikalisches Formatieren der Diskette (Einteilen *}
{*                 in Spuren, Sektoren)                               *}
{* Eingabe       : DRIVE  = Laufwerkscode                             *}
{*                 PDATA  = Physikalische Parameter                   *}
{*                 VERIFY = TRUE, wenn Verify durchgefhrt werden soll*}
{* Ausgabe       : FALSE bei einem Fehler, sonst TRUE                 *}
{**********************************************************************}

function PhysicalFormat( Drive  : byte;
                         PData  : PhysDataType;
                         Verify : boolean ) : boolean;

var versuche : byte;                        { Maximale Anzahl Versuche }
    Regs     : Registers;     { Prozessorregister fr Interrupt-Aufruf }
    Spur,                                              { Aktuelle Spur }
    Seite,                                            { Aktuelle Seite }
    Status   : byte;        { Rckgabewert der aufgerufenen Funktionen }


begin
  {-- Diskette Spur fr Spur formatieren ------------------------------}

  for Spur := 0 to PData.Spuren - 1 do       { alle Spuren durchlaufen }
    for Seite := 0 to PData.Seiten - 1 do    { alle Seiten durchlaufen }
      begin
        Write( #13'Spur: ', Spur : 2, '  Seite: ', Seite : 2 );
        {-- Maximal 5 Versuche ein Spur zu formatieren ----------------}

        versuche := MaxVersuche;      { maximale Versuchsanzahl setzen }
        repeat
          Status := FormatTrack( Drive, Seite, Spur, PData.Sektoren );
          if ( Status = 3 ) then          { Diskette schreibgeschtzt? }
            begin
              PhysicalFormat := false;   { Prozedur mit Fehler beenden }
              WriteLn( #13'Diskette ist schreibgeschtzt' );
              exit;                                 { Prozedur beenden }
            end;
          if ( Status = 0 ) and Verify then
            Status := VerifyTrack( Drive, Seite, Spur, PData.Sektoren );
          dec( versuche );
          if ( Status > 0 ) then          { Formatieren fehlgeschlagen }
            DiskReset;
        until ( Status = 0 ) or ( versuche = 0 );
        if ( Status > 0 )  then             {  Fehler beim Formatieren }
          begin
            PhysicalFormat := false;     { Prozedur mit Fehler beenden }
            WriteLn( #13'Spur fehlerhaft      ' );
            exit;                                   { Prozedur beenden }
          end;
      end;
  PhysicalFormat := true;               { Prozedur ohne Fehler beendet }
end;

{**********************************************************************}
{* LogicalFormat : Logisches Formatieren der Diskette (Schreiben von  *}
{*                 Bootsektor, FAT und Hauptverzeichnis)              *}
{* Eingabe       : DRIVE = Laufwerksnummer                            *}
{*                 PDATA = physikalische Formatinformationen          *}
{*                 LDATA = logische Formatinformationen               *}
{* Ausgabe       : TRUE, wenn kein Fehler aufgetreten ist             *}
{**********************************************************************}

function LogicalFormat( Drive : byte;
                        PData : PhysDataType;
                        LData : LogDataType ) : boolean;

var Status         : byte;   { Rckmeldung der aufgerufenen Funktionen }
    GesamtSektoren : word;               { Gesamte Anzahl der Sektoren }
    i              : byte;                           { Schleifenzhler }
    AktSector,
    AktSide,
    AktTrack       : byte;
    Anzahl         : integer;    { Anzahl noch zu schreibende Sektoren }
    SpurBuffer     : SpurBufType;             { Speicher fr eine Spur }

begin
  fillchar( SpurBuffer, word( PData.Sektoren ) * 512, 0 ); {Buf. leeren}

  {-- Bootsektor: fester Teil -----------------------------------------}

  move( BootMaske, SpurBuffer, 102 );      { Maske Bootsektor kopieren }
  move( BootMes[1], SpurBuffer[ 1, 103 ],        { Boottexte koopieren }
        ord(BootMes[0]) );
  SpurBuffer[ 1, 511 ] := $55;      { Ende-Kennzeichen des Bootsektors }
  SpurBuffer[ 1, 512 ] := $AA;

  {-- Bootsektor: variabler Teil---------------------------------------}

  GesamtSektoren := PData.Spuren * PData.Sektoren * Pdata.Seiten;
  SpurBuffer[ 1, 14 ] := LData.Cluster;                 { Clustergre }
  SpurBuffer[ 1, 18 ] := LData.RootSize;  { Anzahl Eintrge im Hauptv. }
  SpurBuffer[ 1, 20 ] := lo( GesamtSektoren ); { Gesamtanzahl Sektoren }
  SpurBuffer[ 1, 21 ] := hi( GesamtSektoren ); { auf der Diskette      }
  SpurBuffer[ 1, 22 ] := LData.Media;               { Media-Deskriptor }
  SpurBuffer[ 1, 23 ] := LData.FAT;                    { Gre der FAT }
  SpurBuffer[ 1, 25 ] := PData.Sektoren;           { Sektoren pro Spur }
  SpurBuffer[ 1, 27 ] := PData.Seiten;             { Anzahl der Seiten }

  {-- FAT und FAT-Kopie anlegen (Inhalt 00)----------------------------}

  SpurBuffer[ 2, 1 ] := LData.Media;                  { 1. FAT anlegen }
  SpurBuffer[ 2, 2 ] := $FF;
  SpurBuffer[ 2, 3 ] := $FF;
  SpurBuffer[ LData.FAT + 2, 1 ] := LData.Media;      { 2. FAT anlegen }
  SpurBuffer[ LData.FAT + 2, 2 ] := $FF;
  SpurBuffer[ LData.FAT + 2, 3 ] := $FF;

  {-- Boot-Sektor und FAT schreiben -----------------------------------}

  Status := WriteTrack( Drive, 0, 0, 1, PData.Sektoren, SpurBuffer );
  if Status <> 0 then
    LogicalFormat := FALSE

  {-- kein Fehler, Rootverzeichnis schreiben --------------------------}

  else
    begin
      fillchar( SpurBuffer, 512, 0 );                  { Leerer Sektor }
      AktSector := PData.Sektoren;   { Erste Spur komplett geschrieben }
      AktTrack := 0;                                   { Aktuelle Spur }
      AktSide := 0;                                   { Aktuelle Seite }

      {-- Anzahl der verbleibenden Sektoren ermitteln und schreiben ---}

      Anzahl := LData.FAT * 2 + ( LData.Rootsize * 32 div 512 ) +
                1 - PData.Sektoren;

      i := 1;
      repeat
        inc( AktSector );                            { Nchster Sektor }
        if ( AktSector > PData.Sektoren ) then         { Spur zu Ende? }
          begin
            AktSector := 1;                      { Weiter bei Sektor 1 }
            inc( AktSide );                           { Nchste Seite? }
            if ( AktSide = PData.Seiten ) then       { Schon 2. Seite? }
              begin
                AktSide := 0;                     { Wieder auf Seite 0 }
                inc( AktTrack );
              end;
          end;
        Status := WriteTrack( Drive, AktSide, AktTrack,
                              AktSector, 1, SpurBuffer );
        inc( i );
      until ( i > Anzahl ) or ( status <> 0 );
      LogicalFormat := ( status = 0 )
    end;
end;

{**********************************************************************}
{*                         HAUPTPROGRAMM                              *}
{**********************************************************************}

begin
  WriteLn( 'DFP  -  (c) 1992 by Michael Tischer'#13#10 );
  if paramcount > 1 then                        { Parameter angegegen? }
    begin                                              { Ja, auswerten }
      Param := paramstr( 1 );  { Laufwerk ermitteln ( 0 = a:, 1 = b: ) }
      AktDrive := ord( upcase( Param[ 1 ] ) ) - 65;
      AktDriveType := GetDriveType( AktDrive ); { Typ aktuelles Laufw. }
      if ( AktDriveType > 0 ) then               { Laufwerk vorhanden? }
        begin                   { Ja, Programm kann fortgesetzt werden }
          if GetFormatParameter( paramstr( 2 ), AktDriveType,
                                 PData, LData) then
            begin              { Format und Laufwerk passen zueinander }
              DiskPrepare( AktDrive, PData );
              GetIntVec( $1E, AlteDDPT );           { Alte DDPT merken }
              SetIntVec( $1E, PData.DDPT );         { Neue DDPT Setzen }

              Param := paramstr( 3 );
              ok := PhysicalFormat( AktDrive, PData,
                                    upcase( Param[ 1 ] ) <> 'N' );
              if ok then
                begin
                  Write( #13'Bootsektor u. FAT schreiben       ' );
                  ok := LogicalFormat( AktDrive, PData, LData )
                end;

              {-- Auswertung des Formatiervorgangs --------------------}

              if ok then
                begin
                  WriteLn( #13'Formatierung o.k.                 ' );
                  EndeCode := 0;
                end
              else
                begin
                  WriteLn( #13'Formatierung wg. Fehler abgebrochen' );
                  EndeCode := 1;
                end;
              SetIntVec( $1E, AlteDDPT );     { Alte DDPT restaurieren }
            end
          else
            begin
              WriteLn( 'Das angegebene Format kann auf dem Laufwerk ' +
                       'nicht formatiert werden!' );
              EndeCode := 2;     { Rckgabewert an aufrufenden Prozess }
            end
        end
      else
        begin
          WriteLn( 'Das angegebene Diskettenlaufwerk existiert nicht!');
          EndeCode := 3;         { Rckgabewert an aufrufenden Prozess }
        end
    end
  else
    begin
      writeln( 'Aufruf: DFP Laufwerk   Format   [ NV ]' );
      writeln( '                                ' );
      writeln( '                                ' );
      writeln( '  A: oder B:                    ' );
      writeln( '                                 ' );
      writeln( '   360, 720, 1200, 1440          ' );
      writeln( '                                  ' );
      writeln( '                 NV = kein Verify ' );
      EndeCode := 4;             { Rckgabewert an aufrufenden Prozess }
    end;
  Halt( EndeCode );
end.
