{**********************************************************************}
{*                          D I R P 1 . P A S                         *}
{*--------------------------------------------------------------------*}
{*    Aufgabe        : Zeigt alle Dateien in einem beliebigen Ver-    *}
{*                     zeichnis inklusive Unterverzeichnissen und     *}
{*                     Volume-Namen auf dem Bildschirm an.            *}
{*                     Die Abfrage erfolgt dabei ber die beiden      *}
{*                     DOS-Funktionen $4E und $4F.                    *}
{*                     Siehe in disem Zusammenhang auch das Programm  *}
{*                     DIRP2.PAS.                                     *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  :  8.07.1988                                     *}
{*    letztes Update : 15.10.1991                                     *}
{**********************************************************************}
                                                                        
program DIRP1;
                                                                        
Uses Crt, Dos;                   { die CRT- und die DOS-Unit einbinden }
                                                                        
{-- Typdeklarationen --------------------------------------------------}

type DirBufTyp = record     { Datenstruktur der Funktionen $4E und $4F }
                   Reserviert : array [1..21] of char;
                   Attr       : byte;
                   Time       : integer;
                   Date       : integer;
                   Size       : longint;
                   Name       : array [1..13] of char
                 end;

     MonVec    = array[1..12] of string[3];    { Array mit Monatsnamen }

{-- Konstanten --------------------------------------------------------}

const FA_ReadOnly  = $01;                             { Dateiattribute }
      FAHidden     = $02;
      FA_SysFile   = $04;
      FA_VolumeID  = $08;
      FA_Directory = $10;
      FA_Archive   = $20;
      FA_AnyFile   = $3F;

      EINTR = 14;        { soviele Eintrge sind gleichzeitig sichtbar }
      Monate : MonVec = ( 'Jan', 'Feb', 'Mr', 'Apr', 'Mai', 'Jun',
                          'Jul', 'Aug', 'Sep', 'Okt', 'Nov', 'Dez');

{**********************************************************************}
{* FindFirst: den ersten Directory-Eintrag einlesen                   *}
{* Eingabe : keine                                                    *}
{* Ausgabe : TRUE oder FALSE, je nachdem, ob ein Eintrag gefunden     *}
{*           wurde                                                    *}
{**********************************************************************}
                                                                        
function FindFirst(DateiName : string;      { die zu suchenden Dateien }
                   Attribut  : integer) : boolean; { das Such-Attribut }

var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  DateiName := DateiName + #0;      { Dateiname durch NUL abschliessen }
  Regs.ah := $4E;               { Funktionsnummer fr Search for first }
  Regs.cx := Attribut;            { Attribute, nach denen gesucht wird }
  Regs.ds := Seg(DateiName);           { Segmentadresse des Dateinamen }
  Regs.dx := succ(Ofs(DateiName));      { Offsetadresse des Dateinamen }
  MsDos( Regs );                          { DOS Interrupt $21 aufrufen }
  FindFirst := ( (Regs.flags and 1) = 0 )          { Carry-Flag testen }
end;
                                                                        
{**********************************************************************}
{* FindNext: den folgenden Directory-Eintrag einlesen                 *}
{* Eingabe : keine                                                    *}
{* Ausgabe : true oder false, je nachdem, ob noch ein Eintrag ge-     *}
{*             funden wurde                                           *}
{* Info    : diese Funktion darf erst nach einem erfolgreichen Auf-   *}
{*           ruf der Funktion FinFirst aufgerufen werden              *}
{**********************************************************************}
                                                                        
function FindNext : boolean;
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $4F;                { Funktionsnummer fr Search for next }
  MsDos( Regs );                          { DOS Interrupt $21 aufrufen }
  FindNext := ( (Regs.flags and 1) = 0 )           { Carry-Flag testen }
end;
                                                                        
{**********************************************************************}
{* SetDTA: setzt die Adresse der DTA                                  *}
{* Eingabe : SEGMENT = Segmentadresse der neuen DTA                   *}
{*           OFFSET  = Offsetadresse der neuen DTA                    *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure SetDTA(Segment,                { neue Segmentadresse der DTA }
                 Offset  : integer);      { neue Offsetadresse der DTA }
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $1A;                     { Funktionsnummer fr DTA setzen }
  Regs.ds := Segment;                 { Segmentadresse ins DS-Register }
  Regs.dx := Offset;                   { Offsetadresse ins DX-Register }
  MsDos( Regs );                          { DOS-Interrupt $21 aufrufen }
end;
                                                                        
{**********************************************************************}
{* PRINTDATA: Informationen ber einen Eintrag ausgeben               *}
{* Eingabe : DIRBUF = Datenstrktur mit Dateiinformationen             *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure PrintData( DirBuf : DirBufTyp );
                                                                        
var Zaehler : byte;

begin
  writeln;      { dadurch wird das Fenster um eine Zeile hochgescrollt }

  Zaehler := 1;        { mit dem ersten Buchstaben des Namens beginnen }
  while (DirBuf.Name[Zaehler]<>#0) do            { wiederholen bis NUL }
   begin
    write(DirBuf.Name[Zaehler]);         { Zeichen des Namens ausgeben }
    Zaehler := succ(Zaehler)            { nchstes Zeichen verarbeiten }
   end;

  GotoXY(13, EINTR);
  write('', DirBuf.Size:7);
  GotoXY(21, EINTR);
  write('', DirBuf.Date and 31:2,'. ');               { Tag ermitteln }
  write (Monate[DirBuf.Date shr 5 and 15]);           { Monat ausgeben }
  write(DirBuf.Date shr 9 + 1980:5);                  { Jahr ermitteln }
  GotoXY(34, EINTR);
  write('', DirBuf.Time shr 11:2, ' Uhr');         { Stunde ermitteln }
  write(DirBuf.Time shr 5 and 63:3);                { Minute ermitteln }

  GotoXY(44, EINTR);
  write('');                    { Trennstrich zum vorhergehenden Feld }
  Zaehler := 1;                               { die Attribute ausgeben }
  while ( Zaehler < 32 ) do
    begin
      if (DirBuf.Attr and Zaehler) <> 0 then write('X')
                                             else write(' ');
      Zaehler := Zaehler shl 1;
    end;
  write('');                         { rechte Wand des Fensterrahmens }
end;
                                                                        
{**********************************************************************}
{* BildschirmAufbau: bereitet den Bildschirm fr die Ausgabe des      *}
{*                   Directories vor                                  *}
{* Eingabe : keine                                                    *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure BildschirmAufbau;
                                                                        
var Zaehler : integer;                               { Schleifenzhler }
                                                                        
begin
 ClrScr;                                          { Bildschirm lschen }
 Window(14,(20-EINTR) shr 1+1,64,(20-EINTR) shr 1 +5+EINTR);
 GotoXY(1,1);            { Cursor in die obere linke Ecke des Fensters }

 write('ͻ');
 write(' Dateiname   Gre    Datum      Zeit   RHSVD');
 write('Ķ');

 for Zaehler := 1 to EINTR do
  write('                                             ');
 write('ͼ');

 Window(15,(20-EINTR) shr 1+4,66,(20-EINTR) shr 1 +3+EINTR);
 GotoXY(1, EINTR);        { Cursor in die obere linke Ecke des Fenster }
end;
                                                                        
{**********************************************************************}
{* Dir: kontrolliert das Einlesen und die Ausgabe des Directories     *}
{* Eingabe : PFAD     = Such-Pfad mit Dateimaske                      *}
{*           ATTRIBUT = Such-Attribut                                 *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure Dir( Pfad : string; Attr : byte );

var AnzEintraege,              { Gesamt-Anzahl der gefundenen Eintrge }
    AnzBild      : integer;              { Anzahl der Eintrge in Bild }
    Taste        : char;            { dient zum Warten auf Tastendruck }
    DirBuf       : DirBufTyp;    { nimmt jeweils einen Dir-Eintrag auf }

begin
  SetDTA(Seg(DirBuf), Ofs(DirBuf));          { DirBuf ist der neue DTA }
  clrscr;                                         { Bildschirm lschen }
  BildschirmAufbau;        { Bildschirm fr Directory-Ausgabe aufbauen }

  AnzBild := -1;              { noch kein Eintrag im Fenster angezeigt }
  AnzEintraege := 0;                    { noch keinen Eintrag gefunden }
  if FindFirst( Pfad, Attr ) then              { ersten Eintrag suchen }
    repeat
      AnzEintraege := succ(AnzEintraege);  { ein Eintrag mehr gefunden }
      AnzBild := succ(AnzBild);          { ein Eintrag mehr im Fenster }
      if AnzBild = EINTR then                  { ist das Fenster voll? }
        begin                                                     { JA }
          Window( 14, (20-EINTR) shr 1 + 5+ EINTR,
                  66 ,(20-EINTR) shr 1 + 6+ EINTR );
          GotoXY(1, 1);     { Cursor in die unterste Zeile des Fenster }
          TextBackground( LightGray );            { weier Hintergrund }
          TextColor( Black );                       { schwarze Schrift }
          write('                Bitte Taste drcken                ');
          Taste := ReadKey;                           { Taste einlesen }
          GotoXY(1, 1);   { Cursor in die obere linke Ecke des Fenster }
          TextBackground( Black );             { schwarzer Hintergrund }
          TextColor( LightGray );                      { weie Schrift }
          write('                                                   ');
          Window(15,(20-EINTR) shr 1+4,65,(20-EINTR) shr 1 +3+EINTR);
          GotoXY(1, EINTR);     { Cursor an seine alte Position zurck }
          AnzBild := 0;             { mit zhlen wieder bei 0 anfangen }
        end;
      PrintData( DirBuf );                { Daten des Eintrag ausgeben }
    until not(FindNext);                { gibt es noch einen Eintrag ? }

  Window(14,(20-EINTR) shr 1 +5+EINTR,65,(20-EINTR) shr 1 +6+EINTR);
  GotoXY(1, 1);           { Cursor in die obere linke Ecke des Fenster }
  TextBackground( LightGray );                    { weier Hintergrund }
  TextColor( Black );                               { schwarze Schrift }
  write('                                                   ');

  GotoXY(2, 1);
  case AnzEintraege of
    0 : write('keine Datei gefunden');
    1 : write('eine Datei gefunden');
    else write(AnzEintraege,' Dateien gefunden')
  end;

 Window(1, 1, 80, 25);   { wieder ganzen Bildschirm als Fenster setzen }
end;
                                                                        
{**********************************************************************}
{**                           HAUPTPROGRAMM                          **}
{**********************************************************************}
                                                                        
begin
  case ParamCount of                    { je nach Anzahl der Argumente }
    0  : Dir( '*.*', FA_AnyFile );   { alle Dateien im aktuellen Verz. }
    1  : Dir( ParamStr(1), FA_AnyFile );     { angegebene Dat anzeigen }
    else writeln('Ungltige Anzahl Parameter');
  end;
end.
