{********************************************************************}
{                      D M A U T I L . P A S                         }
{*------------------------------------------------------------------*}
{ Aufgabe        : Stellt Funktionen zur Programmierung des DMA-     }
{                  Controllers zur Verfgung.                        }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 20.03.1994                                        }
{ letztes Update :  8.10.1994                                        }
{********************************************************************}
Unit DMAUTIL;

Interface

Const
{- Bits des Statusregisters ($08, $D0 ) Lesen -----------------------}
STATUS_REQ3 = $80;               { Bit Gesetzt: jeweiliger DMA-Kanal }
STATUS_REQ2 = $40;               { erhielt DMA-Anforderung.          }
STATUS_REQ1 = $20;               { Request                           }
STATUS_REQ0 = $10;
STATUS_TC3  = $08;             { Bit gesetzt: Seit letztem Lesen des }
STATUS_TC2  = $04;             { Statusregisters wurde eine DMA-     }
STATUS_TC1  = $02;             { bertragung abgeschlossen.          }
STATUS_TC0  = $01;             { Terminal Count                      }

{= Bit des Befehlsregisters ($08, $D0) Schreiben ====================}

COMMAND_DACKLEVEL = $80; { Bit 7 gesetzt: DMA Acknowledge Leitung }
                         { HIGH aktiv                             }
COMMAND_DREQLEVEL = $40; { Bit 6 gesetzt: REQ Acknowledge Leitung }
                         { LOW aktiv                              }
COMMAND_EXTWRITE = $20; { Bit 5 gesetzt: EXTENDED Write, }
                        { sonst LATE Write               }
COMMAND_FIXEDPRI = $10; { Bit 4 gesetzt: feste Prioritt }
COMMAND_COMPRESS = $08; { Bit 3 gesetzt: komprimierte Taktung }
COMMAND_INACTIVE = $04; { Bit 2 gesetzt: Controller deaktiviert }
COMMAND_ADH0     = $02; { Bit 1 gesetzt: Adress Hold fr Kanal }
                        { 0/4 deaktiviert                      }
COMMAND_MEM2MEM  = $01; {- Bit 0 gesetzt: Speicher/Speicher, }
                        { sonst Speicher/Peripherie          }

{= Bits des Request-Registers ( $09, $D2 ) Schreiben ================}
REQUEST_RESERVED = $F8;            { Reserverte Bits immer =0 setzen }
REQUEST_SET      = $04;                         { DMA Request setzen }
REQUEST_CLR      = $00;                        { DMA-Request lschen }
REQUEST_MSK      = $03;       { In unteren beiden Bits Kanal angeben }

{= Bits des Kanalmaskierungsregisters ( $0A, $D4 ) Schreiben ========}
CHANNEL_RESERVED = $F8;            { Reserverte Bits immer =0 setzen }
CHANNEL_SET      = $04;                { DMA Kanal maskieren/sperren }
CHANNEL_CLR      = $00;                        { DMA Kanal freigeben }
CHANNEL_MSK      = $03;       { In unteren beiden Bits Kanal angeben }

{= Bits des Modusregisters ($0B, $D6) Schreiben =====================}
MODE_DEMAND     = $00;                  { "Auf Abruf" transferieren  }
MODE_SINGLE     = $40;                   { Einzelwerte transferieren }
MODE_BLOCK      = $80;                               { Blocktransfer }
MODE_CASCADE    = $C0;                    { Kaskadiert transferieren }
MODE_DECREMENT  = $20;                             { Dekrementierten }
MODE_AUTOINIT   = $10;               { Autoinitialisierung nach Ende }
MODE_VERIFY     = $00;                                { Verifizieren }
MODE_WRITE      = $04;                       { In Speicher Schreiben }
MODE_READ       = $08;                          { Aus Speicher Lesen }
MODE_INVALID    = $0C;                                    { Ungltig }
MODE_CHANNELMSK = $03;        { In unteren beiden Bits Kanal angeben }


Procedure dma_MasterClear( iChan : Integer );

Procedure dma_SetRequest( iChan : Integer );

Procedure dma_ClrRequest( iChan : Integer );

Procedure dma_SetMask( iChan : Integer );

Procedure dma_ClrMask( iChan : Integer );

Function dma_ReadStatus( iChan : Integer ) : Byte;

Procedure dma_ClrFlipFlop( iChan : Integer );

Function dma_ReadCount( iChan : Integer ) : Word;

Procedure dma_SetChannel( iChan : Integer;
                          lpMem : Pointer;
                          uSize : Word;
                          bMode : Byte );

Procedure dma_Mem2Mem(  bPage : Byte;
                        uSource, uDest,  uSize : Word );

Function dma_Until64kPage( lpMem : Pointer; uSize : Word ) : Word;

Procedure dma_Free( lpMem : Pointer );

Function dma_AllocMem( uSize : Word ) : Pointer;

{---------------------------------------------------------------------}

Implementation

uses DOS;
Const
 dma_adress    : array[0..7] of Byte=($00,$02,$04,$06,$C0,$C4,$C8,$CC);
 dma_count     : array[0..7] of Byte=($01,$03,$05,$07,$C2,$C6,$CA,$CE);
 dma_page      : array[0..7] of Byte=($87,$83,$81,$82,$88,$8B,$89,$8A);

 {-- Register-Offsets fr Master und Slave ---------------------------}
 dma_status    : array[0..1] of Byte=($08,$D0);     { Statusregister [Lesen] }
 dma_command   : array[0..1] of Byte=($08,$D0);{ Befehlsregister [Schreiben] }
 dma_request   : array[0..1] of Byte=($09,$D2);       { DMA-Request auslsen }
 dma_chmask    : array[0..1] of Byte=($0A,$D4);   { Kanle einzeln maskieren }
 dma_mode      : array[0..1] of Byte=($0B,$D6);              { Transfermodus }
 dma_flipflop  : array[0..1] of Byte=($0C,$D8);    { Adress-/Zhler-Flipflop }
 dma_masterclr : array[0..1] of Byte=($0D,$DA);        { Controller resetten }
 dma_temp      : array[0..1] of Byte=($0D,$DA);        { Temporres Register }
 dma_maskclr   : array[0..1] of Byte=($0E,$DC);      { Alle Kanle freigeben }
 dma_mask      : array[0..1] of Byte=($0F,$DE);    { Kanle gesamt maskieren }

{********************************************************************}
{ dma_Masterclear : zu einem Kanal gehrigen Controller resetten     }
{*------------------------------------------------------------------*}
{ Eingabe : iChan : Nummer des Kanals (0-7)                          }
{********************************************************************}
Procedure dma_MasterClear( iChan : Integer );

Begin
  iChan := iChan and $0007;
  port[dma_masterclr[iChan div 4]] := 0 ;
End;

{********************************************************************}
{ dma_SetRequest : bertragung auf angegebenem Kanal auslsen        }
{*------------------------------------------------------------------*}
{ Eingabe : iChan : Nummer des Kanals (0-7)                          }
{********************************************************************}
Procedure dma_SetRequest( iChan : Integer );

Begin
  iChan :=iChan and $0007;
  port[dma_request[iChan div 4]] := REQUEST_SET or ( iChan and $03 ) ;
End;

{********************************************************************}
{ dma_ClrRequest : bertragung auf angegebenem Kanal stoppen         }
{*------------------------------------------------------------------*}
{ Eingabe : iChan : Nummer des Kanals (0-7)                          }
{********************************************************************}
Procedure dma_ClrRequest( iChan : Integer );

Begin
  iChan := iChan and $0007;
  port[dma_request[iChan div 4]] := REQUEST_CLR or ( iChan and $03 );
End;

{********************************************************************}
{ dma_SetMask : Angegebenen Kanal maskieren (sperren)                }
{*------------------------------------------------------------------*}
{ Eingabe : iChan : Nummer des Kanals (0-7)                          }
{********************************************************************}
Procedure dma_SetMask( iChan : Integer );

Begin
  iChan :=iChan and $0007;
  port[dma_chmask[iChan div 4]] := CHANNEL_SET or ( iChan and $03 );
End;

{********************************************************************}
{ dma_ClrMask : Angegebenen Kanal freigeben                          }
{*------------------------------------------------------------------*}
{ Eingabe : iChan : Nummer des Kanals (0-7)                          }
{********************************************************************}
Procedure dma_ClrMask( iChan : Integer );

Begin
  iChan :=iChan and $0007;
  port[dma_chmask[iChan div 4]] := CHANNEL_CLR or ( iChan and $03 );
End;

{********************************************************************}
{ dma_ReadStatus : Status des zum angegebeben Kanal zugehrigen      }
{                  Controllers lesen                                 }
{*------------------------------------------------------------------*}
{ Eingabe : iChan : Nummer des Kanals (0-7)                          }
{ Ausgabe : Status des Controllers                                   }
{********************************************************************}
Function dma_ReadStatus( iChan : Integer ) : Byte;

Begin
  iChan :=iChan and $0007;
  dma_ReadStatus := port[dma_status[iChan div 4]];
End;

{********************************************************************}
{ dma_ClrFlipFlop : FlipFlop des zum angegebeben Kanal zugehrigen   }
{                   Controllers zurcksetzen                         }
{*------------------------------------------------------------------*}
{ Eingabe : iChan : Nummer des Kanals (0-7)                          }
{*------------------------------------------------------------------*}
{ Info : Das FlipFlop dient zur Unterscheidung von LO- und HI-Byte   }
{        der Transferadresse bzw. des Transferzhlers.               }
{********************************************************************}
Procedure dma_ClrFlipFlop( iChan : Integer );

Begin
  iChan :=iChan and $0007;
  port[dma_flipflop[iChan div 4]] := 0;
End;

{********************************************************************}
{ dma_ReadCount : Transferzhler des angegebenen Kanals lesen        }
{*------------------------------------------------------------------*}
{ Eingabe : iChan : Nummer des Kanals (0-7)                          }
{ Ausgabe : Aktueller Transferzhler (0-65535)                       }
{********************************************************************}
Function dma_ReadCount( iChan : Integer ) : Word;

var l, h : Byte;

Begin
  iChan := iChan and $0007;

  dma_ClrFlipFlop( iChan );
  l := port[dma_count[iChan]];
  h := port[dma_count[iChan]];
  dma_ReadCount := h * 256 + l;
End;

{********************************************************************}
{ dma_SetChannel : DMA-Kanal zum Transfer vorbereiten                }
{*------------------------------------------------------------------*}
{ Eingabe : iChan  : Nummer des Kanals (0-7)                         }
{           lpMem  : Adresse des Speichers                           }
{           uSize  : Anzahl zu transferierEnder Bytes                }
{           bMode  : bertragungsmodus                               }
{           bAlign : DMA_BIT8 oder DMA_BIT16 bertragung             }
{*------------------------------------------------------------------*}
{ Info : Um 16-Bit bertragungen zu ermglichen, mu die dem DMA-    }
{        Kontroller bergebene lineare Adresse mit 2 multipliziert   }
{        werden. Bei 16-Bit bertragungen knnen bis 128K Daten      }
{        transferiert werden.                                        }
{********************************************************************}
Procedure dma_SetChannel( iChan : Integer;
                          lpMem : Pointer;
                          uSize : Word;
                          bMode : Byte );
var  uAdress : Word;
     bPage   : Byte;

Begin
  iChan := iChan and $0007;                      { Max. 8 DMA-Kanle }

  dma_SetMask( iChan );                             {- Kanal sperren }
                       { DMA transferiert 1 Byte mehr als angegeben! }
                                    { mind. 1 Byte (0==1) bertragen }
  if uSize <> 0 then usize := usize - 1;

  {- lineare 20-Bit Adresse erzeugen --------------------------------}
  if iChan <= 3 then                                      { 8Bit DMA }
    Begin                 { Adresse = untere 16 Bit der 20-Bit Adresse }
      uAdress := Word ( ( Longint ( lpMem ) and $FFFF0000 ) shr 12 )
                 + ( Longint ( lpMem ) and $FFFF );
                            { Seite = obere 4 Bit der 20-Bit Adresse }
      bPage := Byte ( ( ( ( ( Longint ( lpMem ) ) and $FFFF0000 ) shr 12 )
               + ( Longint ( lpMem ) and $FFFF ) ) shr 16 );
    End
  else                                                  { 16-Bit DMA }
    Begin                 { Adresse = untere 16 Bit der 20-Bit Adresse }
      uAdress := Word ( ( Longint ( lpMem ) and $FFFF0000 ) shr 13 )
                 + ( ( Longint ( lpMem ) and $FFFF ) shr 1 );
                            { Seite = obere 4 Bit der 20-Bit Adresse }
      bPage := Byte ( ( ( ( Longint ( lpMem ) and $FFFF0000 ) shr 12 )
             + ( Longint ( lpMem ) and $FFFF ) ) shr 16 );
      bPage := bpage and $FE;
      uSize := usize div 2; { Es werden Words und nicht Bytes gezhlt! }
    End;

  port[dma_mode[iChan div 4]] := bMode or ( iChan and MODE_CHANNELMSK );

  dma_ClrFlipFlop( iChan );   { Adress-/Zhlerflipflop lschen und...}
                { Adresse an DMA-Controller bermitteln (LO/HI-Byte) }
  port[dma_adress[iChan]] := LO( uAdress );
  port[dma_adress[iChan]] := HI( uAdress );
  port[dma_page[iChan]] := bPage;            { Speicher-Seite setzen }

  dma_ClrFlipFlop( iChan );  { Adress-/Zhlerflipflop lschen und ...}
                 { Zhler an DMA-Controller bermitteln (LO/HI-Byte) }
  port[dma_count[iChan]] := LO( uSize );
  port[dma_count[iChan]] := HI( uSize );

  dma_ClrMask( iChan );                 { DMA-Kanal wieder freigeben }
End;

{********************************************************************}
{ dma_Mem2Mem : Speicherblock innerhalb einer 64K Seite verschieben  }
{*------------------------------------------------------------------*}
{ Eingabe : bPage   : Adresse der 64K-Seite                          }
{           uSource : Quelle                                         }
{           uDest   : Ziel                                           }
{           uSize   : Anzahl zu transferierEnder Bytes               }
{*------------------------------------------------------------------*}
{ Hinweis: Der Speicher-Speicher DMA Transfer ist bei neueren        }
{          Rechnern (ab 80286) nicht implementiert. Im Protected-    }
{          oder Virtual-8086-Mode des Prozessors werden DMA0         }
{          Zugriffe generell abgefangen und fhren zu einem          }
{          Systemabsturz.                                            }
{********************************************************************}
Procedure dma_Mem2Mem( bPage : Byte;
                       uSource, uDest, uSize : Word );

Begin
  dma_MasterClear( 0 );    { ber Kanal 0 Master-Controller resetten }
                       { DMA transferiert 1 Byte mehr als angegeben! }
                                    { mind. 1 Byte (0==1) bertragen }
  if uSize<>0 then uSize :=uSize - 1 ;

  port[dma_adress[0]] := LO( uSource );
  port[dma_adress[0]] := HI( uSource );
  port[dma_page[0]] := bPage;                { Speicher-Seite setzen }

  port[dma_adress[1]] := LO( uDest );
  port[dma_adress[1]] := HI( uDest );
  port[dma_page[1]] := bPage;                { Speicher-Seite setzen }

                 { Zhler an DMA-Controller bermitteln (LO/HI-Byte) }
  port[dma_count[0]] := LO( uSize );
  port[dma_count[0]] := HI( uSize );

      { dma_ClrFlipFlop( 1 );  Adress-/Zhlerflipflop lschen und ...}
  port[dma_count[1]] := LO( uSize );
  port[dma_count[1]] := HI( uSize );

    { Durch setzen von COMMAND_ADH0 wird Adresseregister von Kanal 0 }
    { nicht Inkrementiert. Der Speicherblock den Kanal2 bearbeitet   }
    { wird somit mit einem Wert initialisiert.                       }
  port[dma_command[0]] := COMMAND_MEM2MEM or
                          COMMAND_FIXEDPRI or
                          COMMAND_COMPRESS;

   { Wenn sich die Speicherblcke berlappen mu zw. Inkrementierung }
   { und Dekrementierung unterschieden werden (MODE_DECREMENT). Bei  }
   { Dekrementieren mu dem jeweiligen DMA-Kanal als Adresse die     }
   { Speicheradress + Anzahl zu kopierEnder Bytes bergeben werden!  }
  port[dma_mode[0]] := MODE_BLOCK or MODE_READ  or 0;      { Kanal 0 }
  port[dma_mode[0]] := MODE_BLOCK or MODE_WRITE or 1;      { Kanal 1 }

  dma_SetRequest( 0 );
End;

{********************************************************************}
{ dma_Until64kPage : Anzahl der Bytes eines bergebenen Speicher-    }
{                    bereichs innerhalb dessen 64k Seite ermitteln.  }
{*------------------------------------------------------------------*}
{ Eingabe : lpMem : Adresse des Speichers der untersucht werden soll }
{           uSize : Gre des Speicherbereichs                       }
{ Ausgabe : Anzahl der Bytes innerhalb des Speichers bis zum         }
{           erreichen des SpeicherEndes oder der nchsten 64k-Seite  }
{********************************************************************}
Function dma_Until64kPage( lpMem : Pointer; uSize : Word ) : Word;

var lAdr, lPageEnd : Longint;

Begin
  {-- Ermittle 20Bit lineare Adresse aus far Zeiger -----------------}
  lAdr := ( Longint ( Longint ( lpMem ) shr 16 ) shl 4 ) +
          ( Longint ( lpMem ) and $ffff );
  lPageEnd := lAdr + $FFFF;                       { 1 Seite addieren }
                                    { Bytes bis Seitenende ermitteln }
  lPageEnd := lPageEnd and $FFFF0000;
  lPageEnd := lPageEnd - lAdr;

  if lPageEnd>uSize then dma_Until64kPage := uSize
                    else dma_Until64kPage := lPageEnd;
End;

{********************************************************************}
{ dma_FreeMem : DMA fhigen Speicher freigeben.                      }
{*------------------------------------------------------------------*}
{ Eingabe : uSize : Gre des allokierten Speichers.                 }
{********************************************************************}
Procedure dma_Free( lpMem : Pointer);

var regs : Registers;

Begin
                    { Segment ermitteln und ber DOS-Call freigeben }
  regs.ah := $49;
  regs.es := Longint ( lpMem ) shr 16;
  msdos( regs );
End;

{********************************************************************}
{ dma_AllocMem : DMA fhigen Speicher allokieren.                    }
{*------------------------------------------------------------------*}
{ Eingabe : uSize : Gre des zu allokierEnden Speichers (max. 64k). }
{ Ausgabe : Adresse des allokierten Speichers.                       }
{*------------------------------------------------------------------*}
{ Info : - Diese Funktion allokiert solange Speicher, bis der        }
{          gefundene Speicher innerhalb einer physischen 64K Grenze  }
{          liegt. Hinweis: Die Allokationsstrategie ist NICHT        }
{          optimal (s. HeapWalk).                                    }
{          Der allokierte Speicher Beginnt an einer Segment-Adresse  }
{          und ist somit Page- bzw. 16Byte-Aligned.                  }
{********************************************************************}
Function dma_AllocMem( uSize : Word ) : Pointer;

Const
  uSegment    : Word = $FFFF;
  uOldSegment : Word = $FFFF;
var
  test : Longint;
  regs : Registers;
Begin
                             { Kann noch Speicher allokiert werden ? }
  Repeat
    regs.ah := $48;
    regs.bx := uSize div 16 + 1;
    msdos( regs );

    if ( regs.flags and 1 ) = 0 then
    Begin
      usegment := regs.ax;

      if uOldSegment <> $FFFF then       { alten Speicher freigeben }
        dma_Free( Pointer ( uOldSegment shl 16 ) );
      uOldSegment := uSegment;              { Neuen Speicher merken }
                              { liegt neuer Speicher in 64K Seite ? }
      if dma_Until64kPage(
             Pointer ( Longint ( usegment ) shl 16), uSize ) = uSize  then
        Begin
          DMA_AllocMem := Pointer (Longint ( uSegment ) shl 16 );
          Exit;
        End;
    End;
  Until regs.flags > 0;
  if uSegment <> $FFFF then                { Schade, kein Speicher! }
      dma_Free( Pointer ( uOldSegment shl 16 ) );
  DMA_AllocMem := NIL;
End;

End.
