{********************************************************************}
{                       D S P U T I L  . P A S                       }
{*------------------------------------------------------------------*}
{ Aufgabe        : Stellt Funktionen zur Programmierung des Sound-   }
{                  blaster DSP's zur Verfgung.                      }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 20.03.1994                                        }
{ letztes Update : 10.08.1994                                        }
{*------------------------------------------------------------------*}
{ Hinweis:  Wird dieses Modul mit definiertem Symbol DSP_VERSIONONLY }
{           compiliert, werden nur die zur Ermittlung der Version    }
{           bentigten Funktionen compiliert!                        }
{********************************************************************}

{$X+}   { erweiterte Syntax, Auswertung von Fkt-Ergebnissen optional }

Unit DSPUTIL;

Interface
Uses SBUTIL, DOS;

Type

DSPRECPLAY = record
  iStereo    : Boolean;
  iBits      : Integer;
  uFrequency : Word;
End;

UserFunc     = Procedure( l : Longint ); { Callback-Routinen }
DSP4UserFunc = Procedure( b : Byte );
OldIntproc   = Procedure;              

const
 DSP_ERRRESET     =  1;                                { Fehlercodes }
 DSP_ERRVERSION   =  2;
 DSP_ERRFRQ       =  3;
 DSP_ERR4DACFRQ   =  4;
 DSP_ERR4ADCFRQ   =  5;
 DSP_ERRSPEAKER   =  6;
 DSP_ERRTRANSSIZE =  7;
 DSP_ERRILLSIZE   =  8;
 DSP_ERR8DAC      =  9;
 DSP_ERR8ADC      =  10;
 DSP_ERRPLAY      =  11;
 DSP_ERRRECORD    =  12;

 FRQ1_MONO_ADC    =  13000;           { Hchste AD-Sample Frequenzen }
 FRQ2p_MONO_ADC   =  13000;                      { DSP Version 2.01+ }
 FRQ2p_HIMONO_ADC =  15000;
 FRQ3_MONO_ADC    =  23000;
 FRQ3_HIMONO_ADC  =  44100;
 FRQ3_STEREO_ADC  =  22050;
 FRQ4_ADC         =  44100;

 FRQ1_MONO_DAC    =  23000;           { Hchste DA-Sample Frequenzen }
 FRQ2p_MONO_DAC   =  23000;                      { DSP Version 2.01+ }
 FRQ2p_HIMONO_DAC =  44100;
 FRQ3_MONO_DAC    =  23000;
 FRQ3_HIMONO_DAC  =  44100;
 FRQ3_STEREO_DAC  =  22050;
 FRQ4_DAC         =  44100;

 DSP_1XX = $100;
 DSP_200 = $200;
 DSP_201 = $210;
 DSP_3XX = $300;                                   { Versionsnummern }
 DSP_4XX = $400;

                                              { DSP-Register-Offsets }
 DSP_WRITESTATUS  = $0C;       { Bit 7 zeigt an ob Schreiben erlaubt }
 DSP_WRITECMDDATA = $0C;                              { Schreibdaten }

 DSP_READSTATUS   =    $0E;        { Bit 7 zeigt an ob Lesen erlaubt }
 DSP_IRQACK       =    $0E;                   { Interruptacknowledge }
 DSP_READDATA     =    $0A;                              { Lesedaten }

 DSP_RSET         =    $06;                             { Reset-Port }

 DSP_GETVER       =    $E1;                          { DSP-Kommandos }
 DSP_8DA          =    $10;
 DSP_8DMADAC      =    $14;
 DSP_8DMAAUTODAC  =    $1C;

 DSP_8AD          =    $20;
 DSP_8DMAADC      =    $24;
 DSP_8DMAAUTOADC  =    $2C;

 DSP_SETTIMECONSTANT = $40;
 DSP_SETTRANSSIZE    = $48;

 DSP_DMAPAUSE        = $D0;
 DSP_DMACONTINUE     = $D4;

 DSP2p_8DMAHIAUTODAC = $90;
 DSP2p_8DMAHIDAC     = $91;

 DSP2p_8DMAHIAUTOADC = $98;
 DSP2p_8DMAHIADC     = $99;

 DSP_SPEAKERON       = $D1;
 DSP_SPEAKEROFF      = $D3;
 DSP2p_SPEAKERSTATUS = $D8;
 DSP4_EXIT16DMA      = $D9;
 DSP4_EXIT8DMA       = $DA;
 DSP2p_EXITAUTOINIT  = $DA;

 DSP3_MONOADC        = $A0;                                   { DSP3 }
 DSP3_STEREOADC      = $A8;

 DSP4_DACFR          = $41;
 DSP4_ADCFR          = $42;

 DSP4_CMDADC         = $08;              { Kommando-Bit fr Aufnahme }
 DSP4_CMDAUTOINIT    = $04;                 { Bit fr Autoinit Modus }
 DSP4_CMDFIFO        = $02;
 DSP4_CMDDAC         = $00;
 DSP4_CMD8DMA        = $C0;             { Hi-Nibble fr 8 und 16-Bit }
 DSP4_CMD16DMA       = $B0;             { Kommandos                  }

 DSP4_MODESTEREO   =  $20;
 DSP4_MODEMONO     =  $00;
 DSP4_MODESIGNED   =  $10;
 DSP4_MODEUNSIGNED =  $00;

 _SB_NONE = 0;
 _SB_1XX  = 1;                           { DSP-Version 1.00 bis 1.99 }
 _SB_2XX  = 2;                           { DSP-Version 2.00 bis 2.99 }
 _SB_3XX  = 3;                           { DSP-Version 3.00 bis 3.99 }
 _SB_4XX  = 4;                           { DSP-Version 4.00 bis 4.99 }
 _SB_XXX  = 5;                           { DSP-Version > 4.99 }

 ON       = TRUE;
 OFF      = FALSE;

Procedure DummyUserFunc( l : Longint );
Procedure DummyDSP4UserFunc( b : Byte );

Function _dos_open( FileName: String ) : Integer;

Procedure _dos_close( handle : integer );

Function _dos_read( handle : integer; Buf : Pointer; cnt : Integer ) : word;

Function _dos_write( handle : integer; Buf : Pointer; cnt : Integer ) : word;

Function _dos_filesize( handle : integer ) : longint;

Function dsp_SetBase( var SBBASE : SBBASE;
                          iReset : Boolean ) : Word;

Function dsp_Write( iVal : Word ) : Word;

Function dsp_Read( var Val : Byte) : Word;

Function dsp_Reset : Word;

Function dsp_GetVersion(var SBBASE : SBBASE) : Word;

Function dsp_AdjustFrq( var Frq    : Word;
                            iADC   : Boolean;
                        var Stereo : Boolean ) : Boolean;

Function dsp4_DACFrq( uFrq : Word ) : Word;

Function dsp4_ADCFrq( uFrq : Word ) : Word;

Function dsp_SetFrq( var Frq : Word ) : Word;

Function dsp_CanStereo : Boolean;

Function dsp_IsHIMONOADCFrq( uFrq : Word ) : Boolean;

Function dsp_IsHIMONODACFrq( uFrq : Word ) : Boolean;

Function dsp_MaxBits : Integer;

Function dsp_SetSpeaker( iState : Boolean ) : Word;

Function dsp_SetTransferSize( uSize : Word ) : Word;

Procedure dsp_IrqHandler; interrupt;

Procedure dsp_RestoreIrqHandler;

Procedure dsp_InitIrqHandler;

Procedure dsp_SetUserIRQ( lpFunc : UserFunc );

Procedure dsp4_SetUserIRQ( lpFunc : DSP4UserFunc );

Function dsp_WaitForNextIRQ( lpDoSomething : userfunc;
                             lpar          : Longint ) : Boolean;

Procedure dsp_InitWaitForIRQ;

Function dsp_8DAC( bVal : Byte ) : Word;

Function dsp_8ADC( var Val : Byte ) : Word;

Function dsp_PLAY( pBuffer       : pointer;
                   uSize, uDelay : Word ) : Word;

Function dsp_RECORD( pBuffer       : pointer;
                     uSize, uDelay : Word ) : Word;

Procedure dsp_InitBuffers;

Function dsp_FileOpen( Filename   : string;
                       var Handle : Integer ) : Integer;

Function dsp_FileClose( var Handle : Integer ) : Integer;

Function dsp_ReadHeader( var Handle : Integer;
                         var DRP    : DSPRECPLAY ) : Word;

Function dsp_WriteHeader( var Handle : Integer;
                          var DRP    : DSPRECPLAY  ) : Word;

Function dsp_ReadBuffer( var Handle    : Integer;
                             lpBuffer  : pointer;
                             iHalfSize : Integer ) : Word;

Function dsp_WriteBuffer( var Handle    : Integer;
                              lpBuffer  : pointer;
                              iHalfSize : Integer ) : Word;

Procedure dsp_ClearBuffer( lpBuffer : pointer; uMemSize : Word );

Procedure dsp_DoRECPLAY( var Handle   : Integer;
                             iADC     : Boolean;
                             iSource  : Integer;
                         var DRP     : DSPRECPLAY;
                             iSecs    : Integer;
                             lpBuffer : pointer;
                             uMemSize : Word );

Implementation

Uses MIXUTIL,IRQUTIL,DMAUTIL;

type SampleArrayB = array[0..65534] of byte; { Array mit Sample-Daten }
type SampleArrayBPtr = ^SampleArrayB; { Zeiger auf dieses Array }
type BPTR = ^BYTE;

{-- Globale Variablen -----------------------------------------------}
Const
 pDspNames : Array[0..5] of string = ( 'Unbekannte Soundkarte',
                                       'SB 1.5 oder SB MCV',
                                       'SB 2.0',
                                       'SB Pro oder SB Pro MCV',
                                       'SB 16, SB 16 ASP',
                                       '> SB16' );

 { Welcher Bufferteil soll als nchstes geladen/gespeichert werden ? }
 iBufCnt : Integer = 0;

var DSPBASE : SBBASE;

{$ifndef DSP_VERSIONONLY}

lpUserFunc     : UserFunc;
lpDSP4UserFunc : DSP4UserFunc;
lpOldInt       : OldIntProc;

const
lIRQServed     : Longint = 0;
lOldIRQServed  : Longint = 0;
{$Endif}

{-- Funktionen ------------------------------------------------------}

{********************************************************************}
{ _dos_open : Datei mittels DOS ffnen (lesen und schreiben)         }
{*------------------------------------------------------------------*}
{ Eingabe : FileName : Name der zu ffnenden Datei                   }
{ Ausgabe : gltiges Filehandle oder -1                              }
{*------------------------------------------------------------------*}
{ Hinweis : Die Pascal-Dateifunktionen knnen nicht verwendet werden,}
{           weil sie sich grundstzlich nur auf Objekte auf dem      }
{           Heap beziehen. Der DMA-Buffer kann jedoch nicht ber den }
{           Heap allokiert werden.                                   }
{********************************************************************}
Function _dos_open( FileName: String ) : Integer;
var regs : Registers;
Begin
     regs.ah := $3D;
     regs.al := $2; { lesen und schreiben }
     FileName := FileName + #0;
     regs.ds := LongInt( @Filename[1] ) shr 16;           { Segment }
     regs.dx := LongInt( @FileName[1] ) and $FFFF; { Offset }

     msdos( regs );
     if ( regs.Flags and 1 ) <> 0 then
          _dos_open := -1
     else _dos_open := regs.ax;
End;

{********************************************************************}
{ _dos_close : DOS-Datei schlieen                                   }
{*------------------------------------------------------------------*}
{ Eingabe : handle : Handle der vorher durch _dos_open geffneten    }
{                    Datei.                                          }
{********************************************************************}
Procedure _dos_close( handle : integer );
var regs : Registers;
Begin
     if handle >= 0 then
     Begin
       regs.ah := $3E;
       regs.bx := handle;
       msdos( regs );
     End;
End;

{********************************************************************}
{ _dos_read : Daten aus DOS-Datei lesen                              }
{*------------------------------------------------------------------*}
{ Eingabe : handle : Handle der DOS-Datei                            }
{           Buf    : Untypisierter Zeiger auf Lesebuffer             }
{           cnt    : Anzahl der zu lesenden Zeichen                  }
{********************************************************************}
Function _dos_read( handle : integer; Buf : Pointer; cnt : Integer ) : word;
var regs : Registers;
Begin
     if handle >= 0 then
     Begin
       regs.ah := $3F;
       regs.bx := handle;
       regs.cx := cnt;
       regs.ds := LongInt( Buf ) shr 16;           { Segment }
       regs.dx := LongInt( Buf ) and $FFFF; { Offset }
       msdos( regs );
       if ( regs.Flags and 1 ) <> 0 then
          _dos_read := 0
       else _dos_read := regs.ax;
     End;
End;

{********************************************************************}
{ _dos_write: Daten in DOS-Datei schreiben                           }
{*------------------------------------------------------------------*}
{ Eingabe : handle : Handle der DOS-Datei                            }
{           Buf    : Untypisierter Zeiger auf Schreibdaten           }
{           cnt    : Anzahl der zu schreibenden Zeichen              }
{********************************************************************}
Function _dos_write( handle : integer; Buf : Pointer; cnt : Integer ) : word;
var regs : Registers;
Begin
     if handle >= 0 then
     Begin
       regs.ah := $40;
       regs.bx := handle;
       regs.cx := cnt;
       regs.ds := LongInt( Buf ) shr 16;           { Segment }
       regs.dx := LongInt( Buf ) and $FFFF; { Offset }
       msdos( regs );
       if (regs.Flags and 1) <> 0 then
          _dos_write := 0
       else _dos_write := regs.ax;
     End;
End;

{********************************************************************}
{ _dos_filesize: Lnge einer DOS-Datei ermitteln                     }
{*------------------------------------------------------------------*}
{ Eingabe : handle : Handle der DOS-Datei                            }
{ Ausgabe : Lnge der Datei                                          }
{*------------------------------------------------------------------*}
{ Hinweis : Um die Lnge einer Datei zu ermitteln wird wie folgt ver-}
{           fahren:                                                  }
{           1.) aktuellen Dateizeiger merken                         }
{           2.) Dateizeiger ans Ende der Datei bewegen und           }
{               aktuelle Position zurckgeben                        }
{           3.) Dateizeiger zurck an alte Position bewegen          }
{********************************************************************}
Function _dos_filesize( handle : integer ) : longint;
var regs : Registers;
    actposl, actposh : WORD;  { aktuelle Dateizeiger-Position }
Begin
     _dos_filesize := 0; { zunchst Fehler kennzeichnen }
     if handle >= 0 then
     Begin
       regs.ah := $42;  { aktuelle Position lesen }
       regs.al := 1;    { aktuelle Datei-Position }
       regs.bx := handle;
       regs.cx := 0;     { kein Offset }
       regs.dx := 0;
       msdos( regs );
       if regs.FLags and 1 = 0 then
         Begin
           actposl := regs.ax;
           actposh := regs.dx;
         End
       else exit;

       regs.ah := $42;  { Position am Dateiende lesen }
       regs.al := 2;    { relativ zu Ende der Datei positionieren }
       regs.bx := handle;
       regs.cx := 0;     { Offset = 0 => Dateiende }
       regs.dx := 0;
       msdos( regs );
       if regs.Flags and 1 = 0 then
         Begin
           _dos_filesize := longint( regs.dx ) shl 16 +
                            longint ( regs.ax );
         End
       else exit;

       regs.ah := $42;  { alte Position restaurieren }
       regs.al := 0;    { relativ zu Dateianfang }
       regs.bx := handle;
       regs.cx := actposh;     { kein Offset }
       regs.dx := actposl;
       msdos( regs );
       if regs.Flags and 1 <> 0 then
       Begin
         _dos_filesize := 0;
         exit;
       End;
     End;
End;


{********************************************************************}
{ DummyUserFunc : Dummy-Funktion fr den Prozedur-Typ UserFunc       }
{********************************************************************}
Procedure DummyUserFunc( l : Longint );
begin
end;

{********************************************************************}
{ DummyDSP4UserFunc : Dummy-Funktion fr den Prozedur-Typ            }
{                     DSP4UserFunc                                   }
{********************************************************************}
Procedure DummyDSP4UserFunc( b : Byte );
begin
end;

{********************************************************************}
{ dsp_SetBase : Setze SBBase-Struktur fr Gebrauch der dsp_???       }
{               Funktionen.                                          }
{*------------------------------------------------------------------*}
{ Eingabe : SBBASE - die durch SB_SetEnviron initialisierte Struktur.}
{           iReset - DSP soll zurckgesetzt werden                   }
{ Ausgabe :                 == 0 - DSP-Funktionen konnten ordnungs-  }
{                                  gem initialisiert werden        }
{           DSP_ERRRESET    == 1 - Fehler bei DSP-Reset              }
{           DSP_ERRVERSION  == 2 - Fehler bei Ermittlung der V-Nummer}
{                           ==-1 - Keine SBBASE-Struktur bergeben   }
{                                  oder iDspPort == -1               }
{********************************************************************}
Function dsp_SetBase( var SBBASE : SBBASE; iReset : Boolean ) : Word;

Begin
  if @SBBASE <> NIL then
    if SBBASE.iDspPort <> -1 then
      Begin
        DSPBASE := SBBASE;
        if iReset then if dsp_Reset <> 0 then
          Begin
            dsp_SetBase := DSP_ERRRESET;
            Exit;
          End;

      if dsp_GetVersion( SBBASE ) <> 0 then
        Begin
          dsp_SetBase := DSP_ERRVERSION;
          Exit;
        End;

      if SBBASE.uDspVersion >= DSP_4XX then
        if SBBASE.iDspDmaW = -1 then
            SBBASE.iDspDmaW  := SBBASE.iDspDmaB;

{$ifndef DSP_VERSIONONLY}
      mix_SetBase( SBBASE, FALSE );
{$Endif}

      DSPBASE := SBBASE;
      dsp_SetBase := NO_ERROR;
    End
  else dsp_SetBase := ERROR;
End;

{********************************************************************}
{ dsp_write :  Daten an DSP bergeben                                }
{*------------------------------------------------------------------*}
{ Eingabe : iVal - zu bermittelndes Daten-Byte oder Kommando        }
{ Ausgabe : == 0 - Datum konnte nicht bertragen werden              }
{           <> 0 - Datum wurde an DSP bermittelt                    }
{********************************************************************}
Function dsp_Write( iVal : Word ) : Word;

Const cx : Word = 65535;

Begin                  { Interner Zhler == 0 => bertragungsfehler! }
      { Write-Status lesen, und warten bis OK. Zhler dekrementieren }
  While ( ( ( port[DSPBASE.iDspPort + DSP_WRITESTATUS] and $80 ) <> 0 )
              and ( cx <> 0 ) ) do
    Dec(cx);
        { Zhler != 0, dann Write-Status innerhalb vorgegebener Zeit }
        { bertragung eines DatenBytes also erlaubt :                }
  if cx > 0 then port[DSPBASE.iDspPort + DSP_WRITECMDDATA] := Byte ( iVal );
  dsp_write := cx;          { Zhler als Fehlerindikator zurckgeben }
End;

{********************************************************************}
{ dsp_Read : Daten vom DSP lesen                                     }
{*------------------------------------------------------------------*}
{ Eingabe : Val - Byte-Variable, die den gelesenen Wert entgegen-    }
{                 nehmen soll (Ausgabeparameter)                     }
{ Ausgabe : == 0 - DSP stellte kein Datum zur Verfgung              }
{           <> 0 - DSP stellte Datum zur Verfgung                   }
{*------------------------------------------------------------------*}
{ Info : - Das Auslesen der DSP-Daten macht nur Sinn, wenn vorher    }
{          ein entsprechEnder Abfrage-Befehl gesendet wurde.         }
{********************************************************************}
Function dsp_Read( var Val : Byte ) : Word;

Const cx : Word = 65535;

Begin                  { Interner Zhler == 0 => bertragungsfehler! }
       { Read-Status lesen, und warten bis OK. Zhler dekrementieren }
  while ( not ( ( ( port[DSPBASE.iDspPort + DSP_READSTATUS] and $80 ) <> 0)
          and ( cx <> 0 ) ) ) do
    Dec(cx);
     { Zhler != 0, dann Read-Status innerhalb vorgegebener Zeit OK: }
     { Lesen eines DatenBytes also erlaubt:                          }
  if cx > 0 then Val := Byte ( port[DSPBASE.iDspPort + DSP_READDATA] );
  dsp_Read := cx;           { Zhler als Fehlerindikator zurckgeben }
End;

{********************************************************************}
{ dsp_Reset : DSP in Ausgangszustand zurckversetzen                 }
{*-------------------------------------------------------------------}
{ Ausgabe : NO_ERROR( 0 ) - Reset ordnungsgem ausgefhrt           }
{           ERROR ( -1 )  - Reset nicht ausgefhrt                   }
{*------------------------------------------------------------------*}
{ Info : - Reset-Sequenz - Zuerst 1, dann 0 an Reset-Port des DSP    }
{                          bermitteln, dann DSP-Auslesen und auf    }
{                          Empfang von $AA warten.                   }
{********************************************************************}
Function dsp_Reset : Word;

Const bVal : Byte = 0;

var  dummy : Byte;

Begin
                   { Reset-Port zuerst mit 1, dann mit 0 beschreiben }
  port[DSPBASE.iDspPort + DSP_RSET] := 1 ;
  dummy := port[DSPBASE.iDspPort + DSP_RSET]; { mind. 3              }
  dummy := port[DSPBASE.iDspPort + DSP_RSET]; { Microsekunden warten }
  dummy := port[DSPBASE.iDspPort + DSP_RSET];
  port[DSPBASE.iDspPort + DSP_RSET] := 0;
  dummy := port[DSPBASE.iDspPort + DSP_RSET]; { mind. 3              }
  dummy := port[DSPBASE.iDspPort + DSP_RSET]; { Microsekunden warten }
  dummy := port[DSPBASE.iDspPort + DSP_RSET];
  dsp_Read( bVal );                             { Wert von DSP-Lesen }
  if bVal = $AA then dsp_Reset := NO_ERROR
                else dsp_Reset := ERROR;          { Wert = $AA => OK }
End;

{********************************************************************}
{ dsp_GetVersion : DSP-Versionsnummer ermitteln                      }
{--------------------------------------------------------------------}
{ Eingabe : SBBASE - die zu initialisierende SBase-Struktur          }
{ Ausgabe : NO_ERROR ( 0 )       - kein Fehler                       }
{           DSP_ERRVERSION ( 2 ) - bertragungsfehler                }
{*------------------------------------------------------------------*}
{ Info : - Neben dem Feld uDspVersion wird auch der Name der         }
{          entdeckten SoundBlaster-Karte angegeben                   }
{********************************************************************}
Function dsp_GetVersion( var SBBASE : SBBASE) : Word;

var bV1, bV2 : Byte;

Begin
  if  ( ( dsp_Write(DSP_GETVER ) <> 0 )
    and ( dsp_Read( bV1 ) <> 0 )
    and ( dsp_Read( bV2 ) <> 0 ) ) then
    Begin                                       { Versions-Wort bilden }
      SBBASE.uDspVersion := bV1 * 256 + bV2;
      if ( ( bV1 >= 1 ) and ( bV1 <= 4 ) ) then
        SBBASE.pDspName := pDspNames[bV1]
      else
        SBBASE.pDspName := pDspNames[_SB_NONE];
      dsp_GetVersion := NO_ERROR;
    End
  else                             { Fehler bei DSP-Datenbertragung }
    dsp_GetVersion := DSP_ERRVERSION;
End;                               

{$ifndef DSP_VERSIONONLY}           { Nur Versionkontrolle verwenden }
{********************************************************************}
{ dsp_AdjustFrq : Frequenz an Soundkarte anpassen                    }
{*------------------------------------------------------------------*}
{ Eingabe : Frq     - WORD-Variable, die die gewnschte Frequenz     }
{                     enthlt                                        }
{           iADC    - Aufnahme (TRUE) oder Wiedergabe (FALSE)        }
{           iStereo - Stereomodus (TRUE/FALSE)                       }
{ Ausgabe : TRUE - Mono/Stereo gltig                                }
{           FALSE - kein Stereomodus                                 }
{*------------------------------------------------------------------*}
{ Info : Diese Funktion berprft ob die gegebene Frequenz auf der   }
{        aktuell vorhandenen Soundblaster Karte verwEndet werden     }
{        kann, bzw. setzt andernfalls die grtmgliche Frequenz.    }
{        Wird versucht, den Stereomodus einer Karte zu aktivieren,   }
{        die dies nicht zult, wird FALSE zurckgegegen.            }
{        Einige der hohen Frequenzen knnen nur von den HI-SPEED     }
{        Kommandos verwendet werden.                                 }
{********************************************************************}
Function dsp_AdjustFrq( var Frq    : Word;
                            iADC   : Boolean;
                        var Stereo : Boolean ) : Boolean;

Begin
  if iADC then
    Begin { Aufnahme }
      if DSPBASE.uDspVersion >= DSP_4XX then
        Begin
          if Frq > FRQ4_ADC then Frq := FRQ4_ADC;
          dsp_AdjustFrq := TRUE;
          Exit;
        End;

      if DSPBASE.uDspVersion >= DSP_3XX then
        Begin
          if Stereo then
            if Frq > FRQ3_STEREO_ADC then Frq := FRQ3_STEREO_ADC
          else
            if Frq > FRQ3_HIMONO_ADC then Frq := FRQ3_HIMONO_ADC;
          dsp_AdjustFrq := TRUE;
          Exit;
        End;

      if DSPBASE.uDspVersion >= DSP_201 then
        Begin
          if Frq > FRQ2p_HIMONO_ADC then Frq := FRQ2p_HIMONO_ADC;
          if Stereo then
            Begin
              Stereo := FALSE; dsp_AdjustFrq := FALSE;
              Exit;
            End;
          dsp_ADjustFRQ := TRUE;
          Exit;
        End;

      if DSPBASE.uDspVersion >= DSP_1XX then
        Begin
          if Frq > FRQ1_MONO_ADC then Frq := FRQ1_MONO_ADC;
          if Stereo then
            Begin
              Stereo := FALSE; dsp_AdjustFrq := FALSE;
              Exit;
            End;
          dsp_AdjustFrq := TRUE;
          Exit;
        End;
    End
  else { Widergabe }
    Begin
      if DSPBASE.uDspVersion >= DSP_4XX then
        Begin
          if Frq > FRQ4_DAC then Frq := FRQ4_DAC;
          dsp_AdjustFrq := TRUE;
          Exit;
        End;

      if DSPBASE.uDspVersion >= DSP_3XX then
        Begin
          if Stereo then
            if Frq > FRQ3_STEREO_DAC then Frq := FRQ3_STEREO_DAC
          else
            if Frq > FRQ3_HIMONO_DAC then Frq := FRQ3_HIMONO_DAC;
          dsp_AdjustFrq := TRUE;
          Exit;
        End;

      if DSPBASE.uDspVersion >= DSP_201 then
        Begin
          if Frq > FRQ2p_HIMONO_DAC then Frq := FRQ2p_HIMONO_DAC;
          if Stereo then
            Begin
              Stereo := FALSE;
              dsp_AdjustFrq := FALSE;
              Exit;
            End;
          dsp_AdjustFrq :=TRUE;
          Exit;
        End;

      if DSPBASE.uDspVersion >= DSP_1XX then
        Begin
          if Frq > FRQ1_MONO_DAC then Frq := FRQ1_MONO_DAC;
          if Stereo then
            Begin
              Stereo := FALSE;
              dsp_AdjustFrq := FALSE;
              Exit;
            End;
          dsp_AdjustFrq := TRUE;
          Exit;
        End;
    End;
  dsp_AdjustFrq := FALSE;                       { DSP-Version < 1.00 }
End;

{********************************************************************}
{ dsp4_DACFrq : Ausgabefrequenz fr DSP4.00  setzen                  }
{*------------------------------------------------------------------*}
{ Eingabe : uFrq - einzustellende Ausgabefrequenz in Hertz [Hz]      }
{ Ausgabe : NO_ERROR( 0 )        - Frequenz wurde gesetzt            }
{           DSP_ERR4DACFRQ ( 4 ) - bertragungsfehler                }
{*------------------------------------------------------------------*}
{ Info : - Diese Funktion gibt es erst ab dem DSP4.00!               }
{********************************************************************}
Function dsp4_DACFrq( uFrq : Word ) : Word;

Begin
  if( ( dsp_Write( DSP4_DACFR ) <> 0 ) and
      ( dsp_Write( HI( uFrq ) ) <> 0 ) and
      ( dsp_Write( LO( uFrq ) ) <> 0 ) ) then
    dsp4_DACFrq := NO_ERROR
  else
    dsp4_DACFrq := DSP_ERR4DACFRQ;
End;

{********************************************************************}
{ dsp4_ADCFrq : Eingabefrequenz des DSP4.00 setzen                   }
{*------------------------------------------------------------------*}
{ Eingabe : uFrq - einzustellEnde Eingabefrequenz in Hertz           }
{ Ausgabe : NO_ERROR( 0 )        - Frequenz wurde gesetzt            }
{           DSP_ERR4ADCFRQ ( 5 ) - bertragungsfehler                }
{*------------------------------------------------------------------*}
{ Info : - Diese Funktion gibt es erst ab dem DSP4.00!               }
{********************************************************************}
Function dsp4_ADCFrq( uFrq : Word ) : Word;

Begin
  if( ( dsp_Write( DSP4_ADCFR ) <> 0 ) and
      ( dsp_Write( HI( uFrq ) ) <> 0 ) and
      ( dsp_Write( LO( uFrq ) ) <> 0 ) ) then
    dsp4_ADCFrq := NO_ERROR
  else
    dsp4_ADCFrq := DSP_ERR4ADCFRQ;
End;

{********************************************************************}
{ dsp_SetFrq : Ein-/Ausgabefrequenz des DSP setzen                   }
{*------------------------------------------------------------------*}
{ Eingabe : Frq - Word-Variable, die die Ein-/Ausgabefrequenz in     }
{                 Hertz [Hz] enthlt. Nach dem Aufruf dieser         }
{                 Funktion steht in diesem Word die tatschlich      }
{                 verwEndete Frequenz.                               }
{ Ausgabe : NO_ERROR( 0 )    - Frequenz wurde gesetzt                }
{           DSP_ERRFRQ ( 3 ) - bertragungsfehler                    }
{*------------------------------------------------------------------*}
{ Info : - Beim DSP3.XX kann die Requenz in 256'er Schritten einge-  }
{          stellt werden. Welche Frequenz tatschlich verwendet wird,}
{          wird von dieser Funktion zurckgegeben.                   }
{********************************************************************}
Function dsp_SetFrq( var Frq : Word ) : Word;

var BTC : Byte;

Begin
                                                     { Zeitkonstante }
  bTC  := Byte ( 256 - ( ( 1000000 + ( Frq div 2 ) ) div Frq ) );
  if ( ( dsp_Write( DSP_SETTIMECONSTANT ) <> 0 ) and
       ( dsp_Write( bTC ) <> 0 ) ) then
    Begin
      Frq := Word ( 1000000 div ( 256 - bTC ) );
      dsp_SetFrq := NO_ERROR;
    End
  else
    dsp_SetFrq := ERROR;
End;

{********************************************************************}
{ dsp_CanStereo: Untersttzt aktuelle Soundkarte Stereomodus?        }
{--------------------------------------------------------------------}
{ Ausgabe : TRUE  - Stereomodus wird untersttzt (ab DSP3.00)        }
{           FALSE - Stereomodus wird nicht untersttzt.              }
{********************************************************************}
Function dsp_CanStereo : Boolean;
Begin
  dsp_CanStereo := ( DSPBASE.uDspVersion >= DSP_3XX ) ;
End;

{********************************************************************}
{ dsp_HIMONOADCFrq : Ist angegebene Frequenz eine HISPEED-Frequenz?  }
{--------------------------------------------------------------------}
{ Ausgabe : Frequenz fr Mono-Aufnahmen                              }
{********************************************************************}
Function dsp_IsHIMONOADCFrq( uFrq : Word ) : Boolean;

Begin
  if DSPBASE.uDspVersion <= DSP_1XX then
    dsp_IsHIMONOADCFrq := FALSE
  else
    if DSPBASE.uDspVersion <= DSP_201 then
      dsp_IsHIMONOADCFrq := ( uFrq <= FRQ2p_HIMONO_ADC )
    else
      if DSPBASE.uDspVersion <= DSP_3XX then
        dsp_IsHIMONOADCFrq := ( uFrq <= FRQ3_HIMONO_ADC )
      else
        if DSPBASE.uDspVersion <= DSP_4XX then
          dsp_IsHIMONOADCFrq := ( uFrq <= FRQ4_ADC )
        else
          dsp_IsHIMONOADCFrq := TRUE;
End;

{********************************************************************}
{ dsp_HIMONODACFrq : Ist angegebene Frequenz eine HISPEED-Frequenz?  }
{--------------------------------------------------------------------}
{ Ausgabe : Frquenz fr Mono-Wiedergaben                             }
{********************************************************************}
Function dsp_IsHIMONODACFrq( uFrq : Word ) : Boolean;

Begin
  if DSPBASE.uDspVersion <= DSP_1XX then
    dsp_IsHIMONODACFrq :=FALSE
  else
    if DSPBASE.uDspVersion <= DSP_201 then
      dsp_IsHIMONODACFrq :=( uFrq <= FRQ2p_HIMONO_DAC )
    else
      if DSPBASE.uDspVersion <= DSP_3XX then
        dsp_IsHIMONODACFrq :=( uFrq <= FRQ3_HIMONO_DAC )
      else
        if DSPBASE.uDspVersion <= DSP_4XX then
          dsp_IsHIMONODACFrq := ( uFrq <= FRQ4_DAC )
        else
          dsp_IsHIMONODACFrq := TRUE;
End;

{********************************************************************}
{ dsp_MaxBits : Hchste Sampleauflsung ermitteln                    }
{--------------------------------------------------------------------}
{ Ausgabe : Maximale Anzahl der Bits eines Samples (8, 16)           }
{********************************************************************}
Function dsp_MaxBits : Integer;

Begin
  if DSPBASE.uDspVersion >= DSP_4XX then dsp_MaxBits := 16
                                    else dsp_MaxBits := 8;
End;

{********************************************************************}
{ dsp_SetSpeaker : DSP Ausgabe an-/ abschalten.                      }
{--------------------------------------------------------------------}
{ Eingabe : iState   == 0 DSP Ausgabe ausschalten                    }
{                    <> 0 DSP Ausgabe anschalten                     }
{ Ausgabe : NO_ERROR( 0 )        - Lautsprecher wurde geschaltet     }
{           DSP_ERRSPEAKER ( 6 ) - bertragungsfehler                }
{********************************************************************}
Function dsp_SetSpeaker( iState : Boolean ) : Word;

var return : Word;

Begin
   if iState then return := dsp_Write( DSP_SPEAKERON )
             else return := dsp_Write( DSP_SPEAKEROFF );
   if return <> 0 then dsp_SetSpeaker := NO_ERROR
                  else dsp_SetSpeaker := DSP_ERRSPEAKER;
End;

{********************************************************************}
{ dsp_SetTransferSize : Anzahl zu bertragender Bytes an DSP ber-   }
{                       mitteln.                                     }
{--------------------------------------------------------------------}
{ Eingabe : uSize - Anzahl der Bytes in bertragung                  }
{ Ausgabe : NO_ERROR( 0 )         - Anzahl gesetzt                   }
{           DSP_ERRTRANSSIZE( 7 ) - bertragungsfehler               }
{********************************************************************}
Function dsp_SetTransferSize( uSize : Word ) : Word;

Begin
  if uSize <> 0 then
    Begin         { Nur Wenn Anzahl != 0 bergeben wurd  weitermachen }
      Dec( uSize );                       { bergeben wird Anzahl - 1}
                                               { DSP-Kommando senden }
      if ( ( dsp_Write( DSP_SETTRANSSIZE ) <> 0 ) and
           ( dsp_Write( LO( uSize ) ) <> 0 ) and    { LO-Byte senden }
           ( dsp_Write( HI( uSize ) ) <> 0 ) )      { HI-Byte senden }
      then dsp_SetTransferSize := NO_ERROR
      else dsp_SetTransferSize := DSP_ERRTRANSSIZE;
    End
  else
    dsp_SetTransferSize := DSP_ERRILLSIZE;
End;

{********************************************************************}
{ dsp_IrqHandler : Demo DSP Interrupt-Handler                        }
{*------------------------------------------------------------------*}
{ Info : - Dies ist ein gewhnlicher Interrupt-Handler.              }
{          Um die Programmierung eines eigenen Handlers zu           }
{          vereinfachen, wird in diesem Handler alles ntige unter-  }
{          nommen, um die Interrupts zu besttigen. Im Anschlu      }
{          daran kann eine Benutzer-Funktion aufgerufen werden.      }
{********************************************************************}
Procedure dsp_IrqHandler;

var who, i : Byte;

Begin
  Inc(lIRQServed);           { Anzahl der bisherigen Aufrufe erhhen }
  if DSPBASE.uDspVersion >= $0400 then         { DSP-Version >= 4.00 }
    Begin
                                  { Welcher Chip hat IRQ ausgelst ? }
      who := Byte ( mix_Read( MIX4_IRQSOURCE ) );
                           
      if( @lpDSP4UserFunc <> @DummyDSP4UserFunc ) then
        lpDSP4UserFunc( who ); { Benutzerfunktion aufrufen }

      {-- 8-Bit DMA oder Midi-bertragung ---------------------------}
      if ( who and MIX4_IRQ8DMA ) <> 0 then
        i := port[DSPBASE.iDspPort + $0e];

      {-- 16-Bit DMA ------------------------------------------------}
      if ( who and MIX4_IRQ16DMA ) <> 0 then
        i := port[DSPBASE.iDspPort + $0f];
                                                      
      {-- MPU-401 UART ----------------------------------------------}
      if ( who and MIX4_IRQMPU ) <> 0 then
        i := port[DSPBASE.iMpuPort];
    End
  else
                           { IRQ-Acknowledge aller anderen DSP's.    }
                           { Einzige Verursacher: Midi und 8-Bit DMA }
    i := port[DSPBASE.iDspPort + $0e];

  if @lpUserFunc <> @DummyUserFunc then
    lpUserfunc( lIRQServed );          { Benutzerfunktion aufrufen }

              { Interruptkontroller das Ende des IRQ's signalisieren }
  irq_SendEOI( DSPBASE.iDspIrq );
End;


{********************************************************************}
{ dsp_RestoreIrqHandler : Original IRQ-Handler wieder einrichten.    }
{********************************************************************}
Procedure dsp_RestoreIrqHandler;

Begin
  irq_SetHandler( DSPBASE.iDspIrq, @lpOldInt );
End;

{********************************************************************}
{ dsp_InitIrqHandler : eigenen DSP-IRQ-Handler einrichten.           }
{********************************************************************}
Procedure dsp_InitIrqHandler;

Begin
 @lpOldInt := irq_SetHandler( DSPBASE.iDspIrq, @dsp_irqHandler );
End;

{********************************************************************}
{ dsp_SetUserIRQ : Benutzerdefinierte IRQ-Funktion angeben.          }
{*------------------------------------------------------------------*}
{ Eigabe : lpFunc - Adresse der Benutzerfunktion.                    }
{ Info : - Die hier angegebene Funktion wird vom eigenen DSP-Handler }
{          aufgerufen.                                               }
{          Der Benutzerfunktion wird die Anzahl der bisherigen IRQ-  }
{          Aufrufe bergeben.                                        }
{          Durch Aufruf von dsp_SetUserIRQ(DummyUserFunc) werden     }
{          weitere Aufruf der Benutzerfunktion unterdrckt.          }
{********************************************************************}
Procedure dsp_SetUserIRQ( lpFunc : UserFunc );

Begin
  lpUserFunc := lpFunc;
End;

{********************************************************************}
{ dsp4_SetUserIRQ : Benutzerdefinierte IRQ-Funktion angeben.         }
{*------------------------------------------------------------------*}
{ Eigabe : lpFunc - Adresse der Benutzerfunktion.                    }
{ Info : - Die hier angegebene Funktion wird vom eigenen DSP-Handler }
{          aufgerufen.                                               }
{          Der Benutzerfunktion wird die Anzahl der bisherigen IRQ-  }
{          Aufrufe bergeben.                                        }
{          Durch Aufruf von dsp4_SetUserIRQ(DummyDSP4UserFunc)       }
{          werden weitere Aufruf der Benutzerfunktion unterdrckt.   }
{********************************************************************}
Procedure dsp4_SetUserIRQ( lpFunc : DSP4UserFunc );

Begin
  lpDSP4UserFunc := lpFunc;
End;

{********************************************************************}
{ dsp_WaitForNextIRQ : Warte auf nchsten vom DSP ausgelsten        }
{                      Interrupt                                     }
{*------------------------------------------------------------------*}
{ Eingabe : lpDoSomething - Adresse einer whrend des Wartens auszu- }
{                           fhrenden Funktion.                      }
{           lPar          - Parameter, der der Funktion bergeben    }
{                           werden soll.                             }
{ Ausgabe : TRUE  : Seit dem letzen Aufruf von WaitForNextIRQ ist    }
{                   mehr als 1 IRQ-Aufruf vergangen.                 }
{           FALSE : Seit dem letzen Aufruf von WaitForNextIRQ ist    }
{                   nur 1 IRQ-Aufruf vergangen.                      }
{********************************************************************}
Function dsp_WaitForNextIRQ( lpDoSomething : UserFunc;
                             lpar          : Longint ) : Boolean;

var iret : Boolean;

Begin

  iRet  := ( lIRQServed - lOldIRQServed > 1 );
  lOldIRQServed := lIRQServed;        { aktuellen Zhlerstand retten }
                    { Schleife ausfhren, bis sich IRQ-Zhler ndert }
  while( lOldIRQServed = lIRQServed ) do         { Funktion aufrufen }
    if @lpDoSomething <> @DummyUserFunc then
      lpDoSomething( lPar );

  dsp_WaitForNextIRQ := iRet;
End;

{********************************************************************}
{ dsp_InitWaitForIRQ : Interruptzhler angleichen                    }
{********************************************************************}
Procedure dsp_InitWaitForIRQ;

Begin                    { alter Zhlerstand = aktuellen Zhlerstand }
  lOldIRQServed := lIRQServed;
End;

{********************************************************************}
{ dsp_8DAC : Direktes 8 Bit 'Digital->Analog Converten' (Ausgabe)    }
{--------------------------------------------------------------------}
{ Eingabe : bVal - auszugebender 8-Bit Wert                          }
{ Ausgabe : NO_ERROR ( 0 )   - Byte konnte ausgegeben werden         }
{           DSP_ERR8DAC( 9 ) - bertragungsfehler                    }
{********************************************************************}
Function dsp_8DAC( bVal : Byte ) : Word;

Begin
  if ( ( dsp_Write( DSP_8DA ) <> 0 ) and ( dsp_Write( bVal ) <> 0 ) ) then
    dsp_8DAC := NO_ERROR
  else
    dsp_8DAC := DSP_ERR8DAC;
End;

{********************************************************************}
{ dsp_8ADC : Direktes 8 Bit 'Analog->Digital Converten' (Samplen)    }
{--------------------------------------------------------------------}
{ Eingabe : Val - Byte-Variable, die den gewandelten Wert aufnehmen  }
{                 soll.                                              }
{ Ausgabe : NO_ERROR ( 0 )    - Byte konnte eingelesen werden        }
{           DSP_ERR8ADC( 10 ) - bertragungsfehler                   }
{********************************************************************}
Function dsp_8ADC( var Val : Byte ) : Word;

Begin
  if ( ( dsp_Write( DSP_8AD ) <> 0 ) and ( dsp_Read( Val ) <> 0 ) ) then
    dsp_8ADC := NO_ERROR
  else
    dsp_8ADC := DSP_ERR8DAC;
End;

{********************************************************************}
{ dsp_PLAY : Abspielen von Musikdaten eines Arrays                   }
{--------------------------------------------------------------------}
{ Eingabe : pBuffer  - Adresse des Arrays mit den Sample-Daten       }
{           uSize    - Gre des Arrays                              }
{           uDelay   - Wert fr Verzgerungsschleife (fr Frequenz)  }
{ Ausgabe : NO_ERROR ( 0 )     - Byte konnte ausgegeben werden       }
{           DSP_ERRPLAY ( 11 ) - bertragungsfehler                  }
{********************************************************************}
Function dsp_PLAY( pBuffer       : pointer;
                   uSize, uDelay : Word ) : Word;
var i, j   : Word;
    BufPtr : SampleArrayBPtr;

Begin
  BufPtr := pBuffer;
  if dsp_SetSpeaker( ON ) <> 0 then dsp_PLAY := DSP_ERRPLAY
  else Begin                                        { Tonausgabe an }
    for i := 0 to uSize - 1 do    { Jedes einzelne Byte ausgeben... }
    Begin
      if dsp_8DAC( BufPtr^[i] ) <> 0 then
      Begin
        dsp_PLAY := DSP_ERRPLAY;
        Exit;
      End;
      for j := 1 to uDelay do Begin End;           { ... und warten }
    End;
    if dsp_SetSpeaker( OFF ) <> 0 then
    Begin
      dsp_PLAY := DSP_ERRPLAY; Exit;
    End;
    dsp_PLAY := NO_ERROR;
  End;
End; 

{********************************************************************}
{ dsp_RECORD : Aufnehmen von Musikdaten in ein Array                 }
{--------------------------------------------------------------------}
{ Eingabe : pBuffer - Adresse des Musikdaten aufnehmEnden Arrays     }
{           uSize    - Gre des Arrays                              }
{           uDelay   - Wert fr Verzgerungsschleife (fr Frequenz)  }
{ Ausgabe : NO_ERROR ( 0 )       - Byte konnte ausgegeben werden     }
{           DSP_ERRRECORD ( 12 ) - bertragungsfehler                }
{********************************************************************}
Function dsp_RECORD( pBuffer       : pointer;
                     uSize, uDelay : Word ) : Word;

var i, j   : Word;
    BufPtr : SampleArrayBPtr;

Begin
  BufPtr := pBuffer;
            { Bei Aufnahmen mu DSP-Lautsprecher abgeschaltet sein! }
  if dsp_SetSpeaker( OFF ) <> 0 then
    dsp_RECORD := DSP_ERRRECORD
  else Begin
    for i := 0 to uSize - 1 do
    Begin                                               { Einlesen... }
       if dsp_8ADC( BufPtr^[i] ) <> 0 then
       Begin
         dsp_RECORD := DSP_ERRRECORD; Exit;
       End;
       for j := 1 to uDelay do Begin End;            { ... und warten }
    End;
    dsp_RECORD := NO_ERROR;
  End;
End;


{********************************************************************}
{ dsp_InitBuffers: Buffer fr DSP-Aufnahme/Wiedergabe vorbereiten    }
{********************************************************************}
Procedure dsp_InitBuffers;

Begin
  iBufCnt := 0;
End;

{********************************************************************}
{ dsp_FileOpen: Datei fr DSP-Aufnahme/Wiedergabe ffnen             }
{*------------------------------------------------------------------*}
{ Eingabe : Filename- Dateiname                                      }
{           Handle  - Dateihandle                                    }
{ Ausgabe : 0    - kein Fehler                                       }
{           <> 0 - aufgetretener DOSERROR                            }
{********************************************************************}
Function dsp_FileOpen(     Filename : string;
                       var Handle   : Integer ) : Integer;
Begin
  Handle := _dos_open( FileName );
  dsp_FileOpen := integer( Handle = -1 );
End;

{********************************************************************}
{ dsp_FileClose: DSP-Aufnahme/Wiedergabe Datei schlieen             }
{*------------------------------------------------------------------*}
{ Eingabe : Handle - Datei-Handle der Datei                          }
{ Ausgabe : 0      - kein Fehler                                     }
{           <> 0 - aufgetretener DOSERROR                            }
{********************************************************************}
Function dsp_FileClose( var Handle : Integer ) : Integer;

Begin
  _dos_close( Handle );
End;

{********************************************************************}
{ dsp_ReadHeader : Dateikopf lesen                                   }
{*------------------------------------------------------------------*}
{ Eingabe : Handle - Datei-Handle                                    }
{           pDRP   - RECPLAY-Struktur, die Dateikopf aufnehmen soll  }
{ Ausgabe : Anzahl der gelesenen Zeichen                             }
{********************************************************************}
Function dsp_ReadHeader( var Handle : Integer;
                         var DRP    : DSPRECPLAY ) : Word;
Begin
  dsp_ReadHeader := _dos_read( Handle, @DRP, sizeof( DSPRECPLAY ) );
End;

{********************************************************************}
{ dsp_WriteHeader : Dateikopf schreiben                              }
{*------------------------------------------------------------------*}
{ Eingabe : Handle - Datei-Handle                                    }
{           DRP    - RECPLAY-Struktur die Dateikopf enthlt          }
{ Ausgabe : Anzahl der geschriebenen Zeichen                         }
{********************************************************************}
Function dsp_WriteHeader( var Handle : Integer;
                          var DRP    : DSPRECPLAY  ) : Word;
Begin
  dsp_WriteHeader := _dos_write( Handle, @DRP, sizeof( DSPRECPLAY ) );
End;

{********************************************************************}
{ dsp_ReadBuffer : Nchsten Block fr Wiedergabe lesen               }
{*------------------------------------------------------------------*}
{ Eingabe : Handle    - Datei-Handle                                 }
{           lpBuffer  - Adresse des Speichers                        }
{           iHalfSize - (Gre des Speichers) / 2                    }
{ Ausgabe : Anzahl der gelesenen Zeichen                             }
{*------------------------------------------------------------------*}
{ Hinweis : Diese Funktion wacht automatisch darber, ob die erste   }
{           oder zweite Hlfte des Buffers gelesen werden mu. Dazu  }
{           werden alle geladenen Buffer gezhlt. Welcher Teil des   }
{           Buffers mit neuen Daten geladen wird, wird aufgrund der  }
{           'Geradheit' dieses Zhlers entschieden.                  }
{********************************************************************}
Function dsp_ReadBuffer( var Handle    : Integer;
                             lpBuffer  : pointer;
                             iHalfSize : Integer ) : Word;
var BufPtr : BPTR;
Begin
  BufPtr := lpBuffer;
  if ( iBufCnt and 1 ) <> 0 then
    Inc( BufPtr, iHalfSize );
  BufPtr^:= 255;
  dsp_ReadBuffer := _dos_read( Handle, BufPtr, iHalfSize );
  Inc( iBufCnt );
End;

{********************************************************************}
{ dsp_WriteBuffer : Nchsten Block der Aufnahme sichern              }
{*------------------------------------------------------------------*}
{ Eingabe : Handle    - Datei-Handle                                 }
{           lpBuffer  - Adresse des Speichers                        }
{           iHalfSize - (Gre des Speichers) / 2                    }
{ Ausgabe : Anzahl der geschriebenen Zeichen                         }
{*------------------------------------------------------------------*}
{ Hinweis : Diese Funktion wacht automatisch darber, ob die erste   }
{           oder zweite Hlfte des Buffers gesichert werden mu.     }
{********************************************************************}
Function dsp_WriteBuffer( var Handle    : Integer;
                              lpBuffer  : pointer;
                              iHalfSize : Integer ) : Word;
var BufPtr : BPTR;
Begin
  BufPtr := lpBuffer;
  if ( iBufCnt and 1 ) <> 0 then Inc( BufPtr, iHalfSize );
  dsp_WriteBuffer := _dos_write( Handle, BufPtr, iHalfSize );
  Inc( iBufCnt );
End;

{********************************************************************}
{ dsp_ClearBuffer : Buffer lschen                                   }
{*------------------------------------------------------------------*}
{ Eingabe : lpBuffer - Adresse des Buffers                           }
{           uMemSize - Gre des Buffers                             }
{*------------------------------------------------------------------*}
{ Hinweis : Nachdem der letzte wiederzugebende Block geladen wurde,  }
{           wird diese Funktion aufgerufen, um ein 'Echo' zu ver-    }
{           meiden.                                                  }
{********************************************************************}
Procedure dsp_ClearBuffer( lpBuffer : pointer; uMemSize : Word );
var i      : Word;
    l      : word;
    BufPtr : SampleArrayBPtr;

Begin
  BufPtr := lpBuffer;
  for l := 0 to uMemSize - 1 do
    BufPtr^[l] := 0;
End;

{********************************************************************}
{ dsp_DORecPlay : Harddisk-Recorder / Player                         }
{*------------------------------------------------------------------*}
{ Eingabe : Handle  -  Datei-Handle der Datei, die wiederzugebende   }
{                      Daten enthlt, bzw. Daten aufnehmen soll.     }
{           iADC     - Aufnahme (TRUE) oder Wiedergabe (FALSE)       }
{           iASource - Aufnahmequelle (CD, LINE, MIC)                }
{                      -1: aktuelle Mixereinstellung verwEnden       }
{           DRP      - Dateikopf                                     }
{           iSecs    - Anzahl der aufzunehmEnden Sekunden            }
{                      (wird bei Wiedergabe ignoriert)               }
{           lpBuffer - DMA-fhiger Speicher                          }
{           uMemSize - gre des DMA-Speichers                       }
{********************************************************************}
Procedure dsp_DoRECPLAY( var Handle   : Integer;
                             iADC     : Boolean;
                             iSource  : Integer;
                         var DRP      : DSPRECPLAY;
                             iSecs    : Integer;
                             lpBuffer : pointer;
                             uMemSize : Word );

var iEncore   : Boolean;
    DSP_cmd,
    m,
    cBackup   : Byte;
    lSamples  : Longint;
    BufPtr    : SampleArrayBPtr;

Const
    dsp4_Mode : Byte    = 0;
    iDivisor  : Integer = 2;

Begin
  BufPtr := lpBuffer;
  if DRP.iStereo then
    DRP.iStereo := dsp_CanStereo;

  if DRP.iBits = 16 then
    DRP.iBits := dsp_MaxBits else DRP.iBits := 8;

  dsp_AdjustFrq( DRP.uFrequency, iADC, DRP.iStereo );
  dsp_InitIrqHandler;          { DSP-Interrupt Funktion installieren }
  dsp_InitBuffers;        { DMA-Double/SplitBuffering initialisieren }
  dsp_ClearBuffer( lpBuffer, uMemSize );

  if not iADC then                       { Wiedergabe: Buffer fllen }
    Begin
      lSamples := _dos_filesize( Handle );
      dsp_ReadBuffer( Handle, lpBuffer, uMemSize div 2 );
      dsp_ReadBuffer( Handle, lpBuffer, uMemSize div 2 );
    End
  else
    lSamples := DRP.uFrequency * iSecs;

  {- DSP3xx ---------------------------------------------------------}
  if DSPBASE.uDspVersion < DSP_4XX then
    Begin
      if DRP.iStereo then
        if iADC then mix3_PrepareForStereo( ADC)
                else mix3_PrepareForStereo( DAC);

      if iADC then                                          { Aufnahme }
        Begin
          if iSource >= 0 then mix3_SetADCSource( iSource );
          dsp_SetSpeaker( OFF );   { Bei Aufnahmen immer DSP-Speaker aus }
                                        { Mono oder Stereo Aufnahme? }
          if DSPBASE.uDspVersion >= DSP_3XX then
             if DRP.iStereo then dsp_Write( DSP3_STEREOADC )
                             else dsp_Write( DSP3_MONOADC );
               { Im Stereomodus oder bei Hochgeschwindigkeitstransfers }
               { HI_SPEED DSP-Befehle verwEnden.                       }

          DSP_cmd := Byte ( DSP_8DMAAUTOADC );
          if DSPBASE.uDspVersion >= DSP_201 then
            if ( DRP.iStereo or
               dsp_IsHIMONOADCFrq( DRP.uFrequency ) ) then
              DSP_cmd := Byte ( DSP2p_8DMAHIAUTOADC );
        End
      else                                                { Wiedergabe }
        Begin { DSP3XX }
          dsp_Write( DSP3_MONOADC );          { Aufnahmemodus immer MONO }

          dsp_SetSpeaker( ON );                    { DSP-Lautsprecher an }
          if DRP.iStereo then                       { Stereo-Wiedergabe }
            Begin                { Bei Stereo-Wiedergabe erst mal $80 ber }
                          { DSP unter DMA-Verwendung ausgeben.       }
           { erstes Byte des DMA fhigen Speicher durch $80 ersetzen }
              cBackUp := BufPtr^[0];
              BufPtr^[0] := $80;
                { DMA-Kanal zur Ausgabe eines Bytes programmieren... }
              dma_SetChannel( DSPBASE.iDspDmaB, @lpBuffer, 1,
                              MODE_SINGLE or MODE_READ );

              dsp_Write( DSP_8DMADAC );{ ...und ein Byte ber DSP ausgeben }
              dsp_Write( 0 );
              dsp_Write( 0 );
                                   { Ende des DMA-Transfers abwarten }
              dsp_WaitForNextIRQ( DummyUserFunc, 0 );
              BufPtr^[0] := cBackUp;         { DMA-Buffer wieder herstellen }
              DSP_cmd := DSP2p_8DMAHIAUTODAC;{ Stereowiederg.immer HiSpeed }
            End
          else                                         { Mono-Wiedergabe }
            Begin
              DSP_cmd := Byte ( DSP_8DMAAUTODAC );               { DSP_201 }
              if dsp_IsHIMONODACFrq( DRP.uFrequency ) then
                DSP_cmd := Byte ( DSP2p_8DMAHIAUTODAC );
            End;
        End;

        {-- DMA fr Aufnahme/Wiedergabe programmieren -------------------}
      if iADC then
        dma_SetChannel( DSPBASE.iDspDmaB, lpBuffer, uMemSize,
                        Byte ( MODE_SINGLE or MODE_AUTOINIT or MODE_WRITE ) )
      else
        dma_SetChannel( DSPBASE.iDspDmaB, lpBuffer, uMemSize,
                        Byte ( MODE_SINGLE or MODE_AUTOINIT or MODE_READ ) );

      dsp_SetFrq( DRP.uFrequency );
            { Nach jeder Bufferhlfte soll ein IRQ ausgelst werden: }
      dsp_SetTransferSize( uMemSize div 2 );
      dsp_Write( DSP_cmd );                    { DSP-Kommando absetzen }
    End
  else

  {- DSP 4.XX -------------------------------------------------------}
    Begin
      if iADC then
        case iSource of
          CD:
            Begin
              mix4_SetADCSourceL( CD_L, TRUE );{ Bei Mono-Aufnahmen wird }
              mix4_SetADCSourceR( CD_R, TRUE );{ nur linker ADC verwendet}
            End;

          LINE:
            Begin
              mix4_SetADCSourceL( LINE_L, TRUE );
              mix4_SetADCSourceR( LINE_R, TRUE );
            End;

          MIC:
            Begin
              mix4_SetADCSourceL( MIC, TRUE );
              mix4_SetADCSourceR( MIC, TRUE );
            End;
        End;

      if not iADC then
        mix4_SetVolume( VOICE, 255, 255 )
      else
        if not DRP.iStereo then mix4_PrepareForMonoADC;
                             { DMA-Kanal fr 8 oder 16 Bit Programmieren }
      if iADC then m := Byte ( MODE_SINGLE or MODE_AUTOINIT or MODE_WRITE )
              else m := Byte ( MODE_SINGLE or MODE_AUTOINIT or MODE_READ );

      if DRP.iBits = 16 then
        dma_SetChannel( DSPBASE.iDspDmaW, lpBuffer, uMemSize, m)
      else
        dma_SetChannel( DSPBASE.iDspDmaB, lpBuffer, uMemSize, m);
                                       { Ein-/Ausgabefrequenz setzen }

      if iADC then dsp4_ADCFrq( DRP.uFrequency )
              else dsp4_DACFrq( DRP.uFrequency );

            { Aufnahme/Wiedergabe wird durch Kommando-Byte angegeben }
      if iADC then
        DSP_cmd := DSP4_CMDADC
      else
        DSP_cmd := DSP4_CMDDAC;

      DSP_cmd := DSP_cmd or DSP4_CMDAUTOINIT or DSP4_CMDFIFO;
      if DRP.iBits = 16 then DSP_cmd := DSP_cmd or DSP4_CMD16DMA
                        else DSP_cmd := DSP_cmd or DSP4_CMD8DMA;
      if DRP.iStereo then dsp4_Mode := DSP4_MODESTEREO
                     else dsp4_Mode := DSP4_MODEMONO;
      dsp4_Mode := dsp4_Mode or DSP4_MODESIGNED;

          { Kommando, Modus und bertragungslnge an DSP bermitteln }
      dsp_Write( DSP_cmd );
      dsp_Write( dsp4_Mode );
                                 { Halber Buffer in Words bzw. Bytes }
      if( DRP.iBits = 16 ) then iDivisor := iDivisor * 2;
      dsp_Write( LO( ( uMemSize div iDivisor ) - 1 ) );
      dsp_Write( HI( ( uMemSize div iDivisor ) - 1 ) );
    End;

  dsp_InitWaitForIRQ;                { Auf IRQ-Warten initialisieren }

  Repeat
    if dsp_WaitForNextIRQ( DummyUserFunc, 0 ) then
      Writeln( 'Daten verloren!' );
    write( '.' );
    if iADC then
      Begin
        iEncore := FALSE;          { erst mal nicht weitermachen, oder? }
        if dsp_WriteBuffer( Handle, lpBuffer, uMemSize div 2 )<>0 then
          if lSamples > 0 then
            Begin
              iEncore := TRUE;
              lSamples := lSamples - uMemSize div 2;
            End;
      End
    else
      iEncore := dsp_ReadBuffer( Handle, lpBuffer, uMemSize div 2 )<>0;
  Until not iEncore;

  dsp_ClearBuffer( lpBuffer, uMemSize );

      { HISpeed-Kommandos knnen nur durch Reset unterbrochen werden }
  if ( ( DSP_cmd = DSP2p_8DMAHIAUTODAC ) or
       ( DSP_cmd = DSP2p_8DMAHIAUTOADC ) ) then
    Begin
      dsp_RestoreIrqHandler;           { alten IRQ-Handler instalieren }
      dsp_Reset;
    End
  else
    Begin
      if ( not DRP.iStereo and ( DSPBASE.uDspVersion>= DSP_4XX ) ) then
         mix4_PrepareForMonoADC;

      if ( ( DSP_cmd and $F0 ) = DSP4_CMD8DMA ) then
        dsp_Write( DSP4_EXIT8DMA )      { Nach aktuellem Block beenden }
      else
        if ( ( DSP_cmd and $F0 ) = DSP4_CMD16DMA ) then
          dsp_Write( DSP4_EXIT16DMA )     { Nach aktuellem Block beenden }
        else
          Begin
            if ( DRP.iStereo and ( DSPBASE.uDspVersion <= DSP_3XX ) ) then
              mix3_RestoreFromStereo;
            dsp_SetSpeaker( OFF );                 { DSP-Laustsprecher aus }
            dsp_Write( DSP2p_EXITAUTOINIT );{ Nach aktuellem Block beenden }
          End;
      dsp_WaitForNextIRQ( DummyUserFunc, 0 );                  { auf Ende warten }
      dsp_RestoreIrqHandler;          { alten IRQ-Handler installieren }
    End;
End;
{$Endif}                                     { #ifndef DSP_VERSIONONLY }

{-- UNIT-Initialisierung ---------------------------------------------}

begin
  lpUserFunc     := DummyUserFunc;
  lpDSP4UserFunc := DummyDSP4UserFunc;
End.

