{**********************************************************************}
{*                              D V I P                               *}
{*--------------------------------------------------------------------*}
{*    Aufgabe        : zeigt, wie man aus TURBO-PASCAL heraus direkt  *}
{*                     auf den Video-RAM zugreifen kann.              *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  :  2.10.1987                                     *}
{*    letztes Update : 14.01.1991                                     *}
{**********************************************************************}
                                                                        
program DVIP;
                                                                        
Uses Crt, Dos;                   { die CRT- und die DOS-Unit einbinden }
                                                                        
const NORMAL        = $07;           { Defintion der Zeichen-Attribute }
      HELL          = $0f;           { in bezug auf eine monochrome    }
      INVERS        = $70;           { Bildschirmkarte                 }
      UNTERSTRICHEN = $01;
      BLINKEN       = $80;
                                                                        
      SCHWARZ       = $00;           { Farbattribute der Color-Karte   }
      BLAU          = $01;
      GRUEN         = $02;
      KOBALTBLAU    = $03;
      ROT           = $04;
      VIOLETT       = $05;
      BRAUN         = $06;
      HELLGRAU      = $07;
      DUNKELGRAU    = $01;
      HELLBLAU      = $09;
      HELLGRUEN     = $0A;
      HELLKOBALT    = $0B;
      HELLROT       = $0C;
      HELLVIOLETT   = $0D;
      GELB          = $0E;
      WEISS         = $0F;
                                                                        
type TextTyp = string[80];
                                                                        
var VSeg : word;                        { Segmentadresse des Video-RAM }
                                                                        
{**********************************************************************}
{* InitDPrint: Ermittelt die Segmentadresse des Video-RAM fr DPrint. *}
{* Eingabe : keine                                                    *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure InitDPrint;
                                                                        
var CRTC_PORT : word absolute $0040:0063;  { Variable im BIOS-Var.Seg. }
                                                                        
begin
  if CRTC_PORT = $3B4 then       { monochromer Adapater angeschlossen? }
    VSeg := $B000                        { JA, Video-RAM bei B000:0000 }
  else                                  { Nein, mu Color-Adapter sein }
    VSeg := $B800;                           { Video-RAM bei B800:0000 }
end;
                                                                        
{**********************************************************************}
{* DPrint: Schreibt einen String dirket in den Video-RAM.             *}
{* Eingabe : - SPALTE: die Ausgabespalte                              *}
{*           - ZEILE : die Ausgabezeile                               *}
{*           - FARBE : Farbe (Attribut) fr die einzelnen Zeichen.    *}
{*           - STROUT: der auszugebende String                        *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure DPrint( Spalte, Zeile, Farbe : byte; StrOut : TextTyp);
                                                                        
var PAGE_OFS : word absolute $0040:$004E;  { Variable im BIOS-Var.Seg. }
    Offset   : word;         { Zeiger auf die aktuelle Ausgabeposition }
    i, j     : byte;                                 { Schleifenzhler }
    Attribut : word;                            { Attribut fr Ausgabe }
                                                                        
begin
  Offset := Zeile * 160 + Spalte * 2 + PAGE_OFS;
  Attribut := Farbe shl 8;    { Hi-Byte fr Word-Zugriff auf Video-RAM }
  i := length( StrOut );              { die Lnge des String ermitteln }
  for j:=1 to i do                            { den String durchlaufen }
    begin         { das Zeichen nebst Attribut direkt in den Video-RAM }
      memw[VSeg:Offset] := Attribut or ord( StrOut[j] );
      Offset := Offset + 2;  { auf nchstes ASCII-Attribut-Paar setzen }
    end;
end;
                                                                        
{**********************************************************************}
{* Demo: Demonstriert die Arbeit mit DPrint.                          *}
{* Eingabe : keine                                                    *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure demo;
                                                                        
var Spalte,                                 { aktuelle Ausgabeposition }
    Zeile,
    Farbe   : integer;
                                                                        
begin
  TextBackGround( SCHWARZ );              { Hintergrund schwarz frben }
  ClrScr;                                     { den Bildschirm lschen }
  DPrint( 22, 0, WEISS, 'DVIP  - (c) 1988 by Michael Tischer');
  Randomize;                             { Zufallsgenerator anschalten }
  while not KeyPressed do            { wiederholen, bis Taste bettigt }
    begin
      Spalte := Random( 76 );                { Spalte, Zeile und Farbe }
      Zeile := Random( 22 ) + 1;             { zufllig auswhlen      }
      Farbe := Random( 14 ) + 1;
      DPrint( Spalte, Zeile,   Farbe, '');        { Block ausgeben }
      DPrint( Spalte, Zeile+1, Farbe, '');
    end;
  ClrScr;                                  { Bildschirm wieder lschen }
end;
                                                                        
{**********************************************************************}
{**                           HAUPTPROGRAMM                          **}
{**********************************************************************}
                                                                        
begin
  InitDPrint;                     { Ausgabe ber DPrint initialisieren }
  Demo;                          { die Arbeit mit DPrint demonstrieren }
end.
                                                                        
