{**********************************************************************}
{*                              E M M P                               *}
{*--------------------------------------------------------------------*}
{*    Aufgabe        : Stellt verschiedene Funktionen zur Verfgung,  *}
{*                     mit denen auf den EMS-Speicher bzw. auf den    *}
{*                     EMM zugegriffen werden kann.                   *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  : 30.08.1988                                     *}
{*    letztes Update : 19.02.1992                                     *}
{*--------------------------------------------------------------------*}
{*    Vernderungen  : 19.02.92 : MK_FP durch PTR ersetzt             *}
{**********************************************************************}
                                                                        
program EMMP;
                                                                        
Uses Dos, CRT;                   { die DOS- und die CRT-Unit einbinden }
                                                                        
type  ByteBuf = array[0..1000] of byte; { ein Speicherbereich als Byte }
      CharBuf = array[0..1000] of char; { ein Speicherbereich als Char }
      BytePtr = ^ByteBuf;             { Pointer auf einen Byte-Bereich }
      CharPtr = ^CharBuf;             { Pointer auf einen Char-Bereich }
                                                                        
const EMS_INT   = $67;          { Interruptnr. zum Zugriff auf den EMM }
      EMS_ERR   = -1;          { wird bei einem Fehler zurckgeliefert }
      W_EMS_ERR = $FFFF;                     { Fehlercode in WORD-Form }
      EmmName   : array[0..7] of char = 'EMMXXXX0';     { Name des EMM }
                                                                        
var   EmmEC,               { hier werden die EMM-Fehler-Codes abgelegt }
      i        : byte;                               { Schleifenzhler }
      Handle,                { Handle zum Zugriff auf den EMS-Speicher }
      EmmVer   : integer;                     { Versionsnummer des EMM }
      AnzPage,                                  { Anzahl der EMS-Pages }
      PageSeg  : word;                 { Segmentadresse des Page-Frame }
      Taste    : char;
                                                                        
{**********************************************************************}
{* EmsInst: Stellt fest, ob EMS-Speicher und ein zugehriger EMM      *}
{*          instlliert sind.                                          *}
{* Eingabe : keine                                                    *}
{* Ausgabe : TRUE, wenn EMS-Speicher vorhanden ist, sonst FALSE.      *}
{**********************************************************************}
                                                                        
function EmsInst : boolean;
                                                                        
type EmmName  = array [1..8] of char;   { Name des EMM im Treiber-Kopf }
     EmmNaPtr = ^EmmName;           { Zeiger auf Namen im Treiber-Kopf }

const Name : EmmName = 'EMMXXXX0';             { Name des EMS-Treibers }

var Regs  : Registers;            { Prozessorregs. fr Interruptaufruf }

begin
  Regs.ax := $35 shl 8 + EMS_INT;           { Interruptvektor $67 ber }
  msdos( Regs );                            { DOS-Funktion 35h holen   }

  EmsInst := (EmmNaPtr(Ptr(Regs.ES,10))^ = Name);  {Treiber-Name vergl.}
end;
                                                                        
{**********************************************************************}
{* EmsNumPage: Ermittelt die Gesamtanzahl der EMS-Pages.              *}
{* Eingabe : keine                                                    *}
{* Ausgabe : EMS_ERR bei einem Fehler, sonst die Anzahl der EMS-Pages.*}
{**********************************************************************}
                                                                        
function EmsNumPage : integer;
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $42;                { Fkt.nr.: Anzahl der Pages ermitteln }
  Intr(EMS_INT, Regs);                                  { EMM aufrufen }
  if (Regs.ah <>0 ) then                 { ist ein Fehler aufgetreten? }
    begin                                                         { Ja }
      EmmEC := Regs.ah;                        { den Fehlercode merken }
      EmsNumPage := EMS_ERR;                         { Fehler anzeigen }
    end
  else                                                   { kein Fehler }
    EmsNumPage := Regs.dx;      { Gesamtanzahl der Pages zurckliefern }
end;
                                                                        
{**********************************************************************}
{* EmsFreePage: Ermittelt die Anzahl der noch freien EMS-Pages.       *}
{* Eingabe : keine                                                    *}
{* Ausgabe : EMS_ERR bei einem Fehler, sonst die Anzahl der noch      *}
{*           noch nicht reservierten EMS-Pages.                       *}
{**********************************************************************}
                                                                        
function EmsFreePage : integer;
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $42;                { Fkt.nr.: Anzahl der Pages ermitteln }
  Intr(EMS_INT, Regs);                                  { EMM aufrufen }
  if (Regs.ah <>0 ) then                 { ist ein Fehler aufgetreten? }
    begin                                                         { Ja }
      EmmEC := Regs.ah;                        { den Fehlercode merken }
      EmsFreePage := EMS_ERR;                        { Fehler anzeigen }
    end
  else                                                   { kein Fehler }
    EmsFreePage := Regs.bx;        { Anzahl freier Pages zurckliefern }
end;
                                                                        
{**********************************************************************}
{* EmsFrameSeg: Ermittelt die Segmentadresse des Page-Frame.          *}
{* Eingabe : keine                                                    *}
{* Ausgabe : EMS_ERR bei einem Fehler, sonst die ermittelte Seg.adr.  *}
{**********************************************************************}
                                                                        
function EmsFrameSeg : word;
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $41;              { Fkt.nr.: Segmentadr. Page-Frame holen }
  Intr(EMS_INT, Regs);                                  { EMM aufrufen }
  if (Regs.ah <>0 ) then                 { ist ein Fehler aufgetreten? }
    begin                                                         { Ja }
      EmmEC := Regs.ah;                        { den Fehlercode merken }
      EmsFrameSeg := W_EMS_ERR;                      { Fehler anzeigen }
    end
  else                                                   { kein Fehler }
    EmsFrameSeg := Regs.bx;   { Segmentadresse des Page-Frame zurckl. }
end;
                                                                        
{**********************************************************************}
{* EmsAlloc: Allokiert die angegebene Anzahl Pages und liefert ein    *}
{*           Handle zum Zugriff auf diese Pages zurck.               *}
{* Eingabe : PAGES: die Anzahl zu allokierender Pages.                *}
{* Ausgabe : EMS_ERR bei einem Fehler, sonst das Handle.              *}
{**********************************************************************}
                                                                        
function EmsAlloc( Pages : integer ) : integer;
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $43;                          { Fkt.nr.: Pages allokieren }
  Regs.bx := Pages;            { Anzahl zu allokierender Seiten setzen }
  Intr(EMS_INT, Regs);                                  { EMM aufrufen }
  if (Regs.ah <>0 ) then                 { ist ein Fehler aufgetreten? }
    begin                                                         { Ja }
      EmmEC := Regs.ah;                        { den Fehlercode merken }
      EmsAlloc := EMS_ERR;                           { Fehler anzeigen }
    end
  else                                                   { kein Fehler }
    EmsAlloc := Regs.dx;                        { Handle zurckliefern }
end;
                                                                        
{**********************************************************************}
{* EmsMap  : Bildet eine der unter dem bergebenen Handle allokierten *}
{*           logischen Pages auf eine physikalische Page im Page-     *}
{*           Frame ab.                                                *}
{* Eingabe : HANDLE: das ber EmsAlloc erhaltene Handle.              *}
{*           LOGP  : die abzubildende logische Seite                  *}
{*           PHYSP : die physikalische Seite im Page-Frame.           *}
{* Ausgabe : FALSE bei einem Fehler, sonst TRUE.                      *}
{**********************************************************************}
                                                                        
function EmsMap(Handle, LogP : integer; PhysP : byte) : boolean;
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $44;                            { Fkt.nr.: Mapping setzen }
  Regs.al := PhysP;                       { physikalische Seite setzen }
  Regs.bx := LogP;                             { logische Seite setzen }
  Regs.dx := Handle;                               { EMS-Handle setzen }
  Intr(EMS_INT, Regs);                                  { EMM aufrufen }
  EmmEC := Regs.ah;                                { Fehlercode merken }
  EmsMap := (Regs.ah = 0)       { TRUE zurckliefern, wenn kein Fehler }
end;
                                                                        
{**********************************************************************}
{* EmsFree : Gibt den, unter einem Handle allokierten, Speicher       *}
{*           wieder frei.                                             *}
{* Eingabe : HANDLE: das ber EmsAlloc erhaltene Handle.              *}
{* Ausgabe : FALSE bei einem Fehler, sonst TRUE.                      *}
{**********************************************************************}
                                                                        
function EmsFree(Handle : integer) : boolean;
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $45;                           { Fkt.nr.: Pages freigeben }
  Regs.dx := handle;                               { EMS-Handle setzen }
  Intr(EMS_INT, Regs);                                  { EMM aufrufen }
  EmmEC := Regs.ah;                                { Fehlercode merken }
  EmsFree := (Regs.ah = 0)      { TRUE zurckliefern, wenn kein Fehler }
end;
                                                                        
{**********************************************************************}
{* EmsVersion: Ermittelt die Versionsnummer des EMM.                  *}
{* Eingabe : keine                                                    *}
{* Ausgabe : EMS_ERR bei einem Fehler, sonst die Versionsnummer.      *}
{*           Dabei steht 11 fr 1.1, 40 fr 4.0 etc.                  *}
{**********************************************************************}
                                                                        
function EmsVersion : integer;
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $46;                     { Fkt.nr.: EMM-Version ermitteln }
  Intr(EMS_INT, Regs);                                  { EMM aufrufen }
  if (Regs.ah <>0 ) then                 { ist ein Fehler aufgetreten? }
    begin                                                         { Ja }
      EmmEC := Regs.ah;                        { den Fehlercode merken }
      EmsVersion := EMS_ERR;                         { Fehler anzeigen }
    end
  else            { kein Fehler, Versionsnummer aus BCD-Zahl errechnen }
   EmsVersion := (Regs.al and 15) + (Regs.al shr 4) * 10;
end;
                                                                        
{**********************************************************************}
{* EmsSaveMap: Speichert die Abbildung zwischen logischen und physi-  *}
{*             kalischen Pages des bergebenen Handles.               *}
{* Eingabe : HANDLE: das ber EmsAlloc erhaltene Handle.              *}
{* Ausgabe : FALSE bei einem Fehler, sonst TRUE.                      *}
{**********************************************************************}
                                                                        
function EmsSaveMap( Handle : integer ) : boolean;
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $47;                         { Fkt.nr.: Mapping speichern }
  Regs.dx := handle;                               { EMS-Handle setzen }
  Intr(EMS_INT, Regs);                                  { EMM aufrufen }
  EmmEC := Regs.ah;                                { Fehlercode merken }
  EmsSaveMap := (Regs.ah = 0)   { TRUE zurckliefern, wenn kein Fehler }
end;
                                                                        
{**********************************************************************}
{* EmsRestoreMap: Setzt die Abbildung zischen logischen und physi-    *}
{*                kalischen Seiten, die zuvor ber EmsSaveMap ge-     *}
{*                speichert wurde, wieder zurck.                     *}
{* Eingabe : HANDLE: das ber EmsAlloc erhaltene Handle.              *}
{* Ausgabe : FALSE bei einem Fehler, sonst TRUE.                      *}
{**********************************************************************}
                                                                        
function EmsRestoreMap( Handle : integer ) : boolean;
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $48;                       { Fkt.nr.: Mapping zurckholen }
  Regs.dx := handle;                               { EMS-Handle setzen }
  Intr(EMS_INT, Regs);                                  { EMM aufrufen }
  EmmEC := Regs.ah;                                { Fehlercode merken }
  EmsRestoreMap := (Regs.ah = 0)     { TRUE zurckl., wenn kein Fehler }
end;
                                                                        
{**********************************************************************}
{* PrintErr: Gibt eine EMS-Fehlermeldung aus und beendet das Programm.*}
{* Eingabe : keine                                                    *}
{* Ausgabe : keine                                                    *}
{* Info    : Diese Funktion darf nur aufgerufen werden, wenn bei ei-  *}
{*           nem vorhergehenden Auruf einer Funktion aus diesem Modul *}
{*           ein Fehler gemeldet wurde.                               *}
{**********************************************************************}
                                                                        
procedure PrintErr;
                                                                        
begin
  writeln('ACHTUNG! Fehler bei Zugriff auf EMS-Speicher');
  write('     ... ');
  if ((EmmEC<$80) or (EmmEc>$8E) or (EmmEc=$82)) then
    writeln('nicht identifizierbar')
  else
    case EmmEC of
      $80 : writeln('Fehler im EMS-Treiber (EMM zerstrt)');
      $81 : writeln('Fehler in der EMS-Hardware');
      $83 : writeln('ungltiges EMM-Handle');
      $84 : writeln('aufgerufene EMS-Funtkion existiert nicht');
      $85 : writeln('keine freien EMS-Handles mehr verfgbar');
      $86 : writeln('Fehler beim Sichern oder Zurckspeichern ',
                    'des Mapping');
      $87 : writeln('mehr Seiten angefordert als physikalisch ',
                    'vorhanden');
      $88 : writeln('mehr Seiten angefordert als noch frei sind');
      $89 : writeln('Null Seiten angefordert');
      $8A : writeln('logische Seite gehrt nicht zum Handle');
      $8B : writeln('ungltige physikalische Seitennummer');
      $8C : writeln('Mapping-Speicherbereich ist voll');
      $8D : writeln('Sichern des Mapping wurde bereits durchgefhrt');
      $8E : writeln('Zurckspeichern des Mapping ohne ',
                    'verhergehendes Sichern');
    end;
  Halt;                                             { Programm beenden }
end;
                                                                        
{**********************************************************************}
{* VrAdr: Liefert einen Pointer auf den Video-RAM.                    *}
{* Eingabe : keine                                                    *}
{* Ausgabe : Pointer auf den Video-RAM.                               *}
{**********************************************************************}
                                                                        
function VrAdr : BytePtr;
                                                                        
var Regs : Registers;      { Prozessorregister fr den Interruptaufruf }
                                                                        
begin
  Regs.ah := $0f;                     { Fkt.nr.: Videmomodus ermitteln }
  Intr($10, Regs);                     { BIOS-Video-Interrupt aufrufen }
  if (Regs.al = 7) then                  { monochrome Bildschirmkarte? }
    VrAdr := ptr($B000, 0)               { Ja, Video-RAM bei B000:0000 }
  else                                   { Color-, EGA- oder VGA-KArte }
    VrAdr := ptr($B800, 0);                  { Video-RAM bei B800:0000 }
end;
                                                                        
{**********************************************************************}
{* PageAdr: Liefert die Adresse einer physikalischen Seite im Page-   *}
{*          Frame.                                                    *}
{* Eingabe : PAGE: Die Nummer der physikalischen Page (0 bis 3)       *}
{* Ausgabe : Pointer auf die physikalische Page.                      *}
{**********************************************************************}
                                                                        
function PageAdr( Page : integer ) : BytePtr;
                                                                        
begin
  PageAdr := Ptr( EmsFrameSeg + (Page shl 10), 0 );
end;
                                                                        
{**********************************************************************}
{**                           HAUPTPROGRAMM                          **}
{**********************************************************************}
                                                                        
begin
  ClrScr;                                         { Bildschirm lschen }
  writeln('EMMP  -  (c) 1988, 92 by MICHAEL TISCHER',#13#10);
  if EmsInst then                      { ist EMS-Speicher installiert? }
    begin                                                         { Ja }
      {*-- Informationen ber den EMS-Speicher ausgeben --------------*}
                                                                        
      EmmVer := EmsVersion;             { EMM-Versionsnummer ermitteln }
      if EmmVer = EMS_ERR then           { ist ein Fehler aufgetreten? }
        PrintErr;    { Ja, Fehlermeldung ausgeben und Programm beenden }
      writeln('EMM-Versionsnummer           : ',EmmVer div 10, '.',
              EmmVer mod 10);
                                                                        
      AnzPage := EmsNumPage;        { Gesamtanzahl der Pages ermitteln }
      if AnzPage = EMS_ERR then          { ist ein Fehler aufgetreten? }
        PrintErr;    { Ja, Fehlermeldung ausgeben und Programm beenden }
      writeln('Anzahl der EMS-Pages         : ', AnzPage, ' (',
               AnzPage shl 4, ' KByte)');
                                                                        
      AnzPage := EmsFreePage;           { Anzahl freie Pages ermitteln }
      if AnzPage = EMS_ERR then          { ist ein Fehler aufgetreten? }
        PrintErr;    { Ja, Fehlermeldung ausgeben und Programm beenden }
      writeln('... davon frei               : ', AnzPage, ' (',
               AnzPage shl 4, ' KByte)');
                                                                        
      PageSeg := EmsFrameSeg;          { Segmentadresse des Page-Frame }
      if PageSeg = W_EMS_ERR then        { ist ein Fehler aufgetreten? }
        PrintErr;    { Ja, Fehlermeldung ausgeben und Programm beenden }
      writeln('Segmentadresse des Page-Frame: ', PAgeSeg);
                                                                        
      writeln;
      writeln('Nun wird eine Seite aus dem EMS-Speicher allokiert und');
      writeln('und der  Bildschirminhalt aus dem  VIDEO-RAM in  diese');
      writeln('Seite kopiert.');
      writeln('                    ... bettigen Sie bitte eine Taste');
      Taste := ReadKey;                        { auf eine Taste warten }
                                                                        
      {*-- eine Seite allokieren und sie auf die erste logische ------*}
      {*-- Seite im Page-Frame abbilden                         ------*}
                                                                        
      Handle := EmsAlloc( 1 );                  { ein Seite allokieren }
      if Handle = EMS_ERR then           { ist ein Fehler aufgetreten? }
        PrintErr;    { Ja, Fehlermeldung ausgeben und Programm beenden }
      if not(EmsMap(Handle, 0, 0)) then               { Mapping setzen }
        PrintErr;    { Fehler: Fehlermeldung ausgeben und Prg. beenden }
                                                                        
      {*-- 4000 Bytes aus dem Video-RAM in den EMS-Speicher kopieren -*}
                                                                        
      Move(VrAdr^, PageAdr(0)^, 4000);
                                                                        
      ClrScr;                                 { den Bildschirm lschen }
      while KeyPressed do                { den Tastaturpuffer auslesen }
        Taste := ReadKey;
      writeln('Der alte Bildschirminhalt ist nun gelscht worden und');
      writeln('damit  unwiderbringlich verloren. Da er aber im  EMS-');
      writeln('Speicher  gesichert  wurde, kann er  von dort  wieder');
      writeln('zurck in den Video-RAM kopiert werden.');
      writeln('                   ... bettigen Sie bitte eine Taste');
      Taste := ReadKey;                        { auf eine Taste warten }
                                                                        
      {*-- Inhalt des Video-RAM aus dem EMS-Speicher zurckkopieren --*}
      {*-- und den allokierten EMS-Speicher wieder freigeben        --*}
                                                                        
      Move(PageAdr(0)^, VrAdr^, 4000);      { Video-RAM zurckkopieren }
      if not(EmsFree(Handle)) then                { Speicher freigeben }
        PrintErr;    { Fehler: Fehlermeldung ausgeben und Prg. beenden }
      GotoXY(1, 15);
      writeln('ENDE')
    end
 else                           { der EMS-Treiber wurde nicht entdeckt }
   writeln('ACHTUNG! Kein EMS-Speicher installiert.');
end.
