{***********************************************************************
*                              E X T P . P A S                         *
**--------------------------------------------------------------------**
*  Demmonstriert den Zugriff auf das Extended-Memory ber die BIOS-    *
*  Funktionen des Interrupts 15h bei Bercksichtigung von RAM-Disks    *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    : 18.05.1989                                       *
*  letztes Update am: 19.02.1992                                       *
***********************************************************************}

program ExtP;

uses Dos;

{-- globale Variablen -------------------------------------------------}

var RdLen    : integer;                   { Gre der RAM-Disks in KB  }
    ExtAvail : boolean;                   { Extended Memory verfgbar? }
    ExtStart : longint; { Startadresse des EXT-Memory als lineare Adr. }
    ExtLen   : integer;           { Gre des extended Memory in KByte }

{***********************************************************************
*  ExtAdrConv : konvertiert einen Pointer in eine 32-Bit groe lineare *
*               Adresse, die in Form eines LONGINTS zurckgeliefert    *
*               wird                                                   *
**--------------------------------------------------------------------**
*  Eingabe : Adr = der zu konvertierende Pointer                       *
*  Ausgabe : die konvertierte Adresse                                  *
***********************************************************************}

function ExtAdrConv ( Adr : pointer ) : longint;

type PTRREC = record                       { dient dem Zugriff auf die }
                 Ofs : word;               { Bestandteile eines belie- }
                 Seg : word;               { bigen Pointers            }
               end;

begin
  ExtAdrConv := longint( PTRREC( Adr ).seg ) shl 4 + PTRREC( Adr ).ofs;
end;

{***********************************************************************
*  ExtCopy : kopiert Daten zwischen zwei beliebigen Puffern innerhalb  *
*            des 16-MB groen Adressraums des 80286/386/486.           *
**--------------------------------------------------------------------**
*  Eingabe : Start = Adresse des Start-Puffers als 32-Bit lineare Adr. *
*            Ziel  = Adresse des Ziel-Puffers als 32-Bit lineare Adr.  *
*            Len   = Anzahl der zu kopierenden Bytes                   *
*  Info    : - Die Anzahl der zu kopierenden Bytes mu gradzahlig sein.*
*            - Diese Prozedur ist nur fr die Verwendung innerhalb     *
*              dieser Unit gedacht.                                    *
***********************************************************************}

procedure ExtCopy( Start, Ziel : longint; Len : word );

{-- Datenstrukturen zum Zugriff auf den extended RAM ------------------}

type SDES = record                                { Segmentbeschreiber }
              Laenge   : word;         { Lnge des Segments in Byte    }
              AdrLo    : word;         { Bit  0 bis 15 der Segmentadr. }
              AdrHi    : byte;         { Bit 16 bis 23 der Segmentadr. }
              Attribut : byte;         { Segmentattribut               }
              Res      : word;         { reserviert fr 80386          }
            end;

     GDT  = record                           { Global Descriptor Table }
              Dummy : SDES;
              GDTS  : SDES;
              Start : SDES;                          { kopiere von ... }
              Ziel  : SDES;                          { ... nach        }
              Code  : SDES;
              Stack : SDES;
            end;

     LI = record              { dient dem Zugriff auf die Bestandteile }
           LoWord : word;     { eine LongInts der eine lineare 32-Bit- }
           HiByte : byte;     { Adresse reprsentiert                  }
           dummy  : byte;
         end;

var GTab : GDT;                              { Global Descriptor Table }
    Regs : Registers;             { Prozessorregs. fr Interruptaufruf }
    Adr  : longint;                      { zur Umrechnung der Adressen }

begin
  FillChar( GTab, SizeOf( GTab ), 0 );             { alle Felder auf 0 }

  {-- Segmentdescriptor des Start-Segments aufbauen -------------------}

  GTab.Start.AdrLo    := LI( Start ).LoWord;
  GTab.Start.AdrHi    := LI( Start ).HiByte;
  GTab.Start.Attribut := $92;
  GTab.Start.Laenge   := Len;

  {-- Segmentdescriptor des Ziel-Segments aufbauen --------------------}

  GTab.Ziel.AdrLo    := LI( Ziel ).LoWord;
  GTab.Ziel.AdrHi    := LI( Ziel ).HiByte;
  GTab.Ziel.Attribut := $92;
  GTab.Ziel.Laenge   := Len;

  {-- Speicherbereiche mit Hilfe der Funtion $87 des Kassetten- -------}
  {-- Interrupts $15 kopieren                                   -------}

  Regs.AH := $87;            { Funktionsnummer fr 'Speicher kopieren' }
  Regs.ES := seg( GTab );                            { Adresse der GDT }
  Regs.SI := ofs( GTab );                            { nach ES:SI      }
  Regs.CX := Len shr 1;          { Anzahl zu kopierender Words nach CX }
  intr( $15, Regs );                               { Funktion aufrufen }
  if ( Regs.AH <> 0 ) then                                   { Fehler? }
    begin                                 { Ja, AH enthlt Fehler-Code }
      writeln('Fehler bei Zugriff auf Extended-Ram (', Regs.AH,')!');
      RunError;                { Programm mit Run-Time-Error abbrechen }
    end;
end;

{***********************************************************************
*  ExtRead : liest eine bestimmte Anzahl von Bytes aus dem extended-   *
*            Memory in den Hauptspeicher.                              *
**--------------------------------------------------------------------**
*  Eingabe :  ExtAdr = Source-Adresse im extended-RAM (lineare Adresse)*
*             BuPtr  = Pointer auf den Ziel-Puffer im Hauptspeicher    *
*             Len    = Anzahl der zu kopierenden Bytes                 *
***********************************************************************}

procedure ExtRead( ExtAdr : longint;  BuPtr : pointer;  Len : word );

begin
  ExtCopy( ExtAdr, ExtAdrConv( BuPtr ), len );
end;

{***********************************************************************
*  ExtWrite : schreibt eine bestimmte Anzahl von Bytes aus dem Haupt-  *
*             speicher in das extended Memory.                         *
**--------------------------------------------------------------------**
*  Eingabe :  BuPtr  = Pointer auf den Source-Puffer im Hauptspeicher  *
*             ExtAdr = Ziel-Adresse im extended-RAM (lineare Adresse)  *
*             Len    = Anzahl der zu kopierenden Bytes                 *
***********************************************************************}

procedure ExtWrite( BuPtr : pointer;  ExtAdr : longint;  Len : word );

begin
  ExtCopy( ExtAdrConv( BuPtr ), ExtAdr, len );
end;

{***********************************************************************
*  ExtGetInfo : ermittelt die Startadresse des extended RAM und seine  *
*               Gre und bercksichtigt dabei evtl. dort abgelegte    *
*               RAM-Disks                                              *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : keine                                                     *
*  Globals : ExtAvail/W, ExtStart/W, ExtLen/W                          *
***********************************************************************}

procedure ExtGetInfo;

type NAME_TYP    = array [1..5] of char;
type BOOT_SEKTOR = record                 { Boot-Sektor einer RAM-Disk }
                     dummy1   : array [1..3] of byte;
                     Name     : NAME_TYP;
                     dummy2   : array [1..3] of byte;
                     BpS      : word;
                     dummy3   : array [1..6] of byte;
                     Sektoren : word;
                     dummy4   : byte;         { fllt auf gerade Lnge }
                   end;

const VdiskName : NAME_TYP = 'VDISK';

var BootSek : BOOT_SEKTOR;      { nimmt vermeintlichen Boot-Sektor auf }
    Letzte  : boolean;                       { markiert Schleifeneinde }
    Regs    : Registers;          { Prozessorregs. fr Interruptaufruf }

begin
  {-- Gre des extended Memory ermitteln und dabei feststellen, ob ---}
  {-- berhaupt extended Memory zur Verfgung steht                 ---}

  Regs.ah := $88;   { Funktionsnr.: "Gre des Extended-RAM ermitteln" }
  intr( $15, Regs );                    { Kassetten-Interrupt aufrufen }
  if ( Regs.AX = 0 ) then
    begin                                { kein extended RAM vorhanden }
      ExtAvail := FALSE;
      ExtLen   := 0;
      ExtStart := 0;
      exit;                                      { zurck zum Aufrufer }
    end;

  ExtAvail := TRUE;                        { extended Memory verfgbar }
  ExtLen   := Regs.AX;          { extended RAM vorhanden, Gre merken }

  {-- nach RAM-Disks vom Typ VDISK suchen -----------------------------}

  ExtStart := $100000;                              { bei 1 MB starten }
  Letzte := FALSE;                             { von RAM-Disk ausgehen }
  repeat                                             { Abfrageschleife }
    ExtRead( ExtStart, @BootSek, SizeOf( BootSek ) );
    with BootSek do
      if Name = VDiskName then       { ist Boot-Sektor einer RAM-Disk? }
        inc( ExtStart, longint( Sektoren ) * BpS ) {Ja, hinter RAM-Disk}
      else
        Letzte := TRUE;                 { keine RAM-Disk mehr gefunden }
  until Letzte;

  {-- Gre der RAM-Disks vom freien extended RAM abziehen ------------}

  dec( ExtLen, integer( (ExtStart - longint($100000)) shr 10) );
end;

{***********************************************************************
*  CheckExt : berprft die Konsistenz des freien extended RAM         *
***********************************************************************}

procedure CheckExt;

var AdrTest   : longint;                      { Adresse des Testblocks }
    i, j      : integer;                             { Schleifenzhler }
    WriteBuf,                                             { Testblcke }
    ReadBuf   : array [1..1024] of byte;
    Fehler    : boolean;                    { Zeiger Speicherfehler an }

begin
  Randomize;                         { Zufallsgenerator initialisieren }
  AdrTest := ExtStart;
  for i := 1 to ExtLen do    { den Speicher in 1 KB-Blocks durchlaufen }
    begin
      for j := 1 to 1024 do           { Block mit Zufallszahlen fllen }
        WriteBuf[ j ] := Random( 255 );

      write(#13, AdrTest );  { Adresse des berprften KB-Blocks ausg. }

      {-- Buffer schreiben und dann wieder nach ReadBuf einlesen-------}

      ExtWrite( @WriteBuf, AdrTest, 1024 );
      ExtRead( AdrTest, @ReadBuf, 1024 );

      {-- Identitt von WriteBuf und ReadBuf feststellen --------------}

      for j := 1 to 1024 do
        if WriteBuf[j] <> ReadBuf[j] then    { Bufferinhalt identisch? }
          begin                                        { Nein, Fehler! }
            writeln( '  Fehler! Speicherstelle ',
                     AdrTest + longint(j-1) );
            Fehler := TRUE;
          end;

      inc( AdrTest, longint( 1024 ) ); { AdrTest auf nchsten KB-Block }
    end;                               { setzen                        }
 writeln;
 if not( Fehler ) then                   { ist ein Fehler aufgetreten? }
   writeln( 'Alles o.k.!' );                                    { Nein }
end;

{***********************************************************************
*  H A U P T P R O G R A M M                                           *
***********************************************************************}

begin
  writeln( #13#10'EXTDEMO - (c) 1989 by Michael Tischer'#13#10);
  ExtGetInfo;  { Verfgbarkeit und Gre des extended Memory ermitteln }
  if ExtAvail then                       { ist extended RAM vorhanden? }
    begin                                                         { Ja }
      RdLen := integer( (ExtStart - longint( $100000 ) ) shr 10 );
      if ( RdLen = 0 ) then              { sind RAM-Disks installiert? }
        begin                                                   { Nein }
          writeln( 'Es sind keine RAM-Disks installiert. ');
          writeln( 'Der freie extended RAM beginnt mit der ',
                   '1 MB Speichergrenze. ');
        end
      else                                   { Ja, RAM-Disks vorhanden }
        begin
          writeln( 'Eine oder mehrere RAM-Disks belegen ', RdLen,
                   ' KB des extended RAM.');
          writeln( 'Der freie extended RAM beginnt ', RdLen,
                   ' KB hinter der 1 MB-Speichergrenze.');
        end;
       writeln( 'Die Gre des freien extended RAM betrgt ',
                 ExtLen, ' KB.');
       writeln( #13#10'Der extended RAM wird nun auf seine Konsistenz',
                ' hin berprft...'#13#10);
       CheckExt;
    end
  else
    writeln( 'In Ihrem Rechner ist kein extended RAM installiert!');
end.
