{**********************************************************************}
{*                         F L O C K P . P A S                        *}
{*--------------------------------------------------------------------*}
{*    Aufgabe       : Demonstriert das File-Lockung in einem Netzwerk *}
{*                    mit Hilfe der Funktionen und Prozeduren aus der *}
{*                    Netzwerk-Unit NETFILEP                          *}
{*--------------------------------------------------------------------*}
{*    Autor         : Michael Tischer                                 *}
{*    entwickelt am : 10.02.1992                                      *}
{*    letztes Update: 13.02.1992                                      *}
{**********************************************************************}

program FLockP;

uses Crt, Dos,                   { die Crt- und die DOS-Unit einbinden }
     NetFileP;                            { die Netzwerkunit einbinden }

const DateiName = 'flockp.dat';                   { Name der Testdatei }

type  Test  = array[ 1..4 ] of char;         { Datentyp fr Test-Datei }

{**********************************************************************}
{* FModus   : Dateimodus aus Zugriffsart und Locking erzeugen         *}
{* Eingabe  : Zugriffsart, Schutzmodus der Datei                      *}
{* Ausgabe  : Dateimodus                                              *}
{**********************************************************************}

function FModus( Zugriff, Schutz : byte ) : byte;

{-- Zugriffs- und Dateischutzarten wie in Netfile definiert -----------}

const Zugriff_Art : array[ 1..3 ] of byte = ( FM_R, FM_W, FM_RW );
      Schutz_Art  : array[ 1..5 ] of byte = ( SM_COMP, SM_RW, SM_R,
                                              SM_W, SM_NO );

begin
  Fmodus := Zugriff_Art[ Zugriff ] or Schutz_Art[ Schutz ];
end;

{**********************************************************************}
{* DateiTest   : Demonstration der Zugriffskonflikte oder Datei-      *}
{*               sperrren mit und ohne File-Locking                   *}
{* Eingabe     : Zugriffsart und Schutzmodus der beiden konkurrieren- *}
{*               den Dateien                                          *}
{* Ausgabe     : keine                                                *}
{**********************************************************************}

procedure DateiTest( ZugriffA, SchutzA, ZugriffB, SchutzB : byte );

const TestAAus : Test = 'AAAA';                       { Testdatenstze }
      TestBAus : Test = 'BBBB';

var TestAEin,                                { Datenstze fr Lesetest }
    TestBEin  : Test;
    DateiA,                      { Testdateien fr gemeinsamen Zugriff }
    DateiB    : file of Test;

begin
  window( 1, 11, 80, 25 );
  clrscr;
  writeln( 'Datei A: Name = ', DateiName, ', Zugriffsart = ',
           ZugriffA, ',  Schutzmodus = ', SchutzA );
  writeln( 'Datei B: Name = ', DateiName, ', Zugriffsart = ',
           ZugriffB, ',  Schutzmodus = ', SchutzB );

  {-- Dateien erffnen ------------------------------------------------}

  write( #13#10'ffnen der Datei A:  ' );
  NetReset( DateiName, FModus( ZugriffA, SchutzA ),
            sizeof( Test ), DateiA );
  if ( NetError = NE_FileNotFound ) then
    NetRewrite( Dateiname, FModus( ZugriffA, SchutzA ),
                sizeof( test ), DateiA );
  Writeln( 'Status ', NetError : 2, '  = ', NetErrorMsg( NetError ) );

  write( 'ffnen der Datei B:  ' );
  NetReset( Dateiname, FModus( ZugriffB, SchutzB ),
            sizeof( Test ), DateiB );
  Writeln( 'Status ', NetError : 2, '  = ', NetErrorMsg( NetError ) );

  {-- Dateien schreiben -----------------------------------------------}

  write( #13#10'Schreiben in Datei A:' );
  if ( Is_NetWriteOk( DateiA ) ) then             { Schreiben erlaubt? }
    begin
      Netwrite( DateiA, TestAAus );
      writeln( ' Satz "', TestAAus, '" geschrieben ' );
    end
  else
    writeln( ' Datei nicht zur Ausgabe geffnet' );

  write( 'Schreiben in Datei B:' );
  if ( Is_NetWriteOk( DateiB ) ) then             { Schreiben erlaubt? }
    begin
      Netwrite( DateiB, TestBAus );
      writeln( ' Satz "', TestBAus, '" geschrieben ' );
    end
  else
    writeln( ' Datei nicht zur Ausgabe geffnet' );

  {-- Dateizeiger fr beide Dateien wieder auf den Anfang -------------}

  if Is_NetOpen( DateiA ) then                       { Datei geffnet? }
    NetSeek( DateiA, 0 );
  if Is_NetOpen( DateiB ) then                       { Datei geffnet? }
    NetSeek( DateiB, 0 );

  {-- Dateien auslesen ------------------------------------------------}

  write( #13#10'Lesen aus Datei A:' );
  if ( Is_NetReadOk( DateiA ) ) then                  { Lesen erlaubt? }
    begin
      Netread( DateiA, TestAEin );
      writeln( ' Satz "', TestAEin, '" gelesen ' );
    end
  else
    writeln( ' Datei nicht zur Eingabe geffnet' );

  write( 'Lesen aus Datei B:' );                      { Lesen erlaubt? }
  if ( Is_NetReadOk( DateiB ) ) then
    begin
      Netread( DateiB, TestBEin );
      writeln( ' Satz "', TestBEin, '" gelesen ' );
    end
  else
    writeln( ' Datei nicht zur Eingabe geffnet' );

  {-- Dateien schliessen ----------------------------------------------}

  NetClose( DateiA );
  NetClose( DateiB );
end;

{**********************************************************************}
{*               H A U P T P R O G R A M M                            *}
{**********************************************************************}

var ZugriffA,                              { Zugriffsarten der Dateien }
    ZugriffB,
    SchutzA,                                   { Lock-Modi der Dateien }
    SchutzB   : byte;

begin
  clrscr;
  writeln( 'FLOCKP Demonstration des File-Locking mit DOS       ',
           '(c) 1992 by Michael Tischer' );
  writeln( '====================================================',
           '===========================' );

  if ( ShareInst ) then                          { Schare installiert? }
    begin
      {-- Datei-Modi auswhlen ----------------------------------------}

      writeln( #13#10'Mgliche Zugriffsarten:         ',
                     'Mgliche Schutzarten:' );
      writeln( ' 1: Nur Lesen                    ',
               ' 1: Kompatibilittsmodus (kein Schutz)' );
      writeln( ' 2: Nur Schreiben                ',
               ' 2: Andere Dateizugriffe generell verboten' );
      writeln( ' 3: Lesen und Schreiben          ',
               ' 3: Nur Lese-Zugriffe erlaubt' );
      writeln( '                                 ',
               ' 4: Nur Schreib-Zugriffe erlaubt' );
      writeln( '                                 ',
               ' 5: Alles Erlaubt (Schutz ber Record-Locking)' );

      Write( #13#10'Zugriffsart Testdatei A: ' );
      read( ZugriffA );
      Write( 'Schutzmodus Testdatei A: ' );
      read( SchutzA );
      Write( 'Zugriffsart Testdatei B: ' );
      read( ZugriffB );
      Write( 'Schutzmodus Testdatei B: ' );
      read( SchutzB );

      DateiTest( ZugriffA, SchutzA, ZugriffB, SchutzB );
    end
  else
    writeln( #13#10'Test nicht mglich, SHARE mu installiert sein' );
end.
