{***********************************************************************
*                           H M A P . P A S                            *
**--------------------------------------------------------------------**
*  Aufgabe        : Demonstriert den direkten Zugriff auf die HMA ohne *
*                   die Beihilfe eines speziellen Treibers.            *
**--------------------------------------------------------------------**
*  Autor          : MICHAEL TISCHER                                    *
*  entwickelt am  : 27.07.1990                                         *
*  letztes Update : 29.07.1990                                         *
***********************************************************************}

program HMAP;

uses Crt;                                                 { fr ClrScr }

{***********************************************************************
* HMAAvail : berprft ob ein 80286 oder nachfolgender Prozessor und   *
*            mindestens 64 KB exdtended Memory vorhanden sind          *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Ausgabe : TRUE, wenn die HMA verfgbar ist, sonst FALSE              *
* Info    : - Der Aufruf dieser Funktion mu dem Aufruf aller anderen  *
*             Prozeduren und Funktionen aus diesem Programm voraus-    *
*             gehen.                                                   *
***********************************************************************}

function HMAAvail : boolean;

begin
  inline (
          $33/$C0/                          { xor    ax,ax             }
          $50/                              { push   ax                }
          $9D/                              { popf                     }
          $9C/                              { pushf                    }
          $58/                              { pop    ax                }
          $25/$00/$F0/                      { and    ax,0F000h         }
          $3D/$00/$F0/                      { cmp    ax,0F000h         }
          $74/$0E/                          { je     keinhma >Ŀ   }
          $B4/$88/                          { mov    ah,88h           }
          $CD/$15/                          { int    15h              }
          $3D/$40/$00/                      { cmp    ax,64            }
          $72/$05/                          { jb     keinhma >Ĵ   }
          $B8/$01/$00/                      { mov    ax,0001h         }
          $EB/$02/                          { jmp    ende             }
          $33/$C0/                          { xor    ax,ax <   }
          $88/$46/$FF                       { mov    [bp-1],al         }
        );
end;

{***********************************************************************
* GateA20 : Sperrt die Adressleitung A20 oder schaltet sie frei        *
**--------------------------------------------------------------------**
* Eingabe : FREI = TRUE, wenn die Leitung freigeschaltet werden soll   *
* Ausgabe : TRUE, wenn der Zugriff auf den Tastatur-Controller ge-     *
*           glckt ist, sonst FALSE                                    *
* Info    : - Nachg dem Aufruf dieser Funktion sollte mit Hilfe der    *
*             Funktion IsA20On festgestellt werden, ob die Adresslei-  *
*             tung tatschlich freigegeben wurde, weil dies nach dem   *
*             hier verwandten Verfahren nur bei Maschinen mit ISA-Bus  *
*             mglich ist.                                             *
***********************************************************************}

function GateA20( frei : boolean ) : boolean;

begin
  inline (
          $B4/$DD/                          { mov    ah,11011101b      }
          $83/$7E/$04/$00/                  { cmp    FREI,0            }
          $74/$02/                          { je     g1 Ŀ   }
          $B4/$DF/                          { mov    ah,11011111b     }
          $33/$C9/                          { xor    cx,cx <   }
          $FA/                              { cli                      }
          $E4/$64/                          { in     al,64 <Ŀ   }
          $A8/$02/                          { test   al,02            }
          $E0/$FA/                          { loopnz    }
          $75/$1D/                          { jne    gerr >Ŀ}
          $B0/$D1/                          { mov    al,WO_COMMAND    }
          $E6/$64/                          { out    KB_COMMAND,al    }
          $E4/$64/                          { in     al,64 <Ŀ  }
          $A8/$02/                          { test   al,02           }
          $E0/$FA/                          { loopnz   }
          $75/$11/                          { jne    gerr >Ĵ}
          $8A/$C4/                          { mov    al,ah            }
          $E6/$60/                          { out    KB_DATA,al       }
          $E4/$64/                          { in     al,64 <Ŀ  }
          $A8/$02/                          { test   al,02           }
          $E0/$FA/                          { loopnz   }
          $75/$05/                          { jne    gerr >Ĵ}
          $B8/$01/$00/                      { mov    ax,0001h         }
          $EB/$02/                          { jmp    ende             }
          $33/$C0/                          { xor    ax,ax <}
          $FB/                              { sti                      }
          $88/$46/$FF                       { mov    [bp-1],al         }
         );
end;

{***********************************************************************
* IsA20On : berprft, ob die Adressleitung A20 freigeschaltet ist     *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Ausgabe : TRUE, wenn der Leitung frei ist, sonst FALSE               *
***********************************************************************}

function IsA20On : boolean;

begin
  inline (
          $1E/                              { push   ds                }
          $06/                              { push   es                }
          $33/$F6/                          { xor    si,si             }
          $8E/$DE/                          { mov    ds,si             }
          $BF/$10/$00/                      { mov    di,0010           }
          $B8/$FF/$FF/                      { mov    ax,FFFF           }
          $8E/$C0/                          { mov    es,ax             }
          $B9/$40/$00/                      { mov    cx,64             }
          $FC/                              { cld                      }
          $F3/$A7/                          { repe   cmpsw             }
          $07/                              { pop    es                }
          $1F/                              { pop    ds                }
          $E3/$05/                          { jcxz   a20off Ŀ    }
          $B8/$01/$00/                      { mov    ax,0001h         }
          $EB/$02/                          { jmp    ende             }
          $33/$C0/                          { xor    ax,ax <    }
          $88/$46/$FF                       { mov    [bp-1],al         }
        );
end;

{***********************************************************************
* HMATest : Demonstriert den Zugriff auf die HMA                       *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
***********************************************************************}

procedure HMATest;

type HMAR    = array [1..65520] of BYTE;               { das HMA-Array }
     HMARPTR = ^HMAR;                       { Zeiger auf das HMA-Array }

var hmap  : HMARPTR;                              { Zeiger auf die HMA }
    i,                                               { Schleifenzhler }
    err   : word;                  { Anzahl der Fehler bei HMA-Zugriff }
    dummy : boolean;

begin
  if ( IsA20On ) then
   writeln( 'Die Adressleitung A20 ist bereits freigeschaltet!' )
  else
    if ( GateA20( TRUE ) = FALSE ) or ( IsA20On = FALSE ) then
      begin
        writeln( 'Achtung! Die Adressleitung A20 konnte nicht frei' +
                 'geschaltet werden.' );
        exit;
      end
    else
      writeln( 'Der Zugriff auf die HMA wurde freigeschaltet.' );

  hmap := HMARPTR(Ptr( $FFFF, $0010 ));               { Zeiger auf HMA }

  err := 0;                             { bis jetzt noch keinen Fehler }
  for i := 1 to 65520 do          { jede Speicherstelle einzeln testen }
    begin
      write( #13, 'Speicherstelle: ', i );
      hmap^[i] := i mod 256;              { Speicherstelle beschreiben }
      if ( hmap^[i] <> i mod 256 ) then          { und wieder auslesen }
        begin                                                { Fehler! }
          writeln( ' FEHLER!' );
          inc( err );
        end;
    end;

  writeln( #13 );
  if ( err = 0 ) then                   { Resultat des Tests auswerten }
    writeln( 'HMA ok, keine Speicherstelle fehlerhaft.')
  else
    writeln( 'ACHTUNG! ', err, ' fehlerhafte Speicherstellen in ' +
             'der HMA entdeckt! ');

  dummy := GateA20( FALSE );                { Adressleitung abschalten }
end;

{***********************************************************************
*                        H A U P T P R O G R A M M                     *
***********************************************************************}

begin
 writeln( 'HMAP  -  HMA-Demoprogramm von MICHAEL TISCHER'#10 );
 if HMAAvail then
   begin
     HMATest;                                             { HMA testen }
     writeln;
   end
 else
   writeln( 'Kein Zugriff auf HMA mglich.' );
end.

