{***********************************************************************
*  LaufText : demonstriert die Programmierung von Soft-Scrolling in    *
*             Verbindung mit EGA- und VGA-Karten                       *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    : 23.01.1990                                       *
*  letztes Update am: 14.01.1991                                       *
***********************************************************************}

program LaufText;

uses dos,                                            { Units einbinden }
     crt;

const SLOW      = 1;          { Konstanten fr SPEED in ShowLaufText() }
      MEDIUM    = 2;
      FAST      = 3;

const FARBE     = $5E;                           { Gelb auf Lila Grund }
      FARBE1    = $5F;                           { Wei auf Lila Grund }
      BREITE    = 8;                    { Breite der Zeichen in Pixeln }
      HOEHE     = 14;                { Hhe der Zeichen in Scan-Zeilen }
      SPALTEN   = 216;                { Spalten pro Zeile im Video-RAM }
      BANDSIZE  = 10800;                          { Gre eines Bandes }
      BAENDER   = 3;                               { Anzahl der Bnder }
      MAXLEN    = 61;                    { maximale Anzahl von Zeichen }
      STARTZ    = 6;       { Startzeile der Zeichen auf dem Bildschirm }

      CrtAttr   = $3C0;             { Crt-Attribut-Controller-Register }
      CrtStatus = $3da;                                   { Statusport }
      CrtAdr    = $3d4;                           { Monitor Adressport }


type VRAM = array[1..BAENDER,1..25,1..SPALTEN] of word;    { Video-RAM }
     VPTR = ^VRAM;                              { Zeiger auf Video-RAM }

var vp         : vptr;                           { Zeiger in Video-RAM }

type KARTE = ( EGA, VGA, WEDERNOCH );             { Art der Videokarte }

procedure CLI; inline( $FA );                { Interrupts unterdrcken }
procedure STI; inline( $FB );             { Interrupts wieder erlauben }

{***********************************************************************
*  SetOrigin : Legt den sichtbaren Teil des Video-RAM durch die Pro-   *
*              grammierung des Video-Controllers fest.                 *
**--------------------------------------------------------------------**
*  Eingabe :   Band    = Nummer des anzuzeigenden Bands (1-5)          *
*              Spalte, = Nummer der Spalte und Zeile, die in der ober- *
*              Zeile     en linken Bildschirmecke angezeigt werden     *
*                        sollen (Ursprung ist 0/0)                     *
*              PixX,   = Pixelloffsets                                 *
*              PixY                                                    *
***********************************************************************}

procedure SetOrigin( band, spalte, zeile, pixx, pixy : byte );

var offset : integer;                 { Offset fr Start des Video-RAM }
ch : char;

begin
  offset := ( BANDSIZE div 2 ) * (band-1) + zeile * SPALTEN + spalte;

  {-- Auf erneuten vertikalen Rcklauf und dessen Ende warten ---------}

  repeat until port[CrtStatus] and 8 = 8;
  repeat until port[CrtStatus] and 8 = 0;

  {-- Offset fr Start d. Video-RAM in Register $0C und $0D schreiben, }
  {-- wird aber erst beim nchsten Bildschirmaufbau gltig             }

  CLI;                                                { Interrupts aus }
  portw[ CrtAdr ] := hi( offset ) shl 8 + $0c;
  portw[ CrtAdr ] := lo( offset ) shl 8 + $0d;
  STI;

  {-- auf nchsten Rcklauf warten, damit Pixeloffset geschrieben  ----}
  {-- werden kann und gleichzeitig mit neuer Startadr. gltig wird ----}

  repeat until port[CrtStatus] and 8 = 8;

  {-- Pixeloffsets in das Register $08 bzw. in das Register $13 des ---}
  {-- Attribut-Controllers schreiben                                ---}

  CLI;
  portw[ CrtAdr ] := pixy shl 8 + $08;
  port[ CrtAttr ] := $13 or $20;         { auf den Attribut-Controller }
  port[ CrtAttr ] := pixx;               { nur Byteweise zugreifen     }
  STI;                                          { Interrupts wieder an }

                                                      { ch := readkey; }
end;

{***********************************************************************
*  PrintChar : Baut ein Zeichen auerhalb des sichtbaren Bereichs des  *
*              Video-RAMs auf.                                         *
**--------------------------------------------------------------------**
*  Eingabe :   Zeichen = das aufzubauende Zeichen                      *
*              Band    = Nummer des Bands (0-4)                        *
*              Spalte  = Spalte im Video-RAM, an der das Zeichen be-   *
*                        ginnen soll                                   *
*  Info    :   Das aufgebaute Zeichen kann durch Aufruf von Smooth-    *
*              Left in den sichtbaren Teil des Bildschirms geholt wer- *
*              den.                                                    *
*              Das Punktmuster des Zeichens wird dem 14*8 Punkte Zei-  *
*              chesatz im EGA-/VGA-ROM entnommen.                      *
***********************************************************************}

procedure PrintChar( zeichen : char; band, spalte : byte );

type ZDEF = array[0..255,1..14] of byte;      { Aufbau des Zeichensatz }
     ZPTR = ^ZDEF;                            { Zeiger auf Zeichensatz }

var  Regs  : Registers;                 { Register zum Interruptaufruf }
     ch    : char;                      { einzelner Punkt des Zeichens }
     i, k,                                           { Schleifenzhler }
     Maske : byte;                  { Bitmaske fr Aufbau des Zeichens }

const fptr : ZPTR = NIL;               { Zeiger auf Zeichensatz im ROM }

begin
  if fptr = NIL then         { Zeiger auf Zeichensatz schon ermittelt? }
    begin                                                       { Nein }
      Regs.AH := $11;                   { Funktion $11 des Video-BIOS, }
      Regs.AL := $30;                   { Unterfunktion $30 aufrufen   }
      Regs.BH := 2;            { Zeiger auf 8*14 Zeichensatz ermitteln }
      intr( $10, Regs );
      fptr := ptr( Regs.ES, Regs.BP );         { Zeiger zusammensetzen }
    end;

  {-- das Zeichen Zeile fr Zeile durchlaufen und aufbauen ------------}

  for i := 1 to HOEHE do
    begin
      Maske := fptr^[ord(zeichen),i]; { Bitmuster fr eine Zeile holen }
      for k := 1 to BREITE do         { die einzelnen Bits durchlaufen }
        begin
          if Maske and 128 = 128 then ch := #219       { Punkt gesetzt }
                                 else ch := #32;      { Punkt gelscht }
          vp^[band, STARTZ+i, (spalte-1)*BREITE+k] :=
                                                  ord(ch) + FARBE shl 8;
          Maske := Maske shl 1;              { nchstes Bit bearbeiten }
        end;
    end;
end;

{***********************************************************************
*  IsEgaVga : Stellt fest, ob eine EGA- oder VGA-Karte installiert ist.*
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : EGA, VGA oder WEDERNOCH                                   *
***********************************************************************}

function IsEgaVga : KARTE;

var Regs : Registers;          { Prozessorregister fr Interruptaufruf }

begin
  Regs.AX := $1a00;                 { Funktion 1Ah gibt es nur bei VGA }
  Intr( $10, Regs );
  if ( Regs.AL = $1a ) then              { ist die Funktion verfgbar? }
    IsEgaVga := VGA
  else
    begin
      Regs.ah := $12;                     { Funktion 12h Unterfunktion }
      Regs.bl := $10;                     { 10h aufrufen               }
      intr($10, Regs);                           { Video-BIOS aufrufen }
      if ( Regs.bl <> $10 ) then IsEgaVga := EGA
                            else IsEgaVga := WEDERNOCH;
    end;
end;

{***********************************************************************
*  ShowLaufText : Bringt eien Text als Lauftext auf den Bildschirm     *
**--------------------------------------------------------------------**
*  Eingabe :  LTEXT = der Lauftext als String                          *
*             SPEED = die Geschwindigkeit (Konstanten SLOW, FAST etc.) *
*             VK    = Art der Videokarte (EGA oder VGA)                *
***********************************************************************}

procedure ShowLaufText( ltext : string; speed : byte; vk : karte );

var band,                              { das aktuell durchlaufene Band }
    spalte,                               { aktuell bearbeitete Spalte }
    index,                         { Index in den aufzubauenden String }
    len,                              { Lnge des auszugebenden Textes }
    i, k : integer;                                  { Schleifenzhler }
    step,                                               { Schrittweite }
    uplimit : byte;                   { Anzahl der Schleifendurchlufe }
    Regs : Registers;          { Prozessorregister fr Interruptaufruf }

const steptable : array [EGA..VGA,1..3,1..10] of byte =
        (
          (                                        { EGA-Schrittweiten }
            (   0,   1,   2,   3,   4,   5,   6,   7, 255, 255 ),
            (   0,   2,   4,   6, 255, 255, 255, 255, 255, 255 ),
            (   0,   4, 255, 255, 255, 255, 255, 255, 255, 255 )
          ),
          (                                        { VGA-Schrittweiten }
            (   8,   0,   1,   2,   3,   4,   5,   6,   7, 255 ),
            (   8,   2,   5, 255, 255, 255, 255, 255, 255, 255 ),
            (   8,   3, 255, 255, 255, 255, 255, 255, 255, 255 )
          )
        );

begin
  vp := ptr( $B800, $0000 );             { Zeiger auf Video-RAM setzen }

  {-- gesamten Video-RAM mit Leerzeichen fllen -----------------------}

  for index := 1 to BAENDER do
    for i := 1 to 25 do
      for k := 1 to SPALTEN do
        vp^[ index, i, k ] := FARBE shl 8 + 32;

  {-- horizontale Fhrungsschienen malen ------------------------------}

  for k := 1 to BAENDER do
    for i := 1 to SPALTEN do
      begin
        vp^[ k, STARTZ-2, i ] := ord('') + FARBE1 shl 8;
        vp^[ k, STARTZ + HOEHE + 2, i ] := ord('') + FARBE1 shl 8;
      end;

  gotoxy( 1, 1 );       { Cursor verschwindet beim Verschieben des Bs. }

  {-- Farbe fr den Bildschirmrahmen festlegen ------------------------}

  Regs.AH := $10;           { Funktionsnummer fr "Rahmenfarbe setzen" }
  Regs.AL := $01;                               { Unterfunktionsnummer }
  Regs.BH := FARBE shr 4;                                { Rahmenfarbe }
  intr( $10, Regs );

  {-- Anzahl der Spalten pro Zeile im Video-RAM auf SPALTEN festlegen -}

  portw[ CrtAdr ] := ( SPALTEN div 2 ) shl 8 + $13;

  {-- den Lauftext komplett im Video-RAM aufbauen ---------------------}

  if length( ltext ) > MAXLEN then len := MAXLEN
                              else len := length( ltext );
  spalte := 1;
  band   := 1;
  index  := 1;
  while ( index <= len ) do
    begin
      PrintChar( ltext[index], band, spalte );  { das Zeichen aufbauen }
      inc( spalte );                           { in die nchste Spalte }
      inc( index );                   { das nchste Zeichen bearbeiten }
      if ( spalte > ( SPALTEN div BREITE ) ) then       { Bandwechsel? }
        begin                                                     { Ja }
          spalte := 1;                   { wieder in Spalte 1 beginnen }
          inc( band );                           { in das nchste Band }
          dec( index, ( 80 div BREITE ) );  { Zeichen eine Page zurck }
        end
    end;

  {-- den Lauftext schrittweise von rechts nach links ber den Bild- --}
  {-- schirm laufen lassen                                           --}

  spalte := 0;                { in der Spalte 0 und im Band 1 beginnen }
  band   := 1;
  for i := 1 to (len-( 80 div BREITE )) * BREITE do
    begin
      k := 1;
      while ( steptable[vk, speed, k] <> 255 ) do
        begin
          SetOrigin( band, spalte, 0, steptable[vk, speed, k], 0 );
          inc( k );
        end;

      inc( spalte );                           { in die nchste Spalte }
      if ( spalte = SPALTEN - 80 ) then                 { Bandwechsel? }
        begin                                                     { Ja }
          spalte := 0;                   { wieder in Spalte 0 beginnen }
          inc( band );                           { Band inkrementieren }
        end
    end;

  {-- wieder 80 Spalten pro Zeile im Video-RAM einstellen -------------}

  portw[ CrtAdr ] := 40 shl 8 + $13;

  {-- Farbe fr den Bildschirmrahmen zurcksetzen ---------------------}

  Regs.AH := $10;           { Funktionsnummer fr "Rahmenfarbe setzen" }
  Regs.AL := $01;                               { Unterfunktionsnummer }
  Regs.BH := 0;                                       { Rahmen schwarz }
  intr( $10, Regs );

  if ( vk = EGA ) then                     { Einstellungen auf Default }
    SetOrigin( 1, 0, 0, 0, 0 )                               { fr EGA }
  else
    SetOrigin( 1, 0, 0, 8, 0 );                              { fr VGA }
  ClrScr;
end;

{-- H A U P T P R O G R A M M -----------------------------------------}

var  ch : char; i : integer;
     vk : karte;

begin
  vk := IsEgaVga;                       { Art der Videokarte ermitteln }
  if vk = EGA then halt;
  if ( vk = WEDERNOCH ) then
    begin
      writeln( 'LAUFTEXT  -  (c) 1990 by MICHAEL TISCHER' );
      writeln( #13#10'Achtung! Kein EGA- oder VGA-Karte installiert.');
    end
  else
    ShowLaufText( '+++ PCIntern (c) 1987-1991 by Data Becker +++ '+
                  '          ', FAST, vk );
  while keypressed do
   ch := readkey;

end.
