{***********************************************************************
*                          L O G O P . P A S                           *
**--------------------------------------------------------------------**
*  Aufgabe        : Demonstriert die Definition eigener Zeichenstze   *
*                   bei EGA- und VGA-Karten am Beispiel einer Routine  *
*                   zur Darstellung vo Logos innerhalb des Textmodus   *
**--------------------------------------------------------------------**
*  Autor          : MICHAEL TISCHER                                    *
*  entwickelt am  :  5.08.1990                                         *
*  letztes Update : 14.01.1991                                         *
***********************************************************************}

Program Logop;

uses DOS, CRT;

{-- Konstanten --------------------------------------------------------}

const EGAVGA_SEQUENCER = $3C4;            { Sequencer Adre-/Datenport }
      EGAVGA_MONCTR    = $3D4;                    { Monitor-Controller }
      EGAVGA_GRAPHCTR  = $3CE;  { Graphics-Controller Adre-/Datenport }
      EV_STATC         = $3DA;  { EGA-/VGA-Karten Statusregister Color }
      EV_STATM         = $3BA;   { EGA-/VGA-Karten Statusregister Mono }
      EV_ATTR          = $3C0;   { EGA-/VGA-Karten Attribut-Controller }

{-- Typedeklarationen -------------------------------------------------}

type KARTE = ( EGA, VGA, WEDERNOCH );

procedure CLI; inline( $FA );                { Interrupts unterdrcken }
procedure STI; inline( $FB );             { Interrupts wieder zulassen }

{***********************************************************************
*  SetCharWidth: Legt die Breite der Zeichen bei VGA-Karten auf 8 oder *
*                9 Punkte fest.                                        *
**--------------------------------------------------------------------**
*  Eingabe : BREITE = Breite der Zeichen (8 oder 9)                    *
***********************************************************************}

procedure SetCharWidth( breite : byte );

var Regs : Registers;          { Prozessorregister zum Interruptaufruf }
    x    : byte;                  { nimmt Wert fr Misc-Output-Reg auf }

begin
  if ( breite = 8 ) then Regs.BX := $0001    { BH = horiz. Ausrichtung }
                    else Regs.BX := $0800;   { BL = Wert fr Seq. Reg. }

  x := port[ $3CC ] and not(4+8);  { horizontale Auflsung von 720 auf }
  if ( breite = 9 ) then           { 640 Punkte umschalten und umge-   }
    x := x or 4;                   { kehrt                             }
  port[ $3C2 ] := x;

  CLI;        { Sequencer von 8 auf 9 Punkte nzw. umgekehrt umschalten }
  portw[ EGAVGA_SEQUENCER ] := $0100;
  portw[ EGAVGA_SEQUENCER ] := $01 + Regs.BL shl 8;
  portw[ EGAVGA_SEQUENCER ] := $0300;
  STI;

  Regs.AX := $1000;                   { Bildschirmausrichtung anpassen }
  Regs.BL := $13;
  intr( $10, Regs );
end;

{***********************************************************************
*  IsEgaVga : Stellt fest, ob eine EGA- oder VGA-Karte installiert ist.*
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : EGA, VGA oder WEDERNOCH                                   *
***********************************************************************}

function IsEgaVga : KARTE;

var Regs : Registers;          { Prozessorregister fr Interruptaufruf }

begin
  Regs.AX := $1a00;                 { Funktion 1Ah gibt es nur bei VGA }
  Intr( $10, Regs );
  if ( Regs.AL = $1a ) then              { ist die Funktion verfgbar? }
    IsEgaVga := VGA
  else
    begin
      Regs.ah := $12;                     { Funktion 12h Unterfunktion }
      Regs.bl := $10;                     { 10h aufrufen               }
      intr($10, Regs);                           { Video-BIOS aufrufen }
      if ( Regs.bl <> $10 ) then IsEgaVga := EGA
                            else IsEgaVga := WEDERNOCH;
    end;
end;

{***********************************************************************
*  BuildLogo : Baut ein Logo auf dem Bildschirm auf, indem das Logo aus*
*              verschiedenen Zeichen zusammengesetzt wird, die im deut-*
*              schen Sprachgebrauch nicht vorkommen.                   *
**--------------------------------------------------------------------**
*  Eingabe : SPALTE = Startspalte des Logos (1-80)                     *
*            ZEILE  = Startzeile des Logos (1-25)                      *
*            TIEFE  = Anzahl der Logo-Scan-Zeilen                      *
*            FARBE  = Ausgabefarbe des Logos                           *
*            BUF    = Array mit Strings, die das Zeichenmuster des     *
*                     Logos enthalten                                  *
*  Info    : - Ein Beispiel fr den Aufbau des zu bergebenden Logo-   *
*              Puffers liefert die Prozedur TEST.                      *
*            - Das Logo wird innerhalb seines Zeichenblocks zentriert  *
*              dargestellt.                                            *
***********************************************************************}

procedure BuildLogo( spalte, zeile, tiefe, farbe : byte; var buf );

type BYTEAR = array[0..10000] of byte;     { ein Byte-Array zum Durch- }
     BARPTR = ^BYTEAR;                     { laufen des Logo-Puffers   }

const MAX_CHAR = 32;       { soviele Zeichen knnen umdefiniert werden }

const UseChars : array[1..MAX_CHAR] of byte =   { die umzudef. Zeichen }
                  ( 128, 130, 131, 133, 134, 135, 136, 137, 138, 139,
                    140, 141, 143, 144, 145, 146, 147, 149, 150, 151,
                    152, 155, 156, 157, 158, 159, 160, 161, 162, 163,
                    164, 165 );

var Regs      : Registers;    { Prozessorregister fr Interrupt-Aufruf }
    videok    : KARTE;              { Typ der installierten Videokarte }
    chardef   : array[0..15] of byte;       { Bitmuster eines Zeichens }
    charhoehe,                    { Anzahl der Scan-Zeilen pro Zeichen }
    i, j, k, l,                                   { Schleifenvariablen }
    maske,                 { Bit-Maske fr Erstellung einer Scan-Zeile }
    breite,                                       { Breite der Strings }
    index,                  { Index zum Durchlaufen des UseChars-Array }
    dx,                        { Breite des Logo-Blocks (Text-Spalten) }
    dy,                          { Tiefe des Logo-Blocks (Text-Zeilen) }
    links,                                     { linker Rand in Pixeln }
    rechts,                                   { rechter Rand in Pixeln }
    oben,                                      { oberer Rand in Pixeln }
    unten    : byte;                           { untere Rand in Pixeln }
    bptr     : barptr;             { zur Adressierung des Logo-Puffers }

{-- Unterfunktion IsSet: prft, ob ein Logo-Pixel gesetzt ist ---------}

function IsSet( zeile, spalte : byte ) : boolean;

begin
  if ( zeile < oben ) or ( zeile > unten ) or        { Pixel auerhalb }
     ( spalte < links ) or ( spalte > rechts ) then  { des Rands?      }
   IsSet := false                                  { Ja, nicht gesetzt }
  else                               { Nein, aus Logo-Puffer ermitteln }
   IsSet := bptr^[ (zeile-oben)*(breite+1) + 1 + (spalte-links) ] <> 32;
end;

{-- Hauptprozedur -----------------------------------------------------}

begin
  videok := IsEgaVga;                          { Video-Karte ermitteln }
  case videok of
    WEDERNOCH :
      begin
        writeln( 'Fehler: Kein EGA- oder VGA-Karte installiert!' );
        exit;
      end;

    EGA       :
      charhoehe := 14;               { EGA: 14 Scan-Zeilen pro Zeichen }

    VGA       :
      begin                                                      { VGA }
        SetCharWidth( 8 );             { Zeichen nur noch 8 Pix. breit }
        charhoehe := 16;                  { 16 Scan-Zeilen pro Zeichen }
      end;
    end;

  bptr := @buf;                        { Zeiger auf Logo-Puffer setzen }
  breite := bptr^[0];  { Lnge der Strings und damit Logo-Breite holen }
  dx := ( breite + 7 ) div 8;     { Anzahl der bentigten Zeichen ber. }
  dy := ( tiefe + charhoehe - 1 ) div charhoehe;
  if ( dx*dy > MAX_CHAR ) then
    writeln( 'Fehler: Logo in BuildLogo zu gro! ' )
  else
    begin
      oben   := ( dy*charhoehe-tiefe ) div 2;         { Rand berechnen }
      unten  := tiefe + oben - 1;
      links  := ( dx*8-breite ) div 2;
      rechts := breite + links - 1;

      TextColor( farbe and 15 );                 { Ausgabefarbe setzen }
      TextBackGround( farbe shr 4 );
      index := 1;       { erstes Zeichen aus UseChar-Array heranziehen }
      for i := 0 to dy-1 do              { die Text-Zeilen durchlaufen }
        begin
          GotoXY( spalte, zeile + i );
          for j := 1 to dx do           { die Text-Spalten durchlaufen }
            begin
              write( chr( UseChars[ index ] ) );    { Zeichen ausgeben }

              {-- neues Zeichenmuster fr das ausg. Zeichen berechnen -}

              for k := 0 to charhoehe-1 do   { Scan-Zeilen durchlaufen }
                begin
                  maske := 0;                     { Bit-Maske bisher 0 }
                  for l := 0 to 7 do   { jedes Zeichen ist 8 Pix breit }
                    begin
                      maske := maske shl 1;  {Maske nach links schieben}
                      if IsSet( i*charhoehe+k, (j-1)*8+l ) then
                        maske := maske or 1;   { Punkt im Logo gesetzt }
                    end;
                  chardef[ k ] := maske; { Bitmuster in Zeichen-Puffer }
                end;
              Regs.AX := $1100;          { das neue Zeichenmuster mit  }
              Regs.BH := charhoehe;      { Hilfe der Unterfunktion $00 }
              Regs.BL := 0;              { der Funktion $10 des BIOS-  }
              Regs.CX := 1;              { Video-Interrupts festlegen  }
              Regs.DX := UseChars[ index ];
              Regs.ES := seg( chardef );
              Regs.BP := ofs( chardef );
              intr( $10, Regs );

              inc( index );      { nchstes Zeichen aus UseChar nehmen }
            end;
        end;
    end;
end;

{***********************************************************************
*  ResetLogo : Ldt wieder die ursprnglichen Zeichendefinitionen      *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************}

procedure ResetLogo;

var Regs      : Registers;    { Prozessorregister fr Interrupt-Aufruf }

begin
  case IsEgaVga of
    EGA  : begin
             Regs.AX := $1101;   { 8*14 Zeichensatz komplett neu laden }
             Regs.BL := 0;
             intr( $10, Regs );
           end;

    VGA  : begin
             SetCharWidth( 9 );         { Zeichen wieder mit 9 Punkten }
             Regs.AX := $1104;   { 8*16 Zeichensatz komplett neu laden }
             Regs.BL := 0;
             intr( $10, Regs );
           end;
    end;
end;

{***********************************************************************
*  Test : demonstriert die Arbeit mit der Prozedur BuildLogo           *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
***********************************************************************}

procedure Test;

const MyLogo : array[1..32] of string[38] =
                ( '                  **                  ',
                  '                 ****                 ',
                  '                 ****                 ',
                  '                  **                  ',
                  '                                      ',
                  '                                      ',
                  '                                      ',
                  '                 ****                 ',
                  '                 ****                 ',
                  '**************************************',
                  '**************************************',
                  '***              ****              ***',
                  '**               ****               **',
                  '*                ****                *',
                  '                 ****                 ',
                  '                 ****                 ',
                  '      ********   ****   ********      ',
                  '      ****  **** **** ****  ****      ',
                  '      ****     ********     ****      ',
                  '      ****      ******      ****      ',
                  '      ****       ****       ****      ',
                  '      ****       ****       ****      ',
                  '      ****       ****       ****      ',
                  '      ****       ****       ****      ',
                  '      ****       ****       ****      ',
                  '      ****       ****       ****      ',
                  '      ****       ****       ****      ',
                  '      ****       ****       ****      ',
                  '      ****       ****       ****      ',
                  '      ****       ****       ****      ',
                  '      ****       ****       ****      ',
                  '      ****       ****       ****      ' );

var ch : char;
    i  : byte;

const NeuDef : set of char = [ #128, #130, #131, #133..#141, #143..#147,
                               #149..#152, #155..#165 ];

begin
  TextBackGround( BLACK );
  ClrScr;
  GotoXY( 1, 1 );
  for i := 0 to 255 do           { den kompletten Zeichensatz ausgeben }
    begin
      if ( chr(i) in NeuDef ) then TextColor( WHITE )
                             else TextColor( YELLOW );

      GotoXY( i mod 13 * 6 + 2, i div 13 + 1 );
      write( i:3 , ':' );
      if ( i <> 13 ) and ( i <> 10 ) and ( i <> 7 ) then
       write( chr( i ) );
    end;

  GotoXY( 23, 22 );
  write( 'LOGOP  -  (c) 1990 by MICHAEL TISCHER' );
  BuildLogo( 61, 21, 32, CYAN shl 4 + WHITE, MyLogo );    { Logo aufb. }
  ch := ReadKey;                                    { auf Taste warten }
  ResetLogo;                                     { Logo wieder lschen }
  ClrScr;
  GotoXY( 1, 1 );
end;

{**********************************************************************}
{-- H A U P T P R O G R A M M                                        --}
{**********************************************************************}

begin
  Test;
end.
