{*****************************************************************************
*                             M R C I P . P A S                              *
*----------------------------------------------------------------------------*
*     Aufgabe        : Demonstriert die Nutzung eines MRCI-Servers zur       *
*                      Kompression und Dekompression von Daten               *
*----------------------------------------------------------------------------*
*     Autor          : MICHAEL TISCHER                                       *
*     entwickelt am  : 25.09.1993                                            *
*     letztes Update : 26.09.1993                                            *
*****************************************************************************}

program MrciP;

uses Dos, Crt;

{-- Typen und Strukturen ----------------------------------------------------}

type MRCINFO = record                               { beschreibt MRCI-Server }
       HerstellerID      : longint;                  { 4-byte ASCII-Code mit }
                                                     { Hersteller-Namen      }
       HerstellerVersion : word;        { Versionsnummer des MRC-Servers     }
                                        { High-Byte is Hauptversion, Lo-Byte }
                                        { ist Unterversion                   }
       MrciVersion       : word;       { Versionsnummer des MRCI, das dieser }
                                       { Server untersttzt                  }
       Einsprungspunkt   : pointer;         { Einsprungspunkt fr Ausfhrung }
                                            { von Server-Funktionen          }
       Flags             : word;                        { Server-Fhigkeiten }
       HardwareFlags     : word;            { Server-Fhigkeiten in Hardware }
       MaxBytes          : word;             { grter Block, den der Server }
                                             { verarbeiten kann              }
    end;

type MRCIREQUEST = record                 { fr Aufruf von Server-Funktionen }
       SourcePtr   : pointer;                        { Zeiger in Quellpuffer }
       SourceLen   : word;                  { Lnge des Quellpuffers in Byte }
       Reserviert1 : word;   
       DestPtr     : pointer;                         { Zeiger in Zielpuffer }
       DestLen     : word;                           { Lnge des Zielpuffers }
       ChunkLen    : word;               { Blockgre fr komprimierte Daten }
       IncDecomp   : longint;           { Status fr inkrementelles Dekompr. }
     end;

type BUF = record          { beschreibt einen Puffer fr Compress/Decompress }
   adresse : pointer;                                    { Zeiger auf Puffer }
   lenb    : word;                                  { Lnge des Puffers in Byte }
end;

{-- Konstanten --------------------------------------------------------------}

{-- Flags und Befehlscodes --------------------------------------------------}
const STANDARD_COMPRESS = 1;                         { Standard-Komprimieren }
const STANDARD_DECOMPRESS = 2;                     { Standard-Dekomprimieren }
const MAX_COMPRESS = 8;                            { maximales Komprimieren  }
const INC_DECOMPRESS = 32;                   { inkrementelles Dekomprimieren }

{-- Art des MRCI-Clients ----------------------------------------------------}
const TYP_ANWENDUNG  = 0;                   { MRCI-Client ist eine Anwendung }
const TYP_TSRTREIBER = 1;           { MRCI-Client ist TSR oder Gertetreiber }

{-- Fehlercodes -------------------------------------------------------------}
const MRCI_OK             = 0;                                  { alles o.k. }
const MRCI_ERR_FUNKTION   = 1;                      { falscher Funktionscode }
const MRCI_ERR_BUSY       = 2;                      { Server ist beschftigt }
const MRCI_ERR_OVERFLOW   = 3;                         { Zielpuffer zu klein }
const MRCI_ERR_NOCOMPRESS = 4;       { Daten knnen nicht komprimiert werden }

{-- Konstanten fr Demoprogramm ---------------------------------------------}
const TYP_APP       = TYP_ANWENDUNG;                 { das hier ist kein TSR }
const CHUNK         = 1;               { wenn es geht, bis auf 1 Byte kompr. }

{-- globale Variablen -------------------------------------------------------}

var mrci : mrcinfo;                  { nimmt MRCI-Info-Block nach Aufruf von }
                                     { MrciQuery auf                         }

{*****************************************************************************
* MrciQuery : berprft die Existenz eines MRCI-Servers in Hardware oder     *
*             Software und liefert Informationen ber diesen Server          *
*----------------------------------------------------------------------------*
* Eingabe : MrciInfo = enthlt nach einem erfolgreichen Funktionsaufruf      *
*                      einen MRCI-Info-Block mit Informationen ber den      *
*                      Server                                                *
* Ausgabe : TRUE wenn Mrci-Server installiert ist, sonst FALSE               *
*****************************************************************************}

function MrciQuery ( var mrci : mrcinfo ) : boolean;

var server : boolean;                           { gibt es einen MRCI-Server? }
    regs   : registers;

type mrcip = ^mrcinfo;                                  { Zeiger auf MRCINFO }

begin
  regs.ax := $4A12;                               { MUX-Code fr MRCI-Server }
  regs.cx := ( ord( 'M' ) shl 8 ) + ord( 'R' );                  { CX = 'MR' }
  regs.dx := ( ord( 'C' ) shl 8 ) + ord( 'I' );                  { DX = 'CI' }
  intr( $2F, regs);
  if ( regs.cx = ( (ord( 'I' ) shl 8 ) + ord( 'C' ) ) ) and     { CX == 'IC' }
     ( regs.dx = ( (ord( 'R' ) shl 8 ) + ord( 'M' ) ) ) then    { DX == 'RM' }
    server := true                     { CX und DX o.k. ---> Server gefunden }
  else 
    begin             { kein Sofware-Server gefunden, Hardware-Server testen }
      regs.ax := $B001;                    { Int 1A, Funktion B001h aufrufen }
      regs.cx := ( ord( 'M' ) shl 8 ) + ord( 'R' );              { CX = 'MR' }
      regs.dx := ( ord( 'C' ) shl 8 ) + ord( 'I' );              { DX = 'CI' }
      intr( $1A, regs);
      if ( regs.cx = ( (ord( 'I' ) shl 8 ) + ord( 'C' ) ) ) and
         ( regs.dx = ( (ord( 'R' ) shl 8 ) + ord( 'M' ) ) ) then
        server := true                 { CX und DX o.k. ---> Server gefunden }
    end;

  if server then     { wenn Server gefunden, dann zeigt ES:DI auf Info-Block }
    mrci := mrcip(ptr( Regs.es, Regs.di ))^;
  MrciQuery := server;
end;

{*****************************************************************************
* MrciCall : Ruft einen der verschiedenen Server-Dienste auf                 *
*----------------------------------------------------------------------------*
* Eingabe : OperationCode = Befehlscode (siehe Konstanten)                   *
*           SourceBuf     = Quellpuffer                                      *
*           DestBuf       = Zielpuffer                                       *
* Ausgabe : Fehlercode des MRCI-Servers                                      *
* Info    : Diese Funktion darf erst nach vorhergehendem und erfolgreichem   *
*           Aufruf von MrciQuery aufgerufen werden                           *
* Globals : mrci
*****************************************************************************}

function MRCICall(     OperationCode : word;
                   var sourcebuf     : buf;
                   var destbuf       : buf ) : integer;

var mrcir : MRCIREQUEST;              { zum Aufbau eines MRCI-Request-Blocks }
    mrcrp : pointer;                                       { zeigt auf MRCIR }
    err   : integer;                          { zurckgelieferter Fehlercode }

begin
  mrcir.SourcePtr := sourcebuf.adresse;          { Source-Puffer in Request- }
  mrcir.SourceLen := sourcebuf.lenb;             { Block verzeichnen         }

  mrcir.DestPtr := destbuf.adresse;                { Ziel-Puffer in Request- }
  mrcir.DestLen := destbuf.lenb;                   { Block verzeichnen       }

  mrcir.ChunkLen := CHUNK;                 { Chunk-Gre aus Konstante holen }
  mrcrp := @mrcir;                         { Zeiger auf Request-Block merken }

  asm
    push    bp                                            { Register sichern }
    push    ds

    {-- zunchst in eine Windows-Critical-Section eintreten, damit unter ----}
    {-- Windows im 486 Enhanced Mode nicht mehrere VMs gleichzeitig den  ----}
    {-- MRCI-Server aufrufen knnen                                      ----}

    push    ax                               { Windows erwartet genau diese  }
    mov     ax,8001h                         { Befehlsfolge und keine andere }
    int     2ah                     
    pop     ax                      

    {--- Aufruf dss MRCI-Servers vorbereiten --------------------------------}
    mov     ax,OperationCode
    mov     cx,TYP_APP

    mov     dx,seg mrci              { Zeiger auf MRCI-Info-Block nach ES:BX }
    mov     es,dx
    mov     bx,offset mrci
    lds     si,mrcrp                        { DS:SI auf Request-Block setzen }
    call    es:MRCInfo.Einsprungspunkt[bx]                 { Server aufrufen }

    {-- Windows-Critical-Section wieder verlassen ---------------------------}
    push    ax                             { auch hier mu die Befehlssquenz }
    mov     ax,8101h                       { genau eingehalten werden        }
    int     2ah                 
    pop     ax                  

    pop     ds                                        { Register zurckholen }
    pop     bp

    mov     err,ax                  { zurckgelieferten Fehlercode speichern }
  end;
  MRCICall := err;          { Fehlercode als Funktionsergebnis zurckliefern }
  destbuf.lenb := mrcir.DestLen;       { Lnge des Zielpuffers zurckliefern }
end;

{*****************************************************************************
* Compress : komprimiert einen Datenblock mit Hilfe des MRCI-Servers         *
*----------------------------------------------------------------------------*
* Eingabe : TypCompress = eine der Beiden Konstanten STANDARD_COMPRESS oder  *
*                         MAX_COMPRESS                                       *
*           SourceBuf   = Quellpuffer                                        *
*           DestBuf     = Zielpuffer                                         *
* Ausgabe : Fehlercode des MRCI-Servers                                      *
* Info    : Diese Funktion darf erst nach vorhergehendem und erfolgreichem   *
*           Aufruf von MrciQuery aufgerufen werden                           *
*****************************************************************************}

function Compress(     TypCompress : word;
                   var sourcebuf   : buf;
                   var destbuf     : buf ) : integer;

begin
  Compress := MRCICall( STANDARD_COMPRESS, sourcebuf, destbuf );
end;

{*****************************************************************************
* Decompress : dekomprimiert einen Datenblock mit Hilfe des MRCI-Servers     *
*----------------------------------------------------------------------------*
* Eingabe : SourceBuf = Quellpuffer                                          *
*           DestBuf   = Zielpuffer                                           *
* Ausgabe : Fehlercode des MRCI-Servers                                      *
* Info    : - Diese Funktion darf erst nach vorhergehendem und erfolgreichem *
*             Aufruf von MrciQuery aufgerufen werden                         *
*           - Der Datenblock mu vom MRCI-Server komprimiert worden sein     *
*****************************************************************************}

function Decompress( var sourcebuf : buf; var destbuf : buf ) : integer;

begin
  Decompress := MRCICall( STANDARD_DECOMPRESS, sourcebuf, destbuf );
end;

{*****************************************************************************
* JaNein : berprft Flags und gibt Ja/Nein-Meldung aus                      *
*----------------------------------------------------------------------------*
* Eingabe : Status = Status, in dem sich das gewnschte Flag versteckt       *
*           Flag   = Bitwert des Flags                                       *
* Ausgabe : Ja oder Nein als String, jeweils 4 Zeichen breit                 *
*****************************************************************************}

function JaNein( Status, Flag : word ) : string;

begin
  If (Status and Flag) = Flag then
    JaNein := 'Ja  '
  else
    JaNein := 'Nein'
end;

{*****************************************************************************
* PrintMrciInfos: Gibt Statusinformationen ber MRCI-Server aus              *
*----------------------------------------------------------------------------*
* Eingabe : keine                                                            *
* Ausgabe : keine                                                            *
* Info    : Diese Funktion darf erst nach vorhergehendem und erfolgreichem   *
*           Aufruf von MrciQuery aufgerufen werden                           *
* Globals : mrci
*****************************************************************************}

procedure PrintMrciInfos;

begin
  writeln ( 'Hersteller                   : ',
            chr( ( mrci.HerstellerID and 255) ),
            chr( ( mrci.HerstellerID shr 8 ) and 255 ) ,
            chr( ( mrci.HerstellerID shr 16 ) and 255 ),
            chr( mrci.HerstellerID shr 24 ) );
  writeln ( 'Hersteller-Version           : ',
            mrci.HerstellerVersion shr 8, '.',
            mrci.HerstellerVersion and 255 );
  writeln ( 'MRCI-Version                 : ',
            mrci.MrciVersion shr 8, '.',
            mrci.MrciVersion and 255 );
  writeln;
  writeln ( 'Features                       Software   Hardware');
  writeln ( '----------------------------------------------------' );
  writeln ( 'Standard-Komprimieren        :   ',
            janein( mrci.Flags, STANDARD_COMPRESS ), '       ',
            janein( mrci.HardwareFlags, STANDARD_COMPRESS ) );
  writeln ( 'Standard-Dekomprimieren      :   ',
            janein( mrci.Flags, STANDARD_DECOMPRESS ), '       ',
            janein( mrci.HardwareFlags, STANDARD_DECOMPRESS ) );
  writeln ( 'Maximal Komprimieren         :   ',
            janein( mrci.Flags, MAX_COMPRESS ), '       ',
            janein( mrci.HardwareFlags, MAX_COMPRESS ) );
  writeln ( 'Inkrementelles Dekomprimieren:   ',
            janein( mrci.Flags, INC_DECOMPRESS ), '       ',
            janein( mrci.HardwareFlags, INC_DECOMPRESS ) );
  writeln;
end;

{----------------------------------------------------------------------------}
{-- Hauptprogramm -----------------------------------------------------------}
{----------------------------------------------------------------------------}

const SBUFLEN = 12196;                 { Umfang der zu komprimierenden Daten }

var sourcebuf : array [1..SBUFLEN] of byte;          { Quell- und Zielpuffer }
    destbuf   : array [1..SBUFLEN] of byte;
    sb,                                  { beschreiben Quell- und Zielpuffer }
    db        : BUF;
    err,                                                        { Fehlercode }
    i         : integer;                                   { Schleifenzhler }

begin
  writeln( 'MRCIP  -  (c) 1993 by Michael Tischer');
  writeln;
  if not MrciQuery( mrci ) then
    begin
      writeln ('MRCI nicht installiert');
      exit;
   end;

  {-- MRCI ist installiert, zunchst Infos ber MRCI-Server ausgeben --------}

  writeln ( 'MRCI installiert');
  PrintMrciInfos;

  {-- Puffer mit Daten fllen, komprimieren und dekomprimieren --------------}
  sb.adresse := @sourcebuf;                     { Beschreiber von Quell- und }
  sb.lenb := SBUFLEN;                           { Zielpuffer aufbauen        }
  db.adresse := @destbuf;
  db.lenb := SBUFLEN;

  for i := 1 to SBUFLEN do                    { Quellpuffer mit Daten fllen }
    sourcebuf[i] := 1;

  writeln('Komprimieren');
  writeln('Lnge vorher: ', SBUFLEN, ' Bytes ');
  err := Compress( STANDARD_COMPRESS, sb, db );
  writeln ( 'Fehlercode = ', err );
  writeln('Lnge nachher: ', db.lenb, ' Bytes ');

  writeln;
  writeln('Dekomprimieren');
  writeln('Lnge vorher: ', db.lenb, ' Bytes ');
  err := Decompress( db, sb );
  writeln ( 'Fehlercode = ', err );
  writeln('Lnge nachher: ', sb.lenb, ' Bytes ');
end.