{**********************************************************************}
{*                       R A W C O O K P . P A S                      *}
{*--------------------------------------------------------------------*}
{*    Aufgabe        : stellt zwei Funktionen zur Verfgung, mit      *}
{*                     denen ein Zeichen-Gerte-Treiber in den RAW-   *}
{*                     oder in den COOKED-Modus umgeschaltet werden   *}
{*                     kann                                           *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  : 16.08.1987                                     *}
{*    letztes Update : 28.10.1991                                     *}
{**********************************************************************}
                                                                        
program RAWCOOKP;
                                                                        
Uses Crt, Dos;                   { die CRT- und die DOS-Unit einbinden }
                                                                        
const STANDEIN = 0; { Handle 0 ist mit dem Standardeingabeg. verbunden }
      STANDAUS = 1; { Handle 1 ist mit dem Standardausgabeg. verbunden }
                                                                        
var Taste : char;           { wird nur fr das Demo-Programm gebraucht }
                                                                        
{**********************************************************************}
{* GetMode: liest das Attribut eines Gerte-Treibers ein              *}
{* Eingabe : das bergebene Handle mu mit dem anzusprechenden Gert  *}
{*           verbunden sein                                           *}
{* Ausgabe : das Gerte-Attribut                                      *}
{**********************************************************************}
                                                                        
function GetMode(Handle : integer) : integer;
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ah := $44;                 { Funktionsnummer fr IOCTL: Get Mode }
 Regs.bx := Handle;
 MsDos(  Regs );                          { DOS-Interrupt $21 aufrufen }
 GetMode := Regs.dx                        { Gerte-Attribut bergeben }
end;
                                                                        
{**********************************************************************}
{* SetRaw: versetzt einen Zeichen-Treiber in den RAW-Modus            *}
{* Eingabe : das bergebene Handle mu mit dem anzusprechenden        *}
{*           Gert verbunden sein                                     *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure SetRaw(Handle : integer);
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ax := $4401;               { Funktionsnummer fr IOCTL: Set Mode }
 Regs.bx := Handle;
 Regs.dx := GetMode(Handle) and 255 or 32;     { neues Gerte-Attribut }
 MsDos( Regs );                           { DOS-Interrupt $21 aufrufen }
end;
                                                                        
{**********************************************************************}
{* SetCooked: versetzt einen Zeichen-Treiber in den COOKED-Modus      *}
{* Eingabe : das bergebene Handle mu mit dem anzusprechenden Gert  *}
{*           verbunden sein                                           *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure SetCooked(Handle : integer);
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ax := $4401;               { Funktionsnummer fr IOCTL: Set Mode }
 Regs.bx := Handle;
 Regs.dx := GetMode(Handle) and 223;           { neues Gerte-Attribut }
 MsDos( Regs );                           { DOS-Interrupt $21 aufrufen }
end;
                                                                        
{**********************************************************************}
{* TestAusgabe: gibt einen Test-String 1000 mal auf dem Standard-     *}
{*              ausgabegert aus                                      *}
{* Eingabe : keine                                                    *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure TestAusgabe;
                                                                        
var Regs     : Registers;     { Prozessorregister fr Interrupt-Aufruf }
    Zaehler  : integer;                            { Schleifenvariable }
    Test     : string[9];               { der auszugebende Test-String }
                                                                        
begin
 Test := 'Test.... ';
 Regs.bx := STANDAUS;          { auf das Standardausgabegert ausgeben }
 Regs.cx := 9;                                    { Anzahl der Zeichen }
 Regs.ds := Seg(Test);                     { Segmentadresse des Textes }
 Regs.dx := Ofs(Test)+1;                    { Offsetadresse des Textes }
 for Zaehler := 1 to 1000 do            { den String 1000 mal ausgeben }
  begin
    Regs.ah := $40;             { Funktionsnummer fr Handle-Schreiben }
    MsDos( Regs );                        { DOS-Interrupt $21 aufrufen }
   end;
 writeln;
end;
                                                                        
{**********************************************************************}
{*                            HAUPTPROGRAMM                           *}
{**********************************************************************}
                                                                        
begin
  writeln('RAWCOOKP  -  (c) 1987, 91 by Michael Tischer'#13#10);
  writeln('Der CON-Treiber befindet sich jetzt im RAW-Modus.' +
          ' Steuertasten wie z.B.' + #13#10 +
          '<CTRL-S> werden bei der Ausgabe nicht erkannt. ' + #13#10 +
          'Probieren Sie das ruhig bei den folgenden Ausgaben aus!');
  writeln('Bitte zum Start ein Taste bettigen...');
  Taste := ReadKey;                                 { auf Taste warten }
  SetRaw(STANDEIN);                { Consolen-Treiber in den RAW-Modus }
  TestAusgabe;                         { Test-String 1000 mal ausgeben }

  while KeyPressed do                      { den Tastaturpuffer leeren }
    Taste := ReadKey;                                 { Taste auslesen }

  writeln('Der Consolen-Treiber befindet sich jetzt im COOKED-Modus.' +
          ' Steuertasten wie z.B. <CTRL-S> werden bei der Ausgabe ' +
          ' erkannt und entsprechend beantwortet!' );
  writeln('Bitte zum Start ein Taste bettigen...');
  Taste := ReadKey;                                 { auf Taste warten }
  SetCooked(STANDEIN);         { Treiber in den COOKED-Modus versetzen }
  TestAusgabe;                         { Test-String 1000 mal ausgeben }
end.
