{***********************************************************************
*                           S 3 2 2 0 P . P A S                        *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Arbeit mit Sprites im VGA-      *
*                     320*200-Grafikmodus bei 256 Farben und vier      *
*                     Bildschirmseiten. Das Programm greift dabei      *
*                     auf die Assembler-Routinen aus den Moduln        *
*                     V3220PA.ASM und S3220PA.ASM zurck.              *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    : 12.09.1990                                       *
*  letztes Update am: 14.01.1991                                       *
***********************************************************************}

program S3220P;

uses dos, crt;

{-- externe Referenzen auf die Assembler-Routinen ---------------------}

{$L v3220pa}                               { Assembler-Modul einbinden }

procedure init320200; external;
procedure setpix( x, y : integer; farbe : byte ); external;
function  getpix( x, y: integer ) : byte ; external;
procedure setpage( page : byte ); external;
procedure showpage( page : byte ); external;

{$L s3220pa}                               { Assembler-Modul einbinden }

procedure blockmove( vonpage : byte; vonx, vony : integer;
                     nachpage : byte; nachx, nachy : integer;
                     breite, hoehe: byte; bmskp : pointer ); external;

{-- Konstanten --------------------------------------------------------}

const MAXX = 319;                         { Maximale X- und Y-Ordinate }
      MAXY = 199;

      OUT_LEFT   = 1;       { fr Kollisionsmarkierung in SpriteMove() }
      OUT_TOP    = 2;
      OUT_RIGHT  = 4;
      OUT_BOTTOM = 8;
      OUT_NO     = 0;                              { nirgendwo drauen }

{-- Typdeklarationen --------------------------------------------------}

type SPLOOK = record                  { Erscheinungsbild eines Sprites }
                breite,                                { Breite gesamt }
                hoehe,                           { Hhe in Punktzeilen }
                seite,                         { abgelegt in Seite ... }
                msklen : byte;          { Lnge jeweils eines Eintrags }
                bmskp  : pointer;               { Zeiger auf Bit-Maske }
                zeile  : integer;   { Punktzeile, ab der das Sprite in }
              end;                  { seiner Seite abgelegt wurde      }
     SPLP = ^SPLOOK;                 { Zeiger auf das Erscheinungsbild }

     SPID = record                           { Sprite-Beschreiber (ID) }
              hgseite :byte;                        { Hintergrundseite }
              x, y : array [0..1] of integer; { Koordinaten S. 0 und 1 }
              hgx, hgy : integer;             { Puffer fr Hintergrund }
              splookp : SPLP;            { Zeiger auf Erscheinungsbild }
            end;
     SPIP = ^SPID;                     { Zeiger auf Sprite-Beschreiber }

     BYTEAR = array[0..10000] of byte;     { zur Adressierung der ver- }
     BARPTR = ^BYTEAR;                     { schiedenen Puffer         }

     PTRREC = record       { zur Zerlegung eines Zeigers oder LONINGTS }
                ofs,
                seg : word;
              end;

{***********************************************************************
*  IsVga : Stellt fest, ob eine VGA-Karte installiert ist.             *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : TRUE oder FALSE                                           *
***********************************************************************}

function IsVga : boolean;

var Regs : Registers;          { Prozessorregister fr Interruptaufruf }

begin
  Regs.AX := $1a00;                 { Funktion 1Ah gibt es nur bei VGA }
  Intr( $10, Regs );
  IsVga := ( Regs.AL = $1a );
end;

{***********************************************************************
*  PrintChar : Schreibt ein Zeichen im Grafikmodus in den Bildschirm   *
**--------------------------------------------------------------------**
*  Eingabe :   zeichen = das aufzubauende Zeichen                      *
*              x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************}

procedure PrintChar( zeichen : char; x, y : integer; vg, hg : byte );

type ZDEF = array[0..255,0..7] of byte;       { Aufbau des Zeichensatz }
     ZPTR = ^ZDEF;                            { Zeiger auf Zeichensatz }

var  Regs  : Registers;                 { Register zum Interruptaufruf }
     ch    : char;                      { einzelner Punkt des Zeichens }
     i, k,                                           { Schleifenzhler }
     Maske : byte;                  { Bitmaske fr Aufbau des Zeichens }

const fptr : ZPTR = NIL;               { Zeiger auf Zeichensatz im ROM }

begin
  if fptr = NIL then         { Zeiger auf Zeichensatz schon ermittelt? }
    begin                                                       { Nein }
      Regs.AH := $11;                   { Funktion $11 des Video-BIOS, }
      Regs.AL := $30;                   { Unterfunktion $30 aufrufen   }
      Regs.BH := 3;             { Zeiger auf 8*8 Zeichensatz ermitteln }
      intr( $10, Regs );
      fptr := ptr( Regs.ES, Regs.BP );         { Zeiger zusammensetzen }
    end;

  if ( hg = 255 ) then                 { Zeichen transparent zeichnen? }
    for i := 0 to 7 do             { Ja, nur Vordergrund-Punkte setzen }
      begin
        Maske := fptr^[ord(zeichen),i]; {Bitmuster fr eine Zeile holen}
        for k := 0 to 7 do
          begin
            if ( Maske and 128 <> 0 ) then             { Punkt setzen? }
              setpix( x+k, y+i, vg );                             { Ja }
            Maske := Maske shl 1;
          end;
      end
  else                        { Nein, auch Hintergrund bercksichtigen }
    for i := 0 to 7 do                        { die Zeilen durchlaufen }
      begin
        Maske := fptr^[ord(zeichen),i]; {Bitmuster fr eine Zeile holen}
        for k := 0 to 7 do
          begin
            if ( Maske and 128 <> 0 ) then              { Vordergrund? }
              setpix( x+k, y+i, vg )                              { Ja }
            else
              setpix( x+k, y+i, hg );              { Nein, Hintergrund }
            Maske := Maske shl 1;
         end;
      end;
end;

{***********************************************************************
*  PrintString: Schreibt einen String im Grafikmodus in den Bildschirm *
**--------------------------------------------------------------------**
*  Eingabe :   x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*              tstr    = der auszugebende String                       *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************}

procedure PrintString( x, y : integer; vg, hg : byte; tstr : string );

var i : integer;                                     { Schleifenzhler }

begin
  for i := 1 to length( tstr ) do             { den String durchlaufen }
    begin
      PrintChar( tstr[i], x, y, vg, hg );               { und ausgeben }
      inc( x, 8 );                    { Ausgabeposition inkrementieren }
    end;
end;

{***********************************************************************
*  Line: Zieht eine Linie innerhalb nach dem Bresenham-Algorithmus     *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endekoordinate                                   *
*            FARBE  = Farbe der Linienpunkte                           *
***********************************************************************}

procedure Line( x1, y1, x2, y2 : integer; farbe : byte );

var d, dx, dy,
    aincr, bincr,
    xincr, yincr,
    x, y                 : integer;

{-- Unterprozedur, vertauscht zwei Integer-Variablen ------------------}

procedure SwapInt( var i1, i2: integer );

var dummy : integer;

begin
  dummy := i2;
  i2    := i1;
  i1    := dummy;
end;

{-- Hauptprozedur -----------------------------------------------------}

begin
  if ( abs(x2-x1) < abs(y2-y1) ) then   { ber X- oder Y-Achse laufen? }
    begin                                           { ber die Y-Achse }
      if ( y1 > y2 ) then                              { y1 grer y2? }
        begin
          SwapInt( x1, x2 );                   { Ja, X1 mit X2 und     }
          SwapInt( y1, y2 );                   { Y1 mit Y2 vertauschen }
        end;

      if ( x2 > x1 ) then xincr := 1      { Spalteninkrement festlegen }
                     else xincr := -1;

      dy := y2 - y1;
      dx := abs( x2-x1 );
      d  := 2 * dx - dy;
      aincr := 2 * (dx - dy);
      bincr := 2 * dx;
      x := x1;
      y := y1;

      setpix( x, y, farbe );                     { ersten Punkt setzen }
      for y:=y1+1 to y2 do              { Line auf X-Achse durchlaufen }
        begin
          if ( d >= 0 ) then
            begin
              inc( x, xincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          setpix( x, y, farbe );
        end;
    end
  else                                              { ber die X-Achse }
    begin
      if ( x1 > x2 ) then                              { x1 grer x2? }
        begin
          SwapInt( x1, x2 );                   { Ja, X1 mit X2 und     }
          SwapInt( y1, y2 );                   { Y1 mit Y2 vertauschen }
        end;

      if ( y2 > y1 ) then yincr := 1       { Zeileninkrement festlegen }
                     else yincr := -1;

      dx := x2 - x1;
      dy := abs( y2-y1 );
      d  := 2 * dy - dx;
      aincr := 2 * (dy - dx);
      bincr := 2 * dy;
      x := x1;
      y := y1;

      setpix( x, y, farbe );                     { ersten Punkt setzen }
      for x:=x1+1 to x2 do              { Line auf X-Achse durchlaufen }
        begin
          if ( d >= 0 ) then
            begin
              inc( y, yincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          setpix( x, y, farbe );
       end;
    end;
end;

{***********************************************************************
*  GrafPrint: Gibt einen formatierten String im Grafikbildschirm aus   *
**--------------------------------------------------------------------**
*  Eingabe : X, Y   = Startkoordinate (0 - ...)                        *
*            VG     = Vordergrundfarbe                                 *
*            HG     = Hintergrundfarbe (255 = transparent)             *
*            STRING = String mit Formatinformationen                   *
***********************************************************************}

procedure GrafPrint( x, y : integer; vg, hg : byte; strt : string );

var i : integer;                                     { Schleifenzhler }

begin
 for i:=1 to length( strt ) do
   begin
     printchar( strt[i], x, y, vg, hg );     { ber PrintChar ausgeben }
     inc( x, 8 );                  { X auf die nchste Zeichenposition }
   end;
end;

{***********************************************************************
*  CreateSprite: Erzeugt ein Sprite auf der Basis einer bereits kompi- *
*                lierten Punktmusters                                  *
**--------------------------------------------------------------------**
*  Eingabe : SPLOOKP = Zeiger auf die von CompileSprite erzeugte       *
*                      Datenstruktur                                   *
*            HGSEITE = Bildschirmseite, in der der Sprite-Hintergrund  *
*                      aufbewahrt werden soll                          *
*            HGX,    = Koordniate in der HGSEITE, ab der der Hinter-   *
*            HGY       grund des Sprites abgelegt werden soll          *
*  Ausgabe : Zeiger auf die erstellte Sprite-Struktur                  *
*  Info    : Fr die Ablage des Sprite-Hintergrundes werden zwei ne-   *
*            beneinanderliegende Flchen von der Gre des jeweiligen  *
*            Sprites bentigt.                                         *
***********************************************************************}

function CreateSprite( splookp : SPLP; hgseite : byte;
                       hgx, hgy : integer              ) : SPIP;

var spidp : SPIP;                { Zeiger auf erzeugte Sprite-Struktur }

begin
  new( spidp );           { Speicher fr Sprite-Beschreiber allokieren }
  spidp^.splookp := splookp;                    { Daten in die Sprite- }
  spidp^.hgseite := hgseite;                    { Struktur eintragen   }
  spidp^.hgx := hgx;
  spidp^.hgy := hgy;

  CreateSprite := spidp;    { Zeiger auf Sprite-Struktur zurckliefern }
end;

{***********************************************************************
*  CompileSprite: Erzeugt das Punkt- und Bitmusters eines Sprites,     *
*                 auf der Basis einer zur Laufzeit angegebenen Sprite- *
*                 definition                                           *
**--------------------------------------------------------------------**
*  Eingabe : BUFP    = Zeiger auf Array mit Zeigern auf Strings, die   *
*                      das Zeichenmuster des Sprites enthalten         *
*            HOEHE   = Hoehe des Sprites und damit Anzahl der Strings  *
*            SEITE   = Grafik-Seite zum Aufbau des Sprites             *
*            Y       = Punktzeile, in der das Sprite aufgebaut wird    *
*            FB      = ASCII-Zeichen, das fr die kleinste Farbe steht *
*            VGFARBE = erste Farbcode fr FB                           *
*  Info    : Die Sprites werden in der angegebenen Punktzeile ab dem   *
*            linken Zeilenrand angelegt                                *
***********************************************************************}

function CompileSprite( var buf; hoehe, seite : byte;
                        y : integer; fb : char; vgfarbe : byte ) : SPLP;

type BYPTR  = ^byte;                             { Zeiger auf ein Byte }

var  breite,                                      { Breite der Strings }
     c,                         { nimmt Zeichen aus C-Sprite-Array auf }
     farbe,                           { umgesetzte Farbe eines Punktes }
     i, k, l,                                     { Schleifenvariablen }
     pixc,                  { Punktzhler fr Erstellung der Bit-Maske }
     pixm     : byte;                                     { Punktmaske }
     abstand,             { Abstand von Sprite-Anfang zu Sprite-Anfang }
     lx, ly   : integer;                             { Laufkoordinaten }
     splookp  : SPLP;            { Zeiger auf erzeugte Sprite-Struktur }
     lspb     : BYPTR;                   { Laufzeiger in Sprite-Puffer }
     bptr     : barptr; {zur Adressierung des Puffers mit dem Erschbild}

begin
  {-- Sprite-Look-Struktur erzeugen und mit Daten fllen --------------}

  new( splookp );
  bptr := @buf;                        { Zeiger auf Logo-Puffer setzen }
  breite := bptr^[0];  { Lnge der Strings und damit Logo-Breite holen }
  abstand := ( ( breite + 3 + 3 ) div 4 ) * 4;
  splookp^.breite := abstand;
  splookp^.msklen := (abstand*hoehe+7) div 8;
  getmem( splookp^.bmskp, splookp^.msklen * 4 );
  splookp^.hoehe  := hoehe;
  splookp^.zeile  := y;
  splookp^.seite  := seite;

  {-- Sprite-Hintergrund in seiner Home-Seite mit Codes fr den -------}
  {-- transparenten Zeichenhintergrund fllen                   -------}

  setpage( seite );                     { Seite zum Zeichnen festlegen }
  lx := 4 * abstand - 1;
  for ly:=y+hoehe-1 downto y do
    Line( 0, ly, lx, ly, 255 );

  {-- Sprite in seiner Home-Seite viermal nebeneinander aufbauen ------}

  lx := 0;                         { am linken Rand der Zeile beginnen }
  for l := 1 to 4 do                         { Sprite viermal aufbauen }
    begin
      for i := 0 to hoehe-1 do                    { Zeilen durchlaufen }
        for k := 0 to breite-1 do                { Spalten durchlaufen }
          begin
            c := bptr^[i*(breite+1)+k+1];                { Farbe holen }
            if ( c = 32 ) then                    { Hintergrund-Punkt? }
              setpix( lx+k, y+i, 255 )       { Ja, Farbcode 255 setzen }
            else                 { Nein, Farbcode wie angegeben setzen }
              setpix( lx+k, y+i, vgfarbe+(c-ord(fb)) );
          end;
      inc( lx, abstand+1 );              { Aufbauspalte inkrementieren }
    end;

  {-- die vier erzeugten Sprites durchlaufen und dabei Bit-Maske fr --}
  {-- das Kopieren der Sprites in die Bit-Planes erzeugen            --}

  pixm := 0;
  pixc := 0;
  lx := 0;
  for l := 0 to 3 do
    begin
      lspb := splookp^.bmskp;
      inc( PTRREC( lspb ).ofs, splookp^.msklen * l );

      for i := 0 to hoehe-1 do
        for k := 0 to abstand-1 do
          begin
            pixm := pixm shr 1;         { Pixelmaske 1 Bit nach rechts }
            if ( getpix( lx+k, y+i ) <> 255 ) then  {Hintergrund-Punkt?}
              pixm := pixm or 128;     { Nein, Bit fr Mit-Mask setzen }
            inc( pixc, 1 );
            if ( pixc = 8 ) then       { schon acht Punkte bearbeitet? }
              begin            { Ja Bit-Maske im Sprite-Puffer ablegen }
                lspb^ := pixm;
                inc( PTRREC( lspb ).ofs, 1 );
                pixc := 0;       { Pixelzhler und -maske wieder auf 0 }
                pixm := 0;
              end;
          end;

      if ( pixc <> 0 ) then      { letztes Nibbl noch nicht im Puffer? }
        begin                                       { Nein, jetzt aber }
          lspb^ := pixm shr 4;  { Hi-Nibble in das Lo-Nibble und absp. }
          pixc := 0;             { Pixelzhler und -maske wieder auf 0 }
          pixm := 0;
        end;

      inc( lx, abstand );          { LX auf Start des nchsten Sprites }
    end;

  CompileSprite := splookp;   { Zeiger auf Sprite-Puffer zurckliefern }
end;

{***********************************************************************
*  PrintSprite : Sprite in einer bestimmten Seite ausgeben             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in der das Sprite gezeichnet werden soll   *
*                    (0 oder 1)                                        *
***********************************************************************}

procedure PrintSprite( spidp : SPIP; seite : byte );

var breite  : byte;                        { Breite des Sprites gesamt }
    x       : integer;      { X-Koordinate des Sprites in seiner Seite }
    splookp : SPLP;      { Zeiger auf das Erscheinungsbild des Sprites }

begin
  splookp := spidp^.splookp;
  breite  := splookp^.breite;
  x       := spidp^.x[seite];
  blockmove( splookp^.seite, breite * (x mod 4), splookp^.zeile, seite,
             x and not(3), spidp^.y[seite], breite, splookp^.hoehe,
             @BARPTR(splookp^.bmskp)^[(x mod 4) * splookp^.msklen] );
end;

{***********************************************************************
*  GetSpriteBg: Hintergrund eines Sprites holen und an der dafr vor-  *
*               gesehenen Position ablegen                             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, aus der der Hintergrund geholt werden soll *
*                    (0 oder 1)                                        *
***********************************************************************}

procedure GetSpriteBg( spidp : SPIP; seite : BYTE );

var splookp : SPLP;      { Zeiger auf das Erscheinungsbild des Sprites }

begin
 splookp := spidp^.splookp;
 blockmove( seite, spidp^.x[seite], spidp^.y[seite],
            spidp^.hgseite, spidp^.hgx + ( splookp^.breite * seite ),
            spidp^.hgy, splookp^.breite, splookp^.hoehe, NIL );
end;

{***********************************************************************
*  RestoreSpriteBg: Zuvor gesicherten Hintergrund eines Sprites wieder *
*                   in die ursprngliche Grafikseite kopieren          *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in die der Hinntergrund zurckkopiert      *
*                    werden soll (0 oder 1)                            *
***********************************************************************}

procedure RestoreSpriteBg( spidp : SPIP; seite : BYTE );

var splookp : SPLP;      { Zeiger auf das Erscheinungsbild des Sprites }

begin
 splookp := spidp^.splookp;
 blockmove( spidp^.hgseite, spidp^.hgx + ( splookp^.breite * seite ),
            spidp^.hgy, seite, spidp^.x[seite], spidp^.y[seite],
            splookp^.breite, splookp^.hoehe, NIL );
end;

{***********************************************************************
*  MoveSprite: Sprite innerhlicherten Hintergrund eines Sprites wieder *
*                   in die ursprngliche Grafikseite kopieren          *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP  = Zeiger auf die Sprite-Struktur                   *
*            SEITE  = Seite, in die der Hintergrund zurckkopiert      *
*                     werden soll (0 oder 1)                           *
*            DELTAX = Bewegungszhler in X- und Y-Richtung             *
*            DELTAY                                                    *
*  Ausgabe : Kollisionsmarkierung, siehe Konstanten OUT_...            *
***********************************************************************}

function MoveSprite( spidp : SPIP; seite : byte;
                     deltax, deltay : integer    ) : byte;

var neux, neuy : integer;                     { neue Sprite-Koordinate }
    out        : byte;                   { Kollision mit Rand anzeigen }

begin
 {-- X-Koordinate verschieben und Kollision mit Rand testen -----------}

 neux := spidp^.x[seite] + deltax;
 if ( neux  < 0 ) then
   begin
     neux := 0 - deltax - spidp^.x[seite];
     out := OUT_LEFT;
   end
 else
   if ( neux > MAXX - spidp^.splookp^.breite ) then
     begin
       neux := (2*(MAXX+1))-neux-2*(spidp^.splookp^.breite);
       out := OUT_RIGHT;
     end
  else
     out := OUT_NO;

 {-- Y-Koordinate verschieben und Kollision mit Rand testen -----------}

 neuy := spidp^.y[seite] + deltay;                      { oberer Rand? }
 if ( neuy < 0 ) then                                   { oberer Rand? }
   begin                                 { Ja, deltay mu negativ sein }
     neuy := 0 - deltay - spidp^.y[seite];
     out := out or OUT_TOP;
   end
 else
   if ( neuy + spidp^.splookp^.hoehe > MAXY+1  ) then  { unterer Rand? }
     begin                               { Ja, deltay mu positiv sein }
       neuy := (2*(MAXY+1))-neuy-2*(spidp^.splookp^.hoehe);
       out := out or OUT_BOTTOM;
     end;

 {-- neue Position nur setzen, wenn gegenber der alten verndert -----}

 if ( neux <> spidp^.x[seite] ) or  ( neuy <> spidp^.y[seite] ) then
   begin                                           { ist neue Position }
     RestoreSpriteBg( spidp, seite );       { Hintergrund zurcksetzen }
     spidp^.x[seite] := neux;                { neue Koordinaten merken }
     spidp^.y[seite] := neuy;
     GetSpriteBg( spidp, seite );            { neuen Hintergrund holen }
     PrintSprite( spidp, seite );  { Sprite in angegeb. Seite zeichnen }
   end;

 MoveSprite := out;
end;

{***********************************************************************
*  SetSprite: Sprite an eine bestimmte Position versetzen              *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP  = Zeiger auf die Sprite-Struktur                   *
*            x0, y0 = Sprite-Koordinate fr die Seite 0                *
*            x1, y1 = Sprite-Koordinate fr die Seite 1                *
*  Info    : Der Aufruf dieser Funktion sollte dem ersten Aufruf von   *
*            MoveSprite() vorausgehen                                  *
***********************************************************************}

procedure SetSprite( spidp : SPIP; x0, y0, x1, y1 : integer );

begin
 spidp^.x[0] := x0;            { Koordinaten in Sprite-Struktur merken }
 spidp^.x[1] := x1;
 spidp^.y[0] := y0;
 spidp^.y[1] := y1;

 GetSpriteBg( spidp, 0 );                { Sprite-Hintergrund in Seite }
 GetSpriteBg( spidp, 1 );                { 0 und 1 holen               }
 PrintSprite( spidp, 0 );                          { Sprite in Seite 0 }
 PrintSprite( spidp, 1 );                          { und 1 zeichnen    }
end;

{***********************************************************************
*  RemoveSprite: Entfernt ein Sprite von seiner augenblicklichen       *
*                Position und macht es dadurch unsichtbar              *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*  Info    : Nach dem Aufruf dieser Funktion mu erst wieder die Funk- *
*            tion SetSprite() auferufen werden, bevor das Sprite ber  *
*            MoveSprite() bewegt werden kann                           *
**********************************************************************}

procedure RemoveSprite( spidp : SPIP );

begin
 RestoreSpriteBg( spidp, 0 );            { Sprite-Hintergrund in Seite }
 RestoreSpriteBg( spidp, 1 );            { 0 und 1 wieder zurcksetzen }
end;

{***********************************************************************
*  Demo: Demonstriert den Einsatz der verschiedenen Funktionen aus     *
*        diesem Modul                                                  *
***********************************************************************}

procedure Demo;

const GleiterRauf :array [1..20] of string[32] =
                ( '               AA               ',
                  '              AAAA              ',
                  '              AAAA              ',
                  '               AA               ',
                  '             GGBBGG             ',
                  '            GBBCCBBG            ',
                  '           GBBBCCBBBG           ',
                  '          GBBBBBBBBBBG          ',
                  '          GBBBBBBBBBBG          ',
                  ' G       GBBBBBBBBBBBBG       G ',
                  'GCG     GGDBBBBBBBBBBDGG     GCG',
                  'GCG   GGBBBDBBB  BBBDBBBGG   GCG',
                  'GCBGGGBBBBBDBB    BBDBBBBBGGGBCG',
                  'GCBBBBBBBBBBDB    BDBBBBBBBBBBCG',
                  'BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB',
                  'GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ',
                  '  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ',
                  '    GGBBDDDDDGGGGGDDDDDDBBG     ',
                  '      GDDDDGGG    GGGDDDDG      ',
                  '       DDDD          DDDD       '  );

const GleiterRunter :array [1..20] of string[32] =
                ( '       DDDD          DDDD       ',
                  '      GDDDDGGG    GGGDDDDG      ',
                  '    GGBBDDDDDGGGGGDDDDDDBBG     ',
                  '  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ',
                  'GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ',
                  'BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB',
                  'GCBBBBBBBBBBDB    BDBBBBBBBBBBCG',
                  'GCBGGGBBBBBDBB    BBDBBBBBGGGBCG',
                  'GCG   GGBBBDBBB  BBBDBBBGG   GCG',
                  'GCG     GGDBBBBBBBBBBDGG     GCG',
                  ' G       GBBBBBBBBBBBBG       G ',
                  '          GBBBBBBBBBBG          ',
                  '          GBBBBBBBBBBG          ',
                  '           GBBBCCBBBG           ',
                  '            GBBCCBBG            ',
                  '             GGBBGG             ',
                  '               AA               ',
                  '              AAAA              ',
                  '              AAAA              ',
                  '               AA               ' );

  SPRANZ = 6;                                     { Anzahl der Sprites }
  BREITE = 37;               { Breite der Copyright-Meldung in Zeichen }
  HOEHE  = 6;                             { Hhe der Meldung in Zeilen }
  SX     = (MAXX-(BREITE*8)) div 2;                 { Start-Ordinate X }
  SY     = (MAXY-(HOEHE*8)) div 2;                  { Start-Ordinate Y }

type SPRITE = record                      { zur Verwaltung der Sprites }
                spidp : SPIP;                   { Zeiger auf Sprite-Id }
                deltax,              { X-Verschiebung fr Page 0 und 1 }
                deltay : array [0..1] of integer;     { Y-Verschiebung }
              end;

var sprites    : array [1..spranz] of SPRITE;
    page,                                  { aktuell bearbeitete Seite }
    lc,                             { Zeichen fr den Bildschirmaufbau }
    out        : byte;           { nimmt Flags fr Seitenkollision auf }
    x, y, i,                                         { Schleifenzhler }
    dx, dy     : integer;                             { Bewegungswerte }
    gleiterupp,
    gleiterdnp : SPLP;                        { Zeiger auf die Sprites }
    ch         : char;
begin

  Randomize;                         { Zufallsgenerator initialisieren }

  {-- die ersten beiden Grafikseiten mit Buchstaben fllen ------------}

  for page := 0 to 1 do
    begin
      setpage( page );
      lc := 0;
      y := 0;
      while ( y < 200-8 ) do
        begin
          x := 0;
          while ( x < 320-8 ) do
            begin
              PrintChar( chr(lc and 127), x, y, lc mod 255, 0 );
              inc( lc );
              inc( x, 8 );
            end;
          inc( y, 12 );
        end;

      {-- Copyright-Meldung ausgeben ----------------------------------}

      Line( SX-1, SY-1, SX+BREITE*8, SY-1, 15 );
      Line( SX+BREITE*8, SY-1, SX+BREITE*8, SY+HOEHE*8,15 );
      Line( SX+BREITE*8, SY+HOEHE*8, SX-1, SY+HOEHE*8, 15 );
      Line( SX-1, SY+HOEHE*8, SX-1, SY-1, 15 );
      PrintString( SX, SY,    15, 4,
                   '                                     '  );
      PrintString( SX, SY+8,  15, 4,
                   ' ** S3220P.PAS - (c) 1990 by MITI ** ' );
      PrintString( SX, SY+16, 15, 4,
                   '                                     '  );
      PrintString( SX, SY+24, 15, 4,
                   ' Sprite-Demo fuer den 320*200 Punkte '  );
      PrintString( SX, SY+32, 15, 4,
                   ' Grafikmodus von VGA-Karten          '  );
      PrintString( SX, SY+40, 15, 4,
                   '                                     '  );
    end;

  {-- Vorlagen fr die verschiedenen Sprites aufbauen -----------------}

  gleiterupp := CompileSprite( GleiterRauf,   20, 2, 0, 'A', 1 );
  gleiterdnp := CompileSprite( GleiterRunter, 20, 2, 40, 'A', 1 );

 {-- die verschiedenen Sprites erzeugen -------------------------------}

  for i := 1 to SPRANZ do
    begin
      sprites[ i ].spidp := CreateSprite( gleiterupp, 3, (i mod 3)*100,
                                         (i div 3) * 30 );
    repeat                         { Bewegungswerte der Sprites whlen }
      dx := 0;
      dy := random(8) - 4;
    until ( dx <> 0 ) or ( dy <> 0 );

    sprites[ i ].deltax[0] := dx * 2;
    sprites[ i ].deltay[0] := dy * 2;
    sprites[ i ].deltax[1] := dx * 2;
    sprites[ i ].deltay[1] := dy * 2;

    x := ( (320 div SPRANZ) * (i-1) ) + ((320 div SPRANZ) - 40) div 2 ;
    y := random( 200 - 40 );
    SetSprite( sprites[ i ].spidp, x, y, x - dx, y - dy );
  end;

  {-- die Sprites bewegen und an den Seitenrndern reflektieren -------}

  page := 1;                                      { mit Page 1 starten }
  while ( not keypressed ) do  { ein Tastenderuck beendet die Schleife }
    begin
      showpage( 1 - page );                { die andere Seite anzeigen }

      for i := 1 to spranz do                { die Sprites durchlaufen }
        begin          { Sprite bewegen und Kontakt mit Seite abfragen }
          out := MoveSprite( sprites[i].spidp, page,
                             sprites[i].deltax[page],
                             sprites[i].deltay[page] );
          if ( ( out and OUT_TOP ) <> 0 ) or       {Kontakt oben/unten?}
             ( ( out and OUT_BOTTOM ) <> 0 ) then
            begin
              {-- Ja, Bewegungsrichtung umdrehen und neues Erschei- ---}
              {-- nungsbild setzen                                  ---}
              sprites[i].deltay[page] := -sprites[i].deltay[page];
              if ( ( out and OUT_TOP ) <> 0 ) then
                sprites[i].spidp^.splookp := gleiterdnp
              else
                sprites[i].spidp^.splookp := gleiterupp;
            end;
          if ( ( out and OUT_LEFT ) <> 0 ) or    {Kontakt links/rechts?}
             ( ( out and OUT_RIGHT ) <> 0 ) then
            sprites[i].deltax[page] := -sprites[i].deltax[page];
        end;
      page := (page+1) and 1;   { von 1 auf 0 und umgekehrt umschalten }
    end;
  ch := readkey;              { Taste aus dem Tastaturpuffer entfernen }
end;

{----------------------------------------------------------------------}
{-- H A U P T P R O G R A M M                                      ----}
{----------------------------------------------------------------------}

begin
 if ( IsVga ) then                            { VGA-Karte installiert? }
   begin                                               { Ja, los gehts }
     init320200;                          { Grafikmodus initialisieren }
     Demo;
     Textmode( CO80 );              { wieder in den Textmodus schalten }
   end
 else
   writeln( 'S3220P.PAS  -  (c) 1990 by MICHAEL TISCHER'#13#10#10 +
            'Achtung! Fr die Ausfhrung dieses Programms wird eine ' +
            'VGA-Karte bentigt.'#13#10 );

end.

