{***********************************************************************
*                           S 3 2 4 0 P . P A S                        *
**--------------------------------------------------------------------**
*  Aufgabe          : Demonstriert die Arbeit mit Sprites im VGA-      *
*                     320*400-Grafikmodus bei 256 Farben und zwei      *
*                     Bildschirmseiten. Das Programm greift dabei      *
*                     auf die Assembler-Routinen aus den Moduln        *
*                     V3240PA.ASM und S3240PA.ASM zurck.              *
**--------------------------------------------------------------------**
*  Autor            : MICHAEL TISCHER                                  *
*  entwickelt am    : 12.09.1990                                       *
*  letztes Update am: 14.01.1991                                       *
***********************************************************************}

program S3240P;

uses dos, crt;

{-- externe Referenzen auf die Assembler-Routinen ---------------------}

{$L v3240pa}                               { Assembler-Modul einbinden }

procedure init320400; external;
procedure setpix( x, y : integer; farbe : byte ); external;
function  getpix( x, y: integer ) : byte ; external;
procedure setpage( page : byte ); external;
procedure showpage( page : byte ); external;

{$L s3240pa}                               { Assembler-Modul einbinden }

procedure CopyPlane2Buf( bufptr : pointer;
                         page   : byte;
                         vonx,
                         vony   : integer;
                         breite,
                         hoehe  : byte    ); external;

procedure CopyBuf2Plane( bufptr : pointer;
                         page   : byte;
                         nachx,
                         nachy  : integer;
                         breite,
                         hoehe  : byte;
                         bg     : boolean ); external;

{-- Konstanten --------------------------------------------------------}

const MAXX = 319;                         { Maximale X- und Y-Ordinate }
      MAXY = 399;

      OUT_LEFT   = 1;       { fr Kollisionsmarkierung in SpriteMove() }
      OUT_TOP    = 2;
      OUT_RIGHT  = 4;
      OUT_BOTTOM = 8;
      OUT_NO     = 0;                              { nirgendwo drauen }

{-- Typdeklarationen --------------------------------------------------}

type PIXBUF = record         { Informationen fr GetVideo und PutVideo }
                bitptr : array[0..3] of pointer; {Zeiger auf Bit-Planes}
                byproz : array[0..3] of byte; { Anz. zu kopierender B. }
                hoehe  : byte;                     { Anzahl der Zeilen }
                {-- hier folgen die Bytes aus den Bit-Planes ----------}
              end;
     PIXPTR = ^PIXBUF;                 { Zeiger auf einen Pixel-Puffer }

     SPLOOK = record                  { Erscheinungsbild eines Sprites }
                breite,                                { Breite gesamt }
                hoehe  : byte;                   { Hhe in Punktzeilen }
                pixbp  : PIXPTR;              { Zeiger auf Pixel-Block }
              end;                  { seiner Seite abgelegt wurde      }
     SPLP = ^SPLOOK;                 { Zeiger auf das Erscheinungsbild }

     SPID = record                           { Sprite-Beschreiber (ID) }
              splookp : SPLP;            { Zeiger auf Erscheinungsbild }
              x, y    : array [0..1] of integer;  { Koordinaten S. 0&1 }
              hgptr   : array [0..1] of PIXPTR;   { Zeiger auf Hinter- }
            end;                                  { grund Puffer       }
     SPIP = ^SPID;                     { Zeiger auf Sprite-Beschreiber }

     PTRREC = record       { zur Zerlegung eines Zeigers oder LONINGTS }
                ofs,
                seg : word;
              end;

     BYTEAR = array[0..10000] of byte;     { zur Adressierung der ver- }
     BARPTR = ^BYTEAR;                     { schiedenen Puffer         }

{***********************************************************************
*  IsVga : Stellt fest, ob eine VGA-Karte installiert ist.             *
**--------------------------------------------------------------------**
*  Eingabe : keine                                                     *
*  Ausgabe : TRUE oder FALSE                                           *
***********************************************************************}

function IsVga : boolean;

var Regs : Registers;          { Prozessorregister fr Interruptaufruf }

begin
  Regs.AX := $1a00;                 { Funktion 1Ah gibt es nur bei VGA }
  Intr( $10, Regs );
  IsVga := ( Regs.AL = $1a );
end;

{***********************************************************************
*  PrintChar : Schreibt ein Zeichen im Grafikmodus in den Bildschirm   *
**--------------------------------------------------------------------**
*  Eingabe :   zeichen = das aufzubauende Zeichen                      *
*              x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************}

procedure PrintChar( zeichen : char; x, y : integer; vg, hg : byte );

type ZDEF = array[0..255,0..7] of byte;       { Aufbau des Zeichensatz }
     ZPTR = ^ZDEF;                            { Zeiger auf Zeichensatz }

var  Regs  : Registers;                 { Register zum Interruptaufruf }
     ch    : char;                      { einzelner Punkt des Zeichens }
     i, k,                                           { Schleifenzhler }
     Maske : byte;                  { Bitmaske fr Aufbau des Zeichens }

const fptr : ZPTR = NIL;               { Zeiger auf Zeichensatz im ROM }

begin
  if fptr = NIL then         { Zeiger auf Zeichensatz schon ermittelt? }
    begin                                                       { Nein }
      Regs.AH := $11;                   { Funktion $11 des Video-BIOS, }
      Regs.AL := $30;                   { Unterfunktion $30 aufrufen   }
      Regs.BH := 3;             { Zeiger auf 8*8 Zeichensatz ermitteln }
      intr( $10, Regs );
      fptr := ptr( Regs.ES, Regs.BP );         { Zeiger zusammensetzen }
    end;

  if ( hg = 255 ) then                 { Zeichen transparent zeichnen? }
    for i := 0 to 7 do             { Ja, nur Vordergrund-Punkte setzen }
      begin
        Maske := fptr^[ord(zeichen),i]; {Bitmuster fr eine Zeile holen}
        for k := 0 to 7 do
          begin
            if ( Maske and 128 <> 0 ) then             { Punkt setzen? }
              setpix( x+k, y+i, vg );                             { Ja }
            Maske := Maske shl 1;
          end;
      end
  else                        { Nein, auch Hintergrund bercksichtigen }
    for i := 0 to 7 do                        { die Zeilen durchlaufen }
      begin
        Maske := fptr^[ord(zeichen),i]; {Bitmuster fr eine Zeile holen}
        for k := 0 to 7 do
          begin
            if ( Maske and 128 <> 0 ) then              { Vordergrund? }
              setpix( x+k, y+i, vg )                              { Ja }
            else
              setpix( x+k, y+i, hg );              { Nein, Hintergrund }
            Maske := Maske shl 1;
         end;
      end;
end;

{***********************************************************************
*  PrintString: Schreibt einen String im Grafikmodus in den Bildschirm *
**--------------------------------------------------------------------**
*  Eingabe :   x, y    = X- und Y-Koordinate obere linke Ecke          *
*              vg      = Vordergrundfarbe                              *
*              hg      = Hintergrundfarbe                              *
*              tstr    = der auszugebende String                       *
*  Info    : Das Zeichen wird in einer Matrix aus 8*8 Zeichen aufge-   *
*            baut, wobei der 8*8-ROM-Zeichensatz zu Grunde gelegt wird *
***********************************************************************}

procedure PrintString( x, y : integer; vg, hg : byte; tstr : string );

var i : integer;                                     { Schleifenzhler }

begin
  for i := 1 to length( tstr ) do             { den String durchlaufen }
    begin
      PrintChar( tstr[i], x, y, vg, hg );               { und ausgeben }
      inc( x, 8 );                    { Ausgabeposition inkrementieren }
    end;
end;

{***********************************************************************
*  Line: Zieht eine Linie innerhalb nach dem Bresenham-Algorithmus     *
**--------------------------------------------------------------------**
*  Eingabe : X1, Y1 = Startkoordinate (0 - ...)                        *
*            X2, Y2 = Endekoordinate                                   *
*            FARBE  = Farbe der Linienpunkte                           *
***********************************************************************}

procedure Line( x1, y1, x2, y2 : integer; farbe : byte );

var d, dx, dy,
    aincr, bincr,
    xincr, yincr,
    x, y                 : integer;

{-- Unterprozedur, vertauscht zwei Integer-Variablen ------------------}

procedure SwapInt( var i1, i2: integer );

var dummy : integer;

begin
  dummy := i2;
  i2    := i1;
  i1    := dummy;
end;

{-- Hauptprozedur -----------------------------------------------------}

begin
  if ( abs(x2-x1) < abs(y2-y1) ) then   { ber X- oder Y-Achse laufen? }
    begin                                           { ber die Y-Achse }
      if ( y1 > y2 ) then                              { y1 grer y2? }
        begin
          SwapInt( x1, x2 );                   { Ja, X1 mit X2 und     }
          SwapInt( y1, y2 );                   { Y1 mit Y2 vertauschen }
        end;

      if ( x2 > x1 ) then xincr := 1      { Spalteninkrement festlegen }
                     else xincr := -1;

      dy := y2 - y1;
      dx := abs( x2-x1 );
      d  := 2 * dx - dy;
      aincr := 2 * (dx - dy);
      bincr := 2 * dx;
      x := x1;
      y := y1;

      setpix( x, y, farbe );                     { ersten Punkt setzen }
      for y:=y1+1 to y2 do              { Line auf X-Achse durchlaufen }
        begin
          if ( d >= 0 ) then
            begin
              inc( x, xincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          setpix( x, y, farbe );
        end;
    end
  else                                              { ber die X-Achse }
    begin
      if ( x1 > x2 ) then                              { x1 grer x2? }
        begin
          SwapInt( x1, x2 );                   { Ja, X1 mit X2 und     }
          SwapInt( y1, y2 );                   { Y1 mit Y2 vertauschen }
        end;

      if ( y2 > y1 ) then yincr := 1       { Zeileninkrement festlegen }
                     else yincr := -1;

      dx := x2 - x1;
      dy := abs( y2-y1 );
      d  := 2 * dy - dx;
      aincr := 2 * (dy - dx);
      bincr := 2 * dy;
      x := x1;
      y := y1;

      setpix( x, y, farbe );                     { ersten Punkt setzen }
      for x:=x1+1 to x2 do              { Line auf X-Achse durchlaufen }
        begin
          if ( d >= 0 ) then
            begin
              inc( y, yincr );
              inc( d, aincr );
            end
          else
            inc( d, bincr );
          setpix( x, y, farbe );
       end;
    end;
end;

{***********************************************************************
*  GrafPrint: Gibt einen formatierten String im Grafikbildschirm aus   *
**--------------------------------------------------------------------**
*  Eingabe : X, Y   = Startkoordinate (0 - ...)                        *
*            VG     = Vordergrundfarbe                                 *
*            HG     = Hintergrundfarbe (255 = transparent)             *
*            STRING = String mit Formatinformationen                   *
***********************************************************************}

procedure GrafPrint( x, y : integer; vg, hg : byte; strt : string );

var i : integer;                                     { Schleifenzhler }

begin
 for i:=1 to length( strt ) do
   begin
     printchar( strt[i], x, y, vg, hg );     { ber PrintChar ausgeben }
     inc( x, 8 );                  { X auf die nchste Zeichenposition }
   end;
end;

{***********************************************************************
*  GetVideo: Holt den Inhalt eines rechteckigen Bereichs aus dem       *
*            Video-RAM in einen Puffer                                 *
**--------------------------------------------------------------------**
*  Eingabe : PAGE   = Bildschirmseite (0 oder 1)                       *
*            X1, Y1 = Startkoordinate                                  *
*            BREITE = Breite des rechteckigen Bereichs in Punkten      *
*            HOEHE  = Hhe des rechteckigen Bereichs in Punkten        *
*            BUFPTR = Zeiger auf Pixel-Puffer, in dem die Informa-     *
*                     tionen abgelegt werden sollen                    *
*  Ausgabe : Zeiger auf den angelegten Pixel-Puffer mit dem Inhalt     *
*            des angegebenen Bereichs                                  *
*  Info    : Wird fr den Parameter BUFPTR der Wert NIL bergeben,     *
*            wird ein neuer Pixel-Puffer ber den Heap allokiert und   *
*            zurckgeliefert. Dieser Puffer kann bei einem neuer-      *
*            lichen Aufruf wieder angegeben werden, sofern der bis-    *
*            herige Inhalt nicht mehr bentigt wird und die Gre des  *
*            rechteckigen Bereichs gegenber dem vorhergehenden Aufruf *
*            unverndert bleibt.                                       *
***********************************************************************}

function GetVideo( page : byte; x1, y1 : integer;
                   breite, hoehe : byte; bufptr : PIXPTR ) : PIXPTR;

var i,                                              { Schleifenzhnler }
    aktplane,                          { aktuell bearbeitete Bit-Plane }
    sb,                                { Bit-Plane der Startkoordinate }
    eb,                                  { Bit-Plane der Endkoordinate }
    b,                                { Byte-Anzahl in einer Bit-Plane }
    am  : byte;  { Anzahl Bytes in der Mitte der beiden Vierer-Gruppen }
    rptr : pointer;                        { Laufzhler fr den Offset }

begin
  if ( bufptr = NIL ) then        { kein Puffer beim Aufruf bergeben? }
    getmem( bufptr, sizeof( PIXBUF ) + breite*hoehe );  { Nein, allok. }

  {-- die Anzahl der Bytes pro Bit-Plane berechnen --------------------}

  am := ( ( (x1+breite-1) and not(3) ) - {Anzahl der Bytes in der Mitte}
        ( (x1+4) and not(3) ) ) div 4;
  sb := x1 mod 4;                                    { Start-Bit-Plane }
  eb := (x1+breite-1) mod 4;                           { End-Bit-Plane }

  rptr := ptr( seg(bufptr^), ofs(bufptr^) + sizeof( PIXBUF ));

  {-- die vier Bit-Planes durchlaufen ---------------------------------}

  for i:=0 to 3 do 
    begin
      aktplane := (sb+i) mod 4;
      b := am;                  { Grundanzahl der zu kopierenden Bytes }
      if ( aktplane >= sb ) then         { auch im Start-Vierer-Block? }
        inc( b );              { Ja, in dieser Bit-Plane ein Byte mehr }
      if ( aktplane <= eb ) then           { auch im End-Vierer-Block? }
        inc( b );              { Ja, in dieser Bit-Plane ein Byte mehr }
      bufptr^.bitptr[i] := rptr;  { Zeiger auf Start im Puffer ablegen }
      bufptr^.byproz[i] := b;               { Anzahl im Puffer ablegen }
      CopyPlane2Buf( rptr, page, x1+i,               { Inhalt der Bit- }
                     y1, b, hoehe );                 { Plane holen     }
      inc( PTRREC(rptr).ofs, b * hoehe );    { Zeiger auf nchste Bit- }
    end;                                     { Plane im Puffer setzen  }

  bufptr^.hoehe := hoehe;                                { Hhe merken }

  GetVideo := bufptr;    { Zeiger auf Puffer an Aufrufer zurckliefern }
end;

{***********************************************************************
*  PutVideo: Schreibt den Inhalt eines zuvor ber GetVideo gesicherten *
*            rechteckigen Bildschirmbereichs wieder in den Video-RAM   *
**--------------------------------------------------------------------**
*  Eingabe : BUFPTR = Beim vorherigen Aufruf von Getvideo zurck-      *
*                     gelieferter Zeiger auf den Pixel-Puffer          *
*            PAGE   = Bildschirmseite (0 oder 1)                       *
*            X1, Y1 = Startkoordinate                                  *
*            BG     = Sollen Hintergrundpunkte (Farbcode 255) nicht    *
*                     in den Video-RAM geschrieben werden?             *
*  Info    : Der Pixel-Puffer wird durch diese Prozedur nicht gelscht,*
*             dafr mu die Prozedur FreiPixBuf aufgerufen werden.     *
***********************************************************************}

procedure PutVideo( bufptr : PIXPTR; page : byte; x1, y1 : integer;
                    bg     : boolean );

var aktplane,                         { aktuell durchlaufene Bit-Plane }
    hoehe : byte;

begin
  hoehe := bufptr^.hoehe;                          { Hhe des Bereichs }
  for aktplane:=0 to 3 do            { die vier Bit-Planes durchlaufen }
    CopyBuf2Plane( bufptr^.bitptr[aktplane], page, x1+aktplane,
                   y1, bufptr^.byproz[aktplane], hoehe, bg );
end;

{***********************************************************************
*  FreePixBuf: Lscht einen Pixel-Puffer, der bei einem vorherigen     *
*              Aufruf von GetVideo ber den Heap allokiert wurde       *
**--------------------------------------------------------------------**
*  Eingabe : BUFPTR = Beim vorherigen Aufruf von Getvideo zurck-      *
*                     gelieferter Zeiger auf den Pixel-Puffer          *
*            BREITE = Breite des rechteckigen Bereichs in Punkten      *
*            HOEHE  = Hhe des rechteckigen Bereichs in Punkten        *
***********************************************************************}

procedure FreePixBuf( bufptr : PIXPTR; breite, hoehe : byte );

begin
  freemem( bufptr, sizeof( PIXBUF ) + breite*hoehe );
end;

{***********************************************************************
*  CreateSprite: Erzeugt ein Sprite auf der Basis einer bereits kompi- *
*                lierten Punktmusters                                  *
**--------------------------------------------------------------------**
*  Eingabe : SPLOOKP = Zeiger auf die von CompileSprite erzeugte       *
*                      Datenstruktur                                   *
*  Ausgabe : Zeiger auf die erstellte Sprite-Struktur                  *
***********************************************************************}

function CreateSprite( splookp : SPLP ) : SPIP;

var spidp : SPIP;                { Zeiger auf erzeugte Sprite-Struktur }

begin
  new( spidp );           { Speicher fr Sprite-Beschreiber allokieren }
  spidp^.splookp := splookp;                    { Daten in die Sprite- }

  {-- zwei Hintergrund-Puffer erzeugen, indem ein entsprechend groer -}
  {-- Bereich aus dem Video-RAM ber GetVideo gesichert wird          -}

  spidp^.hgptr[0] := GetVideo( 0, 0, 0, splookp^.breite,
                               splookp^.hoehe, NIL );
  spidp^.hgptr[1] := GetVideo( 0, 0, 0, splookp^.breite,
                               splookp^.hoehe, NIL );
  CreateSprite := spidp;    { Zeiger auf Sprite-Struktur zurckliefern }
end;

{***********************************************************************
*  CompileSprite: Erzeugt das Punkt- und Bitmusters eines Sprites,     *
*                 auf der Basis einer zur Laufzeit angegebenen Sprite- *
*                 definition                                           *
**--------------------------------------------------------------------**
*  Eingabe : BUFP    = Zeiger auf Array mit Zeigern auf Strings, die   *
*                      das Zeichenmuster des Sprites enthalten         *
*            HOEHE   = Hoehe des Sprites und damit Anzahl der Strings  *
*            SEITE   = Grafik-Seite zum Aufbau des Sprites             *
*            FB      = ASCII-Zeichen, das fr die kleinste Farbe steht *
*            VGFARBE = erste Farbcode fr FB                           *
*  Info    : Die Sprites werden in der angegebenen Punktzeile ab dem   *
*            linken Zeilenrand angelegt                                *
***********************************************************************}

function CompileSprite( var buf; hoehe, seite : byte;
                        fb : char; vgfarbe : byte ) : SPLP;

type BYPTR  = ^byte;                             { Zeiger auf ein Byte }

var  breite,                                      { Breite der Strings }
     c,                           { nimmt Zeichen aus Sprite-Array auf }
     farbe,                           { umgesetzte Farbe eines Punktes }
     i, k, l  : byte;                             { Schleifenvariablen }
     splookp  : SPLP;            { Zeiger auf erzeugte Sprite-Struktur }
     bptr     : barptr; {zur Adressierung des Puffers mit dem Erschbild}
     pbptr    : PIXPTR;                 { nimmt Sprite-Hintergrund auf }

begin
  {-- Sprite-Look-Struktur erzeugen und mit Daten fllen --------------}

  new( splookp );
  bptr := @buf;                        { Zeiger auf Logo-Puffer setzen }
  breite := bptr^[0];  { Lnge der Strings und damit Logo-Breite holen }
  splookp^.breite := breite;
  splookp^.hoehe  := hoehe;

  {-- Sprite in der angegebenen Seite ab (0/0) aufbauen ---------------}

  setpage( seite );                     { Seite zum Zeichnen festlegen }
  pbptr := GetVideo( seite, 0, 0, breite, hoehe, nil );   { Hgr. holen }

  for i := 0 to hoehe-1 do                    { die Zeilen durchlaufen }
    for k := 0 to breite-1 do                { die Spalten durchlaufen }
      begin
        c := bptr^[i*(breite+1)+k+1];                    { Pixel holen }
        if ( c = 32 ) then                              { Hintergrund? }
          setpix( k, i, 255 )                { Ja, Farbcode 255 setzen }
        else                     { Nein, Farbcode wie angegeben setzen }
          setpix( k, i, vgfarbe+(c-ord(fb)) );
      end;

  {-- Sprite in Puffer holen und Hintergrund restaurieren -------------}

  splookp^.pixbp := GetVideo( seite, 0, 0, breite, hoehe, NIL );
  PutVideo( pbptr, seite, 0, 0, false );
  FreePixBuf( pbptr, breite, hoehe );        { Puffer wieder freigeben }

  CompileSprite := splookp;   { Zeiger auf Sprite-Puffer zurckliefern }
end;

{***********************************************************************
*  PrintSprite : Sprite in einer bestimmten Seite ausgeben             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in der das Sprite gezeichnet werden soll   *
*                    (0 oder 1)                                        *
***********************************************************************}

procedure PrintSprite( spidp : SPIP; seite : byte );

begin
  PutVideo( spidp^.splookp^.pixbp,
            seite, spidp^.x[seite], spidp^.y[seite], true );
end;

{***********************************************************************
*  GetSpriteBg: Hintergrund eines Sprites holen und an der dafr vor-  *
*               gesehenen Position ablegen                             *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, aus der der Hintergrund geholt werden soll *
*                    (0 oder 1)                                        *
***********************************************************************}

procedure GetSpriteBg( spidp : SPIP; seite : BYTE );

var dummy : PIXPTR;

begin
 dummy := GetVideo( seite, spidp^.x[seite],  spidp^.y[seite],
                    spidp^.splookp^.breite, spidp^.splookp^.hoehe,
                    spidp^.hgptr[seite] );
end;

{***********************************************************************
*  RestoreSpriteBg: Zuvor gesicherten Hintergrund eines Sprites wieder *
*                   in die ursprngliche Grafikseite kopieren          *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP = Zeiger auf die Sprite-Struktur                    *
*            SEITE = Seite, in die der Hinntergrund zurckkopiert      *
*                    werden soll (0 oder 1)                            *
***********************************************************************}

procedure RestoreSpriteBg( spidp : SPIP; seite : BYTE );

begin
 PutVideo( spidp^.hgptr[seite], seite,
           spidp^.x[seite],  spidp^.y[seite], false );
end;

{***********************************************************************
*  MoveSprite: Sprite in einer Bildschirmseite bewegen                 *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP  = Zeiger auf die Sprite-Struktur                   *
*            SEITE  = Seite, in die der Hintergrund zurckkopiert      *
*                     werden soll (0 oder 1)                           *
*            DELTAX = Bewegungszhler in X- und Y-Richtung             *
*            DELTAY                                                    *
*  Ausgabe : Kollisionsmarkierung, siehe Konstanten OUT_...            *
***********************************************************************}

function MoveSprite( spidp : SPIP; seite : byte;
                     deltax, deltay : integer    ) : byte;

var neux, neuy : integer;                     { neue Sprite-Koordinate }
    out        : byte;                   { Kollision mit Rand anzeigen }

begin
 {-- X-Koordinate verschieben und Kollision mit Rand testen -----------}

 neux := spidp^.x[seite] + deltax;
 if ( neux  < 0 ) then
   begin
     neux := 0 - deltax - spidp^.x[seite];
     out := OUT_LEFT;
   end
 else
   if ( neux > MAXX - spidp^.splookp^.breite ) then
     begin
       neux := (2*(MAXX+1))-neux-2*(spidp^.splookp^.breite);
       out := OUT_RIGHT;
     end
  else
     out := OUT_NO;

 {-- Y-Koordinate verschieben und Kollision mit Rand testen -----------}

 neuy := spidp^.y[seite] + deltay;                      { oberer Rand? }
 if ( neuy < 0 ) then                                   { oberer Rand? }
   begin                                 { Ja, deltay mu negativ sein }
     neuy := 0 - deltay - spidp^.y[seite];
     out := out or OUT_TOP;
   end
 else
   if ( neuy + spidp^.splookp^.hoehe > MAXY+1  ) then  { unterer Rand? }
     begin                               { Ja, deltay mu positiv sein }
       neuy := (2*(MAXY+1))-neuy-2*(spidp^.splookp^.hoehe);
       out := out or OUT_BOTTOM;
     end;

 {-- neue Position nur setzen, wenn gegenber der alten verndert -----}

 if ( neux <> spidp^.x[seite] ) or  ( neuy <> spidp^.y[seite] ) then
   begin                                           { ist neue Position }
     RestoreSpriteBg( spidp, seite );       { Hintergrund zurcksetzen }
     spidp^.x[seite] := neux;                { neue Koordinaten merken }
     spidp^.y[seite] := neuy;
     GetSpriteBg( spidp, seite );            { neuen Hintergrund holen }
     PrintSprite( spidp, seite );  { Sprite in angegeb. Seite zeichnen }
   end;

 MoveSprite := out;
end;

{***********************************************************************
*  SetSprite: Sprite an eine bestimmte Position versetzen              *
**--------------------------------------------------------------------**
*  Eingabe : SPIDP  = Zeiger auf die Sprite-Struktur                   *
*            x0, y0 = Sprite-Koordinate fr die Seite 0                *
*            x1, y1 = Sprite-Koordinate fr die Seite 1                *
*  Info    : Der Aufruf dieser Funktion sollte dem ersten Aufruf von   *
*            MoveSprite() vorausgehen                                  *
***********************************************************************}

procedure SetSprite( spidp : SPIP; x0, y0, x1, y1 : integer );

begin
 spidp^.x[0] := x0;            { Koordinaten in Sprite-Struktur merken }
 spidp^.x[1] := x1;
 spidp^.y[0] := y0;
 spidp^.y[1] := y1;

 GetSpriteBg( spidp, 0 );                { Sprite-Hintergrund in Seite }
 GetSpriteBg( spidp, 1 );                { 0 und 1 holen               }
 PrintSprite( spidp, 0 );                          { Sprite in Seite 0 }
 PrintSprite( spidp, 1 );                          { und 1 zeichnen    }
end;

{***********************************************************************
*  Demo: Demonstriert den Einsatz der verschiedenen Funktionen aus     *
*        diesem Modul                                                  *
***********************************************************************}

procedure Demo;

const GleiterRauf :array [1..20] of string[32] =
                ( '               AA               ',
                  '              AAAA              ',
                  '              AAAA              ',
                  '               AA               ',
                  '             GGBBGG             ',
                  '            GBBCCBBG            ',
                  '           GBBBCCBBBG           ',
                  '          GBBBBBBBBBBG          ',
                  '          GBBBBBBBBBBG          ',
                  ' G       GBBBBBBBBBBBBG       G ',
                  'GCG     GGDBBBBBBBBBBDGG     GCG',
                  'GCG   GGBBBDBBB  BBBDBBBGG   GCG',
                  'GCBGGGBBBBBDBB    BBDBBBBBGGGBCG',
                  'GCBBBBBBBBBBDB    BDBBBBBBBBBBCG',
                  'BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB',
                  'GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ',
                  '  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ',
                  '    GGBBDDDDDGGGGGDDDDDDBBG     ',
                  '      GDDDDGGG    GGGDDDDG      ',
                  '       DDDD          DDDD       '  );

const GleiterRunter :array [1..20] of string[32] =
                ( '       DDDD          DDDD       ',
                  '      GDDDDGGG    GGGDDDDG      ',
                  '    GGBBDDDDDGGGGGDDDDDDBBG     ',
                  '  GGCCBBBDDDDDDDDDDDDDDBBBCCG   ',
                  'GGCBBBBBBBDBBBBBBBBBBDBBBBBBBCG ',
                  'BBBBBBBBBBBBDB BB BDBBBBBBBBBBBB',
                  'GCBBBBBBBBBBDB    BDBBBBBBBBBBCG',
                  'GCBGGGBBBBBDBB    BBDBBBBBGGGBCG',
                  'GCG   GGBBBDBBB  BBBDBBBGG   GCG',
                  'GCG     GGDBBBBBBBBBBDGG     GCG',
                  ' G       GBBBBBBBBBBBBG       G ',
                  '          GBBBBBBBBBBG          ',
                  '          GBBBBBBBBBBG          ',
                  '           GBBBCCBBBG           ',
                  '            GBBCCBBG            ',
                  '             GGBBGG             ',
                  '               AA               ',
                  '              AAAA              ',
                  '              AAAA              ',
                  '               AA               ' );

  SPRANZ = 6;                                     { Anzahl der Sprites }
  BREITE = 37;               { Breite der Copyright-Meldung in Zeichen }
  HOEHE  = 6;                             { Hhe der Meldung in Zeilen }
  SX     = (MAXX-(BREITE*8)) div 2;                 { Start-Ordinate X }
  SY     = (MAXY-(HOEHE*8)) div 2;                  { Start-Ordinate Y }

type SPRITE = record                      { zur Verwaltung der Sprites }
                spidp : SPIP;                   { Zeiger auf Sprite-Id }
                deltax,              { X-Verschiebung fr Page 0 und 1 }
                deltay : array [0..1] of integer;     { Y-Verschiebung }
              end;

var sprites    : array [1..spranz] of SPRITE;
    page,                                  { aktuell bearbeitete Seite }
    lc,                             { Zeichen fr den Bildschirmaufbau }
    out        : byte;           { nimmt Flags fr Seitenkollision auf }
    x, y, i,                                         { Schleifenzhler }
    dx, dy     : integer;                             { Bewegungswerte }
    gleiterupp,
    gleiterdnp : SPLP;                        { Zeiger auf die Sprites }
    ch         : char;
begin

  Randomize;                         { Zufallsgenerator initialisieren }

  {-- Vorlagen fr die verschiedenen Sprites aufbauen -----------------}

  gleiterupp := CompileSprite( GleiterRauf,   20, 0, 'A', 1 );
  gleiterdnp := CompileSprite( GleiterRunter, 20, 0, 'A', 1 );

  {-- die ersten beiden Grafikseiten mit Buchstaben fllen ------------}

  for page := 0 to 1 do
    begin
      setpage( page );
      lc := 0;
      y := 0;
      while ( y < (MAXY+1)-8 ) do
        begin
          x := 0;
          while ( x < (MAXX+1)-8 ) do
            begin
              PrintChar( chr(lc and 127), x, y, lc mod 255, 0 );
              inc( lc );
              inc( x, 8 );
            end;
          inc( y, 12 );
        end;

      {-- Copyright-Meldung ausgeben ----------------------------------}

      Line( SX-1, SY-1, SX+BREITE*8, SY-1, 15 );
      Line( SX+BREITE*8, SY-1, SX+BREITE*8, SY+HOEHE*8,15 );
      Line( SX+BREITE*8, SY+HOEHE*8, SX-1, SY+HOEHE*8, 15 );
      Line( SX-1, SY+HOEHE*8, SX-1, SY-1, 15 );
      PrintString( SX, SY,    15, 4,
                   '                                     '  );
      PrintString( SX, SY+8,  15, 4,
                   ' ** S3240P.PAS - (c) 1990 by MITI ** ' );
      PrintString( SX, SY+16, 15, 4,
                   '                                     '  );
      PrintString( SX, SY+24, 15, 4,
                   ' Sprite-Demo fuer den 320*400 Punkte '  );
      PrintString( SX, SY+32, 15, 4,
                   ' Grafikmodus von VGA-Karten          '  );
      PrintString( SX, SY+40, 15, 4,
                   '                                     '  );
    end;

 {-- die verschiedenen Sprites erzeugen -------------------------------}

  for i := 1 to SPRANZ do
    begin
      sprites[ i ].spidp := CreateSprite( gleiterupp );
      repeat                       { Bewegungswerte der Sprites whlen }
        dx := 0;
        dy := random(10) - 5;
      until ( dx <> 0 ) or ( dy <> 0 );

      sprites[ i ].deltax[0] := dx * 2;
      sprites[ i ].deltay[0] := dy * 2;
      sprites[ i ].deltax[1] := dx * 2;
      sprites[ i ].deltay[1] := dy * 2;

      x := ( (320 div SPRANZ) * (i-1) ) + ((320 div SPRANZ)-40) div 2;
      y := random( 200 - 40 );
      SetSprite( sprites[ i ].spidp, x, y, x - dx, y - dy );
    end;

  {-- die Sprites bewegen und an den Seitenrndern reflektieren -------}

  page := 1;                                      { mit Page 1 starten }
  while ( not keypressed ) do  { ein Tastenderuck beendet die Schleife }
    begin
      showpage( 1 - page );                { die andere Seite anzeigen }

      for i := 1 to spranz do                { die Sprites durchlaufen }
        begin          { Sprite bewegen und Kontakt mit Seite abfragen }
          out := MoveSprite( sprites[i].spidp, page,
                             sprites[i].deltax[page],
                             sprites[i].deltay[page] );
          if ( ( out and OUT_TOP ) <> 0 ) or       {Kontakt oben/unten?}
             ( ( out and OUT_BOTTOM ) <> 0 ) then
            begin
              {-- Ja, Bewegungsrichtung umdrehen und neues Erschei- ---}
              {-- nungsbild                                         ---}

              sprites[i].deltay[page] := -sprites[i].deltay[page];
              if ( ( out and OUT_TOP ) <> 0 ) then
                sprites[i].spidp^.splookp := gleiterdnp
              else
                sprites[i].spidp^.splookp := gleiterupp;
            end;
          if ( ( out and OUT_LEFT ) <> 0 ) or    {Kontakt links/rechts?}
             ( ( out and OUT_RIGHT ) <> 0 ) then
            sprites[i].deltax[page] := -sprites[i].deltax[page];
        end;
      page := (page+1) and 1;   { von 1 auf 0 und umgekehrt umschalten }
    end;

  ch := readkey;                                    { auf Taste warten }
end;

{----------------------------------------------------------------------}
{-- H A U P T P R O G R A M M                                      ----}
{----------------------------------------------------------------------}

begin
 if ( IsVga ) then                            { VGA-Karte installiert? }
   begin                                               { Ja, los gehts }
     init320400;                          { Grafikmodus initialisieren }
     Demo;
     Textmode( CO80 );              { wieder in den Textmodus schalten }
   end
 else
   writeln( 'S3240P.PAS  -  (c) 1990 by MICHAEL TISCHER'#13#10#10 +
            'Achtung! Fr die Ausfhrung dieses Programms wird eine ' +
            'VGA-Karte bentigt.'#13#10 );
end .

