{********************************************************************}
{                         S B U T I L . P A S                        }
{*------------------------------------------------------------------*}
{ Aufgabe        : Stellt Funktionen zur Initialisierung der SBBASE  }
{                  Struktur bereit.                                  }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 20.03.1994                                        }
{ letztes Update :  8.10.1994                                        }
{********************************************************************}
Unit SBUTIL;

Interface

Type
SBBASE = Record { beschreibt istallierte Soundkarte }
 iDspPort,                            { Basisport des DS-Prozessors }
 iMixPort,                                   { Basisport des Mixers }
 iMpuPort,                               { Basisport der MP-Einheit }
 iDspDmaB,                            { verwendeter 8-Bit DMA Kanal }
 iDspDmaW,                           { verwendeter 16-Bit DMA Kanal }
 iDspIrq     : Integer;              { verwendete Interrupt-Leitung }
 uDspVersion : Word;                           { DSP Versionsnummer }
 pDspName    : String;                { Name der Soundblaster-Karte }
End;

{-- Prototypen ----------------------------------------------}

Function sb_GetEnviron( var sbbase : SBBASE; Env : String ) : Integer;
Procedure sb_Print( var sbbase : SBBASE );
Function sb_LoadDriver( Name : String ) : Longint;
Procedure sb_UnloadDriver( lpEntry : Longint );

Implementation

Uses DOS,ARGS;

Const
  ERROR    = -1;
  NO_ERROR = 0;

{********************************************************************}
{ Hex : wandelt eine Hex-Zahl in einen Hex-String                    }
{*------------------------------------------------------------------*}
{ Eingabe : hz - der umzuwandelnde Hex-Wert                          }
{ Ausgabe : der umgewandelte Hex-String                              }
{********************************************************************}

Function hex( hz : Word ) : String;
var i  : Integer;
    hs : String;
Begin
  hs := '';
  Repeat
    i := hz mod 16;
    hz := hz div 16;
    case i of
      0..9  : hs := chr( i + ord( '0' ) ) + hs;
      10..15: hs := chr( i - 10 + ord( 'A' ) ) + hs;
    End;
  Until hz = 0;
  if Length( hs ) mod 2 <> 0 then hs := '0' + hs;
  hex := hs;
End;

{********************************************************************}
{ sb_GetEnviron : Initialisiere SBBASE-Struktur aufgrund bergebenem }
{                 Environment-String (BLASTER-Umgebungsvariable)     }
{*------------------------------------------------------------------*}
{ Eingabe : SBBASE - zu initialisierende Struktur                    }
{           Env    - die BALSTER Umgebungsvariable                   }
{ Ausgabe : =  0 : fehlerfreie Initialisierung                       }
{           = -1 : fehler bei Initialisierung                        }
{ Info : - Diese Struktur mu den einzelnen Modulen bergeben        }
{          werden, die daraufhin weitere Initialisierungen der       }
{          Struktur (z.B. DSP-Versionsnummer) durchfhren.           }
{********************************************************************}
Function sb_GetEnviron( var sbbase : SBBASE; Env : String ) : Integer;

Begin
  if Env <> '' then
    Begin
      sbbase.iDspPort := htoi( strichr( Env, 'A' ), -1 );
      sbbase.iMixPort := htoi( strichr( Env, 'M' ), -1 );
      if sbbase.iMixPort = -1 then
        sbbase.iMixPort := sbbase.iDspPort;
      sbbase.iMpuPort := htoi( strichr( Env, 'P' ), -1 );
      sbbase.uDspVersion := $FFFF;     { wird spter initialisiert }
      sbbase.iDspIrq  := htoi( strichr( Env, 'I' ), -1 );
      sbbase.iDspDmaB := htoi( strichr( Env, 'D' ), -1 );
      sbbase.iDspDmaW := htoi( strichr( Env, 'H' ), -1 );
      sb_GetEnviron := NO_ERROR;
    End
  else
    sb_GetEnviron := ERROR;
End;

{********************************************************************}
{ sb_Print : SBBASE-Struktur anzeigen                                }
{*------------------------------------------------------------------*}
{ Eingabe : SBBASE - anzuzeigende Struktur                           }
{********************************************************************}
Procedure sb_Print( var sbbase : SBBASE );

Begin
  Writeln( 'DSP-Port: ', hex( sbbase.iDspPort ) );
  Writeln( 'MIX-Port: ', hex( sbbase.iMixPort ) );
  Writeln( 'MPU-Port: ', hex( sbbase.iMpuPort ) );
  if sbbase.uDspVersion <> $FFFF then
    Begin
      Writeln( 'DSP-Version: ', HI( sbbase.uDspVersion ),'.',
                                LO( sbbase.uDspVersion ) );
      Writeln( 'Karte : ', sbbase.pDspName );
    End
  else
    Writeln( 'DSP nicht Initialisiert!' );

  Writeln( 'DSP-IRQ: ', sbbase.iDspIrq );
  Writeln( 'DSP- 8 Bit DMA: ', sbbase.iDspDmaB );
  Writeln( 'DSP-16 Bit DMA: ', sbbase.iDspDmaW );
End;

{********************************************************************}
{ AllocMem : Speicher ber DOS allokieren                            }
{*------------------------------------------------------------------*}
{ Eingabe : lNumPara - Gre des zu allokierenden Speicherbereichs   }
{                      in Paragraphen                                }
{           uSegment - Word-Variable, in der die Segmentadresse des  }
{                      allokierten Speicherbereichs zurckgeliefert  }
{                      wird                                          }
{********************************************************************}
Function AllocMem( lNumPara : Word; var uSegment : Word ) : Integer;

var regs : Registers;

Begin
  regs.ah := $48;
  regs.bx := lNumPara;
  msdos( regs );
  if ( regs.flags and 1 ) = 1 then
    allocmem := 0 { es konnte kein Speicher allokierte werden }
  else
    Begin
      allocmem := -1; { alles o.k. }
      uSegment := regs.ax; { Segmentadresse zurckliefern }
    End;
End;

{********************************************************************}
{ ReleaseMem : zuvor ber AllocMem allokierten Speicher wieder       }
{              freigeben                                             }
{*------------------------------------------------------------------*}
{ Eingabe : usegment - das zuvor bei einem Aufruf von AllocMem zu-   }
{                      rckgelieferte Segment                        }
{********************************************************************}
Procedure ReleaseMem( uSegment : Word );

var regs : Registers;

Begin
  regs.ah := $49;
  regs.es := uSegment;
  msdos( regs );
End;

{********************************************************************}
{ sb_LoadDriver : Soundblaster-Treiber laden                         }
{*------------------------------------------------------------------*}
{ Eingabe : Name - Name des zu ladenden Treibers (kompl. Dateiname)  }
{ Ausgabe : Adresse des Treibereinsprungs oder NULL, wenn der Trei-  }
{           ber nicht geladen werden konnte                          }
{********************************************************************}
Function sb_LoadDriver( Name:String ) : Longint;

var   iInc, uDummy,
      uSegment      : Word;
      lFileSize,
      lNumSeg,lpMem : Longint;
      iHandle       : File;

Begin
  uSegment := 0;
  {$i-}
    Assign( iHandle, Name );
    Reset( iHandle, 1 );
  {$i+}

  if ioresult = 0 then
    Begin { Datei konnte geffnet werden }
      lFileSize := FileSize( iHandle );  { Ge des Treibers ermitteln }
      lNumSeg := (lFileSize+15) div 16;          { Gre in Paragraphen }
      if lNumSeg < $FFFF then
        if AllocMem( Word ( lNumSeg ), uSegment ) <> 0  then
          Begin { es konnte gengend Speicher allokiert werden }
            lpMem := Longint( uSegment ) shl 16;
            while lFileSize > 0 do
              Begin
                if lFileSize > 65535 then iInc := 65535
                                     else iInc := lFileSize;
                BlockRead( iHandle, Pointer( lpMem )^, iInc, uDummy);
                if iInc = uDummy then
                  Begin
                    lpMem := lpmem + iInc;
                    lFileSize := lFileSize - iInc;
                  End
              else                  { Nach Lesefehler Speicher freigeben }
                Begin
                  ReleaseMem( uSegment );
                  uSegment := 0;
                  lFileSize := 0;
                End;
              End;
          End
        else
          uSegment := 0;
      close( iHandle );
    End;
  sb_Loaddriver := Longint( uSegment ) shl 16 ;
End;

{********************************************************************}
{ sb_UnloadDriver : zuvor ber sb_LoadDriver geladenen Soundblaster- }
{                   Treiber wieder aus Speicher entfernen            }
{*------------------------------------------------------------------*}
{ Eingabe : lpEntry : Adresse des Speicherblocks, den der Treiber    }
{                     belegt.                                        }
{********************************************************************}
Procedure sb_UnloadDriver( lpEntry : Longint );

Begin
  if lpEntry <> 0 then ReleaseMem( lpEntry shr 16 );
End;

End.