{********************************************************************}
{                     S E R T R A N S . P A S                        }
{*------------------------------------------------------------------*}
{ Aufgabe        : Filetransfer mit Fehlerkorrektur ber serielle    }
{                  Schnittstelle                                     }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 08.04.1994                                        }
{ letztes Update : 10.10.1994                                        }
{********************************************************************}
{$X+}                                 { Funktionsergebnisse optional }

Uses SERUTIL,ARGS;

Const
 CCITT_POLYNOM = $1021;          { x^16 + x^12 + x^5 + x^0 = $11021! }
                                                { einige Ascii-Codes }
 ASCII_SOH       = $01;                       { Start of Header      }
 ASCII_STX       = $02;                       { Start of Text / Data }
 ASCII_EOT       = $03;                       { End of Transmission  }
 ASCII_ENQ       = $05;                                    { Inquiry }
 ASCII_ACK       = $06;                       { Positive Acknowledge }
 ASCII_NAK       = $15;                       { Negative Acknowledge }
 ASCII_SYN       = $16;                       { Synchronus Idle      }

 PAKET_DATASIZE = 512;

 PAKET_FILEOPEN =  0;                      { Willkrliche Pakettypen }
 PAKET_FILECLOSE=  1;
 PAKET_FILEDATA =  2;

                                                   { Fehlermeldungen }
 TRANS_SUCCESS      =  0;
 TRANS_TIMEOUT      = -1;
 TRANS_TRANSERROR   = -2;
 TRANS_PROTOCOLL    = -3;
 TRANS_CRC          = -4;
 TRANS_NOFILE       = -5;

                                { Action-Codes fr trans_SendPaket() }
 SEND_RECEIVER   = 0;
 SEND_SENDING    = 1;
 SEND_GETNAKLO   = 2;
 SEND_GETNAKHI   = 3;
 SEND_PUTCRCLO   = 4;
 SEND_PUTCRCHI   = 5;
 SEND_GETREPLY   = 6;

                             { Action-Codes fr trans_ReceivePaket() }
 RECEIVE_RECEIVE   = 0;
 RECEIVE_GETCRCLO  = 1;
 RECEIVE_GETCRCHI  = 2;
 RECEIVE_SIGNAL    = 3;
 RECEIVE_PUTACK    = 4;
 RECEIVE_PUTNAK    = 5;
 RECEIVE_PUTNAKLO  = 6;
 RECEIVE_PUTNAKHI  = 7;

 PAKET_SUCCESS  = 0;
 PAKET_SEQUENCE = -10;
 PAKET_UNKNOWN  = -11;
 PAKET_TIMEOUT  = -12;

Type
Paket = RECORD { Pakete bilden die Grundlage der Kommunikation }
  bType     : Byte;            { Type des Pakets (Benutzerdefiniert) }
  uDataSize,                    { gltige Daten innerhalb des Pakets }
  uNr       : Word;                       { Sequenznummer des Pakets }
  bData     : Array[0..PAKET_DATASIZE - 1] of Char;                              { Daten }
End;

var
 iSerPort, iCom : Integer;
 lBaud          : Longint;
 Argumente      : NArgStrings;

{********************************************************************}
{ CalcCRC : Cyclic-Redundancy-Checksum (CRC) berechnen               }
{*------------------------------------------------------------------*}
{ Eingabe : uCRC  - bisherige Checksum                               }
{           bData - einzubeziehendes Byte                            }
{ Ausgabe : neue Checksumme inkl. des bergebenen Bytes              }
{*------------------------------------------------------------------*}
{ Info : Beim ersten Aufruf dieser Funktion mu die 'bisherige'      }
{        Checksumme auf $FFFF gesetzt werden.                        }
{********************************************************************}
Function CalcCRC( uCRC : Word; bData : Byte ) : Word;

var j : Integer;

Begin
  for j := 0 to 7 do
    Begin
      uCRC := uCRC shl 1;                     { Modulo Polynomdivision }
      if ( bData and $0080 ) <> 0 then
        uCRC := uCRC xor CCITT_POLYNOM;
      bData := bData shl 1;
  End;
  CalcCRC := uCRC;
End;

{********************************************************************}
{ paket_Build : Struktur vom Typ PAKET fr das Senden vorbereiten    }
{*------------------------------------------------------------------*}
{ Eingabe : Paket     - das zu initialisierende Paket                }
{           bType     - Benutzertyp des Pakets                       }
{           pData     - Adresse der Paket-Daten                      }
{           uDataSize - Anzahl der im Paket enthaltenen gltigen     }
{                       Bytes                                        }
{           uNr       - laufende Nummer des Pakets                   }
{********************************************************************}
Procedure paket_Build( var Paket      : PAKET;
                           bType      : Byte;
                           pData      : pointer;
                           uDataSize,
                           uNr        : Word );
var i      : Word;
    BufPtr : ^char;

Begin
  BufPtr := pData;  { Daten in das Paket eintragen }
  Paket.bType := bType;
  Paket.uDataSize := uDataSize;
  Paket.uNr := uNr;
  move( BufPtr^, Paket.bData[0], uDataSize );
End;

{********************************************************************}
{ trans_WriteUINT : Sende einzelnes UINT ber serielle Schnittstelle }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der Schnittstelle                   }
{           uData    - zu versendendes UINT                          }
{ Ausgabe : Fehlerstatus                                             }
{*------------------------------------------------------------------*}
{ Info : Diese Funktion verschickt zunchst das LO-, dann das HI-    }
{        Byte eines UINTs. Achtung! Kein Handshake!                  }
{********************************************************************}
Function trans_WriteUINT( iSerPort : Integer; uData : Word ) : Integer;

Begin
  trans_WriteUINT := TRANS_TRANSERROR;
  if ser_WriteByte(iSerPort, LO(uData), $8000,0,0)=SER_SUCCESS then
   if ser_WriteByte(iSerPort, HI(uData), $8000,0,0) = SER_SUCCESS then
     trans_WriteUINT := TRANS_SUCCESS;
End;

{********************************************************************}
{ trans_ReadUINT : Ein einzelnes UINT von Schnittstelle lesen       }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der Schnittstelle                   }
{           Data     - Integer-Variable, die den empfangenen Wert    }
{                      aufnehmen soll.                               }
{ Ausgabe : Fehlerstatus                                             }
{********************************************************************}
Function trans_ReadUINT(     iSerPort : Integer;
                         var Data     : Word ) : Integer;

var bErrL, bErrH : Byte;

Begin
  trans_ReadUINT := TRANS_TRANSERROR;
  if ser_ReadByte(iSerPort, bErrL, $8000, 0, 0) = SER_SUCCESS then
    if ser_ReadByte(iSerPort, bErrH, $8000, 0, 0) = SER_SUCCESS then
      Begin
        Data := bErrH * 256 + bErrL;
        trans_ReadUINT := TRANS_SUCCESS;
      End;
End;

{********************************************************************}
{ trans_WriteNak : Negative Aknowledge ber Schnittstelle senden     }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der Schnittstelle                   }
{           iCode    - erweiterter Fehlercode                        }
{ Ausgabe : Fehlerstatus                                             }
{********************************************************************}
Function trans_SendNak( iSerPort : Integer; iCode : Word ) : Integer;

Begin
  if ser_WriteByte( iSerPort, ASCII_NAK, $8000,0,0) = SER_SUCCESS then
    trans_SendNak := trans_WriteUINT( iSerPort, iCode )
  else
    trans_SendNak := TRANS_TRANSERROR;
End;

{********************************************************************}
{ trans_WriteAck : (Positives) Aknowledge ber Schnittstelle senden  }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der Schnittstelle                   }
{ Ausgabe : Fehlerstatus                                             }
{********************************************************************}
Function trans_SendAck( iSerPort : Integer ) : Integer;

Begin
  if ser_WriteByte( iSerPort, ASCII_ACK, $8000,0,0) = SER_SUCCESS then
    trans_SendAck := TRANS_SUCCESS
  else
    trans_SendAck := TRANS_TRANSERROR;
End;

{********************************************************************}
{ trans_SendPaket : Ganzes Datenpaket ber Schnittstelle senden      }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort   - Basisport der zu verwendenden Schnittstelle }
{           pPaket     - Adresse der Daten                           }
{           iPaketSize - Gre der Daten                             }
{ Ausgabe : Fehlerstatus                                             }
{********************************************************************}
Function trans_SendPaket( iSerPort   : Integer;
                          pPaket     : pointer;
                          iPaketSize : Word ) : Integer;
var uCRC, uAction,
    uTimeOut,
    uActByte      : Word;
    bData, bNakLo : Byte;          { erstes Byte des NAK Returncodes }
    bDataOk       : Boolean;
    bptr          : ^Byte;

Begin
  uTimeOut := $FFFF;
  uAction := SEND_RECEIVER;

  While uTimeOut <> 0 do
    Begin
      bDataOk := FALSE;                        { Keine Daten vorhanden }
      if ser_IsDataAvaiable( iSerPort ) then
        if ser_ReadByte( iSerPort, bData, 1, 0, 0 ) <> SER_SUCCESS then
          Begin
            trans_SendPaket := TRANS_TRANSERROR;
            Exit;
          End
        else
          bDataOk := TRUE;                             { oder doch? }

      if bDataOk then
        Begin
          case uAction of
            SEND_RECEIVER:
               if bData = ASCII_SYN then
                 Begin
                   uActByte := 0;
                   uCRC := $FFFF;
                   uAction := SEND_SENDING;
                 End;

            {-- Senden eines Paketes wird durch NAK unterbrochen! ---}
            SEND_SENDING:
              if bData = ASCII_NAK then uAction := SEND_GETNAKLO;

            SEND_GETNAKLO:
              Begin
                bNakLo := bData;
                uAction := SEND_GETNAKHI;
              End;

            SEND_GETNAKHI:
              Begin
                trans_SendPaket := bData * 256 + bNakLo;
                Exit;
              End;

            SEND_GETREPLY:
              Begin
                if bData = ASCII_ACK then
                begin
                  trans_SendPaket := TRANS_SUCCESS;
                  exit; { Ende der Finit-State-Machine }
                end;
                if bData = ASCII_NAK then
                begin
                  uAction := SEND_GETNAKLO;
                  trans_SendPaket := TRANS_PROTOCOLL;
                end;
              End;
          End;
        End
      else
       if ser_IsWritingPossible( iSerPort ) then
         Begin
           case uAction of
             SEND_SENDING:
                Begin
                  bptr := pPaket;
                  Inc( bptr, uActByte );
                  if ser_WriteByte( iSerPort, bptr^,
                                    1, 0, 0 ) = SER_SUCCESS then
                    Begin
                      bptr := pPaket;
                      Inc( bptr, uActByte );
                      uCRC := CalcCRC( uCRC, bptr^ );
                      Inc( uActByte );
                      if uActByte = iPaketSize then
                        uAction := SEND_PUTCRCLO;
                      uTimeOut := $FFFF;
                    End
                  else
                    Begin
                      trans_SendPaket := TRANS_TRANSERROR;
                      Exit;
                    End;
                End;

             SEND_PUTCRCLO:
               if ser_WriteByte( iSerPort, LO( uCRC ),
                                 1, 0, 0 ) = SER_SUCCESS then
                 uAction := SEND_PUTCRCHI
               else
                 Begin
                   trans_SendPaket := TRANS_TRANSERROR;
                   Exit;
                 End;

             SEND_PUTCRCHI:
               if ser_WriteByte( iSerPort, HI( uCRC ), 1,
                               0, 0 ) = SER_SUCCESS then
                 uAction := SEND_GETREPLY
               else
                 Begin
                   trans_SendPaket := TRANS_TRANSERROR;
                   Exit;
                 End
             else             { case else }
               Dec( uTimeOut )
           End;
         End;
    End;
  trans_SendPaket := TRANS_TIMEOUT;
End;

{********************************************************************}
{ trans_ReceivePaket: Datenpaket von der Schnittstelle empfangen     }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort   - Basisport der zu verwendenden Schnittstelle }
{           pPaket     - Adresse des Empfangsbuffers                 }
{           iPaketSize - Gre des Buffers                           }
{ Ausgabe : Fehlerstatus                                             }
{********************************************************************}
Function trans_ReceivePaket( iSerPort   : Integer;
                             pPaket     : Pointer;
                             iPaketSize : Word ) : Integer;

var uCRC, uAction,
    uTimeOut,
    uActByte      : Word;
    bData, bCrcLo : Byte;                { erstes Byte des CRC Codes }
    iAckError     : Integer;
    bDataOk       : Boolean;
    bptr          : ^Byte;

Begin
  uTimeOut := $FFFF;
  uAction := RECEIVE_SIGNAL;

  uCRC := $FFFF;         { wenigstens einmal auf Variablen zugreifen }
  uActByte := 0;

  while uTimeOut <> 0 do
    Begin
      bDataOk := FALSE;                       { Keine Daten vorhanden }
      if ser_IsDataAvaiable( iSerPort ) then
        if ser_ReadByte( iSerPort, bData, 1, 0, 0 ) <> SER_SUCCESS then
          Begin
            trans_ReceivePaket := TRANS_TRANSERROR;
            Exit;
          End
        else
          bDataOk := TRUE;                            { oder doch? }

      if bDataOk then
        Begin
          case uAction of
            RECEIVE_RECEIVE:
              Begin
                uCRC := CalcCRC( uCRC, bData );
                bptr := pPaket;
                Inc( bptr, uActByte );
                bptr^ := bData;
                inc( uActByte );
                if uActByte = iPaketSize then
                  uAction := RECEIVE_GETCRCLO;
                uTimeOut := $FFFF;
              End;

            RECEIVE_GETCRCLO:
              Begin
                bCrcLo := bData;
                uAction := RECEIVE_GETCRCHI;
              end;

            RECEIVE_GETCRCHI:
            begin
              if uCRC = bData * 256 + bCrcLo then
              begin
                uAction := RECEIVE_PUTACK;
              end
               else
                 Begin
                   iAckError := TRANS_CRC;
                   uAction := RECEIVE_PUTNAK;
                 End;
            end;
          End;
        End
      else
       if ser_IsWritingPossible( iSerPort ) then
         Begin
           case uAction of
             RECEIVE_SIGNAL:
             begin
                if ser_WriteByte( iSerPort, ASCII_SYN,
                                  1, 0, 0 ) = SER_SUCCESS then
                  Begin
                    uCRC := $FFFF;
                    uActByte := 0;
                    uAction := RECEIVE_RECEIVE;
                  End
               else
                 Begin
                   trans_ReceivePaket := TRANS_TRANSERROR;
                   Exit;
                 End;
             end;
             RECEIVE_PUTACK:
               Begin
                 if ser_WriteByte( iSerPort, ASCII_ACK,
                                   1, 0, 0 ) = SER_SUCCESS then
                   trans_ReceivePaket := TRANS_SUCCESS
                 else
                   trans_ReceivePaket := TRANS_TRANSERROR;
                 Exit; { Ende der Finit-State-Machine }
               End;

             RECEIVE_PUTNAK:
             begin
               if ser_WriteByte( iSerPort, ASCII_NAK,
                                 1, 0, 0 ) = SER_SUCCESS then
                  uAction := RECEIVE_PUTNAKLO
               else
               begin
                  trans_ReceivePaket := TRANS_TRANSERROR;
                  Exit;
               end;
             end;
             RECEIVE_PUTNAKLO:
             begin
               if ser_WriteByte( iSerPort, LO( iAckError ),
                                 1, 0, 0 ) = SER_SUCCESS then
                 uAction := RECEIVE_PUTNAKHI
               else
                 Begin
                   trans_ReceivePaket := TRANS_TRANSERROR;
                   Exit;
                 End;
             end;
             RECEIVE_PUTNAKHI:
               Begin
                 if ser_WriteByte( iSerPort, HI( iAckError ),
                                   1, 0, 0 ) = SER_SUCCESS then
                   trans_ReceivePaket := iAckError
                 else
                 begin
                   trans_ReceivePaket := TRANS_TRANSERROR;
                   Exit;
                 end;
               End;
             else  { case else }
               Dec( uTimeOut );
           End;
         End;
    End;
  trans_ReceivePaket := TRANS_TIMEOUT;
End;

{********************************************************************}
{ trans_ReceiveFile : Ganze Datei ber Schnittstelle empfangen       }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der zu verwendenden Schnittstelle   }
{ Ausgabe : Fehlerstatus                                             }
{********************************************************************}
Function trans_ReceiveFile( iSerPort : Integer ) : Integer;

var Pak      : PAKET;
    iTimeOut,
    i,
    ior,err  : Integer;
    fHandle  : File;
    w        : Word;
    DatName  : string;

Const uNr    : Word = 0;

Begin
  iTimeOut := 10;        { Beim ersten Paket ein wenig lnger warten }
  while iTimeOut <> 0 do
    Begin
      err := trans_ReceivePaket( iSerPort,             { Paket abholen }
                                 @Pak,
                                 sizeof( Paket ) );
      if err = TRANS_SUCCESS then
        Begin
          iTimeOut := 4;                             { Timeout verkrzen }
          if uNr <> Pak.uNr then
            Begin
              Close( fHandle );
              trans_SendNak( iSerPort, Word ( PAKET_SEQUENCE ) );
              trans_ReceiveFile := PAKET_SEQUENCE;
              Exit;
            End
         else
           Begin
             Inc( uNr );
             Write( #13, uNr );
             case Pak.bType of
               PAKET_FILEOPEN:
                 Begin
                   DatName := '';
                   for i := 0 to Pak.uDataSize do
                     DatName := DatName + Pak.bData[i];

                   {$i-}
                     Assign( fhandle, DatName );
                     Rewrite( fhandle, 1 );
                   {$i+}

                   ior := ioresult;
                   if ior <> 0 then
                     Begin
                       trans_SendNak( iSerPort, ior );
                       trans_ReceiveFile := ior;
                       Exit;
                     End;
                 End;

               PAKET_FILEDATA:
                 Begin
                   {$i-}
                      Blockwrite(fhandle,Pak.bData,Pak.uDataSize,w);
                   {$i+}
                   if w <> Pak.uDataSize then
                     Begin
                       ior := ioresult;
                       trans_SendNak( iSerPort, ior );
                       Close( fHandle );
                       trans_ReceiveFile := ior;
                       Exit;
                     End;
                 End;

               PAKET_FILECLOSE:
                 Begin
                   Close( fHandle );
                   trans_ReceiveFile := PAKET_SUCCESS;
                   Exit;
                 End

               else { case else }
                 Begin
                   trans_SendNak( iSerPort, Word ( PAKET_UNKNOWN ) );
                   trans_ReceiveFile := PAKET_UNKNOWN;
                   Exit;
                 End;
             End;
           End;
        End
      else
        Dec( iTimeOut );
    End;
  trans_ReceiveFile := PAKET_TIMEOUT;
End;

{********************************************************************}
{ trans_SendFile : Ganze Datei ber Schnittstelle senden             }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der zu verwendenden Schnittstelle   }
{           Name     - Name der zu versendenden Datei                }
{ Ausgabe : Fehlerstatus                                             }
{********************************************************************}
Function trans_SendFile( iSerPort : Integer; Name : String ) : Integer;

var fHandle : File;
    Pak     : PAKET;
    iSize,
    err,
    iCnt,
    iLen    : Integer;
    n       : string;
Const uNr   : Word = 0;

Begin
  {$i-}
    Assign( fHandle, Name );
    Reset( fhandle, 1 );
  {$i+}
  if ioresult = 0 then
    Begin
      iLen := Length( Name );
      while ( ( iLen > 0 ) and ( Name[iLen] <> '\' )
              and ( Name[iLen] <> ':' ) ) do
               Dec( iLen );

      paket_Build( Pak, PAKET_FILEOPEN, @Name[iLen+1],
                   Length( Name ) - iLen, uNr );
      Inc( uNr );

      iCnt := 25;
      Write(#13, uNr );
      Repeat
        Dec( iCnt );
        err := trans_SendPaket( iSerPort, @Pak, sizeof( PAKET ) );
      Until ( ( err <> TRANS_TIMEOUT ) or ( iCnt = 0 ) );

      if err <> TRANS_SUCCESS then
        Begin
          Close( fHandle );
          trans_SendFile := err;
          Exit;
        End;

      Repeat
        BlockRead( fHandle, Pak.bData, PAKET_DATASIZE, iSize );
        if iSize <> 0 then
          Begin
            paket_Build( Pak, PAKET_FILEDATA, @Pak.bData, iSize, uNr );
            Inc( uNr );
            iCnt := 10;
            Write(#13, uNr );
            Repeat
              Dec( iCnt );
              err := trans_SendPaket( iSerPort, @Pak, sizeof( PAKET ) );
            Until ( ( err <> TRANS_TIMEOUT ) or ( iCnt = 0 ) );
            if err <> TRANS_SUCCESS then
              Begin
                Close( fHandle );
                trans_SendFile := err;
                Exit;
             End;
          End;
      Until ( iSize <> PAKET_DATASIZE );

      Close( fHandle );
      paket_Build( Pak, PAKET_FILECLOSE, @Name[iLen],
                   Length( Name ) - iLen, uNr );
        Inc( uNr );

      iCnt := 10;
      Write(#13, uNr );
      Repeat
        Dec( iCnt );
        err := trans_SendPaket( iSerPort, @Pak, sizeof( PAKET ) );
      Until ( ( err <> TRANS_TIMEOUT ) or ( iCnt = 0 ) );
      trans_SendFile := err;
      Exit;
    End;
  trans_SendFile := TRANS_NOFILE;
End;

{********************************************************************}
{ trans_PError : Fehlermeldung ausgeben                              }
{*------------------------------------------------------------------*}
{ Eingabe : iError - aufgetretener Fehler                            }
{********************************************************************}
Procedure trans_PError( iError : Integer );

Begin
  case iError of
    TRANS_SUCCESS:
         Writeln( 'bertragung OK!' );
    TRANS_TIMEOUT:
         Writeln( 'FEHLER: Byte-Timeout' );
    TRANS_TRANSERROR:
         Writeln( 'FEHLER: Genereller Fehler' );
    TRANS_PROTOCOLL:
         Writeln( 'FEHLER: Ack oder NAck fehlt!' );
    TRANS_CRC:
         Writeln( 'FEHLER: CRC-Fehler!' );
    TRANS_NOFILE:
         Writeln( 'FEHLER: Kann Datei nicht ffnen!' );
    PAKET_SEQUENCE:
         Writeln( 'FEHLER: Ungltige Paket-Reihenfolge!' );
    PAKET_UNKNOWN:
         Writeln( 'FEHLER: Unbekannte Paket-ID!' );
    PAKET_TIMEOUT:
         Writeln( 'FEHLER: Paket-Timeout' );
  End;
End;

{********************************************************************}
{ H A U P T P R O G R A M M                                          }
{********************************************************************}
Begin
  if GetArg( '?', _none, NIL, 1 ) <> 0 then
    Begin
      Writeln( 'Aufruf:' );
      Writeln( 'SERTRANS [SendeDatei] [-COM:port] [-BAUD:bdrate]' );
      Writeln( 'comport  = 1 oder 2    (Default: 1 )' );
      Writeln( 'baudrate = 50 - 115200 (Default: 9600)' );
      Halt( 0 );
    End;

  if GetArg( '-COM:', _int, @iCom, 1 ) <> 0 then
    Begin
      if iCom = 1 then       { Nur COM1 und COM2 sind 'standardisiert' }
        iSerPort := SER_COM1
      else
        if iCom = 2 then       { Nur COM1 und COM2 sind 'standardisiert' }
          iSerPort := SER_COM2
       else
         Begin
           Writeln( 'Nicht untersttzter Com-Port!' );
           Halt( 0 );
         End;
    End
  else                     { Nur COM1 und COM2 sind 'standardisiert' }
    iSerPort := SER_COM1;

  if GetArg( '-BAUD:', _long, @lBaud, 1 ) = 0 then
    lBaud := 9600;              { Maximale Baudrate bei UART 8450A }
  if lBaud > SER_MAXBAUD then
    Begin
      Writeln( 'Baudrate zu hoch!' );
      Writeln( 'Maximum: ',SER_MAXBAUD,' Bd' );
    End;

  ser_FIFOLevel( iSerPort, 0 ); { FIFO abschalten }
  if ser_Init( iSerPort, lBaud,             { Maximum bei UART 8450A }
               SER_LCR_8BITS or
               SER_LCR_1STOPBIT or
               SER_LCR_NOPARITY ) = 0 then
    Begin
      Writeln( 'Keine Schnittstelle!' );
      Halt( 0 );
    End;

  if GetNArg( '-', Argumente ) <> 0 then
    Begin
      Writeln( 'Sending' );
      trans_PError( trans_SendFile( iSerPort, Argumente[0] ) );
    End
  else
    Begin
      Writeln( 'Receiving' );
      trans_PError( trans_ReceiveFile( iSerPort ) );
    End;
End.
