{********************************************************************}
{                       S E R U T I L . P A S                        }
{*------------------------------------------------------------------*}
{ Aufgabe        : Funktionen fr Direktzugriff auf serielle         }
{                  Schnittstelle                                     }
{*------------------------------------------------------------------*}
{ Autor          : Michael Tischer / Bruno Jennrich                  }
{ entwickelt am  : 08.04.1994                                        }
{ letztes Update : 10.10.1994                                        }
{********************************************************************}
{$X+}                                 { Funktionsergebnisse optional }
Unit SERUTIL;

Interface

Uses WIN;
Const
 SER_COM1 = $3F8;                                { Basisadresse COM1 }
 SER_COM2 = $2F8;                                { Basisadresse COM2 }

 SER_IRQ_COM1 = 4;                              { IRQ 4 = Vektor $0C }
 SER_IRQ_COM2 = 3;                              { IRQ 3 = Vektor $0B }

 SER_TXBUFFER     =  $00;                        { Transmit Register }
 SER_RXBUFFER     =  $00;                         { Receive Register }
 SER_DIVISOR_LSB  =  $00;                    { Baudraten Divisor LSB }
 SER_DIVISOR_MSB  =  $01;                    { Baudraten Divisor MSB }
 SER_IRQ_ENABLE   =  $01;                { Interrupt-Enable-Register }
 SER_IRQ_ID       =  $02;                    { Interrupt ID-Register }
 SER_FIFO         =  $02;                            { FIFO-Register }
 SER_2Function    =  $02;              { Alternate-Function-Register }
 SER_LINE_CONTROL =  $03;                        { Leitungskontrolle }
 SER_MODEM_CONTROL=  $04;                           { Modemkontrolle }
 SER_LINE_STATUS  =  $05;                           { Leitungsstatus }
 SER_MODEM_STATUS =  $06;                              { Modemstatus }
 SER_SCRATCH      =  $07;                         { Scratch-Register }

              { IRQ-Enable-Register-Bits (Interrupts an-/abschalten) }
 SER_IER_RECEIVED = $01;                     { IRQ nach Datenempfang }
 SER_IER_SENT     = $02;                  { IRQ nach gesendetem Byte }
 SER_IER_LINE     = $04;        { IRQ nach Leitungszustands-nderung }
 SER_IER_MODEM    = $08;           { IRQ nach Modemzustands-nderung }

                    { IRQ-ID - Bits (Durch was wurde IRQ ausgelst?) }
 SER_ID_PENDING    = $01;              { steht ein serieller IRQ an? }
 SER_ID_MASK       = $06;          { ID wird in Bits 1 und 2 kodiert }
 SER_ID_LINESTATUS = $06;       { Leitungszustand (Error oder Break) }
 SER_ID_RECEIVED   = $04;                          { Daten empfangen }
 SER_ID_SENT       = $02;                      { Byte wurde gesendet }
 SER_ID_MODEMSTATUS= $00;          { CTS, DSR, RI oder RLSD nderung }

            { Bitbelegung im FIFO-Register (Falls mind. 16550A-UART) }
 SER_FIFO_ENABLE       = $01;
 SER_FIFO_RESETRECEIVE = $02;
 SER_FIFO_RESETTRANSMIT= $04;

      { FIFO-Bits (Nach wievielen Bytes im FIFO wird IRQ ausgelst ) }
 SER_FIFO_TRIGGER0   = $00;                                 { Normal }
 SER_FIFO_TRIGGER4   = $40;                                { 4 Bytes }
 SER_FIFO_TRIGGER8   = $80;                                { 8 Bytes }
 SER_FIFO_TRIGGER14  = $C0;                               { 14 Bytes }

              { Line-Control-Register - Bits (bertragungsparameter) }
 SER_LCR_WordLEN    = $03;        { Anzahl der zu bertragenden Bits }
 SER_LCR_5BITS      = $00;
 SER_LCR_6BITS      = $01;
 SER_LCR_7BITS      = $02;
 SER_LCR_8BITS      = $03;
 SER_LCR_2STOPBITS  = $04;                     { 2 bzw. 1.5 Stopbits }
 SER_LCR_1STOPBIT   = $00;                               { 1 Stopbit }

 SER_LCR_NOPARITY   = $00;              { Parittsprfung abschalten }
 SER_LCR_ODDPARITY  = $08;                        { ungerade Paritt }
 SER_LCR_EVENPARITY = $18;                          { gerade Paritt }
 SER_LCR_PARITYSET  = $28;               { Parittsbit immer gesetzt }
 SER_LCR_PARITYCLR  = $38;              { Parittsbit immer gelscht }
 SER_LCR_PARITYMSK  = $38;

 SER_LCR_SENDBREAK  = $40;         { Sende Break solange Bit gesetzt }
 SER_LCR_SETDIVISOR = $80;       { fr Zugriff auf Baudraten-Divisor }

                  { Modem-Control-Register - Bits (Signal Kontrolle) }
 SER_MCR_DTR        = $01;                       { DTR-Signal setzen }
 SER_MCR_RTS        = $02;                       { RTS-Signal setzen }
 SER_MCR_UNUSED     = $04;
 SER_MCR_IRQENABLED = $08;          { IRQ's an IRQ-Controller melden }
 SER_MCR_LOOP       = $10;                              { Selbsttest }

                  { Line-Status-Register - Bits (bertragungsfehler) }
 SER_LSR_DATARECEIVED = $01;      { Datenwort (5 - 8 Bits) empfangen }
 SER_LSR_OVERRUNERROR = $02;         { vorheriges Datenwort verloren }
 SER_LSR_PARITYERROR  = $04;                        { Parittsfehler }
 SER_LSR_FRAMINGERROR = $08;                 { Start-/Stopbit Fehler }
 SER_LSR_BREAKDETECT  = $10;                        { Break entdeckt }
 SER_LSR_ERRORMSK = ( SER_LSR_OVERRUNERROR or SER_LSR_PARITYERROR or
                      SER_LSR_FRAMINGERROR or SER_LSR_BREAKDETECT );
 SER_LSR_THREMPTY     = $20;
 SER_LSR_TSREMPTY     = $40;

{ Modem Status Register - Bits (Welche Signale sind gesetzt)         }
{ Die Delta...-Bits geben an, ob sich der Zustand der entsprechenden }
{ Signale seit dem letzten Lesen des Modem Status Register verndert }
{ hat.                                                               }
 SER_MSR_DCTS = $01;                    { Delta CTS (Zustand in CTS) }
 SER_MSR_DDSR = $02;                    { Delta DSR (Zustand in DSR) }
 SER_MSR_DRI  = $04;                      { Delta RI (Zustand in RI) }
 SER_MSR_DCD  = $08;                      { Delta CD (Zustand in CD) }
 SER_MSR_CTS  = $10;                         { Clear To Send gesetzt }
 SER_MSR_DSR  = $20;                        { Data Set Ready gesetzt }
 SER_MSR_RI   = $40;                         { Rin Indicator gesetzt }
 SER_MSR_CD   = $80;                        { Carrier Detect gesetzt }

 NOSER    =0;
 INS8250  =1;                        { National Semiconductor UART's }
 NS16450  =2;
 NS16550A =3;
 NS16C552 =4;

 SER_MAXBAUD = 115200;                           { Maximale Baudrate }

 SER_SUCCESS    = 0;
 SER_ERRSIGNALS = $0300;
 SER_ERRTIMEOUT = $0400;

Function ser_UARTType( iSerPort : Integer ) : Integer;

Function ser_Init( iSerPort  : Integer;
                   lBaudRate : longint;
                   bParams   : Byte ) : Integer;

Procedure ser_FIFOLevel( iSerPort : Integer; bLevel : Byte );

Function ser_IsDataAvaiable( iSerPort : Integer ) : Boolean;

Function ser_IsWritingPossible( iSerPort : Integer ) : Boolean;

Function ser_IsModemStatusSet( iSerPort    : Integer;
                               bTestStatus : Byte ) : Boolean;

Procedure ser_SetModemControl( iSerPort    : Integer;
                               bNewControl : Byte );

Function ser_WriteByte( iSerPort : Integer;
                        bData    : Byte;
                        uTimeOut : Word;
                        bSigMask,
                        bSigVals : Byte ) : Integer;

Function ser_ReadByte(     iSerPort : Integer;
                       var Data    : Byte;
                           uTimeOut : Word;
                           bSigMask,
                           bSigVals : Byte ) : Integer;

Function ser_WritePaket( iSerPort : Integer;
                         pData    : pointer;
                         iLen     : Integer;
                         uTimeOut : Word;
                         bSigMask,
                         bSigVals : Byte ) : Integer;

Function ser_ReadPaket( iSerPort : Integer;
                        pData    : pointer;
                        iLen     : Integer;
                        uTimeOut : Word;
                        bSigMask,
                        bSigVals : Byte ) : Integer;

Procedure ser_CLRIRQ( iSerPort : Integer );

Procedure ser_SETIRQ( iSerPort : Integer );

Function ser_SetIRQHandler( iSerPort,
                            iSerIRQ   : Integer;
                            lpHandler : Pointer;
                            bEnablers : Byte) : Pointer;

Procedure ser_RestoreIRQHandler( iSerPort,
                                 iSerIRQ   : Integer;
                                 lpHandler : Pointer);

Procedure ser_PrintError( var Win : WINDOW; e : Integer );

Procedure ser_PrintModemStatus( var Win : WINDOW; iSerPort : Integer );

Procedure ser_PrintLineStatus( var Win : WINDOW; iSerPort : Integer);

Function ser_GetBaud( iSerPort : Integer ) : Longint;

Procedure ser_PrintCardSettings( var Win : WINDOW; iSerPort : Integer );

Implementation

Uses IRQUTIL;

type SerBuf    = array[0..65534] of byte;
     SerBufPtr = ^SerBuf;

{********************************************************************}
{ ser_UARTType : Typ des UART-Chips ermitteln                        }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort     - Basisport der zu testenden Schnittstelle  }
{ Ausgabe : 0 (NOSER)    - Kein UART-Chip gefunden                   }
{           1 (INS8250)  - INS8250 oder INS8250-B Chip               }
{           2 (NS16450)  - INS8250A, INS82C50A, NS16450, NS16C450    }
{           3 (NS16550A) - NS16550A Chip                             }
{           4 (NS16C552) - NS16C552 Chip                             }
{********************************************************************}
Function ser_UARTType( iSerPort : Integer ) : Integer;

var b          : Byte;
    UartDetect : integer;
Begin
  UartDetect := -1; { -1 zeigt an, da noch nicht initialisiert }

  {- Grundfunktionalitt prfen ------------------------------------ }
  port[iSerPort + SER_LINE_CONTROL] := $AA;  { Divisor-Latch gesetzt }
  if port[iSerPort + SER_LINE_CONTROL] <> $AA then
    UartDetect := NOSER
  else
    Begin
      port[iSerPort + SER_DIVISOR_MSB] := $55;   { Divisor beschreiben }
      if port[iSerPort + SER_DIVISOR_MSB] <> $55
        then UartDetect := NOSER
      else                              { Divisor-Latch lschen }
        Begin
          port[iSerPort + SER_LINE_CONTROL] := $55;
          if port[iSerPort + SER_LINE_CONTROL] <> $55 then
            UartDetect := NOSER
          else
            Begin
              port[iSerPort + SER_IRQ_ENABLE] := $55;
              if port[iSerPort + SER_IRQ_ENABLE] <> $05 then
                UartDetect := NOSER
              else
                Begin
                  port[iSerPort + SER_FIFO] := 0;     { FIFO und IRQ lschen }
                  port[iSerPort + SER_IRQ_ENABLE] := 0;
                  if port[iSerPort + SER_IRQ_ID] <> 1 then
                    UartDetect := NOSER
                  else
                    Begin
                      port[iSerPort + SER_MODEM_CONTROL] := $F5;
                      if port[iSerPort + SER_MODEM_CONTROL] <> $15 then
                        UartDetect := NOSER
                    end;
                end;
            end;
        end;
    end;
  if UartDetect = -1  then { noch nicht herausgefiltert? }
    Begin { Looping }
      port[iSerPort + SER_MODEM_CONTROL] := SER_MCR_LOOP;
      b := port[iSerPort + SER_MODEM_STATUS];
      if ( port[iSerPort + SER_MODEM_STATUS] and $F0 ) <> 0 then
        UartDetect := NOSER
      else
        Begin
          port[iSerPort + SER_MODEM_CONTROL] := $1F;
          if ( port[iSerPort + SER_MODEM_STATUS] and $F0 ) <> $F0 then
            UartDetect := NOSER
          else
            Begin
              port[iSerPort + SER_MODEM_CONTROL] := SER_MCR_DTR or
                                                    SER_MCR_RTS;
                                       { Scratch-Register vorhanden? }
              port[iSerPort + SER_SCRATCH] := $55;
              if port[iSerPort + SER_SCRATCH] <> $55 then
                UartDetect := INS8250
              else                      { FIFO vorhanden ? }
                Begin
                  port[iSerPort + SER_SCRATCH] := 0;
                  port[iSerPort + SER_FIFO] := $CF;
                  if ( port[iSerPort + SER_IRQ_ID] and $C0 ) <> $C0 then
                    UartDetect := NS16450
                  else
                    Begin
                      port[iSerPort + SER_FIFO] := 0;
                            { Alternate-Function Register vorhanden? }
                      port[iSerPort + SER_LINE_CONTROL] := SER_LCR_SETDIVISOR;
                      port[iSerPort + SER_2Function] := $07;
                      if port[iSerPort + SER_2Function] <> $07 then
                        Begin
                          port[iSerPort + SER_LINE_CONTROL] := 0;
                          UartDetect := NS16550A;
                        End
                      else
                        Begin
                          port[iSerPort + SER_LINE_CONTROL] := 0;               { Rcksetzen }
                          port[iSerPort + SER_2Function] := 0;
                          UartDetect := NS16C552;
                        End;
                    End;
                End;
            End;
        End;
    End;
  ser_UARTType := UartDetect;
End;

{********************************************************************}
{ ser_Init : serielle Schnittstelle initialisieren                   }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der Schnittstelle, die              }
{                      initialisiert werden soll.                    }
{           lBaud    - Baudrate ( von 1 - 115200 )                   }
{           bParams  - Bitmaske der restlichen Parameter             }
{                      (s. SER_LCR_...-Bits)                         }
{ Ausgabe : TRUE  - Schnittstelle konnte initialisiert werden        }
{           FALSE - keine Schnittstelle gefunden                     }
{********************************************************************}
Function ser_Init( iSerPort  : Integer;
                   lBaudRate : longint;
                   bParams   : Byte ) : Integer;

var uDivisor : Word;
    b        : Byte;
    uART     : Integer;

Begin
  uART := ser_UARTType( iSerPort );
  if uart = NOSER then
    Begin                                 { Baudratendivisor berechnen }
      ser_Init := NOSER;
      exit;
    end;

  uDivisor := ( SER_MAXBAUD div lBaudRate );
                                       
  {-- Baudrate eintellen --------}
  port[iSerPort + SER_LINE_CONTROL] :=  { Divisor-Zugriff ermglichen }
     port[SER_LINE_CONTROL] or SER_LCR_SETDIVISOR;

  port[iSerPort + SER_DIVISOR_LSB] := LO( uDivisor );
  port[iSerPort + SER_DIVISOR_MSB] := HI( uDivisor );

  port[iSerPort + SER_LINE_CONTROL] := { Divisor-Zugriff verhindern }
     port[SER_LINE_CONTROL] and not SER_LCR_SETDIVISOR;

  {-- Andere Parameter erst nach Rcksetzen des Baudraten Latches    --}
  {-- setzen, da durch diese Operation alle Schnittstellen Parameter --}
  {-- gelscht werden!                                               --}

  port[iSerPort + SER_LINE_CONTROL] := bParams;
        { Ein Byte lesen, um evtl. vorhandenen Fehler zurckzusetzen }
  b := port[iSerPort + SER_TXBUFFER];
  ser_Init := uART;
End;

{********************************************************************}
{ ser_FIFOLevel : Setze Gre des FIFO-Buffers                       }
{*------------------------------------------------------------------*}
{ Eingabe : 0                      - FIFO Buffergre = 0, Disable   }
{                                    und Reset (1Byte)               }
{           SER_FIFO_TRIGGER4/8/14 - Gre = 4, 8 oder 14 Byte       }
{********************************************************************}
Procedure ser_FIFOLevel( iSerPort : Integer; bLevel : Byte );

Begin
  if bLevel <> 0 then
    port[iSerPort + SER_FIFO] := bLevel or SER_FIFO_ENABLE
  else
    port[iSerPort + SER_FIFO] := SER_FIFO_RESETRECEIVE or
                                 SER_FIFO_RESETTRANSMIT;
End;

{********************************************************************}
{ ser_IsDataAvaiable : Stehen Daten zum Lesen bereit?                }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der zu berprfenden Schnittstelle. }
{ Ausgabe : = 0 : Kein Byte zum Lesen vorhanden                      }
{          <> 0 : Byte steht bereit                                  }
{*------------------------------------------------------------------*}
{ Info : Ein Byte wird Bit fr Bit versandt, und ist somit erst      }
{        dann wieder ein vollstndiges Byte, wenn die empfangende    }
{        Schnittstelle die einzelnen Bits zusammengesetzt hat. Dies  }
{        wird von dieser Funktion geprft.                           }
{********************************************************************}
Function ser_IsDataAvaiable( iSerPort : Integer ) : Boolean;

Begin
  ser_IsDataAvaiable := ( port[iSerPort + SER_LINE_STATUS]
                          and SER_LSR_DATARECEIVED ) <> 0;
End;

{********************************************************************}
{ ser_IsWritingPossible : Kann Schnittstelle nchstes Byte senden?   }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der zu berprfenden Schnittstelle. }
{ Ausgabe : = 0 : Byte darf nicht gesendet werden.                   }
{           <>0 : Schnittstelle zum Senden bereit.                   }
{*------------------------------------------------------------------*}
{ Info : Eine serielle Schnittstelle sollte in folgenden Fllen      }
{        nicht zum Senden eines Bytes herangezogen werden:           }
{        1. Ein empfangenes Byte wurde noch nicht von Schnittstelle  }
{           'abgeholt'.                                              }
{        2. Ein alter Sendeauftrag ist noch nicht abgeschlossen.     }
{********************************************************************}
Function ser_IsWritingPossible( iSerPort : Integer ) : Boolean;

Begin
  ser_IsWritingPossible := ( port[iSerPort + SER_LINE_STATUS]
                             and SER_LSR_TSREMPTY ) <> 0;
End;

{********************************************************************}
{ ser_IsModemStatusSet : Zustand der Eingangs-Leitungen berprfen   }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort    - Basisport der Schnittstelle.               }
{           bTestStatus - Bitmuster der zu testenden Leitungen       }
{                         (CTS, DSR, RI, CD)                         }
{********************************************************************}
Function ser_IsModemStatusSet( iSerPort : Integer;
                               bTestStatus : Byte ) : Boolean;
Begin
  ser_IsModemStatusSet := ( port[iSerPort + SER_MODEM_STATUS]
                            and bTestStatus ) = bTestStatus;
End;

{********************************************************************}
{ ser_SetModemControl : Signalleitungen fr Kommunikation mit Modem  }
{                       etc. setzen.                                 }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort    - Basisport der Schnittstelle.               }
{           bNewControl - Neuer Zustand der DTR, RTS etc. Leitungen  }
{********************************************************************}
Procedure ser_SetModemControl( iSerPort : Integer; bNewControl : Byte );

Begin
  port[iSerPort + SER_MODEM_CONTROL] := bNewControl;
End;

{********************************************************************}
{ ser_WriteByte : Ein Byte senden                                    }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der Schnittstelle, ber die ein     }
{                      Byte versendet werden soll.                   }
{           bData    - zu versendendes Byte                          }
{           uTimeOut - Anzahl der Schleifendurchlufe nach denen bei }
{                      Misserfolg des Sendens ein TimeOut-Fehler     }
{                      gemeldet wird. (Ist iTimeOut = 0 wird un-     }
{                      endlich lange gewartet.)                      }
{           bSigMask - Bitmaske der zu testenden Signalleitungen     }
{                      (RTS, CTS, CD, RI)                            }
{           bSigVals - Zustand der Signalleitungen nach dem aus-     }
{                      maskieren mit obiger Maske.                   }
{ Ausgabe :  = 0 - Byte wurde gesendet                               }
{           <> 0 - Fehler                                            }
{********************************************************************}
Function ser_WriteByte( iSerPort           : Integer;
                        bData              : Byte;
                        uTimeOut           : Word;
                        bSigMask, bSigVals : Byte ) : Integer;
Begin
  if uTimeOut <> 0 then                           { TimeOut-Schleife }
    Begin
      While( not ser_IsWritingPossible( iSerPort )
             and ( uTimeOut<> 0 ) )
        do Dec( uTimeOut );
      if uTimeOut = 0 then Begin
        ser_WriteByte := SER_ERRTIMEOUT;
        Exit;
      End;
    End
  else { Warten! }
    Repeat
    Until ser_IsWritingPossible( iSerPort );     

  {-- Signalleitungen testen ---------}
  if ( port[iSerPort + SER_MODEM_STATUS] and bSigMask ) = bSigVals then
    Begin                 { Byte zum Senden an Schnittstelle bergeben }
      port[iSerPort + SER_TXBUFFER] := bData;
                                 { Schnittstellen-Fehler zurckgeben }
      ser_WriteByte := port[iSerPort + SER_LINE_STATUS] and
                       SER_LSR_ERRORMSK;
    End
  else
    ser_WriteByte := SER_ERRSIGNALS;
End;

{********************************************************************}
{ ser_ReadByte : Byte empfangen                                      }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der Schnittstelle, ber die ein     }
{                      Byte empfangen werden soll.                   }
{           Data     - die Byte-Variable, die das empfangende Byte   }
{                      aufnehmen soll.                               }
{           uTimeOut - Anzahl der Schleifendurchlufe nach denen bei }
{                      Misserfolg des Empfangens ein TimeOut-Fehler  }
{                      gemeldet wird. (Ist iTimeOut = 0 wird un-     }
{                      endlich lange gewartet.)                      }
{           bSigMask - Bitmaske der zu testenden Signalleitungen     }
{                      (RTS, CTS, CD, RI)                            }
{           bSigVals - Zustand der Signalleitungen nach dem aus-     }
{                      maskieren mit obiger Maske.                   }
{ Ausgabe :  = 0 - Byte wurde gesendet                               }
{           != 0 - Fehler                                            }
{********************************************************************}
Function ser_ReadByte(     iSerPort           : Integer;
                       var Data               : Byte;
                           uTimeOut           : Word;
                           bSigMask, bSigVals : Byte ) : Integer;
Begin
  if uTimeOut <> 0 then                           { TimeOut-Schleife }
    Begin
      while( not ser_IsDataAvaiable( iSerPort )
             and ( uTimeOut <> 0 ) )
        do Dec( uTimeOut );
      if uTimeOut = 0 then
        Begin
          ser_ReadByte := SER_ERRTIMEOUT;
          Exit;
        End;
    End
  else                                           { Warten! }
    Repeat
    Until ser_IsDataAvaiable( iSerPort );        

  {-- Signalleitungen testen -------------------------------------}
  if ( port[iSerPort+SER_MODEM_STATUS] and bSigMask ) = bSigVals then
    Begin                   { empfangenes Byte von Schnittstelle lesen }
      Data := port[iSerPort + SER_RXBUFFER];
      ser_ReadByte := port[iSerPort + SER_LINE_STATUS] and
                      SER_LSR_ERRORMSK;
    End
  else
    ser_ReadByte := SER_ERRSIGNALS;
End;

{********************************************************************}
{ ser_WritePaket : Daten-Paket senden                                }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der Schnittstelle, ber die Daten   }
{                      versendet werden soll.                        }
{           pData    - Adresse der zu versendenden Daten             }
{           iLen     - >= 0 : Anzahl der zu sendenden Bytes.         }
{                       < 0 : Gre des Buffer = strlen( pData )     }
{           uTimeOut - Anzahl der Schleifendurchlufe nach denen bei }
{                      Misserfolg des Sendens ein TimeOut-Fehler     }
{                      gemeldet wird. (Ist iTimeOut = 0 wird un-     }
{                      endlich lange gewartet.)                      }
{           bSigMask - Bitmaske der zu testenden Signalleitungen     }
{                      (RTS, CTS, CD, RI)                            }
{           bSigVals - Zustand der Signalleitungen nach dem aus-     }
{                      maskieren mit obiger Maske.                   }
{ Ausgabe :  = 0 - Byte wurde gesendet                               }
{           <> 0 - Fehler                                            }
{********************************************************************}
Function ser_WritePaket( iSerPort : Integer;
                         pData    : pointer;
                         iLen     : Integer;
                         uTimeOut : Word;
                         bSigMask,
                         bSigVals : Byte ) : Integer;
var i, e   : Integer;
    BufPtr : SerBufPtr;

Begin
  BufPtr := pData;
  if iLen < 0 then { Lnge nicht angegeben, dann nach }
    Begin          { erstem NULL-Byte im Paket suchen }
      iLen := 0;
      While BufPtr^[iLen] <> 0 do
        Inc(iLen);
    End;

  {-- Paket durchlaufen und jedes Byte einzeln senden --------------}
  for i := 0 to iLen - 1 do
    Begin
      e := ser_WriteByte( iSerPort, BufPtr^[i], uTimeOut,
                          bSigMask, bSigVals );
      if e <> 0 then
        Begin
          ser_WritePaket := e;
          Exit;
        End;
    End;
  ser_WritePaket := SER_SUCCESS;
End;

{********************************************************************}
{ ser_ReadPaket : Daten-Paket empfangen                              }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der Schnittstelle, ber die Daten   }
{                      empfangen werden soll.                        }
{           pData    - Adresse der zu sendenden Daten                }
{           iLen     - Gre des Empfangsbuffers                     }
{           uTimeOut - Anzahl der Schleifendurchlufe nach denen bei }
{                      Misserfolg des Sendens ein TimeOut-Fehler     }
{                      gemeldet wird. (Ist iTimeOut = 0 wird unend-  }
{                      lich lnage gewartet.)                         }
{           bSigMask - Bitmaske der zu testenden Signalleitungen     }
{                      (RTS, CTS, CD, RI)                            }
{           bSigVals - Zustand der Signalleitungen nach dem aus-     }
{                      maskieren mit obiger Maske.                   }
{ Ausgabe :  = 0 - Byte wurde gesendet                               }
{           <> 0 - Fehler                                            }
{********************************************************************}
Function ser_ReadPaket( iSerPort : Integer;
                        pData    : pointer;
                        iLen     : Integer;
                        uTimeOut : Word;
                        bSigMask,
                        bSigVals : Byte ) : Integer;

var i, e   : Integer;
    BufPtr : SerBufPtr;

Begin
  BufPtr := pData;
  {-- den Puffer durchlaufen und jedes Byte einzeln empfangen ---}
  for i := 0 to iLen - 1 do
    Begin
      e := ser_ReadByte( iSerPort, BufPtr^[i], uTimeOut,
                         bSigMask, bSigVals );
      if e <> 0 then
        Begin
          ser_ReadPaket := e;
          Exit;
        End;
    End;
  ser_ReadPaket := SER_SUCCESS;
End;

{********************************************************************}
{ ser_CLRIRQ : Meldung serieller Interrupts an IRQ-Controller        }
{              unterbinden.                                          }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der Schnittstelle, die keine IRQ's  }
{                      mehr an den IRQ-Controller weiterleiten darf. }
{********************************************************************}
Procedure ser_CLRIRQ( iSerPort : Integer );

Begin
  port[iSerPort + SER_MODEM_CONTROL] :=
    port[iSerPort + SER_MODEM_CONTROL] and not SER_MCR_IRQENABLED;
End;

{********************************************************************}
{ ser_SETIRQ : Meldung serieller Interrupts an IRQ-Controller        }
{              erlauben.                                             }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisport der Schnittstelle, der IRQ's an den }
{                      IRQ-Controller weiterleiten mu.              }
{********************************************************************}
Procedure ser_SETIRQ( iSerPort : Integer );

Begin
  port[iSerPort + SER_MODEM_CONTROL] :=
    port[iSerPort + SER_MODEM_CONTROL] or SER_MCR_IRQENABLED;
End;

{********************************************************************}
{ ser_SetIRQHandler : Interrupt-Handler setzen                       }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort  - Basisport der Seriellenschnittstelle, fr    }
{                       die ein Interrupt-Handler eingerichtet       }
{                       werden soll.                                 }
{           iSerIRQ   - von der Schnittstelle belegte IRQ(!)-Leitung.}
{           lpHandler - Adresse des Interrupt-Handlers.              }
{           bEnablers - Zustnde, durch die ein IRQ ausgelst werden }
{                       soll (s. SER_IER_...-Bits)                   }
{ Ausgabe : Adresse des alten IRQ-Handlers                           }
{********************************************************************}
Function ser_SetIRQHandler( iSerPort,
                            iSerIRQ   : Integer;
                            lpHandler : Pointer;
                            bEnablers : Byte ) : Pointer;
Begin                                          
  port[iSerPort + SER_IRQ_ENABLE] := bEnablers; {IRQ-Auslser setzen}
  ser_SETIRQ( iSerPort );           { IRQ's an IRQ-Controller melden }

  {-- Handler setzen (IRQ wird dort 'enabled') ---------------}
  ser_SetIRQHandler := irq_SetHandler( iSerIRQ, lpHandler );
End;

{********************************************************************}
{ ser_RestoreIRQHandler : Alten IRQ-Handler restaurieren             }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort  - Basisport der Seriellenschnittstelle, deren  }
{                       alter Interrupt-Handler wieder eingerichtet  }
{                       werden soll.                                 }
{           iSerIRQ   - von der Schnittstelle belgte IRQ(!)-Leitung. }
{           lpHandler - Adresse des alten Interrupt-Handlers.        }
{********************************************************************}
Procedure ser_RestoreIRQHandler( iSerPort,
                                 iSerIRQ   : Integer;
                                 lpHandler : Pointer);
Begin
  {-- IRQ's nicht mehr an IRQ-Controller melden ---------------------}
  {-- Handler setzen und alle 'Enabler' lschen                      }

  ser_CLRIRQ( iSerPort );
  ser_SetIRQHandler( iSerPort, iSerIRQ, lpHandler, 0 );
  irq_Disable( iSerIRQ );     { IRQ auch controllerseitig 'disablen' }
End;

{********************************************************************}
{ ser_PrintError : Fehlermeldung ausgeben                            }
{*------------------------------------------------------------------*}
{ Eingabe : Win - Windows, in dem die Ausgabe erfolgen soll          }
{           e   - Fehlercode                                         }
{********************************************************************}
Procedure ser_PrintError( var Win : WINDOW; e : Integer );

Begin
  case e of
    SER_LSR_DATARECEIVED:
      win_print( Win, 'Alte Daten!'#10 );

    SER_ERRTIMEOUT:
      win_print( Win, 'TimeOut-Error!' );

    SER_ERRSIGNALS:
      win_print( Win, 'Signalleitungen!' );

  else
    begin
      if ( e and SER_LSR_OVERRUNERROR ) <> 0 then
        win_print( Win, 'Overrun Error!'#10 );

      if ( e and SER_LSR_PARITYERROR ) <> 0 then
        win_print( Win, 'Parity Error!'#10 );

      if ( e and SER_LSR_FRAMINGERROR ) <> 0 then
        win_print( Win, 'Framing Error!'#10 );

      if ( e and SER_LSR_BREAKDETECT ) <> 0 then
        win_print( Win, 'Break detect!'#10 );
    end;
  End;
End;

{********************************************************************}
{ ser_PrintModemStatus : Zustand der Signalleitungen anzeigen        }
{*------------------------------------------------------------------*}
{ Eingabe : Win      - Fenster, in dem die Ausgabe erfolgen soll     }
{           iSerPort - Basisport der Schnittstelle, deren Leitungs-  }
{                      Zustnde ausgegeben werden sollen.            }
{********************************************************************}
Procedure ser_PrintModemStatus( var Win : WINDOW; iSerPort:Integer );

var b : Byte;

Begin
  b := port[iSerPort + SER_MODEM_STATUS];
  if ( b and SER_MSR_DCTS ) <> 0 then
    win_print( Win,'DCTS :' )
  else
    win_print( Win,' CTS :' );

  if ( b and SER_MSR_CTS ) <> 0 then
    win_print( Win,'[X]'#10 )
  else
    win_print( Win,'[ ]'#10 );

  if ( b and SER_MSR_DDSR ) <> 0 then
    win_print( Win,'DDSR :' )
  else
    win_print( Win,' DSR :'  );

  if ( b and SER_MSR_DSR ) <> 0 then
    win_print( Win,'[X]'#10 )
  else
    win_print( Win,'[ ]'#10 );

  if ( b and SER_MSR_DRI ) <> 0 then
    win_print( Win,'DRI  :' )
  else
    win_print( Win,' RI  :' );

  if ( b and SER_MSR_RI ) <> 0 then
    win_print( Win,'[X]'#10 )
  else
    win_print( Win,'[ ]'#10  );

  if ( b and SER_MSR_DCD ) <> 0 then
    win_print( Win,'DCD  :' )
  else
    win_print( Win,' CD  :' );

  if ( b and SER_MSR_CD ) <> 0 then
    win_print( Win,'[X]'#10 )
  else
    win_print( Win,'[ ]'#10 );
End;

{********************************************************************}
{ ser_PrintLineStatus : Zustand der Schnittstelle anzeigen           }
{*------------------------------------------------------------------*}
{ Eingabe : Win      - das Fenster, in dem die Ausgabe erfolgen soll }
{           iSerPort - Basisport der Schnittstelle, deren interne-   }
{                      Zustnde ausgegeben werden sollen.            }
{********************************************************************}
Procedure ser_PrintLineStatus( var Win : WINDOW; iSerPort : Integer );

var b : Byte;

Begin
  b := port[iSerPort + SER_MODEM_STATUS];
  if ( b and SER_LSR_DATARECEIVED ) <> 0 then
    win_print( Win, 'Data received [X]'#10 )
  else
    win_print( Win, 'Data received [ ]' );

  if ( b and SER_LSR_OVERRUNERROR ) <> 0 then
    win_print( Win, 'Overrun Error [X]'#10 )
  else
    win_print( Win, 'Overrun Error [ ]'#10 );

  if ( b and SER_LSR_PARITYERROR ) <> 0 then
    win_print( Win, 'Parity Error  [X]'#10 )
  else
    win_print( Win, 'Parity Error  [ ]'#10 );

  if ( b and SER_LSR_FRAMINGERROR ) <> 0 then
    win_print( Win, 'Framing Error [X]'#10 )
  else
    win_print( Win, 'Framing Error [ ]'#10 );

  if ( b and SER_LSR_BREAKDETECT ) <> 0 then
    win_print( Win, 'Break detect  [X]'#10 )
  else
    win_print( Win, 'Break detect  [ ]'#10 );

  if ( b and SER_LSR_THREMPTY ) <> 0 then
    win_print( Win, 'THR Empty     [X]'#10 )
  else
    win_print( Win, 'THR Empty     [ ]'#10 );

  if ( b and SER_LSR_TSREMPTY ) <> 0 then
    win_print( Win, 'TSR Empty     [X]'#10 )
  else
    win_print( Win, 'TSR Empty     [ ]'#10 );
End;

{********************************************************************}
{ ser_GetBaud : Aktuelle Baudrate einer Schnittstelle ermitteln      }
{*------------------------------------------------------------------*}
{ Eingabe : iSerPort - Basisadresse der Schnittstelle, deren Baud-   }
{                      rate ermittelt werden soll.                   }
{ Ausgabe : Baudrate                                                 }
{********************************************************************}
Function ser_GetBaud( iSerPort : Integer ) : Longint;

var uDivisor  : Word;
    bSettings : Byte;

Begin
  asm cli End; { Interrupts ausschalten }
  bSettings := port[iSerPort + SER_LINE_CONTROL];

  {-- Baudratendivisor lesen ----------------------------------------}
  port[iSerPort + SER_LINE_CONTROL] := bSettings or SER_LCR_SETDIVISOR;
  uDivisor := port[iSerPort + SER_DIVISOR_MSB] * 256 +
              port[iSerPort + SER_DIVISOR_LSB];

  port[iSerPort + SER_LINE_CONTROL] := bSettings;
  asm sti End; { Interrupts wieder zulassen }
  if uDivisor <> 0 then
    ser_GetBaud:= SER_MAXBAUD div uDivisor
  else
    ser_GetBaud := 0;
End;

{********************************************************************}
{ ser_PrintCardSettings : bertragungsparameter der Schnittstelle    }
{                         anzeigen                                   }
{*------------------------------------------------------------------*}
{ Eingabe : Win      - das Fenster, in dem die Ausgabe erfolgen soll }
{           iSerPort - Basisport der Schnittstelle, deren ber-      }
{                      tragungsparameter ausgegeben werden sollen.   }
{********************************************************************}
Procedure ser_PrintCardSettings( var Win : WINDOW; iSerPort : Integer );

var lBaudRate : Longint;
    bSettings : Byte;
    WordLen   : Integer;
    s         : string;

Begin
  case ser_UARTType( iSerPort ) of
     NOSER:
       Begin
         win_print (Win, ' Keine Schnittstelle entdeckt!'#10 );
         Exit;
       End;

     INS8250:
       win_print( Win, ' INS8250 UART-Chip'#10 );

     NS16450:
       win_print( Win, ' NS16450 UART-Chip'#10 );

     NS16550A:
       win_print( Win, ' NS16550A UART-Chip'#10 );

     NS16C552:
       win_print( Win, ' NS16C552 UART-Chip'#10 );
  End;

  lBaudRate := ser_GetBaud( iSerPort );
  str( lBaudRate, s );
  win_print( Win, ' Baudrate '+s+#10 );

  bSettings := port[iSerPort + SER_LINE_CONTROL];
  WordLen := 5 + ( bSettings and SER_LCR_WordLEN );
  str( WordLen, s );
  win_print( Win, ' Datenbits : '+s+#10 );

  if ( bSettings and SER_LCR_2STOPBITS ) <> 0 then
    if WordLen = 5 then
      s := '1.5'
    else
      s := '2'
  else
    s := '1';

  win_print( Win, ' StopBits  : '+s+#10 );
  win_print( Win, ' Paritt   : ' );

  case bSettings and SER_LCR_PARITYMSK of
     SER_LCR_ODDPARITY:
       win_print( Win, 'ungerade' );
     SER_LCR_EVENPARITY:
       win_print( Win, 'gerade' );
     SER_LCR_PARITYSET:
       win_print( Win, 'Immer gesetzt' );
     SER_LCR_PARITYCLR:
       win_print( Win, 'Immer gelscht' );
     else
       win_print( Win, 'keine' );
  End;                               

  win_print( Win, #10 );
  if ( bSettings and SER_LCR_SENDBREAK ) <> 0 then
    win_print(Win, 'Sende Break-Signal'#10);
End;

End.
