;**********************************************************************;
;*                       V 8 0 6 0 P A . A S M                        *;
;*--------------------------------------------------------------------*;
;*    Aufgabe        : enthlt verschiedene Routinen zur Arbeit im    *;
;*                     800*600 Punkte-Grafikmodus der Super-VGA-      *;
;*                     Karten bei 16 Farben                           *;
;*--------------------------------------------------------------------*;
;*    Autor          : MICHAEL TISCHER                                *;
;*    entwickelt am  : 14.01.1991                                     *;
;*    letztes Update : 14.01.1991                                     *;
;*--------------------------------------------------------------------*;
;*    assemblieren   : MASM /mx V8060PA;     oder   TASM -mx V8060PA  *;
;*                       ... dann mit V8060P.PAS linken               *;
;**********************************************************************;

;== Konstanten =========================================================

GC_INDEX       = 3ceh             ;Indexregister des Graphics-Ctrl.
GC_READ_MAP    = 4                ;Nummer des Read-Map-Registers
GC_BIT_MASK    = 8                ;Nummer des Bit-Mask-Registers
GC_GRAPH_MODE  = 5                ;Nummer des Graphics-Mode-Registers

;== Datensegment =======================================================

DATA   segment word public        ;mu zur Laufzeit initialisiert werden
DATA   ends

;== Programm ===========================================================

CODE       segment byte public    ;das Programmsegment

	   assume cs:code, ds:data

;-- Public-Deklarationen -----------------------------------------------

public    init800600              ;800*600-Punkte-Modus initialisieren
public    setpix                  ;Punkt setzen
public    getpix                  ;Farbe eines Punktes ermitteln

;-- Daten im Codesegment -----------------------------------------------

	   ;-- Codenummer, die der 800*600-Punkte-Modus bei verschie-
	   ;-- denen Super-VGA-Karten trgt

modenr     db 6Ah, 58h, 29h, 54h, 16h, 79h
modenrend  equ this byte

;-----------------------------------------------------------------------
;-- INIT800600: initialisiert den 800*600 Super-VGA-Grafikmodus bei
;--             16 Farben
;-- Aufruf von TP: function init800600 : boolean;
;-- Retrun-Wert : TRUE = Modus wurde initialisiert, FALSE = Fehler

init800600 proc near

	   ;-- alle Modi aus der Tabelle MODENR ausprobieren, bis ein
	   ;-- Modus vom BIOS akzeptiert wurde

	   mov  si,offset modenr  ;mit erstem Modus aus Tab. beginnen
it1:       xor  ah,ah             ;Funktion 00h: Modus initialisieren
	   mov  al,cs:[si]        ;Codenummer aus Tabelle laden
	   int  10h               ;Modus intialisieren
	   mov  ah,0fh            ;Funktion 0Fh: Modus abfragen
	   int  10h
	   cmp  al,cs:[si]        ;wurde Modus eingestellt?
	   je   it2               ;Ja --> OK

	   ;-- falsche Codenummer, nchste aus Tabelle whlen ----------

	   inc  si                  ;SI auf nchste Codenummer
	   cmp  si,offset modenrend ;gesamte Tabelle durchlaufen?
	   jne  it1                 ;Nein, mach's nochmal, Sam

	   mov  al,0              ;Ja, Funktion mit Fehler beenden
	   ret                    ;zurck zum Aufrufer

it2:       ;-- der Modus konnte initialisiert werden -------------------

	   mov  al,1              ;alles OK
	   ret                    ;zurck zum Aufrufer

init800600 endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- SETPIX: verleiht einem Punkt eine bestimmte Farbe
;-- Aufruf von TP: setpix( x , y : integer; farbe : byte );

setpix     proc near

sframe     struc                  ;Struktur zum Zugriff auf den Stack
bp0        dw ?                   ;nimmt BP auf
ret_adr0   dw ?                   ;Rcksprungadresse zum Aufrufer
farbe      dw ?                   ;Farbe
y0         dw ?                   ;Y-Ordinate
x0         dw ?                   ;X-Ordinate
sframe     ends                   ;Ende der Struktur

frame      equ [ bp - bp0 ]       ;adressiert die Elemente der Struktur

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

           ;-- zunchst Offset in Video-RAM und Shift-Wert berechnen ---

	   mov   ax,frame.y0      ;Y-Koordinate laden
	   mov   dx,800 / 8       ;mit Zeilenbreite multiplizieren
	   mul   dx
	   mov   bx,frame.x0      ;X-Koordinate laden
	   mov   cl,bl            ;Lo-Byte fr Shift-Berechnung merken

	   shr   bx,1             ;X-Koordinate durch acht teilen
	   shr   bx,1
	   shr   bx,1
	   add   bx,ax            ;dazu Offset aus Multiplikation add.

	   and   cl,7             ;Bit-Maske aus X-Koordinate berechnen
	   xor   cl,7
	   mov   ah,1
	   shl   ah,cl

	   mov   dx,GC_INDEX      ;Zugriff auf Graphics-Controller
	   mov   al,GC_BIT_MASK   ;Bit-Maske in das Bit-Mask-Register
	   out   dx,ax            ;schreiben

	   mov   ax,(02h shl 8) + GC_GRAPH_MODE;Write-Modus 2 & Read-
	   out   dx,ax                         ;Modus 0 einstellen

	   mov   ax,0A000h        ;Segmentadresse des Video-RAM
	   mov   es,ax            ;nach ES

	   mov   al,es:[bx]               ;Latch-Register laden
	   mov   al,byte ptr frame.farbe  ;Farbwert fr Punkt setzen
	   mov   es:[bx],al               ;Latch-Reg. zurckschreiben

	   ;-- Defaultwerte in den verschiedenen Registern des Graphics-
	   ;-- Controllers einstellen, die verndert wurden

	   mov   ax,(0FFh shl 8 ) + GC_BIT_MASK
	   out   dx,ax

	   mov   ax,(00h shl 8) + GC_GRAPH_MODE
	   out   dx,ax

	   pop   bp
	   ret   6                ;zurck zum Aufrufer, dabei Argumente
				  ;vom Stack entfernen

setpix     endp                   ;Ende der Prozedur

;-----------------------------------------------------------------------
;-- GETPIX: liefert die Farbe eines Punktes
;-- Aufruf von TP: x := getpix( x , y : integer );

getpix     proc near

sframe1    struc                  ;Struktur zum Zugriff auf den Stack
bp1        dw ?                   ;nimmt BP auf
ret_adr1   dw ?                   ;Rcksprungadresse zum Aufrufer
y1         dw ?                   ;Y-Ordinate
x1         dw ?                   ;X-Ordinate
sframe1    ends                   ;Ende der Struktur

frame      equ [ bp - bp1 ]       ;adressiert die Elemente der Struktur

	   push  bp               ;Adressierung der Parameter ber
	   mov   bp,sp            ;das BP-Register vorbereiten

           ;-- zunchst Offset in Video-RAM und Shift-Wert berechnen ---

	   mov   ax,frame.y1      ;Y-Koordinate laden
	   mov   dx,800 / 8       ;mit Zeilenbreite multiplizieren
	   mul   dx
	   mov   si,frame.x1      ;X-Koordinate laden
	   mov   cx,si            ;fr Shift-Berechnung merken

	   shr   si,1             ;X-Koordinate durch acht teilen
	   shr   si,1
	   shr   si,1
	   add   si,ax            ;dazu Offset aus Multiplikation add.

	   and   cl,7             ;Bit-Maske aus X-Koordinate berechnen
	   xor   cl,7
           mov   ch,1
	   shl   ch,cl

	   mov   ax,0A000h        ;Segmentadresse des Video-RAM
           mov   es,ax            ;nach ES

	   mov   dx,GC_INDEX      ;Zugriff auf Graphics-Controller
	   mov   ax,(3 shl 8)+ GC_READ_MAP    ;zunchst Plane #3
           xor   bl,bl                        ;auslesen

gp1:       out   dx,ax            ;zu lesende Bit-Plane angeben
           mov   bh,es:[si]       ;Wert aus Latch-Register laden
           and   bh,ch            ;nur gewnschten Pixel stehenlassen
           neg   bh               ;Bit 7 entsprechend des Pixels setzen
           rol   bx,1             ;Bit 7 aus BH nach Bit 1 in BL rot.

           dec   ah               ;nchste Bit-Plane bearbeiten
           jge   gp1              ;grer oder gleich Null? ---> weiter

           mov   al,bl            ;Funktionsergebnis nach AL

	   pop   bp
	   ret   4                ;zurck zum Aufrufer, dabei Argumente
				  ;vom Stack entfernen

getpix     endp                   ;Ende der Prozedur

;== Ende ===============================================================

CODE       ends                   ;Ende des Codesegments
	   end                    ;Ende des Programms


