{**********************************************************************}
{*                            V I D E O P                             *}
{*--------------------------------------------------------------------*}
{*    Aufgabe        : stellt einige Funktionen zur Verfgung, die    *}
{*                     auf dem BIOS-Video-Interrupt beruhen, aber     *}
{*                     von TURBO PASCAL noch nicht zur Verfgung      *}
{*                     gestellt werden                                *}
{*--------------------------------------------------------------------*}
{*    Autor          : MICHAEL TISCHER                                *}
{*    entwickelt am  : 10.07.1987                                     *}
{*    letztes Update : 14.01.1991                                     *}
{**********************************************************************}
                                                                        
program VIDEOP;
                                                                        
Uses Crt, Dos;                   { die CRT- und die DOS-Unit einbinden }
                                                                        
const NORMAL        = $07;           { Defintion der Zeichen-Attribute }
      HELL          = $0f;           { in bezug auf eine monochrome    }
      INVERS        = $70;           { Bildschirmkarte                 }
      UNTERSTRICHEN = $01;
      BLINKEN       = $80;
                                                                        
type TextTyp = string[80];
                                                                        
var  i,                     { Schleifenvariablen fr das Hauptprogramm }
     j,
     k,
     l       : integer;
     IString : string[2];                { nimmt Anzahl der Pfeile auf }
                                                                        
{**********************************************************************}
{* GetVideoMode: aktuellen Videomodus sowie Parameter auslesen        *}
{* Eingabe : keine                                                    *}
{* Ausgabe : die unten aufgefhrten Variablen enthalten nach dem Auf- *}
{*           ruf der Prozedur die ermittelten Werte                   *}
{**********************************************************************}
                                                                        
procedure GetVideoMode(var VideoModus,{ Nummer des aktuellen Videmodus }
                           Anzahl,      { Anzahl der Spalten pro Zeile }
                           Seite       : integer);  { akt. Bilds.seite }
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ah := $0F;                                     { Funktionsnummer }
 Intr($10, Regs);                      { BIOS-Video-Interrupt aufrufen }
 VideoModus := Regs.al;                        { Nummer des Videomodus }
 Anzahl := Regs.ah;                         { Anzahl Zeichen pro Zeile }
 Seite := Regs.bh;              { Nummer der aktuellen Bildschirmseite }
end;
                                                                        
{**********************************************************************}
{* SetCursorType: definiert das Erscheinungsbild des blinkenden       *}
{*                Bildschirmcursors                                   *}
{* Eingabe : s.u.                                                     *}
{* Ausgabe : keine                                                    *}
{* Info    : bei einer monochromen Bildschirmkarte knnen die Para-   *}
{*           meter im Bereich zwischen 0 und 13, bei einer Color-     *}
{*           Bildschirmkarte zwischen 0 und 7 liegen                  *}
{**********************************************************************}
                                                                        
procedure SetCursorType(Anfang,             { Anfangszeile des Cursors }
                        Ende    : integer);     { Endzeile des Cursors }
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ah := 1;                                       { Funktionsnummer }
 Regs.ch := Anfang;                   { Anfangszeile fr Cursor setzen }
 Regs.cl := Ende;                         { Endzeile fr Cursor setzen }
 Intr($10, Regs);                      { BIOS-Video-Interrupt aufrufen }
end;
                                                                        
{**********************************************************************}
{* SetCursorPos: definiert die Position des Cursor in der angegebenen *}
{*               Bildschirmseite                                      *}
{* Eingabe : s.u.                                                     *}
{* Ausgabe : keine                                                    *}
{* Info    : die Position des blinkenden Bildschirmcursors ndert     *}
{*           sich durch den Aufruf dieser Prozedur nur dann, wenn die *}
{*           angegebene Bildschirmseite die aktuelle Bildschirmseite  *}
{*           ist                                                      *}
{**********************************************************************}
                                                                        
procedure SetCursorPos(Seite, { Bildschirms. deren Cursor gesetzt wird }
                       Spalte,                { neue Spalte des Cursor }
                       Zeile   : integer);     { neue Zeile des Cursor }
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ah := 2;                                       { Funktionsnummer }
 Regs.bh := Seite;                                   { Bildschirmseite }
 Regs.dh := Zeile;                             { Bildschirmkoordinaten }
 Regs.dl := Spalte;                            { setzen                }
 Intr($10, Regs);                      { BIOS-Video-Interrupt aufrufen }
end;
                                                                        
{**********************************************************************}
{* GetCursorPos: ermittelt die Position des Cursor in einer Bild-     *}
{*               schirmseite sowie seine Start- und Endzeile          *}
{* Eingabe : s.u.                                                     *}
{* Ausgabe : die unten aufgefhrten Variablen enthalten nach dem      *}
{*           Aufruf der Prozedur die ermittelten Werte                *}
{* Info    : das Start- und Endzeile des Cursor ist unabhngig von    *}
{*           der angegebenen Bildschirmseite                          *}
{**********************************************************************}
                                                                        
procedure GetCursorPos(Seite : integer;          { die Bildschirmseite }
                       var Spalte,                { Spalte des Cursors }
                           Zeile,                  { Zeile des Cursors }
                           Anfang,            { Startzeile des Cursors }
                           Ende    : integer);  { Endzeile des Cursors }
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ah := 3;                                       { Funktionsnummer }
 Regs.bh := Seite;                                   { Bildschirmseite }
 Intr($10, Regs);                      { BIOS-Video-Interrupt aufrufen }
 Spalte := Regs.dl;                          { Ergebnisse der Funktion }
 Zeile := Regs.dh;                           { aus den Registern lesen }
 Anfang := Regs.ch;                          { und den entsprechenden  }
 Ende := Regs.cl;                            { Variablen zuweisen      }
end;
                                                                        
{**********************************************************************}
{* SetDisplayPage: setzt die auf dem Bildschirm darzustellende Bild-  *}
{*                 schirmseite                                        *}
{* Eingabe : s.u.                                                     *}
{* Ausgabe : keine                                                    *}
{**********************************************************************}
                                                                        
procedure SetDisplayPage(Seite : integer);  { die neue Bildschirmseite }
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ah := 5;                          { Funktionsnummer fr Set Page }
 Regs.al := Seite;                                   { Bildschirmseite }
 Intr($10, Regs);                      { BIOS-Video-Interrupt aufrufen }
end;
                                                                        
{**********************************************************************}
{* ScrollUp: verschiebt einen Bildschirmbereich um eine oder mehrere  *}
{*           Zeilen nach oben oder lscht ihn                         *}
{* Eingabe : s.u.                                                     *}
{* Ausgabe : keine                                                    *}
{* Info    : wird als Anzahl 0 bergeben, wird der Bildschirmbereich  *}
{*           mit Leerzeichen gefllt                                  *}
{**********************************************************************}
                                                                        
procedure ScrollUp(Anzahl,   { Anzahl der Zeilen um die gescrollt wird }
                   Farbe, { Attribut fr die entstehende(n) Leerzeilen }
                   SpalteOL,        { Spalte in der oberen linken Ecke }
                   ZeileOL,          { Zeile in der oberen linken Ecke }
                   SpalteUR,      { Spalte in der unteren rechten Ecke }
                   ZeileUR   : integer);{ Zeile in der unteren r. Ecke }
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ah := 6;                                       { Funktionsnummer }
 Regs.al := Anzahl;                     { Anzahl zu scrollender Zeilen }
 Regs.bh := Farbe;                            { Farbe der Leerzeile(n) }
 Regs.ch := ZeileOL;                          { Koordinaten oben links }
 Regs.cl := SpalteOL;                         { setzen                 }
 Regs.dh := ZeileUR;                        { Koordinaten unten rechts }
 Regs.dl := SpalteUR;                       { setzen                   }
 Intr($10, Regs);                      { BIOS-Video-Interrupt aufrufen }
end;
                                                                        
{**********************************************************************}
{* ScrollDown: verschiebt einen Bildschirmbereich um eine oder meh-   *}
{*             rer Zeilen nach unten oder lscht ihn                  *}
{* Eingabe : s.u.                                                     *}
{* Ausgabe : keine                                                    *}
{* Info    : wird als Anzahl 0 bergeben, wird der Bildschirmbereich  *}
{*           mit Leerzeichen gefllt                                  *}
{**********************************************************************}
                                                                        
procedure ScrollDown(Anzahl,{ Anzahl der Zeilen, um die gescrollt wird }
                     Farbe,   { Attribut fr die entstehende(n) Leerz. }
                     SpalteOL,      { Spalte in der oberen linken Ecke }
                     ZeileOL,        { Zeile in der oberen linken Ecke }
                     SpalteUR,    { Spalte in der unteren rechten Ecke }
                     ZeileUR   : integer);   { Zeile in der u. r. Ecke }
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ah := 7;                                       { Funktionsnummer }
 Regs.al := Anzahl;                     { Anzahl zu scrollender Zeilen }
 Regs.bh := Farbe;                            { Farbe der Leerzeile(n) }
 Regs.ch := ZeileOL;                          { Koordinaten oben links }
 Regs.cl := SpalteOL;                         { setzen                 }
 Regs.dh := ZeileUR;                        { Koordinaten unten rechts }
 Regs.dl := SpalteUR;                       { setzen                   }
 Intr($10, Regs);                      { BIOS-Video-Interrupt aufrufen }
end;
                                                                        
{**********************************************************************}
{* GetChar: ein Zeichen samt Attribut aus einer vorgegebenen Position *}
{*          innerhalb einer Bildschirmseite auslesen                  *}
{* Eingabe : s.u.                                                     *}
{* Ausgabe : s.u.                                                     *}
{**********************************************************************}
                                                                        
procedure GetChar(Seite,   { Bildschirmseite, auf die zugegriffen wird }
                  Spalte,                           { Bildschirmspalte }
                  Zeile       : integer;             { Bildschirmzeile }
                  var Zeichen : char;            { das dortige Zeichen }
                  var Farbe   : integer);              { sein Attribut }
                                                                        
var Regs     : Registers;     { Prozessorregister fr Interrupt-Aufruf }
    AktSpalte,                             { aktuelle Bildschirmspalte }
    AktZeile,                               { aktuelle Bildschirmzeile }
    AktSeite,                               { aktuelle Bildschirmseite }
    Dummy    : integer;{ nimmt Variablen auf,die nicht bentigt werden }
                                                                        
begin
 GetVideoMode(Dummy, Dummy, AktSeite);    { akt. Bilds.seite ermitteln }
 GetCursorPos(AktSeite, AktSpalte, AktZeile,     { akt. Cursorposition }
              Dummy, Dummy);       { in der akt. Bildschirmseite holen }
 SetCursorPos(Seite, Spalte, Zeile);   { Cursor an die bergebene Pos. }
                                                                        
 Regs.ah := 8;           { Funktionsnr. fr Zeichen und Attribut holen }
 Regs.bh := Seite;                                   { Bildschirmseite }
 Intr($10, Regs);                      { BIOS-Video-Interrupt aufrufen }
 Zeichen := chr(Regs.al);                     { ASCII-Code des Zeichen }
 Farbe := Regs.ah;                              { Attribut des Zeichen }
 SetCursorPos(AktSeite, AktSpalte, AktZeile);{ Cursor an alte Position }
end;
                                                                        
{**********************************************************************}
{* WRITECHAR: schreibt ein Zeichen mit einer vorgegebenen Farbe an    *}
{*            die aktuelle Cursorposition in der vorgegebenen Bild-   *}
{*            schirmseite                                             *}
{* Eingabe : s.u.                                                     *}
{* Ausgabe : keine                                                    *}
{* Info    : bei der Ausgabe des Zeichens werden auch die Steuer-     *}
{*           zeichen wie z.B. Carriage-Return als normale ASCII-Codes *}
{*           behandelt                                                *}
{**********************************************************************}
                                                                        
procedure WriteChar(Seite   : integer; { Bildschirmseite zum Schreiben }
                    Zeichen : char;           { ASCII-Code des Zeichen }
                    Farbe   : integer);                { sein Attribut }
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
                                                                        
begin
 Regs.ah := 9;                    { Funktionsnr. fr Zeichen schreiben }
 Regs.al := ord(Zeichen);             { Code des auszugebenden Zeichen }
 Regs.bh := Seite;                            { Bildschirmseite setzen }
 Regs.bl := Farbe;                          { Farbe des Zeichen setzen }
 Regs.cx := 1;                       { das Zeichen nur einmal ausgeben }
 Intr($10, Regs);                      { BIOS-Video-Interrupt aufrufen }
end;
                                                                        
{**********************************************************************}
{* WriteText: schreibt einen String ab einer vorgegebenen Position in *}
{*            eine Bildschirmseite.                                   *}
{* Eingabe : s.u.                                                     *}
{* Ausgabe : keine                                                    *}
{* Info    : bei der Ausgabe des Zeichens werden die Steuerzeichen wie*}
{*           z.B. Carriage-Return auch als solche behandelt           *}
{*           wird ber das Ende des Bildschirms hinaus geschrieben,   *}
{*           wird er um eine Zeile hochgescrollt                      *}
{**********************************************************************}
                                                                        
procedure WriteText(Seite,   { Bildschirmseite, in der ausgegeben wird }
                    Spalte,           { Spalte, ab der ausgegeben wird }
                    Zeile,             { Zeile, ab der ausgegeben wird }
                    Farbe  : integer;         { Farbe fr alle Zeichen }
                    Text   : TextTyp);         { der auszugebende Text }
                                                                        
var Regs : Registers;         { Prozessorregister fr Interrupt-Aufruf }
    Zaehler  : integer;                              { Schleifenzhler }
                                                                        
begin
 SetCursorPos(Seite, Spalte, Zeile);                   { Cursor setzen }
 for Zaehler:=1 to length(Text) do              { Zeichen nacheinander }
  begin                                         { abarbeiten           }
   WriteChar(Seite, ' ', Farbe);     { Farbe an der aktuellen Position }
   Regs.ah := 14;                          { Fktnr. fr Teletype-Ouput }
   Regs.al := ord(Text[zaehler]);     { Code des auszugebenden Zeichen }
   Regs.bh := Seite;                                 { Bildschirmseite }
   Intr($10, Regs);                    { BIOS-Video-Interrupt aufrufen }
  end;
end;
                                                                        
{**********************************************************************}
{**                           HAUPTPROGRAMM                          **}
{**********************************************************************}
                                                                        
begin
 ClrScr;                                          { Bildschirm lschen }
 for i := 1 to 24 do                      { Zeile 1 bis 24 durchlaufen }
  for j := 0 to 79 do                       { alle Spalten durchlaufen }
   begin
    SetCursorPos(0, j, i);                      { Cursor positionieren }
    WriteChar(0, chr(i*80+j and 255), NORMAL); { ein Zeichen schreiben }
   end;
 ScrollDown(0, NORMAL, 5, 8, 19, 22);              { Fenster 1 lschen }
 WriteText(0, 5, 8, INVERS, '   Fenster 1   ');
 ScrollDown(0, NORMAL, 60, 2, 74, 16);             { Fenster 2 lschen }
 WriteText(0, 60, 2, INVERS, '   Fenster 2   ');
 WriteText(0, 30, 12, INVERS or BLINKEN, ' >>> PC INTERN <<< ');
 WriteText(0, 0, 0, INVERS, '                      es werden noch    '+
                            'Pfeile gezeichnet                       ');
 for i := 49 downto 0 do                { insgesamt 50 Pfeile zeichnen }
  begin
   Str(i:2, IString);                    { i in ASCII-String umwandeln }
   WriteText(0, 37, 0, INVERS, IString);
   j := 1;                         { jeder Pfeil besteht aus 16 Zeilen }
   while j <= 15 do
    begin
      k := 0;
      while k < j do                 { eine Zeile des Pfeils erstellen }
       begin
        SetCursorPos(0, 12-(j shr 1)+k, 9);          { Pfeil Fenster 1 }
        WriteChar(0, '*', HELL);
        SetCursorPos(0, 67-(j shr 1)+k, 16);         { Pfeil Fenster 2 }
        WriteChar(0, '*', HELL);
        k := succ(k);
       end;
      ScrollDown(1, NORMAL, 5, 9, 19, 22);        { Fenster 1 scrollen }
      ScrollUp(1, NORMAL, 60, 3, 74, 16);         { Fenster 2 scrollen }
      for l := 0 to 8000 do                            { Warteschleife }
       ;
      j := j+2;
    end;
  end;
 ClrScr;                                          { Bildschirm lschen }
end.
