{***********************************************************************
*                            X M S P . P A S                           *
**--------------------------------------------------------------------**
*  Aufgabe        : Demonstriert den Zugriff auf extended Memory und   *
*                   High-Memory-Area mit Hilfe der XMS-Funktionen, wie *
*                   sie z.B. durch den Gertetreiber HIMEM.SYS imple-  *
*                   mentiert werden.                                   *
**--------------------------------------------------------------------**
*  Autor          : MICHAEL TISCHER                                    *
*  entwickelt am  : 27.07.1990                                         *
*  letztes Update : 29.07.1990                                         *
***********************************************************************}

program XMSP;

uses Crt, Dos;              { fr Interrupt-Aufruf und Tastaturzugriff }

const ERR_NOERR          = $00;                          { kein Fehler }
      ERR_NOTIMPLEMENTED = $80;    { angegebene Funktion nicht bekannt }
      ERR_VDISKFOUND     = $81;               { VDISK-RAMDISK entdeckt }
      ERR_A20            = $82;         { Fehler auf Adressleitung A20 }
      ERR_GENERAL        = $8E;            { allgemeiner Treiberfehler }
      ERR_UNRECOVERABLE  = $8F;              { nicht-behebbarer Fehler }
      ERR_HMANOTEXIST    = $90;                  { keine HMA vorhanden }
      ERR_HMAINUSE       = $91;                 { HMA bereits vergeben }
      ERR_HMAMINSIZE     = $92;                { zu wenig Platz in HMA }
      ERR_HMANOTALLOCED  = $93;                  { HMA nicht allokiert }
      ERR_A20STILLON     = $94;            { Adressleitung A20 noch an }
      ERR_OUTOMEMORY     = $A0;       { kein extended Memory mehr frei }
      ERR_OUTOHANDLES    = $A1;              { alle XMS-Handles belegt }
      ERR_INVALIDHANDLE  = $A2;                    { ungltiges Handle }
      ERR_SHINVALID      = $A3;               { Source-Handle ungltig }
      ERR_SOINVALID      = $A4;               { Source-Offset ungltig }
      ERR_DHINVALID      = $A5;          { Destination-Handle ungltig }
      ERR_DOINVALID      = $A6;          { Destination-Offset ungltig }
      ERR_LENINVALID     = $A7;   { ungltige Lnge fr Move- Funktion }
      ERR_OVERLAP        = $A8;                { verbotene berlappung }
      ERR_PARITY         = $A9;                      { Partits-Fehler }
      ERR_EMBUNLOCKED    = $AA;               { UMB ist nicht gesperrt }
      ERR_EMBLOCKED      = $AB;                { UMB ist noch gesperrt }
      ERR_LOCKOVERFLOW   = $AC;        { berlauf des UMB-Sperrzhlers }
      ERR_LOCKFAIL       = $AD;       { UMB kann nicht gesperrt werden }
      ERR_UMBSIZETOOBIG  = $B0;              { kleinerer UMB verfgbar }
      ERR_NOUMBS         = $B1;              { kein UMB mehr verfgbar }
      ERR_INVALIDUMB     = $B2;      { UMB-Segmentadresse ist ungltig }

type XMSRegs = record                   { Informationen fr XMS-Aufruf }
                 AX,              { nur die Register AX, BX, DX und SI }
                 BX,              { werden je nach aufgerufener Funk-  }
                 DX,              { tion bentigt, dazu noch eine Seg- }
                 SI,              { mentadresse                        }
                 Segment : word
               end;

{-- globale Variablen -------------------------------------------------}

var XMSPtr : pointer;   { Zeiger auf den Extended-Memory-Manager (XMM) }
    XMSErr : BYTE;                  { Fehlercode der letzten Operation }

{***********************************************************************
* XMSInit : Initialisiert die Routinen zum Aufruf der XMS-Funktionen   *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Ausgabe : TRUE, wenn ein XMS-Treiber entdeckt wurde, sonst FALSE     *
* Info    : - Der Aufruf dieser Funktion mu dem Aufruf aller anderen  *
*             Prozeduren und Funktionen aus diesem Programm voraus-    *
*             gehen.                                                   *
***********************************************************************}

function XMSInit : boolean;

var Regs : Registers;                   { Register zum Interruptaufruf }
    xr : XMSRegs;

begin
  Regs.AX := $4300;       { Verfgbarkeit des XMS-Managers feststellen }
  intr( $2F, Regs );
  if ( Regs.AL = $80 ) then                 { XMS-Manager angetroffen? }
    begin                                                         { Ja }
      Regs.AX := $4310;            { Einsprungspunkt des XMM ermitteln }
      intr( $2F, Regs );
      XMSPtr := ptr( Regs.ES, Regs.BX );   { Adresse in glob. Var. sp. }
      XMSErr := ERR_NOERR;              { noch kein Fehler aufgetreten }
      XMSInit := true;      { Handler angetroffen, Modul initialisiert }
    end
  else                                  { kein XMS-Handler installiert }
   XMSInit := false;
end;

{***********************************************************************
* XMSCall : Allgmeine Routine zum Aufruf einer XMS-Funktion            *
**--------------------------------------------------------------------**
* Eingabe : FktNr = Nummer der aufzurufenden XMS-Funktion              *
*           XRegs = Struktur mit Registern fr Funktionsaufruf         *
* Info    : - Es mssen vor dem Aufruf dieser Prozedur immer nur die   *
*             Register geladen werden, die fr den Aufruf der angege-  *
*             benen Funktion tatschlich bentigt werden.              *
*           - Nach dem XMS-Funktionsaufruf werden die Inhalte der ver- *
*             schiedenen Prozessorregister in die korrespondierenden   *
*             Komponenten der bergebenen Struktur kopiert.            *
*           - Vor dem ersten Aufruf dieser Prozedur mu die Prozedur   *
*             XMSInit erfolgreich aufgerufen worden sein.              *
***********************************************************************}

procedure XMSCall( FktNr : byte; var XRegs : XMSRegs );

begin
  inline ( $8C / $D9 /                           { mov    cx,ds        }
           $51 /                                 { push   cx           }
           $C5 / $BE / $04 / $00 /               { lds    di,[bp+0004] }
           $8A / $66 / $08 /                     { mov    ah,[bp+0008] }
           $8B / $9D / $02 / $00 /               { mov    bx,[di+0002] }
           $8B / $95 / $04 / $00 /               { mov    dx,[di+0004] }
           $8B / $B5 / $06 / $00 /               { mov    si,[di+0006] }
           $8E / $5D / $08 /                     { mov    ds,[di+08]   }
           $8E / $C1 /                           { mov    es,cx        }
           $26 / $FF / $1E / XMSPtr /            { call   es:[XMSPTr]  }
           $8C / $D9 /                           { mov    cx,ds        }
           $C5 / $7E / $04 /                     { lds    di,[bp+04]   }
           $89 / $05 /                           { mov    [di],ax      }
           $89 / $5D / $02 /                     { mov    [di+02],bx   }
           $89 / $55 / $04 /                     { mov    [di+04],dx   }
           $89 / $75 / $06 /                     { mov    [di+06],si   }
           $89 / $4D / $08 /                     { mov    [di+08],cx   }
           $1F                                   { pop    ds           }
        );

  {-- auf Fehlercode testen -------------------------------------------}

  if ( XRegs.AX = 0 ) and ( XRegs.BX >= 128 ) then
    begin
      XMSErr := Lo(XRegs.BX)               { Fehler, Fehlercode merken }
      {
       .
       .
       .
         hier knnte eine weitere Fehlerbehandlung erfolgen
       .
       .
       .
      }
    end
  else
    XMSErr := ERR_NOERR;                     { Kein Fehler, alles o.k. }
end;

{***********************************************************************
* XMSQueryVer: liefert die XMS-Versionsnummer und andere Statusinfor-  *
*              mationen                                                *
**--------------------------------------------------------------------**
* Eingabe : VerNr = nimmt die Versionsnummer nach dem Funktionsaufruf  *
*                   auf (Format: 235 = 2.35)                           *
*           RevNr = nimmt die Revisionsnummer nach dem Funktionsaufruf *
*                   auf                                                *
* Ausgabe : TRUE, wenn eine HMA zur Verfgung steht, sonst FALSE       *
***********************************************************************}

function XMSQueryVer( var VerNr, RevNr : integer ): boolean;

var XR : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  XmsCall( 0, XR );
  VerNr := Hi(XR.AX)*100 + ( Lo(XR.AX) shr 4 ) * 10 +
           ( Lo(XR.AX) and 15 );
  RevNr := Hi(XR.BX)*100 + ( Lo(XR.BX) shr 4 ) * 10 +
           ( Lo(XR.BX) and 15 );
  XMSQueryVer := ( XR.DX = 1 );
end;

{***********************************************************************
* XMSGetHMA : Liefert dem Aufrufer das Zugriffsrecht auf die HMA.      *
**--------------------------------------------------------------------**
* Eingabe : LenB = Anzahl der zu allokierenden Bytes                   *
* Info    : TSR-Programme sollten immer nur genau die tatschlich be-  *
*           ntigte Speichergre anfordern, Applikationen hingegen    *
*           den Wert $FFFF angeben.                                    *
* Ausgabe : TRUE, wenn die HMA zur Verfgung gestellt werden konnte,   *
*           sonst FALSE;                                               *
***********************************************************************}

function XMSGetHMA( LenB : word ) : boolean;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  XR.DX := LenB;                      { Lnge im Register DX bergeben }
  XmsCall( 1, Xr );                         { XMS-Funktion #1 aufrufen }
  XMSGetHMA := ( XMSErr = ERR_NOERR );
end;

{***********************************************************************
* XMSReleaseHMA : Gibt die HMA wieder frei und ermglicht dadurch ihre *
*                 Weitergabe an andere Programme.                      *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Info    : - Vor der Beendigung eines Programms sollte diese Prozedur *
*             aufgerufen werden, sofern die HMA zuvor ber einen Aufruf*
*             von XMSGetHMA allokiert wurde, weil sie sonst nicht an   *
*             nachfolgend aufgerufene Programme vergeben werden kann.  *
*           - Die in der HAM gespeicherten Daten gehen durch den Auf-  *
*             ruf dieser Prozedur verloren.                            *
***********************************************************************}

procedure XMSReleaseHMA;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  XmsCall( 2, Xr );                         { XMS-Funktion #2 aufrufen }
end;

{***********************************************************************
* XMSA20OnGlobal: Schaltet die Adressleitung A20 frei, soda ein di-   *
*                 rekter Zugriff auf die HMA mglich wird.             *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Info    : - Die Freischaltung der Adressleitung A20 ist bei vielen   *
*             Rechnern ein relativ zeitaufwendiger Vorgagn. Rufen Sie  *
*             dieser Prozedur deshalb nicht unntig oft auf.           *
***********************************************************************}

procedure XMSA20OnGlobal;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  XmsCall( 3, Xr );                         { XMS-Funktion #3 aufrufen }
end;

{***********************************************************************
* XMSA20OffGlobal: Als Gegenstck zu der Prozedur XMSA20OnGlobal       *
*                  schaltet diese Prozedur die Adressleitung A20 wie-  *
*                  der ab, so da kein direkter Zugriff auf die HMA    *
*                  mehr mglich ist.                                   *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Info    : - Diese Prozedur sollte vor der Beendigung eines Programms *
*             immer aufgerufen werden, falls zuvor die Leitung A20     *
*             mittels eines Aufrufs von XMSA20OnGlobal freigeschaltet  *
*             wurde.                                                   *
***********************************************************************}

procedure XMSA20OffGlobal;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  XmsCall( 4, Xr );                         { XMS-Funktion #4 aufrufen }
end;

{***********************************************************************
* XMSA20OnLocal: Siehe XMSA20OnGlobal                                  *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Info    : - Von der gleichartigen globalen Prozedur unterscheidet    *
*             sich diese lokale Prozedur dadurch, da die Freischal-   *
*             tung nur dann erfolgt, wenn dies nicht bereits durch ei- *
*             nen vorhergehenden Aufruf erfolgt ist.                   *
***********************************************************************}

procedure XMSA20OnLocal;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  XmsCall( 5, Xr );                         { XMS-Funktion #5 aufrufen }
end;

{***********************************************************************
* XMSA20OffLocal : Siehe XMSA29OffGlobal                               *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Info    : - Von der gleichartigen globalen Prozedur unterscheidet    *
*             sich diese lokale Prozedur dadurch, da die Abschaltung  *
*             nur dann erfolgt, wenn dies nicht bereits durch einen    *
*             vorhergehenden Aufruf erfolgt ist.                       *
***********************************************************************}

procedure XMSA20OffLocal;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  XmsCall( 6, Xr );                         { XMS-Funktion #6 aufrufen }
end;

{***********************************************************************
* XMSIsA20On : Liefert den Status der Adressleitung A20                *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
* Ausgabe : TRUE, wenn die Leitungs A20 freigeschaltet ist, sonst      *
*           FALSE.                                                     *
***********************************************************************}

function XMSIsA20On : boolean;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  XmsCall( 7, Xr );                         { XMS-Funktion #7 aufrufen }
  XMSIsA20On := ( Xr.AX = 1 );          { AX = 1 ---> Leitung ist frei }
end;

{***********************************************************************
* XMSQueryFree : Liefert die Gre des freien extended Memory und die  *
*                des grten freien Blocks                             *
**--------------------------------------------------------------------**
* Eingabe : GesFrei: Nimmt die Gesamtgre des freien EM auf.          *
*           MaxBl  : Nimmt die Gre des grten freien Blocks auf.    *
* Info    : - Beide Angabe in KB                                       *
*           - Die Gre der HMA wird nicht mitgezhlt, auch wenn sie   *
*             noch nicht an ein Programm vergeben wurde.               *
***********************************************************************}

procedure XMSQueryFree( var GesFrei, MaxBl : integer );

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  XmsCall( 8, Xr );                         { XMS-Funktion #8 aufrufen }
  GesFrei := Xr.AX;                          { Gesamtgre steht in AX }
  MaxBl   := Xr.DX;                      { freier Speicher steht in DX }
end;

{***********************************************************************
* XMSGetMem : Allokiert einen extended-Memory-Block (EMB)              *
**--------------------------------------------------------------------**
* Eingabe : LenKB : Gre des angeforderten Blocks in KB               *
* Ausgabe : Handle zum weiteren Zugriff auf dne Block oder 0, wenn     *
*           kein Block allokiert werden konnte. Dann steht auch in der *
*           globalen Variable XMSErr ein entsprechender Fehlercode.    *
***********************************************************************}

function XMSGetMem( LenKb : integer ) : integer;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  Xr.DX := LenKB;                { Lnge wird im DX-Register bergeben }
  XmsCall( 9, Xr );                         { XMS-Funktion #9 aufrufen }
  XMSGetMem := Xr.DX                            { Handle zurckliefern }
end;

{***********************************************************************
* XMSFreeMem : Gibt einen zuvor allokierten extended-Memory-Block      *
*              (EMB) wieder frei.                                      *
**--------------------------------------------------------------------**
* Eingabe : Handle : Das Handle zum Zugriff auf den Block, das beim    *
*                    Aufruf von XMSGetMem zurckgeliefert wurde.       *
* Info    : - Der Inhalt des EMB geht durch diesen Aufruf unwider-     *
*             bringlich verloren und auch das Handle wird ungltig.    *
*           - Vor der Beendigung eines Programms sollten alle allo-    *
*             kierten Bereich mit Hilfe dieser Prozedur wieder frei-   *
*             gegeben werden, damit sie an nachfolgend aufgerufene     *
*             Programm wieder vergeben werden knnen.                  *
***********************************************************************}

procedure XMSFreeMem( Handle : integer );

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  Xr.DX := Handle;              { Handle wird im DX-Register bergeben }
  XmsCall( 10, Xr );                       { XMS-Funktion #10 aufrufen }
end;

{***********************************************************************
* XMSCopy : Kopiert Speicherbereiche zwischen dem extended Memory und  *
*           dem konventionellen Speicher bzw. innerhalb der beiden     *
*           Speichergruppen.                                           *
**--------------------------------------------------------------------**
* Eingabe : VonHandle  : Handle des Speicherbereichs, der verschoben   *
*                        werden soll.                                  *
*           VonOffset  : Offset in diesen Block ab dem verschoben wird.*
*           NachHandle : Handle des Speicherbereichs, der das Ziel der *
*                        Verschiebung darstellt.                       *
*           NachOffset : Offset in den Ziel-Block.                     *
*           LenW       : Anzahl der zu verschiebenden Worte            *
* Info    : - Um normalen Speicher in die Operation einzubeziehen,     *
*             mu als Handle der Wert 0 und fr den Offset die Seg-    *
*             ment und Offsetadresse in der gewohnte Form (Offset vor  *
*             Segment) angegeben werden.                               *
***********************************************************************}

procedure XMSCopy( VonHandle  : integer; VonOffset  : longint;
                   NachHandle : integer; NachOffset : longint;
                   LenW       : longint  );

type EMMS = record                { eine extended-Memory-Move-Struktur }
              LenB    : longint;             { Anzahl zu versch. Bytes }
              SHandle : integer;                       { Source-Handle }
              SOffset : longint;                       { Source-Offset }
              DHandle : integer;                  { Destination-Handle }
              DOffset : longint;                  { Destination-Offset }
            end;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }
    Mi : EMMS;                                        { nimmt EEMS auf }

begin
  with Mi do                           { zunchst den EMMS aufbereiten }
    begin
      LenB := 2 * LenW;
      SHandle := VonHandle;
      SOffset := VonOffset;
      DHandle := NachHandle;
      DOffset := NachOffset
    end;

  Xr.Si      := Ofs( Mi );                    { Offsetadresse des EMMS }
  Xr.Segment := Seg(Mi);                     { Segmentadresse des EMMS }
  XmsCall( 11, Xr );                       { XMS-Funktion #11 aufrufen }
end;

{***********************************************************************
* XMSLock : Sperrt einen extended-Memory-Block gegen sein Verschiebung *
*           durch den XMM und liefert gleichzeitig seine absolute      *
*           Adresse zurck.                                            *
**--------------------------------------------------------------------**
* Eingabe : Handle : Handle des Speicherbereichs, das bei einem vor-   *
*                    hergehenden Aufruf von XMSGetMem zurckgeliefert  *
*                    wurde.                                            *
* Ausgabe : Die lineare Adresse des zugehrigen Speicherblocks.        *
***********************************************************************}

function XMSLock( Handle : integer ) : longint;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  Xr.DX := Handle;                                    { Handle des EMB }
  XmsCall( 12, Xr );                       { XMS-Funktion #12 aufrufen }
  XMSLock := longint(Xr.DX) shl 16 + Xr.BX;    { 32-Bit-Adr. berechnen }
end;

{***********************************************************************
* XMSUnlock : Gibt einen gesperrten extended-Memory-Block wieder fr   *
*             die Verschiebung frei.                                   *
**--------------------------------------------------------------------**
* Eingabe : Handle : Handle des Speicherbereichs, das bei einem vor-   *
*                    hergehenden Aufruf von XMSGetMem zurckgeliefert  *
*                    wurde.                                            *
***********************************************************************}

procedure XMSUnLock( Handle : integer );

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  Xr.DX := Handle;                                    { Handle des EMB }
  XmsCall( 13, Xr );                       { XMS-Funktion #13 aufrufen }
end;

{***********************************************************************
* XMSQueryInfo : Liefert verschiedene Informationen ber einen bereits *
*                allokierten extended-Memory-Block                     *
**--------------------------------------------------------------------**
* Eingabe : Handle : Handle des Speicherbereichs                       *
*           Lock   : Variable, in die der Lock-Zhler eingetragen wird *
*           LenKB  : Variable, in die die Lnge des Blocks in KB ein-  *
*                    getragen wird                                     *
*           FreeH  : Anzahl der noch freien Handles                    *
* Info    : Die Startadresse eines Blocks knnen Sie mit Hilfe dieser  *
*           Prozedur nicht ermitteln, verwenden Sie dafr die Funktion *
*           XMSLock.                                                   *
***********************************************************************}

procedure XMSQueryInfo( Handle    : integer; var Lock, LenKB : integer;
                        var FreeH : integer );

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  Xr.DX := Handle;                                    { Handle des EMB }
  XmsCall( 14, Xr );                       { XMS-Funktion #14 aufrufen }
  Lock  := Hi( Xr.BX );                           { Register auswerten }
  FreeH := Lo( Xr.BX );
  LenKB := Xr.DX;
end;

{***********************************************************************
* XMSRealloc : Vergert oder verkleinert einen zuvor ber XMSGetMem   *
*              allokierten extended-Memory-Block                       *
**--------------------------------------------------------------------**
* Eingabe : Handle   : Handle des Speicherbereichs                     *
*           NeuLenKB : Neue Lnge des Speicherbereichs in KB           *
* Ausgabe : TRUE, wenn der Block in seiner Gre verndert werden      *
*           konnte, sonst FALSE                                        *
* Info    : Der angegebene Block darf nicht gesperrt sein!             *
***********************************************************************}

function XMSRealloc( Handle, NeuLenKB : integer ) : boolean;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  Xr.DX := Handle;                                    { Handle des EMB }
  Xr.BX := NeuLenKB;               { die neue Lnge in das BX-Register }
  XmsCall( 15, Xr );                       { XMS-Funktion #15 aufrufen }
  XMSRealloc := ( XMSErr = ERR_NOERR );
end;

{***********************************************************************
* XMSGetUMB : Allokiert einen upper-Memory-Block (UMB).                *
**--------------------------------------------------------------------**
* Eingabe : LenPara : Gre des zu allokierenden Bereichs in Para-     *
*                     graphen (jeweils 16 Byte)                        *
*           Seg     : Variable, die die Segmentadresse des allo-       *
*                     kierten UMB im Erfolgsfall aufnimmt              *
*           MaxPara : Variable, die die Lnge des grten verfgbaren  *
*                     UMB im Fehlerfall angibt                         *
* Ausgabe : TRUE, wenn ein UMB allokierte werden konnte, sonst FALSE   *
* Info    : Achtung! Diese Funktion wird nicht bei allen XMS-Treibern  *
*                    untersttzt und ist extrem Hardware-abhngig.     *
***********************************************************************}

function XMSGetUMB( LenPara          : integer;
                    var Seg, MaxPara : word ) : boolean;

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  Xr.DX := LenPara;                         { gewnschte Lnge nach DX }
  XmsCall( 16, Xr );                       { XMS-Funktion #16 aufrufen }
  Seg := Xr.BX;                         { Segmentadresse zurckliefern }
  MaxPara := Xr.DX;                            { Lnge des grten UMB }
  XMSGetUMB := ( XMSErr = ERR_NOERR );
end;

{***********************************************************************
* XMSFreeUMB : Gibt einen zuvor ber XMSGetUMB allokierten UMB wieder  *
*              frei.                                                   *
**--------------------------------------------------------------------**
* Eingabe : Seg : Segmentadresse des freizugebenden UMB                *
* Info    : Achtung! Diese Funktion wird nicht bei allen XMS-Treibern  *
*                    untersttzt und ist extrem Hardware-abhngig.     *
***********************************************************************}

procedure XMSFreeUMB( var Seg : word );

var Xr : XMSRegs;                 { Register fr Kommunikation mit XMS }

begin
  Xr.DX := Seg;                       { Segmentadresse des UMB nach DX }
  XmsCall( 17, Xr );                       { XMS-Funktion #17 aufrufen }
end;

{----------------------------------------------------------------------}
{-- Test- und Demoprozeduren                                         --}
{----------------------------------------------------------------------}

{***********************************************************************
* HMATest : Testet die Verfgbarkeit der HMA und demonstriert den Um-  *
*           gang mit ihr.                                              *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
***********************************************************************}

procedure HMATest;

type HMAR = array [1..65520] of BYTE;                  { das HMA-Array }
     HMARPTR = ^HMAR;                       { Zeiger auf das HMA-Array }

var ch   : char;                              { zur Abfrage von Tasten }
    A20  : boolean;            { aktuelle Status der Adressleitung A20 }
    hmap : HMARPTR;                               { Zeiger auf die HMA }
    i,                                               { Schleifenzhler }
    err  : word;                   { Anzahl der Fehler bei HMA-Zugriff }

begin
  write( 'HMA-Test  -  Bettigen Sie bitte eine Taste, um den Test ' +
         'zu starten...' );
  ch := ReadKey;
  writeln( #10 );

  {-- HMA allokieren und jede Speicherstelle testen -------------------}

  if ( XMSGetHMA( $FFFF) ) then              { HMA in Besitz gebracht? }
    begin                                                         { Ja }
      A20 := XMSIsA20On;                    { Leitungsstatus ermitteln }
      If ( A20 = FALSE ) then              { ist die Leitung A20 frei? }
        XMSA20OnGlobal;                     { Nein, jetzt freischalten }

      hmap := HMARPTR(Ptr( $FFFF, $0010 ));           { Zeiger auf HMA }

      err := 0;                         { bis jetzt noch keinen Fehler }
      for i := 1 to 65520 do      { jede Speicherstelle einzeln testen }
        begin
          write( #13, 'Speicherstelle: ', i );
          hmap^[i] := i mod 256;          { Speicherstelle beschreiben }
          if ( hmap^[i] <> i mod 256 ) then      { und wieder auslesen }
            begin                                            { Fehler! }
              writeln( ' FEHLER!' );
              inc( err );
            end;
        end;

      XMSReleaseHMA;                        { die HMA wieder freigeben }
      If ( A20 = FALSE ) then              { war die Leitung A20 frei? }
        XMSA20OffGlobal;               { Nein, jetzt wieder abschalten }

      writeln( #13 );
      if ( err = 0 ) then               { Resultat des Tests auswerten }
        writeln( 'HMA ok, keine Speicherstelle fehlerhaft.')
      else
        writeln( 'ACHTUNG! ', err, ' fehlerhafte Speicherstellen in ' +
                 'der HMA entdeckt! ');
    end
  else
   writeln( 'ACHTUNG! Kein Zugriff auf die HMA mglich.' );
end;

{***********************************************************************
* EMBTest : Testet den extended Memory und demonstriert den Aufruf     *
*           verschiedener XMS-Funktionen                               *
**--------------------------------------------------------------------**
* Eingabe : keine                                                      *
***********************************************************************}

procedure EMBTest;

type BAR = array [1..1024] of BYTE;              { Byte-Array mit 1 KB }
     BARPTR = ^BAR;                        { Zeiger auf das Byte-Array }

var ch      : char;                           { zur Abfrage von Tasten }
    Adr     : longint;                          { Startadresse des EMB }
    barp    : BARPTR;                         { Zeiger auf 1 KB-Puffer }
    i, j,                                            { Schleifenzhler }
    err,                           { Anzahl der Fehler bei HMA-Zugriff }
    Handle,                               { Handle zum Zugriff auf EMB }
    GesFrei,               { Gre des gesamten freien extended-Memory }
    MaxBl   : integer;                          { grter freier Block }

begin
  write( 'EMB-Test  -  Bettigen Sie bitte eine Taste, um den Test ' +
         'zu starten...' );
  ch := ReadKey;
  writeln( #10 );

  XMSQueryFree( GesFrei, MaxBl );  { Gre des extended Mem. ermitteln }
  writeln( 'Gesamtgre des freien extended Memory (inkl. HMA): ',
            GesFrei, ' KB' );
  writeln( '                        davon grter freier Block: ',
            MaxBl, ' KB' );

  GesFrei := GesFrei - 64;          { tatschliche Gre ohne HMA ber. }
  if ( MaxBl >= GesFrei ) then                { kann der Wert stimmen? }
    MaxBl := MaxBl - 64;                                        { Nein }

  if ( MaxBl <> 0 ) then                { noch gengend Speicher frei? }
    begin                                                         { Ja }
      Handle := XMSGetMem( MaxBl );
      writeln( MaxBl, ' KB allokiert.' );
      writeln( 'Handle       = ', Handle );
      Adr := XMSLock( Handle );                    { Adresse ermitteln }
      XMSUnlock( Handle );                      { Lock wieder aufheben }
      writeln( 'Startadresse = ', Adr, ' (', Adr div 1024, 'KB)' );

      GetMem( barp, 1024 );        { Puffer auf Turbo-Heap allokierten }
      err := 0;                         { bis jetzt noch keinen Fehler }

      {-- den allokierten EMB KB fr KB durchlaufen und testen --------}

      for  i := 0 to MaxBl-1 do
        begin
          write( #13, 'KB-Test: ', i+1 );
          FillChar( barp^, 1024, i mod 255 );
          XMSCopy( 0, longint(barp), Handle, longint(i)*1024, 512 );
          FillChar( barp^, 1024, 255 );
          XMSCopy( Handle, longint(i)*1024, 0, longint(barp), 512 );

          {-- zurckkopierten Puffer mit erwartetem Resultat vergl. ---}

          j := 1;
          while ( j <= 1024 ) do
            if ( barp^[j] <> i mod 255 ) then
              begin                                          { Fehler! }
                writeln( ' FEHLER!' );
                inc( err );
                j := 1025;
              end
           else                  { kein Fehler, nchste Speicherstelle }
            inc( j );
        end;

      writeln( #13 );
      if ( err = 0 ) then               { Resultat des Tests auswerten }
        writeln( 'EMB ok, keiner der getesten 1 KB-Blcke ' +
                 'war fehlerhaft.')
      else
        writeln( 'ACHTUNG! ', err, ' fehlerhafte 1 KB-Blcke im EMB' +
                 ' entdeckt' );

      FreeMem( barp, 1024 );                 { Puffer wieder freigeben }
      XMSFreeMem( Handle );                     { EMB wieder freigeben }
   end;
end;

{***********************************************************************
*                        H A U P T P R O G R A M M                     *
***********************************************************************}

var VerNr,
    RevNr  : integer;

begin
  ClrScr;
  writeln( 'XMSP  -  XMS-Demoprogramm von MICHAEL TISCHER' );
  writeln;
  if XMSInit then
   begin
     if XMSQueryVer( VerNr, RevNr ) then
      writeln( 'Zugriff auf HMA mglich.' )
     else
      writeln( 'Kein Zugriff auf HMA.' );
     writeln( 'XMS-Versionsnummer: ', VerNr div 100,
               '.', VerNr mod 100 );
     writeln( 'Revisionsnummer   : ', RevNr div 100,
              '.', RevNr mod 100 );
     writeln;
     HMATest;                                             { HMA testen }
     writeln;
     EMBTest;                                 { extended Memory testen }
   end
  else
    writeln( 'Kein XMS-Treiber installiert!');
end.
